/****************************************************************************
** Meta object code from reading C++ file 'dlgtrackinfomulti.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/library/dlgtrackinfomulti.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dlgtrackinfomulti.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17DlgTrackInfoMultiE_t {};
} // unnamed namespace

template <> constexpr inline auto DlgTrackInfoMulti::qt_create_metaobjectdata<qt_meta_tag_ZN17DlgTrackInfoMultiE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DlgTrackInfoMulti",
        "slotOk",
        "",
        "slotApply",
        "slotCancel",
        "slotImportMetadataFromFiles",
        "slotTrackChanged",
        "TrackId",
        "trackId",
        "slotTagBoxIndexChanged",
        "slotCommentBoxIndexChanged",
        "commentTextChanged",
        "slotEditingFinished",
        "QComboBox*",
        "pBox",
        "QLineEdit*",
        "pLine",
        "slotKeyTextChanged",
        "slotColorButtonClicked",
        "slotColorPicked",
        "mixxx::RgbColor::optional_t",
        "newColor",
        "slotStarRatingChanged",
        "rating",
        "slotCoverFound",
        "const QObject*",
        "pRequester",
        "CoverInfo",
        "info",
        "QPixmap",
        "pixmap",
        "slotCoverInfoSelected",
        "CoverInfoRelative",
        "coverInfo",
        "slotReloadCoverArt",
        "slotOpenInFileBrowser"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotOk'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotApply'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotCancel'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotImportMetadataFromFiles'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotTrackChanged'
        QtMocHelpers::SlotData<void(TrackId)>(6, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'slotTagBoxIndexChanged'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotCommentBoxIndexChanged'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'commentTextChanged'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotEditingFinished'
        QtMocHelpers::SlotData<void(QComboBox *, QLineEdit *)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { 0x80000000 | 15, 16 },
        }}),
        // Slot 'slotKeyTextChanged'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotColorButtonClicked'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotColorPicked'
        QtMocHelpers::SlotData<void(const mixxx::RgbColor::optional_t &)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'slotStarRatingChanged'
        QtMocHelpers::SlotData<void(int)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 23 },
        }}),
        // Slot 'slotCoverFound'
        QtMocHelpers::SlotData<void(const QObject *, const CoverInfo &, const QPixmap &)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 25, 26 }, { 0x80000000 | 27, 28 }, { 0x80000000 | 29, 30 },
        }}),
        // Slot 'slotCoverInfoSelected'
        QtMocHelpers::SlotData<void(const CoverInfoRelative &)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Slot 'slotReloadCoverArt'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotOpenInFileBrowser'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DlgTrackInfoMulti, qt_meta_tag_ZN17DlgTrackInfoMultiE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DlgTrackInfoMulti::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17DlgTrackInfoMultiE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17DlgTrackInfoMultiE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17DlgTrackInfoMultiE_t>.metaTypes,
    nullptr
} };

void DlgTrackInfoMulti::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DlgTrackInfoMulti *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotOk(); break;
        case 1: _t->slotApply(); break;
        case 2: _t->slotCancel(); break;
        case 3: _t->slotImportMetadataFromFiles(); break;
        case 4: _t->slotTrackChanged((*reinterpret_cast<std::add_pointer_t<TrackId>>(_a[1]))); break;
        case 5: _t->slotTagBoxIndexChanged(); break;
        case 6: _t->slotCommentBoxIndexChanged(); break;
        case 7: _t->commentTextChanged(); break;
        case 8: _t->slotEditingFinished((*reinterpret_cast<std::add_pointer_t<QComboBox*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QLineEdit*>>(_a[2]))); break;
        case 9: _t->slotKeyTextChanged(); break;
        case 10: _t->slotColorButtonClicked(); break;
        case 11: _t->slotColorPicked((*reinterpret_cast<std::add_pointer_t<mixxx::RgbColor::optional_t>>(_a[1]))); break;
        case 12: _t->slotStarRatingChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 13: _t->slotCoverFound((*reinterpret_cast<std::add_pointer_t<const QObject*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CoverInfo>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QPixmap>>(_a[3]))); break;
        case 14: _t->slotCoverInfoSelected((*reinterpret_cast<std::add_pointer_t<CoverInfoRelative>>(_a[1]))); break;
        case 15: _t->slotReloadCoverArt(); break;
        case 16: _t->slotOpenInFileBrowser(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackId >(); break;
            }
            break;
        }
    }
}

const QMetaObject *DlgTrackInfoMulti::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DlgTrackInfoMulti::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17DlgTrackInfoMultiE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::DlgTrackInfoMulti"))
        return static_cast< Ui::DlgTrackInfoMulti*>(this);
    return QDialog::qt_metacast(_clname);
}

int DlgTrackInfoMulti::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    return _id;
}
QT_WARNING_POP
