/****************************************************************************
** Meta object code from reading C++ file 'scannertask.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/library/scanner/scannertask.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'scannertask.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11ScannerTaskE_t {};
} // unnamed namespace

template <> constexpr inline auto ScannerTask::qt_create_metaobjectdata<qt_meta_tag_ZN11ScannerTaskE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ScannerTask",
        "taskDone",
        "",
        "success",
        "queueTask",
        "ScannerTask*",
        "pTask",
        "directoryHashedAndScanned",
        "directoryPath",
        "newDirectory",
        "mixxx::cache_key_t",
        "hash",
        "directoryUnchanged",
        "trackExists",
        "filePath",
        "addNewTrack",
        "progressLoading",
        "fileName",
        "progressHashing"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'taskDone'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'queueTask'
        QtMocHelpers::SignalData<void(ScannerTask *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'directoryHashedAndScanned'
        QtMocHelpers::SignalData<void(const QString &, bool, mixxx::cache_key_t)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 }, { QMetaType::Bool, 9 }, { 0x80000000 | 10, 11 },
        }}),
        // Signal 'directoryUnchanged'
        QtMocHelpers::SignalData<void(const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Signal 'trackExists'
        QtMocHelpers::SignalData<void(const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Signal 'addNewTrack'
        QtMocHelpers::SignalData<void(const QString &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Signal 'progressLoading'
        QtMocHelpers::SignalData<void(const QString &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 17 },
        }}),
        // Signal 'progressHashing'
        QtMocHelpers::SignalData<void(const QString &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ScannerTask, qt_meta_tag_ZN11ScannerTaskE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ScannerTask::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11ScannerTaskE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11ScannerTaskE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11ScannerTaskE_t>.metaTypes,
    nullptr
} };

void ScannerTask::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ScannerTask *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->taskDone((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->queueTask((*reinterpret_cast<std::add_pointer_t<ScannerTask*>>(_a[1]))); break;
        case 2: _t->directoryHashedAndScanned((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<mixxx::cache_key_t>>(_a[3]))); break;
        case 3: _t->directoryUnchanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->trackExists((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->addNewTrack((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->progressLoading((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->progressHashing((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ScannerTask* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< mixxx::cache_key_t >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ScannerTask::*)(bool )>(_a, &ScannerTask::taskDone, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ScannerTask::*)(ScannerTask * )>(_a, &ScannerTask::queueTask, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ScannerTask::*)(const QString & , bool , mixxx::cache_key_t )>(_a, &ScannerTask::directoryHashedAndScanned, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ScannerTask::*)(const QString & )>(_a, &ScannerTask::directoryUnchanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ScannerTask::*)(const QString & )>(_a, &ScannerTask::trackExists, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ScannerTask::*)(const QString & )>(_a, &ScannerTask::addNewTrack, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ScannerTask::*)(const QString & )>(_a, &ScannerTask::progressLoading, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (ScannerTask::*)(const QString & )>(_a, &ScannerTask::progressHashing, 7))
            return;
    }
}

const QMetaObject *ScannerTask::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ScannerTask::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11ScannerTaskE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QRunnable"))
        return static_cast< QRunnable*>(this);
    return QObject::qt_metacast(_clname);
}

int ScannerTask::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void ScannerTask::taskDone(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void ScannerTask::queueTask(ScannerTask * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void ScannerTask::directoryHashedAndScanned(const QString & _t1, bool _t2, mixxx::cache_key_t _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3);
}

// SIGNAL 3
void ScannerTask::directoryUnchanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void ScannerTask::trackExists(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void ScannerTask::addNewTrack(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void ScannerTask::progressLoading(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void ScannerTask::progressHashing(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}
QT_WARNING_POP
