/****************************************************************************
** Meta object code from reading C++ file 'midicontroller.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/controllers/midi/midicontroller.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'midicontroller.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN22MidiInputHandleJSProxyE_t {};
} // unnamed namespace

template <> constexpr inline auto MidiInputHandleJSProxy::qt_create_metaobjectdata<qt_meta_tag_ZN22MidiInputHandleJSProxyE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MidiInputHandleJSProxy",
        "disconnect",
        ""
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'disconnect'
        QtMocHelpers::MethodData<bool()>(1, 2, QMC::AccessPublic, QMetaType::Bool),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MidiInputHandleJSProxy, qt_meta_tag_ZN22MidiInputHandleJSProxyE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MidiInputHandleJSProxy::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22MidiInputHandleJSProxyE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22MidiInputHandleJSProxyE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22MidiInputHandleJSProxyE_t>.metaTypes,
    nullptr
} };

void MidiInputHandleJSProxy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MidiInputHandleJSProxy *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { bool _r = _t->disconnect();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

const QMetaObject *MidiInputHandleJSProxy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MidiInputHandleJSProxy::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22MidiInputHandleJSProxyE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int MidiInputHandleJSProxy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN14MidiControllerE_t {};
} // unnamed namespace

template <> constexpr inline auto MidiController::qt_create_metaobjectdata<qt_meta_tag_ZN14MidiControllerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MidiController",
        "messageReceived",
        "",
        "status",
        "control",
        "value",
        "receivedShortMessage",
        "mixxx::Duration",
        "timestamp",
        "receive",
        "data",
        "slotBeforeEngineShutdown",
        "learnTemporaryInputMappings",
        "MidiInputMappings",
        "mappings",
        "clearTemporaryInputMappings",
        "commitTemporaryInputMappings"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'messageReceived'
        QtMocHelpers::SignalData<void(unsigned char, unsigned char, unsigned char)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UChar, 3 }, { QMetaType::UChar, 4 }, { QMetaType::UChar, 5 },
        }}),
        // Slot 'receivedShortMessage'
        QtMocHelpers::SlotData<void(unsigned char, unsigned char, unsigned char, mixxx::Duration)>(6, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::UChar, 3 }, { QMetaType::UChar, 4 }, { QMetaType::UChar, 5 }, { 0x80000000 | 7, 8 },
        }}),
        // Slot 'receive'
        QtMocHelpers::SlotData<void(const QByteArray &, mixxx::Duration)>(9, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QByteArray, 10 }, { 0x80000000 | 7, 8 },
        }}),
        // Slot 'slotBeforeEngineShutdown'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'learnTemporaryInputMappings'
        QtMocHelpers::SlotData<void(const MidiInputMappings &)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'clearTemporaryInputMappings'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'commitTemporaryInputMappings'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MidiController, qt_meta_tag_ZN14MidiControllerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MidiController::staticMetaObject = { {
    QMetaObject::SuperData::link<Controller::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14MidiControllerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14MidiControllerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14MidiControllerE_t>.metaTypes,
    nullptr
} };

void MidiController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MidiController *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->messageReceived((*reinterpret_cast<std::add_pointer_t<uchar>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uchar>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<uchar>>(_a[3]))); break;
        case 1: _t->receivedShortMessage((*reinterpret_cast<std::add_pointer_t<uchar>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uchar>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<uchar>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<mixxx::Duration>>(_a[4]))); break;
        case 2: _t->receive((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<mixxx::Duration>>(_a[2]))); break;
        case 3: _t->slotBeforeEngineShutdown(); break;
        case 4: _t->learnTemporaryInputMappings((*reinterpret_cast<std::add_pointer_t<MidiInputMappings>>(_a[1]))); break;
        case 5: _t->clearTemporaryInputMappings(); break;
        case 6: _t->commitTemporaryInputMappings(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 3:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< mixxx::Duration >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< mixxx::Duration >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MidiController::*)(unsigned char , unsigned char , unsigned char )>(_a, &MidiController::messageReceived, 0))
            return;
    }
}

const QMetaObject *MidiController::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MidiController::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14MidiControllerE_t>.strings))
        return static_cast<void*>(this);
    return Controller::qt_metacast(_clname);
}

int MidiController::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Controller::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void MidiController::messageReceived(unsigned char _t1, unsigned char _t2, unsigned char _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}
namespace {
struct qt_meta_tag_ZN21MidiControllerJSProxyE_t {};
} // unnamed namespace

template <> constexpr inline auto MidiControllerJSProxy::qt_create_metaobjectdata<qt_meta_tag_ZN21MidiControllerJSProxyE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MidiControllerJSProxy",
        "sendShortMsg",
        "",
        "status",
        "byte1",
        "byte2",
        "sendSysexMsg",
        "QList<int>",
        "data",
        "length",
        "makeInputHandler",
        "QJSValue",
        "control",
        "scriptCode"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'sendShortMsg'
        QtMocHelpers::MethodData<void(unsigned char, unsigned char, unsigned char)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UChar, 3 }, { QMetaType::UChar, 4 }, { QMetaType::UChar, 5 },
        }}),
        // Method 'sendSysexMsg'
        QtMocHelpers::MethodData<void(const QList<int> &, unsigned int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 }, { QMetaType::UInt, 9 },
        }}),
        // Method 'sendSysexMsg'
        QtMocHelpers::MethodData<void(const QList<int> &)>(6, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Method 'makeInputHandler'
        QtMocHelpers::MethodData<QJSValue(unsigned char, unsigned char, const QJSValue &)>(10, 2, QMC::AccessPublic, 0x80000000 | 11, {{
            { QMetaType::UChar, 3 }, { QMetaType::UChar, 12 }, { 0x80000000 | 11, 13 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MidiControllerJSProxy, qt_meta_tag_ZN21MidiControllerJSProxyE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MidiControllerJSProxy::staticMetaObject = { {
    QMetaObject::SuperData::link<ControllerJSProxy::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21MidiControllerJSProxyE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21MidiControllerJSProxyE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21MidiControllerJSProxyE_t>.metaTypes,
    nullptr
} };

void MidiControllerJSProxy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MidiControllerJSProxy *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sendShortMsg((*reinterpret_cast<std::add_pointer_t<uchar>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uchar>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<uchar>>(_a[3]))); break;
        case 1: _t->sendSysexMsg((*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2]))); break;
        case 2: _t->sendSysexMsg((*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[1]))); break;
        case 3: { QJSValue _r = _t->makeInputHandler((*reinterpret_cast<std::add_pointer_t<uchar>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uchar>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QJSValue >(); break;
            }
            break;
        }
    }
}

const QMetaObject *MidiControllerJSProxy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MidiControllerJSProxy::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21MidiControllerJSProxyE_t>.strings))
        return static_cast<void*>(this);
    return ControllerJSProxy::qt_metacast(_clname);
}

int MidiControllerJSProxy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ControllerJSProxy::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}
QT_WARNING_POP
