// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: waveform.proto
// Protobuf C++ Version: 7.35.0

#ifndef waveform_2eproto_2epb_2eh
#define waveform_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

// clang-format off
#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 7035000
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/generated_enum_util.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_waveform_2eproto

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct TableStruct_waveform_2eproto {
  static const ::uint32_t offsets[];
};
namespace mixxx {
namespace track {
namespace io {
enum Waveform_Units : int;
extern const uint32_t Waveform_Units_internal_data_[];
class Waveform;
struct WaveformGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern WaveformGlobalsTypeInternal Waveform_globals_;
extern const ::google::protobuf::internal::ClassDataLite Waveform_class_data_;
#else
extern const WaveformGlobalsTypeInternal Waveform_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Waveform_FilteredSignal;
struct Waveform_FilteredSignalGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern Waveform_FilteredSignalGlobalsTypeInternal Waveform_FilteredSignal_globals_;
extern const ::google::protobuf::internal::ClassDataLite Waveform_FilteredSignal_class_data_;
#else
extern const Waveform_FilteredSignalGlobalsTypeInternal Waveform_FilteredSignal_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Waveform_Signal;
struct Waveform_SignalGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern Waveform_SignalGlobalsTypeInternal Waveform_Signal_globals_;
extern const ::google::protobuf::internal::ClassDataLite Waveform_Signal_class_data_;
#else
extern const Waveform_SignalGlobalsTypeInternal Waveform_Signal_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
}  // namespace io
}  // namespace track
}  // namespace mixxx
namespace google {
namespace protobuf {
template <>
internal::EnumTraitsT<::mixxx::track::io::Waveform_Units_internal_data_>
    internal::EnumTraitsImpl::value<::mixxx::track::io::Waveform_Units>;
}  // namespace protobuf
}  // namespace google

namespace mixxx {
namespace track {
namespace io {
enum Waveform_Units : int {
  Waveform_Units_RMS = 0,
  Waveform_Units_AMPLITUDE = 1,
};

extern const uint32_t Waveform_Units_internal_data_[];
inline constexpr Waveform_Units Waveform_Units_Units_MIN =
    static_cast<Waveform_Units>(0);
inline constexpr Waveform_Units Waveform_Units_Units_MAX =
    static_cast<Waveform_Units>(1);
[[nodiscard]] inline bool Waveform_Units_IsValid(int value) {
  return 0 <= value && value <= 1;
}
inline constexpr int Waveform_Units_Units_ARRAYSIZE = 1 + 1;
[[nodiscard]] const ::std::string& Waveform_Units_Name(Waveform_Units value);
template <typename T>
[[nodiscard]] const ::std::string& Waveform_Units_Name(T value) {
  static_assert(::std::is_same<T, Waveform_Units>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to Units_Name().");
  return Waveform_Units_Name(static_cast<Waveform_Units>(value));
}
[[nodiscard]] bool Waveform_Units_Parse(
    ::absl::string_view name, Waveform_Units* PROTOBUF_NONNULL value);
using ::google::protobuf::internal::generated_enum::AbslParseFlag;
using ::google::protobuf::internal::generated_enum::AbslUnparseFlag;

// ===================================================================


// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Waveform_Signal final : public ::google::protobuf::MessageLite
/* @@protoc_insertion_point(class_definition:mixxx.track.io.Waveform.Signal) */ {
 public:
  inline Waveform_Signal() : Waveform_Signal(nullptr) {}
  ~Waveform_Signal() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Waveform_Signal* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Waveform_Signal));
  }
#endif

  template <typename = void>
  explicit constexpr Waveform_Signal(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Waveform_Signal(const Waveform_Signal& from) : Waveform_Signal(nullptr, from) {}
  inline Waveform_Signal(Waveform_Signal&& from) noexcept : Waveform_Signal(nullptr, ::std::move(from)) {}
  inline Waveform_Signal& operator=(const Waveform_Signal& from) {
    CopyFrom(from);
    return *this;
  }
  inline Waveform_Signal& operator=(Waveform_Signal&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::std::string& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString);
  }
  [[nodiscard]] inline ::std::string* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::std::string>();
  }

  [[nodiscard]] static const Waveform_Signal& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Waveform_Signal>(&Waveform_Signal_globals_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(Waveform_Signal& a, Waveform_Signal& b) { a.Swap(&b); }
  inline void Swap(Waveform_Signal* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Waveform_Signal* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Waveform_Signal* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::MessageLite::DefaultConstruct<Waveform_Signal>(arena);
  }
  void CopyFrom(const Waveform_Signal& from);
  void MergeFrom(const Waveform_Signal& from) { Waveform_Signal::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Waveform_Signal* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "mixxx.track.io.Waveform.Signal"; }

  explicit Waveform_Signal(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Waveform_Signal(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Waveform_Signal& from);
  Waveform_Signal(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Waveform_Signal&& from) noexcept
      : Waveform_Signal(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kValueFieldNumber = 1,
    kUnitsFieldNumber = 3,
    kMaxValueFieldNumber = 4,
    kMinValueFieldNumber = 5,
    kChannelsFieldNumber = 2,
  };
  // repeated int32 value = 1;
  [[nodiscard]] int value_size()
      const;
  private:
  int _internal_value_size() const;

  public:
  void clear_value() ;
  [[nodiscard]] ::int32_t value(int index) const;
  void set_value(int index, ::int32_t value);
  void add_value(::int32_t value);
  [[nodiscard]] const ::google::protobuf::RepeatedField<::int32_t>& value()
      const;
  ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL mutable_value();

  private:
  const ::google::protobuf::RepeatedField<::int32_t>& _internal_value() const;
  ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL _internal_mutable_value();

  public:
  // optional .mixxx.track.io.Waveform.Units units = 3 [default = RMS];
  [[nodiscard]] bool has_units()
      const;
  void clear_units() ;
  [[nodiscard]] ::mixxx::track::io::Waveform_Units units() const;
  void set_units(::mixxx::track::io::Waveform_Units value);

  private:
  ::mixxx::track::io::Waveform_Units _internal_units() const;
  void _internal_set_units(::mixxx::track::io::Waveform_Units value);

  public:
  // optional int32 max_value = 4;
  [[nodiscard]] bool has_max_value()
      const;
  void clear_max_value() ;
  [[nodiscard]] ::int32_t max_value() const;
  void set_max_value(::int32_t value);

  private:
  ::int32_t _internal_max_value() const;
  void _internal_set_max_value(::int32_t value);

  public:
  // optional int32 min_value = 5;
  [[nodiscard]] bool has_min_value()
      const;
  void clear_min_value() ;
  [[nodiscard]] ::int32_t min_value() const;
  void set_min_value(::int32_t value);

  private:
  ::int32_t _internal_min_value() const;
  void _internal_set_min_value(::int32_t value);

  public:
  // optional int32 channels = 2 [default = 2];
  [[nodiscard]] bool has_channels()
      const;
  void clear_channels() ;
  [[nodiscard]] ::int32_t channels() const;
  void set_channels(::int32_t value);

  private:
  ::int32_t _internal_channels() const;
  void _internal_set_channels(::int32_t value);

  public:
  // @@protoc_insertion_point(class_scope:mixxx.track.io.Waveform.Signal)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 5,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Waveform_Signal& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedField<::int32_t> value_;
    int units_;
    ::int32_t max_value_;
    ::int32_t min_value_;
    ::int32_t channels_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_waveform_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Waveform_FilteredSignal final : public ::google::protobuf::MessageLite
/* @@protoc_insertion_point(class_definition:mixxx.track.io.Waveform.FilteredSignal) */ {
 public:
  inline Waveform_FilteredSignal() : Waveform_FilteredSignal(nullptr) {}
  ~Waveform_FilteredSignal() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Waveform_FilteredSignal* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Waveform_FilteredSignal));
  }
#endif

  template <typename = void>
  explicit constexpr Waveform_FilteredSignal(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Waveform_FilteredSignal(const Waveform_FilteredSignal& from) : Waveform_FilteredSignal(nullptr, from) {}
  inline Waveform_FilteredSignal(Waveform_FilteredSignal&& from) noexcept : Waveform_FilteredSignal(nullptr, ::std::move(from)) {}
  inline Waveform_FilteredSignal& operator=(const Waveform_FilteredSignal& from) {
    CopyFrom(from);
    return *this;
  }
  inline Waveform_FilteredSignal& operator=(Waveform_FilteredSignal&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::std::string& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString);
  }
  [[nodiscard]] inline ::std::string* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::std::string>();
  }

  [[nodiscard]] static const Waveform_FilteredSignal& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Waveform_FilteredSignal>(&Waveform_FilteredSignal_globals_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(Waveform_FilteredSignal& a, Waveform_FilteredSignal& b) { a.Swap(&b); }
  inline void Swap(Waveform_FilteredSignal* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Waveform_FilteredSignal* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Waveform_FilteredSignal* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::MessageLite::DefaultConstruct<Waveform_FilteredSignal>(arena);
  }
  void CopyFrom(const Waveform_FilteredSignal& from);
  void MergeFrom(const Waveform_FilteredSignal& from) { Waveform_FilteredSignal::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Waveform_FilteredSignal* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "mixxx.track.io.Waveform.FilteredSignal"; }

  explicit Waveform_FilteredSignal(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Waveform_FilteredSignal(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Waveform_FilteredSignal& from);
  Waveform_FilteredSignal(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Waveform_FilteredSignal&& from) noexcept
      : Waveform_FilteredSignal(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kLowFieldNumber = 1,
    kMidFieldNumber = 2,
    kHighFieldNumber = 3,
    kLowCutoffFrequencyFieldNumber = 4,
    kMidLowCutoffFrequencyFieldNumber = 5,
    kMidHighCutoffFrequencyFieldNumber = 6,
    kHighCutoffFrequencyFieldNumber = 7,
  };
  // optional .mixxx.track.io.Waveform.Signal low = 1;
  [[nodiscard]] bool has_low()
      const;
  void clear_low() ;
  [[nodiscard]] const ::mixxx::track::io::Waveform_Signal& low() const;
  [[nodiscard]] ::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE release_low();
  ::mixxx::track::io::Waveform_Signal* PROTOBUF_NONNULL mutable_low();
  void set_allocated_low(::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_low(::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE value);
  ::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE unsafe_arena_release_low();

  private:
  const ::mixxx::track::io::Waveform_Signal& _internal_low() const;
  ::mixxx::track::io::Waveform_Signal* PROTOBUF_NONNULL _internal_mutable_low();

  public:
  // optional .mixxx.track.io.Waveform.Signal mid = 2;
  [[nodiscard]] bool has_mid()
      const;
  void clear_mid() ;
  [[nodiscard]] const ::mixxx::track::io::Waveform_Signal& mid() const;
  [[nodiscard]] ::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE release_mid();
  ::mixxx::track::io::Waveform_Signal* PROTOBUF_NONNULL mutable_mid();
  void set_allocated_mid(::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_mid(::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE value);
  ::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE unsafe_arena_release_mid();

  private:
  const ::mixxx::track::io::Waveform_Signal& _internal_mid() const;
  ::mixxx::track::io::Waveform_Signal* PROTOBUF_NONNULL _internal_mutable_mid();

  public:
  // optional .mixxx.track.io.Waveform.Signal high = 3;
  [[nodiscard]] bool has_high()
      const;
  void clear_high() ;
  [[nodiscard]] const ::mixxx::track::io::Waveform_Signal& high() const;
  [[nodiscard]] ::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE release_high();
  ::mixxx::track::io::Waveform_Signal* PROTOBUF_NONNULL mutable_high();
  void set_allocated_high(::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_high(::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE value);
  ::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE unsafe_arena_release_high();

  private:
  const ::mixxx::track::io::Waveform_Signal& _internal_high() const;
  ::mixxx::track::io::Waveform_Signal* PROTOBUF_NONNULL _internal_mutable_high();

  public:
  // optional double low_cutoff_frequency = 4;
  [[nodiscard]] bool has_low_cutoff_frequency()
      const;
  void clear_low_cutoff_frequency() ;
  [[nodiscard]] double low_cutoff_frequency() const;
  void set_low_cutoff_frequency(double value);

  private:
  double _internal_low_cutoff_frequency() const;
  void _internal_set_low_cutoff_frequency(double value);

  public:
  // optional double mid_low_cutoff_frequency = 5;
  [[nodiscard]] bool has_mid_low_cutoff_frequency()
      const;
  void clear_mid_low_cutoff_frequency() ;
  [[nodiscard]] double mid_low_cutoff_frequency() const;
  void set_mid_low_cutoff_frequency(double value);

  private:
  double _internal_mid_low_cutoff_frequency() const;
  void _internal_set_mid_low_cutoff_frequency(double value);

  public:
  // optional double mid_high_cutoff_frequency = 6;
  [[nodiscard]] bool has_mid_high_cutoff_frequency()
      const;
  void clear_mid_high_cutoff_frequency() ;
  [[nodiscard]] double mid_high_cutoff_frequency() const;
  void set_mid_high_cutoff_frequency(double value);

  private:
  double _internal_mid_high_cutoff_frequency() const;
  void _internal_set_mid_high_cutoff_frequency(double value);

  public:
  // optional double high_cutoff_frequency = 7;
  [[nodiscard]] bool has_high_cutoff_frequency()
      const;
  void clear_high_cutoff_frequency() ;
  [[nodiscard]] double high_cutoff_frequency() const;
  void set_high_cutoff_frequency(double value);

  private:
  double _internal_high_cutoff_frequency() const;
  void _internal_set_high_cutoff_frequency(double value);

  public:
  // @@protoc_insertion_point(class_scope:mixxx.track.io.Waveform.FilteredSignal)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 7,
                          3, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Waveform_FilteredSignal& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE low_;
    ::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE mid_;
    ::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE high_;
    double low_cutoff_frequency_;
    double mid_low_cutoff_frequency_;
    double mid_high_cutoff_frequency_;
    double high_cutoff_frequency_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_waveform_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Waveform final : public ::google::protobuf::MessageLite
/* @@protoc_insertion_point(class_definition:mixxx.track.io.Waveform) */ {
 public:
  inline Waveform() : Waveform(nullptr) {}
  ~Waveform() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Waveform* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Waveform));
  }
#endif

  template <typename = void>
  explicit constexpr Waveform(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Waveform(const Waveform& from) : Waveform(nullptr, from) {}
  inline Waveform(Waveform&& from) noexcept : Waveform(nullptr, ::std::move(from)) {}
  inline Waveform& operator=(const Waveform& from) {
    CopyFrom(from);
    return *this;
  }
  inline Waveform& operator=(Waveform&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::std::string& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString);
  }
  [[nodiscard]] inline ::std::string* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::std::string>();
  }

  [[nodiscard]] static const Waveform& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Waveform>(&Waveform_globals_);
  }
  static constexpr int kIndexInFileMessages = 2;
  friend void swap(Waveform& a, Waveform& b) { a.Swap(&b); }
  inline void Swap(Waveform* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Waveform* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Waveform* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::MessageLite::DefaultConstruct<Waveform>(arena);
  }
  void CopyFrom(const Waveform& from);
  void MergeFrom(const Waveform& from) { Waveform::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Waveform* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "mixxx.track.io.Waveform"; }

  explicit Waveform(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Waveform(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Waveform& from);
  Waveform(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Waveform&& from) noexcept
      : Waveform(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  // nested types ----------------------------------------------------
  using Signal = Waveform_Signal;
  using FilteredSignal = Waveform_FilteredSignal;
  using Units = Waveform_Units;
  static constexpr Units RMS = Waveform_Units_RMS;
  static constexpr Units AMPLITUDE = Waveform_Units_AMPLITUDE;
  [[nodiscard]] static inline bool Units_IsValid(int value) {
    return Waveform_Units_IsValid(value);
  }
  static constexpr Units Units_MIN = Waveform_Units_Units_MIN;
  static constexpr Units Units_MAX = Waveform_Units_Units_MAX;
  static constexpr int Units_ARRAYSIZE = Waveform_Units_Units_ARRAYSIZE;
  template <typename T>
  [[nodiscard]] static inline const ::std::string& Units_Name(T value) {
    return Waveform_Units_Name(value);
  }
  [[nodiscard]] static inline bool Units_Parse(
      ::absl::string_view name, Units* PROTOBUF_NONNULL value) {
    return Waveform_Units_Parse(name, value);
  }

  // accessors -------------------------------------------------------
  enum : int {
    kSignalAllFieldNumber = 3,
    kSignalFilteredFieldNumber = 4,
    kVisualSampleRateFieldNumber = 1,
    kAudioVisualRatioFieldNumber = 2,
  };
  // optional .mixxx.track.io.Waveform.Signal signal_all = 3;
  [[nodiscard]] bool has_signal_all()
      const;
  void clear_signal_all() ;
  [[nodiscard]] const ::mixxx::track::io::Waveform_Signal& signal_all() const;
  [[nodiscard]] ::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE release_signal_all();
  ::mixxx::track::io::Waveform_Signal* PROTOBUF_NONNULL mutable_signal_all();
  void set_allocated_signal_all(::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_signal_all(::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE value);
  ::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE unsafe_arena_release_signal_all();

  private:
  const ::mixxx::track::io::Waveform_Signal& _internal_signal_all() const;
  ::mixxx::track::io::Waveform_Signal* PROTOBUF_NONNULL _internal_mutable_signal_all();

  public:
  // optional .mixxx.track.io.Waveform.FilteredSignal signal_filtered = 4;
  [[nodiscard]] bool has_signal_filtered()
      const;
  void clear_signal_filtered() ;
  [[nodiscard]] const ::mixxx::track::io::Waveform_FilteredSignal& signal_filtered() const;
  [[nodiscard]] ::mixxx::track::io::Waveform_FilteredSignal* PROTOBUF_NULLABLE release_signal_filtered();
  ::mixxx::track::io::Waveform_FilteredSignal* PROTOBUF_NONNULL mutable_signal_filtered();
  void set_allocated_signal_filtered(::mixxx::track::io::Waveform_FilteredSignal* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_signal_filtered(::mixxx::track::io::Waveform_FilteredSignal* PROTOBUF_NULLABLE value);
  ::mixxx::track::io::Waveform_FilteredSignal* PROTOBUF_NULLABLE unsafe_arena_release_signal_filtered();

  private:
  const ::mixxx::track::io::Waveform_FilteredSignal& _internal_signal_filtered() const;
  ::mixxx::track::io::Waveform_FilteredSignal* PROTOBUF_NONNULL _internal_mutable_signal_filtered();

  public:
  // optional double visual_sample_rate = 1;
  [[nodiscard]] bool has_visual_sample_rate()
      const;
  void clear_visual_sample_rate() ;
  [[nodiscard]] double visual_sample_rate() const;
  void set_visual_sample_rate(double value);

  private:
  double _internal_visual_sample_rate() const;
  void _internal_set_visual_sample_rate(double value);

  public:
  // optional double audio_visual_ratio = 2;
  [[nodiscard]] bool has_audio_visual_ratio()
      const;
  void clear_audio_visual_ratio() ;
  [[nodiscard]] double audio_visual_ratio() const;
  void set_audio_visual_ratio(double value);

  private:
  double _internal_audio_visual_ratio() const;
  void _internal_set_audio_visual_ratio(double value);

  public:
  // @@protoc_insertion_point(class_scope:mixxx.track.io.Waveform)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<2, 4,
                          2, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Waveform& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE signal_all_;
    ::mixxx::track::io::Waveform_FilteredSignal* PROTOBUF_NULLABLE signal_filtered_;
    double visual_sample_rate_;
    double audio_visual_ratio_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_waveform_2eproto;
};

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// Waveform_Signal

// repeated int32 value = 1;
inline int Waveform_Signal::_internal_value_size() const {
  return _internal_value().size();
}
inline int Waveform_Signal::value_size() const {
  return _internal_value_size();
}
inline void Waveform_Signal::clear_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.value_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline ::int32_t Waveform_Signal::value(int index) const {
  // @@protoc_insertion_point(field_get:mixxx.track.io.Waveform.Signal.value)
  return _internal_value().Get(index);
}
inline void Waveform_Signal::set_value(int index, ::int32_t value) {
  _internal_mutable_value()->Set(index, value);
  // @@protoc_insertion_point(field_set:mixxx.track.io.Waveform.Signal.value)
}
inline void Waveform_Signal::add_value(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_value()
      ->InternalAddWithArena<const ::google::protobuf::MessageLite*>(
          internal_visibility(), this, value);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:mixxx.track.io.Waveform.Signal.value)
}
inline const ::google::protobuf::RepeatedField<::int32_t>& Waveform_Signal::value() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:mixxx.track.io.Waveform.Signal.value)
  return _internal_value();
}
inline ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL Waveform_Signal::mutable_value()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:mixxx.track.io.Waveform.Signal.value)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_value();
}
inline const ::google::protobuf::RepeatedField<::int32_t>&
Waveform_Signal::_internal_value() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.value_;
}
inline ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL
Waveform_Signal::_internal_mutable_value() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.value_;
}

// optional int32 channels = 2 [default = 2];
inline bool Waveform_Signal::has_channels() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000010U);
  return value;
}
inline void Waveform_Signal::clear_channels() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.channels_ = 2;
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::int32_t Waveform_Signal::channels() const {
  // @@protoc_insertion_point(field_get:mixxx.track.io.Waveform.Signal.channels)
  return _internal_channels();
}
inline void Waveform_Signal::set_channels(::int32_t value) {
  _internal_set_channels(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:mixxx.track.io.Waveform.Signal.channels)
}
inline ::int32_t Waveform_Signal::_internal_channels() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.channels_;
}
inline void Waveform_Signal::_internal_set_channels(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.channels_ = value;
}

// optional .mixxx.track.io.Waveform.Units units = 3 [default = RMS];
inline bool Waveform_Signal::has_units() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  return value;
}
inline void Waveform_Signal::clear_units() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.units_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::mixxx::track::io::Waveform_Units Waveform_Signal::units() const {
  // @@protoc_insertion_point(field_get:mixxx.track.io.Waveform.Signal.units)
  return _internal_units();
}
inline void Waveform_Signal::set_units(::mixxx::track::io::Waveform_Units value) {
  _internal_set_units(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:mixxx.track.io.Waveform.Signal.units)
}
inline ::mixxx::track::io::Waveform_Units Waveform_Signal::_internal_units() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::mixxx::track::io::Waveform_Units>(_impl_.units_);
}
inline void Waveform_Signal::_internal_set_units(::mixxx::track::io::Waveform_Units value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);

                                          assert(::google::protobuf::internal::ValidateEnum(
                                              value, ::mixxx::track::io::Waveform_Units_internal_data_));
                                          _impl_.units_ = value;
}

// optional int32 max_value = 4;
inline bool Waveform_Signal::has_max_value() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  return value;
}
inline void Waveform_Signal::clear_max_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.max_value_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline ::int32_t Waveform_Signal::max_value() const {
  // @@protoc_insertion_point(field_get:mixxx.track.io.Waveform.Signal.max_value)
  return _internal_max_value();
}
inline void Waveform_Signal::set_max_value(::int32_t value) {
  _internal_set_max_value(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:mixxx.track.io.Waveform.Signal.max_value)
}
inline ::int32_t Waveform_Signal::_internal_max_value() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.max_value_;
}
inline void Waveform_Signal::_internal_set_max_value(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.max_value_ = value;
}

// optional int32 min_value = 5;
inline bool Waveform_Signal::has_min_value() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000008U);
  return value;
}
inline void Waveform_Signal::clear_min_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.min_value_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline ::int32_t Waveform_Signal::min_value() const {
  // @@protoc_insertion_point(field_get:mixxx.track.io.Waveform.Signal.min_value)
  return _internal_min_value();
}
inline void Waveform_Signal::set_min_value(::int32_t value) {
  _internal_set_min_value(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:mixxx.track.io.Waveform.Signal.min_value)
}
inline ::int32_t Waveform_Signal::_internal_min_value() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.min_value_;
}
inline void Waveform_Signal::_internal_set_min_value(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.min_value_ = value;
}

// -------------------------------------------------------------------

// Waveform_FilteredSignal

// optional .mixxx.track.io.Waveform.Signal low = 1;
inline bool Waveform_FilteredSignal::has_low() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.low_ != nullptr);
  return value;
}
inline void Waveform_FilteredSignal::clear_low() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.low_ != nullptr) _impl_.low_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::mixxx::track::io::Waveform_Signal& Waveform_FilteredSignal::_internal_low() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::mixxx::track::io::Waveform_Signal* p = _impl_.low_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::mixxx::track::io::Waveform_Signal>(&::mixxx::track::io::Waveform_Signal_globals_);
}
inline const ::mixxx::track::io::Waveform_Signal& Waveform_FilteredSignal::low() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:mixxx.track.io.Waveform.FilteredSignal.low)
  return _internal_low();
}
inline void Waveform_FilteredSignal::unsafe_arena_set_allocated_low(
    ::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.low_);
  }
  _impl_.low_ = reinterpret_cast<::mixxx::track::io::Waveform_Signal*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:mixxx.track.io.Waveform.FilteredSignal.low)
}
inline ::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE Waveform_FilteredSignal::release_low() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::mixxx::track::io::Waveform_Signal* released = _impl_.low_;
  _impl_.low_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE Waveform_FilteredSignal::unsafe_arena_release_low() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:mixxx.track.io.Waveform.FilteredSignal.low)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::mixxx::track::io::Waveform_Signal* temp = _impl_.low_;
  _impl_.low_ = nullptr;
  return temp;
}
inline ::mixxx::track::io::Waveform_Signal* PROTOBUF_NONNULL Waveform_FilteredSignal::_internal_mutable_low() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.low_ == nullptr) {
    auto* p = ::google::protobuf::MessageLite::DefaultConstruct<::mixxx::track::io::Waveform_Signal>(GetArena());
    _impl_.low_ = reinterpret_cast<::mixxx::track::io::Waveform_Signal*>(p);
  }
  return _impl_.low_;
}
inline ::mixxx::track::io::Waveform_Signal* PROTOBUF_NONNULL Waveform_FilteredSignal::mutable_low()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::mixxx::track::io::Waveform_Signal* _msg = _internal_mutable_low();
  // @@protoc_insertion_point(field_mutable:mixxx.track.io.Waveform.FilteredSignal.low)
  return _msg;
}
inline void Waveform_FilteredSignal::set_allocated_low(::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.low_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.low_ = reinterpret_cast<::mixxx::track::io::Waveform_Signal*>(value);
  // @@protoc_insertion_point(field_set_allocated:mixxx.track.io.Waveform.FilteredSignal.low)
}

// optional .mixxx.track.io.Waveform.Signal mid = 2;
inline bool Waveform_FilteredSignal::has_mid() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.mid_ != nullptr);
  return value;
}
inline void Waveform_FilteredSignal::clear_mid() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.mid_ != nullptr) _impl_.mid_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::mixxx::track::io::Waveform_Signal& Waveform_FilteredSignal::_internal_mid() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::mixxx::track::io::Waveform_Signal* p = _impl_.mid_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::mixxx::track::io::Waveform_Signal>(&::mixxx::track::io::Waveform_Signal_globals_);
}
inline const ::mixxx::track::io::Waveform_Signal& Waveform_FilteredSignal::mid() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:mixxx.track.io.Waveform.FilteredSignal.mid)
  return _internal_mid();
}
inline void Waveform_FilteredSignal::unsafe_arena_set_allocated_mid(
    ::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.mid_);
  }
  _impl_.mid_ = reinterpret_cast<::mixxx::track::io::Waveform_Signal*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:mixxx.track.io.Waveform.FilteredSignal.mid)
}
inline ::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE Waveform_FilteredSignal::release_mid() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::mixxx::track::io::Waveform_Signal* released = _impl_.mid_;
  _impl_.mid_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE Waveform_FilteredSignal::unsafe_arena_release_mid() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:mixxx.track.io.Waveform.FilteredSignal.mid)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::mixxx::track::io::Waveform_Signal* temp = _impl_.mid_;
  _impl_.mid_ = nullptr;
  return temp;
}
inline ::mixxx::track::io::Waveform_Signal* PROTOBUF_NONNULL Waveform_FilteredSignal::_internal_mutable_mid() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.mid_ == nullptr) {
    auto* p = ::google::protobuf::MessageLite::DefaultConstruct<::mixxx::track::io::Waveform_Signal>(GetArena());
    _impl_.mid_ = reinterpret_cast<::mixxx::track::io::Waveform_Signal*>(p);
  }
  return _impl_.mid_;
}
inline ::mixxx::track::io::Waveform_Signal* PROTOBUF_NONNULL Waveform_FilteredSignal::mutable_mid()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::mixxx::track::io::Waveform_Signal* _msg = _internal_mutable_mid();
  // @@protoc_insertion_point(field_mutable:mixxx.track.io.Waveform.FilteredSignal.mid)
  return _msg;
}
inline void Waveform_FilteredSignal::set_allocated_mid(::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.mid_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.mid_ = reinterpret_cast<::mixxx::track::io::Waveform_Signal*>(value);
  // @@protoc_insertion_point(field_set_allocated:mixxx.track.io.Waveform.FilteredSignal.mid)
}

// optional .mixxx.track.io.Waveform.Signal high = 3;
inline bool Waveform_FilteredSignal::has_high() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.high_ != nullptr);
  return value;
}
inline void Waveform_FilteredSignal::clear_high() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.high_ != nullptr) _impl_.high_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline const ::mixxx::track::io::Waveform_Signal& Waveform_FilteredSignal::_internal_high() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::mixxx::track::io::Waveform_Signal* p = _impl_.high_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::mixxx::track::io::Waveform_Signal>(&::mixxx::track::io::Waveform_Signal_globals_);
}
inline const ::mixxx::track::io::Waveform_Signal& Waveform_FilteredSignal::high() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:mixxx.track.io.Waveform.FilteredSignal.high)
  return _internal_high();
}
inline void Waveform_FilteredSignal::unsafe_arena_set_allocated_high(
    ::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.high_);
  }
  _impl_.high_ = reinterpret_cast<::mixxx::track::io::Waveform_Signal*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:mixxx.track.io.Waveform.FilteredSignal.high)
}
inline ::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE Waveform_FilteredSignal::release_high() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::mixxx::track::io::Waveform_Signal* released = _impl_.high_;
  _impl_.high_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE Waveform_FilteredSignal::unsafe_arena_release_high() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:mixxx.track.io.Waveform.FilteredSignal.high)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::mixxx::track::io::Waveform_Signal* temp = _impl_.high_;
  _impl_.high_ = nullptr;
  return temp;
}
inline ::mixxx::track::io::Waveform_Signal* PROTOBUF_NONNULL Waveform_FilteredSignal::_internal_mutable_high() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.high_ == nullptr) {
    auto* p = ::google::protobuf::MessageLite::DefaultConstruct<::mixxx::track::io::Waveform_Signal>(GetArena());
    _impl_.high_ = reinterpret_cast<::mixxx::track::io::Waveform_Signal*>(p);
  }
  return _impl_.high_;
}
inline ::mixxx::track::io::Waveform_Signal* PROTOBUF_NONNULL Waveform_FilteredSignal::mutable_high()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::mixxx::track::io::Waveform_Signal* _msg = _internal_mutable_high();
  // @@protoc_insertion_point(field_mutable:mixxx.track.io.Waveform.FilteredSignal.high)
  return _msg;
}
inline void Waveform_FilteredSignal::set_allocated_high(::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.high_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.high_ = reinterpret_cast<::mixxx::track::io::Waveform_Signal*>(value);
  // @@protoc_insertion_point(field_set_allocated:mixxx.track.io.Waveform.FilteredSignal.high)
}

// optional double low_cutoff_frequency = 4;
inline bool Waveform_FilteredSignal::has_low_cutoff_frequency() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000008U);
  return value;
}
inline void Waveform_FilteredSignal::clear_low_cutoff_frequency() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.low_cutoff_frequency_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline double Waveform_FilteredSignal::low_cutoff_frequency() const {
  // @@protoc_insertion_point(field_get:mixxx.track.io.Waveform.FilteredSignal.low_cutoff_frequency)
  return _internal_low_cutoff_frequency();
}
inline void Waveform_FilteredSignal::set_low_cutoff_frequency(double value) {
  _internal_set_low_cutoff_frequency(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:mixxx.track.io.Waveform.FilteredSignal.low_cutoff_frequency)
}
inline double Waveform_FilteredSignal::_internal_low_cutoff_frequency() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.low_cutoff_frequency_;
}
inline void Waveform_FilteredSignal::_internal_set_low_cutoff_frequency(double value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.low_cutoff_frequency_ = value;
}

// optional double mid_low_cutoff_frequency = 5;
inline bool Waveform_FilteredSignal::has_mid_low_cutoff_frequency() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000010U);
  return value;
}
inline void Waveform_FilteredSignal::clear_mid_low_cutoff_frequency() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.mid_low_cutoff_frequency_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline double Waveform_FilteredSignal::mid_low_cutoff_frequency() const {
  // @@protoc_insertion_point(field_get:mixxx.track.io.Waveform.FilteredSignal.mid_low_cutoff_frequency)
  return _internal_mid_low_cutoff_frequency();
}
inline void Waveform_FilteredSignal::set_mid_low_cutoff_frequency(double value) {
  _internal_set_mid_low_cutoff_frequency(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:mixxx.track.io.Waveform.FilteredSignal.mid_low_cutoff_frequency)
}
inline double Waveform_FilteredSignal::_internal_mid_low_cutoff_frequency() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.mid_low_cutoff_frequency_;
}
inline void Waveform_FilteredSignal::_internal_set_mid_low_cutoff_frequency(double value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.mid_low_cutoff_frequency_ = value;
}

// optional double mid_high_cutoff_frequency = 6;
inline bool Waveform_FilteredSignal::has_mid_high_cutoff_frequency() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000020U);
  return value;
}
inline void Waveform_FilteredSignal::clear_mid_high_cutoff_frequency() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.mid_high_cutoff_frequency_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline double Waveform_FilteredSignal::mid_high_cutoff_frequency() const {
  // @@protoc_insertion_point(field_get:mixxx.track.io.Waveform.FilteredSignal.mid_high_cutoff_frequency)
  return _internal_mid_high_cutoff_frequency();
}
inline void Waveform_FilteredSignal::set_mid_high_cutoff_frequency(double value) {
  _internal_set_mid_high_cutoff_frequency(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:mixxx.track.io.Waveform.FilteredSignal.mid_high_cutoff_frequency)
}
inline double Waveform_FilteredSignal::_internal_mid_high_cutoff_frequency() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.mid_high_cutoff_frequency_;
}
inline void Waveform_FilteredSignal::_internal_set_mid_high_cutoff_frequency(double value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.mid_high_cutoff_frequency_ = value;
}

// optional double high_cutoff_frequency = 7;
inline bool Waveform_FilteredSignal::has_high_cutoff_frequency() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000040U);
  return value;
}
inline void Waveform_FilteredSignal::clear_high_cutoff_frequency() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.high_cutoff_frequency_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
}
inline double Waveform_FilteredSignal::high_cutoff_frequency() const {
  // @@protoc_insertion_point(field_get:mixxx.track.io.Waveform.FilteredSignal.high_cutoff_frequency)
  return _internal_high_cutoff_frequency();
}
inline void Waveform_FilteredSignal::set_high_cutoff_frequency(double value) {
  _internal_set_high_cutoff_frequency(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_set:mixxx.track.io.Waveform.FilteredSignal.high_cutoff_frequency)
}
inline double Waveform_FilteredSignal::_internal_high_cutoff_frequency() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.high_cutoff_frequency_;
}
inline void Waveform_FilteredSignal::_internal_set_high_cutoff_frequency(double value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.high_cutoff_frequency_ = value;
}

// -------------------------------------------------------------------

// Waveform

// optional double visual_sample_rate = 1;
inline bool Waveform::has_visual_sample_rate() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  return value;
}
inline void Waveform::clear_visual_sample_rate() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.visual_sample_rate_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline double Waveform::visual_sample_rate() const {
  // @@protoc_insertion_point(field_get:mixxx.track.io.Waveform.visual_sample_rate)
  return _internal_visual_sample_rate();
}
inline void Waveform::set_visual_sample_rate(double value) {
  _internal_set_visual_sample_rate(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:mixxx.track.io.Waveform.visual_sample_rate)
}
inline double Waveform::_internal_visual_sample_rate() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.visual_sample_rate_;
}
inline void Waveform::_internal_set_visual_sample_rate(double value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.visual_sample_rate_ = value;
}

// optional double audio_visual_ratio = 2;
inline bool Waveform::has_audio_visual_ratio() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000008U);
  return value;
}
inline void Waveform::clear_audio_visual_ratio() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.audio_visual_ratio_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline double Waveform::audio_visual_ratio() const {
  // @@protoc_insertion_point(field_get:mixxx.track.io.Waveform.audio_visual_ratio)
  return _internal_audio_visual_ratio();
}
inline void Waveform::set_audio_visual_ratio(double value) {
  _internal_set_audio_visual_ratio(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:mixxx.track.io.Waveform.audio_visual_ratio)
}
inline double Waveform::_internal_audio_visual_ratio() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.audio_visual_ratio_;
}
inline void Waveform::_internal_set_audio_visual_ratio(double value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.audio_visual_ratio_ = value;
}

// optional .mixxx.track.io.Waveform.Signal signal_all = 3;
inline bool Waveform::has_signal_all() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.signal_all_ != nullptr);
  return value;
}
inline void Waveform::clear_signal_all() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.signal_all_ != nullptr) _impl_.signal_all_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::mixxx::track::io::Waveform_Signal& Waveform::_internal_signal_all() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::mixxx::track::io::Waveform_Signal* p = _impl_.signal_all_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::mixxx::track::io::Waveform_Signal>(&::mixxx::track::io::Waveform_Signal_globals_);
}
inline const ::mixxx::track::io::Waveform_Signal& Waveform::signal_all() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:mixxx.track.io.Waveform.signal_all)
  return _internal_signal_all();
}
inline void Waveform::unsafe_arena_set_allocated_signal_all(
    ::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.signal_all_);
  }
  _impl_.signal_all_ = reinterpret_cast<::mixxx::track::io::Waveform_Signal*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:mixxx.track.io.Waveform.signal_all)
}
inline ::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE Waveform::release_signal_all() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::mixxx::track::io::Waveform_Signal* released = _impl_.signal_all_;
  _impl_.signal_all_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE Waveform::unsafe_arena_release_signal_all() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:mixxx.track.io.Waveform.signal_all)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::mixxx::track::io::Waveform_Signal* temp = _impl_.signal_all_;
  _impl_.signal_all_ = nullptr;
  return temp;
}
inline ::mixxx::track::io::Waveform_Signal* PROTOBUF_NONNULL Waveform::_internal_mutable_signal_all() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.signal_all_ == nullptr) {
    auto* p = ::google::protobuf::MessageLite::DefaultConstruct<::mixxx::track::io::Waveform_Signal>(GetArena());
    _impl_.signal_all_ = reinterpret_cast<::mixxx::track::io::Waveform_Signal*>(p);
  }
  return _impl_.signal_all_;
}
inline ::mixxx::track::io::Waveform_Signal* PROTOBUF_NONNULL Waveform::mutable_signal_all()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::mixxx::track::io::Waveform_Signal* _msg = _internal_mutable_signal_all();
  // @@protoc_insertion_point(field_mutable:mixxx.track.io.Waveform.signal_all)
  return _msg;
}
inline void Waveform::set_allocated_signal_all(::mixxx::track::io::Waveform_Signal* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.signal_all_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.signal_all_ = reinterpret_cast<::mixxx::track::io::Waveform_Signal*>(value);
  // @@protoc_insertion_point(field_set_allocated:mixxx.track.io.Waveform.signal_all)
}

// optional .mixxx.track.io.Waveform.FilteredSignal signal_filtered = 4;
inline bool Waveform::has_signal_filtered() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.signal_filtered_ != nullptr);
  return value;
}
inline void Waveform::clear_signal_filtered() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.signal_filtered_ != nullptr) _impl_.signal_filtered_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::mixxx::track::io::Waveform_FilteredSignal& Waveform::_internal_signal_filtered() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::mixxx::track::io::Waveform_FilteredSignal* p = _impl_.signal_filtered_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::mixxx::track::io::Waveform_FilteredSignal>(&::mixxx::track::io::Waveform_FilteredSignal_globals_);
}
inline const ::mixxx::track::io::Waveform_FilteredSignal& Waveform::signal_filtered() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:mixxx.track.io.Waveform.signal_filtered)
  return _internal_signal_filtered();
}
inline void Waveform::unsafe_arena_set_allocated_signal_filtered(
    ::mixxx::track::io::Waveform_FilteredSignal* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.signal_filtered_);
  }
  _impl_.signal_filtered_ = reinterpret_cast<::mixxx::track::io::Waveform_FilteredSignal*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:mixxx.track.io.Waveform.signal_filtered)
}
inline ::mixxx::track::io::Waveform_FilteredSignal* PROTOBUF_NULLABLE Waveform::release_signal_filtered() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::mixxx::track::io::Waveform_FilteredSignal* released = _impl_.signal_filtered_;
  _impl_.signal_filtered_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::mixxx::track::io::Waveform_FilteredSignal* PROTOBUF_NULLABLE Waveform::unsafe_arena_release_signal_filtered() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:mixxx.track.io.Waveform.signal_filtered)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::mixxx::track::io::Waveform_FilteredSignal* temp = _impl_.signal_filtered_;
  _impl_.signal_filtered_ = nullptr;
  return temp;
}
inline ::mixxx::track::io::Waveform_FilteredSignal* PROTOBUF_NONNULL Waveform::_internal_mutable_signal_filtered() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.signal_filtered_ == nullptr) {
    auto* p = ::google::protobuf::MessageLite::DefaultConstruct<::mixxx::track::io::Waveform_FilteredSignal>(GetArena());
    _impl_.signal_filtered_ = reinterpret_cast<::mixxx::track::io::Waveform_FilteredSignal*>(p);
  }
  return _impl_.signal_filtered_;
}
inline ::mixxx::track::io::Waveform_FilteredSignal* PROTOBUF_NONNULL Waveform::mutable_signal_filtered()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::mixxx::track::io::Waveform_FilteredSignal* _msg = _internal_mutable_signal_filtered();
  // @@protoc_insertion_point(field_mutable:mixxx.track.io.Waveform.signal_filtered)
  return _msg;
}
inline void Waveform::set_allocated_signal_filtered(::mixxx::track::io::Waveform_FilteredSignal* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.signal_filtered_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.signal_filtered_ = reinterpret_cast<::mixxx::track::io::Waveform_FilteredSignal*>(value);
  // @@protoc_insertion_point(field_set_allocated:mixxx.track.io.Waveform.signal_filtered)
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace io
}  // namespace track
}  // namespace mixxx


namespace google {
namespace protobuf {

template <>
struct is_proto_enum<::mixxx::track::io::Waveform_Units> : std::true_type {};
template <>
struct internal::LiteEnumFuncs<::mixxx::track::io::Waveform_Units> {
  static constexpr bool kIsDefined = true;
  static constexpr auto kParseFunc = ::mixxx::track::io::Waveform_Units_Parse;
  static constexpr auto kNameFunc = ::mixxx::track::io::Waveform_Units_Name<int>;
};

}  // namespace protobuf
}  // namespace google

// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"
// clang-format on

#endif  // waveform_2eproto_2epb_2eh
