/********************************************************************************
** Form generated from reading UI file 'dlgprefkeydlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLGPREFKEYDLG_H
#define UI_DLGPREFKEYDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DlgPrefKeyDlg
{
public:
    QVBoxLayout *verticalLayout;
    QCheckBox *banalyzerenabled;
    QGroupBox *groupPlugin;
    QHBoxLayout *horizontalLayout;
    QComboBox *plugincombo;
    QGroupBox *groupOptions;
    QVBoxLayout *verticalLayout_2;
    QCheckBox *bfastAnalysisEnabled;
    QCheckBox *breanalyzeEnabled;
    QGroupBox *groupNotation;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *notationChoice1;
    QRadioButton *radioNotationLancelot;
    QRadioButton *radioNotationLancelotAndTraditional;
    QRadioButton *radioNotationOpenKey;
    QHBoxLayout *notationChoice2;
    QRadioButton *radioNotationOpenKeyAndTraditional;
    QRadioButton *radioNotationTraditional;
    QRadioButton *radioNotationCustom;
    QGridLayout *customNotation;
    QLabel *a_major_label;
    QLineEdit *a_major_edit;
    QLabel *b_flat_major_label;
    QLineEdit *b_flat_major_edit;
    QLabel *b_major_label;
    QLineEdit *b_major_edit;
    QLabel *c_major_label;
    QLineEdit *c_major_edit;
    QLabel *d_flat_major_label;
    QLineEdit *d_flat_major_edit;
    QLabel *d_major_label;
    QLineEdit *d_major_edit;
    QLabel *e_flat_major_label;
    QLineEdit *e_flat_major_edit;
    QLabel *e_major_label;
    QLineEdit *e_major_edit;
    QLabel *f_major_label;
    QLineEdit *f_major_edit;
    QLabel *f_sharp_major_label;
    QLineEdit *f_sharp_major_edit;
    QLabel *g_major_label;
    QLineEdit *g_major_edit;
    QLabel *a_flat_major_label;
    QLineEdit *a_flat_major_edit;
    QLabel *a_minor_label;
    QLineEdit *a_minor_edit;
    QLabel *b_flat_minor_label;
    QLineEdit *b_flat_minor_edit;
    QLabel *b_minor_label;
    QLineEdit *b_minor_edit;
    QLabel *c_minor_label;
    QLineEdit *c_minor_edit;
    QLabel *c_sharp_minor_label;
    QLineEdit *c_sharp_minor_edit;
    QLabel *d_minor_label;
    QLineEdit *d_minor_edit;
    QLabel *e_flat_minor_label;
    QLineEdit *e_flat_minor_edit;
    QLabel *e_minor_label;
    QLineEdit *e_minor_edit;
    QLabel *f_minor_label;
    QLineEdit *f_minor_edit;
    QLabel *f_sharp_minor_label;
    QLineEdit *f_sharp_minor_edit;
    QLabel *g_minor_label;
    QLineEdit *g_minor_edit;
    QLabel *g_sharp_minor_label;
    QLineEdit *g_sharp_minor_edit;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *DlgPrefKeyDlg)
    {
        if (DlgPrefKeyDlg->objectName().isEmpty())
            DlgPrefKeyDlg->setObjectName("DlgPrefKeyDlg");
        DlgPrefKeyDlg->resize(536, 500);
        verticalLayout = new QVBoxLayout(DlgPrefKeyDlg);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        banalyzerenabled = new QCheckBox(DlgPrefKeyDlg);
        banalyzerenabled->setObjectName("banalyzerenabled");

        verticalLayout->addWidget(banalyzerenabled);

        groupPlugin = new QGroupBox(DlgPrefKeyDlg);
        groupPlugin->setObjectName("groupPlugin");
        groupPlugin->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        horizontalLayout = new QHBoxLayout(groupPlugin);
        horizontalLayout->setSpacing(6);
        horizontalLayout->setContentsMargins(11, 11, 11, 11);
        horizontalLayout->setObjectName("horizontalLayout");
        plugincombo = new QComboBox(groupPlugin);
        plugincombo->setObjectName("plugincombo");
        plugincombo->setMinimumSize(QSize(200, 0));
        plugincombo->setMaximumSize(QSize(2600, 26));
        plugincombo->setLayoutDirection(Qt::LeftToRight);

        horizontalLayout->addWidget(plugincombo);


        verticalLayout->addWidget(groupPlugin);

        groupOptions = new QGroupBox(DlgPrefKeyDlg);
        groupOptions->setObjectName("groupOptions");
        verticalLayout_2 = new QVBoxLayout(groupOptions);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        bfastAnalysisEnabled = new QCheckBox(groupOptions);
        bfastAnalysisEnabled->setObjectName("bfastAnalysisEnabled");

        verticalLayout_2->addWidget(bfastAnalysisEnabled);

        breanalyzeEnabled = new QCheckBox(groupOptions);
        breanalyzeEnabled->setObjectName("breanalyzeEnabled");

        verticalLayout_2->addWidget(breanalyzeEnabled);


        verticalLayout->addWidget(groupOptions);

        groupNotation = new QGroupBox(DlgPrefKeyDlg);
        groupNotation->setObjectName("groupNotation");
        groupNotation->setMinimumSize(QSize(409, 200));
        verticalLayout_3 = new QVBoxLayout(groupNotation);
        verticalLayout_3->setSpacing(6);
        verticalLayout_3->setContentsMargins(11, 11, 11, 11);
        verticalLayout_3->setObjectName("verticalLayout_3");
        notationChoice1 = new QHBoxLayout();
        notationChoice1->setSpacing(6);
        notationChoice1->setObjectName("notationChoice1");
        radioNotationLancelot = new QRadioButton(groupNotation);
        radioNotationLancelot->setObjectName("radioNotationLancelot");

        notationChoice1->addWidget(radioNotationLancelot);

        radioNotationLancelotAndTraditional = new QRadioButton(groupNotation);
        radioNotationLancelotAndTraditional->setObjectName("radioNotationLancelotAndTraditional");

        notationChoice1->addWidget(radioNotationLancelotAndTraditional);

        radioNotationOpenKey = new QRadioButton(groupNotation);
        radioNotationOpenKey->setObjectName("radioNotationOpenKey");

        notationChoice1->addWidget(radioNotationOpenKey);


        verticalLayout_3->addLayout(notationChoice1);

        notationChoice2 = new QHBoxLayout();
        notationChoice2->setSpacing(6);
        notationChoice2->setObjectName("notationChoice2");
        radioNotationOpenKeyAndTraditional = new QRadioButton(groupNotation);
        radioNotationOpenKeyAndTraditional->setObjectName("radioNotationOpenKeyAndTraditional");

        notationChoice2->addWidget(radioNotationOpenKeyAndTraditional);

        radioNotationTraditional = new QRadioButton(groupNotation);
        radioNotationTraditional->setObjectName("radioNotationTraditional");

        notationChoice2->addWidget(radioNotationTraditional);

        radioNotationCustom = new QRadioButton(groupNotation);
        radioNotationCustom->setObjectName("radioNotationCustom");

        notationChoice2->addWidget(radioNotationCustom);


        verticalLayout_3->addLayout(notationChoice2);

        customNotation = new QGridLayout();
        customNotation->setSpacing(6);
        customNotation->setObjectName("customNotation");
        a_major_label = new QLabel(groupNotation);
        a_major_label->setObjectName("a_major_label");
        a_major_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        customNotation->addWidget(a_major_label, 0, 0, 1, 1);

        a_major_edit = new QLineEdit(groupNotation);
        a_major_edit->setObjectName("a_major_edit");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(a_major_edit->sizePolicy().hasHeightForWidth());
        a_major_edit->setSizePolicy(sizePolicy);
        a_major_edit->setMaxLength(20);

        customNotation->addWidget(a_major_edit, 0, 1, 1, 1);

        b_flat_major_label = new QLabel(groupNotation);
        b_flat_major_label->setObjectName("b_flat_major_label");
        b_flat_major_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        customNotation->addWidget(b_flat_major_label, 0, 2, 1, 1);

        b_flat_major_edit = new QLineEdit(groupNotation);
        b_flat_major_edit->setObjectName("b_flat_major_edit");
        sizePolicy.setHeightForWidth(b_flat_major_edit->sizePolicy().hasHeightForWidth());
        b_flat_major_edit->setSizePolicy(sizePolicy);
        b_flat_major_edit->setMaxLength(20);

        customNotation->addWidget(b_flat_major_edit, 0, 3, 1, 1);

        b_major_label = new QLabel(groupNotation);
        b_major_label->setObjectName("b_major_label");
        b_major_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        customNotation->addWidget(b_major_label, 0, 4, 1, 1);

        b_major_edit = new QLineEdit(groupNotation);
        b_major_edit->setObjectName("b_major_edit");
        sizePolicy.setHeightForWidth(b_major_edit->sizePolicy().hasHeightForWidth());
        b_major_edit->setSizePolicy(sizePolicy);
        b_major_edit->setMaxLength(20);

        customNotation->addWidget(b_major_edit, 0, 5, 1, 1);

        c_major_label = new QLabel(groupNotation);
        c_major_label->setObjectName("c_major_label");
        c_major_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        customNotation->addWidget(c_major_label, 0, 6, 1, 1);

        c_major_edit = new QLineEdit(groupNotation);
        c_major_edit->setObjectName("c_major_edit");
        sizePolicy.setHeightForWidth(c_major_edit->sizePolicy().hasHeightForWidth());
        c_major_edit->setSizePolicy(sizePolicy);
        c_major_edit->setMaxLength(20);

        customNotation->addWidget(c_major_edit, 0, 7, 1, 1);

        d_flat_major_label = new QLabel(groupNotation);
        d_flat_major_label->setObjectName("d_flat_major_label");
        d_flat_major_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        customNotation->addWidget(d_flat_major_label, 0, 8, 1, 1);

        d_flat_major_edit = new QLineEdit(groupNotation);
        d_flat_major_edit->setObjectName("d_flat_major_edit");
        sizePolicy.setHeightForWidth(d_flat_major_edit->sizePolicy().hasHeightForWidth());
        d_flat_major_edit->setSizePolicy(sizePolicy);
        d_flat_major_edit->setMaxLength(20);

        customNotation->addWidget(d_flat_major_edit, 0, 9, 1, 1);

        d_major_label = new QLabel(groupNotation);
        d_major_label->setObjectName("d_major_label");
        d_major_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        customNotation->addWidget(d_major_label, 0, 10, 1, 1);

        d_major_edit = new QLineEdit(groupNotation);
        d_major_edit->setObjectName("d_major_edit");
        sizePolicy.setHeightForWidth(d_major_edit->sizePolicy().hasHeightForWidth());
        d_major_edit->setSizePolicy(sizePolicy);
        d_major_edit->setMaxLength(20);

        customNotation->addWidget(d_major_edit, 0, 11, 1, 1);

        e_flat_major_label = new QLabel(groupNotation);
        e_flat_major_label->setObjectName("e_flat_major_label");
        e_flat_major_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        customNotation->addWidget(e_flat_major_label, 1, 0, 1, 1);

        e_flat_major_edit = new QLineEdit(groupNotation);
        e_flat_major_edit->setObjectName("e_flat_major_edit");
        sizePolicy.setHeightForWidth(e_flat_major_edit->sizePolicy().hasHeightForWidth());
        e_flat_major_edit->setSizePolicy(sizePolicy);
        e_flat_major_edit->setMaxLength(20);

        customNotation->addWidget(e_flat_major_edit, 1, 1, 1, 1);

        e_major_label = new QLabel(groupNotation);
        e_major_label->setObjectName("e_major_label");
        e_major_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        customNotation->addWidget(e_major_label, 1, 2, 1, 1);

        e_major_edit = new QLineEdit(groupNotation);
        e_major_edit->setObjectName("e_major_edit");
        sizePolicy.setHeightForWidth(e_major_edit->sizePolicy().hasHeightForWidth());
        e_major_edit->setSizePolicy(sizePolicy);
        e_major_edit->setMaxLength(20);

        customNotation->addWidget(e_major_edit, 1, 3, 1, 1);

        f_major_label = new QLabel(groupNotation);
        f_major_label->setObjectName("f_major_label");
        f_major_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        customNotation->addWidget(f_major_label, 1, 4, 1, 1);

        f_major_edit = new QLineEdit(groupNotation);
        f_major_edit->setObjectName("f_major_edit");
        sizePolicy.setHeightForWidth(f_major_edit->sizePolicy().hasHeightForWidth());
        f_major_edit->setSizePolicy(sizePolicy);
        f_major_edit->setMaxLength(20);

        customNotation->addWidget(f_major_edit, 1, 5, 1, 1);

        f_sharp_major_label = new QLabel(groupNotation);
        f_sharp_major_label->setObjectName("f_sharp_major_label");
        f_sharp_major_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        customNotation->addWidget(f_sharp_major_label, 1, 6, 1, 1);

        f_sharp_major_edit = new QLineEdit(groupNotation);
        f_sharp_major_edit->setObjectName("f_sharp_major_edit");
        sizePolicy.setHeightForWidth(f_sharp_major_edit->sizePolicy().hasHeightForWidth());
        f_sharp_major_edit->setSizePolicy(sizePolicy);
        f_sharp_major_edit->setMaxLength(20);

        customNotation->addWidget(f_sharp_major_edit, 1, 7, 1, 1);

        g_major_label = new QLabel(groupNotation);
        g_major_label->setObjectName("g_major_label");
        g_major_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        customNotation->addWidget(g_major_label, 1, 8, 1, 1);

        g_major_edit = new QLineEdit(groupNotation);
        g_major_edit->setObjectName("g_major_edit");
        sizePolicy.setHeightForWidth(g_major_edit->sizePolicy().hasHeightForWidth());
        g_major_edit->setSizePolicy(sizePolicy);
        g_major_edit->setMaxLength(20);

        customNotation->addWidget(g_major_edit, 1, 9, 1, 1);

        a_flat_major_label = new QLabel(groupNotation);
        a_flat_major_label->setObjectName("a_flat_major_label");
        a_flat_major_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        customNotation->addWidget(a_flat_major_label, 1, 10, 1, 1);

        a_flat_major_edit = new QLineEdit(groupNotation);
        a_flat_major_edit->setObjectName("a_flat_major_edit");
        sizePolicy.setHeightForWidth(a_flat_major_edit->sizePolicy().hasHeightForWidth());
        a_flat_major_edit->setSizePolicy(sizePolicy);
        a_flat_major_edit->setMaxLength(20);

        customNotation->addWidget(a_flat_major_edit, 1, 11, 1, 1);

        a_minor_label = new QLabel(groupNotation);
        a_minor_label->setObjectName("a_minor_label");
        a_minor_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        customNotation->addWidget(a_minor_label, 2, 0, 1, 1);

        a_minor_edit = new QLineEdit(groupNotation);
        a_minor_edit->setObjectName("a_minor_edit");
        sizePolicy.setHeightForWidth(a_minor_edit->sizePolicy().hasHeightForWidth());
        a_minor_edit->setSizePolicy(sizePolicy);
        a_minor_edit->setMaxLength(20);

        customNotation->addWidget(a_minor_edit, 2, 1, 1, 1);

        b_flat_minor_label = new QLabel(groupNotation);
        b_flat_minor_label->setObjectName("b_flat_minor_label");
        b_flat_minor_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        customNotation->addWidget(b_flat_minor_label, 2, 2, 1, 1);

        b_flat_minor_edit = new QLineEdit(groupNotation);
        b_flat_minor_edit->setObjectName("b_flat_minor_edit");
        sizePolicy.setHeightForWidth(b_flat_minor_edit->sizePolicy().hasHeightForWidth());
        b_flat_minor_edit->setSizePolicy(sizePolicy);
        b_flat_minor_edit->setMaxLength(20);

        customNotation->addWidget(b_flat_minor_edit, 2, 3, 1, 1);

        b_minor_label = new QLabel(groupNotation);
        b_minor_label->setObjectName("b_minor_label");
        b_minor_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        customNotation->addWidget(b_minor_label, 2, 4, 1, 1);

        b_minor_edit = new QLineEdit(groupNotation);
        b_minor_edit->setObjectName("b_minor_edit");
        sizePolicy.setHeightForWidth(b_minor_edit->sizePolicy().hasHeightForWidth());
        b_minor_edit->setSizePolicy(sizePolicy);
        b_minor_edit->setMaxLength(20);

        customNotation->addWidget(b_minor_edit, 2, 5, 1, 1);

        c_minor_label = new QLabel(groupNotation);
        c_minor_label->setObjectName("c_minor_label");
        c_minor_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        customNotation->addWidget(c_minor_label, 2, 6, 1, 1);

        c_minor_edit = new QLineEdit(groupNotation);
        c_minor_edit->setObjectName("c_minor_edit");
        sizePolicy.setHeightForWidth(c_minor_edit->sizePolicy().hasHeightForWidth());
        c_minor_edit->setSizePolicy(sizePolicy);
        c_minor_edit->setMaxLength(20);

        customNotation->addWidget(c_minor_edit, 2, 7, 1, 1);

        c_sharp_minor_label = new QLabel(groupNotation);
        c_sharp_minor_label->setObjectName("c_sharp_minor_label");
        c_sharp_minor_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        customNotation->addWidget(c_sharp_minor_label, 2, 8, 1, 1);

        c_sharp_minor_edit = new QLineEdit(groupNotation);
        c_sharp_minor_edit->setObjectName("c_sharp_minor_edit");
        sizePolicy.setHeightForWidth(c_sharp_minor_edit->sizePolicy().hasHeightForWidth());
        c_sharp_minor_edit->setSizePolicy(sizePolicy);
        c_sharp_minor_edit->setMaxLength(20);

        customNotation->addWidget(c_sharp_minor_edit, 2, 9, 1, 1);

        d_minor_label = new QLabel(groupNotation);
        d_minor_label->setObjectName("d_minor_label");
        d_minor_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        customNotation->addWidget(d_minor_label, 2, 10, 1, 1);

        d_minor_edit = new QLineEdit(groupNotation);
        d_minor_edit->setObjectName("d_minor_edit");
        sizePolicy.setHeightForWidth(d_minor_edit->sizePolicy().hasHeightForWidth());
        d_minor_edit->setSizePolicy(sizePolicy);
        d_minor_edit->setMaxLength(20);

        customNotation->addWidget(d_minor_edit, 2, 11, 1, 1);

        e_flat_minor_label = new QLabel(groupNotation);
        e_flat_minor_label->setObjectName("e_flat_minor_label");
        e_flat_minor_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        customNotation->addWidget(e_flat_minor_label, 3, 0, 1, 1);

        e_flat_minor_edit = new QLineEdit(groupNotation);
        e_flat_minor_edit->setObjectName("e_flat_minor_edit");
        sizePolicy.setHeightForWidth(e_flat_minor_edit->sizePolicy().hasHeightForWidth());
        e_flat_minor_edit->setSizePolicy(sizePolicy);
        e_flat_minor_edit->setMaxLength(20);

        customNotation->addWidget(e_flat_minor_edit, 3, 1, 1, 1);

        e_minor_label = new QLabel(groupNotation);
        e_minor_label->setObjectName("e_minor_label");
        e_minor_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        customNotation->addWidget(e_minor_label, 3, 2, 1, 1);

        e_minor_edit = new QLineEdit(groupNotation);
        e_minor_edit->setObjectName("e_minor_edit");
        sizePolicy.setHeightForWidth(e_minor_edit->sizePolicy().hasHeightForWidth());
        e_minor_edit->setSizePolicy(sizePolicy);
        e_minor_edit->setMaxLength(20);

        customNotation->addWidget(e_minor_edit, 3, 3, 1, 1);

        f_minor_label = new QLabel(groupNotation);
        f_minor_label->setObjectName("f_minor_label");
        f_minor_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        customNotation->addWidget(f_minor_label, 3, 4, 1, 1);

        f_minor_edit = new QLineEdit(groupNotation);
        f_minor_edit->setObjectName("f_minor_edit");
        sizePolicy.setHeightForWidth(f_minor_edit->sizePolicy().hasHeightForWidth());
        f_minor_edit->setSizePolicy(sizePolicy);
        f_minor_edit->setMaxLength(20);

        customNotation->addWidget(f_minor_edit, 3, 5, 1, 1);

        f_sharp_minor_label = new QLabel(groupNotation);
        f_sharp_minor_label->setObjectName("f_sharp_minor_label");
        f_sharp_minor_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        customNotation->addWidget(f_sharp_minor_label, 3, 6, 1, 1);

        f_sharp_minor_edit = new QLineEdit(groupNotation);
        f_sharp_minor_edit->setObjectName("f_sharp_minor_edit");
        sizePolicy.setHeightForWidth(f_sharp_minor_edit->sizePolicy().hasHeightForWidth());
        f_sharp_minor_edit->setSizePolicy(sizePolicy);
        f_sharp_minor_edit->setMaxLength(20);

        customNotation->addWidget(f_sharp_minor_edit, 3, 7, 1, 1);

        g_minor_label = new QLabel(groupNotation);
        g_minor_label->setObjectName("g_minor_label");
        g_minor_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        customNotation->addWidget(g_minor_label, 3, 8, 1, 1);

        g_minor_edit = new QLineEdit(groupNotation);
        g_minor_edit->setObjectName("g_minor_edit");
        sizePolicy.setHeightForWidth(g_minor_edit->sizePolicy().hasHeightForWidth());
        g_minor_edit->setSizePolicy(sizePolicy);
        g_minor_edit->setMaxLength(20);

        customNotation->addWidget(g_minor_edit, 3, 9, 1, 1);

        g_sharp_minor_label = new QLabel(groupNotation);
        g_sharp_minor_label->setObjectName("g_sharp_minor_label");
        g_sharp_minor_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        customNotation->addWidget(g_sharp_minor_label, 3, 10, 1, 1);

        g_sharp_minor_edit = new QLineEdit(groupNotation);
        g_sharp_minor_edit->setObjectName("g_sharp_minor_edit");
        sizePolicy.setHeightForWidth(g_sharp_minor_edit->sizePolicy().hasHeightForWidth());
        g_sharp_minor_edit->setSizePolicy(sizePolicy);
        g_sharp_minor_edit->setMaxLength(20);

        customNotation->addWidget(g_sharp_minor_edit, 3, 11, 1, 1);


        verticalLayout_3->addLayout(customNotation);


        verticalLayout->addWidget(groupNotation);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);


        retranslateUi(DlgPrefKeyDlg);

        QMetaObject::connectSlotsByName(DlgPrefKeyDlg);
    } // setupUi

    void retranslateUi(QWidget *DlgPrefKeyDlg)
    {
        DlgPrefKeyDlg->setWindowTitle(QCoreApplication::translate("DlgPrefKeyDlg", "Key Notation Format Settings", nullptr));
#if QT_CONFIG(tooltip)
        banalyzerenabled->setToolTip(QCoreApplication::translate("DlgPrefKeyDlg", "When key detection is enabled, Mixxx detects the musical key of your tracks\n"
"and allows you to pitch adjust them for harmonic mixing.", nullptr));
#endif // QT_CONFIG(tooltip)
        banalyzerenabled->setText(QCoreApplication::translate("DlgPrefKeyDlg", "Enable Key Detection", nullptr));
        groupPlugin->setTitle(QCoreApplication::translate("DlgPrefKeyDlg", "Choose Analyzer", nullptr));
#if QT_CONFIG(tooltip)
        plugincombo->setToolTip(QCoreApplication::translate("DlgPrefKeyDlg", "Choose between different algorithms to detect keys.", nullptr));
#endif // QT_CONFIG(tooltip)
        groupOptions->setTitle(QCoreApplication::translate("DlgPrefKeyDlg", "Analyzer Settings", nullptr));
        bfastAnalysisEnabled->setText(QCoreApplication::translate("DlgPrefKeyDlg", "Enable Fast Analysis (For slow computers, may be less accurate)", nullptr));
        breanalyzeEnabled->setText(QCoreApplication::translate("DlgPrefKeyDlg", "Re-analyze keys when settings change or 3rd-party keys are present", nullptr));
        groupNotation->setTitle(QCoreApplication::translate("DlgPrefKeyDlg", "Key Notation", nullptr));
        radioNotationLancelot->setText(QCoreApplication::translate("DlgPrefKeyDlg", "Lancelot", nullptr));
        radioNotationLancelotAndTraditional->setText(QCoreApplication::translate("DlgPrefKeyDlg", "Lancelot/Traditional", nullptr));
        radioNotationOpenKey->setText(QCoreApplication::translate("DlgPrefKeyDlg", "OpenKey", nullptr));
        radioNotationOpenKeyAndTraditional->setText(QCoreApplication::translate("DlgPrefKeyDlg", "OpenKey/Traditional", nullptr));
        radioNotationTraditional->setText(QCoreApplication::translate("DlgPrefKeyDlg", "Traditional", nullptr));
        radioNotationCustom->setText(QCoreApplication::translate("DlgPrefKeyDlg", "Custom", nullptr));
        a_major_label->setText(QCoreApplication::translate("DlgPrefKeyDlg", "A", nullptr));
        b_flat_major_label->setText(QCoreApplication::translate("DlgPrefKeyDlg", "Bb", nullptr));
        b_major_label->setText(QCoreApplication::translate("DlgPrefKeyDlg", "B", nullptr));
        c_major_label->setText(QCoreApplication::translate("DlgPrefKeyDlg", "C", nullptr));
        d_flat_major_label->setText(QCoreApplication::translate("DlgPrefKeyDlg", "Db", nullptr));
        d_major_label->setText(QCoreApplication::translate("DlgPrefKeyDlg", "D", nullptr));
        e_flat_major_label->setText(QCoreApplication::translate("DlgPrefKeyDlg", "Eb", nullptr));
        e_major_label->setText(QCoreApplication::translate("DlgPrefKeyDlg", "E", nullptr));
        f_major_label->setText(QCoreApplication::translate("DlgPrefKeyDlg", "F", nullptr));
        f_sharp_major_label->setText(QCoreApplication::translate("DlgPrefKeyDlg", "F#", nullptr));
        g_major_label->setText(QCoreApplication::translate("DlgPrefKeyDlg", "G", nullptr));
        a_flat_major_label->setText(QCoreApplication::translate("DlgPrefKeyDlg", "Ab", nullptr));
        a_minor_label->setText(QCoreApplication::translate("DlgPrefKeyDlg", "Am", nullptr));
        b_flat_minor_label->setText(QCoreApplication::translate("DlgPrefKeyDlg", "Bbm", nullptr));
        b_minor_label->setText(QCoreApplication::translate("DlgPrefKeyDlg", "Bm", nullptr));
        c_minor_label->setText(QCoreApplication::translate("DlgPrefKeyDlg", "Cm", nullptr));
        c_sharp_minor_label->setText(QCoreApplication::translate("DlgPrefKeyDlg", "C#m", nullptr));
        d_minor_label->setText(QCoreApplication::translate("DlgPrefKeyDlg", "Dm", nullptr));
        e_flat_minor_label->setText(QCoreApplication::translate("DlgPrefKeyDlg", "Ebm", nullptr));
        e_minor_label->setText(QCoreApplication::translate("DlgPrefKeyDlg", "Em", nullptr));
        f_minor_label->setText(QCoreApplication::translate("DlgPrefKeyDlg", "Fm", nullptr));
        f_sharp_minor_label->setText(QCoreApplication::translate("DlgPrefKeyDlg", "F#m", nullptr));
        g_minor_label->setText(QCoreApplication::translate("DlgPrefKeyDlg", "Gm", nullptr));
        g_sharp_minor_label->setText(QCoreApplication::translate("DlgPrefKeyDlg", "G#m", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgPrefKeyDlg: public Ui_DlgPrefKeyDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLGPREFKEYDLG_H
