// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: beats.proto
// Protobuf C++ Version: 7.35.0

#ifndef beats_2eproto_2epb_2eh
#define beats_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

// clang-format off
#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 7035000
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/generated_enum_util.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_beats_2eproto

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct TableStruct_beats_2eproto {
  static const ::uint32_t offsets[];
};
namespace mixxx {
namespace track {
namespace io {
enum Source : int;
extern const uint32_t Source_internal_data_[];
class Beat;
struct BeatGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern BeatGlobalsTypeInternal Beat_globals_;
extern const ::google::protobuf::internal::ClassDataLite Beat_class_data_;
#else
extern const BeatGlobalsTypeInternal Beat_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class BeatGrid;
struct BeatGridGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern BeatGridGlobalsTypeInternal BeatGrid_globals_;
extern const ::google::protobuf::internal::ClassDataLite BeatGrid_class_data_;
#else
extern const BeatGridGlobalsTypeInternal BeatGrid_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class BeatMap;
struct BeatMapGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern BeatMapGlobalsTypeInternal BeatMap_globals_;
extern const ::google::protobuf::internal::ClassDataLite BeatMap_class_data_;
#else
extern const BeatMapGlobalsTypeInternal BeatMap_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Bpm;
struct BpmGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern BpmGlobalsTypeInternal Bpm_globals_;
extern const ::google::protobuf::internal::ClassDataLite Bpm_class_data_;
#else
extern const BpmGlobalsTypeInternal Bpm_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
}  // namespace io
}  // namespace track
}  // namespace mixxx
namespace google {
namespace protobuf {
template <>
internal::EnumTraitsT<::mixxx::track::io::Source_internal_data_>
    internal::EnumTraitsImpl::value<::mixxx::track::io::Source>;
}  // namespace protobuf
}  // namespace google

namespace mixxx {
namespace track {
namespace io {
enum Source : int {
  ANALYZER = 0,
  FILE_METADATA = 1,
  USER = 2,
};

extern const uint32_t Source_internal_data_[];
inline constexpr Source Source_MIN =
    static_cast<Source>(0);
inline constexpr Source Source_MAX =
    static_cast<Source>(2);
[[nodiscard]] inline bool Source_IsValid(int value) {
  return 0 <= value && value <= 2;
}
inline constexpr int Source_ARRAYSIZE = 2 + 1;
[[nodiscard]] const ::std::string& Source_Name(Source value);
template <typename T>
[[nodiscard]] const ::std::string& Source_Name(T value) {
  static_assert(::std::is_same<T, Source>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to Source_Name().");
  return Source_Name(static_cast<Source>(value));
}
[[nodiscard]] bool Source_Parse(
    ::absl::string_view name, Source* PROTOBUF_NONNULL value);
using ::google::protobuf::internal::generated_enum::AbslParseFlag;
using ::google::protobuf::internal::generated_enum::AbslUnparseFlag;

// ===================================================================


// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Bpm final : public ::google::protobuf::MessageLite
/* @@protoc_insertion_point(class_definition:mixxx.track.io.Bpm) */ {
 public:
  inline Bpm() : Bpm(nullptr) {}
  ~Bpm() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Bpm* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Bpm));
  }
#endif

  template <typename = void>
  explicit constexpr Bpm(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Bpm(const Bpm& from) : Bpm(nullptr, from) {}
  inline Bpm(Bpm&& from) noexcept : Bpm(nullptr, ::std::move(from)) {}
  inline Bpm& operator=(const Bpm& from) {
    CopyFrom(from);
    return *this;
  }
  inline Bpm& operator=(Bpm&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::std::string& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString);
  }
  [[nodiscard]] inline ::std::string* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::std::string>();
  }

  [[nodiscard]] static const Bpm& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Bpm>(&Bpm_globals_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(Bpm& a, Bpm& b) { a.Swap(&b); }
  inline void Swap(Bpm* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Bpm* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Bpm* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::MessageLite::DefaultConstruct<Bpm>(arena);
  }
  void CopyFrom(const Bpm& from);
  void MergeFrom(const Bpm& from) { Bpm::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Bpm* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "mixxx.track.io.Bpm"; }

  explicit Bpm(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Bpm(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Bpm& from);
  Bpm(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Bpm&& from) noexcept
      : Bpm(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kBpmFieldNumber = 1,
    kSourceFieldNumber = 2,
  };
  // optional double bpm = 1;
  [[nodiscard]] bool has_bpm()
      const;
  void clear_bpm() ;
  [[nodiscard]] double bpm() const;
  void set_bpm(double value);

  private:
  double _internal_bpm() const;
  void _internal_set_bpm(double value);

  public:
  // optional .mixxx.track.io.Source source = 2 [default = ANALYZER];
  [[nodiscard]] bool has_source()
      const;
  void clear_source() ;
  [[nodiscard]] ::mixxx::track::io::Source source() const;
  void set_source(::mixxx::track::io::Source value);

  private:
  ::mixxx::track::io::Source _internal_source() const;
  void _internal_set_source(::mixxx::track::io::Source value);

  public:
  // @@protoc_insertion_point(class_scope:mixxx.track.io.Bpm)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<1, 2,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Bpm& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    double bpm_;
    int source_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_beats_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Beat final : public ::google::protobuf::MessageLite
/* @@protoc_insertion_point(class_definition:mixxx.track.io.Beat) */ {
 public:
  inline Beat() : Beat(nullptr) {}
  ~Beat() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Beat* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Beat));
  }
#endif

  template <typename = void>
  explicit constexpr Beat(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Beat(const Beat& from) : Beat(nullptr, from) {}
  inline Beat(Beat&& from) noexcept : Beat(nullptr, ::std::move(from)) {}
  inline Beat& operator=(const Beat& from) {
    CopyFrom(from);
    return *this;
  }
  inline Beat& operator=(Beat&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::std::string& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString);
  }
  [[nodiscard]] inline ::std::string* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::std::string>();
  }

  [[nodiscard]] static const Beat& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Beat>(&Beat_globals_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(Beat& a, Beat& b) { a.Swap(&b); }
  inline void Swap(Beat* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Beat* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Beat* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::MessageLite::DefaultConstruct<Beat>(arena);
  }
  void CopyFrom(const Beat& from);
  void MergeFrom(const Beat& from) { Beat::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Beat* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "mixxx.track.io.Beat"; }

  explicit Beat(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Beat(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Beat& from);
  Beat(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Beat&& from) noexcept
      : Beat(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kFramePositionFieldNumber = 1,
    kSourceFieldNumber = 3,
    kEnabledFieldNumber = 2,
  };
  // optional int32 frame_position = 1;
  [[nodiscard]] bool has_frame_position()
      const;
  void clear_frame_position() ;
  [[nodiscard]] ::int32_t frame_position() const;
  void set_frame_position(::int32_t value);

  private:
  ::int32_t _internal_frame_position() const;
  void _internal_set_frame_position(::int32_t value);

  public:
  // optional .mixxx.track.io.Source source = 3 [default = ANALYZER];
  [[nodiscard]] bool has_source()
      const;
  void clear_source() ;
  [[nodiscard]] ::mixxx::track::io::Source source() const;
  void set_source(::mixxx::track::io::Source value);

  private:
  ::mixxx::track::io::Source _internal_source() const;
  void _internal_set_source(::mixxx::track::io::Source value);

  public:
  // optional bool enabled = 2 [default = true];
  [[nodiscard]] bool has_enabled()
      const;
  void clear_enabled() ;
  [[nodiscard]] bool enabled() const;
  void set_enabled(bool value);

  private:
  bool _internal_enabled() const;
  void _internal_set_enabled(bool value);

  public:
  // @@protoc_insertion_point(class_scope:mixxx.track.io.Beat)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<2, 3,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Beat& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::int32_t frame_position_;
    int source_;
    bool enabled_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_beats_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED BeatMap final : public ::google::protobuf::MessageLite
/* @@protoc_insertion_point(class_definition:mixxx.track.io.BeatMap) */ {
 public:
  inline BeatMap() : BeatMap(nullptr) {}
  ~BeatMap() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(BeatMap* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(BeatMap));
  }
#endif

  template <typename = void>
  explicit constexpr BeatMap(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline BeatMap(const BeatMap& from) : BeatMap(nullptr, from) {}
  inline BeatMap(BeatMap&& from) noexcept : BeatMap(nullptr, ::std::move(from)) {}
  inline BeatMap& operator=(const BeatMap& from) {
    CopyFrom(from);
    return *this;
  }
  inline BeatMap& operator=(BeatMap&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::std::string& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString);
  }
  [[nodiscard]] inline ::std::string* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::std::string>();
  }

  [[nodiscard]] static const BeatMap& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<BeatMap>(&BeatMap_globals_);
  }
  static constexpr int kIndexInFileMessages = 2;
  friend void swap(BeatMap& a, BeatMap& b) { a.Swap(&b); }
  inline void Swap(BeatMap* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(BeatMap* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] BeatMap* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::MessageLite::DefaultConstruct<BeatMap>(arena);
  }
  void CopyFrom(const BeatMap& from);
  void MergeFrom(const BeatMap& from) { BeatMap::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(BeatMap* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "mixxx.track.io.BeatMap"; }

  explicit BeatMap(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  BeatMap(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const BeatMap& from);
  BeatMap(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, BeatMap&& from) noexcept
      : BeatMap(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kBeatFieldNumber = 1,
  };
  // repeated .mixxx.track.io.Beat beat = 1;
  [[nodiscard]] int beat_size()
      const;
  private:
  int _internal_beat_size() const;

  public:
  void clear_beat() ;
  [[nodiscard]] const ::mixxx::track::io::Beat& beat(int index) const;
  [[nodiscard]] ::mixxx::track::io::Beat* PROTOBUF_NONNULL mutable_beat(int index);
  ::mixxx::track::io::Beat* PROTOBUF_NONNULL add_beat();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::mixxx::track::io::Beat>&
  beat() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::mixxx::track::io::Beat>* PROTOBUF_NONNULL
  mutable_beat();

  private:
  const ::google::protobuf::RepeatedPtrField<::mixxx::track::io::Beat>& _internal_beat() const;
  ::google::protobuf::RepeatedPtrField<::mixxx::track::io::Beat>* PROTOBUF_NONNULL _internal_mutable_beat();

  public:
  // @@protoc_insertion_point(class_scope:mixxx.track.io.BeatMap)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 1,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const BeatMap& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::mixxx::track::io::Beat > beat_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_beats_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED BeatGrid final : public ::google::protobuf::MessageLite
/* @@protoc_insertion_point(class_definition:mixxx.track.io.BeatGrid) */ {
 public:
  inline BeatGrid() : BeatGrid(nullptr) {}
  ~BeatGrid() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(BeatGrid* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(BeatGrid));
  }
#endif

  template <typename = void>
  explicit constexpr BeatGrid(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline BeatGrid(const BeatGrid& from) : BeatGrid(nullptr, from) {}
  inline BeatGrid(BeatGrid&& from) noexcept : BeatGrid(nullptr, ::std::move(from)) {}
  inline BeatGrid& operator=(const BeatGrid& from) {
    CopyFrom(from);
    return *this;
  }
  inline BeatGrid& operator=(BeatGrid&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::std::string& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString);
  }
  [[nodiscard]] inline ::std::string* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::std::string>();
  }

  [[nodiscard]] static const BeatGrid& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<BeatGrid>(&BeatGrid_globals_);
  }
  static constexpr int kIndexInFileMessages = 3;
  friend void swap(BeatGrid& a, BeatGrid& b) { a.Swap(&b); }
  inline void Swap(BeatGrid* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(BeatGrid* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] BeatGrid* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::MessageLite::DefaultConstruct<BeatGrid>(arena);
  }
  void CopyFrom(const BeatGrid& from);
  void MergeFrom(const BeatGrid& from) { BeatGrid::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(BeatGrid* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "mixxx.track.io.BeatGrid"; }

  explicit BeatGrid(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  BeatGrid(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const BeatGrid& from);
  BeatGrid(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, BeatGrid&& from) noexcept
      : BeatGrid(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kBpmFieldNumber = 1,
    kFirstBeatFieldNumber = 2,
  };
  // optional .mixxx.track.io.Bpm bpm = 1;
  [[nodiscard]] bool has_bpm()
      const;
  void clear_bpm() ;
  [[nodiscard]] const ::mixxx::track::io::Bpm& bpm() const;
  [[nodiscard]] ::mixxx::track::io::Bpm* PROTOBUF_NULLABLE release_bpm();
  ::mixxx::track::io::Bpm* PROTOBUF_NONNULL mutable_bpm();
  void set_allocated_bpm(::mixxx::track::io::Bpm* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_bpm(::mixxx::track::io::Bpm* PROTOBUF_NULLABLE value);
  ::mixxx::track::io::Bpm* PROTOBUF_NULLABLE unsafe_arena_release_bpm();

  private:
  const ::mixxx::track::io::Bpm& _internal_bpm() const;
  ::mixxx::track::io::Bpm* PROTOBUF_NONNULL _internal_mutable_bpm();

  public:
  // optional .mixxx.track.io.Beat first_beat = 2;
  [[nodiscard]] bool has_first_beat()
      const;
  void clear_first_beat() ;
  [[nodiscard]] const ::mixxx::track::io::Beat& first_beat() const;
  [[nodiscard]] ::mixxx::track::io::Beat* PROTOBUF_NULLABLE release_first_beat();
  ::mixxx::track::io::Beat* PROTOBUF_NONNULL mutable_first_beat();
  void set_allocated_first_beat(::mixxx::track::io::Beat* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_first_beat(::mixxx::track::io::Beat* PROTOBUF_NULLABLE value);
  ::mixxx::track::io::Beat* PROTOBUF_NULLABLE unsafe_arena_release_first_beat();

  private:
  const ::mixxx::track::io::Beat& _internal_first_beat() const;
  ::mixxx::track::io::Beat* PROTOBUF_NONNULL _internal_mutable_first_beat();

  public:
  // @@protoc_insertion_point(class_scope:mixxx.track.io.BeatGrid)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<1, 2,
                          2, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const BeatGrid& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::mixxx::track::io::Bpm* PROTOBUF_NULLABLE bpm_;
    ::mixxx::track::io::Beat* PROTOBUF_NULLABLE first_beat_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_beats_2eproto;
};

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// Beat

// optional int32 frame_position = 1;
inline bool Beat::has_frame_position() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  return value;
}
inline void Beat::clear_frame_position() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.frame_position_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline ::int32_t Beat::frame_position() const {
  // @@protoc_insertion_point(field_get:mixxx.track.io.Beat.frame_position)
  return _internal_frame_position();
}
inline void Beat::set_frame_position(::int32_t value) {
  _internal_set_frame_position(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_set:mixxx.track.io.Beat.frame_position)
}
inline ::int32_t Beat::_internal_frame_position() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.frame_position_;
}
inline void Beat::_internal_set_frame_position(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.frame_position_ = value;
}

// optional bool enabled = 2 [default = true];
inline bool Beat::has_enabled() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  return value;
}
inline void Beat::clear_enabled() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.enabled_ = true;
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline bool Beat::enabled() const {
  // @@protoc_insertion_point(field_get:mixxx.track.io.Beat.enabled)
  return _internal_enabled();
}
inline void Beat::set_enabled(bool value) {
  _internal_set_enabled(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:mixxx.track.io.Beat.enabled)
}
inline bool Beat::_internal_enabled() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.enabled_;
}
inline void Beat::_internal_set_enabled(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.enabled_ = value;
}

// optional .mixxx.track.io.Source source = 3 [default = ANALYZER];
inline bool Beat::has_source() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  return value;
}
inline void Beat::clear_source() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.source_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::mixxx::track::io::Source Beat::source() const {
  // @@protoc_insertion_point(field_get:mixxx.track.io.Beat.source)
  return _internal_source();
}
inline void Beat::set_source(::mixxx::track::io::Source value) {
  _internal_set_source(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:mixxx.track.io.Beat.source)
}
inline ::mixxx::track::io::Source Beat::_internal_source() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::mixxx::track::io::Source>(_impl_.source_);
}
inline void Beat::_internal_set_source(::mixxx::track::io::Source value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);

                                          assert(::google::protobuf::internal::ValidateEnum(
                                              value, ::mixxx::track::io::Source_internal_data_));
                                          _impl_.source_ = value;
}

// -------------------------------------------------------------------

// Bpm

// optional double bpm = 1;
inline bool Bpm::has_bpm() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  return value;
}
inline void Bpm::clear_bpm() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.bpm_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline double Bpm::bpm() const {
  // @@protoc_insertion_point(field_get:mixxx.track.io.Bpm.bpm)
  return _internal_bpm();
}
inline void Bpm::set_bpm(double value) {
  _internal_set_bpm(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_set:mixxx.track.io.Bpm.bpm)
}
inline double Bpm::_internal_bpm() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.bpm_;
}
inline void Bpm::_internal_set_bpm(double value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.bpm_ = value;
}

// optional .mixxx.track.io.Source source = 2 [default = ANALYZER];
inline bool Bpm::has_source() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  return value;
}
inline void Bpm::clear_source() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.source_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::mixxx::track::io::Source Bpm::source() const {
  // @@protoc_insertion_point(field_get:mixxx.track.io.Bpm.source)
  return _internal_source();
}
inline void Bpm::set_source(::mixxx::track::io::Source value) {
  _internal_set_source(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:mixxx.track.io.Bpm.source)
}
inline ::mixxx::track::io::Source Bpm::_internal_source() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::mixxx::track::io::Source>(_impl_.source_);
}
inline void Bpm::_internal_set_source(::mixxx::track::io::Source value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);

                                          assert(::google::protobuf::internal::ValidateEnum(
                                              value, ::mixxx::track::io::Source_internal_data_));
                                          _impl_.source_ = value;
}

// -------------------------------------------------------------------

// BeatMap

// repeated .mixxx.track.io.Beat beat = 1;
inline int BeatMap::_internal_beat_size() const {
  return _internal_beat().size();
}
inline int BeatMap::beat_size() const {
  return _internal_beat_size();
}
inline void BeatMap::clear_beat() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.beat_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::mixxx::track::io::Beat& BeatMap::beat(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:mixxx.track.io.BeatMap.beat)
  return _internal_beat().Get(index);
}
inline ::mixxx::track::io::Beat* PROTOBUF_NONNULL BeatMap::mutable_beat(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:mixxx.track.io.BeatMap.beat)
  return _internal_mutable_beat()->Mutable(index);
}
inline ::mixxx::track::io::Beat* PROTOBUF_NONNULL BeatMap::add_beat()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::mixxx::track::io::Beat* _add =
      _internal_mutable_beat()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:mixxx.track.io.BeatMap.beat)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::mixxx::track::io::Beat>& BeatMap::beat() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:mixxx.track.io.BeatMap.beat)
  return _internal_beat();
}
inline ::google::protobuf::RepeatedPtrField<::mixxx::track::io::Beat>* PROTOBUF_NONNULL
BeatMap::mutable_beat() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:mixxx.track.io.BeatMap.beat)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_beat();
}
inline const ::google::protobuf::RepeatedPtrField<::mixxx::track::io::Beat>&
BeatMap::_internal_beat() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.beat_;
}
inline ::google::protobuf::RepeatedPtrField<::mixxx::track::io::Beat>* PROTOBUF_NONNULL
BeatMap::_internal_mutable_beat() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.beat_;
}

// -------------------------------------------------------------------

// BeatGrid

// optional .mixxx.track.io.Bpm bpm = 1;
inline bool BeatGrid::has_bpm() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.bpm_ != nullptr);
  return value;
}
inline void BeatGrid::clear_bpm() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.bpm_ != nullptr) _impl_.bpm_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::mixxx::track::io::Bpm& BeatGrid::_internal_bpm() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::mixxx::track::io::Bpm* p = _impl_.bpm_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::mixxx::track::io::Bpm>(&::mixxx::track::io::Bpm_globals_);
}
inline const ::mixxx::track::io::Bpm& BeatGrid::bpm() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:mixxx.track.io.BeatGrid.bpm)
  return _internal_bpm();
}
inline void BeatGrid::unsafe_arena_set_allocated_bpm(
    ::mixxx::track::io::Bpm* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.bpm_);
  }
  _impl_.bpm_ = reinterpret_cast<::mixxx::track::io::Bpm*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:mixxx.track.io.BeatGrid.bpm)
}
inline ::mixxx::track::io::Bpm* PROTOBUF_NULLABLE BeatGrid::release_bpm() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::mixxx::track::io::Bpm* released = _impl_.bpm_;
  _impl_.bpm_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::mixxx::track::io::Bpm* PROTOBUF_NULLABLE BeatGrid::unsafe_arena_release_bpm() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:mixxx.track.io.BeatGrid.bpm)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::mixxx::track::io::Bpm* temp = _impl_.bpm_;
  _impl_.bpm_ = nullptr;
  return temp;
}
inline ::mixxx::track::io::Bpm* PROTOBUF_NONNULL BeatGrid::_internal_mutable_bpm() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.bpm_ == nullptr) {
    auto* p = ::google::protobuf::MessageLite::DefaultConstruct<::mixxx::track::io::Bpm>(GetArena());
    _impl_.bpm_ = reinterpret_cast<::mixxx::track::io::Bpm*>(p);
  }
  return _impl_.bpm_;
}
inline ::mixxx::track::io::Bpm* PROTOBUF_NONNULL BeatGrid::mutable_bpm()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::mixxx::track::io::Bpm* _msg = _internal_mutable_bpm();
  // @@protoc_insertion_point(field_mutable:mixxx.track.io.BeatGrid.bpm)
  return _msg;
}
inline void BeatGrid::set_allocated_bpm(::mixxx::track::io::Bpm* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.bpm_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.bpm_ = reinterpret_cast<::mixxx::track::io::Bpm*>(value);
  // @@protoc_insertion_point(field_set_allocated:mixxx.track.io.BeatGrid.bpm)
}

// optional .mixxx.track.io.Beat first_beat = 2;
inline bool BeatGrid::has_first_beat() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.first_beat_ != nullptr);
  return value;
}
inline void BeatGrid::clear_first_beat() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.first_beat_ != nullptr) _impl_.first_beat_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::mixxx::track::io::Beat& BeatGrid::_internal_first_beat() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::mixxx::track::io::Beat* p = _impl_.first_beat_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::mixxx::track::io::Beat>(&::mixxx::track::io::Beat_globals_);
}
inline const ::mixxx::track::io::Beat& BeatGrid::first_beat() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:mixxx.track.io.BeatGrid.first_beat)
  return _internal_first_beat();
}
inline void BeatGrid::unsafe_arena_set_allocated_first_beat(
    ::mixxx::track::io::Beat* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.first_beat_);
  }
  _impl_.first_beat_ = reinterpret_cast<::mixxx::track::io::Beat*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:mixxx.track.io.BeatGrid.first_beat)
}
inline ::mixxx::track::io::Beat* PROTOBUF_NULLABLE BeatGrid::release_first_beat() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::mixxx::track::io::Beat* released = _impl_.first_beat_;
  _impl_.first_beat_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::mixxx::track::io::Beat* PROTOBUF_NULLABLE BeatGrid::unsafe_arena_release_first_beat() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:mixxx.track.io.BeatGrid.first_beat)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::mixxx::track::io::Beat* temp = _impl_.first_beat_;
  _impl_.first_beat_ = nullptr;
  return temp;
}
inline ::mixxx::track::io::Beat* PROTOBUF_NONNULL BeatGrid::_internal_mutable_first_beat() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.first_beat_ == nullptr) {
    auto* p = ::google::protobuf::MessageLite::DefaultConstruct<::mixxx::track::io::Beat>(GetArena());
    _impl_.first_beat_ = reinterpret_cast<::mixxx::track::io::Beat*>(p);
  }
  return _impl_.first_beat_;
}
inline ::mixxx::track::io::Beat* PROTOBUF_NONNULL BeatGrid::mutable_first_beat()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::mixxx::track::io::Beat* _msg = _internal_mutable_first_beat();
  // @@protoc_insertion_point(field_mutable:mixxx.track.io.BeatGrid.first_beat)
  return _msg;
}
inline void BeatGrid::set_allocated_first_beat(::mixxx::track::io::Beat* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.first_beat_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.first_beat_ = reinterpret_cast<::mixxx::track::io::Beat*>(value);
  // @@protoc_insertion_point(field_set_allocated:mixxx.track.io.BeatGrid.first_beat)
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace io
}  // namespace track
}  // namespace mixxx


namespace google {
namespace protobuf {

template <>
struct is_proto_enum<::mixxx::track::io::Source> : std::true_type {};
template <>
struct internal::LiteEnumFuncs<::mixxx::track::io::Source> {
  static constexpr bool kIsDefined = true;
  static constexpr auto kParseFunc = ::mixxx::track::io::Source_Parse;
  static constexpr auto kNameFunc = ::mixxx::track::io::Source_Name<int>;
};

}  // namespace protobuf
}  // namespace google

// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"
// clang-format on

#endif  // beats_2eproto_2epb_2eh
