/********************************************************************************
** Form generated from reading UI file 'dlgreplacecuecolordlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLGREPLACECUECOLORDLG_H
#define UI_DLGREPLACECUECOLORDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>

QT_BEGIN_NAMESPACE

class Ui_DlgReplaceCueColor
{
public:
    QGridLayout *gridLayout;
    QGroupBox *groupBoxConditions;
    QGridLayout *gridLayout_2;
    QSpinBox *spinBoxHotcueIndex;
    QCheckBox *checkBoxHotcueIndexCondition;
    QComboBox *comboBoxHotcueIndexCompare;
    QCheckBox *checkBoxCurrentColorCondition;
    QComboBox *comboBoxCurrentColorCompare;
    QPushButton *pushButtonCurrentColor;
    QFrame *frameReplaceAllColors;
    QHBoxLayout *horizontalLayout;
    QLabel *labelReplaceAllColorsIcon;
    QLabel *labelReplaceAllColorsText;
    QGroupBox *groupBoxReplacement;
    QGridLayout *gridLayout_3;
    QLabel *labelNewColor;
    QPushButton *pushButtonNewColor;
    QDialogButtonBox *buttonBox;
    QSpacerItem *verticalSpacer;

    void setupUi(QDialog *DlgReplaceCueColor)
    {
        if (DlgReplaceCueColor->objectName().isEmpty())
            DlgReplaceCueColor->setObjectName("DlgReplaceCueColor");
        DlgReplaceCueColor->resize(315, 249);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(DlgReplaceCueColor->sizePolicy().hasHeightForWidth());
        DlgReplaceCueColor->setSizePolicy(sizePolicy);
        gridLayout = new QGridLayout(DlgReplaceCueColor);
        gridLayout->setObjectName("gridLayout");
        groupBoxConditions = new QGroupBox(DlgReplaceCueColor);
        groupBoxConditions->setObjectName("groupBoxConditions");
        gridLayout_2 = new QGridLayout(groupBoxConditions);
        gridLayout_2->setObjectName("gridLayout_2");
        spinBoxHotcueIndex = new QSpinBox(groupBoxConditions);
        spinBoxHotcueIndex->setObjectName("spinBoxHotcueIndex");
        spinBoxHotcueIndex->setEnabled(true);
        spinBoxHotcueIndex->setMinimum(1);

        gridLayout_2->addWidget(spinBoxHotcueIndex, 1, 2, 1, 1);

        checkBoxHotcueIndexCondition = new QCheckBox(groupBoxConditions);
        checkBoxHotcueIndexCondition->setObjectName("checkBoxHotcueIndexCondition");
        checkBoxHotcueIndexCondition->setEnabled(true);

        gridLayout_2->addWidget(checkBoxHotcueIndexCondition, 1, 0, 1, 1);

        comboBoxHotcueIndexCompare = new QComboBox(groupBoxConditions);
        comboBoxHotcueIndexCompare->addItem(QString());
        comboBoxHotcueIndexCompare->addItem(QString());
        comboBoxHotcueIndexCompare->setObjectName("comboBoxHotcueIndexCompare");
        comboBoxHotcueIndexCompare->setEnabled(true);

        gridLayout_2->addWidget(comboBoxHotcueIndexCompare, 1, 1, 1, 1);

        checkBoxCurrentColorCondition = new QCheckBox(groupBoxConditions);
        checkBoxCurrentColorCondition->setObjectName("checkBoxCurrentColorCondition");
        checkBoxCurrentColorCondition->setChecked(true);

        gridLayout_2->addWidget(checkBoxCurrentColorCondition, 0, 0, 1, 1);

        comboBoxCurrentColorCompare = new QComboBox(groupBoxConditions);
        comboBoxCurrentColorCompare->addItem(QString());
        comboBoxCurrentColorCompare->addItem(QString());
        comboBoxCurrentColorCompare->setObjectName("comboBoxCurrentColorCompare");

        gridLayout_2->addWidget(comboBoxCurrentColorCompare, 0, 1, 1, 1);

        pushButtonCurrentColor = new QPushButton(groupBoxConditions);
        pushButtonCurrentColor->setObjectName("pushButtonCurrentColor");

        gridLayout_2->addWidget(pushButtonCurrentColor, 0, 2, 1, 1);

        frameReplaceAllColors = new QFrame(groupBoxConditions);
        frameReplaceAllColors->setObjectName("frameReplaceAllColors");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(frameReplaceAllColors->sizePolicy().hasHeightForWidth());
        frameReplaceAllColors->setSizePolicy(sizePolicy1);
        frameReplaceAllColors->setFrameShape(QFrame::NoFrame);
        frameReplaceAllColors->setFrameShadow(QFrame::Plain);
        horizontalLayout = new QHBoxLayout(frameReplaceAllColors);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        labelReplaceAllColorsIcon = new QLabel(frameReplaceAllColors);
        labelReplaceAllColorsIcon->setObjectName("labelReplaceAllColorsIcon");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(labelReplaceAllColorsIcon->sizePolicy().hasHeightForWidth());
        labelReplaceAllColorsIcon->setSizePolicy(sizePolicy2);

        horizontalLayout->addWidget(labelReplaceAllColorsIcon);

        labelReplaceAllColorsText = new QLabel(frameReplaceAllColors);
        labelReplaceAllColorsText->setObjectName("labelReplaceAllColorsText");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Minimum);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(labelReplaceAllColorsText->sizePolicy().hasHeightForWidth());
        labelReplaceAllColorsText->setSizePolicy(sizePolicy3);
        labelReplaceAllColorsText->setLineWidth(1);
        labelReplaceAllColorsText->setTextFormat(Qt::PlainText);
        labelReplaceAllColorsText->setScaledContents(false);
        labelReplaceAllColorsText->setWordWrap(true);

        horizontalLayout->addWidget(labelReplaceAllColorsText);


        gridLayout_2->addWidget(frameReplaceAllColors, 2, 0, 1, 3);


        gridLayout->addWidget(groupBoxConditions, 0, 0, 1, 2);

        groupBoxReplacement = new QGroupBox(DlgReplaceCueColor);
        groupBoxReplacement->setObjectName("groupBoxReplacement");
        gridLayout_3 = new QGridLayout(groupBoxReplacement);
        gridLayout_3->setObjectName("gridLayout_3");
        labelNewColor = new QLabel(groupBoxReplacement);
        labelNewColor->setObjectName("labelNewColor");

        gridLayout_3->addWidget(labelNewColor, 0, 0, 1, 1);

        pushButtonNewColor = new QPushButton(groupBoxReplacement);
        pushButtonNewColor->setObjectName("pushButtonNewColor");

        gridLayout_3->addWidget(pushButtonNewColor, 0, 1, 1, 1);


        gridLayout->addWidget(groupBoxReplacement, 1, 0, 1, 2);

        buttonBox = new QDialogButtonBox(DlgReplaceCueColor);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Apply|QDialogButtonBox::Close);

        gridLayout->addWidget(buttonBox, 3, 0, 1, 2);

        verticalSpacer = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 2, 0, 1, 2);


        retranslateUi(DlgReplaceCueColor);

        QMetaObject::connectSlotsByName(DlgReplaceCueColor);
    } // setupUi

    void retranslateUi(QDialog *DlgReplaceCueColor)
    {
        DlgReplaceCueColor->setWindowTitle(QCoreApplication::translate("DlgReplaceCueColor", "Replace Hotcue Color", nullptr));
        groupBoxConditions->setTitle(QCoreApplication::translate("DlgReplaceCueColor", "Replace cue color if \342\200\246", nullptr));
        checkBoxHotcueIndexCondition->setText(QCoreApplication::translate("DlgReplaceCueColor", "Hotcue index", nullptr));
        comboBoxHotcueIndexCompare->setItemText(0, QCoreApplication::translate("DlgReplaceCueColor", "is", nullptr));
        comboBoxHotcueIndexCompare->setItemText(1, QCoreApplication::translate("DlgReplaceCueColor", "is not", nullptr));

        checkBoxCurrentColorCondition->setText(QCoreApplication::translate("DlgReplaceCueColor", "Current cue color", nullptr));
        comboBoxCurrentColorCompare->setItemText(0, QCoreApplication::translate("DlgReplaceCueColor", "is", nullptr));
        comboBoxCurrentColorCompare->setItemText(1, QCoreApplication::translate("DlgReplaceCueColor", "is not", nullptr));

        pushButtonCurrentColor->setText(QString());
        labelReplaceAllColorsIcon->setText(QString());
        labelReplaceAllColorsText->setText(QCoreApplication::translate("DlgReplaceCueColor", "If you don't specify any conditions, the colors of all cues in the library will be replaced.", nullptr));
        groupBoxReplacement->setTitle(QCoreApplication::translate("DlgReplaceCueColor", "\342\200\246 by:", nullptr));
        labelNewColor->setText(QCoreApplication::translate("DlgReplaceCueColor", "New cue color", nullptr));
        pushButtonNewColor->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class DlgReplaceCueColor: public Ui_DlgReplaceCueColor {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLGREPLACECUECOLORDLG_H
