/****************************************************************************
** Meta object code from reading C++ file 'autodjprocessor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/library/autodj/autodjprocessor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'autodjprocessor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14DeckAttributesE_t {};
} // unnamed namespace

template <> constexpr inline auto DeckAttributes::qt_create_metaobjectdata<qt_meta_tag_ZN14DeckAttributesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DeckAttributes",
        "playChanged",
        "",
        "DeckAttributes*",
        "pDeck",
        "playing",
        "playPositionChanged",
        "playPosition",
        "introStartPositionChanged",
        "introStartPosition",
        "introEndPositionChanged",
        "introEndPosition",
        "outroStartPositionChanged",
        "outtroStartPosition",
        "outroEndPositionChanged",
        "outroEndPosition",
        "trackLoaded",
        "TrackPointer",
        "pTrack",
        "loadingTrack",
        "pNewTrack",
        "pOldTrack",
        "playerEmpty",
        "rateChanged",
        "slotPlayPosChanged",
        "v",
        "slotPlayChanged",
        "slotIntroStartPositionChanged",
        "slotIntroEndPositionChanged",
        "slotOutroStartPositionChanged",
        "slotOutroEndPositionChanged",
        "slotTrackLoaded",
        "slotLoadingTrack",
        "slotPlayerEmpty",
        "slotRateChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'playChanged'
        QtMocHelpers::SignalData<void(DeckAttributes *, bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Bool, 5 },
        }}),
        // Signal 'playPositionChanged'
        QtMocHelpers::SignalData<void(DeckAttributes *, double)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Double, 7 },
        }}),
        // Signal 'introStartPositionChanged'
        QtMocHelpers::SignalData<void(DeckAttributes *, double)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Double, 9 },
        }}),
        // Signal 'introEndPositionChanged'
        QtMocHelpers::SignalData<void(DeckAttributes *, double)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Double, 11 },
        }}),
        // Signal 'outroStartPositionChanged'
        QtMocHelpers::SignalData<void(DeckAttributes *, double)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Double, 13 },
        }}),
        // Signal 'outroEndPositionChanged'
        QtMocHelpers::SignalData<void(DeckAttributes *, double)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Double, 15 },
        }}),
        // Signal 'trackLoaded'
        QtMocHelpers::SignalData<void(DeckAttributes *, TrackPointer)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 17, 18 },
        }}),
        // Signal 'loadingTrack'
        QtMocHelpers::SignalData<void(DeckAttributes *, TrackPointer, TrackPointer)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 17, 20 }, { 0x80000000 | 17, 21 },
        }}),
        // Signal 'playerEmpty'
        QtMocHelpers::SignalData<void(DeckAttributes *)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'rateChanged'
        QtMocHelpers::SignalData<void(DeckAttributes *)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotPlayPosChanged'
        QtMocHelpers::SlotData<void(double)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 25 },
        }}),
        // Slot 'slotPlayChanged'
        QtMocHelpers::SlotData<void(double)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 25 },
        }}),
        // Slot 'slotIntroStartPositionChanged'
        QtMocHelpers::SlotData<void(double)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 25 },
        }}),
        // Slot 'slotIntroEndPositionChanged'
        QtMocHelpers::SlotData<void(double)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 25 },
        }}),
        // Slot 'slotOutroStartPositionChanged'
        QtMocHelpers::SlotData<void(double)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 25 },
        }}),
        // Slot 'slotOutroEndPositionChanged'
        QtMocHelpers::SlotData<void(double)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 25 },
        }}),
        // Slot 'slotTrackLoaded'
        QtMocHelpers::SlotData<void(TrackPointer)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'slotLoadingTrack'
        QtMocHelpers::SlotData<void(TrackPointer, TrackPointer)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 20 }, { 0x80000000 | 17, 21 },
        }}),
        // Slot 'slotPlayerEmpty'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotRateChanged'
        QtMocHelpers::SlotData<void(double)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 25 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DeckAttributes, qt_meta_tag_ZN14DeckAttributesE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DeckAttributes::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14DeckAttributesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14DeckAttributesE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14DeckAttributesE_t>.metaTypes,
    nullptr
} };

void DeckAttributes::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DeckAttributes *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->playChanged((*reinterpret_cast<std::add_pointer_t<DeckAttributes*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 1: _t->playPositionChanged((*reinterpret_cast<std::add_pointer_t<DeckAttributes*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 2: _t->introStartPositionChanged((*reinterpret_cast<std::add_pointer_t<DeckAttributes*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 3: _t->introEndPositionChanged((*reinterpret_cast<std::add_pointer_t<DeckAttributes*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 4: _t->outroStartPositionChanged((*reinterpret_cast<std::add_pointer_t<DeckAttributes*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 5: _t->outroEndPositionChanged((*reinterpret_cast<std::add_pointer_t<DeckAttributes*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 6: _t->trackLoaded((*reinterpret_cast<std::add_pointer_t<DeckAttributes*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[2]))); break;
        case 7: _t->loadingTrack((*reinterpret_cast<std::add_pointer_t<DeckAttributes*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[3]))); break;
        case 8: _t->playerEmpty((*reinterpret_cast<std::add_pointer_t<DeckAttributes*>>(_a[1]))); break;
        case 9: _t->rateChanged((*reinterpret_cast<std::add_pointer_t<DeckAttributes*>>(_a[1]))); break;
        case 10: _t->slotPlayPosChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 11: _t->slotPlayChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 12: _t->slotIntroStartPositionChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 13: _t->slotIntroEndPositionChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 14: _t->slotOutroStartPositionChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 15: _t->slotOutroEndPositionChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 16: _t->slotTrackLoaded((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1]))); break;
        case 17: _t->slotLoadingTrack((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[2]))); break;
        case 18: _t->slotPlayerEmpty(); break;
        case 19: _t->slotRateChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeckAttributes* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeckAttributes* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeckAttributes* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeckAttributes* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeckAttributes* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeckAttributes* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeckAttributes* >(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeckAttributes* >(); break;
            case 2:
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeckAttributes* >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeckAttributes* >(); break;
            }
            break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DeckAttributes::*)(DeckAttributes * , bool )>(_a, &DeckAttributes::playChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeckAttributes::*)(DeckAttributes * , double )>(_a, &DeckAttributes::playPositionChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeckAttributes::*)(DeckAttributes * , double )>(_a, &DeckAttributes::introStartPositionChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeckAttributes::*)(DeckAttributes * , double )>(_a, &DeckAttributes::introEndPositionChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeckAttributes::*)(DeckAttributes * , double )>(_a, &DeckAttributes::outroStartPositionChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeckAttributes::*)(DeckAttributes * , double )>(_a, &DeckAttributes::outroEndPositionChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeckAttributes::*)(DeckAttributes * , TrackPointer )>(_a, &DeckAttributes::trackLoaded, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeckAttributes::*)(DeckAttributes * , TrackPointer , TrackPointer )>(_a, &DeckAttributes::loadingTrack, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeckAttributes::*)(DeckAttributes * )>(_a, &DeckAttributes::playerEmpty, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (DeckAttributes::*)(DeckAttributes * )>(_a, &DeckAttributes::rateChanged, 9))
            return;
    }
}

const QMetaObject *DeckAttributes::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DeckAttributes::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14DeckAttributesE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int DeckAttributes::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    return _id;
}

// SIGNAL 0
void DeckAttributes::playChanged(DeckAttributes * _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void DeckAttributes::playPositionChanged(DeckAttributes * _t1, double _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void DeckAttributes::introStartPositionChanged(DeckAttributes * _t1, double _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void DeckAttributes::introEndPositionChanged(DeckAttributes * _t1, double _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void DeckAttributes::outroStartPositionChanged(DeckAttributes * _t1, double _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void DeckAttributes::outroEndPositionChanged(DeckAttributes * _t1, double _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void DeckAttributes::trackLoaded(DeckAttributes * _t1, TrackPointer _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void DeckAttributes::loadingTrack(DeckAttributes * _t1, TrackPointer _t2, TrackPointer _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2, _t3);
}

// SIGNAL 8
void DeckAttributes::playerEmpty(DeckAttributes * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void DeckAttributes::rateChanged(DeckAttributes * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN15AutoDJProcessorE_t {};
} // unnamed namespace

template <> constexpr inline auto AutoDJProcessor::qt_create_metaobjectdata<qt_meta_tag_ZN15AutoDJProcessorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AutoDJProcessor",
        "loadTrackToPlayer",
        "",
        "TrackPointer",
        "pTrack",
        "group",
        "play",
        "autoDJStateChanged",
        "AutoDJProcessor::AutoDJState",
        "state",
        "autoDJError",
        "AutoDJProcessor::AutoDJError",
        "error",
        "transitionTimeChanged",
        "time",
        "randomTrackRequested",
        "tracksToAdd",
        "crossfaderChanged",
        "value",
        "playerPositionChanged",
        "DeckAttributes*",
        "pDeck",
        "position",
        "playerPlayChanged",
        "playing",
        "playerIntroStartChanged",
        "playerIntroEndChanged",
        "playerOutroStartChanged",
        "playerOutroEndChanged",
        "playerTrackLoaded",
        "playerLoadingTrack",
        "pNewTrack",
        "pOldTrack",
        "playerEmpty",
        "playerRateChanged",
        "playlistFirstTrackChanged",
        "controlEnableChangeRequest",
        "controlFadeNow",
        "controlShuffle",
        "controlSkipNext",
        "controlAddRandomTrack",
        "slotNumberOfDecksChanged",
        "decks"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'loadTrackToPlayer'
        QtMocHelpers::SignalData<void(TrackPointer, const QString &, bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 5 }, { QMetaType::Bool, 6 },
        }}),
        // Signal 'autoDJStateChanged'
        QtMocHelpers::SignalData<void(AutoDJProcessor::AutoDJState)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'autoDJError'
        QtMocHelpers::SignalData<void(AutoDJProcessor::AutoDJError)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'transitionTimeChanged'
        QtMocHelpers::SignalData<void(int)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 14 },
        }}),
        // Signal 'randomTrackRequested'
        QtMocHelpers::SignalData<void(int)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 16 },
        }}),
        // Slot 'crossfaderChanged'
        QtMocHelpers::SlotData<void(double)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 18 },
        }}),
        // Slot 'playerPositionChanged'
        QtMocHelpers::SlotData<void(DeckAttributes *, double)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 20, 21 }, { QMetaType::Double, 22 },
        }}),
        // Slot 'playerPlayChanged'
        QtMocHelpers::SlotData<void(DeckAttributes *, bool)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 20, 21 }, { QMetaType::Bool, 24 },
        }}),
        // Slot 'playerIntroStartChanged'
        QtMocHelpers::SlotData<void(DeckAttributes *, double)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 20, 21 }, { QMetaType::Double, 22 },
        }}),
        // Slot 'playerIntroEndChanged'
        QtMocHelpers::SlotData<void(DeckAttributes *, double)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 20, 21 }, { QMetaType::Double, 22 },
        }}),
        // Slot 'playerOutroStartChanged'
        QtMocHelpers::SlotData<void(DeckAttributes *, double)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 20, 21 }, { QMetaType::Double, 22 },
        }}),
        // Slot 'playerOutroEndChanged'
        QtMocHelpers::SlotData<void(DeckAttributes *, double)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 20, 21 }, { QMetaType::Double, 22 },
        }}),
        // Slot 'playerTrackLoaded'
        QtMocHelpers::SlotData<void(DeckAttributes *, TrackPointer)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 20, 21 }, { 0x80000000 | 3, 4 },
        }}),
        // Slot 'playerLoadingTrack'
        QtMocHelpers::SlotData<void(DeckAttributes *, TrackPointer, TrackPointer)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 20, 21 }, { 0x80000000 | 3, 31 }, { 0x80000000 | 3, 32 },
        }}),
        // Slot 'playerEmpty'
        QtMocHelpers::SlotData<void(DeckAttributes *)>(33, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'playerRateChanged'
        QtMocHelpers::SlotData<void(DeckAttributes *)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'playlistFirstTrackChanged'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'controlEnableChangeRequest'
        QtMocHelpers::SlotData<void(double)>(36, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 18 },
        }}),
        // Slot 'controlFadeNow'
        QtMocHelpers::SlotData<void(double)>(37, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 18 },
        }}),
        // Slot 'controlShuffle'
        QtMocHelpers::SlotData<void(double)>(38, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 18 },
        }}),
        // Slot 'controlSkipNext'
        QtMocHelpers::SlotData<void(double)>(39, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 18 },
        }}),
        // Slot 'controlAddRandomTrack'
        QtMocHelpers::SlotData<void(double)>(40, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 18 },
        }}),
        // Slot 'slotNumberOfDecksChanged'
        QtMocHelpers::SlotData<void(int)>(41, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 42 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AutoDJProcessor, qt_meta_tag_ZN15AutoDJProcessorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AutoDJProcessor::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15AutoDJProcessorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15AutoDJProcessorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15AutoDJProcessorE_t>.metaTypes,
    nullptr
} };

void AutoDJProcessor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AutoDJProcessor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->loadTrackToPlayer((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 1: _t->autoDJStateChanged((*reinterpret_cast<std::add_pointer_t<AutoDJProcessor::AutoDJState>>(_a[1]))); break;
        case 2: _t->autoDJError((*reinterpret_cast<std::add_pointer_t<AutoDJProcessor::AutoDJError>>(_a[1]))); break;
        case 3: _t->transitionTimeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->randomTrackRequested((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->crossfaderChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 6: _t->playerPositionChanged((*reinterpret_cast<std::add_pointer_t<DeckAttributes*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 7: _t->playerPlayChanged((*reinterpret_cast<std::add_pointer_t<DeckAttributes*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 8: _t->playerIntroStartChanged((*reinterpret_cast<std::add_pointer_t<DeckAttributes*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 9: _t->playerIntroEndChanged((*reinterpret_cast<std::add_pointer_t<DeckAttributes*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 10: _t->playerOutroStartChanged((*reinterpret_cast<std::add_pointer_t<DeckAttributes*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 11: _t->playerOutroEndChanged((*reinterpret_cast<std::add_pointer_t<DeckAttributes*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 12: _t->playerTrackLoaded((*reinterpret_cast<std::add_pointer_t<DeckAttributes*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[2]))); break;
        case 13: _t->playerLoadingTrack((*reinterpret_cast<std::add_pointer_t<DeckAttributes*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[3]))); break;
        case 14: _t->playerEmpty((*reinterpret_cast<std::add_pointer_t<DeckAttributes*>>(_a[1]))); break;
        case 15: _t->playerRateChanged((*reinterpret_cast<std::add_pointer_t<DeckAttributes*>>(_a[1]))); break;
        case 16: _t->playlistFirstTrackChanged(); break;
        case 17: _t->controlEnableChangeRequest((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 18: _t->controlFadeNow((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 19: _t->controlShuffle((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 20: _t->controlSkipNext((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 21: _t->controlAddRandomTrack((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 22: _t->slotNumberOfDecksChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeckAttributes* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeckAttributes* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeckAttributes* >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeckAttributes* >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeckAttributes* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeckAttributes* >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeckAttributes* >(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeckAttributes* >(); break;
            case 2:
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeckAttributes* >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeckAttributes* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AutoDJProcessor::*)(TrackPointer , const QString & , bool )>(_a, &AutoDJProcessor::loadTrackToPlayer, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AutoDJProcessor::*)(AutoDJProcessor::AutoDJState )>(_a, &AutoDJProcessor::autoDJStateChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AutoDJProcessor::*)(AutoDJProcessor::AutoDJError )>(_a, &AutoDJProcessor::autoDJError, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AutoDJProcessor::*)(int )>(_a, &AutoDJProcessor::transitionTimeChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AutoDJProcessor::*)(int )>(_a, &AutoDJProcessor::randomTrackRequested, 4))
            return;
    }
}

const QMetaObject *AutoDJProcessor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AutoDJProcessor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15AutoDJProcessorE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int AutoDJProcessor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    return _id;
}

// SIGNAL 0
void AutoDJProcessor::loadTrackToPlayer(TrackPointer _t1, const QString & _t2, bool _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void AutoDJProcessor::autoDJStateChanged(AutoDJProcessor::AutoDJState _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void AutoDJProcessor::autoDJError(AutoDJProcessor::AutoDJError _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void AutoDJProcessor::transitionTimeChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void AutoDJProcessor::randomTrackRequested(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
