/****************************************************************************
** Meta object code from reading C++ file 'effectchain.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/effects/effectchain.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'effectchain.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11EffectChainE_t {};
} // unnamed namespace

template <> constexpr inline auto EffectChain::qt_create_metaobjectdata<qt_meta_tag_ZN11EffectChainE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "EffectChain",
        "chainPresetChanged",
        "",
        "name",
        "slotControlClear",
        "value",
        "sendParameterUpdate",
        "slotControlChainSuperParameter",
        "v",
        "force",
        "slotEffectChainPresetRenamed",
        "oldName",
        "newName",
        "slotPresetListUpdated",
        "slotControlLoadedChainPresetRequest",
        "slotControlChainPresetSelector",
        "slotControlNextChainPreset",
        "slotControlPrevChainPreset",
        "slotChannelStatusChanged",
        "ChannelHandleAndGroup",
        "handleGroup"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'chainPresetChanged'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'slotControlClear'
        QtMocHelpers::SlotData<void(double)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 5 },
        }}),
        // Slot 'sendParameterUpdate'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotControlChainSuperParameter'
        QtMocHelpers::SlotData<void(double, bool)>(7, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Double, 8 }, { QMetaType::Bool, 9 },
        }}),
        // Slot 'slotEffectChainPresetRenamed'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(10, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 11 }, { QMetaType::QString, 12 },
        }}),
        // Slot 'slotPresetListUpdated'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotControlLoadedChainPresetRequest'
        QtMocHelpers::SlotData<void(double)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 5 },
        }}),
        // Slot 'slotControlChainPresetSelector'
        QtMocHelpers::SlotData<void(double)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 5 },
        }}),
        // Slot 'slotControlNextChainPreset'
        QtMocHelpers::SlotData<void(double)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 5 },
        }}),
        // Slot 'slotControlPrevChainPreset'
        QtMocHelpers::SlotData<void(double)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 5 },
        }}),
        // Slot 'slotChannelStatusChanged'
        QtMocHelpers::SlotData<void(double, const ChannelHandleAndGroup &)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 5 }, { 0x80000000 | 19, 20 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<EffectChain, qt_meta_tag_ZN11EffectChainE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject EffectChain::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11EffectChainE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11EffectChainE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11EffectChainE_t>.metaTypes,
    nullptr
} };

void EffectChain::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<EffectChain *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->chainPresetChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->slotControlClear((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 2: _t->sendParameterUpdate(); break;
        case 3: _t->slotControlChainSuperParameter((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 4: _t->slotEffectChainPresetRenamed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 5: _t->slotPresetListUpdated(); break;
        case 6: _t->slotControlLoadedChainPresetRequest((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 7: _t->slotControlChainPresetSelector((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 8: _t->slotControlNextChainPreset((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 9: _t->slotControlPrevChainPreset((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 10: _t->slotChannelStatusChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ChannelHandleAndGroup>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (EffectChain::*)(const QString & )>(_a, &EffectChain::chainPresetChanged, 0))
            return;
    }
}

const QMetaObject *EffectChain::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *EffectChain::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11EffectChainE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int EffectChain::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void EffectChain::chainPresetChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
