/****************************************************************************
** Meta object code from reading C++ file 'baseplaylistfeature.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/library/trackset/baseplaylistfeature.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QSet>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'baseplaylistfeature.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19BasePlaylistFeatureE_t {};
} // unnamed namespace

template <> constexpr inline auto BasePlaylistFeature::qt_create_metaobjectdata<qt_meta_tag_ZN19BasePlaylistFeatureE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "BasePlaylistFeature",
        "activateChild",
        "",
        "QModelIndex",
        "index",
        "activatePlaylist",
        "playlistId",
        "htmlLinkClicked",
        "QUrl",
        "link",
        "slotPlaylistTableChanged",
        "slotPlaylistTableChangedAndSelect",
        "slotPlaylistTableChangedAndScrollTo",
        "slotPlaylistContentOrLockChanged",
        "QSet<int>",
        "playlistIds",
        "slotPlaylistTableRenamed",
        "newName",
        "slotCreatePlaylist",
        "renameItem",
        "deleteItem",
        "slotDeletePlaylist",
        "slotDuplicatePlaylist",
        "slotAddToAutoDJ",
        "slotAddToAutoDJTop",
        "slotAddToAutoDJReplace",
        "slotRenamePlaylist",
        "slotTogglePlaylistLock",
        "slotImportPlaylist",
        "slotImportPlaylistFile",
        "playlistFile",
        "slotCreateImportPlaylist",
        "slotExportPlaylist",
        "slotExportTrackFiles",
        "slotAnalyzePlaylist",
        "slotTrackSelected",
        "TrackId",
        "trackId",
        "slotResetSelectedTrack"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'activateChild'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'activatePlaylist'
        QtMocHelpers::SlotData<void(int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Slot 'htmlLinkClicked'
        QtMocHelpers::SlotData<void(const QUrl &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'slotPlaylistTableChanged'
        QtMocHelpers::SlotData<void(int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Slot 'slotPlaylistTableChangedAndSelect'
        QtMocHelpers::SlotData<void(int)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Slot 'slotPlaylistTableChangedAndScrollTo'
        QtMocHelpers::SlotData<void(int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Slot 'slotPlaylistContentOrLockChanged'
        QtMocHelpers::SlotData<void(const QSet<int> &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'slotPlaylistTableRenamed'
        QtMocHelpers::SlotData<void(int, const QString &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 }, { QMetaType::QString, 17 },
        }}),
        // Slot 'slotCreatePlaylist'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'renameItem'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'deleteItem'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotDeletePlaylist'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotDuplicatePlaylist'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotAddToAutoDJ'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotAddToAutoDJTop'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotAddToAutoDJReplace'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotRenamePlaylist'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotTogglePlaylistLock'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotImportPlaylist'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotImportPlaylistFile'
        QtMocHelpers::SlotData<void(const QString &, int)>(29, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 30 }, { QMetaType::Int, 6 },
        }}),
        // Slot 'slotCreateImportPlaylist'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotExportPlaylist'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotExportTrackFiles'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotAnalyzePlaylist'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotTrackSelected'
        QtMocHelpers::SlotData<void(TrackId)>(35, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 36, 37 },
        }}),
        // Slot 'slotResetSelectedTrack'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<BasePlaylistFeature, qt_meta_tag_ZN19BasePlaylistFeatureE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject BasePlaylistFeature::staticMetaObject = { {
    QMetaObject::SuperData::link<BaseTrackSetFeature::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19BasePlaylistFeatureE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19BasePlaylistFeatureE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19BasePlaylistFeatureE_t>.metaTypes,
    nullptr
} };

void BasePlaylistFeature::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<BasePlaylistFeature *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->activateChild((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 1: _t->activatePlaylist((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->htmlLinkClicked((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 3: _t->slotPlaylistTableChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->slotPlaylistTableChangedAndSelect((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->slotPlaylistTableChangedAndScrollTo((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->slotPlaylistContentOrLockChanged((*reinterpret_cast<std::add_pointer_t<QSet<int>>>(_a[1]))); break;
        case 7: _t->slotPlaylistTableRenamed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 8: _t->slotCreatePlaylist(); break;
        case 9: _t->renameItem((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 10: _t->deleteItem((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 11: _t->slotDeletePlaylist(); break;
        case 12: _t->slotDuplicatePlaylist(); break;
        case 13: _t->slotAddToAutoDJ(); break;
        case 14: _t->slotAddToAutoDJTop(); break;
        case 15: _t->slotAddToAutoDJReplace(); break;
        case 16: _t->slotRenamePlaylist(); break;
        case 17: _t->slotTogglePlaylistLock(); break;
        case 18: _t->slotImportPlaylist(); break;
        case 19: _t->slotImportPlaylistFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 20: _t->slotCreateImportPlaylist(); break;
        case 21: _t->slotExportPlaylist(); break;
        case 22: _t->slotExportTrackFiles(); break;
        case 23: _t->slotAnalyzePlaylist(); break;
        case 24: _t->slotTrackSelected((*reinterpret_cast<std::add_pointer_t<TrackId>>(_a[1]))); break;
        case 25: _t->slotResetSelectedTrack(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<int> >(); break;
            }
            break;
        case 24:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackId >(); break;
            }
            break;
        }
    }
}

const QMetaObject *BasePlaylistFeature::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *BasePlaylistFeature::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19BasePlaylistFeatureE_t>.strings))
        return static_cast<void*>(this);
    return BaseTrackSetFeature::qt_metacast(_clname);
}

int BasePlaylistFeature::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = BaseTrackSetFeature::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    }
    return _id;
}
QT_WARNING_POP
