/****************************************************************************
** Meta object code from reading C++ file 'dlgautodj.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/library/autodj/dlgautodj.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dlgautodj.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9DlgAutoDJE_t {};
} // unnamed namespace

template <> constexpr inline auto DlgAutoDJ::qt_create_metaobjectdata<qt_meta_tag_ZN9DlgAutoDJE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DlgAutoDJ",
        "addRandomTrackButton",
        "",
        "buttonChecked",
        "loadTrack",
        "TrackPointer",
        "tio",
        "loadTrackToPlayer",
        "group",
        "trackSelected",
        "pTrack",
        "shufflePlaylistButton",
        "skipNextButton",
        "fadeNowButton",
        "toggleAutoDJButton",
        "enable",
        "autoDJError",
        "AutoDJProcessor::AutoDJError",
        "error",
        "transitionTimeChanged",
        "time",
        "transitionSliderChanged",
        "value",
        "autoDJStateChanged",
        "AutoDJProcessor::AutoDJState",
        "state",
        "updateSelectionInfo",
        "slotTransitionModeChanged",
        "comboboxIndex",
        "slotRepeatPlaylistChanged",
        "checked"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'addRandomTrackButton'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'loadTrack'
        QtMocHelpers::SignalData<void(TrackPointer)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'loadTrackToPlayer'
        QtMocHelpers::SignalData<void(TrackPointer, const QString &, bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { QMetaType::QString, 8 }, { QMetaType::Bool, 2 },
        }}),
        // Signal 'trackSelected'
        QtMocHelpers::SignalData<void(TrackPointer)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 10 },
        }}),
        // Slot 'shufflePlaylistButton'
        QtMocHelpers::SlotData<void(bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'skipNextButton'
        QtMocHelpers::SlotData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'fadeNowButton'
        QtMocHelpers::SlotData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'toggleAutoDJButton'
        QtMocHelpers::SlotData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
        // Slot 'autoDJError'
        QtMocHelpers::SlotData<void(AutoDJProcessor::AutoDJError)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'transitionTimeChanged'
        QtMocHelpers::SlotData<void(int)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 20 },
        }}),
        // Slot 'transitionSliderChanged'
        QtMocHelpers::SlotData<void(int)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 22 },
        }}),
        // Slot 'autoDJStateChanged'
        QtMocHelpers::SlotData<void(AutoDJProcessor::AutoDJState)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Slot 'updateSelectionInfo'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotTransitionModeChanged'
        QtMocHelpers::SlotData<void(int)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 28 },
        }}),
        // Slot 'slotRepeatPlaylistChanged'
        QtMocHelpers::SlotData<void(bool)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 30 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DlgAutoDJ, qt_meta_tag_ZN9DlgAutoDJE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DlgAutoDJ::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9DlgAutoDJE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9DlgAutoDJE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9DlgAutoDJE_t>.metaTypes,
    nullptr
} };

void DlgAutoDJ::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DlgAutoDJ *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->addRandomTrackButton((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->loadTrack((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1]))); break;
        case 2: _t->loadTrackToPlayer((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 3: _t->trackSelected((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1]))); break;
        case 4: _t->shufflePlaylistButton((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->skipNextButton((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->fadeNowButton((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->toggleAutoDJButton((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->autoDJError((*reinterpret_cast<std::add_pointer_t<AutoDJProcessor::AutoDJError>>(_a[1]))); break;
        case 9: _t->transitionTimeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->transitionSliderChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->autoDJStateChanged((*reinterpret_cast<std::add_pointer_t<AutoDJProcessor::AutoDJState>>(_a[1]))); break;
        case 12: _t->updateSelectionInfo(); break;
        case 13: _t->slotTransitionModeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 14: _t->slotRepeatPlaylistChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DlgAutoDJ::*)(bool )>(_a, &DlgAutoDJ::addRandomTrackButton, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgAutoDJ::*)(TrackPointer )>(_a, &DlgAutoDJ::loadTrack, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgAutoDJ::*)(TrackPointer , const QString & , bool )>(_a, &DlgAutoDJ::loadTrackToPlayer, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgAutoDJ::*)(TrackPointer )>(_a, &DlgAutoDJ::trackSelected, 3))
            return;
    }
}

const QMetaObject *DlgAutoDJ::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DlgAutoDJ::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9DlgAutoDJE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::DlgAutoDJ"))
        return static_cast< Ui::DlgAutoDJ*>(this);
    if (!strcmp(_clname, "LibraryView"))
        return static_cast< LibraryView*>(this);
    return QWidget::qt_metacast(_clname);
}

int DlgAutoDJ::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void DlgAutoDJ::addRandomTrackButton(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void DlgAutoDJ::loadTrack(TrackPointer _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void DlgAutoDJ::loadTrackToPlayer(TrackPointer _t1, const QString & _t2, bool _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3);
}

// SIGNAL 3
void DlgAutoDJ::trackSelected(TrackPointer _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
