// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: headers.proto
// Protobuf C++ Version: 7.35.0

#include "headers.pb.h"

#include <algorithm>
#include <type_traits>
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/generated_message_tctable_impl.h"
#include "google/protobuf/internal_visibility.h"
#include "google/protobuf/extension_set.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/wire_format_lite.h"
#include "google/protobuf/io/zero_copy_stream_impl_lite.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"
PROTOBUF_PRAGMA_INIT_SEG
namespace _pb = ::google::protobuf;
namespace _pbi = ::google::protobuf::internal;
namespace _fl = ::google::protobuf::internal::field_layout;
namespace mixxx {
namespace library {
class HeaderViewState_HeaderState::_Internal {
 public:
  using HasBits = decltype(::std::declval<HeaderViewState_HeaderState>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(HeaderViewState_HeaderState, _impl_._has_bits_);
};

constexpr HeaderViewState_HeaderState::ParseTableT_ HeaderViewState_HeaderState::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(HeaderViewState_HeaderState, _impl_._has_bits_),
      0, // no _extensions_
      5, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967264,  // skipmap
      offsetof(ParseTableT_, field_entries),
      5,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallbackLite,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::mixxx::library::HeaderViewState_HeaderState>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // optional bool hidden = 1;
      {::_pbi::TcParser::FastV8S1,
       {8, 1, 0,
        PROTOBUF_FIELD_OFFSET(HeaderViewState_HeaderState, _impl_.hidden_)}},
      // optional int32 size = 2;
      {::_pbi::TcParser::FastV32S1,
       {16, 2, 0,
        PROTOBUF_FIELD_OFFSET(HeaderViewState_HeaderState, _impl_.size_)}},
      // optional int32 logical_index = 3;
      {::_pbi::TcParser::FastV32S1,
       {24, 3, 0,
        PROTOBUF_FIELD_OFFSET(HeaderViewState_HeaderState, _impl_.logical_index_)}},
      // optional int32 visual_index = 4;
      {::_pbi::TcParser::FastV32S1,
       {32, 4, 0,
        PROTOBUF_FIELD_OFFSET(HeaderViewState_HeaderState, _impl_.visual_index_)}},
      // optional string column_name = 5;
      {::_pbi::TcParser::FastBS1,
       {42, 0, 0,
        PROTOBUF_FIELD_OFFSET(HeaderViewState_HeaderState, _impl_.column_name_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // optional bool hidden = 1;
      {PROTOBUF_FIELD_OFFSET(HeaderViewState_HeaderState, _impl_.hidden_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
      // optional int32 size = 2;
      {PROTOBUF_FIELD_OFFSET(HeaderViewState_HeaderState, _impl_.size_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // optional int32 logical_index = 3;
      {PROTOBUF_FIELD_OFFSET(HeaderViewState_HeaderState, _impl_.logical_index_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // optional int32 visual_index = 4;
      {PROTOBUF_FIELD_OFFSET(HeaderViewState_HeaderState, _impl_.visual_index_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // optional string column_name = 5;
      {PROTOBUF_FIELD_OFFSET(HeaderViewState_HeaderState, _impl_.column_name_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kBytes | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr HeaderViewState_HeaderState::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        column_name_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        hidden_{false},
        size_{0},
        logical_index_{0},
        visual_index_{0} {}

template <typename>
constexpr HeaderViewState_HeaderState::HeaderViewState_HeaderState(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::MessageLite(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL HeaderViewState_HeaderState::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) HeaderViewState_HeaderState(arena);
}
constexpr auto HeaderViewState_HeaderState::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(HeaderViewState_HeaderState), alignof(HeaderViewState_HeaderState));
}
constexpr auto HeaderViewState_HeaderState::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataLite{
      {
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &HeaderViewState_HeaderState::MergeImpl,
          ::google::protobuf::MessageLite::GetNewImpl<HeaderViewState_HeaderState>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &HeaderViewState_HeaderState::SharedDtor,
          ::google::protobuf::MessageLite::GetClearImpl<HeaderViewState_HeaderState>(), &HeaderViewState_HeaderState::ByteSizeLong,
              &HeaderViewState_HeaderState::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(HeaderViewState_HeaderState, _impl_._cached_size_),
          true,
      },
      "mixxx.library.HeaderViewState.HeaderState",
  };
}
struct HeaderViewState_HeaderStateGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr HeaderViewState_HeaderStateGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 HeaderViewState_HeaderState_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(HeaderViewState_HeaderState::InternalGenerateClassData_(
            _default, &HeaderViewState_HeaderState_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<HeaderViewState_HeaderState>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~HeaderViewState_HeaderStateGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) HeaderViewState_HeaderState _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<HeaderViewState_HeaderState>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(HeaderViewState_HeaderStateGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST HeaderViewState_HeaderStateGlobalsTypeInternal HeaderViewState_HeaderState_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* HeaderViewState_HeaderState_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return HeaderViewState_HeaderState_globals_.GetClassData();
#else
  return HeaderViewState_HeaderState_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class HeaderViewState::_Internal {
 public:
  using HasBits = decltype(::std::declval<HeaderViewState>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(HeaderViewState, _impl_._has_bits_);
};

constexpr HeaderViewState::ParseTableT_ HeaderViewState::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(HeaderViewState, _impl_._has_bits_),
      0, // no _extensions_
      4, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967280,  // skipmap
      offsetof(ParseTableT_, field_entries),
      4,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallbackLite,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::mixxx::library::HeaderViewState>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // optional int32 sort_order = 4;
      {::_pbi::TcParser::FastV32S1,
       {32, 3, 0,
        PROTOBUF_FIELD_OFFSET(HeaderViewState, _impl_.sort_order_)}},
      // repeated .mixxx.library.HeaderViewState.HeaderState header_state = 1;
      {::_pbi::TcParser::FastMtR1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(HeaderViewState, _impl_.header_state_)}},
      // optional bool sort_indicator_shown = 2;
      {::_pbi::TcParser::FastV8S1,
       {16, 1, 0,
        PROTOBUF_FIELD_OFFSET(HeaderViewState, _impl_.sort_indicator_shown_)}},
      // optional int32 sort_indicator_section = 3;
      {::_pbi::TcParser::FastV32S1,
       {24, 2, 0,
        PROTOBUF_FIELD_OFFSET(HeaderViewState, _impl_.sort_indicator_section_)}},
    }}, {{
      65535, 65535
    }}, {{
      // repeated .mixxx.library.HeaderViewState.HeaderState header_state = 1;
      {PROTOBUF_FIELD_OFFSET(HeaderViewState, _impl_.header_state_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kMessage | ::_fl::kTvTable)},
      // optional bool sort_indicator_shown = 2;
      {PROTOBUF_FIELD_OFFSET(HeaderViewState, _impl_.sort_indicator_shown_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
      // optional int32 sort_indicator_section = 3;
      {PROTOBUF_FIELD_OFFSET(HeaderViewState, _impl_.sort_indicator_section_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // optional int32 sort_order = 4;
      {PROTOBUF_FIELD_OFFSET(HeaderViewState, _impl_.sort_order_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::mixxx::library::HeaderViewState_HeaderState>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::mixxx::library::HeaderViewState_HeaderState_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr HeaderViewState::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        header_state_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::mixxx::library::HeaderViewState,
            PROTOBUF_FIELD_OFFSET(::mixxx::library::HeaderViewState, _impl_.header_state_)>()
         }
        ,
        sort_indicator_shown_{false},
        sort_indicator_section_{0},
        sort_order_{0} {}

template <typename>
constexpr HeaderViewState::HeaderViewState(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::MessageLite(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL HeaderViewState::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) HeaderViewState(arena);
}
constexpr auto HeaderViewState::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(HeaderViewState), alignof(HeaderViewState));
}
constexpr auto HeaderViewState::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataLite{
      {
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &HeaderViewState::MergeImpl,
          ::google::protobuf::MessageLite::GetNewImpl<HeaderViewState>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &HeaderViewState::SharedDtor,
          ::google::protobuf::MessageLite::GetClearImpl<HeaderViewState>(), &HeaderViewState::ByteSizeLong,
              &HeaderViewState::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(HeaderViewState, _impl_._cached_size_),
          true,
      },
      "mixxx.library.HeaderViewState",
  };
}
struct HeaderViewStateGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr HeaderViewStateGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 HeaderViewState_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(HeaderViewState::InternalGenerateClassData_(
            _default, &HeaderViewState_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<HeaderViewState>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~HeaderViewStateGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) HeaderViewState _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<HeaderViewState>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(HeaderViewStateGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST HeaderViewStateGlobalsTypeInternal HeaderViewState_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* HeaderViewState_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return HeaderViewState_globals_.GetClassData();
#else
  return HeaderViewState_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
}  // namespace library
}  // namespace mixxx
namespace mixxx {
namespace library {
// ===================================================================

HeaderViewState_HeaderState::HeaderViewState_HeaderState(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::MessageLite(arena, HeaderViewState_HeaderState_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::MessageLite(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:mixxx.library.HeaderViewState.HeaderState)
}
PROTOBUF_NDEBUG_INLINE HeaderViewState_HeaderState::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::mixxx::library::HeaderViewState_HeaderState& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        column_name_(arena, from.column_name_) {}

HeaderViewState_HeaderState::HeaderViewState_HeaderState(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const HeaderViewState_HeaderState& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::MessageLite(arena, HeaderViewState_HeaderState_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::MessageLite(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  HeaderViewState_HeaderState* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::std::string>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, hidden_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, hidden_),
           offsetof(Impl_, visual_index_) -
               offsetof(Impl_, hidden_) +
               sizeof(Impl_::visual_index_));

  // @@protoc_insertion_point(copy_constructor:mixxx.library.HeaderViewState.HeaderState)
}
PROTOBUF_NDEBUG_INLINE HeaderViewState_HeaderState::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        column_name_(arena) {}

inline void HeaderViewState_HeaderState::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, hidden_),
           0,
           offsetof(Impl_, visual_index_) -
               offsetof(Impl_, hidden_) +
               sizeof(Impl_::visual_index_));
}
HeaderViewState_HeaderState::~HeaderViewState_HeaderState() {
  // @@protoc_insertion_point(destructor:mixxx.library.HeaderViewState.HeaderState)
  SharedDtor(*this);
}
inline void HeaderViewState_HeaderState::SharedDtor(MessageLite& self) {
  HeaderViewState_HeaderState& this_ = static_cast<HeaderViewState_HeaderState&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::std::string>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.column_name_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1
const ::google::protobuf::internal::ClassDataLite HeaderViewState_HeaderState_class_data_ =
    HeaderViewState_HeaderState::InternalGenerateClassData_(HeaderViewState_HeaderState_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
HeaderViewState_HeaderState::GetClassData() const {
  return HeaderViewState_HeaderState_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
HeaderViewState_HeaderState::GetClassData() const {
  return HeaderViewState_HeaderState_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const HeaderViewState_HeaderState::ParseTableT_
    HeaderViewState_HeaderState::_table_ =
        HeaderViewState_HeaderState::InternalGenerateParseTable_(HeaderViewState_HeaderState_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void HeaderViewState_HeaderState::Clear() {
// @@protoc_insertion_point(message_clear_start:mixxx.library.HeaderViewState.HeaderState)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.column_name_.ClearNonDefaultToEmpty();
  }
  if (BatchCheckHasBit(cached_has_bits, 0x0000001eU)) {
    ::memset(&_impl_.hidden_, 0, static_cast<::size_t>(
        reinterpret_cast<char*>(&_impl_.visual_index_) -
        reinterpret_cast<char*>(&_impl_.hidden_)) + sizeof(_impl_.visual_index_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::std::string>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL HeaderViewState_HeaderState::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const HeaderViewState_HeaderState& this_ = static_cast<const HeaderViewState_HeaderState&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL HeaderViewState_HeaderState::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const HeaderViewState_HeaderState& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:mixxx.library.HeaderViewState.HeaderState)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // optional bool hidden = 1;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(
        1, this_._internal_hidden(), target);
  }

  // optional int32 size = 2;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target =
        ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<2>(
            stream, this_._internal_size(), target);
  }

  // optional int32 logical_index = 3;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    target =
        ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<3>(
            stream, this_._internal_logical_index(), target);
  }

  // optional int32 visual_index = 4;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    target =
        ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<4>(
            stream, this_._internal_visual_index(), target);
  }

  // optional string column_name = 5;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    const ::std::string& _s = this_._internal_column_name();
    target = stream->WriteStringMaybeAliased(5, _s, target);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target = stream->WriteRaw(
        this_._internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString).data(),
        static_cast<int>(this_._internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString).size()), target);
  }
  // @@protoc_insertion_point(serialize_to_array_end:mixxx.library.HeaderViewState.HeaderState)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t HeaderViewState_HeaderState::ByteSizeLong(const MessageLite& base) {
  const HeaderViewState_HeaderState& this_ = static_cast<const HeaderViewState_HeaderState&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t HeaderViewState_HeaderState::ByteSizeLong() const {
  const HeaderViewState_HeaderState& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:mixxx.library.HeaderViewState.HeaderState)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  total_size += static_cast<bool>(0x00000002U & cached_has_bits) * 2;
  if (BatchCheckHasBit(cached_has_bits, 0x0000001dU)) {
    // optional string column_name = 5;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                      this_._internal_column_name());
    }
    // optional int32 size = 2;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
          this_._internal_size());
    }
    // optional int32 logical_index = 3;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
          this_._internal_logical_index());
    }
    // optional int32 visual_index = 4;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
          this_._internal_visual_index());
    }
  }
  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    total_size += this_._internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString).size();
  }
  this_._impl_._cached_size_.Set(::_pbi::ToCachedSize(total_size));
  return total_size;
}

void HeaderViewState_HeaderState::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<HeaderViewState_HeaderState*>(&to_msg);
  auto& from = static_cast<const HeaderViewState_HeaderState&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:mixxx.library.HeaderViewState.HeaderState)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000001fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_set_column_name(from._internal_column_name());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _this->_impl_.hidden_ = from._impl_.hidden_;
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _this->_impl_.size_ = from._impl_.size_;
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      _this->_impl_.logical_index_ = from._impl_.logical_index_;
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      _this->_impl_.visual_index_ = from._impl_.visual_index_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::std::string>(
      from._internal_metadata_);
}

void HeaderViewState_HeaderState::CopyFrom(const HeaderViewState_HeaderState& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:mixxx.library.HeaderViewState.HeaderState)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void HeaderViewState_HeaderState::InternalSwap(HeaderViewState_HeaderState* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.column_name_, &other->_impl_.column_name_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(HeaderViewState_HeaderState, _impl_.visual_index_)
      + sizeof(HeaderViewState_HeaderState::_impl_.visual_index_)
      - PROTOBUF_FIELD_OFFSET(HeaderViewState_HeaderState, _impl_.hidden_)>(
          reinterpret_cast<char*>(&_impl_.hidden_),
          reinterpret_cast<char*>(&other->_impl_.hidden_));
}

// ===================================================================

HeaderViewState::HeaderViewState(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::MessageLite(arena, HeaderViewState_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::MessageLite(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:mixxx.library.HeaderViewState)
}
PROTOBUF_NDEBUG_INLINE HeaderViewState::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::mixxx::library::HeaderViewState& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        header_state_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::mixxx::library::HeaderViewState,
              PROTOBUF_FIELD_OFFSET(::mixxx::library::HeaderViewState, _impl_.header_state_)>()
          , from.header_state_
        }
     {}

HeaderViewState::HeaderViewState(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const HeaderViewState& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::MessageLite(arena, HeaderViewState_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::MessageLite(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  HeaderViewState* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::std::string>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, sort_indicator_shown_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, sort_indicator_shown_),
           offsetof(Impl_, sort_order_) -
               offsetof(Impl_, sort_indicator_shown_) +
               sizeof(Impl_::sort_order_));

  // @@protoc_insertion_point(copy_constructor:mixxx.library.HeaderViewState)
}
PROTOBUF_NDEBUG_INLINE HeaderViewState::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        header_state_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::mixxx::library::HeaderViewState,
            PROTOBUF_FIELD_OFFSET(::mixxx::library::HeaderViewState, _impl_.header_state_)>()
         }
     {}

inline void HeaderViewState::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, sort_indicator_shown_),
           0,
           offsetof(Impl_, sort_order_) -
               offsetof(Impl_, sort_indicator_shown_) +
               sizeof(Impl_::sort_order_));
}
HeaderViewState::~HeaderViewState() {
  // @@protoc_insertion_point(destructor:mixxx.library.HeaderViewState)
  SharedDtor(*this);
}
inline void HeaderViewState::SharedDtor(MessageLite& self) {
  HeaderViewState& this_ = static_cast<HeaderViewState&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::std::string>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1
const ::google::protobuf::internal::ClassDataLite HeaderViewState_class_data_ =
    HeaderViewState::InternalGenerateClassData_(HeaderViewState_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
HeaderViewState::GetClassData() const {
  return HeaderViewState_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
HeaderViewState::GetClassData() const {
  return HeaderViewState_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const HeaderViewState::ParseTableT_
    HeaderViewState::_table_ =
        HeaderViewState::InternalGenerateParseTable_(HeaderViewState_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void HeaderViewState::Clear() {
// @@protoc_insertion_point(message_clear_start:mixxx.library.HeaderViewState)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.header_state_.Clear();
  }
  if (BatchCheckHasBit(cached_has_bits, 0x0000000eU)) {
    ::memset(&_impl_.sort_indicator_shown_, 0, static_cast<::size_t>(
        reinterpret_cast<char*>(&_impl_.sort_order_) -
        reinterpret_cast<char*>(&_impl_.sort_indicator_shown_)) + sizeof(_impl_.sort_order_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::std::string>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL HeaderViewState::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const HeaderViewState& this_ = static_cast<const HeaderViewState&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL HeaderViewState::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const HeaderViewState& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:mixxx.library.HeaderViewState)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // repeated .mixxx.library.HeaderViewState.HeaderState header_state = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (unsigned i = 0, n = static_cast<unsigned>(
                             this_._internal_header_state_size());
         i < n; i++) {
      const auto& repfield = this_._internal_header_state().Get(i);
      target =
          ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
              1, repfield, repfield.GetCachedSize(),
              target, stream);
    }
  }

  // optional bool sort_indicator_shown = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(
        2, this_._internal_sort_indicator_shown(), target);
  }

  // optional int32 sort_indicator_section = 3;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target =
        ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<3>(
            stream, this_._internal_sort_indicator_section(), target);
  }

  // optional int32 sort_order = 4;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    target =
        ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<4>(
            stream, this_._internal_sort_order(), target);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target = stream->WriteRaw(
        this_._internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString).data(),
        static_cast<int>(this_._internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString).size()), target);
  }
  // @@protoc_insertion_point(serialize_to_array_end:mixxx.library.HeaderViewState)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t HeaderViewState::ByteSizeLong(const MessageLite& base) {
  const HeaderViewState& this_ = static_cast<const HeaderViewState&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t HeaderViewState::ByteSizeLong() const {
  const HeaderViewState& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:mixxx.library.HeaderViewState)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  total_size += static_cast<bool>(0x00000002U & cached_has_bits) * 2;
  if (BatchCheckHasBit(cached_has_bits, 0x0000000dU)) {
    // repeated .mixxx.library.HeaderViewState.HeaderState header_state = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1UL * this_._internal_header_state_size();
      for (const auto& msg : this_._internal_header_state()) {
        total_size += ::google::protobuf::internal::WireFormatLite::MessageSize(msg);
      }
    }
    // optional int32 sort_indicator_section = 3;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
          this_._internal_sort_indicator_section());
    }
    // optional int32 sort_order = 4;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
          this_._internal_sort_order());
    }
  }
  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    total_size += this_._internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString).size();
  }
  this_._impl_._cached_size_.Set(::_pbi::ToCachedSize(total_size));
  return total_size;
}

void HeaderViewState::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<HeaderViewState*>(&to_msg);
  auto& from = static_cast<const HeaderViewState&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:mixxx.library.HeaderViewState)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_header_state()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_header_state());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _this->_impl_.sort_indicator_shown_ = from._impl_.sort_indicator_shown_;
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _this->_impl_.sort_indicator_section_ = from._impl_.sort_indicator_section_;
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      _this->_impl_.sort_order_ = from._impl_.sort_order_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::std::string>(
      from._internal_metadata_);
}

void HeaderViewState::CopyFrom(const HeaderViewState& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:mixxx.library.HeaderViewState)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void HeaderViewState::InternalSwap(HeaderViewState* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.header_state_.InternalSwap(&other->_impl_.header_state_);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(HeaderViewState, _impl_.sort_order_)
      + sizeof(HeaderViewState::_impl_.sort_order_)
      - PROTOBUF_FIELD_OFFSET(HeaderViewState, _impl_.sort_indicator_shown_)>(
          reinterpret_cast<char*>(&_impl_.sort_indicator_shown_),
          reinterpret_cast<char*>(&other->_impl_.sort_indicator_shown_));
}

// @@protoc_insertion_point(namespace_scope)
}  // namespace library
}  // namespace mixxx
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google
// @@protoc_insertion_point(global_scope)
#include "google/protobuf/port_undef.inc"
