// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: waveform.proto
// Protobuf C++ Version: 7.35.0

#include "waveform.pb.h"

#include <algorithm>
#include <type_traits>
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/generated_message_tctable_impl.h"
#include "google/protobuf/internal_visibility.h"
#include "google/protobuf/extension_set.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/wire_format_lite.h"
#include "google/protobuf/io/zero_copy_stream_impl_lite.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"
PROTOBUF_PRAGMA_INIT_SEG
namespace _pb = ::google::protobuf;
namespace _pbi = ::google::protobuf::internal;
namespace _fl = ::google::protobuf::internal::field_layout;
namespace mixxx {
namespace track {
namespace io {
class Waveform_Signal::_Internal {
 public:
  using HasBits = decltype(::std::declval<Waveform_Signal>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Waveform_Signal, _impl_._has_bits_);
};

constexpr Waveform_Signal::ParseTableT_ Waveform_Signal::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Waveform_Signal, _impl_._has_bits_),
      0, // no _extensions_
      5, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967264,  // skipmap
      offsetof(ParseTableT_, field_entries),
      5,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallbackLite,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::mixxx::track::io::Waveform_Signal>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // repeated int32 value = 1;
      {::_pbi::TcParser::FastV32R1,
       {8, 0, 0,
        PROTOBUF_FIELD_OFFSET(Waveform_Signal, _impl_.value_)}},
      // optional int32 channels = 2 [default = 2];
      {::_pbi::TcParser::FastV32S1,
       {16, 4, 0,
        PROTOBUF_FIELD_OFFSET(Waveform_Signal, _impl_.channels_)}},
      // optional .mixxx.track.io.Waveform.Units units = 3 [default = RMS];
      {::_pbi::TcParser::FastEr0S1,
       {24, 1, 1,
        PROTOBUF_FIELD_OFFSET(Waveform_Signal, _impl_.units_)}},
      // optional int32 max_value = 4;
      {::_pbi::TcParser::FastV32S1,
       {32, 2, 0,
        PROTOBUF_FIELD_OFFSET(Waveform_Signal, _impl_.max_value_)}},
      // optional int32 min_value = 5;
      {::_pbi::TcParser::FastV32S1,
       {40, 3, 0,
        PROTOBUF_FIELD_OFFSET(Waveform_Signal, _impl_.min_value_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // repeated int32 value = 1;
      {PROTOBUF_FIELD_OFFSET(Waveform_Signal, _impl_.value_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kInt32)},
      // optional int32 channels = 2 [default = 2];
      {PROTOBUF_FIELD_OFFSET(Waveform_Signal, _impl_.channels_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // optional .mixxx.track.io.Waveform.Units units = 3 [default = RMS];
      {PROTOBUF_FIELD_OFFSET(Waveform_Signal, _impl_.units_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kEnumRange)},
      // optional int32 max_value = 4;
      {PROTOBUF_FIELD_OFFSET(Waveform_Signal, _impl_.max_value_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // optional int32 min_value = 5;
      {PROTOBUF_FIELD_OFFSET(Waveform_Signal, _impl_.min_value_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
    }},
    {{
        {0, 1},
    }},
    {{
    }},
  };
}


inline constexpr Waveform_Signal::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        value_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::mixxx::track::io::Waveform_Signal,
            PROTOBUF_FIELD_OFFSET(::mixxx::track::io::Waveform_Signal, _impl_.value_)>()
         }
        ,
        units_{static_cast< ::mixxx::track::io::Waveform_Units >(0)},
        max_value_{0},
        min_value_{0},
        channels_{2} {}

template <typename>
constexpr Waveform_Signal::Waveform_Signal(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::MessageLite(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Waveform_Signal::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Waveform_Signal(arena);
}
constexpr auto Waveform_Signal::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(Waveform_Signal), alignof(Waveform_Signal));
}
constexpr auto Waveform_Signal::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataLite{
      {
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Waveform_Signal::MergeImpl,
          ::google::protobuf::MessageLite::GetNewImpl<Waveform_Signal>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Waveform_Signal::SharedDtor,
          ::google::protobuf::MessageLite::GetClearImpl<Waveform_Signal>(), &Waveform_Signal::ByteSizeLong,
              &Waveform_Signal::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Waveform_Signal, _impl_._cached_size_),
          true,
      },
      "mixxx.track.io.Waveform.Signal",
  };
}
struct Waveform_SignalGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr Waveform_SignalGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Waveform_Signal_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Waveform_Signal::InternalGenerateClassData_(
            _default, &Waveform_Signal_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Waveform_Signal>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~Waveform_SignalGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Waveform_Signal _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Waveform_Signal>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(Waveform_SignalGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST Waveform_SignalGlobalsTypeInternal Waveform_Signal_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Waveform_Signal_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Waveform_Signal_globals_.GetClassData();
#else
  return Waveform_Signal_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Waveform_FilteredSignal::_Internal {
 public:
  using HasBits = decltype(::std::declval<Waveform_FilteredSignal>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Waveform_FilteredSignal, _impl_._has_bits_);
};

constexpr Waveform_FilteredSignal::ParseTableT_ Waveform_FilteredSignal::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Waveform_FilteredSignal, _impl_._has_bits_),
      0, // no _extensions_
      7, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967168,  // skipmap
      offsetof(ParseTableT_, field_entries),
      7,  // num_field_entries
      3,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallbackLite,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::mixxx::track::io::Waveform_FilteredSignal>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // optional .mixxx.track.io.Waveform.Signal low = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(Waveform_FilteredSignal, _impl_.low_)}},
      // optional .mixxx.track.io.Waveform.Signal mid = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 1, 1,
        PROTOBUF_FIELD_OFFSET(Waveform_FilteredSignal, _impl_.mid_)}},
      // optional .mixxx.track.io.Waveform.Signal high = 3;
      {::_pbi::TcParser::FastMtS1,
       {26, 2, 2,
        PROTOBUF_FIELD_OFFSET(Waveform_FilteredSignal, _impl_.high_)}},
      // optional double low_cutoff_frequency = 4;
      {::_pbi::TcParser::FastF64S1,
       {33, 3, 0,
        PROTOBUF_FIELD_OFFSET(Waveform_FilteredSignal, _impl_.low_cutoff_frequency_)}},
      // optional double mid_low_cutoff_frequency = 5;
      {::_pbi::TcParser::FastF64S1,
       {41, 4, 0,
        PROTOBUF_FIELD_OFFSET(Waveform_FilteredSignal, _impl_.mid_low_cutoff_frequency_)}},
      // optional double mid_high_cutoff_frequency = 6;
      {::_pbi::TcParser::FastF64S1,
       {49, 5, 0,
        PROTOBUF_FIELD_OFFSET(Waveform_FilteredSignal, _impl_.mid_high_cutoff_frequency_)}},
      // optional double high_cutoff_frequency = 7;
      {::_pbi::TcParser::FastF64S1,
       {57, 6, 0,
        PROTOBUF_FIELD_OFFSET(Waveform_FilteredSignal, _impl_.high_cutoff_frequency_)}},
    }}, {{
      65535, 65535
    }}, {{
      // optional .mixxx.track.io.Waveform.Signal low = 1;
      {PROTOBUF_FIELD_OFFSET(Waveform_FilteredSignal, _impl_.low_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // optional .mixxx.track.io.Waveform.Signal mid = 2;
      {PROTOBUF_FIELD_OFFSET(Waveform_FilteredSignal, _impl_.mid_), _Internal::kHasBitsOffset + 1, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // optional .mixxx.track.io.Waveform.Signal high = 3;
      {PROTOBUF_FIELD_OFFSET(Waveform_FilteredSignal, _impl_.high_), _Internal::kHasBitsOffset + 2, 2, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // optional double low_cutoff_frequency = 4;
      {PROTOBUF_FIELD_OFFSET(Waveform_FilteredSignal, _impl_.low_cutoff_frequency_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kDouble)},
      // optional double mid_low_cutoff_frequency = 5;
      {PROTOBUF_FIELD_OFFSET(Waveform_FilteredSignal, _impl_.mid_low_cutoff_frequency_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kDouble)},
      // optional double mid_high_cutoff_frequency = 6;
      {PROTOBUF_FIELD_OFFSET(Waveform_FilteredSignal, _impl_.mid_high_cutoff_frequency_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kDouble)},
      // optional double high_cutoff_frequency = 7;
      {PROTOBUF_FIELD_OFFSET(Waveform_FilteredSignal, _impl_.high_cutoff_frequency_), _Internal::kHasBitsOffset + 6, 0, (0 | ::_fl::kFcOptional | ::_fl::kDouble)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::mixxx::track::io::Waveform_Signal>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::mixxx::track::io::Waveform_Signal_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::mixxx::track::io::Waveform_Signal>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::mixxx::track::io::Waveform_Signal_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::mixxx::track::io::Waveform_Signal>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::mixxx::track::io::Waveform_Signal_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr Waveform_FilteredSignal::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        low_{nullptr},
        mid_{nullptr},
        high_{nullptr},
        low_cutoff_frequency_{0},
        mid_low_cutoff_frequency_{0},
        mid_high_cutoff_frequency_{0},
        high_cutoff_frequency_{0} {}

template <typename>
constexpr Waveform_FilteredSignal::Waveform_FilteredSignal(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::MessageLite(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Waveform_FilteredSignal::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Waveform_FilteredSignal(arena);
}
constexpr auto Waveform_FilteredSignal::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(Waveform_FilteredSignal), alignof(Waveform_FilteredSignal));
}
constexpr auto Waveform_FilteredSignal::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataLite{
      {
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Waveform_FilteredSignal::MergeImpl,
          ::google::protobuf::MessageLite::GetNewImpl<Waveform_FilteredSignal>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Waveform_FilteredSignal::SharedDtor,
          ::google::protobuf::MessageLite::GetClearImpl<Waveform_FilteredSignal>(), &Waveform_FilteredSignal::ByteSizeLong,
              &Waveform_FilteredSignal::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Waveform_FilteredSignal, _impl_._cached_size_),
          true,
      },
      "mixxx.track.io.Waveform.FilteredSignal",
  };
}
struct Waveform_FilteredSignalGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr Waveform_FilteredSignalGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Waveform_FilteredSignal_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Waveform_FilteredSignal::InternalGenerateClassData_(
            _default, &Waveform_FilteredSignal_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Waveform_FilteredSignal>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~Waveform_FilteredSignalGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Waveform_FilteredSignal _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Waveform_FilteredSignal>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(Waveform_FilteredSignalGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST Waveform_FilteredSignalGlobalsTypeInternal Waveform_FilteredSignal_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Waveform_FilteredSignal_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Waveform_FilteredSignal_globals_.GetClassData();
#else
  return Waveform_FilteredSignal_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Waveform::_Internal {
 public:
  using HasBits = decltype(::std::declval<Waveform>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Waveform, _impl_._has_bits_);
};

constexpr Waveform::ParseTableT_ Waveform::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Waveform, _impl_._has_bits_),
      0, // no _extensions_
      4, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967280,  // skipmap
      offsetof(ParseTableT_, field_entries),
      4,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallbackLite,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::mixxx::track::io::Waveform>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // optional .mixxx.track.io.Waveform.FilteredSignal signal_filtered = 4;
      {::_pbi::TcParser::FastMtS1,
       {34, 1, 1,
        PROTOBUF_FIELD_OFFSET(Waveform, _impl_.signal_filtered_)}},
      // optional double visual_sample_rate = 1;
      {::_pbi::TcParser::FastF64S1,
       {9, 2, 0,
        PROTOBUF_FIELD_OFFSET(Waveform, _impl_.visual_sample_rate_)}},
      // optional double audio_visual_ratio = 2;
      {::_pbi::TcParser::FastF64S1,
       {17, 3, 0,
        PROTOBUF_FIELD_OFFSET(Waveform, _impl_.audio_visual_ratio_)}},
      // optional .mixxx.track.io.Waveform.Signal signal_all = 3;
      {::_pbi::TcParser::FastMtS1,
       {26, 0, 0,
        PROTOBUF_FIELD_OFFSET(Waveform, _impl_.signal_all_)}},
    }}, {{
      65535, 65535
    }}, {{
      // optional double visual_sample_rate = 1;
      {PROTOBUF_FIELD_OFFSET(Waveform, _impl_.visual_sample_rate_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kDouble)},
      // optional double audio_visual_ratio = 2;
      {PROTOBUF_FIELD_OFFSET(Waveform, _impl_.audio_visual_ratio_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kDouble)},
      // optional .mixxx.track.io.Waveform.Signal signal_all = 3;
      {PROTOBUF_FIELD_OFFSET(Waveform, _impl_.signal_all_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // optional .mixxx.track.io.Waveform.FilteredSignal signal_filtered = 4;
      {PROTOBUF_FIELD_OFFSET(Waveform, _impl_.signal_filtered_), _Internal::kHasBitsOffset + 1, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::mixxx::track::io::Waveform_Signal>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::mixxx::track::io::Waveform_Signal_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::mixxx::track::io::Waveform_FilteredSignal>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::mixxx::track::io::Waveform_FilteredSignal_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr Waveform::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        signal_all_{nullptr},
        signal_filtered_{nullptr},
        visual_sample_rate_{0},
        audio_visual_ratio_{0} {}

template <typename>
constexpr Waveform::Waveform(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::MessageLite(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Waveform::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Waveform(arena);
}
constexpr auto Waveform::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(Waveform), alignof(Waveform));
}
constexpr auto Waveform::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataLite{
      {
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Waveform::MergeImpl,
          ::google::protobuf::MessageLite::GetNewImpl<Waveform>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Waveform::SharedDtor,
          ::google::protobuf::MessageLite::GetClearImpl<Waveform>(), &Waveform::ByteSizeLong,
              &Waveform::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Waveform, _impl_._cached_size_),
          true,
      },
      "mixxx.track.io.Waveform",
  };
}
struct WaveformGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr WaveformGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Waveform_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Waveform::InternalGenerateClassData_(
            _default, &Waveform_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Waveform>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~WaveformGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Waveform _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Waveform>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(WaveformGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST WaveformGlobalsTypeInternal Waveform_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Waveform_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Waveform_globals_.GetClassData();
#else
  return Waveform_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
}  // namespace io
}  // namespace track
}  // namespace mixxx
namespace mixxx {
namespace track {
namespace io {
PROTOBUF_CONSTINIT const uint32_t Waveform_Units_internal_data_[] = {
    131072u, 0u, };
static const char Waveform_Units_names[] = {
    "AMPLITUDE"
    "RMS"
};

static const ::google::protobuf::internal::EnumEntry Waveform_Units_entries[] = {
    {{&Waveform_Units_names[0], 9}, 1},
    {{&Waveform_Units_names[9], 3}, 0},
};

static const int Waveform_Units_entries_by_number[] = {
    1,  // 0 -> RMS
    0,  // 1 -> AMPLITUDE
};

[[nodiscard]] bool Waveform_Units_Parse(::absl::string_view name,
                                  Waveform_Units* PROTOBUF_NONNULL value) {
  int int_value;
  bool success = ::google::protobuf::internal::LookUpEnumValue(
      Waveform_Units_entries, 2, name, &int_value);
  if (success) {
    *value = static_cast<Waveform_Units>(int_value);
  }
  return success;
}
static ::google::protobuf::internal::ExplicitlyConstructed<::std::string>
    Waveform_Units_strings[2] = {};

[[nodiscard]] const ::std::string& Waveform_Units_Name(Waveform_Units value) {
  static const bool kDummy = ::google::protobuf::internal::InitializeEnumStrings(
      Waveform_Units_entries, Waveform_Units_entries_by_number, 2,
      Waveform_Units_strings);
  (void)kDummy;

  int idx = ::google::protobuf::internal::LookUpEnumName(Waveform_Units_entries,
                                  Waveform_Units_entries_by_number,
                                  2, value);
  return idx == -1 ? ::google::protobuf::internal::GetEmptyString() : Waveform_Units_strings[idx].get();
}
// ===================================================================

Waveform_Signal::Waveform_Signal(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::MessageLite(arena, Waveform_Signal_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::MessageLite(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:mixxx.track.io.Waveform.Signal)
}
PROTOBUF_NDEBUG_INLINE Waveform_Signal::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::mixxx::track::io::Waveform_Signal& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        value_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::mixxx::track::io::Waveform_Signal,
              PROTOBUF_FIELD_OFFSET(::mixxx::track::io::Waveform_Signal, _impl_.value_)>()
          , from.value_
        }
     {}

Waveform_Signal::Waveform_Signal(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Waveform_Signal& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::MessageLite(arena, Waveform_Signal_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::MessageLite(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Waveform_Signal* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::std::string>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, units_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, units_),
           offsetof(Impl_, channels_) -
               offsetof(Impl_, units_) +
               sizeof(Impl_::channels_));

  // @@protoc_insertion_point(copy_constructor:mixxx.track.io.Waveform.Signal)
}
PROTOBUF_NDEBUG_INLINE Waveform_Signal::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        value_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::mixxx::track::io::Waveform_Signal,
            PROTOBUF_FIELD_OFFSET(::mixxx::track::io::Waveform_Signal, _impl_.value_)>()
         }
        ,
        channels_{2} {}

inline void Waveform_Signal::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, units_),
           0,
           offsetof(Impl_, min_value_) -
               offsetof(Impl_, units_) +
               sizeof(Impl_::min_value_));
}
Waveform_Signal::~Waveform_Signal() {
  // @@protoc_insertion_point(destructor:mixxx.track.io.Waveform.Signal)
  SharedDtor(*this);
}
inline void Waveform_Signal::SharedDtor(MessageLite& self) {
  Waveform_Signal& this_ = static_cast<Waveform_Signal&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::std::string>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1
const ::google::protobuf::internal::ClassDataLite Waveform_Signal_class_data_ =
    Waveform_Signal::InternalGenerateClassData_(Waveform_Signal_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Waveform_Signal::GetClassData() const {
  return Waveform_Signal_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Waveform_Signal::GetClassData() const {
  return Waveform_Signal_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Waveform_Signal::ParseTableT_
    Waveform_Signal::_table_ =
        Waveform_Signal::InternalGenerateParseTable_(Waveform_Signal_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Waveform_Signal::Clear() {
// @@protoc_insertion_point(message_clear_start:mixxx.track.io.Waveform.Signal)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.value_.Clear();
  }
  if (BatchCheckHasBit(cached_has_bits, 0x0000001eU)) {
    ::memset(&_impl_.units_, 0, static_cast<::size_t>(
        reinterpret_cast<char*>(&_impl_.min_value_) -
        reinterpret_cast<char*>(&_impl_.units_)) + sizeof(_impl_.min_value_));
    _impl_.channels_ = 2;
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::std::string>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Waveform_Signal::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Waveform_Signal& this_ = static_cast<const Waveform_Signal&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Waveform_Signal::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Waveform_Signal& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:mixxx.track.io.Waveform.Signal)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // repeated int32 value = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (int i = 0, n = this_._internal_value_size(); i < n; ++i) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteInt32ToArray(
          1, this_._internal_value().Get(i), target);
    }
  }

  // optional int32 channels = 2 [default = 2];
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    target =
        ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<2>(
            stream, this_._internal_channels(), target);
  }

  // optional .mixxx.track.io.Waveform.Units units = 3 [default = RMS];
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteEnumToArray(
        3, this_._internal_units(), target);
  }

  // optional int32 max_value = 4;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target =
        ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<4>(
            stream, this_._internal_max_value(), target);
  }

  // optional int32 min_value = 5;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    target =
        ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<5>(
            stream, this_._internal_min_value(), target);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target = stream->WriteRaw(
        this_._internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString).data(),
        static_cast<int>(this_._internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString).size()), target);
  }
  // @@protoc_insertion_point(serialize_to_array_end:mixxx.track.io.Waveform.Signal)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Waveform_Signal::ByteSizeLong(const MessageLite& base) {
  const Waveform_Signal& this_ = static_cast<const Waveform_Signal&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Waveform_Signal::ByteSizeLong() const {
  const Waveform_Signal& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:mixxx.track.io.Waveform.Signal)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000001fU)) {
    // repeated int32 value = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ::size_t data_size = ::_pbi::WireFormatLite::Int32Size(
          this_._internal_value());
      ::size_t tag_size = ::size_t{1} *
          ::_pbi::FromIntSize(this_._internal_value_size());
      total_size += tag_size + data_size;
    }
    // optional .mixxx.track.io.Waveform.Units units = 3 [default = RMS];
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::_pbi::WireFormatLite::EnumSize(this_._internal_units());
    }
    // optional int32 max_value = 4;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
          this_._internal_max_value());
    }
    // optional int32 min_value = 5;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
          this_._internal_min_value());
    }
    // optional int32 channels = 2 [default = 2];
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
          this_._internal_channels());
    }
  }
  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    total_size += this_._internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString).size();
  }
  this_._impl_._cached_size_.Set(::_pbi::ToCachedSize(total_size));
  return total_size;
}

void Waveform_Signal::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Waveform_Signal*>(&to_msg);
  auto& from = static_cast<const Waveform_Signal&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:mixxx.track.io.Waveform.Signal)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000001fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_value()->MergeFrom(from._internal_value());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _this->_impl_.units_ = from._impl_.units_;
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _this->_impl_.max_value_ = from._impl_.max_value_;
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      _this->_impl_.min_value_ = from._impl_.min_value_;
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      _this->_impl_.channels_ = from._impl_.channels_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::std::string>(
      from._internal_metadata_);
}

void Waveform_Signal::CopyFrom(const Waveform_Signal& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:mixxx.track.io.Waveform.Signal)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Waveform_Signal::InternalSwap(Waveform_Signal* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.value_.InternalSwap(&other->_impl_.value_);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Waveform_Signal, _impl_.channels_)
      + sizeof(Waveform_Signal::_impl_.channels_)
      - PROTOBUF_FIELD_OFFSET(Waveform_Signal, _impl_.units_)>(
          reinterpret_cast<char*>(&_impl_.units_),
          reinterpret_cast<char*>(&other->_impl_.units_));
}

// ===================================================================

Waveform_FilteredSignal::Waveform_FilteredSignal(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::MessageLite(arena, Waveform_FilteredSignal_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::MessageLite(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:mixxx.track.io.Waveform.FilteredSignal)
}
PROTOBUF_NDEBUG_INLINE Waveform_FilteredSignal::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::mixxx::track::io::Waveform_FilteredSignal& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0} {}

Waveform_FilteredSignal::Waveform_FilteredSignal(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Waveform_FilteredSignal& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::MessageLite(arena, Waveform_FilteredSignal_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::MessageLite(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Waveform_FilteredSignal* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::std::string>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.low_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                ? ::google::protobuf::MessageLite::CopyConstruct(arena, *from._impl_.low_)
                : nullptr;
  _impl_.mid_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                ? ::google::protobuf::MessageLite::CopyConstruct(arena, *from._impl_.mid_)
                : nullptr;
  _impl_.high_ = (CheckHasBit(cached_has_bits, 0x00000004U))
                ? ::google::protobuf::MessageLite::CopyConstruct(arena, *from._impl_.high_)
                : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, low_cutoff_frequency_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, low_cutoff_frequency_),
           offsetof(Impl_, high_cutoff_frequency_) -
               offsetof(Impl_, low_cutoff_frequency_) +
               sizeof(Impl_::high_cutoff_frequency_));

  // @@protoc_insertion_point(copy_constructor:mixxx.track.io.Waveform.FilteredSignal)
}
PROTOBUF_NDEBUG_INLINE Waveform_FilteredSignal::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void Waveform_FilteredSignal::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, low_),
           0,
           offsetof(Impl_, high_cutoff_frequency_) -
               offsetof(Impl_, low_) +
               sizeof(Impl_::high_cutoff_frequency_));
}
Waveform_FilteredSignal::~Waveform_FilteredSignal() {
  // @@protoc_insertion_point(destructor:mixxx.track.io.Waveform.FilteredSignal)
  SharedDtor(*this);
}
inline void Waveform_FilteredSignal::SharedDtor(MessageLite& self) {
  Waveform_FilteredSignal& this_ = static_cast<Waveform_FilteredSignal&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::std::string>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.low_;
  delete this_._impl_.mid_;
  delete this_._impl_.high_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1
const ::google::protobuf::internal::ClassDataLite Waveform_FilteredSignal_class_data_ =
    Waveform_FilteredSignal::InternalGenerateClassData_(Waveform_FilteredSignal_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Waveform_FilteredSignal::GetClassData() const {
  return Waveform_FilteredSignal_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Waveform_FilteredSignal::GetClassData() const {
  return Waveform_FilteredSignal_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Waveform_FilteredSignal::ParseTableT_
    Waveform_FilteredSignal::_table_ =
        Waveform_FilteredSignal::InternalGenerateParseTable_(Waveform_FilteredSignal_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Waveform_FilteredSignal::Clear() {
// @@protoc_insertion_point(message_clear_start:mixxx.track.io.Waveform.FilteredSignal)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(_impl_.low_ != nullptr);
      _impl_.low_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(_impl_.mid_ != nullptr);
      _impl_.mid_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(_impl_.high_ != nullptr);
      _impl_.high_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00000078U)) {
    ::memset(&_impl_.low_cutoff_frequency_, 0, static_cast<::size_t>(
        reinterpret_cast<char*>(&_impl_.high_cutoff_frequency_) -
        reinterpret_cast<char*>(&_impl_.low_cutoff_frequency_)) + sizeof(_impl_.high_cutoff_frequency_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::std::string>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Waveform_FilteredSignal::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Waveform_FilteredSignal& this_ = static_cast<const Waveform_FilteredSignal&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Waveform_FilteredSignal::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Waveform_FilteredSignal& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:mixxx.track.io.Waveform.FilteredSignal)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // optional .mixxx.track.io.Waveform.Signal low = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.low_, this_._impl_.low_->GetCachedSize(), target,
        stream);
  }

  // optional .mixxx.track.io.Waveform.Signal mid = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.mid_, this_._impl_.mid_->GetCachedSize(), target,
        stream);
  }

  // optional .mixxx.track.io.Waveform.Signal high = 3;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        3, *this_._impl_.high_, this_._impl_.high_->GetCachedSize(), target,
        stream);
  }

  // optional double low_cutoff_frequency = 4;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteDoubleToArray(
        4, this_._internal_low_cutoff_frequency(), target);
  }

  // optional double mid_low_cutoff_frequency = 5;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteDoubleToArray(
        5, this_._internal_mid_low_cutoff_frequency(), target);
  }

  // optional double mid_high_cutoff_frequency = 6;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteDoubleToArray(
        6, this_._internal_mid_high_cutoff_frequency(), target);
  }

  // optional double high_cutoff_frequency = 7;
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteDoubleToArray(
        7, this_._internal_high_cutoff_frequency(), target);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target = stream->WriteRaw(
        this_._internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString).data(),
        static_cast<int>(this_._internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString).size()), target);
  }
  // @@protoc_insertion_point(serialize_to_array_end:mixxx.track.io.Waveform.FilteredSignal)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Waveform_FilteredSignal::ByteSizeLong(const MessageLite& base) {
  const Waveform_FilteredSignal& this_ = static_cast<const Waveform_FilteredSignal&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Waveform_FilteredSignal::ByteSizeLong() const {
  const Waveform_FilteredSignal& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:mixxx.track.io.Waveform.FilteredSignal)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  total_size += ::absl::popcount(0x00000078U & cached_has_bits) * 9;
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    // optional .mixxx.track.io.Waveform.Signal low = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.low_);
    }
    // optional .mixxx.track.io.Waveform.Signal mid = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.mid_);
    }
    // optional .mixxx.track.io.Waveform.Signal high = 3;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.high_);
    }
  }
  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    total_size += this_._internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString).size();
  }
  this_._impl_._cached_size_.Set(::_pbi::ToCachedSize(total_size));
  return total_size;
}

void Waveform_FilteredSignal::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Waveform_FilteredSignal*>(&to_msg);
  auto& from = static_cast<const Waveform_FilteredSignal&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:mixxx.track.io.Waveform.FilteredSignal)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000007fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(from._impl_.low_ != nullptr);
      if (_this->_impl_.low_ == nullptr) {
        _this->_impl_.low_ = ::google::protobuf::MessageLite::CopyConstruct(arena, *from._impl_.low_);
      } else {
        _this->_impl_.low_->MergeFrom(*from._impl_.low_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.mid_ != nullptr);
      if (_this->_impl_.mid_ == nullptr) {
        _this->_impl_.mid_ = ::google::protobuf::MessageLite::CopyConstruct(arena, *from._impl_.mid_);
      } else {
        _this->_impl_.mid_->MergeFrom(*from._impl_.mid_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(from._impl_.high_ != nullptr);
      if (_this->_impl_.high_ == nullptr) {
        _this->_impl_.high_ = ::google::protobuf::MessageLite::CopyConstruct(arena, *from._impl_.high_);
      } else {
        _this->_impl_.high_->MergeFrom(*from._impl_.high_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      _this->_impl_.low_cutoff_frequency_ = from._impl_.low_cutoff_frequency_;
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      _this->_impl_.mid_low_cutoff_frequency_ = from._impl_.mid_low_cutoff_frequency_;
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      _this->_impl_.mid_high_cutoff_frequency_ = from._impl_.mid_high_cutoff_frequency_;
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      _this->_impl_.high_cutoff_frequency_ = from._impl_.high_cutoff_frequency_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::std::string>(
      from._internal_metadata_);
}

void Waveform_FilteredSignal::CopyFrom(const Waveform_FilteredSignal& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:mixxx.track.io.Waveform.FilteredSignal)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Waveform_FilteredSignal::InternalSwap(Waveform_FilteredSignal* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Waveform_FilteredSignal, _impl_.high_cutoff_frequency_)
      + sizeof(Waveform_FilteredSignal::_impl_.high_cutoff_frequency_)
      - PROTOBUF_FIELD_OFFSET(Waveform_FilteredSignal, _impl_.low_)>(
          reinterpret_cast<char*>(&_impl_.low_),
          reinterpret_cast<char*>(&other->_impl_.low_));
}

// ===================================================================

Waveform::Waveform(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::MessageLite(arena, Waveform_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::MessageLite(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:mixxx.track.io.Waveform)
}
PROTOBUF_NDEBUG_INLINE Waveform::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::mixxx::track::io::Waveform& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0} {}

Waveform::Waveform(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Waveform& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::MessageLite(arena, Waveform_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::MessageLite(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Waveform* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::std::string>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.signal_all_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                ? ::google::protobuf::MessageLite::CopyConstruct(arena, *from._impl_.signal_all_)
                : nullptr;
  _impl_.signal_filtered_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                ? ::google::protobuf::MessageLite::CopyConstruct(arena, *from._impl_.signal_filtered_)
                : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, visual_sample_rate_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, visual_sample_rate_),
           offsetof(Impl_, audio_visual_ratio_) -
               offsetof(Impl_, visual_sample_rate_) +
               sizeof(Impl_::audio_visual_ratio_));

  // @@protoc_insertion_point(copy_constructor:mixxx.track.io.Waveform)
}
PROTOBUF_NDEBUG_INLINE Waveform::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void Waveform::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, signal_all_),
           0,
           offsetof(Impl_, audio_visual_ratio_) -
               offsetof(Impl_, signal_all_) +
               sizeof(Impl_::audio_visual_ratio_));
}
Waveform::~Waveform() {
  // @@protoc_insertion_point(destructor:mixxx.track.io.Waveform)
  SharedDtor(*this);
}
inline void Waveform::SharedDtor(MessageLite& self) {
  Waveform& this_ = static_cast<Waveform&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::std::string>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.signal_all_;
  delete this_._impl_.signal_filtered_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1
const ::google::protobuf::internal::ClassDataLite Waveform_class_data_ =
    Waveform::InternalGenerateClassData_(Waveform_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Waveform::GetClassData() const {
  return Waveform_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Waveform::GetClassData() const {
  return Waveform_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Waveform::ParseTableT_
    Waveform::_table_ =
        Waveform::InternalGenerateParseTable_(Waveform_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Waveform::Clear() {
// @@protoc_insertion_point(message_clear_start:mixxx.track.io.Waveform)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(_impl_.signal_all_ != nullptr);
      _impl_.signal_all_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(_impl_.signal_filtered_ != nullptr);
      _impl_.signal_filtered_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x0000000cU)) {
    ::memset(&_impl_.visual_sample_rate_, 0, static_cast<::size_t>(
        reinterpret_cast<char*>(&_impl_.audio_visual_ratio_) -
        reinterpret_cast<char*>(&_impl_.visual_sample_rate_)) + sizeof(_impl_.audio_visual_ratio_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::std::string>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Waveform::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Waveform& this_ = static_cast<const Waveform&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Waveform::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Waveform& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:mixxx.track.io.Waveform)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // optional double visual_sample_rate = 1;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteDoubleToArray(
        1, this_._internal_visual_sample_rate(), target);
  }

  // optional double audio_visual_ratio = 2;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteDoubleToArray(
        2, this_._internal_audio_visual_ratio(), target);
  }

  // optional .mixxx.track.io.Waveform.Signal signal_all = 3;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        3, *this_._impl_.signal_all_, this_._impl_.signal_all_->GetCachedSize(), target,
        stream);
  }

  // optional .mixxx.track.io.Waveform.FilteredSignal signal_filtered = 4;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        4, *this_._impl_.signal_filtered_, this_._impl_.signal_filtered_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target = stream->WriteRaw(
        this_._internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString).data(),
        static_cast<int>(this_._internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString).size()), target);
  }
  // @@protoc_insertion_point(serialize_to_array_end:mixxx.track.io.Waveform)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Waveform::ByteSizeLong(const MessageLite& base) {
  const Waveform& this_ = static_cast<const Waveform&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Waveform::ByteSizeLong() const {
  const Waveform& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:mixxx.track.io.Waveform)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  total_size += ::absl::popcount(0x0000000cU & cached_has_bits) * 9;
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // optional .mixxx.track.io.Waveform.Signal signal_all = 3;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.signal_all_);
    }
    // optional .mixxx.track.io.Waveform.FilteredSignal signal_filtered = 4;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.signal_filtered_);
    }
  }
  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    total_size += this_._internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString).size();
  }
  this_._impl_._cached_size_.Set(::_pbi::ToCachedSize(total_size));
  return total_size;
}

void Waveform::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Waveform*>(&to_msg);
  auto& from = static_cast<const Waveform&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:mixxx.track.io.Waveform)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(from._impl_.signal_all_ != nullptr);
      if (_this->_impl_.signal_all_ == nullptr) {
        _this->_impl_.signal_all_ = ::google::protobuf::MessageLite::CopyConstruct(arena, *from._impl_.signal_all_);
      } else {
        _this->_impl_.signal_all_->MergeFrom(*from._impl_.signal_all_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.signal_filtered_ != nullptr);
      if (_this->_impl_.signal_filtered_ == nullptr) {
        _this->_impl_.signal_filtered_ = ::google::protobuf::MessageLite::CopyConstruct(arena, *from._impl_.signal_filtered_);
      } else {
        _this->_impl_.signal_filtered_->MergeFrom(*from._impl_.signal_filtered_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _this->_impl_.visual_sample_rate_ = from._impl_.visual_sample_rate_;
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      _this->_impl_.audio_visual_ratio_ = from._impl_.audio_visual_ratio_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::std::string>(
      from._internal_metadata_);
}

void Waveform::CopyFrom(const Waveform& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:mixxx.track.io.Waveform)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Waveform::InternalSwap(Waveform* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Waveform, _impl_.audio_visual_ratio_)
      + sizeof(Waveform::_impl_.audio_visual_ratio_)
      - PROTOBUF_FIELD_OFFSET(Waveform, _impl_.signal_all_)>(
          reinterpret_cast<char*>(&_impl_.signal_all_),
          reinterpret_cast<char*>(&other->_impl_.signal_all_));
}

// @@protoc_insertion_point(namespace_scope)
}  // namespace io
}  // namespace track
}  // namespace mixxx
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google
// @@protoc_insertion_point(global_scope)
#include "google/protobuf/port_undef.inc"
