/********************************************************************************
** Form generated from reading UI file 'dlgtrackinfomulti.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLGTRACKINFOMULTI_H
#define UI_DLGTRACKINFOMULTI_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DlgTrackInfoMulti
{
public:
    QVBoxLayout *main_layout;
    QGroupBox *tags_groupBox;
    QGridLayout *tags_layout;
    QLabel *lblTitle;
    QComboBox *txtTitle;
    QWidget *coverWidget;
    QHBoxLayout *coverLayout;
    QLabel *lblArtist;
    QComboBox *txtArtist;
    QLabel *lblAlbum;
    QComboBox *txtAlbum;
    QLabel *lblAlbumArtist;
    QComboBox *txtAlbumArtist;
    QWidget *starsWidget;
    QHBoxLayout *starsLayout;
    QLabel *lblComposer;
    QComboBox *txtComposer;
    QLabel *lblYear;
    QComboBox *txtYear;
    QLabel *lblGenre;
    QComboBox *txtGenre;
    QLabel *lblKey;
    QComboBox *txtKey;
    QLabel *lblGrouping;
    QComboBox *txtGrouping;
    QLabel *lblTrackNumber;
    QComboBox *txtTrackNumber;
    QLabel *lblTrackComment;
    QComboBox *txtCommentBox;
    QPlainTextEdit *txtComment;
    QPushButton *btnImportMetadataFromFile;
    QLabel *lblTrackColor;
    QPushButton *btnColorPicker;
    QSpacerItem *spacerColorPicker;
    QGroupBox *properties_groupBox;
    QGridLayout *properties_layout;
    QLabel *lblDuration;
    QLabel *txtDuration;
    QLabel *lblType;
    QLabel *txtType;
    QLabel *lblBpm;
    QLabel *txtBpm;
    QLabel *lblBitrate;
    QLabel *txtBitrate;
    QLabel *lblSamplerate;
    QLabel *txtSamplerate;
    QLabel *lblLocation;
    QLabel *txtLocation;
    QHBoxLayout *file_browser_btn_layout;
    QPushButton *btnOpenFileBrowser;
    QSpacerItem *file_browser_spacer;
    QSpacerItem *properties_bottom_spacer;
    QHBoxLayout *buttonLayout;
    QPushButton *btnReset;
    QSpacerItem *reset_button_spacer;
    QPushButton *btnCancel;
    QPushButton *btnApply;
    QPushButton *btnOK;

    void setupUi(QDialog *DlgTrackInfoMulti)
    {
        if (DlgTrackInfoMulti->objectName().isEmpty())
            DlgTrackInfoMulti->setObjectName("DlgTrackInfoMulti");
        DlgTrackInfoMulti->resize(575, 642);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(DlgTrackInfoMulti->sizePolicy().hasHeightForWidth());
        DlgTrackInfoMulti->setSizePolicy(sizePolicy);
        DlgTrackInfoMulti->setSizeGripEnabled(true);
        main_layout = new QVBoxLayout(DlgTrackInfoMulti);
        main_layout->setSpacing(6);
        main_layout->setContentsMargins(11, 11, 11, 11);
        main_layout->setObjectName("main_layout");
        tags_groupBox = new QGroupBox(DlgTrackInfoMulti);
        tags_groupBox->setObjectName("tags_groupBox");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Ignored, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(tags_groupBox->sizePolicy().hasHeightForWidth());
        tags_groupBox->setSizePolicy(sizePolicy1);
        tags_layout = new QGridLayout(tags_groupBox);
        tags_layout->setSpacing(6);
        tags_layout->setContentsMargins(11, 11, 11, 11);
        tags_layout->setObjectName("tags_layout");
        lblTitle = new QLabel(tags_groupBox);
        lblTitle->setObjectName("lblTitle");
        sizePolicy.setHeightForWidth(lblTitle->sizePolicy().hasHeightForWidth());
        lblTitle->setSizePolicy(sizePolicy);
        QFont font;
        font.setItalic(false);
        lblTitle->setFont(font);
        lblTitle->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        tags_layout->addWidget(lblTitle, 0, 0, 1, 1);

        txtTitle = new QComboBox(tags_groupBox);
        txtTitle->setObjectName("txtTitle");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(txtTitle->sizePolicy().hasHeightForWidth());
        txtTitle->setSizePolicy(sizePolicy2);
        txtTitle->setMinimumSize(QSize(0, 0));

        tags_layout->addWidget(txtTitle, 0, 1, 1, 1);

        coverWidget = new QWidget(tags_groupBox);
        coverWidget->setObjectName("coverWidget");
        sizePolicy.setHeightForWidth(coverWidget->sizePolicy().hasHeightForWidth());
        coverWidget->setSizePolicy(sizePolicy);
        coverLayout = new QHBoxLayout(coverWidget);
        coverLayout->setSpacing(0);
        coverLayout->setContentsMargins(11, 11, 11, 11);
        coverLayout->setObjectName("coverLayout");
        coverLayout->setSizeConstraint(QLayout::SetDefaultConstraint);

        tags_layout->addWidget(coverWidget, 0, 2, 3, 2);

        lblArtist = new QLabel(tags_groupBox);
        lblArtist->setObjectName("lblArtist");
        sizePolicy.setHeightForWidth(lblArtist->sizePolicy().hasHeightForWidth());
        lblArtist->setSizePolicy(sizePolicy);
        lblArtist->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        tags_layout->addWidget(lblArtist, 1, 0, 1, 1);

        txtArtist = new QComboBox(tags_groupBox);
        txtArtist->setObjectName("txtArtist");
        sizePolicy2.setHeightForWidth(txtArtist->sizePolicy().hasHeightForWidth());
        txtArtist->setSizePolicy(sizePolicy2);
        txtArtist->setMinimumSize(QSize(0, 0));

        tags_layout->addWidget(txtArtist, 1, 1, 1, 1);

        lblAlbum = new QLabel(tags_groupBox);
        lblAlbum->setObjectName("lblAlbum");
        sizePolicy.setHeightForWidth(lblAlbum->sizePolicy().hasHeightForWidth());
        lblAlbum->setSizePolicy(sizePolicy);
        lblAlbum->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        tags_layout->addWidget(lblAlbum, 2, 0, 1, 1);

        txtAlbum = new QComboBox(tags_groupBox);
        txtAlbum->setObjectName("txtAlbum");
        sizePolicy2.setHeightForWidth(txtAlbum->sizePolicy().hasHeightForWidth());
        txtAlbum->setSizePolicy(sizePolicy2);
        txtAlbum->setMinimumSize(QSize(0, 0));

        tags_layout->addWidget(txtAlbum, 2, 1, 1, 1);

        lblAlbumArtist = new QLabel(tags_groupBox);
        lblAlbumArtist->setObjectName("lblAlbumArtist");
        sizePolicy.setHeightForWidth(lblAlbumArtist->sizePolicy().hasHeightForWidth());
        lblAlbumArtist->setSizePolicy(sizePolicy);
        lblAlbumArtist->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        tags_layout->addWidget(lblAlbumArtist, 3, 0, 1, 1);

        txtAlbumArtist = new QComboBox(tags_groupBox);
        txtAlbumArtist->setObjectName("txtAlbumArtist");
        sizePolicy2.setHeightForWidth(txtAlbumArtist->sizePolicy().hasHeightForWidth());
        txtAlbumArtist->setSizePolicy(sizePolicy2);
        txtAlbumArtist->setMinimumSize(QSize(0, 0));

        tags_layout->addWidget(txtAlbumArtist, 3, 1, 1, 1);

        starsWidget = new QWidget(tags_groupBox);
        starsWidget->setObjectName("starsWidget");
        sizePolicy.setHeightForWidth(starsWidget->sizePolicy().hasHeightForWidth());
        starsWidget->setSizePolicy(sizePolicy);
        starsLayout = new QHBoxLayout(starsWidget);
        starsLayout->setSpacing(0);
        starsLayout->setContentsMargins(11, 11, 11, 11);
        starsLayout->setObjectName("starsLayout");
        starsLayout->setSizeConstraint(QLayout::SetDefaultConstraint);

        tags_layout->addWidget(starsWidget, 3, 2, 1, 2);

        lblComposer = new QLabel(tags_groupBox);
        lblComposer->setObjectName("lblComposer");
        lblComposer->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        tags_layout->addWidget(lblComposer, 4, 0, 1, 1);

        txtComposer = new QComboBox(tags_groupBox);
        txtComposer->setObjectName("txtComposer");
        sizePolicy2.setHeightForWidth(txtComposer->sizePolicy().hasHeightForWidth());
        txtComposer->setSizePolicy(sizePolicy2);
        txtComposer->setMinimumSize(QSize(0, 0));

        tags_layout->addWidget(txtComposer, 4, 1, 1, 1);

        lblYear = new QLabel(tags_groupBox);
        lblYear->setObjectName("lblYear");
        lblYear->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        tags_layout->addWidget(lblYear, 4, 2, 1, 1);

        txtYear = new QComboBox(tags_groupBox);
        txtYear->setObjectName("txtYear");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(txtYear->sizePolicy().hasHeightForWidth());
        txtYear->setSizePolicy(sizePolicy3);

        tags_layout->addWidget(txtYear, 4, 3, 1, 1);

        lblGenre = new QLabel(tags_groupBox);
        lblGenre->setObjectName("lblGenre");
        lblGenre->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        tags_layout->addWidget(lblGenre, 5, 0, 1, 1);

        txtGenre = new QComboBox(tags_groupBox);
        txtGenre->setObjectName("txtGenre");
        sizePolicy2.setHeightForWidth(txtGenre->sizePolicy().hasHeightForWidth());
        txtGenre->setSizePolicy(sizePolicy2);
        txtGenre->setMinimumSize(QSize(0, 0));

        tags_layout->addWidget(txtGenre, 5, 1, 1, 1);

        lblKey = new QLabel(tags_groupBox);
        lblKey->setObjectName("lblKey");
        lblKey->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        tags_layout->addWidget(lblKey, 5, 2, 1, 1);

        txtKey = new QComboBox(tags_groupBox);
        txtKey->setObjectName("txtKey");
        sizePolicy3.setHeightForWidth(txtKey->sizePolicy().hasHeightForWidth());
        txtKey->setSizePolicy(sizePolicy3);
        txtKey->setMinimumSize(QSize(0, 0));

        tags_layout->addWidget(txtKey, 5, 3, 1, 1);

        lblGrouping = new QLabel(tags_groupBox);
        lblGrouping->setObjectName("lblGrouping");
        lblGrouping->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        tags_layout->addWidget(lblGrouping, 6, 0, 1, 1);

        txtGrouping = new QComboBox(tags_groupBox);
        txtGrouping->setObjectName("txtGrouping");
        sizePolicy2.setHeightForWidth(txtGrouping->sizePolicy().hasHeightForWidth());
        txtGrouping->setSizePolicy(sizePolicy2);
        txtGrouping->setMinimumSize(QSize(0, 0));

        tags_layout->addWidget(txtGrouping, 6, 1, 1, 1);

        lblTrackNumber = new QLabel(tags_groupBox);
        lblTrackNumber->setObjectName("lblTrackNumber");
        lblTrackNumber->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        tags_layout->addWidget(lblTrackNumber, 6, 2, 1, 1);

        txtTrackNumber = new QComboBox(tags_groupBox);
        txtTrackNumber->setObjectName("txtTrackNumber");
        sizePolicy3.setHeightForWidth(txtTrackNumber->sizePolicy().hasHeightForWidth());
        txtTrackNumber->setSizePolicy(sizePolicy3);
        txtTrackNumber->setMinimumSize(QSize(0, 0));

        tags_layout->addWidget(txtTrackNumber, 6, 3, 1, 1);

        lblTrackComment = new QLabel(tags_groupBox);
        lblTrackComment->setObjectName("lblTrackComment");
        lblTrackComment->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        tags_layout->addWidget(lblTrackComment, 7, 0, 1, 1);

        txtCommentBox = new QComboBox(tags_groupBox);
        txtCommentBox->setObjectName("txtCommentBox");
        sizePolicy2.setHeightForWidth(txtCommentBox->sizePolicy().hasHeightForWidth());
        txtCommentBox->setSizePolicy(sizePolicy2);
        txtCommentBox->setMinimumSize(QSize(0, 0));

        tags_layout->addWidget(txtCommentBox, 7, 1, 1, 3);

        txtComment = new QPlainTextEdit(tags_groupBox);
        txtComment->setObjectName("txtComment");
        sizePolicy2.setHeightForWidth(txtComment->sizePolicy().hasHeightForWidth());
        txtComment->setSizePolicy(sizePolicy2);
        txtComment->setTabChangesFocus(true);

        tags_layout->addWidget(txtComment, 8, 1, 1, 3);

        btnImportMetadataFromFile = new QPushButton(tags_groupBox);
        btnImportMetadataFromFile->setObjectName("btnImportMetadataFromFile");

        tags_layout->addWidget(btnImportMetadataFromFile, 9, 1, 1, 3);

        lblTrackColor = new QLabel(tags_groupBox);
        lblTrackColor->setObjectName("lblTrackColor");
        lblTrackColor->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        tags_layout->addWidget(lblTrackColor, 10, 0, 1, 1);

        btnColorPicker = new QPushButton(tags_groupBox);
        btnColorPicker->setObjectName("btnColorPicker");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(btnColorPicker->sizePolicy().hasHeightForWidth());
        btnColorPicker->setSizePolicy(sizePolicy4);

        tags_layout->addWidget(btnColorPicker, 10, 1, 1, 3);

        spacerColorPicker = new QSpacerItem(40, 10, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        tags_layout->addItem(spacerColorPicker, 0, 0, 1, 1);

        tags_layout->setColumnStretch(1, 10);

        main_layout->addWidget(tags_groupBox);

        properties_groupBox = new QGroupBox(DlgTrackInfoMulti);
        properties_groupBox->setObjectName("properties_groupBox");
        QSizePolicy sizePolicy5(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(properties_groupBox->sizePolicy().hasHeightForWidth());
        properties_groupBox->setSizePolicy(sizePolicy5);
        properties_layout = new QGridLayout(properties_groupBox);
        properties_layout->setSpacing(6);
        properties_layout->setContentsMargins(11, 11, 11, 11);
        properties_layout->setObjectName("properties_layout");
        lblDuration = new QLabel(properties_groupBox);
        lblDuration->setObjectName("lblDuration");

        properties_layout->addWidget(lblDuration, 0, 0, 1, 1);

        txtDuration = new QLabel(properties_groupBox);
        txtDuration->setObjectName("txtDuration");
        QFont font1;
        font1.setBold(true);
        txtDuration->setFont(font1);
        txtDuration->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        properties_layout->addWidget(txtDuration, 0, 1, 1, 1);

        lblType = new QLabel(properties_groupBox);
        lblType->setObjectName("lblType");

        properties_layout->addWidget(lblType, 0, 2, 1, 1);

        txtType = new QLabel(properties_groupBox);
        txtType->setObjectName("txtType");
        txtType->setFont(font1);
        txtType->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        properties_layout->addWidget(txtType, 0, 3, 1, 1);

        lblBpm = new QLabel(properties_groupBox);
        lblBpm->setObjectName("lblBpm");

        properties_layout->addWidget(lblBpm, 1, 0, 1, 1);

        txtBpm = new QLabel(properties_groupBox);
        txtBpm->setObjectName("txtBpm");
        txtBpm->setFont(font1);
        txtBpm->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        properties_layout->addWidget(txtBpm, 1, 1, 1, 1);

        lblBitrate = new QLabel(properties_groupBox);
        lblBitrate->setObjectName("lblBitrate");

        properties_layout->addWidget(lblBitrate, 1, 2, 1, 1);

        txtBitrate = new QLabel(properties_groupBox);
        txtBitrate->setObjectName("txtBitrate");
        txtBitrate->setFont(font1);
        txtBitrate->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        properties_layout->addWidget(txtBitrate, 1, 3, 1, 1);

        lblSamplerate = new QLabel(properties_groupBox);
        lblSamplerate->setObjectName("lblSamplerate");

        properties_layout->addWidget(lblSamplerate, 2, 2, 1, 1);

        txtSamplerate = new QLabel(properties_groupBox);
        txtSamplerate->setObjectName("txtSamplerate");
        txtSamplerate->setFont(font1);

        properties_layout->addWidget(txtSamplerate, 2, 3, 1, 1);

        lblLocation = new QLabel(properties_groupBox);
        lblLocation->setObjectName("lblLocation");
        lblLocation->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        properties_layout->addWidget(lblLocation, 3, 0, 1, 1);

        txtLocation = new QLabel(properties_groupBox);
        txtLocation->setObjectName("txtLocation");
        txtLocation->setWordWrap(true);
        txtLocation->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        properties_layout->addWidget(txtLocation, 3, 1, 1, 3);

        file_browser_btn_layout = new QHBoxLayout();
        file_browser_btn_layout->setSpacing(6);
        file_browser_btn_layout->setObjectName("file_browser_btn_layout");
        btnOpenFileBrowser = new QPushButton(properties_groupBox);
        btnOpenFileBrowser->setObjectName("btnOpenFileBrowser");

        file_browser_btn_layout->addWidget(btnOpenFileBrowser);

        file_browser_spacer = new QSpacerItem(40, 10, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        file_browser_btn_layout->addItem(file_browser_spacer);


        properties_layout->addLayout(file_browser_btn_layout, 4, 1, 1, 3);

        properties_bottom_spacer = new QSpacerItem(40, 10, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        properties_layout->addItem(properties_bottom_spacer, 5, 1, 1, 3);

        properties_layout->setColumnStretch(1, 1);
        properties_layout->setColumnStretch(3, 1);

        main_layout->addWidget(properties_groupBox);

        buttonLayout = new QHBoxLayout();
        buttonLayout->setSpacing(6);
        buttonLayout->setObjectName("buttonLayout");
        btnReset = new QPushButton(DlgTrackInfoMulti);
        btnReset->setObjectName("btnReset");

        buttonLayout->addWidget(btnReset);

        reset_button_spacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        buttonLayout->addItem(reset_button_spacer);

        btnCancel = new QPushButton(DlgTrackInfoMulti);
        btnCancel->setObjectName("btnCancel");
        btnCancel->setMinimumSize(QSize(100, 0));

        buttonLayout->addWidget(btnCancel);

        btnApply = new QPushButton(DlgTrackInfoMulti);
        btnApply->setObjectName("btnApply");
        btnApply->setMinimumSize(QSize(100, 0));

        buttonLayout->addWidget(btnApply);

        btnOK = new QPushButton(DlgTrackInfoMulti);
        btnOK->setObjectName("btnOK");
        btnOK->setMinimumSize(QSize(100, 0));

        buttonLayout->addWidget(btnOK);


        main_layout->addLayout(buttonLayout);

        QWidget::setTabOrder(txtTitle, txtArtist);
        QWidget::setTabOrder(txtArtist, txtAlbum);
        QWidget::setTabOrder(txtAlbum, txtAlbumArtist);
        QWidget::setTabOrder(txtAlbumArtist, txtComposer);
        QWidget::setTabOrder(txtComposer, txtGenre);
        QWidget::setTabOrder(txtGenre, txtGrouping);
        QWidget::setTabOrder(txtGrouping, txtCommentBox);
        QWidget::setTabOrder(txtCommentBox, txtComment);
        QWidget::setTabOrder(txtComment, txtYear);
        QWidget::setTabOrder(txtYear, txtKey);
        QWidget::setTabOrder(txtKey, txtTrackNumber);
        QWidget::setTabOrder(txtTrackNumber, btnImportMetadataFromFile);
        QWidget::setTabOrder(btnImportMetadataFromFile, btnColorPicker);
        QWidget::setTabOrder(btnColorPicker, btnCancel);
        QWidget::setTabOrder(btnCancel, btnApply);
        QWidget::setTabOrder(btnApply, btnOK);

        retranslateUi(DlgTrackInfoMulti);

        btnCancel->setDefault(false);
        btnApply->setDefault(true);
        btnOK->setDefault(false);


        QMetaObject::connectSlotsByName(DlgTrackInfoMulti);
    } // setupUi

    void retranslateUi(QDialog *DlgTrackInfoMulti)
    {
        DlgTrackInfoMulti->setWindowTitle(QCoreApplication::translate("DlgTrackInfoMulti", "Track Editor", nullptr));
#if QT_CONFIG(tooltip)
        DlgTrackInfoMulti->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        lblTitle->setText(QCoreApplication::translate("DlgTrackInfoMulti", "Title", nullptr));
        lblArtist->setText(QCoreApplication::translate("DlgTrackInfoMulti", "Artist", nullptr));
        lblAlbum->setText(QCoreApplication::translate("DlgTrackInfoMulti", "Album", nullptr));
        lblAlbumArtist->setText(QCoreApplication::translate("DlgTrackInfoMulti", "Album Artist", nullptr));
        lblComposer->setText(QCoreApplication::translate("DlgTrackInfoMulti", "Composer", nullptr));
        lblYear->setText(QCoreApplication::translate("DlgTrackInfoMulti", "Year", nullptr));
        lblGenre->setText(QCoreApplication::translate("DlgTrackInfoMulti", "Genre", nullptr));
        lblKey->setText(QCoreApplication::translate("DlgTrackInfoMulti", "Key", nullptr));
        lblGrouping->setText(QCoreApplication::translate("DlgTrackInfoMulti", "Grouping", nullptr));
        lblTrackNumber->setText(QCoreApplication::translate("DlgTrackInfoMulti", "Track #", nullptr));
        lblTrackComment->setText(QCoreApplication::translate("DlgTrackInfoMulti", "Comments", nullptr));
        btnImportMetadataFromFile->setText(QCoreApplication::translate("DlgTrackInfoMulti", "Re-Import Metadata from files", nullptr));
        lblTrackColor->setText(QCoreApplication::translate("DlgTrackInfoMulti", "Color", nullptr));
        lblDuration->setText(QCoreApplication::translate("DlgTrackInfoMulti", "Duration:", nullptr));
        txtDuration->setText(QString());
        lblType->setText(QCoreApplication::translate("DlgTrackInfoMulti", "Filetype:", nullptr));
        txtType->setText(QString());
        lblBpm->setText(QCoreApplication::translate("DlgTrackInfoMulti", "BPM:", nullptr));
        txtBpm->setText(QString());
        lblBitrate->setText(QCoreApplication::translate("DlgTrackInfoMulti", "Bitrate:", nullptr));
        txtBitrate->setText(QString());
        lblSamplerate->setText(QCoreApplication::translate("DlgTrackInfoMulti", "Samplerate:", nullptr));
        txtSamplerate->setText(QString());
        lblLocation->setText(QCoreApplication::translate("DlgTrackInfoMulti", "Location:", nullptr));
        txtLocation->setText(QString());
        btnOpenFileBrowser->setText(QCoreApplication::translate("DlgTrackInfoMulti", "Open in File Browser", nullptr));
#if QT_CONFIG(tooltip)
        btnReset->setToolTip(QCoreApplication::translate("DlgTrackInfoMulti", "Discard changes and reload saved values.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnReset->setText(QCoreApplication::translate("DlgTrackInfoMulti", "&Reset", nullptr));
#if QT_CONFIG(tooltip)
        btnCancel->setToolTip(QCoreApplication::translate("DlgTrackInfoMulti", "Discard changes and close the window.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnCancel->setText(QCoreApplication::translate("DlgTrackInfoMulti", "&Cancel", nullptr));
#if QT_CONFIG(tooltip)
        btnApply->setToolTip(QCoreApplication::translate("DlgTrackInfoMulti", "Save changes and keep the window open.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnApply->setText(QCoreApplication::translate("DlgTrackInfoMulti", "&Apply", nullptr));
#if QT_CONFIG(tooltip)
        btnOK->setToolTip(QCoreApplication::translate("DlgTrackInfoMulti", "Save changes and close the window.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnOK->setText(QCoreApplication::translate("DlgTrackInfoMulti", "&OK", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgTrackInfoMulti: public Ui_DlgTrackInfoMulti {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLGTRACKINFOMULTI_H
