/********************************************************************************
** Form generated from reading UI file 'dlgprefwaveformdlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLGPREFWAVEFORMDLG_H
#define UI_DLGPREFWAVEFORMDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DlgPrefWaveformDlg
{
public:
    QGridLayout *gridLayout;
    QGridLayout *gridLayout_2;
    QLabel *waveformTypeLabel;
    QLabel *waveformOverviewLabel;
    QComboBox *waveformOverviewComboBox;
    QComboBox *waveformTypeComboBox;
    QLabel *frameRateLabel;
    QSlider *frameRateSlider;
    QSpinBox *frameRateSpinBox;
    QLabel *endOfTrackWarningTimeLabel;
    QSlider *endOfTrackWarningTimeSlider;
    QSpinBox *endOfTrackWarningTimeSpinBox;
    QLabel *BeatgridWaveform;
    QSlider *beatGridAlphaSlider;
    QSpinBox *beatGridAlphaSpinBox;
    QLabel *label;
    QSlider *playMarkerPositionSlider;
    QLabel *defaultZoomLabel;
    QComboBox *defaultZoomComboBox;
    QCheckBox *synchronizeZoomCheckBox;
    QCheckBox *normalizeOverviewCheckBox;
    QLabel *visualGainLabel;
    QGridLayout *gridLayout_3;
    QDoubleSpinBox *midVisualGain;
    QLabel *visualGainLabel_5;
    QLabel *visualGainLabel_4;
    QLabel *visualGainLabel_2;
    QDoubleSpinBox *highVisualGain;
    QDoubleSpinBox *lowVisualGain;
    QLabel *visualGainLabel_3;
    QDoubleSpinBox *allVisualGain;
    QLabel *untilMarkLabel;
    QGridLayout *gridLayout_4;
    QCheckBox *untilMarkShowBeatsCheckBox;
    QLabel *untilMarkAlignLabel;
    QLabel *untilMarkTextPointSizeLabel;
    QLabel *untilMarkTextHeightLimitLabel;
    QCheckBox *untilMarkShowTimeCheckBox;
    QComboBox *untilMarkAlignComboBox;
    QSpinBox *untilMarkTextPointSizeSpinBox;
    QComboBox *untilMarkTextHeightLimitComboBox;
    QLabel *requiresGLSLLabel;
    QLabel *cachedWaveforms;
    QGridLayout *cachingGridLayout;
    QPushButton *clearCachedWaveforms;
    QLabel *waveformCachingInfo;
    QLabel *waveformDiskUsage;
    QCheckBox *enableWaveformCaching;
    QCheckBox *enableWaveformGenerationWithAnalysis;
    QGroupBox *openGLStatus;
    QGridLayout *GridLayout1;
    QLabel *openGlStatusData;
    QLabel *openGlStatusLabel_2;
    QLabel *frameRateAverage;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *DlgPrefWaveformDlg)
    {
        if (DlgPrefWaveformDlg->objectName().isEmpty())
            DlgPrefWaveformDlg->setObjectName("DlgPrefWaveformDlg");
        DlgPrefWaveformDlg->resize(677, 528);
        gridLayout = new QGridLayout(DlgPrefWaveformDlg);
        gridLayout->setObjectName("gridLayout");
        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        waveformTypeLabel = new QLabel(DlgPrefWaveformDlg);
        waveformTypeLabel->setObjectName("waveformTypeLabel");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(waveformTypeLabel->sizePolicy().hasHeightForWidth());
        waveformTypeLabel->setSizePolicy(sizePolicy);
        waveformTypeLabel->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        gridLayout_2->addWidget(waveformTypeLabel, 0, 0, 1, 1);

        waveformOverviewLabel = new QLabel(DlgPrefWaveformDlg);
        waveformOverviewLabel->setObjectName("waveformOverviewLabel");

        gridLayout_2->addWidget(waveformOverviewLabel, 1, 0, 1, 1);

        waveformOverviewComboBox = new QComboBox(DlgPrefWaveformDlg);
        waveformOverviewComboBox->setObjectName("waveformOverviewComboBox");

        gridLayout_2->addWidget(waveformOverviewComboBox, 1, 1, 1, 3);

        waveformTypeComboBox = new QComboBox(DlgPrefWaveformDlg);
        waveformTypeComboBox->setObjectName("waveformTypeComboBox");

        gridLayout_2->addWidget(waveformTypeComboBox, 0, 1, 1, 3);

        frameRateLabel = new QLabel(DlgPrefWaveformDlg);
        frameRateLabel->setObjectName("frameRateLabel");
        frameRateLabel->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        gridLayout_2->addWidget(frameRateLabel, 2, 0, 1, 1);

        frameRateSlider = new QSlider(DlgPrefWaveformDlg);
        frameRateSlider->setObjectName("frameRateSlider");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(frameRateSlider->sizePolicy().hasHeightForWidth());
        frameRateSlider->setSizePolicy(sizePolicy1);
        frameRateSlider->setMaximumSize(QSize(16777215, 16777215));
        frameRateSlider->setMinimum(10);
        frameRateSlider->setMaximum(60);
        frameRateSlider->setValue(30);
        frameRateSlider->setOrientation(Qt::Horizontal);

        gridLayout_2->addWidget(frameRateSlider, 2, 1, 1, 2);

        frameRateSpinBox = new QSpinBox(DlgPrefWaveformDlg);
        frameRateSpinBox->setObjectName("frameRateSpinBox");
        frameRateSpinBox->setMinimum(10);
        frameRateSpinBox->setMaximum(60);
        frameRateSpinBox->setValue(30);

        gridLayout_2->addWidget(frameRateSpinBox, 2, 3, 1, 1);

        endOfTrackWarningTimeLabel = new QLabel(DlgPrefWaveformDlg);
        endOfTrackWarningTimeLabel->setObjectName("endOfTrackWarningTimeLabel");
        endOfTrackWarningTimeLabel->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        gridLayout_2->addWidget(endOfTrackWarningTimeLabel, 3, 0, 1, 1);

        endOfTrackWarningTimeSlider = new QSlider(DlgPrefWaveformDlg);
        endOfTrackWarningTimeSlider->setObjectName("endOfTrackWarningTimeSlider");
        sizePolicy1.setHeightForWidth(endOfTrackWarningTimeSlider->sizePolicy().hasHeightForWidth());
        endOfTrackWarningTimeSlider->setSizePolicy(sizePolicy1);
        endOfTrackWarningTimeSlider->setMaximumSize(QSize(16777215, 16777215));
        endOfTrackWarningTimeSlider->setMinimum(0);
        endOfTrackWarningTimeSlider->setMaximum(60);
        endOfTrackWarningTimeSlider->setValue(30);
        endOfTrackWarningTimeSlider->setOrientation(Qt::Horizontal);

        gridLayout_2->addWidget(endOfTrackWarningTimeSlider, 3, 1, 1, 2);

        endOfTrackWarningTimeSpinBox = new QSpinBox(DlgPrefWaveformDlg);
        endOfTrackWarningTimeSpinBox->setObjectName("endOfTrackWarningTimeSpinBox");
        endOfTrackWarningTimeSpinBox->setMinimum(0);
        endOfTrackWarningTimeSpinBox->setMaximum(60);
        endOfTrackWarningTimeSpinBox->setValue(30);

        gridLayout_2->addWidget(endOfTrackWarningTimeSpinBox, 3, 3, 1, 1);

        BeatgridWaveform = new QLabel(DlgPrefWaveformDlg);
        BeatgridWaveform->setObjectName("BeatgridWaveform");
        BeatgridWaveform->setMidLineWidth(6);
        BeatgridWaveform->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        gridLayout_2->addWidget(BeatgridWaveform, 4, 0, 1, 1);

        beatGridAlphaSlider = new QSlider(DlgPrefWaveformDlg);
        beatGridAlphaSlider->setObjectName("beatGridAlphaSlider");
        beatGridAlphaSlider->setMaximum(100);
        beatGridAlphaSlider->setValue(90);
        beatGridAlphaSlider->setOrientation(Qt::Horizontal);

        gridLayout_2->addWidget(beatGridAlphaSlider, 4, 1, 1, 2);

        beatGridAlphaSpinBox = new QSpinBox(DlgPrefWaveformDlg);
        beatGridAlphaSpinBox->setObjectName("beatGridAlphaSpinBox");
        beatGridAlphaSpinBox->setMaximum(100);
        beatGridAlphaSpinBox->setValue(90);

        gridLayout_2->addWidget(beatGridAlphaSpinBox, 4, 3, 1, 1);

        label = new QLabel(DlgPrefWaveformDlg);
        label->setObjectName("label");

        gridLayout_2->addWidget(label, 5, 0, 1, 1);

        playMarkerPositionSlider = new QSlider(DlgPrefWaveformDlg);
        playMarkerPositionSlider->setObjectName("playMarkerPositionSlider");
        sizePolicy1.setHeightForWidth(playMarkerPositionSlider->sizePolicy().hasHeightForWidth());
        playMarkerPositionSlider->setSizePolicy(sizePolicy1);
        playMarkerPositionSlider->setMaximumSize(QSize(16777215, 16777215));
        playMarkerPositionSlider->setMinimum(0);
        playMarkerPositionSlider->setMaximum(100);
        playMarkerPositionSlider->setValue(50);
        playMarkerPositionSlider->setOrientation(Qt::Horizontal);

        gridLayout_2->addWidget(playMarkerPositionSlider, 5, 1, 1, 3);

        defaultZoomLabel = new QLabel(DlgPrefWaveformDlg);
        defaultZoomLabel->setObjectName("defaultZoomLabel");
        defaultZoomLabel->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        gridLayout_2->addWidget(defaultZoomLabel, 6, 0, 1, 1);

        defaultZoomComboBox = new QComboBox(DlgPrefWaveformDlg);
        defaultZoomComboBox->setObjectName("defaultZoomComboBox");

        gridLayout_2->addWidget(defaultZoomComboBox, 6, 1, 1, 3);

        synchronizeZoomCheckBox = new QCheckBox(DlgPrefWaveformDlg);
        synchronizeZoomCheckBox->setObjectName("synchronizeZoomCheckBox");

        gridLayout_2->addWidget(synchronizeZoomCheckBox, 7, 1, 1, 3);

        normalizeOverviewCheckBox = new QCheckBox(DlgPrefWaveformDlg);
        normalizeOverviewCheckBox->setObjectName("normalizeOverviewCheckBox");

        gridLayout_2->addWidget(normalizeOverviewCheckBox, 9, 1, 1, 1);

        visualGainLabel = new QLabel(DlgPrefWaveformDlg);
        visualGainLabel->setObjectName("visualGainLabel");
        visualGainLabel->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        gridLayout_2->addWidget(visualGainLabel, 10, 0, 1, 1);

        gridLayout_3 = new QGridLayout();
        gridLayout_3->setObjectName("gridLayout_3");
        midVisualGain = new QDoubleSpinBox(DlgPrefWaveformDlg);
        midVisualGain->setObjectName("midVisualGain");
        midVisualGain->setDecimals(1);
        midVisualGain->setMinimum(0.500000000000000);
        midVisualGain->setMaximum(5.000000000000000);
        midVisualGain->setSingleStep(0.100000000000000);

        gridLayout_3->addWidget(midVisualGain, 1, 2, 1, 1);

        visualGainLabel_5 = new QLabel(DlgPrefWaveformDlg);
        visualGainLabel_5->setObjectName("visualGainLabel_5");
        visualGainLabel_5->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        gridLayout_3->addWidget(visualGainLabel_5, 0, 3, 1, 1);

        visualGainLabel_4 = new QLabel(DlgPrefWaveformDlg);
        visualGainLabel_4->setObjectName("visualGainLabel_4");
        visualGainLabel_4->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        gridLayout_3->addWidget(visualGainLabel_4, 0, 2, 1, 1);

        visualGainLabel_2 = new QLabel(DlgPrefWaveformDlg);
        visualGainLabel_2->setObjectName("visualGainLabel_2");
        visualGainLabel_2->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        gridLayout_3->addWidget(visualGainLabel_2, 0, 0, 1, 1);

        highVisualGain = new QDoubleSpinBox(DlgPrefWaveformDlg);
        highVisualGain->setObjectName("highVisualGain");
        highVisualGain->setDecimals(1);
        highVisualGain->setMinimum(0.500000000000000);
        highVisualGain->setMaximum(5.000000000000000);
        highVisualGain->setSingleStep(0.100000000000000);

        gridLayout_3->addWidget(highVisualGain, 1, 3, 1, 1);

        lowVisualGain = new QDoubleSpinBox(DlgPrefWaveformDlg);
        lowVisualGain->setObjectName("lowVisualGain");
        lowVisualGain->setDecimals(1);
        lowVisualGain->setMinimum(0.500000000000000);
        lowVisualGain->setMaximum(5.000000000000000);
        lowVisualGain->setSingleStep(0.100000000000000);

        gridLayout_3->addWidget(lowVisualGain, 1, 1, 1, 1);

        visualGainLabel_3 = new QLabel(DlgPrefWaveformDlg);
        visualGainLabel_3->setObjectName("visualGainLabel_3");
        visualGainLabel_3->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        gridLayout_3->addWidget(visualGainLabel_3, 0, 1, 1, 1);

        allVisualGain = new QDoubleSpinBox(DlgPrefWaveformDlg);
        allVisualGain->setObjectName("allVisualGain");
        allVisualGain->setDecimals(1);
        allVisualGain->setMinimum(0.500000000000000);
        allVisualGain->setMaximum(5.000000000000000);
        allVisualGain->setSingleStep(0.100000000000000);

        gridLayout_3->addWidget(allVisualGain, 1, 0, 1, 1);


        gridLayout_2->addLayout(gridLayout_3, 10, 1, 1, 3);

        untilMarkLabel = new QLabel(DlgPrefWaveformDlg);
        untilMarkLabel->setObjectName("untilMarkLabel");
        untilMarkLabel->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        gridLayout_2->addWidget(untilMarkLabel, 11, 0, 1, 1);

        gridLayout_4 = new QGridLayout();
        gridLayout_4->setObjectName("gridLayout_4");
        untilMarkShowBeatsCheckBox = new QCheckBox(DlgPrefWaveformDlg);
        untilMarkShowBeatsCheckBox->setObjectName("untilMarkShowBeatsCheckBox");

        gridLayout_4->addWidget(untilMarkShowBeatsCheckBox, 0, 0, 1, 1);

        untilMarkAlignLabel = new QLabel(DlgPrefWaveformDlg);
        untilMarkAlignLabel->setObjectName("untilMarkAlignLabel");
        untilMarkAlignLabel->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        gridLayout_4->addWidget(untilMarkAlignLabel, 0, 1, 1, 1);

        untilMarkTextPointSizeLabel = new QLabel(DlgPrefWaveformDlg);
        untilMarkTextPointSizeLabel->setObjectName("untilMarkTextPointSizeLabel");
        untilMarkTextPointSizeLabel->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        gridLayout_4->addWidget(untilMarkTextPointSizeLabel, 0, 2, 1, 1);

        untilMarkTextHeightLimitLabel = new QLabel(DlgPrefWaveformDlg);
        untilMarkTextHeightLimitLabel->setObjectName("untilMarkTextHeightLimitLabel");
        untilMarkTextHeightLimitLabel->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        gridLayout_4->addWidget(untilMarkTextHeightLimitLabel, 0, 3, 1, 1);

        untilMarkShowTimeCheckBox = new QCheckBox(DlgPrefWaveformDlg);
        untilMarkShowTimeCheckBox->setObjectName("untilMarkShowTimeCheckBox");

        gridLayout_4->addWidget(untilMarkShowTimeCheckBox, 1, 0, 1, 1);

        untilMarkAlignComboBox = new QComboBox(DlgPrefWaveformDlg);
        untilMarkAlignComboBox->setObjectName("untilMarkAlignComboBox");

        gridLayout_4->addWidget(untilMarkAlignComboBox, 1, 1, 1, 1);

        untilMarkTextPointSizeSpinBox = new QSpinBox(DlgPrefWaveformDlg);
        untilMarkTextPointSizeSpinBox->setObjectName("untilMarkTextPointSizeSpinBox");
        untilMarkTextPointSizeSpinBox->setMinimum(10);
        untilMarkTextPointSizeSpinBox->setMaximum(100);
        untilMarkTextPointSizeSpinBox->setValue(30);

        gridLayout_4->addWidget(untilMarkTextPointSizeSpinBox, 1, 2, 1, 1);

        untilMarkTextHeightLimitComboBox = new QComboBox(DlgPrefWaveformDlg);
        untilMarkTextHeightLimitComboBox->setObjectName("untilMarkTextHeightLimitComboBox");

        gridLayout_4->addWidget(untilMarkTextHeightLimitComboBox, 1, 3, 1, 1);


        gridLayout_2->addLayout(gridLayout_4, 11, 1, 1, 3);

        requiresGLSLLabel = new QLabel(DlgPrefWaveformDlg);
        requiresGLSLLabel->setObjectName("requiresGLSLLabel");
        requiresGLSLLabel->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        gridLayout_2->addWidget(requiresGLSLLabel, 14, 1, 1, 2);

        cachedWaveforms = new QLabel(DlgPrefWaveformDlg);
        cachedWaveforms->setObjectName("cachedWaveforms");
        cachedWaveforms->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        gridLayout_2->addWidget(cachedWaveforms, 15, 0, 1, 1);

        cachingGridLayout = new QGridLayout();
        cachingGridLayout->setObjectName("cachingGridLayout");
        clearCachedWaveforms = new QPushButton(DlgPrefWaveformDlg);
        clearCachedWaveforms->setObjectName("clearCachedWaveforms");

        cachingGridLayout->addWidget(clearCachedWaveforms, 4, 0, 1, 1);

        waveformCachingInfo = new QLabel(DlgPrefWaveformDlg);
        waveformCachingInfo->setObjectName("waveformCachingInfo");
        waveformCachingInfo->setWordWrap(true);

        cachingGridLayout->addWidget(waveformCachingInfo, 2, 0, 1, 2);

        waveformDiskUsage = new QLabel(DlgPrefWaveformDlg);
        waveformDiskUsage->setObjectName("waveformDiskUsage");
        waveformDiskUsage->setText(QString::fromUtf8("PLACEHOLDER FOR DISK USAGE"));
        waveformDiskUsage->setWordWrap(true);
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(waveformDiskUsage->sizePolicy().hasHeightForWidth());
        waveformDiskUsage->setSizePolicy(sizePolicy2);

        cachingGridLayout->addWidget(waveformDiskUsage, 3, 0, 1, 1);

        enableWaveformCaching = new QCheckBox(DlgPrefWaveformDlg);
        enableWaveformCaching->setObjectName("enableWaveformCaching");

        cachingGridLayout->addWidget(enableWaveformCaching, 0, 0, 1, 2);

        enableWaveformGenerationWithAnalysis = new QCheckBox(DlgPrefWaveformDlg);
        enableWaveformGenerationWithAnalysis->setObjectName("enableWaveformGenerationWithAnalysis");

        cachingGridLayout->addWidget(enableWaveformGenerationWithAnalysis, 1, 0, 1, 2);


        gridLayout_2->addLayout(cachingGridLayout, 15, 1, 1, 3);

        openGLStatus = new QGroupBox(DlgPrefWaveformDlg);
        openGLStatus->setObjectName("openGLStatus");
        GridLayout1 = new QGridLayout(openGLStatus);
        GridLayout1->setObjectName("GridLayout1");
        openGlStatusData = new QLabel(openGLStatus);
        openGlStatusData->setObjectName("openGlStatusData");
        openGlStatusData->setTextInteractionFlags(Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);
        openGlStatusData->setWordWrap(true);
        sizePolicy2.setHeightForWidth(openGlStatusData->sizePolicy().hasHeightForWidth());
        openGlStatusData->setSizePolicy(sizePolicy2);

        GridLayout1->addWidget(openGlStatusData, 0, 0, 1, 3);

        openGlStatusLabel_2 = new QLabel(openGLStatus);
        openGlStatusLabel_2->setObjectName("openGlStatusLabel_2");
        openGlStatusLabel_2->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        GridLayout1->addWidget(openGlStatusLabel_2, 1, 0, 1, 1);

        frameRateAverage = new QLabel(openGLStatus);
        frameRateAverage->setObjectName("frameRateAverage");

        GridLayout1->addWidget(frameRateAverage, 1, 1, 1, 2);


        gridLayout_2->addWidget(openGLStatus, 17, 0, 1, 4);


        gridLayout->addLayout(gridLayout_2, 0, 0, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 1, 0, 1, 1);

#if QT_CONFIG(shortcut)
        waveformTypeLabel->setBuddy(waveformTypeComboBox);
        frameRateLabel->setBuddy(frameRateSlider);
        endOfTrackWarningTimeLabel->setBuddy(endOfTrackWarningTimeSlider);
        defaultZoomLabel->setBuddy(defaultZoomComboBox);
        visualGainLabel->setBuddy(allVisualGain);
        visualGainLabel_5->setBuddy(allVisualGain);
        visualGainLabel_4->setBuddy(allVisualGain);
        visualGainLabel_2->setBuddy(allVisualGain);
        visualGainLabel_3->setBuddy(allVisualGain);
        untilMarkAlignLabel->setBuddy(untilMarkAlignComboBox);
        untilMarkTextPointSizeLabel->setBuddy(untilMarkTextPointSizeSpinBox);
        untilMarkTextHeightLimitLabel->setBuddy(untilMarkTextHeightLimitComboBox);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(waveformTypeComboBox, waveformOverviewComboBox);
        QWidget::setTabOrder(waveformOverviewComboBox, frameRateSlider);
        QWidget::setTabOrder(frameRateSlider, frameRateSpinBox);
        QWidget::setTabOrder(frameRateSpinBox, endOfTrackWarningTimeSlider);
        QWidget::setTabOrder(endOfTrackWarningTimeSlider, endOfTrackWarningTimeSpinBox);
        QWidget::setTabOrder(endOfTrackWarningTimeSpinBox, beatGridAlphaSlider);
        QWidget::setTabOrder(beatGridAlphaSlider, beatGridAlphaSpinBox);
        QWidget::setTabOrder(beatGridAlphaSpinBox, playMarkerPositionSlider);
        QWidget::setTabOrder(playMarkerPositionSlider, defaultZoomComboBox);
        QWidget::setTabOrder(defaultZoomComboBox, synchronizeZoomCheckBox);
        QWidget::setTabOrder(synchronizeZoomCheckBox, normalizeOverviewCheckBox);
        QWidget::setTabOrder(normalizeOverviewCheckBox, allVisualGain);
        QWidget::setTabOrder(allVisualGain, lowVisualGain);
        QWidget::setTabOrder(lowVisualGain, midVisualGain);
        QWidget::setTabOrder(midVisualGain, highVisualGain);
        QWidget::setTabOrder(highVisualGain, untilMarkShowBeatsCheckBox);
        QWidget::setTabOrder(untilMarkShowBeatsCheckBox, untilMarkShowTimeCheckBox);
        QWidget::setTabOrder(untilMarkShowTimeCheckBox, untilMarkAlignComboBox);
        QWidget::setTabOrder(untilMarkAlignComboBox, untilMarkTextPointSizeSpinBox);
        QWidget::setTabOrder(untilMarkTextPointSizeSpinBox, enableWaveformCaching);
        QWidget::setTabOrder(enableWaveformCaching, enableWaveformGenerationWithAnalysis);
        QWidget::setTabOrder(enableWaveformGenerationWithAnalysis, clearCachedWaveforms);

        retranslateUi(DlgPrefWaveformDlg);

        QMetaObject::connectSlotsByName(DlgPrefWaveformDlg);
    } // setupUi

    void retranslateUi(QWidget *DlgPrefWaveformDlg)
    {
        DlgPrefWaveformDlg->setWindowTitle(QCoreApplication::translate("DlgPrefWaveformDlg", "Waveform Preferences", nullptr));
#if QT_CONFIG(tooltip)
        waveformTypeLabel->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        waveformTypeLabel->setText(QCoreApplication::translate("DlgPrefWaveformDlg", "Waveform type", nullptr));
#if QT_CONFIG(tooltip)
        waveformOverviewLabel->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        waveformOverviewLabel->setText(QCoreApplication::translate("DlgPrefWaveformDlg", "Waveform overview type", nullptr));
#if QT_CONFIG(tooltip)
        waveformOverviewComboBox->setToolTip(QCoreApplication::translate("DlgPrefWaveformDlg", "The waveform overview shows the waveform envelope of the entire track.\n"
"Select from different types of displays for the waveform overview, which differ primarily in the level of detail shown in the waveform.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        waveformTypeComboBox->setToolTip(QCoreApplication::translate("DlgPrefWaveformDlg", "The waveform shows the waveform envelope of the track near the current playback position.\n"
"Select from different types of displays for the waveform, which differ primarily in the level of detail shown in the waveform.", nullptr));
#endif // QT_CONFIG(tooltip)
        frameRateLabel->setText(QCoreApplication::translate("DlgPrefWaveformDlg", "Frame rate", nullptr));
#if QT_CONFIG(tooltip)
        frameRateSpinBox->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        frameRateSpinBox->setSuffix(QCoreApplication::translate("DlgPrefWaveformDlg", " fps", nullptr));
        endOfTrackWarningTimeLabel->setText(QCoreApplication::translate("DlgPrefWaveformDlg", "End of track warning", nullptr));
#if QT_CONFIG(tooltip)
        endOfTrackWarningTimeSpinBox->setToolTip(QCoreApplication::translate("DlgPrefWaveformDlg", "Highlight the waveforms when the last seconds of a track remains.", nullptr));
#endif // QT_CONFIG(tooltip)
        endOfTrackWarningTimeSpinBox->setSuffix(QCoreApplication::translate("DlgPrefWaveformDlg", " seconds", nullptr));
#if QT_CONFIG(tooltip)
        BeatgridWaveform->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        BeatgridWaveform->setText(QCoreApplication::translate("DlgPrefWaveformDlg", "Beat grid opacity", nullptr));
#if QT_CONFIG(tooltip)
        beatGridAlphaSpinBox->setToolTip(QCoreApplication::translate("DlgPrefWaveformDlg", "Set amount of opacity on beat grid lines.", nullptr));
#endif // QT_CONFIG(tooltip)
        beatGridAlphaSpinBox->setSuffix(QCoreApplication::translate("DlgPrefWaveformDlg", "%", nullptr));
        label->setText(QCoreApplication::translate("DlgPrefWaveformDlg", "Play marker position", nullptr));
#if QT_CONFIG(tooltip)
        playMarkerPositionSlider->setToolTip(QCoreApplication::translate("DlgPrefWaveformDlg", "Moves the play marker position on the waveforms to the left, right or center (default).", nullptr));
#endif // QT_CONFIG(tooltip)
        defaultZoomLabel->setText(QCoreApplication::translate("DlgPrefWaveformDlg", "Default zoom level", nullptr));
#if QT_CONFIG(tooltip)
        synchronizeZoomCheckBox->setToolTip(QCoreApplication::translate("DlgPrefWaveformDlg", "Synchronize zoom level across all waveform displays.", nullptr));
#endif // QT_CONFIG(tooltip)
        synchronizeZoomCheckBox->setText(QCoreApplication::translate("DlgPrefWaveformDlg", "Synchronize zoom level across all waveforms", nullptr));
        normalizeOverviewCheckBox->setText(QCoreApplication::translate("DlgPrefWaveformDlg", "Normalize waveform overview", nullptr));
        visualGainLabel->setText(QCoreApplication::translate("DlgPrefWaveformDlg", "Visual gain", nullptr));
#if QT_CONFIG(tooltip)
        midVisualGain->setToolTip(QCoreApplication::translate("DlgPrefWaveformDlg", "Visual gain of the middle frequencies", nullptr));
#endif // QT_CONFIG(tooltip)
        visualGainLabel_5->setText(QCoreApplication::translate("DlgPrefWaveformDlg", "High", nullptr));
        visualGainLabel_4->setText(QCoreApplication::translate("DlgPrefWaveformDlg", "Middle", nullptr));
        visualGainLabel_2->setText(QCoreApplication::translate("DlgPrefWaveformDlg", "Global", nullptr));
#if QT_CONFIG(tooltip)
        highVisualGain->setToolTip(QCoreApplication::translate("DlgPrefWaveformDlg", "Visual gain of the high frequencies", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        lowVisualGain->setToolTip(QCoreApplication::translate("DlgPrefWaveformDlg", "Visual gain of the low frequencies", nullptr));
#endif // QT_CONFIG(tooltip)
        visualGainLabel_3->setText(QCoreApplication::translate("DlgPrefWaveformDlg", "Low", nullptr));
#if QT_CONFIG(tooltip)
        allVisualGain->setToolTip(QCoreApplication::translate("DlgPrefWaveformDlg", "Global visual gain", nullptr));
#endif // QT_CONFIG(tooltip)
        untilMarkLabel->setText(QCoreApplication::translate("DlgPrefWaveformDlg", "Play marker hints", nullptr));
        untilMarkShowBeatsCheckBox->setText(QCoreApplication::translate("DlgPrefWaveformDlg", "Beats until next marker", nullptr));
        untilMarkAlignLabel->setText(QCoreApplication::translate("DlgPrefWaveformDlg", "Placement", nullptr));
        untilMarkTextPointSizeLabel->setText(QCoreApplication::translate("DlgPrefWaveformDlg", "Preferred font size", nullptr));
        untilMarkTextHeightLimitLabel->setText(QCoreApplication::translate("DlgPrefWaveformDlg", "Text height limit", nullptr));
        untilMarkShowTimeCheckBox->setText(QCoreApplication::translate("DlgPrefWaveformDlg", "Time until next marker", nullptr));
#if QT_CONFIG(tooltip)
        untilMarkTextPointSizeSpinBox->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        untilMarkTextPointSizeSpinBox->setSuffix(QCoreApplication::translate("DlgPrefWaveformDlg", " pt", nullptr));
        requiresGLSLLabel->setText(QCoreApplication::translate("DlgPrefWaveformDlg", "This functionality requires a waveform type marked \"(GLSL)\".", nullptr));
        cachedWaveforms->setText(QCoreApplication::translate("DlgPrefWaveformDlg", "Caching", nullptr));
        clearCachedWaveforms->setText(QCoreApplication::translate("DlgPrefWaveformDlg", "Clear Cached Waveforms", nullptr));
        waveformCachingInfo->setText(QCoreApplication::translate("DlgPrefWaveformDlg", "Mixxx caches the waveforms of your tracks on disk the first time you load a track. This reduces CPU usage when you are playing live but requires extra disk space.", nullptr));
        enableWaveformCaching->setText(QCoreApplication::translate("DlgPrefWaveformDlg", "Enable waveform caching", nullptr));
        enableWaveformGenerationWithAnalysis->setText(QCoreApplication::translate("DlgPrefWaveformDlg", "Generate waveforms when analyzing library", nullptr));
        openGLStatus->setTitle(QCoreApplication::translate("DlgPrefWaveformDlg", "OpenGL status", nullptr));
#if QT_CONFIG(tooltip)
        openGlStatusData->setToolTip(QCoreApplication::translate("DlgPrefWaveformDlg", "Displays which OpenGL version is supported by the current platform.", nullptr));
#endif // QT_CONFIG(tooltip)
        openGlStatusData->setText(QString());
#if QT_CONFIG(tooltip)
        openGlStatusLabel_2->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        openGlStatusLabel_2->setText(QCoreApplication::translate("DlgPrefWaveformDlg", "Average frame rate", nullptr));
#if QT_CONFIG(tooltip)
        frameRateAverage->setToolTip(QCoreApplication::translate("DlgPrefWaveformDlg", "Displays the actual frame rate.", nullptr));
#endif // QT_CONFIG(tooltip)
        frameRateAverage->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class DlgPrefWaveformDlg: public Ui_DlgPrefWaveformDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLGPREFWAVEFORMDLG_H
