/****************************************************************************
** Meta object code from reading C++ file 'dlgpreflibrary.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/preferences/dialog/dlgpreflibrary.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dlgpreflibrary.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14DlgPrefLibraryE_t {};
} // unnamed namespace

template <> constexpr inline auto DlgPrefLibrary::qt_create_metaobjectdata<qt_meta_tag_ZN14DlgPrefLibraryE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DlgPrefLibrary",
        "apply",
        "",
        "scanLibrary",
        "requestAddDir",
        "dir",
        "requestRemoveDir",
        "LibraryRemovalType",
        "removalType",
        "requestRelocateDir",
        "currentDir",
        "newDir",
        "slotUpdate",
        "slotShow",
        "slotHide",
        "slotResetToDefaults",
        "slotApply",
        "slotCancel",
        "slotAddDir",
        "slotRemoveDir",
        "slotRelocateDir",
        "slotRowHeightValueChanged",
        "slotSelectFont",
        "slotSyncTrackMetadataToggled",
        "slotSearchDebouncingTimeoutMillisChanged",
        "slotBpmRangeSelected",
        "index",
        "slotBpmColumnPrecisionChanged",
        "bpmPrecision",
        "slotSeratoMetadataExportClicked"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'apply'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'scanLibrary'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestAddDir'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Signal 'requestRemoveDir'
        QtMocHelpers::SignalData<void(const QString &, LibraryRemovalType)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 }, { 0x80000000 | 7, 8 },
        }}),
        // Signal 'requestRelocateDir'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 }, { QMetaType::QString, 11 },
        }}),
        // Slot 'slotUpdate'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotShow'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotHide'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotResetToDefaults'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotApply'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotCancel'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotAddDir'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotRemoveDir'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotRelocateDir'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotRowHeightValueChanged'
        QtMocHelpers::SlotData<void(int)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'slotSelectFont'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSyncTrackMetadataToggled'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSearchDebouncingTimeoutMillisChanged'
        QtMocHelpers::SlotData<void(int)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'slotBpmRangeSelected'
        QtMocHelpers::SlotData<void(int)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 26 },
        }}),
        // Slot 'slotBpmColumnPrecisionChanged'
        QtMocHelpers::SlotData<void(int)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 28 },
        }}),
        // Slot 'slotSeratoMetadataExportClicked'
        QtMocHelpers::SlotData<void(bool)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DlgPrefLibrary, qt_meta_tag_ZN14DlgPrefLibraryE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DlgPrefLibrary::staticMetaObject = { {
    QMetaObject::SuperData::link<DlgPreferencePage::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14DlgPrefLibraryE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14DlgPrefLibraryE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14DlgPrefLibraryE_t>.metaTypes,
    nullptr
} };

void DlgPrefLibrary::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DlgPrefLibrary *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->apply(); break;
        case 1: _t->scanLibrary(); break;
        case 2: _t->requestAddDir((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->requestRemoveDir((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<LibraryRemovalType>>(_a[2]))); break;
        case 4: _t->requestRelocateDir((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 5: _t->slotUpdate(); break;
        case 6: _t->slotShow(); break;
        case 7: _t->slotHide(); break;
        case 8: _t->slotResetToDefaults(); break;
        case 9: _t->slotApply(); break;
        case 10: _t->slotCancel(); break;
        case 11: _t->slotAddDir(); break;
        case 12: _t->slotRemoveDir(); break;
        case 13: _t->slotRelocateDir(); break;
        case 14: _t->slotRowHeightValueChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 15: _t->slotSelectFont(); break;
        case 16: _t->slotSyncTrackMetadataToggled(); break;
        case 17: _t->slotSearchDebouncingTimeoutMillisChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 18: _t->slotBpmRangeSelected((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 19: _t->slotBpmColumnPrecisionChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 20: _t->slotSeratoMetadataExportClicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DlgPrefLibrary::*)()>(_a, &DlgPrefLibrary::apply, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgPrefLibrary::*)()>(_a, &DlgPrefLibrary::scanLibrary, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgPrefLibrary::*)(const QString & )>(_a, &DlgPrefLibrary::requestAddDir, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgPrefLibrary::*)(const QString & , LibraryRemovalType )>(_a, &DlgPrefLibrary::requestRemoveDir, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgPrefLibrary::*)(const QString & , const QString & )>(_a, &DlgPrefLibrary::requestRelocateDir, 4))
            return;
    }
}

const QMetaObject *DlgPrefLibrary::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DlgPrefLibrary::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14DlgPrefLibraryE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::DlgPrefLibraryDlg"))
        return static_cast< Ui::DlgPrefLibraryDlg*>(this);
    return DlgPreferencePage::qt_metacast(_clname);
}

int DlgPrefLibrary::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DlgPreferencePage::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 21;
    }
    return _id;
}

// SIGNAL 0
void DlgPrefLibrary::apply()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void DlgPrefLibrary::scanLibrary()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void DlgPrefLibrary::requestAddDir(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void DlgPrefLibrary::requestRemoveDir(const QString & _t1, LibraryRemovalType _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void DlgPrefLibrary::requestRelocateDir(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}
QT_WARNING_POP
