/****************************************************************************
** Meta object code from reading C++ file 'qmlcontrolproxy.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/qml/qmlcontrolproxy.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qmlcontrolproxy.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN5mixxx3qml15QmlControlProxyE_t {};
} // unnamed namespace

template <> constexpr inline auto mixxx::qml::QmlControlProxy::qt_create_metaobjectdata<qt_meta_tag_ZN5mixxx3qml15QmlControlProxyE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "mixxx::qml::QmlControlProxy",
        "QML.Element",
        "ControlProxy",
        "groupChanged",
        "",
        "group",
        "keyChanged",
        "key",
        "keyValidChanged",
        "valid",
        "initializedChanged",
        "initialized",
        "valueChanged",
        "newValue",
        "parameterChanged",
        "newParameter",
        "slotControlProxyValueChanged",
        "reset",
        "keyValid",
        "value",
        "parameter"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'groupChanged'
        QtMocHelpers::SignalData<void(const QString &)>(3, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Signal 'keyChanged'
        QtMocHelpers::SignalData<void(const QString &)>(6, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Signal 'keyValidChanged'
        QtMocHelpers::SignalData<void(bool)>(8, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'initializedChanged'
        QtMocHelpers::SignalData<void(bool)>(10, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Signal 'valueChanged'
        QtMocHelpers::SignalData<void(double)>(12, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 13 },
        }}),
        // Signal 'parameterChanged'
        QtMocHelpers::SignalData<void(double)>(14, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 15 },
        }}),
        // Slot 'slotControlProxyValueChanged'
        QtMocHelpers::SlotData<void(double)>(16, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 13 },
        }}),
        // Method 'reset'
        QtMocHelpers::MethodData<void()>(17, 4, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'group'
        QtMocHelpers::PropertyData<QString>(5, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Required, 0),
        // property 'key'
        QtMocHelpers::PropertyData<QString>(7, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Required, 1),
        // property 'keyValid'
        QtMocHelpers::PropertyData<bool>(18, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
        // property 'initialized'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags, 3),
        // property 'value'
        QtMocHelpers::PropertyData<double>(19, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'parameter'
        QtMocHelpers::PropertyData<double>(20, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<QmlControlProxy, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject mixxx::qml::QmlControlProxy::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5mixxx3qml15QmlControlProxyE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5mixxx3qml15QmlControlProxyE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN5mixxx3qml15QmlControlProxyE_t>.metaTypes,
    nullptr
} };

void mixxx::qml::QmlControlProxy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QmlControlProxy *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->groupChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->keyChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->keyValidChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->initializedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->valueChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 5: _t->parameterChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 6: _t->slotControlProxyValueChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 7: _t->reset(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QmlControlProxy::*)(const QString & )>(_a, &QmlControlProxy::groupChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlControlProxy::*)(const QString & )>(_a, &QmlControlProxy::keyChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlControlProxy::*)(bool )>(_a, &QmlControlProxy::keyValidChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlControlProxy::*)(bool )>(_a, &QmlControlProxy::initializedChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlControlProxy::*)(double )>(_a, &QmlControlProxy::valueChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlControlProxy::*)(double )>(_a, &QmlControlProxy::parameterChanged, 5))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->getGroup(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->getKey(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->isKeyValid(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isInitialized(); break;
        case 4: *reinterpret_cast<double*>(_v) = _t->getValue(); break;
        case 5: *reinterpret_cast<double*>(_v) = _t->getParameter(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setGroup(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setKey(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setValue(*reinterpret_cast<double*>(_v)); break;
        case 5: _t->setParameter(*reinterpret_cast<double*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *mixxx::qml::QmlControlProxy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *mixxx::qml::QmlControlProxy::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5mixxx3qml15QmlControlProxyE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QObject::qt_metacast(_clname);
}

int mixxx::qml::QmlControlProxy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void mixxx::qml::QmlControlProxy::groupChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void mixxx::qml::QmlControlProxy::keyChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void mixxx::qml::QmlControlProxy::keyValidChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void mixxx::qml::QmlControlProxy::initializedChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void mixxx::qml::QmlControlProxy::valueChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void mixxx::qml::QmlControlProxy::parameterChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
QT_WARNING_POP
