/****************************************************************************
** Meta object code from reading C++ file 'qmlwaveformoverview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/qml/qmlwaveformoverview.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qmlwaveformoverview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN5mixxx3qml19QmlWaveformOverviewE_t {};
} // unnamed namespace

template <> constexpr inline auto mixxx::qml::QmlWaveformOverview::qt_create_metaobjectdata<qt_meta_tag_ZN5mixxx3qml19QmlWaveformOverviewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "mixxx::qml::QmlWaveformOverview",
        "QML.Element",
        "WaveformOverview",
        "playerChanged",
        "",
        "channelsChanged",
        "mixxx::qml::QmlWaveformOverview::Channels",
        "channels",
        "rendererChanged",
        "Renderer",
        "renderer",
        "colorHighChanged",
        "QColor",
        "color",
        "colorMidChanged",
        "colorLowChanged",
        "slotTrackLoaded",
        "TrackPointer",
        "pLoadedTrack",
        "slotTrackLoading",
        "pNewTrack",
        "pOldTrack",
        "slotTrackUnloaded",
        "slotWaveformUpdated",
        "player",
        "mixxx::qml::QmlPlayerProxy*",
        "Channels",
        "colorHigh",
        "colorMid",
        "colorLow",
        "ChannelFlag",
        "LeftChannel",
        "RightChannel",
        "BothChannels",
        "RGB",
        "Filtered"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'playerChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'channelsChanged'
        QtMocHelpers::SignalData<void(mixxx::qml::QmlWaveformOverview::Channels)>(5, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'rendererChanged'
        QtMocHelpers::SignalData<void(enum Renderer)>(8, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'colorHighChanged'
        QtMocHelpers::SignalData<void(const QColor &)>(11, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'colorMidChanged'
        QtMocHelpers::SignalData<void(const QColor &)>(14, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'colorLowChanged'
        QtMocHelpers::SignalData<void(const QColor &)>(15, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'slotTrackLoaded'
        QtMocHelpers::SlotData<void(TrackPointer)>(16, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'slotTrackLoading'
        QtMocHelpers::SlotData<void(TrackPointer, TrackPointer)>(19, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 20 }, { 0x80000000 | 17, 21 },
        }}),
        // Slot 'slotTrackUnloaded'
        QtMocHelpers::SlotData<void()>(22, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotWaveformUpdated'
        QtMocHelpers::SlotData<void()>(23, 4, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'player'
        QtMocHelpers::PropertyData<mixxx::qml::QmlPlayerProxy*>(24, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Required, 0),
        // property 'channels'
        QtMocHelpers::PropertyData<Channels>(7, 0x80000000 | 26, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'renderer'
        QtMocHelpers::PropertyData<enum Renderer>(10, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, 2),
        // property 'colorHigh'
        QtMocHelpers::PropertyData<QColor>(27, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, 3),
        // property 'colorMid'
        QtMocHelpers::PropertyData<QColor>(28, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, 4),
        // property 'colorLow'
        QtMocHelpers::PropertyData<QColor>(29, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, 5),
    };
    QtMocHelpers::UintData qt_enums {
        // flag 'Channels'
        QtMocHelpers::EnumData<Channels>(26, 30, QMC::EnumIsFlag | QMC::EnumIsScoped).add({
            {   31, ChannelFlag::LeftChannel },
            {   32, ChannelFlag::RightChannel },
            {   33, ChannelFlag::BothChannels },
        }),
        // enum 'Renderer'
        QtMocHelpers::EnumData<enum Renderer>(9, 9, QMC::EnumIsScoped).add({
            {   34, Renderer::RGB },
            {   35, Renderer::Filtered },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<QmlWaveformOverview, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject mixxx::qml::QmlWaveformOverview::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickPaintedItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5mixxx3qml19QmlWaveformOverviewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5mixxx3qml19QmlWaveformOverviewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN5mixxx3qml19QmlWaveformOverviewE_t>.metaTypes,
    nullptr
} };

void mixxx::qml::QmlWaveformOverview::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QmlWaveformOverview *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->playerChanged(); break;
        case 1: _t->channelsChanged((*reinterpret_cast<std::add_pointer_t<mixxx::qml::QmlWaveformOverview::Channels>>(_a[1]))); break;
        case 2: _t->rendererChanged((*reinterpret_cast<std::add_pointer_t<enum Renderer>>(_a[1]))); break;
        case 3: _t->colorHighChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 4: _t->colorMidChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 5: _t->colorLowChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 6: _t->slotTrackLoaded((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1]))); break;
        case 7: _t->slotTrackLoading((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[2]))); break;
        case 8: _t->slotTrackUnloaded(); break;
        case 9: _t->slotWaveformUpdated(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QmlWaveformOverview::*)()>(_a, &QmlWaveformOverview::playerChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlWaveformOverview::*)(mixxx::qml::QmlWaveformOverview::Channels )>(_a, &QmlWaveformOverview::channelsChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlWaveformOverview::*)(Renderer )>(_a, &QmlWaveformOverview::rendererChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlWaveformOverview::*)(const QColor & )>(_a, &QmlWaveformOverview::colorHighChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlWaveformOverview::*)(const QColor & )>(_a, &QmlWaveformOverview::colorMidChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlWaveformOverview::*)(const QColor & )>(_a, &QmlWaveformOverview::colorLowChanged, 5))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< mixxx::qml::QmlPlayerProxy* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<mixxx::qml::QmlPlayerProxy**>(_v) = _t->getPlayer(); break;
        case 1: QtMocHelpers::assignFlags<Channels>(_v, _t->getChannels()); break;
        case 2: *reinterpret_cast<enum Renderer*>(_v) = _t->m_renderer; break;
        case 3: *reinterpret_cast<QColor*>(_v) = _t->m_colorHigh; break;
        case 4: *reinterpret_cast<QColor*>(_v) = _t->m_colorMid; break;
        case 5: *reinterpret_cast<QColor*>(_v) = _t->m_colorLow; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setPlayer(*reinterpret_cast<mixxx::qml::QmlPlayerProxy**>(_v)); break;
        case 1: _t->setChannels(*reinterpret_cast<Channels*>(_v)); break;
        case 2:
            if (QtMocHelpers::setProperty(_t->m_renderer, *reinterpret_cast<enum Renderer*>(_v)))
                Q_EMIT _t->rendererChanged(_t->m_renderer);
            break;
        case 3:
            if (QtMocHelpers::setProperty(_t->m_colorHigh, *reinterpret_cast<QColor*>(_v)))
                Q_EMIT _t->colorHighChanged(_t->m_colorHigh);
            break;
        case 4:
            if (QtMocHelpers::setProperty(_t->m_colorMid, *reinterpret_cast<QColor*>(_v)))
                Q_EMIT _t->colorMidChanged(_t->m_colorMid);
            break;
        case 5:
            if (QtMocHelpers::setProperty(_t->m_colorLow, *reinterpret_cast<QColor*>(_v)))
                Q_EMIT _t->colorLowChanged(_t->m_colorLow);
            break;
        default: break;
        }
    }
}

const QMetaObject *mixxx::qml::QmlWaveformOverview::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *mixxx::qml::QmlWaveformOverview::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5mixxx3qml19QmlWaveformOverviewE_t>.strings))
        return static_cast<void*>(this);
    return QQuickPaintedItem::qt_metacast(_clname);
}

int mixxx::qml::QmlWaveformOverview::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickPaintedItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void mixxx::qml::QmlWaveformOverview::playerChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void mixxx::qml::QmlWaveformOverview::channelsChanged(mixxx::qml::QmlWaveformOverview::Channels _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void mixxx::qml::QmlWaveformOverview::rendererChanged(Renderer _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void mixxx::qml::QmlWaveformOverview::colorHighChanged(const QColor & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void mixxx::qml::QmlWaveformOverview::colorMidChanged(const QColor & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void mixxx::qml::QmlWaveformOverview::colorLowChanged(const QColor & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
QT_WARNING_POP
