/********************************************************************************
** Form generated from reading UI file 'dlgtrackexport.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLGTRACKEXPORT_H
#define UI_DLGTRACKEXPORT_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DlgTrackExport
{
public:
    QGridLayout *gridLayout;
    QGroupBox *groupBox;
    QWidget *layoutWidget;
    QVBoxLayout *verticalLayout_2;
    QProgressBar *exportProgress;
    QLabel *statusLabel;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *horizontalLayout3;
    QPushButton *cancelButton;
    QSpacerItem *horizontalSpacer_3;

    void setupUi(QDialog *DlgTrackExport)
    {
        if (DlgTrackExport->objectName().isEmpty())
            DlgTrackExport->setObjectName("DlgTrackExport");
        DlgTrackExport->resize(600, 200);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(DlgTrackExport->sizePolicy().hasHeightForWidth());
        DlgTrackExport->setSizePolicy(sizePolicy);
        DlgTrackExport->setMinimumSize(QSize(0, 200));
        DlgTrackExport->setMaximumSize(QSize(16777215, 16777215));
        gridLayout = new QGridLayout(DlgTrackExport);
        gridLayout->setObjectName("gridLayout");
        groupBox = new QGroupBox(DlgTrackExport);
        groupBox->setObjectName("groupBox");
        groupBox->setEnabled(true);
        QSizePolicy sizePolicy1(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(groupBox->sizePolicy().hasHeightForWidth());
        groupBox->setSizePolicy(sizePolicy1);
        layoutWidget = new QWidget(groupBox);
        layoutWidget->setObjectName("layoutWidget");
        layoutWidget->setGeometry(QRect(10, 26, 561, 150));
        sizePolicy.setHeightForWidth(layoutWidget->sizePolicy().hasHeightForWidth());
        layoutWidget->setSizePolicy(sizePolicy);
        verticalLayout_2 = new QVBoxLayout(layoutWidget);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        exportProgress = new QProgressBar(layoutWidget);
        exportProgress->setObjectName("exportProgress");
        exportProgress->setMaximum(10);
        exportProgress->setValue(0);

        verticalLayout_2->addWidget(exportProgress);

        statusLabel = new QLabel(layoutWidget);
        statusLabel->setObjectName("statusLabel");
        statusLabel->setWordWrap(true);

        verticalLayout_2->addWidget(statusLabel);

        verticalSpacer = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);

        horizontalLayout3 = new QHBoxLayout();
        horizontalLayout3->setSpacing(10);
        horizontalLayout3->setObjectName("horizontalLayout3");
        cancelButton = new QPushButton(layoutWidget);
        cancelButton->setObjectName("cancelButton");

        horizontalLayout3->addWidget(cancelButton);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout3->addItem(horizontalSpacer_3);


        verticalLayout_2->addLayout(horizontalLayout3);

        verticalLayout_2->setStretch(0, 2);

        gridLayout->addWidget(groupBox, 0, 0, 1, 1);


        retranslateUi(DlgTrackExport);

        QMetaObject::connectSlotsByName(DlgTrackExport);
    } // setupUi

    void retranslateUi(QDialog *DlgTrackExport)
    {
        DlgTrackExport->setWindowTitle(QCoreApplication::translate("DlgTrackExport", "Export Tracks", nullptr));
        groupBox->setTitle(QCoreApplication::translate("DlgTrackExport", "Exporting Tracks", nullptr));
        exportProgress->setFormat(QString());
        statusLabel->setText(QCoreApplication::translate("DlgTrackExport", "(status text)", nullptr));
        cancelButton->setText(QCoreApplication::translate("DlgTrackExport", "&Cancel", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgTrackExport: public Ui_DlgTrackExport {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLGTRACKEXPORT_H
