/****************************************************************************
** Meta object code from reading C++ file 'libraryscanner.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/library/scanner/libraryscanner.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#include <QtCore/QSet>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'libraryscanner.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14LibraryScannerE_t {};
} // unnamed namespace

template <> constexpr inline auto LibraryScanner::qt_create_metaobjectdata<qt_meta_tag_ZN14LibraryScannerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LibraryScanner",
        "scanStarted",
        "",
        "scanFinished",
        "progressHashing",
        "progressLoading",
        "path",
        "progressCoverArt",
        "file",
        "trackAdded",
        "TrackPointer",
        "pTrack",
        "tracksChanged",
        "QSet<TrackId>",
        "changedTrackIds",
        "tracksRelocated",
        "QList<RelocatedTrack>",
        "relocatedTracks",
        "startScan",
        "scan",
        "slotCancel",
        "queueTask",
        "ScannerTask*",
        "pTask",
        "slotStartScan",
        "slotFinishHashedScan",
        "slotFinishUnhashedScan",
        "slotDirectoryHashedAndScanned",
        "directoryPath",
        "newDirectory",
        "mixxx::cache_key_t",
        "hash",
        "slotDirectoryUnchanged",
        "slotTrackExists",
        "trackPath",
        "slotAddNewTrack"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'scanStarted'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'scanFinished'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'progressHashing'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'progressLoading'
        QtMocHelpers::SignalData<void(const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'progressCoverArt'
        QtMocHelpers::SignalData<void(const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Signal 'trackAdded'
        QtMocHelpers::SignalData<void(TrackPointer)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'tracksChanged'
        QtMocHelpers::SignalData<void(const QSet<TrackId> &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Signal 'tracksRelocated'
        QtMocHelpers::SignalData<void(const QList<RelocatedTrack> &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Signal 'startScan'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'scan'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotCancel'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'queueTask'
        QtMocHelpers::SlotData<void(ScannerTask *)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'slotStartScan'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotFinishHashedScan'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotFinishUnhashedScan'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotDirectoryHashedAndScanned'
        QtMocHelpers::SlotData<void(const QString &, bool, mixxx::cache_key_t)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 28 }, { QMetaType::Bool, 29 }, { 0x80000000 | 30, 31 },
        }}),
        // Slot 'slotDirectoryUnchanged'
        QtMocHelpers::SlotData<void(const QString &)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 28 },
        }}),
        // Slot 'slotTrackExists'
        QtMocHelpers::SlotData<void(const QString &)>(33, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 34 },
        }}),
        // Slot 'slotAddNewTrack'
        QtMocHelpers::SlotData<void(const QString &)>(35, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 34 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LibraryScanner, qt_meta_tag_ZN14LibraryScannerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject LibraryScanner::staticMetaObject = { {
    QMetaObject::SuperData::link<QThread::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14LibraryScannerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14LibraryScannerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14LibraryScannerE_t>.metaTypes,
    nullptr
} };

void LibraryScanner::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LibraryScanner *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->scanStarted(); break;
        case 1: _t->scanFinished(); break;
        case 2: _t->progressHashing((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->progressLoading((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->progressCoverArt((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->trackAdded((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1]))); break;
        case 6: _t->tracksChanged((*reinterpret_cast<std::add_pointer_t<QSet<TrackId>>>(_a[1]))); break;
        case 7: _t->tracksRelocated((*reinterpret_cast<std::add_pointer_t<QList<RelocatedTrack>>>(_a[1]))); break;
        case 8: _t->startScan(); break;
        case 9: _t->scan(); break;
        case 10: _t->slotCancel(); break;
        case 11: _t->queueTask((*reinterpret_cast<std::add_pointer_t<ScannerTask*>>(_a[1]))); break;
        case 12: _t->slotStartScan(); break;
        case 13: _t->slotFinishHashedScan(); break;
        case 14: _t->slotFinishUnhashedScan(); break;
        case 15: _t->slotDirectoryHashedAndScanned((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<mixxx::cache_key_t>>(_a[3]))); break;
        case 16: _t->slotDirectoryUnchanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 17: _t->slotTrackExists((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 18: _t->slotAddNewTrack((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<TrackId> >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<RelocatedTrack> >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< mixxx::cache_key_t >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (LibraryScanner::*)()>(_a, &LibraryScanner::scanStarted, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (LibraryScanner::*)()>(_a, &LibraryScanner::scanFinished, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (LibraryScanner::*)(const QString & )>(_a, &LibraryScanner::progressHashing, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (LibraryScanner::*)(const QString & )>(_a, &LibraryScanner::progressLoading, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (LibraryScanner::*)(const QString & )>(_a, &LibraryScanner::progressCoverArt, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (LibraryScanner::*)(TrackPointer )>(_a, &LibraryScanner::trackAdded, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (LibraryScanner::*)(const QSet<TrackId> & )>(_a, &LibraryScanner::tracksChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (LibraryScanner::*)(const QList<RelocatedTrack> & )>(_a, &LibraryScanner::tracksRelocated, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (LibraryScanner::*)()>(_a, &LibraryScanner::startScan, 8))
            return;
    }
}

const QMetaObject *LibraryScanner::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LibraryScanner::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14LibraryScannerE_t>.strings))
        return static_cast<void*>(this);
    return QThread::qt_metacast(_clname);
}

int LibraryScanner::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QThread::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    return _id;
}

// SIGNAL 0
void LibraryScanner::scanStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void LibraryScanner::scanFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void LibraryScanner::progressHashing(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void LibraryScanner::progressLoading(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void LibraryScanner::progressCoverArt(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void LibraryScanner::trackAdded(TrackPointer _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void LibraryScanner::tracksChanged(const QSet<TrackId> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void LibraryScanner::tracksRelocated(const QList<RelocatedTrack> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void LibraryScanner::startScan()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}
QT_WARNING_POP
