/****************************************************************************
** Meta object code from reading C++ file 'cratefeature.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/library/trackset/crate/cratefeature.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QSet>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'cratefeature.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12CrateFeatureE_t {};
} // unnamed namespace

template <> constexpr inline auto CrateFeature::qt_create_metaobjectdata<qt_meta_tag_ZN12CrateFeatureE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CrateFeature",
        "exportAllCrates",
        "",
        "exportCrate",
        "CrateId",
        "crateId",
        "activate",
        "activateChild",
        "QModelIndex",
        "index",
        "onRightClick",
        "QPoint",
        "globalPos",
        "onRightClickChild",
        "slotCreateCrate",
        "deleteItem",
        "renameItem",
        "slotDeleteCrate",
        "slotRenameCrate",
        "slotDuplicateCrate",
        "slotAutoDjTrackSourceChanged",
        "slotToggleCrateLock",
        "slotImportPlaylist",
        "slotImportPlaylistFile",
        "playlistFile",
        "slotCreateImportCrate",
        "slotExportPlaylist",
        "slotExportTrackFiles",
        "slotAnalyzeCrate",
        "slotCrateTableChanged",
        "slotCrateContentChanged",
        "htmlLinkClicked",
        "QUrl",
        "link",
        "slotTrackSelected",
        "TrackId",
        "trackId",
        "slotResetSelectedTrack",
        "slotUpdateCrateLabels",
        "QSet<CrateId>",
        "updatedCrateIds"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'exportAllCrates'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'exportCrate'
        QtMocHelpers::SignalData<void(CrateId)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'activate'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'activateChild'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'onRightClick'
        QtMocHelpers::SlotData<void(const QPoint &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'onRightClickChild'
        QtMocHelpers::SlotData<void(const QPoint &, const QModelIndex &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 }, { 0x80000000 | 8, 9 },
        }}),
        // Slot 'slotCreateCrate'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'deleteItem'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'renameItem'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'slotDeleteCrate'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotRenameCrate'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotDuplicateCrate'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotAutoDjTrackSourceChanged'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotToggleCrateLock'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotImportPlaylist'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotImportPlaylistFile'
        QtMocHelpers::SlotData<void(const QString &, CrateId)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 24 }, { 0x80000000 | 4, 5 },
        }}),
        // Slot 'slotCreateImportCrate'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotExportPlaylist'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotExportTrackFiles'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotAnalyzeCrate'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotCrateTableChanged'
        QtMocHelpers::SlotData<void(CrateId)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'slotCrateContentChanged'
        QtMocHelpers::SlotData<void(CrateId)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'htmlLinkClicked'
        QtMocHelpers::SlotData<void(const QUrl &)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Slot 'slotTrackSelected'
        QtMocHelpers::SlotData<void(TrackId)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 35, 36 },
        }}),
        // Slot 'slotResetSelectedTrack'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotUpdateCrateLabels'
        QtMocHelpers::SlotData<void(const QSet<CrateId> &)>(38, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 39, 40 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CrateFeature, qt_meta_tag_ZN12CrateFeatureE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CrateFeature::staticMetaObject = { {
    QMetaObject::SuperData::link<BaseTrackSetFeature::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12CrateFeatureE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12CrateFeatureE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12CrateFeatureE_t>.metaTypes,
    nullptr
} };

void CrateFeature::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CrateFeature *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->exportAllCrates(); break;
        case 1: _t->exportCrate((*reinterpret_cast<std::add_pointer_t<CrateId>>(_a[1]))); break;
        case 2: _t->activate(); break;
        case 3: _t->activateChild((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 4: _t->onRightClick((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 5: _t->onRightClickChild((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 6: _t->slotCreateCrate(); break;
        case 7: _t->deleteItem((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 8: _t->renameItem((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 9: _t->slotDeleteCrate(); break;
        case 10: _t->slotRenameCrate(); break;
        case 11: _t->slotDuplicateCrate(); break;
        case 12: _t->slotAutoDjTrackSourceChanged(); break;
        case 13: _t->slotToggleCrateLock(); break;
        case 14: _t->slotImportPlaylist(); break;
        case 15: _t->slotImportPlaylistFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CrateId>>(_a[2]))); break;
        case 16: _t->slotCreateImportCrate(); break;
        case 17: _t->slotExportPlaylist(); break;
        case 18: _t->slotExportTrackFiles(); break;
        case 19: _t->slotAnalyzeCrate(); break;
        case 20: _t->slotCrateTableChanged((*reinterpret_cast<std::add_pointer_t<CrateId>>(_a[1]))); break;
        case 21: _t->slotCrateContentChanged((*reinterpret_cast<std::add_pointer_t<CrateId>>(_a[1]))); break;
        case 22: _t->htmlLinkClicked((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 23: _t->slotTrackSelected((*reinterpret_cast<std::add_pointer_t<TrackId>>(_a[1]))); break;
        case 24: _t->slotResetSelectedTrack(); break;
        case 25: _t->slotUpdateCrateLabels((*reinterpret_cast<std::add_pointer_t<QSet<CrateId>>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CrateId >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CrateId >(); break;
            }
            break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CrateId >(); break;
            }
            break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CrateId >(); break;
            }
            break;
        case 23:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackId >(); break;
            }
            break;
        case 25:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<CrateId> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CrateFeature::*)()>(_a, &CrateFeature::exportAllCrates, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (CrateFeature::*)(CrateId )>(_a, &CrateFeature::exportCrate, 1))
            return;
    }
}

const QMetaObject *CrateFeature::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CrateFeature::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12CrateFeatureE_t>.strings))
        return static_cast<void*>(this);
    return BaseTrackSetFeature::qt_metacast(_clname);
}

int CrateFeature::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = BaseTrackSetFeature::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    }
    return _id;
}

// SIGNAL 0
void CrateFeature::exportAllCrates()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void CrateFeature::exportCrate(CrateId _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
