/****************************************************************************
** Meta object code from reading C++ file 'engineprimeexportjob.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/library/export/engineprimeexportjob.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QSet>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'engineprimeexportjob.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN5mixxx20EnginePrimeExportJobE_t {};
} // unnamed namespace

template <> constexpr inline auto mixxx::EnginePrimeExportJob::qt_create_metaobjectdata<qt_meta_tag_ZN5mixxx20EnginePrimeExportJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "mixxx::EnginePrimeExportJob",
        "jobMaximum",
        "",
        "maximum",
        "jobProgress",
        "progress",
        "completed",
        "numTracksExported",
        "numCratesExported",
        "failed",
        "message",
        "slotCancel",
        "loadIds",
        "QSet<CrateId>",
        "crateIdsToExport",
        "loadTrack",
        "TrackRef",
        "trackRef",
        "loadCrate",
        "CrateId",
        "crateId"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'jobMaximum'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'jobProgress'
        QtMocHelpers::SignalData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Signal 'completed'
        QtMocHelpers::SignalData<void(int, int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 }, { QMetaType::Int, 8 },
        }}),
        // Signal 'failed'
        QtMocHelpers::SignalData<void(const QString &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Slot 'slotCancel'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'loadIds'
        QtMocHelpers::SlotData<void(const QSet<CrateId> &)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'loadTrack'
        QtMocHelpers::SlotData<void(const TrackRef &)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'loadCrate'
        QtMocHelpers::SlotData<void(const CrateId &)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<EnginePrimeExportJob, qt_meta_tag_ZN5mixxx20EnginePrimeExportJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject mixxx::EnginePrimeExportJob::staticMetaObject = { {
    QMetaObject::SuperData::link<QThread::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5mixxx20EnginePrimeExportJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5mixxx20EnginePrimeExportJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN5mixxx20EnginePrimeExportJobE_t>.metaTypes,
    nullptr
} };

void mixxx::EnginePrimeExportJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<EnginePrimeExportJob *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->jobMaximum((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->jobProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->completed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 3: _t->failed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->slotCancel(); break;
        case 5: _t->loadIds((*reinterpret_cast<std::add_pointer_t<QSet<CrateId>>>(_a[1]))); break;
        case 6: _t->loadTrack((*reinterpret_cast<std::add_pointer_t<TrackRef>>(_a[1]))); break;
        case 7: _t->loadCrate((*reinterpret_cast<std::add_pointer_t<CrateId>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<CrateId> >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackRef >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CrateId >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (EnginePrimeExportJob::*)(int )>(_a, &EnginePrimeExportJob::jobMaximum, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (EnginePrimeExportJob::*)(int )>(_a, &EnginePrimeExportJob::jobProgress, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (EnginePrimeExportJob::*)(int , int )>(_a, &EnginePrimeExportJob::completed, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (EnginePrimeExportJob::*)(const QString & )>(_a, &EnginePrimeExportJob::failed, 3))
            return;
    }
}

const QMetaObject *mixxx::EnginePrimeExportJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *mixxx::EnginePrimeExportJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5mixxx20EnginePrimeExportJobE_t>.strings))
        return static_cast<void*>(this);
    return QThread::qt_metacast(_clname);
}

int mixxx::EnginePrimeExportJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QThread::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void mixxx::EnginePrimeExportJob::jobMaximum(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void mixxx::EnginePrimeExportJob::jobProgress(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void mixxx::EnginePrimeExportJob::completed(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void mixxx::EnginePrimeExportJob::failed(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
