/****************************************************************************
** Meta object code from reading C++ file 'constants.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/preferences/constants.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'constants.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN5mixxx11preferences9constantsE_t {};
} // unnamed namespace

template <> constexpr inline auto mixxx::preferences::constants::qt_create_metaobjectdata<qt_meta_tag_ZN5mixxx11preferences9constantsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "mixxx::preferences::constants",
        "Tooltips",
        "Off",
        "On",
        "OnlyInLibrary",
        "ScreenSaver",
        "OnPlay",
        "MultiSamplingMode",
        "Disabled",
        "Two",
        "Four",
        "Eight",
        "Sixteen"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Tooltips'
        QtMocHelpers::EnumData<Tooltips>(1, 1, QMC::EnumIsScoped).add({
            {    2, Tooltips::Off },
            {    3, Tooltips::On },
            {    4, Tooltips::OnlyInLibrary },
        }),
        // enum 'ScreenSaver'
        QtMocHelpers::EnumData<ScreenSaver>(5, 5, QMC::EnumIsScoped).add({
            {    2, ScreenSaver::Off },
            {    3, ScreenSaver::On },
            {    6, ScreenSaver::OnPlay },
        }),
        // enum 'MultiSamplingMode'
        QtMocHelpers::EnumData<MultiSamplingMode>(7, 7, QMC::EnumIsScoped).add({
            {    8, MultiSamplingMode::Disabled },
            {    9, MultiSamplingMode::Two },
            {   10, MultiSamplingMode::Four },
            {   11, MultiSamplingMode::Eight },
            {   12, MultiSamplingMode::Sixteen },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN5mixxx11preferences9constantsE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN5mixxx11preferences9constantsE =
    mixxx::preferences::constants::qt_create_metaobjectdata<qt_meta_tag_ZN5mixxx11preferences9constantsE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN5mixxx11preferences9constantsE =
    qt_staticMetaObjectContent_ZN5mixxx11preferences9constantsE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN5mixxx11preferences9constantsE =
    qt_staticMetaObjectContent_ZN5mixxx11preferences9constantsE.relocatingData;

Q_CONSTINIT const QMetaObject mixxx::preferences::constants::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN5mixxx11preferences9constantsE.stringdata,
    qt_staticMetaObjectStaticContent_ZN5mixxx11preferences9constantsE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN5mixxx11preferences9constantsE.metaTypes,
    nullptr
} };

QT_WARNING_POP
