/****************************************************************************
** Meta object code from reading C++ file 'loopingcontrol.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/engine/controls/loopingcontrol.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'loopingcontrol.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14LoopingControlE_t {};
} // unnamed namespace

template <> constexpr inline auto LoopingControl::qt_create_metaobjectdata<qt_meta_tag_ZN14LoopingControlE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LoopingControl",
        "loopReset",
        "",
        "loopEnabledChanged",
        "enabled",
        "loopUpdated",
        "mixxx::audio::FramePos",
        "startPosition",
        "endPosition",
        "slotLoopIn",
        "pressed",
        "slotLoopInGoto",
        "slotLoopOut",
        "slotLoopOutGoto",
        "slotLoopExit",
        "slotReloopToggle",
        "slotReloopAndStop",
        "slotLoopStartPos",
        "slotLoopEndPos",
        "slotBeatLoop",
        "loopSize",
        "keepSetPoint",
        "enable",
        "LoopingControl::LoopAnchorPoint",
        "forcedAnchor",
        "slotBeatLoopSizeChangeRequest",
        "beats",
        "slotBeatLoopToggle",
        "slotBeatLoopRollActivate",
        "slotBeatLoopActivate",
        "BeatLoopingControl*",
        "pBeatLoopControl",
        "slotBeatLoopActivateRoll",
        "slotBeatLoopDeactivate",
        "slotBeatLoopDeactivateRoll",
        "slotBeatJump",
        "slotBeatJumpSizeChangeRequest",
        "slotBeatJumpSizeHalve",
        "slotBeatJumpSizeDouble",
        "slotBeatJumpForward",
        "slotBeatJumpBackward",
        "slotLoopMove",
        "slotLoopScale",
        "scaleFactor",
        "slotLoopDouble",
        "slotLoopHalve",
        "slotLoopRemove",
        "slotLoopEnabledValueChangeRequest"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'loopReset'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'loopEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 4 },
        }}),
        // Signal 'loopUpdated'
        QtMocHelpers::SignalData<void(mixxx::audio::FramePos, mixxx::audio::FramePos)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 6, 8 },
        }}),
        // Slot 'slotLoopIn'
        QtMocHelpers::SlotData<void(double)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 10 },
        }}),
        // Slot 'slotLoopInGoto'
        QtMocHelpers::SlotData<void(double)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotLoopOut'
        QtMocHelpers::SlotData<void(double)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 10 },
        }}),
        // Slot 'slotLoopOutGoto'
        QtMocHelpers::SlotData<void(double)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotLoopExit'
        QtMocHelpers::SlotData<void(double)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotReloopToggle'
        QtMocHelpers::SlotData<void(double)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotReloopAndStop'
        QtMocHelpers::SlotData<void(double)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotLoopStartPos'
        QtMocHelpers::SlotData<void(double)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotLoopEndPos'
        QtMocHelpers::SlotData<void(double)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotBeatLoop'
        QtMocHelpers::SlotData<void(double, bool, bool, LoopingControl::LoopAnchorPoint)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 20 }, { QMetaType::Bool, 21 }, { QMetaType::Bool, 22 }, { 0x80000000 | 23, 24 },
        }}),
        // Slot 'slotBeatLoop'
        QtMocHelpers::SlotData<void(double, bool, bool)>(19, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Double, 20 }, { QMetaType::Bool, 21 }, { QMetaType::Bool, 22 },
        }}),
        // Slot 'slotBeatLoop'
        QtMocHelpers::SlotData<void(double, bool)>(19, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Double, 20 }, { QMetaType::Bool, 21 },
        }}),
        // Slot 'slotBeatLoop'
        QtMocHelpers::SlotData<void(double)>(19, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Double, 20 },
        }}),
        // Slot 'slotBeatLoopSizeChangeRequest'
        QtMocHelpers::SlotData<void(double)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 26 },
        }}),
        // Slot 'slotBeatLoopToggle'
        QtMocHelpers::SlotData<void(double)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 10 },
        }}),
        // Slot 'slotBeatLoopRollActivate'
        QtMocHelpers::SlotData<void(double)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 10 },
        }}),
        // Slot 'slotBeatLoopActivate'
        QtMocHelpers::SlotData<void(BeatLoopingControl *, LoopingControl::LoopAnchorPoint)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 30, 31 }, { 0x80000000 | 23, 24 },
        }}),
        // Slot 'slotBeatLoopActivate'
        QtMocHelpers::SlotData<void(BeatLoopingControl *)>(29, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 30, 31 },
        }}),
        // Slot 'slotBeatLoopActivateRoll'
        QtMocHelpers::SlotData<void(BeatLoopingControl *, LoopingControl::LoopAnchorPoint)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 30, 31 }, { 0x80000000 | 23, 24 },
        }}),
        // Slot 'slotBeatLoopActivateRoll'
        QtMocHelpers::SlotData<void(BeatLoopingControl *)>(32, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 30, 31 },
        }}),
        // Slot 'slotBeatLoopDeactivate'
        QtMocHelpers::SlotData<void(BeatLoopingControl *)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 30, 31 },
        }}),
        // Slot 'slotBeatLoopDeactivateRoll'
        QtMocHelpers::SlotData<void(BeatLoopingControl *)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 30, 31 },
        }}),
        // Slot 'slotBeatJump'
        QtMocHelpers::SlotData<void(double)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 26 },
        }}),
        // Slot 'slotBeatJumpSizeChangeRequest'
        QtMocHelpers::SlotData<void(double)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 26 },
        }}),
        // Slot 'slotBeatJumpSizeHalve'
        QtMocHelpers::SlotData<void(double)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 10 },
        }}),
        // Slot 'slotBeatJumpSizeDouble'
        QtMocHelpers::SlotData<void(double)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 10 },
        }}),
        // Slot 'slotBeatJumpForward'
        QtMocHelpers::SlotData<void(double)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 10 },
        }}),
        // Slot 'slotBeatJumpBackward'
        QtMocHelpers::SlotData<void(double)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 10 },
        }}),
        // Slot 'slotLoopMove'
        QtMocHelpers::SlotData<void(double)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 26 },
        }}),
        // Slot 'slotLoopScale'
        QtMocHelpers::SlotData<void(double)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 43 },
        }}),
        // Slot 'slotLoopDouble'
        QtMocHelpers::SlotData<void(double)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 10 },
        }}),
        // Slot 'slotLoopHalve'
        QtMocHelpers::SlotData<void(double)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 10 },
        }}),
        // Slot 'slotLoopRemove'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotLoopEnabledValueChangeRequest'
        QtMocHelpers::SlotData<void(double)>(47, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LoopingControl, qt_meta_tag_ZN14LoopingControlE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject LoopingControl::staticMetaObject = { {
    QMetaObject::SuperData::link<EngineControl::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14LoopingControlE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14LoopingControlE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14LoopingControlE_t>.metaTypes,
    nullptr
} };

void LoopingControl::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LoopingControl *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->loopReset(); break;
        case 1: _t->loopEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->loopUpdated((*reinterpret_cast<std::add_pointer_t<mixxx::audio::FramePos>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<mixxx::audio::FramePos>>(_a[2]))); break;
        case 3: _t->slotLoopIn((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 4: _t->slotLoopInGoto((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 5: _t->slotLoopOut((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 6: _t->slotLoopOutGoto((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 7: _t->slotLoopExit((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 8: _t->slotReloopToggle((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 9: _t->slotReloopAndStop((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 10: _t->slotLoopStartPos((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 11: _t->slotLoopEndPos((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 12: _t->slotBeatLoop((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<LoopingControl::LoopAnchorPoint>>(_a[4]))); break;
        case 13: _t->slotBeatLoop((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 14: _t->slotBeatLoop((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 15: _t->slotBeatLoop((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 16: _t->slotBeatLoopSizeChangeRequest((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 17: _t->slotBeatLoopToggle((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 18: _t->slotBeatLoopRollActivate((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 19: _t->slotBeatLoopActivate((*reinterpret_cast<std::add_pointer_t<BeatLoopingControl*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<LoopingControl::LoopAnchorPoint>>(_a[2]))); break;
        case 20: _t->slotBeatLoopActivate((*reinterpret_cast<std::add_pointer_t<BeatLoopingControl*>>(_a[1]))); break;
        case 21: _t->slotBeatLoopActivateRoll((*reinterpret_cast<std::add_pointer_t<BeatLoopingControl*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<LoopingControl::LoopAnchorPoint>>(_a[2]))); break;
        case 22: _t->slotBeatLoopActivateRoll((*reinterpret_cast<std::add_pointer_t<BeatLoopingControl*>>(_a[1]))); break;
        case 23: _t->slotBeatLoopDeactivate((*reinterpret_cast<std::add_pointer_t<BeatLoopingControl*>>(_a[1]))); break;
        case 24: _t->slotBeatLoopDeactivateRoll((*reinterpret_cast<std::add_pointer_t<BeatLoopingControl*>>(_a[1]))); break;
        case 25: _t->slotBeatJump((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 26: _t->slotBeatJumpSizeChangeRequest((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 27: _t->slotBeatJumpSizeHalve((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 28: _t->slotBeatJumpSizeDouble((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 29: _t->slotBeatJumpForward((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 30: _t->slotBeatJumpBackward((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 31: _t->slotLoopMove((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 32: _t->slotLoopScale((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 33: _t->slotLoopDouble((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 34: _t->slotLoopHalve((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 35: _t->slotLoopRemove(); break;
        case 36: _t->slotLoopEnabledValueChangeRequest((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< mixxx::audio::FramePos >(); break;
            }
            break;
        case 19:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< BeatLoopingControl* >(); break;
            }
            break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< BeatLoopingControl* >(); break;
            }
            break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< BeatLoopingControl* >(); break;
            }
            break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< BeatLoopingControl* >(); break;
            }
            break;
        case 23:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< BeatLoopingControl* >(); break;
            }
            break;
        case 24:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< BeatLoopingControl* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (LoopingControl::*)()>(_a, &LoopingControl::loopReset, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (LoopingControl::*)(bool )>(_a, &LoopingControl::loopEnabledChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (LoopingControl::*)(mixxx::audio::FramePos , mixxx::audio::FramePos )>(_a, &LoopingControl::loopUpdated, 2))
            return;
    }
}

const QMetaObject *LoopingControl::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LoopingControl::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14LoopingControlE_t>.strings))
        return static_cast<void*>(this);
    return EngineControl::qt_metacast(_clname);
}

int LoopingControl::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = EngineControl::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 37)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 37;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 37)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 37;
    }
    return _id;
}

// SIGNAL 0
void LoopingControl::loopReset()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void LoopingControl::loopEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void LoopingControl::loopUpdated(mixxx::audio::FramePos _t1, mixxx::audio::FramePos _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN15LoopMoveControlE_t {};
} // unnamed namespace

template <> constexpr inline auto LoopMoveControl::qt_create_metaobjectdata<qt_meta_tag_ZN15LoopMoveControlE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LoopMoveControl",
        "loopMove",
        "",
        "beats",
        "slotMoveForward",
        "value",
        "slotMoveBackward"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'loopMove'
        QtMocHelpers::SignalData<void(double)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'slotMoveForward'
        QtMocHelpers::SlotData<void(double)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 5 },
        }}),
        // Slot 'slotMoveBackward'
        QtMocHelpers::SlotData<void(double)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LoopMoveControl, qt_meta_tag_ZN15LoopMoveControlE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject LoopMoveControl::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15LoopMoveControlE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15LoopMoveControlE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15LoopMoveControlE_t>.metaTypes,
    nullptr
} };

void LoopMoveControl::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LoopMoveControl *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->loopMove((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 1: _t->slotMoveForward((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 2: _t->slotMoveBackward((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (LoopMoveControl::*)(double )>(_a, &LoopMoveControl::loopMove, 0))
            return;
    }
}

const QMetaObject *LoopMoveControl::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LoopMoveControl::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15LoopMoveControlE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int LoopMoveControl::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void LoopMoveControl::loopMove(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN15BeatJumpControlE_t {};
} // unnamed namespace

template <> constexpr inline auto BeatJumpControl::qt_create_metaobjectdata<qt_meta_tag_ZN15BeatJumpControlE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "BeatJumpControl",
        "beatJump",
        "",
        "beats",
        "slotJumpForward",
        "pressed",
        "slotJumpBackward"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'beatJump'
        QtMocHelpers::SignalData<void(double)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 },
        }}),
        // Slot 'slotJumpForward'
        QtMocHelpers::SlotData<void(double)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 5 },
        }}),
        // Slot 'slotJumpBackward'
        QtMocHelpers::SlotData<void(double)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<BeatJumpControl, qt_meta_tag_ZN15BeatJumpControlE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject BeatJumpControl::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15BeatJumpControlE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15BeatJumpControlE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15BeatJumpControlE_t>.metaTypes,
    nullptr
} };

void BeatJumpControl::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<BeatJumpControl *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->beatJump((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 1: _t->slotJumpForward((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 2: _t->slotJumpBackward((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (BeatJumpControl::*)(double )>(_a, &BeatJumpControl::beatJump, 0))
            return;
    }
}

const QMetaObject *BeatJumpControl::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *BeatJumpControl::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15BeatJumpControlE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int BeatJumpControl::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void BeatJumpControl::beatJump(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN18BeatLoopingControlE_t {};
} // unnamed namespace

template <> constexpr inline auto BeatLoopingControl::qt_create_metaobjectdata<qt_meta_tag_ZN18BeatLoopingControlE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "BeatLoopingControl",
        "activateBeatLoop",
        "",
        "BeatLoopingControl*",
        "LoopingControl::LoopAnchorPoint",
        "forcedAnchor",
        "deactivateBeatLoop",
        "activateBeatLoopRoll",
        "deactivateBeatLoopRoll",
        "slotLegacy",
        "value",
        "slotActivate",
        "slotActivateRoll",
        "slotToggle",
        "slotReverseActivate",
        "slotReverseActivateRoll",
        "slotReverseToggle"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'activateBeatLoop'
        QtMocHelpers::SignalData<void(BeatLoopingControl *, LoopingControl::LoopAnchorPoint)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 4, 5 },
        }}),
        // Signal 'deactivateBeatLoop'
        QtMocHelpers::SignalData<void(BeatLoopingControl *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'activateBeatLoopRoll'
        QtMocHelpers::SignalData<void(BeatLoopingControl *, LoopingControl::LoopAnchorPoint)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 4, 5 },
        }}),
        // Signal 'deactivateBeatLoopRoll'
        QtMocHelpers::SignalData<void(BeatLoopingControl *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Slot 'slotLegacy'
        QtMocHelpers::SlotData<void(double)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 10 },
        }}),
        // Slot 'slotActivate'
        QtMocHelpers::SlotData<void(double, LoopingControl::LoopAnchorPoint)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 10 }, { 0x80000000 | 4, 5 },
        }}),
        // Slot 'slotActivateRoll'
        QtMocHelpers::SlotData<void(double, LoopingControl::LoopAnchorPoint)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 10 }, { 0x80000000 | 4, 5 },
        }}),
        // Slot 'slotToggle'
        QtMocHelpers::SlotData<void(double, LoopingControl::LoopAnchorPoint)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 10 }, { 0x80000000 | 4, 5 },
        }}),
        // Slot 'slotReverseActivate'
        QtMocHelpers::SlotData<void(double)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 10 },
        }}),
        // Slot 'slotReverseActivateRoll'
        QtMocHelpers::SlotData<void(double)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 10 },
        }}),
        // Slot 'slotReverseToggle'
        QtMocHelpers::SlotData<void(double)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 10 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<BeatLoopingControl, qt_meta_tag_ZN18BeatLoopingControlE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject BeatLoopingControl::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18BeatLoopingControlE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18BeatLoopingControlE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18BeatLoopingControlE_t>.metaTypes,
    nullptr
} };

void BeatLoopingControl::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<BeatLoopingControl *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->activateBeatLoop((*reinterpret_cast<std::add_pointer_t<BeatLoopingControl*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<LoopingControl::LoopAnchorPoint>>(_a[2]))); break;
        case 1: _t->deactivateBeatLoop((*reinterpret_cast<std::add_pointer_t<BeatLoopingControl*>>(_a[1]))); break;
        case 2: _t->activateBeatLoopRoll((*reinterpret_cast<std::add_pointer_t<BeatLoopingControl*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<LoopingControl::LoopAnchorPoint>>(_a[2]))); break;
        case 3: _t->deactivateBeatLoopRoll((*reinterpret_cast<std::add_pointer_t<BeatLoopingControl*>>(_a[1]))); break;
        case 4: _t->slotLegacy((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 5: _t->slotActivate((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<LoopingControl::LoopAnchorPoint>>(_a[2]))); break;
        case 6: _t->slotActivateRoll((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<LoopingControl::LoopAnchorPoint>>(_a[2]))); break;
        case 7: _t->slotToggle((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<LoopingControl::LoopAnchorPoint>>(_a[2]))); break;
        case 8: _t->slotReverseActivate((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 9: _t->slotReverseActivateRoll((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 10: _t->slotReverseToggle((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< BeatLoopingControl* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< BeatLoopingControl* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< BeatLoopingControl* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< BeatLoopingControl* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (BeatLoopingControl::*)(BeatLoopingControl * , LoopingControl::LoopAnchorPoint )>(_a, &BeatLoopingControl::activateBeatLoop, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (BeatLoopingControl::*)(BeatLoopingControl * )>(_a, &BeatLoopingControl::deactivateBeatLoop, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (BeatLoopingControl::*)(BeatLoopingControl * , LoopingControl::LoopAnchorPoint )>(_a, &BeatLoopingControl::activateBeatLoopRoll, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (BeatLoopingControl::*)(BeatLoopingControl * )>(_a, &BeatLoopingControl::deactivateBeatLoopRoll, 3))
            return;
    }
}

const QMetaObject *BeatLoopingControl::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *BeatLoopingControl::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18BeatLoopingControlE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int BeatLoopingControl::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void BeatLoopingControl::activateBeatLoop(BeatLoopingControl * _t1, LoopingControl::LoopAnchorPoint _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void BeatLoopingControl::deactivateBeatLoop(BeatLoopingControl * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void BeatLoopingControl::activateBeatLoopRoll(BeatLoopingControl * _t1, LoopingControl::LoopAnchorPoint _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void BeatLoopingControl::deactivateBeatLoopRoll(BeatLoopingControl * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
