// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: skin.proto
// Protobuf C++ Version: 7.35.0

#ifndef skin_2eproto_2epb_2eh
#define skin_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

// clang-format off
#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 7035000
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_skin_2eproto

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct TableStruct_skin_2eproto {
  static const ::uint32_t offsets[];
};
namespace mixxx {
namespace skin {
class SkinManifest;
struct SkinManifestGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern SkinManifestGlobalsTypeInternal SkinManifest_globals_;
extern const ::google::protobuf::internal::ClassDataLite SkinManifest_class_data_;
#else
extern const SkinManifestGlobalsTypeInternal SkinManifest_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class SkinManifest_Attribute;
struct SkinManifest_AttributeGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern SkinManifest_AttributeGlobalsTypeInternal SkinManifest_Attribute_globals_;
extern const ::google::protobuf::internal::ClassDataLite SkinManifest_Attribute_class_data_;
#else
extern const SkinManifest_AttributeGlobalsTypeInternal SkinManifest_Attribute_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
}  // namespace skin
}  // namespace mixxx
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google

namespace mixxx {
namespace skin {

// ===================================================================


// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED SkinManifest_Attribute final : public ::google::protobuf::MessageLite
/* @@protoc_insertion_point(class_definition:mixxx.skin.SkinManifest.Attribute) */ {
 public:
  inline SkinManifest_Attribute() : SkinManifest_Attribute(nullptr) {}
  ~SkinManifest_Attribute() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(SkinManifest_Attribute* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(SkinManifest_Attribute));
  }
#endif

  template <typename = void>
  explicit constexpr SkinManifest_Attribute(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline SkinManifest_Attribute(const SkinManifest_Attribute& from) : SkinManifest_Attribute(nullptr, from) {}
  inline SkinManifest_Attribute(SkinManifest_Attribute&& from) noexcept : SkinManifest_Attribute(nullptr, ::std::move(from)) {}
  inline SkinManifest_Attribute& operator=(const SkinManifest_Attribute& from) {
    CopyFrom(from);
    return *this;
  }
  inline SkinManifest_Attribute& operator=(SkinManifest_Attribute&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::std::string& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString);
  }
  [[nodiscard]] inline ::std::string* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::std::string>();
  }

  [[nodiscard]] static const SkinManifest_Attribute& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<SkinManifest_Attribute>(&SkinManifest_Attribute_globals_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(SkinManifest_Attribute& a, SkinManifest_Attribute& b) { a.Swap(&b); }
  inline void Swap(SkinManifest_Attribute* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(SkinManifest_Attribute* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] SkinManifest_Attribute* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::MessageLite::DefaultConstruct<SkinManifest_Attribute>(arena);
  }
  void CopyFrom(const SkinManifest_Attribute& from);
  void MergeFrom(const SkinManifest_Attribute& from) { SkinManifest_Attribute::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(SkinManifest_Attribute* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "mixxx.skin.SkinManifest.Attribute"; }

  explicit SkinManifest_Attribute(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  SkinManifest_Attribute(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const SkinManifest_Attribute& from);
  SkinManifest_Attribute(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, SkinManifest_Attribute&& from) noexcept
      : SkinManifest_Attribute(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kConfigKeyFieldNumber = 1,
    kValueFieldNumber = 2,
    kPersistFieldNumber = 3,
  };
  // optional string config_key = 1;
  [[nodiscard]] bool has_config_key()
      const;
  void clear_config_key() ;
  [[nodiscard]] const ::std::string& config_key() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_config_key(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_config_key();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_config_key();
  void set_allocated_config_key(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_config_key() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_config_key(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_config_key();

  public:
  // optional string value = 2;
  [[nodiscard]] bool has_value()
      const;
  void clear_value() ;
  [[nodiscard]] const ::std::string& value() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_value(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_value();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_value();
  void set_allocated_value(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_value() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_value(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_value();

  public:
  // optional bool persist = 3;
  [[nodiscard]] bool has_persist()
      const;
  void clear_persist() ;
  [[nodiscard]] bool persist() const;
  void set_persist(bool value);

  private:
  bool _internal_persist() const;
  void _internal_set_persist(bool value);

  public:
  // @@protoc_insertion_point(class_scope:mixxx.skin.SkinManifest.Attribute)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<2, 3,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const SkinManifest_Attribute& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr config_key_;
    ::google::protobuf::internal::ArenaStringPtr value_;
    bool persist_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_skin_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED SkinManifest final : public ::google::protobuf::MessageLite
/* @@protoc_insertion_point(class_definition:mixxx.skin.SkinManifest) */ {
 public:
  inline SkinManifest() : SkinManifest(nullptr) {}
  ~SkinManifest() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(SkinManifest* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(SkinManifest));
  }
#endif

  template <typename = void>
  explicit constexpr SkinManifest(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline SkinManifest(const SkinManifest& from) : SkinManifest(nullptr, from) {}
  inline SkinManifest(SkinManifest&& from) noexcept : SkinManifest(nullptr, ::std::move(from)) {}
  inline SkinManifest& operator=(const SkinManifest& from) {
    CopyFrom(from);
    return *this;
  }
  inline SkinManifest& operator=(SkinManifest&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::std::string& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString);
  }
  [[nodiscard]] inline ::std::string* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::std::string>();
  }

  [[nodiscard]] static const SkinManifest& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<SkinManifest>(&SkinManifest_globals_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(SkinManifest& a, SkinManifest& b) { a.Swap(&b); }
  inline void Swap(SkinManifest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(SkinManifest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] SkinManifest* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::MessageLite::DefaultConstruct<SkinManifest>(arena);
  }
  void CopyFrom(const SkinManifest& from);
  void MergeFrom(const SkinManifest& from) { SkinManifest::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(SkinManifest* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "mixxx.skin.SkinManifest"; }

  explicit SkinManifest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  SkinManifest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const SkinManifest& from);
  SkinManifest(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, SkinManifest&& from) noexcept
      : SkinManifest(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  // nested types ----------------------------------------------------
  using Attribute = SkinManifest_Attribute;

  // accessors -------------------------------------------------------
  enum : int {
    kAttributeFieldNumber = 7,
    kTitleFieldNumber = 1,
    kAuthorFieldNumber = 2,
    kVersionFieldNumber = 3,
    kLanguageFieldNumber = 4,
    kDescriptionFieldNumber = 5,
    kLicenseFieldNumber = 6,
  };
  // repeated .mixxx.skin.SkinManifest.Attribute attribute = 7;
  [[nodiscard]] int attribute_size()
      const;
  private:
  int _internal_attribute_size() const;

  public:
  void clear_attribute() ;
  [[nodiscard]] const ::mixxx::skin::SkinManifest_Attribute& attribute(int index) const;
  [[nodiscard]] ::mixxx::skin::SkinManifest_Attribute* PROTOBUF_NONNULL mutable_attribute(int index);
  ::mixxx::skin::SkinManifest_Attribute* PROTOBUF_NONNULL add_attribute();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::mixxx::skin::SkinManifest_Attribute>&
  attribute() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::mixxx::skin::SkinManifest_Attribute>* PROTOBUF_NONNULL
  mutable_attribute();

  private:
  const ::google::protobuf::RepeatedPtrField<::mixxx::skin::SkinManifest_Attribute>& _internal_attribute() const;
  ::google::protobuf::RepeatedPtrField<::mixxx::skin::SkinManifest_Attribute>* PROTOBUF_NONNULL _internal_mutable_attribute();

  public:
  // optional string title = 1;
  [[nodiscard]] bool has_title()
      const;
  void clear_title() ;
  [[nodiscard]] const ::std::string& title() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_title(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_title();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_title();
  void set_allocated_title(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_title() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_title(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_title();

  public:
  // optional string author = 2;
  [[nodiscard]] bool has_author()
      const;
  void clear_author() ;
  [[nodiscard]] const ::std::string& author() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_author(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_author();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_author();
  void set_allocated_author(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_author() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_author(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_author();

  public:
  // optional string version = 3;
  [[nodiscard]] bool has_version()
      const;
  void clear_version() ;
  [[nodiscard]] const ::std::string& version() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_version(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_version();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_version();
  void set_allocated_version(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_version() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_version(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_version();

  public:
  // optional string language = 4;
  [[nodiscard]] bool has_language()
      const;
  void clear_language() ;
  [[nodiscard]] const ::std::string& language() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_language(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_language();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_language();
  void set_allocated_language(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_language() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_language(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_language();

  public:
  // optional string description = 5;
  [[nodiscard]] bool has_description()
      const;
  void clear_description() ;
  [[nodiscard]] const ::std::string& description() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_description(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_description();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_description();
  void set_allocated_description(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_description() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_description(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_description();

  public:
  // optional string license = 6;
  [[nodiscard]] bool has_license()
      const;
  void clear_license() ;
  [[nodiscard]] const ::std::string& license() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_license(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_license();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_license();
  void set_allocated_license(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_license() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_license(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_license();

  public:
  // @@protoc_insertion_point(class_scope:mixxx.skin.SkinManifest)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 7,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const SkinManifest& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::mixxx::skin::SkinManifest_Attribute > attribute_;
    ::google::protobuf::internal::ArenaStringPtr title_;
    ::google::protobuf::internal::ArenaStringPtr author_;
    ::google::protobuf::internal::ArenaStringPtr version_;
    ::google::protobuf::internal::ArenaStringPtr language_;
    ::google::protobuf::internal::ArenaStringPtr description_;
    ::google::protobuf::internal::ArenaStringPtr license_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_skin_2eproto;
};

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// SkinManifest_Attribute

// optional string config_key = 1;
inline bool SkinManifest_Attribute::has_config_key() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  return value;
}
inline void SkinManifest_Attribute::clear_config_key() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.config_key_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& SkinManifest_Attribute::config_key() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:mixxx.skin.SkinManifest.Attribute.config_key)
  return _internal_config_key();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void SkinManifest_Attribute::set_config_key(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.config_key_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:mixxx.skin.SkinManifest.Attribute.config_key)
}
inline ::std::string* PROTOBUF_NONNULL SkinManifest_Attribute::mutable_config_key()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_config_key();
  // @@protoc_insertion_point(field_mutable:mixxx.skin.SkinManifest.Attribute.config_key)
  return _s;
}
inline const ::std::string& SkinManifest_Attribute::_internal_config_key() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.config_key_.Get();
}
inline void SkinManifest_Attribute::_internal_set_config_key(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.config_key_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL SkinManifest_Attribute::_internal_mutable_config_key() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.config_key_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE SkinManifest_Attribute::release_config_key() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:mixxx.skin.SkinManifest.Attribute.config_key)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.config_key_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.config_key_.Set("", GetArena());
  }
  return released;
}
inline void SkinManifest_Attribute::set_allocated_config_key(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.config_key_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.config_key_.IsDefault()) {
    _impl_.config_key_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:mixxx.skin.SkinManifest.Attribute.config_key)
}

// optional string value = 2;
inline bool SkinManifest_Attribute::has_value() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  return value;
}
inline void SkinManifest_Attribute::clear_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.value_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& SkinManifest_Attribute::value() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:mixxx.skin.SkinManifest.Attribute.value)
  return _internal_value();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void SkinManifest_Attribute::set_value(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.value_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:mixxx.skin.SkinManifest.Attribute.value)
}
inline ::std::string* PROTOBUF_NONNULL SkinManifest_Attribute::mutable_value()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_value();
  // @@protoc_insertion_point(field_mutable:mixxx.skin.SkinManifest.Attribute.value)
  return _s;
}
inline const ::std::string& SkinManifest_Attribute::_internal_value() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.value_.Get();
}
inline void SkinManifest_Attribute::_internal_set_value(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.value_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL SkinManifest_Attribute::_internal_mutable_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.value_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE SkinManifest_Attribute::release_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:mixxx.skin.SkinManifest.Attribute.value)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.value_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.value_.Set("", GetArena());
  }
  return released;
}
inline void SkinManifest_Attribute::set_allocated_value(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.value_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.value_.IsDefault()) {
    _impl_.value_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:mixxx.skin.SkinManifest.Attribute.value)
}

// optional bool persist = 3;
inline bool SkinManifest_Attribute::has_persist() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  return value;
}
inline void SkinManifest_Attribute::clear_persist() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.persist_ = false;
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline bool SkinManifest_Attribute::persist() const {
  // @@protoc_insertion_point(field_get:mixxx.skin.SkinManifest.Attribute.persist)
  return _internal_persist();
}
inline void SkinManifest_Attribute::set_persist(bool value) {
  _internal_set_persist(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:mixxx.skin.SkinManifest.Attribute.persist)
}
inline bool SkinManifest_Attribute::_internal_persist() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.persist_;
}
inline void SkinManifest_Attribute::_internal_set_persist(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.persist_ = value;
}

// -------------------------------------------------------------------

// SkinManifest

// optional string title = 1;
inline bool SkinManifest::has_title() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  return value;
}
inline void SkinManifest::clear_title() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.title_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& SkinManifest::title() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:mixxx.skin.SkinManifest.title)
  return _internal_title();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void SkinManifest::set_title(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.title_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:mixxx.skin.SkinManifest.title)
}
inline ::std::string* PROTOBUF_NONNULL SkinManifest::mutable_title()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_title();
  // @@protoc_insertion_point(field_mutable:mixxx.skin.SkinManifest.title)
  return _s;
}
inline const ::std::string& SkinManifest::_internal_title() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.title_.Get();
}
inline void SkinManifest::_internal_set_title(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.title_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL SkinManifest::_internal_mutable_title() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.title_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE SkinManifest::release_title() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:mixxx.skin.SkinManifest.title)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.title_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.title_.Set("", GetArena());
  }
  return released;
}
inline void SkinManifest::set_allocated_title(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.title_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.title_.IsDefault()) {
    _impl_.title_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:mixxx.skin.SkinManifest.title)
}

// optional string author = 2;
inline bool SkinManifest::has_author() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  return value;
}
inline void SkinManifest::clear_author() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.author_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline const ::std::string& SkinManifest::author() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:mixxx.skin.SkinManifest.author)
  return _internal_author();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void SkinManifest::set_author(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  _impl_.author_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:mixxx.skin.SkinManifest.author)
}
inline ::std::string* PROTOBUF_NONNULL SkinManifest::mutable_author()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::std::string* _s = _internal_mutable_author();
  // @@protoc_insertion_point(field_mutable:mixxx.skin.SkinManifest.author)
  return _s;
}
inline const ::std::string& SkinManifest::_internal_author() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.author_.Get();
}
inline void SkinManifest::_internal_set_author(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.author_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL SkinManifest::_internal_mutable_author() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.author_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE SkinManifest::release_author() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:mixxx.skin.SkinManifest.author)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000004U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  auto* released = _impl_.author_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.author_.Set("", GetArena());
  }
  return released;
}
inline void SkinManifest::set_allocated_author(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  _impl_.author_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.author_.IsDefault()) {
    _impl_.author_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:mixxx.skin.SkinManifest.author)
}

// optional string version = 3;
inline bool SkinManifest::has_version() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000008U);
  return value;
}
inline void SkinManifest::clear_version() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.version_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline const ::std::string& SkinManifest::version() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:mixxx.skin.SkinManifest.version)
  return _internal_version();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void SkinManifest::set_version(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  _impl_.version_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:mixxx.skin.SkinManifest.version)
}
inline ::std::string* PROTOBUF_NONNULL SkinManifest::mutable_version()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::std::string* _s = _internal_mutable_version();
  // @@protoc_insertion_point(field_mutable:mixxx.skin.SkinManifest.version)
  return _s;
}
inline const ::std::string& SkinManifest::_internal_version() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.version_.Get();
}
inline void SkinManifest::_internal_set_version(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.version_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL SkinManifest::_internal_mutable_version() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.version_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE SkinManifest::release_version() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:mixxx.skin.SkinManifest.version)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000008U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  auto* released = _impl_.version_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.version_.Set("", GetArena());
  }
  return released;
}
inline void SkinManifest::set_allocated_version(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }
  _impl_.version_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.version_.IsDefault()) {
    _impl_.version_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:mixxx.skin.SkinManifest.version)
}

// optional string language = 4;
inline bool SkinManifest::has_language() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000010U);
  return value;
}
inline void SkinManifest::clear_language() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.language_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline const ::std::string& SkinManifest::language() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:mixxx.skin.SkinManifest.language)
  return _internal_language();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void SkinManifest::set_language(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  _impl_.language_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:mixxx.skin.SkinManifest.language)
}
inline ::std::string* PROTOBUF_NONNULL SkinManifest::mutable_language()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::std::string* _s = _internal_mutable_language();
  // @@protoc_insertion_point(field_mutable:mixxx.skin.SkinManifest.language)
  return _s;
}
inline const ::std::string& SkinManifest::_internal_language() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.language_.Get();
}
inline void SkinManifest::_internal_set_language(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.language_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL SkinManifest::_internal_mutable_language() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.language_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE SkinManifest::release_language() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:mixxx.skin.SkinManifest.language)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000010U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  auto* released = _impl_.language_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.language_.Set("", GetArena());
  }
  return released;
}
inline void SkinManifest::set_allocated_language(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  }
  _impl_.language_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.language_.IsDefault()) {
    _impl_.language_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:mixxx.skin.SkinManifest.language)
}

// optional string description = 5;
inline bool SkinManifest::has_description() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000020U);
  return value;
}
inline void SkinManifest::clear_description() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.description_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline const ::std::string& SkinManifest::description() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:mixxx.skin.SkinManifest.description)
  return _internal_description();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void SkinManifest::set_description(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  _impl_.description_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:mixxx.skin.SkinManifest.description)
}
inline ::std::string* PROTOBUF_NONNULL SkinManifest::mutable_description()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  ::std::string* _s = _internal_mutable_description();
  // @@protoc_insertion_point(field_mutable:mixxx.skin.SkinManifest.description)
  return _s;
}
inline const ::std::string& SkinManifest::_internal_description() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.description_.Get();
}
inline void SkinManifest::_internal_set_description(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.description_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL SkinManifest::_internal_mutable_description() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.description_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE SkinManifest::release_description() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:mixxx.skin.SkinManifest.description)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000020U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
  auto* released = _impl_.description_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.description_.Set("", GetArena());
  }
  return released;
}
inline void SkinManifest::set_allocated_description(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
  }
  _impl_.description_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.description_.IsDefault()) {
    _impl_.description_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:mixxx.skin.SkinManifest.description)
}

// optional string license = 6;
inline bool SkinManifest::has_license() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000040U);
  return value;
}
inline void SkinManifest::clear_license() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.license_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
}
inline const ::std::string& SkinManifest::license() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:mixxx.skin.SkinManifest.license)
  return _internal_license();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void SkinManifest::set_license(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  _impl_.license_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:mixxx.skin.SkinManifest.license)
}
inline ::std::string* PROTOBUF_NONNULL SkinManifest::mutable_license()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  ::std::string* _s = _internal_mutable_license();
  // @@protoc_insertion_point(field_mutable:mixxx.skin.SkinManifest.license)
  return _s;
}
inline const ::std::string& SkinManifest::_internal_license() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.license_.Get();
}
inline void SkinManifest::_internal_set_license(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.license_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL SkinManifest::_internal_mutable_license() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.license_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE SkinManifest::release_license() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:mixxx.skin.SkinManifest.license)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000040U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
  auto* released = _impl_.license_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.license_.Set("", GetArena());
  }
  return released;
}
inline void SkinManifest::set_allocated_license(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
  }
  _impl_.license_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.license_.IsDefault()) {
    _impl_.license_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:mixxx.skin.SkinManifest.license)
}

// repeated .mixxx.skin.SkinManifest.Attribute attribute = 7;
inline int SkinManifest::_internal_attribute_size() const {
  return _internal_attribute().size();
}
inline int SkinManifest::attribute_size() const {
  return _internal_attribute_size();
}
inline void SkinManifest::clear_attribute() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.attribute_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::mixxx::skin::SkinManifest_Attribute& SkinManifest::attribute(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:mixxx.skin.SkinManifest.attribute)
  return _internal_attribute().Get(index);
}
inline ::mixxx::skin::SkinManifest_Attribute* PROTOBUF_NONNULL SkinManifest::mutable_attribute(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:mixxx.skin.SkinManifest.attribute)
  return _internal_mutable_attribute()->Mutable(index);
}
inline ::mixxx::skin::SkinManifest_Attribute* PROTOBUF_NONNULL SkinManifest::add_attribute()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::mixxx::skin::SkinManifest_Attribute* _add =
      _internal_mutable_attribute()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:mixxx.skin.SkinManifest.attribute)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::mixxx::skin::SkinManifest_Attribute>& SkinManifest::attribute() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:mixxx.skin.SkinManifest.attribute)
  return _internal_attribute();
}
inline ::google::protobuf::RepeatedPtrField<::mixxx::skin::SkinManifest_Attribute>* PROTOBUF_NONNULL
SkinManifest::mutable_attribute() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:mixxx.skin.SkinManifest.attribute)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_attribute();
}
inline const ::google::protobuf::RepeatedPtrField<::mixxx::skin::SkinManifest_Attribute>&
SkinManifest::_internal_attribute() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.attribute_;
}
inline ::google::protobuf::RepeatedPtrField<::mixxx::skin::SkinManifest_Attribute>* PROTOBUF_NONNULL
SkinManifest::_internal_mutable_attribute() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.attribute_;
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace skin
}  // namespace mixxx


// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"
// clang-format on

#endif  // skin_2eproto_2epb_2eh
