/****************************************************************************
** Meta object code from reading C++ file 'dlgtagfetcher.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/library/dlgtagfetcher.h"
#include <QtNetwork/QSslError>
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dlgtagfetcher.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13DlgTagFetcherE_t {};
} // unnamed namespace

template <> constexpr inline auto DlgTagFetcher::qt_create_metaobjectdata<qt_meta_tag_ZN13DlgTagFetcherE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DlgTagFetcher",
        "next",
        "",
        "previous",
        "loadTrack",
        "TrackPointer",
        "pTrack",
        "QModelIndex",
        "index",
        "fetchTagFinished",
        "QList<mixxx::musicbrainz::TrackRelease>",
        "guessedTrackReleases",
        "whyEmptyMessage",
        "tagSelected",
        "showProgressOfConstantTask",
        "setPercentOfEachRecordings",
        "totalRecordingsFound",
        "showProgressOfRecordingTask",
        "slotNetworkResult",
        "httpStatus",
        "app",
        "message",
        "code",
        "slotTrackChanged",
        "TrackId",
        "trackId",
        "apply",
        "applyTags",
        "applyCover",
        "retry",
        "quit",
        "reject",
        "slotNext",
        "slotPrev",
        "slotCoverFound",
        "const QObject*",
        "pRequester",
        "CoverInfo",
        "coverInfo",
        "QPixmap",
        "pixmap",
        "slotStartFetchCoverArt",
        "QUuid",
        "albumReleaseId",
        "allUrls",
        "slotLoadFetchedCoverArt",
        "data",
        "slotCoverArtLinkNotFound",
        "slotWorkerStarted",
        "slotWorkerAskOverwrite",
        "coverArtAbsolutePath",
        "std::promise<CoverArtCopyWorker::OverwriteAnswer>*",
        "pPromise",
        "slotWorkerCoverArtCopyFailed",
        "errorMessage",
        "slotWorkerCoverArtUpdated",
        "CoverInfoRelative",
        "slotWorkerFinished"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'next'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'previous'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'loadTrack'
        QtMocHelpers::SlotData<void(const TrackPointer &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'loadTrack'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'fetchTagFinished'
        QtMocHelpers::SlotData<void(TrackPointer, const QList<mixxx::musicbrainz::TrackRelease> &, const QString &)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 10, 11 }, { QMetaType::QString, 12 },
        }}),
        // Slot 'tagSelected'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showProgressOfConstantTask'
        QtMocHelpers::SlotData<void(const QString &)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'setPercentOfEachRecordings'
        QtMocHelpers::SlotData<void(int)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 16 },
        }}),
        // Slot 'showProgressOfRecordingTask'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotNetworkResult'
        QtMocHelpers::SlotData<void(int, const QString &, const QString &, int)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 19 }, { QMetaType::QString, 20 }, { QMetaType::QString, 21 }, { QMetaType::Int, 22 },
        }}),
        // Slot 'slotTrackChanged'
        QtMocHelpers::SlotData<void(TrackId)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Slot 'apply'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'applyTags'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'applyCover'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'retry'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'quit'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'reject'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotNext'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotPrev'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotCoverFound'
        QtMocHelpers::SlotData<void(const QObject *, const CoverInfo &, const QPixmap &)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 35, 36 }, { 0x80000000 | 37, 38 }, { 0x80000000 | 39, 40 },
        }}),
        // Slot 'slotStartFetchCoverArt'
        QtMocHelpers::SlotData<void(const QUuid &, const QList<QString> &)>(41, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 42, 43 }, { QMetaType::QStringList, 44 },
        }}),
        // Slot 'slotLoadFetchedCoverArt'
        QtMocHelpers::SlotData<void(const QUuid &, const QByteArray &)>(45, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 42, 43 }, { QMetaType::QByteArray, 46 },
        }}),
        // Slot 'slotCoverArtLinkNotFound'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotWorkerStarted'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotWorkerAskOverwrite'
        QtMocHelpers::SlotData<void(const QString &, std::promise<CoverArtCopyWorker::OverwriteAnswer> *)>(49, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 50 }, { 0x80000000 | 51, 52 },
        }}),
        // Slot 'slotWorkerCoverArtCopyFailed'
        QtMocHelpers::SlotData<void(const QString &)>(53, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 54 },
        }}),
        // Slot 'slotWorkerCoverArtUpdated'
        QtMocHelpers::SlotData<void(const CoverInfoRelative &)>(55, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 56, 38 },
        }}),
        // Slot 'slotWorkerFinished'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DlgTagFetcher, qt_meta_tag_ZN13DlgTagFetcherE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DlgTagFetcher::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13DlgTagFetcherE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13DlgTagFetcherE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13DlgTagFetcherE_t>.metaTypes,
    nullptr
} };

void DlgTagFetcher::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DlgTagFetcher *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->next(); break;
        case 1: _t->previous(); break;
        case 2: _t->loadTrack((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1]))); break;
        case 3: _t->loadTrack((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 4: _t->fetchTagFinished((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<mixxx::musicbrainz::TrackRelease>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 5: _t->tagSelected(); break;
        case 6: _t->showProgressOfConstantTask((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->setPercentOfEachRecordings((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->showProgressOfRecordingTask(); break;
        case 9: _t->slotNetworkResult((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 10: _t->slotTrackChanged((*reinterpret_cast<std::add_pointer_t<TrackId>>(_a[1]))); break;
        case 11: _t->apply(); break;
        case 12: _t->applyTags(); break;
        case 13: _t->applyCover(); break;
        case 14: _t->retry(); break;
        case 15: _t->quit(); break;
        case 16: _t->reject(); break;
        case 17: _t->slotNext(); break;
        case 18: _t->slotPrev(); break;
        case 19: _t->slotCoverFound((*reinterpret_cast<std::add_pointer_t<const QObject*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CoverInfo>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QPixmap>>(_a[3]))); break;
        case 20: _t->slotStartFetchCoverArt((*reinterpret_cast<std::add_pointer_t<QUuid>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QString>>>(_a[2]))); break;
        case 21: _t->slotLoadFetchedCoverArt((*reinterpret_cast<std::add_pointer_t<QUuid>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 22: _t->slotCoverArtLinkNotFound(); break;
        case 23: _t->slotWorkerStarted(); break;
        case 24: _t->slotWorkerAskOverwrite((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<std::promise<CoverArtCopyWorker::OverwriteAnswer>*>>(_a[2]))); break;
        case 25: _t->slotWorkerCoverArtCopyFailed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 26: _t->slotWorkerCoverArtUpdated((*reinterpret_cast<std::add_pointer_t<CoverInfoRelative>>(_a[1]))); break;
        case 27: _t->slotWorkerFinished(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<mixxx::musicbrainz::TrackRelease> >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackId >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DlgTagFetcher::*)()>(_a, &DlgTagFetcher::next, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgTagFetcher::*)()>(_a, &DlgTagFetcher::previous, 1))
            return;
    }
}

const QMetaObject *DlgTagFetcher::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DlgTagFetcher::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13DlgTagFetcherE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::DlgTagFetcher"))
        return static_cast< Ui::DlgTagFetcher*>(this);
    return QDialog::qt_metacast(_clname);
}

int DlgTagFetcher::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 28)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 28;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 28)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 28;
    }
    return _id;
}

// SIGNAL 0
void DlgTagFetcher::next()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void DlgTagFetcher::previous()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
