/********************************************************************************
** Form generated from reading UI file 'dlgprefinterfacedlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLGPREFINTERFACEDLG_H
#define UI_DLGPREFINTERFACEDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DlgPrefControlsDlg
{
public:
    QVBoxLayout *gridLayout_skin;
    QGroupBox *groupBoxSkinOptions;
    QVBoxLayout *skinLayout;
    QComboBox *ComboBoxSkinconf;
    QLabel *labelColorScheme;
    QComboBox *ComboBoxSchemeconf;
    QLabel *skinDescriptionText;
    QLabel *warningLabel;
    QLabel *skinPreviewLabel;
    QGroupBox *groupBoxMiscOptions;
    QGridLayout *gridLayout_Misc;
    QLabel *textLabelLocale;
    QComboBox *ComboBoxLocale;
    QLabel *labelScaleFactor;
    QHBoxLayout *horizontalLayout_2;
    QDoubleSpinBox *spinBoxScaleFactor;
    QSpacerItem *horizontalSpacer_2;
    QLabel *labelFullscreenOption;
    QCheckBox *checkBoxStartFullScreen;
    QLabel *labelHideMenuBarOption;
    QCheckBox *checkBoxHideMenuBar;
    QLabel *labelTooltips;
    QHBoxLayout *horizontalLayout;
    QRadioButton *radioButtonTooltipsOff;
    QRadioButton *radioButtonTooltipsLibrary;
    QRadioButton *radioButtonTooltipsLibraryAndSkin;
    QLabel *label;
    QComboBox *comboBoxScreensaver;
    QLabel *multiSamplingLabel;
    QComboBox *multiSamplingComboBox;
    QSpacerItem *verticalSpacer;
    QButtonGroup *buttonGroupTooltips;

    void setupUi(QWidget *DlgPrefControlsDlg)
    {
        if (DlgPrefControlsDlg->objectName().isEmpty())
            DlgPrefControlsDlg->setObjectName("DlgPrefControlsDlg");
        DlgPrefControlsDlg->resize(602, 723);
        gridLayout_skin = new QVBoxLayout(DlgPrefControlsDlg);
        gridLayout_skin->setSpacing(6);
        gridLayout_skin->setContentsMargins(11, 11, 11, 11);
        gridLayout_skin->setObjectName("gridLayout_skin");
        groupBoxSkinOptions = new QGroupBox(DlgPrefControlsDlg);
        groupBoxSkinOptions->setObjectName("groupBoxSkinOptions");
        skinLayout = new QVBoxLayout(groupBoxSkinOptions);
        skinLayout->setSpacing(6);
        skinLayout->setContentsMargins(11, 11, 11, 11);
        skinLayout->setObjectName("skinLayout");
        ComboBoxSkinconf = new QComboBox(groupBoxSkinOptions);
        ComboBoxSkinconf->setObjectName("ComboBoxSkinconf");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(ComboBoxSkinconf->sizePolicy().hasHeightForWidth());
        ComboBoxSkinconf->setSizePolicy(sizePolicy);

        skinLayout->addWidget(ComboBoxSkinconf);

        labelColorScheme = new QLabel(groupBoxSkinOptions);
        labelColorScheme->setObjectName("labelColorScheme");
        labelColorScheme->setEnabled(true);
        QFont font;
        labelColorScheme->setFont(font);
        labelColorScheme->setWordWrap(false);

        skinLayout->addWidget(labelColorScheme);

        ComboBoxSchemeconf = new QComboBox(groupBoxSkinOptions);
        ComboBoxSchemeconf->setObjectName("ComboBoxSchemeconf");
        sizePolicy.setHeightForWidth(ComboBoxSchemeconf->sizePolicy().hasHeightForWidth());
        ComboBoxSchemeconf->setSizePolicy(sizePolicy);

        skinLayout->addWidget(ComboBoxSchemeconf);

        skinDescriptionText = new QLabel(groupBoxSkinOptions);
        skinDescriptionText->setObjectName("skinDescriptionText");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(skinDescriptionText->sizePolicy().hasHeightForWidth());
        skinDescriptionText->setSizePolicy(sizePolicy1);
        skinDescriptionText->setText(QString::fromUtf8("skin description goes here"));
        skinDescriptionText->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        skinDescriptionText->setWordWrap(true);

        skinLayout->addWidget(skinDescriptionText);

        warningLabel = new QLabel(groupBoxSkinOptions);
        warningLabel->setObjectName("warningLabel");
        warningLabel->setAlignment(Qt::AlignJustify|Qt::AlignVCenter);

        skinLayout->addWidget(warningLabel);

        skinPreviewLabel = new QLabel(groupBoxSkinOptions);
        skinPreviewLabel->setObjectName("skinPreviewLabel");
        skinPreviewLabel->setText(QString::fromUtf8("skin preview screenshot goes here"));

        skinLayout->addWidget(skinPreviewLabel);


        gridLayout_skin->addWidget(groupBoxSkinOptions);

        groupBoxMiscOptions = new QGroupBox(DlgPrefControlsDlg);
        groupBoxMiscOptions->setObjectName("groupBoxMiscOptions");
        gridLayout_Misc = new QGridLayout(groupBoxMiscOptions);
        gridLayout_Misc->setSpacing(6);
        gridLayout_Misc->setContentsMargins(11, 11, 11, 11);
        gridLayout_Misc->setObjectName("gridLayout_Misc");
        textLabelLocale = new QLabel(groupBoxMiscOptions);
        textLabelLocale->setObjectName("textLabelLocale");

        gridLayout_Misc->addWidget(textLabelLocale, 0, 0, 1, 1);

        ComboBoxLocale = new QComboBox(groupBoxMiscOptions);
        ComboBoxLocale->setObjectName("ComboBoxLocale");

        gridLayout_Misc->addWidget(ComboBoxLocale, 0, 1, 1, 2);

        labelScaleFactor = new QLabel(groupBoxMiscOptions);
        labelScaleFactor->setObjectName("labelScaleFactor");

        gridLayout_Misc->addWidget(labelScaleFactor, 1, 0, 1, 1);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        spinBoxScaleFactor = new QDoubleSpinBox(groupBoxMiscOptions);
        spinBoxScaleFactor->setObjectName("spinBoxScaleFactor");
        spinBoxScaleFactor->setSuffix(QString::fromUtf8(" %"));
        spinBoxScaleFactor->setDecimals(0);
        spinBoxScaleFactor->setMinimum(50.000000000000000);
        spinBoxScaleFactor->setMaximum(400.000000000000000);
        spinBoxScaleFactor->setSingleStep(25.000000000000000);

        horizontalLayout_2->addWidget(spinBoxScaleFactor);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);


        gridLayout_Misc->addLayout(horizontalLayout_2, 1, 1, 1, 1);

        labelFullscreenOption = new QLabel(groupBoxMiscOptions);
        labelFullscreenOption->setObjectName("labelFullscreenOption");

        gridLayout_Misc->addWidget(labelFullscreenOption, 2, 0, 1, 1);

        checkBoxStartFullScreen = new QCheckBox(groupBoxMiscOptions);
        checkBoxStartFullScreen->setObjectName("checkBoxStartFullScreen");

        gridLayout_Misc->addWidget(checkBoxStartFullScreen, 2, 1, 1, 2);

        labelHideMenuBarOption = new QLabel(groupBoxMiscOptions);
        labelHideMenuBarOption->setObjectName("labelHideMenuBarOption");

        gridLayout_Misc->addWidget(labelHideMenuBarOption, 3, 0, 1, 1);

        checkBoxHideMenuBar = new QCheckBox(groupBoxMiscOptions);
        checkBoxHideMenuBar->setObjectName("checkBoxHideMenuBar");

        gridLayout_Misc->addWidget(checkBoxHideMenuBar, 3, 1, 1, 2);

        labelTooltips = new QLabel(groupBoxMiscOptions);
        labelTooltips->setObjectName("labelTooltips");
        labelTooltips->setEnabled(true);
        labelTooltips->setFont(font);
        labelTooltips->setWordWrap(false);

        gridLayout_Misc->addWidget(labelTooltips, 4, 0, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        radioButtonTooltipsOff = new QRadioButton(groupBoxMiscOptions);
        buttonGroupTooltips = new QButtonGroup(DlgPrefControlsDlg);
        buttonGroupTooltips->setObjectName("buttonGroupTooltips");
        buttonGroupTooltips->addButton(radioButtonTooltipsOff);
        radioButtonTooltipsOff->setObjectName("radioButtonTooltipsOff");

        horizontalLayout->addWidget(radioButtonTooltipsOff);

        radioButtonTooltipsLibrary = new QRadioButton(groupBoxMiscOptions);
        buttonGroupTooltips->addButton(radioButtonTooltipsLibrary);
        radioButtonTooltipsLibrary->setObjectName("radioButtonTooltipsLibrary");

        horizontalLayout->addWidget(radioButtonTooltipsLibrary);

        radioButtonTooltipsLibraryAndSkin = new QRadioButton(groupBoxMiscOptions);
        buttonGroupTooltips->addButton(radioButtonTooltipsLibraryAndSkin);
        radioButtonTooltipsLibraryAndSkin->setObjectName("radioButtonTooltipsLibraryAndSkin");

        horizontalLayout->addWidget(radioButtonTooltipsLibraryAndSkin);


        gridLayout_Misc->addLayout(horizontalLayout, 4, 1, 1, 2);

        label = new QLabel(groupBoxMiscOptions);
        label->setObjectName("label");

        gridLayout_Misc->addWidget(label, 5, 0, 1, 1);

        comboBoxScreensaver = new QComboBox(groupBoxMiscOptions);
        comboBoxScreensaver->setObjectName("comboBoxScreensaver");

        gridLayout_Misc->addWidget(comboBoxScreensaver, 5, 1, 1, 2);

        multiSamplingLabel = new QLabel(groupBoxMiscOptions);
        multiSamplingLabel->setObjectName("multiSamplingLabel");

        gridLayout_Misc->addWidget(multiSamplingLabel, 5, 0, 1, 1);

        multiSamplingComboBox = new QComboBox(groupBoxMiscOptions);
        multiSamplingComboBox->setObjectName("multiSamplingComboBox");

        gridLayout_Misc->addWidget(multiSamplingComboBox, 5, 1, 1, 2);


        gridLayout_skin->addWidget(groupBoxMiscOptions);

        verticalSpacer = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_skin->addItem(verticalSpacer);

#if QT_CONFIG(shortcut)
        labelColorScheme->setBuddy(ComboBoxSchemeconf);
        textLabelLocale->setBuddy(ComboBoxLocale);
        labelScaleFactor->setBuddy(spinBoxScaleFactor);
        labelFullscreenOption->setBuddy(checkBoxStartFullScreen);
        labelHideMenuBarOption->setBuddy(checkBoxHideMenuBar);
        labelTooltips->setBuddy(radioButtonTooltipsOff);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(ComboBoxSkinconf, ComboBoxSchemeconf);
        QWidget::setTabOrder(ComboBoxSchemeconf, ComboBoxLocale);
        QWidget::setTabOrder(ComboBoxLocale, spinBoxScaleFactor);
        QWidget::setTabOrder(spinBoxScaleFactor, checkBoxStartFullScreen);
        QWidget::setTabOrder(checkBoxStartFullScreen, checkBoxHideMenuBar);
        QWidget::setTabOrder(checkBoxHideMenuBar, radioButtonTooltipsOff);
        QWidget::setTabOrder(radioButtonTooltipsOff, radioButtonTooltipsLibrary);
        QWidget::setTabOrder(radioButtonTooltipsLibrary, radioButtonTooltipsLibraryAndSkin);
        QWidget::setTabOrder(radioButtonTooltipsLibraryAndSkin, multiSamplingComboBox);

        retranslateUi(DlgPrefControlsDlg);

        QMetaObject::connectSlotsByName(DlgPrefControlsDlg);
    } // setupUi

    void retranslateUi(QWidget *DlgPrefControlsDlg)
    {
        DlgPrefControlsDlg->setWindowTitle(QCoreApplication::translate("DlgPrefControlsDlg", "Interface Preferences", nullptr));
        groupBoxSkinOptions->setTitle(QCoreApplication::translate("DlgPrefControlsDlg", "Skin", nullptr));
#if QT_CONFIG(accessibility)
        ComboBoxSkinconf->setAccessibleName(QCoreApplication::translate("DlgPrefControlsDlg", "Skin selector", nullptr));
#endif // QT_CONFIG(accessibility)
        labelColorScheme->setText(QCoreApplication::translate("DlgPrefControlsDlg", "Color scheme", nullptr));
#if QT_CONFIG(tooltip)
        ComboBoxSchemeconf->setToolTip(QCoreApplication::translate("DlgPrefControlsDlg", "Select from different color schemes of a skin if available.", nullptr));
#endif // QT_CONFIG(tooltip)
        warningLabel->setText(QString());
        groupBoxMiscOptions->setTitle(QCoreApplication::translate("DlgPrefControlsDlg", "Miscellaneous", nullptr));
        textLabelLocale->setText(QCoreApplication::translate("DlgPrefControlsDlg", "Locale", nullptr));
#if QT_CONFIG(tooltip)
        ComboBoxLocale->setToolTip(QCoreApplication::translate("DlgPrefControlsDlg", "Locales determine country and language specific settings.", nullptr));
#endif // QT_CONFIG(tooltip)
        labelScaleFactor->setText(QCoreApplication::translate("DlgPrefControlsDlg", "HiDPI / Retina scaling", nullptr));
#if QT_CONFIG(tooltip)
        spinBoxScaleFactor->setToolTip(QCoreApplication::translate("DlgPrefControlsDlg", "Change the size of text, buttons, and other items.", nullptr));
#endif // QT_CONFIG(tooltip)
        labelFullscreenOption->setText(QCoreApplication::translate("DlgPrefControlsDlg", "Full-screen mode", nullptr));
        checkBoxStartFullScreen->setText(QCoreApplication::translate("DlgPrefControlsDlg", "Start in full-screen mode", nullptr));
        labelHideMenuBarOption->setText(QCoreApplication::translate("DlgPrefControlsDlg", "Menu bar", nullptr));
        checkBoxHideMenuBar->setText(QCoreApplication::translate("DlgPrefControlsDlg", "Auto-hide the menu bar, toggle it with a single Alt key press", nullptr));
        labelTooltips->setText(QCoreApplication::translate("DlgPrefControlsDlg", "Tool tips", nullptr));
        radioButtonTooltipsOff->setText(QCoreApplication::translate("DlgPrefControlsDlg", "Off", nullptr));
        radioButtonTooltipsLibrary->setText(QCoreApplication::translate("DlgPrefControlsDlg", "Library only", nullptr));
        radioButtonTooltipsLibraryAndSkin->setText(QCoreApplication::translate("DlgPrefControlsDlg", "Library and Skin", nullptr));
        label->setText(QCoreApplication::translate("DlgPrefControlsDlg", "Screen saver", nullptr));
        multiSamplingLabel->setText(QCoreApplication::translate("DlgPrefControlsDlg", "Multi-Sampling", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgPrefControlsDlg: public Ui_DlgPrefControlsDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLGPREFINTERFACEDLG_H
