/********************************************************************************
** Form generated from reading UI file 'dlgprefcontrollerdlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLGPREFCONTROLLERDLG_H
#define UI_DLGPREFCONTROLLERDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTableView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DlgPrefControllerDlg
{
public:
    QVBoxLayout *verticalLayout_2;
    QTabWidget *controllerTabs;
    QWidget *mappingInfoTab;
    QGridLayout *gridLayout_2;
    QGroupBox *groupBoxWarning;
    QGridLayout *gridLayout_3;
    QLabel *labelWarningIcon;
    QLabel *labelWarning;
    QLabel *label_LoadMapping;
    QLabel *labelDeviceCategory;
    QComboBox *comboBoxMapping;
    QPushButton *btnLearningWizard;
    QLabel *labelDeviceName;
    QCheckBox *chkEnabledDevice;
    QSpacerItem *verticalSpacer;
    QGroupBox *groupBox;
    QGridLayout *gridLayout;
    QLabel *labelMappingName;
    QLabel *labelLoadedMapping;
    QLabel *label_author;
    QLabel *labelLoadedMappingAuthor;
    QLabel *label_description;
    QLabel *labelLoadedMappingDescription;
    QLabel *label_support;
    QLabel *labelLoadedMappingSupportLinks;
    QLabel *label_scriptFiles;
    QLabel *labelLoadedMappingScriptFileLinks;
    QGroupBox *groupBoxSettings;
    QVBoxLayout *verticalLayout_3;
    QWidget *inputMappingsTab;
    QVBoxLayout *verticalLayout_4;
    QHBoxLayout *layout_inputSearch;
    QLineEdit *inputControlSearch;
    QPushButton *inputControlSearchBtn;
    QTableView *m_pInputMappingTableView;
    QGroupBox *groupBoxInputManagement;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *btnAddInputMapping;
    QPushButton *btnRemoveInputMappings;
    QSpacerItem *horizontalSpacer_3;
    QPushButton *btnClearAllInputMappings;
    QWidget *outputMappingsTab;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *layout_outputSearch;
    QLineEdit *outputControlSearch;
    QPushButton *outputControlSearchBtn;
    QTableView *m_pOutputMappingTableView;
    QGroupBox *groupBoxOutputs;
    QHBoxLayout *horizontalLayout;
    QPushButton *btnAddOutputMapping;
    QPushButton *btnRemoveOutputMappings;
    QSpacerItem *horizontalSpacer_2;
    QPushButton *btnClearAllOutputMappings;

    void setupUi(QWidget *DlgPrefControllerDlg)
    {
        if (DlgPrefControllerDlg->objectName().isEmpty())
            DlgPrefControllerDlg->setObjectName("DlgPrefControllerDlg");
        DlgPrefControllerDlg->resize(507, 437);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(DlgPrefControllerDlg->sizePolicy().hasHeightForWidth());
        DlgPrefControllerDlg->setSizePolicy(sizePolicy);
        verticalLayout_2 = new QVBoxLayout(DlgPrefControllerDlg);
        verticalLayout_2->setObjectName("verticalLayout_2");
        controllerTabs = new QTabWidget(DlgPrefControllerDlg);
        controllerTabs->setObjectName("controllerTabs");
        mappingInfoTab = new QWidget();
        mappingInfoTab->setObjectName("mappingInfoTab");
        gridLayout_2 = new QGridLayout(mappingInfoTab);
        gridLayout_2->setObjectName("gridLayout_2");
        groupBoxWarning = new QGroupBox(mappingInfoTab);
        groupBoxWarning->setObjectName("groupBoxWarning");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(groupBoxWarning->sizePolicy().hasHeightForWidth());
        groupBoxWarning->setSizePolicy(sizePolicy1);
        gridLayout_3 = new QGridLayout(groupBoxWarning);
        gridLayout_3->setObjectName("gridLayout_3");
        labelWarningIcon = new QLabel(groupBoxWarning);
        labelWarningIcon->setObjectName("labelWarningIcon");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(labelWarningIcon->sizePolicy().hasHeightForWidth());
        labelWarningIcon->setSizePolicy(sizePolicy2);
        labelWarningIcon->setMinimumSize(QSize(50, 50));
        labelWarningIcon->setMaximumSize(QSize(50, 50));
        labelWarningIcon->setText(QString::fromUtf8("(icon)"));

        gridLayout_3->addWidget(labelWarningIcon, 0, 0, 1, 1);

        labelWarning = new QLabel(groupBoxWarning);
        labelWarning->setObjectName("labelWarning");
        labelWarning->setText(QString::fromUtf8("(Warning message goes here)"));
        labelWarning->setWordWrap(true);
        labelWarning->setOpenExternalLinks(true);
        labelWarning->setTextInteractionFlags(Qt::TextBrowserInteraction);

        gridLayout_3->addWidget(labelWarning, 0, 1, 1, 1);


        gridLayout_2->addWidget(groupBoxWarning, 13, 0, 1, 2);

        label_LoadMapping = new QLabel(mappingInfoTab);
        label_LoadMapping->setObjectName("label_LoadMapping");
        sizePolicy.setHeightForWidth(label_LoadMapping->sizePolicy().hasHeightForWidth());
        label_LoadMapping->setSizePolicy(sizePolicy);
        label_LoadMapping->setMaximumSize(QSize(16777215, 16777215));
        label_LoadMapping->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(label_LoadMapping, 2, 0, 1, 1);

        labelDeviceCategory = new QLabel(mappingInfoTab);
        labelDeviceCategory->setObjectName("labelDeviceCategory");
        labelDeviceCategory->setEnabled(true);
        sizePolicy2.setHeightForWidth(labelDeviceCategory->sizePolicy().hasHeightForWidth());
        labelDeviceCategory->setSizePolicy(sizePolicy2);
#if QT_CONFIG(tooltip)
        labelDeviceCategory->setToolTip(QString::fromUtf8(""));
#endif // QT_CONFIG(tooltip)
        labelDeviceCategory->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(labelDeviceCategory, 0, 1, 1, 1);

        comboBoxMapping = new QComboBox(mappingInfoTab);
        comboBoxMapping->setObjectName("comboBoxMapping");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(comboBoxMapping->sizePolicy().hasHeightForWidth());
        comboBoxMapping->setSizePolicy(sizePolicy3);
        comboBoxMapping->setMaximumSize(QSize(16777215, 16777215));

        gridLayout_2->addWidget(comboBoxMapping, 2, 1, 1, 1);

        btnLearningWizard = new QPushButton(mappingInfoTab);
        btnLearningWizard->setObjectName("btnLearningWizard");
        btnLearningWizard->setStyleSheet(QString::fromUtf8(""));
        btnLearningWizard->setCheckable(false);
        btnLearningWizard->setChecked(false);

        gridLayout_2->addWidget(btnLearningWizard, 1, 1, 1, 1);

        labelDeviceName = new QLabel(mappingInfoTab);
        labelDeviceName->setObjectName("labelDeviceName");
        labelDeviceName->setEnabled(true);
        sizePolicy2.setHeightForWidth(labelDeviceName->sizePolicy().hasHeightForWidth());
        labelDeviceName->setSizePolicy(sizePolicy2);
        QFont font;
        font.setPointSize(14);
        font.setBold(true);
        labelDeviceName->setFont(font);

        gridLayout_2->addWidget(labelDeviceName, 0, 0, 1, 1);

        chkEnabledDevice = new QCheckBox(mappingInfoTab);
        chkEnabledDevice->setObjectName("chkEnabledDevice");

        gridLayout_2->addWidget(chkEnabledDevice, 1, 0, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_2->addItem(verticalSpacer, 12, 0, 1, 1);

        groupBox = new QGroupBox(mappingInfoTab);
        groupBox->setObjectName("groupBox");
        gridLayout = new QGridLayout(groupBox);
        gridLayout->setObjectName("gridLayout");
        labelMappingName = new QLabel(groupBox);
        labelMappingName->setObjectName("labelMappingName");
        sizePolicy2.setHeightForWidth(labelMappingName->sizePolicy().hasHeightForWidth());
        labelMappingName->setSizePolicy(sizePolicy2);
        labelMappingName->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(labelMappingName, 0, 0, 1, 1);

        labelLoadedMapping = new QLabel(groupBox);
        labelLoadedMapping->setObjectName("labelLoadedMapping");
        sizePolicy2.setHeightForWidth(labelLoadedMapping->sizePolicy().hasHeightForWidth());
        labelLoadedMapping->setSizePolicy(sizePolicy2);
        labelLoadedMapping->setMinimumSize(QSize(0, 0));
#if QT_CONFIG(tooltip)
        labelLoadedMapping->setToolTip(QString::fromUtf8(""));
#endif // QT_CONFIG(tooltip)
        labelLoadedMapping->setText(QString::fromUtf8("(mapping name goes here)"));
        labelLoadedMapping->setWordWrap(true);

        gridLayout->addWidget(labelLoadedMapping, 0, 1, 1, 1);

        label_author = new QLabel(groupBox);
        label_author->setObjectName("label_author");
        sizePolicy2.setHeightForWidth(label_author->sizePolicy().hasHeightForWidth());
        label_author->setSizePolicy(sizePolicy2);
        label_author->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_author, 1, 0, 1, 1);

        labelLoadedMappingAuthor = new QLabel(groupBox);
        labelLoadedMappingAuthor->setObjectName("labelLoadedMappingAuthor");
        sizePolicy2.setHeightForWidth(labelLoadedMappingAuthor->sizePolicy().hasHeightForWidth());
        labelLoadedMappingAuthor->setSizePolicy(sizePolicy2);
        labelLoadedMappingAuthor->setText(QString::fromUtf8("(mapping author goes here)"));

        gridLayout->addWidget(labelLoadedMappingAuthor, 1, 1, 1, 1);

        label_description = new QLabel(groupBox);
        label_description->setObjectName("label_description");
        sizePolicy2.setHeightForWidth(label_description->sizePolicy().hasHeightForWidth());
        label_description->setSizePolicy(sizePolicy2);
        label_description->setAlignment(Qt::AlignRight|Qt::AlignTop|Qt::AlignTrailing);

        gridLayout->addWidget(label_description, 2, 0, 1, 1);

        labelLoadedMappingDescription = new QLabel(groupBox);
        labelLoadedMappingDescription->setObjectName("labelLoadedMappingDescription");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(labelLoadedMappingDescription->sizePolicy().hasHeightForWidth());
        labelLoadedMappingDescription->setSizePolicy(sizePolicy4);
        labelLoadedMappingDescription->setMinimumSize(QSize(0, 0));
        labelLoadedMappingDescription->setFocusPolicy(Qt::NoFocus);
#if QT_CONFIG(tooltip)
        labelLoadedMappingDescription->setToolTip(QString::fromUtf8(""));
#endif // QT_CONFIG(tooltip)
        labelLoadedMappingDescription->setText(QString::fromUtf8("(mapping description goes here) even when it is a damn long text which should wrap but does not"));
        labelLoadedMappingDescription->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        labelLoadedMappingDescription->setWordWrap(true);

        gridLayout->addWidget(labelLoadedMappingDescription, 2, 1, 1, 1);

        label_support = new QLabel(groupBox);
        label_support->setObjectName("label_support");
        sizePolicy2.setHeightForWidth(label_support->sizePolicy().hasHeightForWidth());
        label_support->setSizePolicy(sizePolicy2);
        label_support->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_support, 3, 0, 1, 1);

        labelLoadedMappingSupportLinks = new QLabel(groupBox);
        labelLoadedMappingSupportLinks->setObjectName("labelLoadedMappingSupportLinks");
        sizePolicy2.setHeightForWidth(labelLoadedMappingSupportLinks->sizePolicy().hasHeightForWidth());
        labelLoadedMappingSupportLinks->setSizePolicy(sizePolicy2);
        labelLoadedMappingSupportLinks->setFocusPolicy(Qt::ClickFocus);
#if QT_CONFIG(tooltip)
        labelLoadedMappingSupportLinks->setToolTip(QString::fromUtf8(""));
#endif // QT_CONFIG(tooltip)
        labelLoadedMappingSupportLinks->setInputMethodHints(Qt::ImhUrlCharactersOnly);
        labelLoadedMappingSupportLinks->setText(QString::fromUtf8("(forum link for mapping goes here)"));
        labelLoadedMappingSupportLinks->setOpenExternalLinks(true);
        labelLoadedMappingSupportLinks->setTextInteractionFlags(Qt::LinksAccessibleByKeyboard|Qt::LinksAccessibleByMouse);

        gridLayout->addWidget(labelLoadedMappingSupportLinks, 3, 1, 1, 1);

        label_scriptFiles = new QLabel(groupBox);
        label_scriptFiles->setObjectName("label_scriptFiles");
        sizePolicy2.setHeightForWidth(label_scriptFiles->sizePolicy().hasHeightForWidth());
        label_scriptFiles->setSizePolicy(sizePolicy2);
        label_scriptFiles->setAlignment(Qt::AlignRight|Qt::AlignTop|Qt::AlignTrailing);

        gridLayout->addWidget(label_scriptFiles, 4, 0, 1, 1);

        labelLoadedMappingScriptFileLinks = new QLabel(groupBox);
        labelLoadedMappingScriptFileLinks->setObjectName("labelLoadedMappingScriptFileLinks");
        sizePolicy2.setHeightForWidth(labelLoadedMappingScriptFileLinks->sizePolicy().hasHeightForWidth());
        labelLoadedMappingScriptFileLinks->setSizePolicy(sizePolicy2);
        labelLoadedMappingScriptFileLinks->setFocusPolicy(Qt::ClickFocus);
#if QT_CONFIG(tooltip)
        labelLoadedMappingScriptFileLinks->setToolTip(QString::fromUtf8(""));
#endif // QT_CONFIG(tooltip)
        labelLoadedMappingScriptFileLinks->setInputMethodHints(Qt::ImhUrlCharactersOnly);
        labelLoadedMappingScriptFileLinks->setText(QString::fromUtf8("(links to loaded mapping script files go here)"));
        labelLoadedMappingScriptFileLinks->setOpenExternalLinks(false);
        labelLoadedMappingScriptFileLinks->setTextInteractionFlags(Qt::LinksAccessibleByKeyboard|Qt::LinksAccessibleByMouse);

        gridLayout->addWidget(labelLoadedMappingScriptFileLinks, 4, 1, 1, 1);


        gridLayout_2->addWidget(groupBox, 10, 0, 1, 2);

        groupBoxSettings = new QGroupBox(mappingInfoTab);
        groupBoxSettings->setObjectName("groupBoxSettings");
        QSizePolicy sizePolicy5(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Maximum);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(groupBoxSettings->sizePolicy().hasHeightForWidth());
        groupBoxSettings->setSizePolicy(sizePolicy5);
        groupBoxSettings->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        verticalLayout_3 = new QVBoxLayout(groupBoxSettings);
        verticalLayout_3->setObjectName("verticalLayout_3");

        gridLayout_2->addWidget(groupBoxSettings, 11, 0, 1, 2);

        controllerTabs->addTab(mappingInfoTab, QString());
        inputMappingsTab = new QWidget();
        inputMappingsTab->setObjectName("inputMappingsTab");
        verticalLayout_4 = new QVBoxLayout(inputMappingsTab);
        verticalLayout_4->setObjectName("verticalLayout_4");
        layout_inputSearch = new QHBoxLayout();
        layout_inputSearch->setObjectName("layout_inputSearch");
        inputControlSearch = new QLineEdit(inputMappingsTab);
        inputControlSearch->setObjectName("inputControlSearch");

        layout_inputSearch->addWidget(inputControlSearch);

        inputControlSearchBtn = new QPushButton(inputMappingsTab);
        inputControlSearchBtn->setObjectName("inputControlSearchBtn");

        layout_inputSearch->addWidget(inputControlSearchBtn);


        verticalLayout_4->addLayout(layout_inputSearch);

        m_pInputMappingTableView = new QTableView(inputMappingsTab);
        m_pInputMappingTableView->setObjectName("m_pInputMappingTableView");
        m_pInputMappingTableView->setEditTriggers(QAbstractItemView::AnyKeyPressed|QAbstractItemView::DoubleClicked|QAbstractItemView::EditKeyPressed|QAbstractItemView::SelectedClicked);

        verticalLayout_4->addWidget(m_pInputMappingTableView);

        groupBoxInputManagement = new QGroupBox(inputMappingsTab);
        groupBoxInputManagement->setObjectName("groupBoxInputManagement");
        groupBoxInputManagement->setMinimumSize(QSize(300, 0));
        horizontalLayout_2 = new QHBoxLayout(groupBoxInputManagement);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        btnAddInputMapping = new QPushButton(groupBoxInputManagement);
        btnAddInputMapping->setObjectName("btnAddInputMapping");
        btnAddInputMapping->setStyleSheet(QString::fromUtf8(""));

        horizontalLayout_2->addWidget(btnAddInputMapping);

        btnRemoveInputMappings = new QPushButton(groupBoxInputManagement);
        btnRemoveInputMappings->setObjectName("btnRemoveInputMappings");
        btnRemoveInputMappings->setStyleSheet(QString::fromUtf8(""));

        horizontalLayout_2->addWidget(btnRemoveInputMappings);

        horizontalSpacer_3 = new QSpacerItem(219, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_3);

        btnClearAllInputMappings = new QPushButton(groupBoxInputManagement);
        btnClearAllInputMappings->setObjectName("btnClearAllInputMappings");
        btnClearAllInputMappings->setAutoFillBackground(false);
        btnClearAllInputMappings->setStyleSheet(QString::fromUtf8(""));

        horizontalLayout_2->addWidget(btnClearAllInputMappings);


        verticalLayout_4->addWidget(groupBoxInputManagement);

        controllerTabs->addTab(inputMappingsTab, QString());
        outputMappingsTab = new QWidget();
        outputMappingsTab->setObjectName("outputMappingsTab");
        verticalLayout = new QVBoxLayout(outputMappingsTab);
        verticalLayout->setObjectName("verticalLayout");
        layout_outputSearch = new QHBoxLayout();
        layout_outputSearch->setObjectName("layout_outputSearch");
        outputControlSearch = new QLineEdit(outputMappingsTab);
        outputControlSearch->setObjectName("outputControlSearch");

        layout_outputSearch->addWidget(outputControlSearch);

        outputControlSearchBtn = new QPushButton(outputMappingsTab);
        outputControlSearchBtn->setObjectName("outputControlSearchBtn");

        layout_outputSearch->addWidget(outputControlSearchBtn);


        verticalLayout->addLayout(layout_outputSearch);

        m_pOutputMappingTableView = new QTableView(outputMappingsTab);
        m_pOutputMappingTableView->setObjectName("m_pOutputMappingTableView");
        m_pOutputMappingTableView->setEnabled(true);
        m_pOutputMappingTableView->setEditTriggers(QAbstractItemView::AnyKeyPressed|QAbstractItemView::DoubleClicked|QAbstractItemView::EditKeyPressed|QAbstractItemView::SelectedClicked);

        verticalLayout->addWidget(m_pOutputMappingTableView);

        groupBoxOutputs = new QGroupBox(outputMappingsTab);
        groupBoxOutputs->setObjectName("groupBoxOutputs");
        horizontalLayout = new QHBoxLayout(groupBoxOutputs);
        horizontalLayout->setObjectName("horizontalLayout");
        btnAddOutputMapping = new QPushButton(groupBoxOutputs);
        btnAddOutputMapping->setObjectName("btnAddOutputMapping");
        btnAddOutputMapping->setEnabled(true);

        horizontalLayout->addWidget(btnAddOutputMapping);

        btnRemoveOutputMappings = new QPushButton(groupBoxOutputs);
        btnRemoveOutputMappings->setObjectName("btnRemoveOutputMappings");

        horizontalLayout->addWidget(btnRemoveOutputMappings);

        horizontalSpacer_2 = new QSpacerItem(219, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);

        btnClearAllOutputMappings = new QPushButton(groupBoxOutputs);
        btnClearAllOutputMappings->setObjectName("btnClearAllOutputMappings");

        horizontalLayout->addWidget(btnClearAllOutputMappings);


        verticalLayout->addWidget(groupBoxOutputs);

        controllerTabs->addTab(outputMappingsTab, QString());

        verticalLayout_2->addWidget(controllerTabs);

#if QT_CONFIG(shortcut)
        label_LoadMapping->setBuddy(comboBoxMapping);
#endif // QT_CONFIG(shortcut)

        retranslateUi(DlgPrefControllerDlg);

        controllerTabs->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(DlgPrefControllerDlg);
    } // setupUi

    void retranslateUi(QWidget *DlgPrefControllerDlg)
    {
        DlgPrefControllerDlg->setWindowTitle(QCoreApplication::translate("DlgPrefControllerDlg", "Controller Preferences", nullptr));
        groupBoxWarning->setTitle(QString());
#if QT_CONFIG(tooltip)
        labelWarning->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        label_LoadMapping->setText(QCoreApplication::translate("DlgPrefControllerDlg", "Load Mapping:", nullptr));
        labelDeviceCategory->setText(QCoreApplication::translate("DlgPrefControllerDlg", "(device category goes here)", nullptr));
#if QT_CONFIG(tooltip)
        btnLearningWizard->setToolTip(QCoreApplication::translate("DlgPrefControllerDlg", "Click to start the Controller Learning wizard.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnLearningWizard->setText(QCoreApplication::translate("DlgPrefControllerDlg", "Learning Wizard (MIDI Only)", nullptr));
        labelDeviceName->setText(QCoreApplication::translate("DlgPrefControllerDlg", "Controller Name", nullptr));
        chkEnabledDevice->setText(QCoreApplication::translate("DlgPrefControllerDlg", "Enabled", nullptr));
        groupBox->setTitle(QCoreApplication::translate("DlgPrefControllerDlg", "Mapping Info", nullptr));
        labelMappingName->setText(QCoreApplication::translate("DlgPrefControllerDlg", "Name:", nullptr));
        label_author->setText(QCoreApplication::translate("DlgPrefControllerDlg", "Author:", nullptr));
        label_description->setText(QCoreApplication::translate("DlgPrefControllerDlg", "Description:", nullptr));
        label_support->setText(QCoreApplication::translate("DlgPrefControllerDlg", "Support:", nullptr));
        label_scriptFiles->setText(QCoreApplication::translate("DlgPrefControllerDlg", "Mapping Files:", nullptr));
        groupBoxSettings->setTitle(QCoreApplication::translate("DlgPrefControllerDlg", "Mapping settings", nullptr));
        controllerTabs->setTabText(controllerTabs->indexOf(mappingInfoTab), QCoreApplication::translate("DlgPrefControllerDlg", "Controller Setup", nullptr));
        inputControlSearchBtn->setText(QCoreApplication::translate("DlgPrefControllerDlg", "Search", nullptr));
        groupBoxInputManagement->setTitle(QString());
        btnAddInputMapping->setText(QCoreApplication::translate("DlgPrefControllerDlg", "Add", nullptr));
        btnRemoveInputMappings->setText(QCoreApplication::translate("DlgPrefControllerDlg", "Remove", nullptr));
        btnClearAllInputMappings->setText(QCoreApplication::translate("DlgPrefControllerDlg", "Clear All", nullptr));
        controllerTabs->setTabText(controllerTabs->indexOf(inputMappingsTab), QCoreApplication::translate("DlgPrefControllerDlg", "Input Mappings", nullptr));
        outputControlSearchBtn->setText(QCoreApplication::translate("DlgPrefControllerDlg", "Search", nullptr));
        groupBoxOutputs->setTitle(QString());
        btnAddOutputMapping->setText(QCoreApplication::translate("DlgPrefControllerDlg", "Add", nullptr));
        btnRemoveOutputMappings->setText(QCoreApplication::translate("DlgPrefControllerDlg", "Remove", nullptr));
        btnClearAllOutputMappings->setText(QCoreApplication::translate("DlgPrefControllerDlg", "Clear All", nullptr));
        controllerTabs->setTabText(controllerTabs->indexOf(outputMappingsTab), QCoreApplication::translate("DlgPrefControllerDlg", "Output Mappings", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgPrefControllerDlg: public Ui_DlgPrefControllerDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLGPREFCONTROLLERDLG_H
