/****************************************************************************
** Meta object code from reading C++ file 'wlibrarytableview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/widget/wlibrarytableview.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wlibrarytableview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17WLibraryTableViewE_t {};
} // unnamed namespace

template <> constexpr inline auto WLibraryTableView::qt_create_metaobjectdata<qt_meta_tag_ZN17WLibraryTableViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "WLibraryTableView",
        "loadTrack",
        "",
        "TrackPointer",
        "pTrack",
        "loadTrackToPlayer",
        "group",
        "play",
        "trackSelected",
        "onlyCachedCoverArt",
        "scrollValueChanged",
        "setLibraryFocus",
        "FocusWidget",
        "newFocus",
        "setTrackTableFont",
        "QFont",
        "font",
        "setTrackTableRowHeight",
        "rowHeight",
        "setSelectedClick",
        "enable"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'loadTrack'
        QtMocHelpers::SignalData<void(TrackPointer)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'loadTrackToPlayer'
        QtMocHelpers::SignalData<void(TrackPointer, const QString &, bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 6 }, { QMetaType::Bool, 7 },
        }}),
        // Signal 'loadTrackToPlayer'
        QtMocHelpers::SignalData<void(TrackPointer, const QString &)>(5, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 6 },
        }}),
        // Signal 'trackSelected'
        QtMocHelpers::SignalData<void(TrackPointer)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'onlyCachedCoverArt'
        QtMocHelpers::SignalData<void(bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'scrollValueChanged'
        QtMocHelpers::SignalData<void(int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Signal 'setLibraryFocus'
        QtMocHelpers::SignalData<FocusWidget(FocusWidget)>(11, 2, QMC::AccessPublic, 0x80000000 | 12, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'setTrackTableFont'
        QtMocHelpers::SlotData<void(const QFont &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'setTrackTableRowHeight'
        QtMocHelpers::SlotData<void(int)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 18 },
        }}),
        // Slot 'setSelectedClick'
        QtMocHelpers::SlotData<void(bool)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 20 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WLibraryTableView, qt_meta_tag_ZN17WLibraryTableViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject WLibraryTableView::staticMetaObject = { {
    QMetaObject::SuperData::link<QTableView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17WLibraryTableViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17WLibraryTableViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17WLibraryTableViewE_t>.metaTypes,
    nullptr
} };

void WLibraryTableView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WLibraryTableView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->loadTrack((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1]))); break;
        case 1: _t->loadTrackToPlayer((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 2: _t->loadTrackToPlayer((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 3: _t->trackSelected((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1]))); break;
        case 4: _t->onlyCachedCoverArt((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->scrollValueChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: { FocusWidget _r = _t->setLibraryFocus((*reinterpret_cast<std::add_pointer_t<FocusWidget>>(_a[1])));
            if (_a[0]) *reinterpret_cast<FocusWidget*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->setTrackTableFont((*reinterpret_cast<std::add_pointer_t<QFont>>(_a[1]))); break;
        case 8: _t->setTrackTableRowHeight((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->setSelectedClick((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WLibraryTableView::*)(TrackPointer )>(_a, &WLibraryTableView::loadTrack, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WLibraryTableView::*)(TrackPointer , const QString & , bool )>(_a, &WLibraryTableView::loadTrackToPlayer, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WLibraryTableView::*)(TrackPointer )>(_a, &WLibraryTableView::trackSelected, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (WLibraryTableView::*)(bool )>(_a, &WLibraryTableView::onlyCachedCoverArt, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (WLibraryTableView::*)(int )>(_a, &WLibraryTableView::scrollValueChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<FocusWidget (WLibraryTableView::*)(FocusWidget )>(_a, &WLibraryTableView::setLibraryFocus, 6))
            return;
    }
}

const QMetaObject *WLibraryTableView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *WLibraryTableView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17WLibraryTableViewE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "LibraryView"))
        return static_cast< LibraryView*>(this);
    return QTableView::qt_metacast(_clname);
}

int WLibraryTableView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTableView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void WLibraryTableView::loadTrack(TrackPointer _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void WLibraryTableView::loadTrackToPlayer(TrackPointer _t1, const QString & _t2, bool _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}

// SIGNAL 3
void WLibraryTableView::trackSelected(TrackPointer _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void WLibraryTableView::onlyCachedCoverArt(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void WLibraryTableView::scrollValueChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
FocusWidget WLibraryTableView::setLibraryFocus(FocusWidget _t1)
{
    FocusWidget _t0{};
    QMetaObject::activate<FocusWidget>(this, &staticMetaObject, 6, std::addressof(_t0), _t1);
    return _t0;
}
QT_WARNING_POP
