/********************************************************************************
** Form generated from reading UI file 'dlghidden.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLGHIDDEN_H
#define UI_DLGHIDDEN_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DlgHidden
{
public:
    QVBoxLayout *verticalLayout;
    QWidget *LibraryFeatureControls;
    QHBoxLayout *horizontalLayout;
    QPushButton *btnSelect;
    QPushButton *btnPurge;
    QPushButton *btnUnhide;
    QPushButton *btnDelete;
    QSpacerItem *horizontalSpacer;
    QTableView *m_pTrackTablePlaceholder;

    void setupUi(QWidget *DlgHidden)
    {
        if (DlgHidden->objectName().isEmpty())
            DlgHidden->setObjectName("DlgHidden");
        DlgHidden->resize(560, 399);
        verticalLayout = new QVBoxLayout(DlgHidden);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        LibraryFeatureControls = new QWidget(DlgHidden);
        LibraryFeatureControls->setObjectName("LibraryFeatureControls");
        horizontalLayout = new QHBoxLayout(LibraryFeatureControls);
        horizontalLayout->setSpacing(0);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        btnSelect = new QPushButton(LibraryFeatureControls);
        btnSelect->setObjectName("btnSelect");
        btnSelect->setFocusPolicy(Qt::NoFocus);

        horizontalLayout->addWidget(btnSelect);

        btnPurge = new QPushButton(LibraryFeatureControls);
        btnPurge->setObjectName("btnPurge");
        btnPurge->setFocusPolicy(Qt::NoFocus);
        btnPurge->setCheckable(false);

        horizontalLayout->addWidget(btnPurge);

        btnUnhide = new QPushButton(LibraryFeatureControls);
        btnUnhide->setObjectName("btnUnhide");
        btnUnhide->setFocusPolicy(Qt::NoFocus);
        btnUnhide->setCheckable(false);

        horizontalLayout->addWidget(btnUnhide);

        btnDelete = new QPushButton(LibraryFeatureControls);
        btnDelete->setObjectName("btnDelete");
        btnDelete->setFocusPolicy(Qt::NoFocus);
        btnDelete->setCheckable(false);

        horizontalLayout->addWidget(btnDelete);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout->addWidget(LibraryFeatureControls);

        m_pTrackTablePlaceholder = new QTableView(DlgHidden);
        m_pTrackTablePlaceholder->setObjectName("m_pTrackTablePlaceholder");
        m_pTrackTablePlaceholder->setShowGrid(true);

        verticalLayout->addWidget(m_pTrackTablePlaceholder);


        retranslateUi(DlgHidden);

        QMetaObject::connectSlotsByName(DlgHidden);
    } // setupUi

    void retranslateUi(QWidget *DlgHidden)
    {
        DlgHidden->setWindowTitle(QCoreApplication::translate("DlgHidden", "Hidden Tracks", nullptr));
#if QT_CONFIG(tooltip)
        btnSelect->setToolTip(QCoreApplication::translate("DlgHidden", "Selects all tracks in the table below.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnSelect->setText(QCoreApplication::translate("DlgHidden", "Select All", nullptr));
#if QT_CONFIG(tooltip)
        btnPurge->setToolTip(QCoreApplication::translate("DlgHidden", "Purge selected tracks from the library.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnPurge->setText(QCoreApplication::translate("DlgHidden", "Purge", nullptr));
#if QT_CONFIG(tooltip)
        btnUnhide->setToolTip(QCoreApplication::translate("DlgHidden", "Unhide selected tracks from the library.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnUnhide->setText(QCoreApplication::translate("DlgHidden", "Unhide", nullptr));
#if QT_CONFIG(shortcut)
        btnUnhide->setShortcut(QCoreApplication::translate("DlgHidden", "Ctrl+S", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        btnDelete->setToolTip(QCoreApplication::translate("DlgHidden", "Purge selected tracks from the library and delete files from disk.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnDelete->setText(QCoreApplication::translate("DlgHidden", "Purge And Delete Files", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgHidden: public Ui_DlgHidden {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLGHIDDEN_H
