/****************************************************************************
** Meta object code from reading C++ file 'playermanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/mixer/playermanager.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'playermanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN22PlayerManagerInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto PlayerManagerInterface::qt_create_metaobjectdata<qt_meta_tag_ZN22PlayerManagerInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PlayerManagerInterface",
        "numberOfDecksChanged",
        "",
        "decks",
        "numberOfSamplersChanged",
        "samplers"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'numberOfDecksChanged'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'numberOfSamplersChanged'
        QtMocHelpers::SignalData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PlayerManagerInterface, qt_meta_tag_ZN22PlayerManagerInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PlayerManagerInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22PlayerManagerInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22PlayerManagerInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22PlayerManagerInterfaceE_t>.metaTypes,
    nullptr
} };

void PlayerManagerInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PlayerManagerInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->numberOfDecksChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->numberOfSamplersChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PlayerManagerInterface::*)(int )>(_a, &PlayerManagerInterface::numberOfDecksChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerManagerInterface::*)(int )>(_a, &PlayerManagerInterface::numberOfSamplersChanged, 1))
            return;
    }
}

const QMetaObject *PlayerManagerInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PlayerManagerInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22PlayerManagerInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int PlayerManagerInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void PlayerManagerInterface::numberOfDecksChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void PlayerManagerInterface::numberOfSamplersChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN13PlayerManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto PlayerManager::qt_create_metaobjectdata<qt_meta_tag_ZN13PlayerManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PlayerManager",
        "loadLocationToPlayer",
        "",
        "location",
        "group",
        "play",
        "noMicrophoneInputConfigured",
        "noAuxiliaryInputConfigured",
        "noDeckPassthroughInputConfigured",
        "noVinylControlInputConfigured",
        "trackAnalyzerProgress",
        "TrackId",
        "trackId",
        "AnalyzerProgress",
        "analyzerProgress",
        "trackAnalyzerIdle",
        "slotLoadTrackToPlayer",
        "TrackPointer",
        "pTrack",
        "slotLoadLocationToPlayer",
        "slotLoadLocationToPlayerMaybePlay",
        "slotCloneDeck",
        "source_group",
        "target_group",
        "slotLoadTrackIntoNextAvailableDeck",
        "slotLoadLocationIntoNextAvailableDeck",
        "slotLoadToDeck",
        "deckNumber",
        "slotLoadToPreviewDeck",
        "previewDeckNumber",
        "slotLoadTrackIntoNextAvailableSampler",
        "slotLoadToSampler",
        "samplerNumber",
        "slotChangeNumDecks",
        "v",
        "slotChangeNumSamplers",
        "slotChangeNumPreviewDecks",
        "slotChangeNumMicrophones",
        "slotChangeNumAuxiliaries",
        "slotSaveEjectedTrack",
        "track",
        "slotAnalyzeTrack",
        "onTrackAnalysisProgress",
        "onTrackAnalysisFinished"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'loadLocationToPlayer'
        QtMocHelpers::SignalData<void(const QString &, const QString &, bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::Bool, 5 },
        }}),
        // Signal 'noMicrophoneInputConfigured'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'noAuxiliaryInputConfigured'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'noDeckPassthroughInputConfigured'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'noVinylControlInputConfigured'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'trackAnalyzerProgress'
        QtMocHelpers::SignalData<void(TrackId, AnalyzerProgress)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 }, { 0x80000000 | 13, 14 },
        }}),
        // Signal 'trackAnalyzerIdle'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotLoadTrackToPlayer'
        QtMocHelpers::SlotData<void(TrackPointer, const QString &, bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { QMetaType::QString, 4 }, { QMetaType::Bool, 5 },
        }}),
        // Slot 'slotLoadLocationToPlayer'
        QtMocHelpers::SlotData<void(const QString &, const QString &, bool)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::Bool, 5 },
        }}),
        // Slot 'slotLoadLocationToPlayerMaybePlay'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 },
        }}),
        // Slot 'slotCloneDeck'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 22 }, { QMetaType::QString, 23 },
        }}),
        // Slot 'slotLoadTrackIntoNextAvailableDeck'
        QtMocHelpers::SlotData<void(TrackPointer)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'slotLoadLocationIntoNextAvailableDeck'
        QtMocHelpers::SlotData<void(const QString &, bool)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Bool, 5 },
        }}),
        // Slot 'slotLoadLocationIntoNextAvailableDeck'
        QtMocHelpers::SlotData<void(const QString &)>(25, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'slotLoadToDeck'
        QtMocHelpers::SlotData<void(const QString &, int)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Int, 27 },
        }}),
        // Slot 'slotLoadToPreviewDeck'
        QtMocHelpers::SlotData<void(const QString &, int)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Int, 29 },
        }}),
        // Slot 'slotLoadTrackIntoNextAvailableSampler'
        QtMocHelpers::SlotData<void(TrackPointer)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'slotLoadToSampler'
        QtMocHelpers::SlotData<void(const QString &, int)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Int, 32 },
        }}),
        // Slot 'slotChangeNumDecks'
        QtMocHelpers::SlotData<void(double)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 34 },
        }}),
        // Slot 'slotChangeNumSamplers'
        QtMocHelpers::SlotData<void(double)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 34 },
        }}),
        // Slot 'slotChangeNumPreviewDecks'
        QtMocHelpers::SlotData<void(double)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 34 },
        }}),
        // Slot 'slotChangeNumMicrophones'
        QtMocHelpers::SlotData<void(double)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 34 },
        }}),
        // Slot 'slotChangeNumAuxiliaries'
        QtMocHelpers::SlotData<void(double)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 34 },
        }}),
        // Slot 'slotSaveEjectedTrack'
        QtMocHelpers::SlotData<void(TrackPointer)>(39, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 17, 40 },
        }}),
        // Slot 'slotAnalyzeTrack'
        QtMocHelpers::SlotData<void(TrackPointer)>(41, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 40 },
        }}),
        // Slot 'onTrackAnalysisProgress'
        QtMocHelpers::SlotData<void(TrackId, AnalyzerProgress)>(42, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 11, 12 }, { 0x80000000 | 13, 14 },
        }}),
        // Slot 'onTrackAnalysisFinished'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PlayerManager, qt_meta_tag_ZN13PlayerManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PlayerManager::staticMetaObject = { {
    QMetaObject::SuperData::link<PlayerManagerInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13PlayerManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13PlayerManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13PlayerManagerE_t>.metaTypes,
    nullptr
} };

void PlayerManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PlayerManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->loadLocationToPlayer((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 1: _t->noMicrophoneInputConfigured(); break;
        case 2: _t->noAuxiliaryInputConfigured(); break;
        case 3: _t->noDeckPassthroughInputConfigured(); break;
        case 4: _t->noVinylControlInputConfigured(); break;
        case 5: _t->trackAnalyzerProgress((*reinterpret_cast<std::add_pointer_t<TrackId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<AnalyzerProgress>>(_a[2]))); break;
        case 6: _t->trackAnalyzerIdle(); break;
        case 7: _t->slotLoadTrackToPlayer((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 8: _t->slotLoadLocationToPlayer((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 9: _t->slotLoadLocationToPlayerMaybePlay((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 10: _t->slotCloneDeck((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 11: _t->slotLoadTrackIntoNextAvailableDeck((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1]))); break;
        case 12: _t->slotLoadLocationIntoNextAvailableDeck((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 13: _t->slotLoadLocationIntoNextAvailableDeck((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 14: _t->slotLoadToDeck((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 15: _t->slotLoadToPreviewDeck((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 16: _t->slotLoadTrackIntoNextAvailableSampler((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1]))); break;
        case 17: _t->slotLoadToSampler((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 18: _t->slotChangeNumDecks((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 19: _t->slotChangeNumSamplers((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 20: _t->slotChangeNumPreviewDecks((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 21: _t->slotChangeNumMicrophones((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 22: _t->slotChangeNumAuxiliaries((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 23: _t->slotSaveEjectedTrack((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1]))); break;
        case 24: _t->slotAnalyzeTrack((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1]))); break;
        case 25: _t->onTrackAnalysisProgress((*reinterpret_cast<std::add_pointer_t<TrackId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<AnalyzerProgress>>(_a[2]))); break;
        case 26: _t->onTrackAnalysisFinished(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AnalyzerProgress >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackId >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 23:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 24:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 25:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AnalyzerProgress >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackId >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PlayerManager::*)(const QString & , const QString & , bool )>(_a, &PlayerManager::loadLocationToPlayer, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerManager::*)()>(_a, &PlayerManager::noMicrophoneInputConfigured, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerManager::*)()>(_a, &PlayerManager::noAuxiliaryInputConfigured, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerManager::*)()>(_a, &PlayerManager::noDeckPassthroughInputConfigured, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerManager::*)()>(_a, &PlayerManager::noVinylControlInputConfigured, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerManager::*)(TrackId , AnalyzerProgress )>(_a, &PlayerManager::trackAnalyzerProgress, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerManager::*)()>(_a, &PlayerManager::trackAnalyzerIdle, 6))
            return;
    }
}

const QMetaObject *PlayerManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PlayerManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13PlayerManagerE_t>.strings))
        return static_cast<void*>(this);
    return PlayerManagerInterface::qt_metacast(_clname);
}

int PlayerManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PlayerManagerInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    return _id;
}

// SIGNAL 0
void PlayerManager::loadLocationToPlayer(const QString & _t1, const QString & _t2, bool _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void PlayerManager::noMicrophoneInputConfigured()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void PlayerManager::noAuxiliaryInputConfigured()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void PlayerManager::noDeckPassthroughInputConfigured()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void PlayerManager::noVinylControlInputConfigured()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void PlayerManager::trackAnalyzerProgress(TrackId _t1, AnalyzerProgress _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void PlayerManager::trackAnalyzerIdle()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
