/****************************************************************************
** Meta object code from reading C++ file 'webtask.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/network/webtask.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'webtask.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN5mixxx7network7WebTaskE_t {};
} // unnamed namespace

template <> constexpr inline auto mixxx::network::WebTask::qt_create_metaobjectdata<qt_meta_tag_ZN5mixxx7network7WebTaskE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "mixxx::network::WebTask",
        "networkError",
        "",
        "QNetworkReply::NetworkError",
        "errorCode",
        "errorString",
        "mixxx::network::WebResponseWithContent",
        "responseWithContent",
        "slotStart",
        "timeoutMillis",
        "delayMillis",
        "slotAbort",
        "slotNetworkReplyFinished"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'networkError'
        QtMocHelpers::SignalData<void(QNetworkReply::NetworkError, const QString &, const mixxx::network::WebResponseWithContent &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 5 }, { 0x80000000 | 6, 7 },
        }}),
        // Slot 'slotStart'
        QtMocHelpers::SlotData<void(int, int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 }, { QMetaType::Int, 10 },
        }}),
        // Slot 'slotStart'
        QtMocHelpers::SlotData<void(int)>(8, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Slot 'slotStart'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'slotAbort'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotNetworkReplyFinished'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WebTask, qt_meta_tag_ZN5mixxx7network7WebTaskE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject mixxx::network::WebTask::staticMetaObject = { {
    QMetaObject::SuperData::link<NetworkTask::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5mixxx7network7WebTaskE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5mixxx7network7WebTaskE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN5mixxx7network7WebTaskE_t>.metaTypes,
    nullptr
} };

void mixxx::network::WebTask::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WebTask *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->networkError((*reinterpret_cast<std::add_pointer_t<QNetworkReply::NetworkError>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<mixxx::network::WebResponseWithContent>>(_a[3]))); break;
        case 1: _t->slotStart((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 2: _t->slotStart((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->slotStart(); break;
        case 4: _t->slotAbort(); break;
        case 5: _t->slotNetworkReplyFinished(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNetworkReply::NetworkError >(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< mixxx::network::WebResponseWithContent >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WebTask::*)(QNetworkReply::NetworkError , const QString & , const mixxx::network::WebResponseWithContent & )>(_a, &WebTask::networkError, 0))
            return;
    }
}

const QMetaObject *mixxx::network::WebTask::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *mixxx::network::WebTask::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5mixxx7network7WebTaskE_t>.strings))
        return static_cast<void*>(this);
    return NetworkTask::qt_metacast(_clname);
}

int mixxx::network::WebTask::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = NetworkTask::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void mixxx::network::WebTask::networkError(QNetworkReply::NetworkError _t1, const QString & _t2, const mixxx::network::WebResponseWithContent & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
