/****************************************************************************
** Meta object code from reading C++ file 'browsefeature.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/library/browse/browsefeature.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'browsefeature.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13BrowseFeatureE_t {};
} // unnamed namespace

template <> constexpr inline auto BrowseFeature::qt_create_metaobjectdata<qt_meta_tag_ZN13BrowseFeatureE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "BrowseFeature",
        "setRootIndex",
        "",
        "QModelIndex",
        "requestAddDir",
        "scanLibrary",
        "slotAddQuickLink",
        "slotRemoveQuickLink",
        "slotAddToLibrary",
        "slotRefreshDirectoryTree",
        "activate",
        "activateChild",
        "index",
        "onRightClickChild",
        "QPoint",
        "globalPos",
        "onLazyChildExpandation",
        "slotLibraryScanStarted",
        "slotLibraryScanFinished",
        "invalidateRightClickIndex"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'setRootIndex'
        QtMocHelpers::SignalData<void(const QModelIndex &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'requestAddDir'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'scanLibrary'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotAddQuickLink'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotRemoveQuickLink'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotAddToLibrary'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotRefreshDirectoryTree'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'activate'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'activateChild'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 12 },
        }}),
        // Slot 'onRightClickChild'
        QtMocHelpers::SlotData<void(const QPoint &, const QModelIndex &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 }, { 0x80000000 | 3, 12 },
        }}),
        // Slot 'onLazyChildExpandation'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 12 },
        }}),
        // Slot 'slotLibraryScanStarted'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotLibraryScanFinished'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'invalidateRightClickIndex'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<BrowseFeature, qt_meta_tag_ZN13BrowseFeatureE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject BrowseFeature::staticMetaObject = { {
    QMetaObject::SuperData::link<LibraryFeature::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13BrowseFeatureE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13BrowseFeatureE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13BrowseFeatureE_t>.metaTypes,
    nullptr
} };

void BrowseFeature::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<BrowseFeature *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setRootIndex((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 1: _t->requestAddDir((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->scanLibrary(); break;
        case 3: _t->slotAddQuickLink(); break;
        case 4: _t->slotRemoveQuickLink(); break;
        case 5: _t->slotAddToLibrary(); break;
        case 6: _t->slotRefreshDirectoryTree(); break;
        case 7: _t->activate(); break;
        case 8: _t->activateChild((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 9: _t->onRightClickChild((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 10: _t->onLazyChildExpandation((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 11: _t->slotLibraryScanStarted(); break;
        case 12: _t->slotLibraryScanFinished(); break;
        case 13: _t->invalidateRightClickIndex(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (BrowseFeature::*)(const QModelIndex & )>(_a, &BrowseFeature::setRootIndex, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (BrowseFeature::*)(const QString & )>(_a, &BrowseFeature::requestAddDir, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (BrowseFeature::*)()>(_a, &BrowseFeature::scanLibrary, 2))
            return;
    }
}

const QMetaObject *BrowseFeature::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *BrowseFeature::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13BrowseFeatureE_t>.strings))
        return static_cast<void*>(this);
    return LibraryFeature::qt_metacast(_clname);
}

int BrowseFeature::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = LibraryFeature::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void BrowseFeature::setRootIndex(const QModelIndex & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void BrowseFeature::requestAddDir(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void BrowseFeature::scanLibrary()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
