/********************************************************************************
** Form generated from reading UI file 'dlgkeywheel.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLGKEYWHEEL_H
#define UI_DLGKEYWHEEL_H

#include <QtCore/QVariant>
#include <QtSvgWidgets/QSvgWidget>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_DlgKeywheel
{
public:
    QVBoxLayout *verticalLayout_2;
    QGridLayout *gridLayout;
    QSpacerItem *horizontalSpacer_2;
    QSvgWidget *graphic;
    QSpacerItem *horizontalSpacer;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer_3;
    QPushButton *closeButton;
    QSpacerItem *horizontalSpacer_4;

    void setupUi(QDialog *DlgKeywheel)
    {
        if (DlgKeywheel->objectName().isEmpty())
            DlgKeywheel->setObjectName("DlgKeywheel");
        DlgKeywheel->resize(520, 520);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(DlgKeywheel->sizePolicy().hasHeightForWidth());
        DlgKeywheel->setSizePolicy(sizePolicy);
        DlgKeywheel->setWindowFlags(Qt::Tool|Qt::WindowStaysOnTopHint);
        DlgKeywheel->setLayoutDirection(Qt::LeftToRight);
        verticalLayout_2 = new QVBoxLayout(DlgKeywheel);
        verticalLayout_2->setObjectName("verticalLayout_2");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        gridLayout->setSizeConstraint(QLayout::SetDefaultConstraint);
        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer_2, 0, 2, 1, 1);

        graphic = new QSvgWidget(DlgKeywheel);
        graphic->setObjectName("graphic");
        sizePolicy.setHeightForWidth(graphic->sizePolicy().hasHeightForWidth());
        graphic->setSizePolicy(sizePolicy);

        gridLayout->addWidget(graphic, 0, 1, 1, 1);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer, 0, 0, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_3);

        closeButton = new QPushButton(DlgKeywheel);
        closeButton->setObjectName("closeButton");

        horizontalLayout->addWidget(closeButton);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_4);


        gridLayout->addLayout(horizontalLayout, 2, 1, 1, 1);


        verticalLayout_2->addLayout(gridLayout);


        retranslateUi(DlgKeywheel);

        QMetaObject::connectSlotsByName(DlgKeywheel);
    } // setupUi

    void retranslateUi(QDialog *DlgKeywheel)
    {
        DlgKeywheel->setWindowTitle(QCoreApplication::translate("DlgKeywheel", "Keywheel", nullptr));
        closeButton->setText(QCoreApplication::translate("DlgKeywheel", "&Close", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgKeywheel: public Ui_DlgKeywheel {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLGKEYWHEEL_H
