/****************************************************************************
** Meta object code from reading C++ file 'dlgprefsound.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/preferences/dialog/dlgprefsound.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dlgprefsound.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12DlgPrefSoundE_t {};
} // unnamed namespace

template <> constexpr inline auto DlgPrefSound::qt_create_metaobjectdata<qt_meta_tag_ZN12DlgPrefSoundE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DlgPrefSound",
        "loadPaths",
        "",
        "SoundManagerConfig",
        "config",
        "writePaths",
        "SoundManagerConfig*",
        "refreshOutputDevices",
        "QList<SoundDevicePointer>",
        "devices",
        "refreshInputDevices",
        "updatingAPI",
        "updatedAPI",
        "slotUpdate",
        "slotApply",
        "slotResetToDefaults",
        "bufferUnderflow",
        "count",
        "outputLatencyChanged",
        "latency",
        "latencyCompensationSpinboxChanged",
        "value",
        "mainDelaySpinboxChanged",
        "headDelaySpinboxChanged",
        "boothDelaySpinboxChanged",
        "mainMixChanged",
        "mainEnabledChanged",
        "mainOutputModeComboBoxChanged",
        "mainMonoMixdownChanged",
        "micMonitorModeComboBoxChanged",
        "addPath",
        "AudioOutput",
        "output",
        "AudioInput",
        "input",
        "loadSettings",
        "apiChanged",
        "index",
        "updateAPIs",
        "sampleRateChanged",
        "audioBufferChanged",
        "updateAudioBufferSizes",
        "sampleRateIndex",
        "syncBuffersChanged",
        "engineClockChanged",
        "refreshDevices",
        "settingChanged",
        "deviceChanged",
        "deviceChannelsChanged",
        "configuredDeviceNotFound",
        "queryClicked"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'loadPaths'
        QtMocHelpers::SignalData<void(const SoundManagerConfig &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'writePaths'
        QtMocHelpers::SignalData<void(SoundManagerConfig *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 4 },
        }}),
        // Signal 'refreshOutputDevices'
        QtMocHelpers::SignalData<void(const QList<SoundDevicePointer> &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'refreshInputDevices'
        QtMocHelpers::SignalData<void(const QList<SoundDevicePointer> &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'updatingAPI'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'updatedAPI'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotUpdate'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotApply'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotResetToDefaults'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'bufferUnderflow'
        QtMocHelpers::SlotData<void(double)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 17 },
        }}),
        // Slot 'outputLatencyChanged'
        QtMocHelpers::SlotData<void(double)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 19 },
        }}),
        // Slot 'latencyCompensationSpinboxChanged'
        QtMocHelpers::SlotData<void(double)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 21 },
        }}),
        // Slot 'mainDelaySpinboxChanged'
        QtMocHelpers::SlotData<void(double)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 21 },
        }}),
        // Slot 'headDelaySpinboxChanged'
        QtMocHelpers::SlotData<void(double)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 21 },
        }}),
        // Slot 'boothDelaySpinboxChanged'
        QtMocHelpers::SlotData<void(double)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 21 },
        }}),
        // Slot 'mainMixChanged'
        QtMocHelpers::SlotData<void(int)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 21 },
        }}),
        // Slot 'mainEnabledChanged'
        QtMocHelpers::SlotData<void(double)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 21 },
        }}),
        // Slot 'mainOutputModeComboBoxChanged'
        QtMocHelpers::SlotData<void(int)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 21 },
        }}),
        // Slot 'mainMonoMixdownChanged'
        QtMocHelpers::SlotData<void(double)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 21 },
        }}),
        // Slot 'micMonitorModeComboBoxChanged'
        QtMocHelpers::SlotData<void(int)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 21 },
        }}),
        // Slot 'addPath'
        QtMocHelpers::SlotData<void(const AudioOutput &)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 31, 32 },
        }}),
        // Slot 'addPath'
        QtMocHelpers::SlotData<void(const AudioInput &)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 33, 34 },
        }}),
        // Slot 'loadSettings'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'apiChanged'
        QtMocHelpers::SlotData<void(int)>(36, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 37 },
        }}),
        // Slot 'updateAPIs'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'sampleRateChanged'
        QtMocHelpers::SlotData<void(int)>(39, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 37 },
        }}),
        // Slot 'audioBufferChanged'
        QtMocHelpers::SlotData<void(int)>(40, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 37 },
        }}),
        // Slot 'updateAudioBufferSizes'
        QtMocHelpers::SlotData<void(int)>(41, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 42 },
        }}),
        // Slot 'syncBuffersChanged'
        QtMocHelpers::SlotData<void(int)>(43, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 37 },
        }}),
        // Slot 'engineClockChanged'
        QtMocHelpers::SlotData<void(int)>(44, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 37 },
        }}),
        // Slot 'refreshDevices'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'settingChanged'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'deviceChanged'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'deviceChannelsChanged'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'configuredDeviceNotFound'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'queryClicked'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DlgPrefSound, qt_meta_tag_ZN12DlgPrefSoundE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DlgPrefSound::staticMetaObject = { {
    QMetaObject::SuperData::link<DlgPreferencePage::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12DlgPrefSoundE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12DlgPrefSoundE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12DlgPrefSoundE_t>.metaTypes,
    nullptr
} };

void DlgPrefSound::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DlgPrefSound *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->loadPaths((*reinterpret_cast<std::add_pointer_t<SoundManagerConfig>>(_a[1]))); break;
        case 1: _t->writePaths((*reinterpret_cast<std::add_pointer_t<SoundManagerConfig*>>(_a[1]))); break;
        case 2: _t->refreshOutputDevices((*reinterpret_cast<std::add_pointer_t<QList<SoundDevicePointer>>>(_a[1]))); break;
        case 3: _t->refreshInputDevices((*reinterpret_cast<std::add_pointer_t<QList<SoundDevicePointer>>>(_a[1]))); break;
        case 4: _t->updatingAPI(); break;
        case 5: _t->updatedAPI(); break;
        case 6: _t->slotUpdate(); break;
        case 7: _t->slotApply(); break;
        case 8: _t->slotResetToDefaults(); break;
        case 9: _t->bufferUnderflow((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 10: _t->outputLatencyChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 11: _t->latencyCompensationSpinboxChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 12: _t->mainDelaySpinboxChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 13: _t->headDelaySpinboxChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 14: _t->boothDelaySpinboxChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 15: _t->mainMixChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 16: _t->mainEnabledChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 17: _t->mainOutputModeComboBoxChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 18: _t->mainMonoMixdownChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 19: _t->micMonitorModeComboBoxChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 20: _t->addPath((*reinterpret_cast<std::add_pointer_t<AudioOutput>>(_a[1]))); break;
        case 21: _t->addPath((*reinterpret_cast<std::add_pointer_t<AudioInput>>(_a[1]))); break;
        case 22: _t->loadSettings(); break;
        case 23: _t->apiChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 24: _t->updateAPIs(); break;
        case 25: _t->sampleRateChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 26: _t->audioBufferChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 27: _t->updateAudioBufferSizes((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 28: _t->syncBuffersChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 29: _t->engineClockChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 30: _t->refreshDevices(); break;
        case 31: _t->settingChanged(); break;
        case 32: _t->deviceChanged(); break;
        case 33: _t->deviceChannelsChanged(); break;
        case 34: _t->configuredDeviceNotFound(); break;
        case 35: _t->queryClicked(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DlgPrefSound::*)(const SoundManagerConfig & )>(_a, &DlgPrefSound::loadPaths, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgPrefSound::*)(SoundManagerConfig * )>(_a, &DlgPrefSound::writePaths, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgPrefSound::*)(const QList<SoundDevicePointer> & )>(_a, &DlgPrefSound::refreshOutputDevices, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgPrefSound::*)(const QList<SoundDevicePointer> & )>(_a, &DlgPrefSound::refreshInputDevices, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgPrefSound::*)()>(_a, &DlgPrefSound::updatingAPI, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgPrefSound::*)()>(_a, &DlgPrefSound::updatedAPI, 5))
            return;
    }
}

const QMetaObject *DlgPrefSound::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DlgPrefSound::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12DlgPrefSoundE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::DlgPrefSoundDlg"))
        return static_cast< Ui::DlgPrefSoundDlg*>(this);
    return DlgPreferencePage::qt_metacast(_clname);
}

int DlgPrefSound::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DlgPreferencePage::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 36)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 36;
    }
    return _id;
}

// SIGNAL 0
void DlgPrefSound::loadPaths(const SoundManagerConfig & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void DlgPrefSound::writePaths(SoundManagerConfig * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void DlgPrefSound::refreshOutputDevices(const QList<SoundDevicePointer> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void DlgPrefSound::refreshInputDevices(const QList<SoundDevicePointer> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void DlgPrefSound::updatingAPI()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void DlgPrefSound::updatedAPI()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
