/********************************************************************************
** Form generated from reading UI file 'dlgcoverartfullsize.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLGCOVERARTFULLSIZE_H
#define UI_DLGCOVERARTFULLSIZE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_DlgCoverArtFullSize
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *coverArt;

    void setupUi(QDialog *DlgCoverArtFullSize)
    {
        if (DlgCoverArtFullSize->objectName().isEmpty())
            DlgCoverArtFullSize->setObjectName("DlgCoverArtFullSize");
        DlgCoverArtFullSize->resize(300, 300);
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(DlgCoverArtFullSize->sizePolicy().hasHeightForWidth());
        DlgCoverArtFullSize->setSizePolicy(sizePolicy);
        DlgCoverArtFullSize->setMinimumSize(QSize(100, 100));
        verticalLayout = new QVBoxLayout(DlgCoverArtFullSize);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        coverArt = new QLabel(DlgCoverArtFullSize);
        coverArt->setObjectName("coverArt");
        sizePolicy.setHeightForWidth(coverArt->sizePolicy().hasHeightForWidth());
        coverArt->setSizePolicy(sizePolicy);
        coverArt->setAlignment(Qt::AlignCenter);

        verticalLayout->addWidget(coverArt);


        retranslateUi(DlgCoverArtFullSize);

        QMetaObject::connectSlotsByName(DlgCoverArtFullSize);
    } // setupUi

    void retranslateUi(QDialog *DlgCoverArtFullSize)
    {
        coverArt->setText(QString());
        (void)DlgCoverArtFullSize;
    } // retranslateUi

};

namespace Ui {
    class DlgCoverArtFullSize: public Ui_DlgCoverArtFullSize {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLGCOVERARTFULLSIZE_H
