/****************************************************************************
** Meta object code from reading C++ file 'library.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/library/library.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'library.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7LibraryE_t {};
} // unnamed namespace

template <> constexpr inline auto Library::qt_create_metaobjectdata<qt_meta_tag_ZN7LibraryE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Library",
        "showTrackModel",
        "",
        "QAbstractItemModel*",
        "model",
        "restoreState",
        "switchToView",
        "view",
        "loadTrack",
        "TrackPointer",
        "pTrack",
        "loadTrackToPlayer",
        "group",
        "play",
        "restoreSearch",
        "search",
        "text",
        "disableSearch",
        "pasteFromSidebar",
        "enableCoverArtDisplay",
        "selectTrack",
        "TrackId",
        "trackSelected",
        "analyzeTracks",
        "QList<AnalyzerScheduledTrack>",
        "tracks",
        "exportLibrary",
        "exportCrate",
        "CrateId",
        "crateId",
        "saveModelState",
        "restoreModelState",
        "setTrackTableFont",
        "QFont",
        "font",
        "setTrackTableRowHeight",
        "rowHeight",
        "setSelectedClick",
        "enable",
        "onTrackAnalyzerProgress",
        "trackId",
        "AnalyzerProgress",
        "analyzerProgress",
        "slotShowTrackModel",
        "slotSwitchToView",
        "slotLoadTrack",
        "slotLoadTrackToPlayer",
        "slotLoadLocationToPlayer",
        "location",
        "slotRefreshLibraryModels",
        "slotCreatePlaylist",
        "slotCreateCrate",
        "onSkinLoadFinished",
        "slotSaveCurrentViewState",
        "slotRestoreCurrentViewState",
        "onPlayerManagerTrackAnalyzerProgress",
        "onPlayerManagerTrackAnalyzerIdle"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'showTrackModel'
        QtMocHelpers::SignalData<void(QAbstractItemModel *, bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Bool, 5 },
        }}),
        // Signal 'showTrackModel'
        QtMocHelpers::SignalData<void(QAbstractItemModel *)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'switchToView'
        QtMocHelpers::SignalData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Signal 'loadTrack'
        QtMocHelpers::SignalData<void(TrackPointer)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'loadTrackToPlayer'
        QtMocHelpers::SignalData<void(TrackPointer, const QString &, bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::QString, 12 }, { QMetaType::Bool, 13 },
        }}),
        // Signal 'loadTrackToPlayer'
        QtMocHelpers::SignalData<void(TrackPointer, const QString &)>(11, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::QString, 12 },
        }}),
        // Signal 'restoreSearch'
        QtMocHelpers::SignalData<void(const QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'search'
        QtMocHelpers::SignalData<void(const QString &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 },
        }}),
        // Signal 'disableSearch'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pasteFromSidebar'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'enableCoverArtDisplay'
        QtMocHelpers::SignalData<void(bool)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'selectTrack'
        QtMocHelpers::SignalData<void(const TrackId &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 2 },
        }}),
        // Signal 'trackSelected'
        QtMocHelpers::SignalData<void(TrackPointer)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'analyzeTracks'
        QtMocHelpers::SignalData<void(const QList<AnalyzerScheduledTrack> &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Signal 'exportLibrary'
        QtMocHelpers::SignalData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'exportCrate'
        QtMocHelpers::SignalData<void(CrateId)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 28, 29 },
        }}),
        // Signal 'saveModelState'
        QtMocHelpers::SignalData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'restoreModelState'
        QtMocHelpers::SignalData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'setTrackTableFont'
        QtMocHelpers::SignalData<void(const QFont &)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 33, 34 },
        }}),
        // Signal 'setTrackTableRowHeight'
        QtMocHelpers::SignalData<void(int)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 36 },
        }}),
        // Signal 'setSelectedClick'
        QtMocHelpers::SignalData<void(bool)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Signal 'onTrackAnalyzerProgress'
        QtMocHelpers::SignalData<void(TrackId, AnalyzerProgress)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 40 }, { 0x80000000 | 41, 42 },
        }}),
        // Slot 'slotShowTrackModel'
        QtMocHelpers::SlotData<void(QAbstractItemModel *)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotSwitchToView'
        QtMocHelpers::SlotData<void(const QString &)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'slotLoadTrack'
        QtMocHelpers::SlotData<void(TrackPointer)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'slotLoadTrackToPlayer'
        QtMocHelpers::SlotData<void(TrackPointer, const QString &, bool)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::QString, 12 }, { QMetaType::Bool, 13 },
        }}),
        // Slot 'slotLoadLocationToPlayer'
        QtMocHelpers::SlotData<void(const QString &, const QString &, bool)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 48 }, { QMetaType::QString, 12 }, { QMetaType::Bool, 13 },
        }}),
        // Slot 'slotRefreshLibraryModels'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotCreatePlaylist'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotCreateCrate'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onSkinLoadFinished'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSaveCurrentViewState'
        QtMocHelpers::SlotData<void() const>(53, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotRestoreCurrentViewState'
        QtMocHelpers::SlotData<void() const>(54, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onPlayerManagerTrackAnalyzerProgress'
        QtMocHelpers::SlotData<void(TrackId, AnalyzerProgress)>(55, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 21, 40 }, { 0x80000000 | 41, 42 },
        }}),
        // Slot 'onPlayerManagerTrackAnalyzerIdle'
        QtMocHelpers::SlotData<void()>(56, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Library, qt_meta_tag_ZN7LibraryE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Library::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7LibraryE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7LibraryE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7LibraryE_t>.metaTypes,
    nullptr
} };

void Library::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Library *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->showTrackModel((*reinterpret_cast<std::add_pointer_t<QAbstractItemModel*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 1: _t->showTrackModel((*reinterpret_cast<std::add_pointer_t<QAbstractItemModel*>>(_a[1]))); break;
        case 2: _t->switchToView((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->loadTrack((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1]))); break;
        case 4: _t->loadTrackToPlayer((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 5: _t->loadTrackToPlayer((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 6: _t->restoreSearch((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->search((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->disableSearch(); break;
        case 9: _t->pasteFromSidebar(); break;
        case 10: _t->enableCoverArtDisplay((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->selectTrack((*reinterpret_cast<std::add_pointer_t<TrackId>>(_a[1]))); break;
        case 12: _t->trackSelected((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1]))); break;
        case 13: _t->analyzeTracks((*reinterpret_cast<std::add_pointer_t<QList<AnalyzerScheduledTrack>>>(_a[1]))); break;
        case 14: _t->exportLibrary(); break;
        case 15: _t->exportCrate((*reinterpret_cast<std::add_pointer_t<CrateId>>(_a[1]))); break;
        case 16: _t->saveModelState(); break;
        case 17: _t->restoreModelState(); break;
        case 18: _t->setTrackTableFont((*reinterpret_cast<std::add_pointer_t<QFont>>(_a[1]))); break;
        case 19: _t->setTrackTableRowHeight((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 20: _t->setSelectedClick((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 21: _t->onTrackAnalyzerProgress((*reinterpret_cast<std::add_pointer_t<TrackId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<AnalyzerProgress>>(_a[2]))); break;
        case 22: _t->slotShowTrackModel((*reinterpret_cast<std::add_pointer_t<QAbstractItemModel*>>(_a[1]))); break;
        case 23: _t->slotSwitchToView((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 24: _t->slotLoadTrack((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1]))); break;
        case 25: _t->slotLoadTrackToPlayer((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 26: _t->slotLoadLocationToPlayer((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 27: _t->slotRefreshLibraryModels(); break;
        case 28: _t->slotCreatePlaylist(); break;
        case 29: _t->slotCreateCrate(); break;
        case 30: _t->onSkinLoadFinished(); break;
        case 31: _t->slotSaveCurrentViewState(); break;
        case 32: _t->slotRestoreCurrentViewState(); break;
        case 33: _t->onPlayerManagerTrackAnalyzerProgress((*reinterpret_cast<std::add_pointer_t<TrackId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<AnalyzerProgress>>(_a[2]))); break;
        case 34: _t->onPlayerManagerTrackAnalyzerIdle(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackId >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CrateId >(); break;
            }
            break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AnalyzerProgress >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackId >(); break;
            }
            break;
        case 24:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 25:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 33:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AnalyzerProgress >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackId >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Library::*)(QAbstractItemModel * , bool )>(_a, &Library::showTrackModel, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Library::*)(const QString & )>(_a, &Library::switchToView, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Library::*)(TrackPointer )>(_a, &Library::loadTrack, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Library::*)(TrackPointer , const QString & , bool )>(_a, &Library::loadTrackToPlayer, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Library::*)(const QString & )>(_a, &Library::restoreSearch, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Library::*)(const QString & )>(_a, &Library::search, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Library::*)()>(_a, &Library::disableSearch, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (Library::*)()>(_a, &Library::pasteFromSidebar, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (Library::*)(bool )>(_a, &Library::enableCoverArtDisplay, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (Library::*)(const TrackId & )>(_a, &Library::selectTrack, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (Library::*)(TrackPointer )>(_a, &Library::trackSelected, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (Library::*)(const QList<AnalyzerScheduledTrack> & )>(_a, &Library::analyzeTracks, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (Library::*)()>(_a, &Library::exportLibrary, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (Library::*)(CrateId )>(_a, &Library::exportCrate, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (Library::*)()>(_a, &Library::saveModelState, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (Library::*)()>(_a, &Library::restoreModelState, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (Library::*)(const QFont & )>(_a, &Library::setTrackTableFont, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (Library::*)(int )>(_a, &Library::setTrackTableRowHeight, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (Library::*)(bool )>(_a, &Library::setSelectedClick, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (Library::*)(TrackId , AnalyzerProgress )>(_a, &Library::onTrackAnalyzerProgress, 21))
            return;
    }
}

const QMetaObject *Library::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Library::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7LibraryE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Library::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 35)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 35;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 35)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 35;
    }
    return _id;
}

// SIGNAL 0
void Library::showTrackModel(QAbstractItemModel * _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 2
void Library::switchToView(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Library::loadTrack(TrackPointer _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Library::loadTrackToPlayer(TrackPointer _t1, const QString & _t2, bool _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2, _t3);
}

// SIGNAL 6
void Library::restoreSearch(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void Library::search(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void Library::disableSearch()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void Library::pasteFromSidebar()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void Library::enableCoverArtDisplay(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void Library::selectTrack(const TrackId & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void Library::trackSelected(TrackPointer _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void Library::analyzeTracks(const QList<AnalyzerScheduledTrack> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void Library::exportLibrary()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void Library::exportCrate(CrateId _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void Library::saveModelState()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void Library::restoreModelState()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void Library::setTrackTableFont(const QFont & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void Library::setTrackTableRowHeight(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}

// SIGNAL 20
void Library::setSelectedClick(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void Library::onTrackAnalyzerProgress(TrackId _t1, AnalyzerProgress _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1, _t2);
}
QT_WARNING_POP
