/****************************************************************************
** Meta object code from reading C++ file 'dlgprefsounditem.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/preferences/dialog/dlgprefsounditem.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dlgprefsounditem.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16DlgPrefSoundItemE_t {};
} // unnamed namespace

template <> constexpr inline auto DlgPrefSoundItem::qt_create_metaobjectdata<qt_meta_tag_ZN16DlgPrefSoundItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DlgPrefSoundItem",
        "selectedDeviceChanged",
        "",
        "selectedChannelsChanged",
        "configuredDeviceNotFound",
        "refreshDevices",
        "QList<SoundDevicePointer>",
        "devices",
        "deviceChanged",
        "index",
        "channelChanged",
        "loadPath",
        "SoundManagerConfig",
        "config",
        "writePath",
        "SoundManagerConfig*",
        "save",
        "reload"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'selectedDeviceChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'selectedChannelsChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'configuredDeviceNotFound'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'refreshDevices'
        QtMocHelpers::SlotData<void(const QList<SoundDevicePointer> &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'deviceChanged'
        QtMocHelpers::SlotData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Slot 'channelChanged'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'loadPath'
        QtMocHelpers::SlotData<void(const SoundManagerConfig &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'writePath'
        QtMocHelpers::SlotData<void(SoundManagerConfig *) const>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 13 },
        }}),
        // Slot 'save'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'reload'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DlgPrefSoundItem, qt_meta_tag_ZN16DlgPrefSoundItemE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DlgPrefSoundItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16DlgPrefSoundItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16DlgPrefSoundItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16DlgPrefSoundItemE_t>.metaTypes,
    nullptr
} };

void DlgPrefSoundItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DlgPrefSoundItem *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->selectedDeviceChanged(); break;
        case 1: _t->selectedChannelsChanged(); break;
        case 2: _t->configuredDeviceNotFound(); break;
        case 3: _t->refreshDevices((*reinterpret_cast<std::add_pointer_t<QList<SoundDevicePointer>>>(_a[1]))); break;
        case 4: _t->deviceChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->channelChanged(); break;
        case 6: _t->loadPath((*reinterpret_cast<std::add_pointer_t<SoundManagerConfig>>(_a[1]))); break;
        case 7: _t->writePath((*reinterpret_cast<std::add_pointer_t<SoundManagerConfig*>>(_a[1]))); break;
        case 8: _t->save(); break;
        case 9: _t->reload(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DlgPrefSoundItem::*)()>(_a, &DlgPrefSoundItem::selectedDeviceChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgPrefSoundItem::*)()>(_a, &DlgPrefSoundItem::selectedChannelsChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgPrefSoundItem::*)()>(_a, &DlgPrefSoundItem::configuredDeviceNotFound, 2))
            return;
    }
}

const QMetaObject *DlgPrefSoundItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DlgPrefSoundItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16DlgPrefSoundItemE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::DlgPrefSoundItem"))
        return static_cast< Ui::DlgPrefSoundItem*>(this);
    return QWidget::qt_metacast(_clname);
}

int DlgPrefSoundItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void DlgPrefSoundItem::selectedDeviceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void DlgPrefSoundItem::selectedChannelsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void DlgPrefSoundItem::configuredDeviceNotFound()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
