/****************************************************************************
** Meta object code from reading C++ file 'controllerscriptinterfacelegacy.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/controllers/scripting/legacy/controllerscriptinterfacelegacy.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'controllerscriptinterfacelegacy.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN31ControllerScriptInterfaceLegacyE_t {};
} // unnamed namespace

template <> constexpr inline auto ControllerScriptInterfaceLegacy::qt_create_metaobjectdata<qt_meta_tag_ZN31ControllerScriptInterfaceLegacyE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ControllerScriptInterfaceLegacy",
        "getSetting",
        "QJSValue",
        "",
        "name",
        "getValue",
        "group",
        "setValue",
        "newValue",
        "getParameter",
        "setParameter",
        "getParameterForValue",
        "value",
        "reset",
        "getDefaultValue",
        "getDefaultParameter",
        "makeConnection",
        "callback",
        "makeUnbufferedConnection",
        "connectControl",
        "passedCallback",
        "disconnect",
        "trigger",
        "log",
        "message",
        "beginTimer",
        "interval",
        "scriptCode",
        "oneShot",
        "stopTimer",
        "timerId",
        "scratchEnable",
        "deck",
        "intervalsPerRev",
        "rpm",
        "alpha",
        "beta",
        "ramp",
        "scratchTick",
        "scratchDisable",
        "isScratching",
        "softTakeover",
        "set",
        "softTakeoverIgnoreNextValue",
        "brake",
        "activate",
        "factor",
        "rate",
        "spinback",
        "softStart"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'getSetting'
        QtMocHelpers::MethodData<QJSValue(const QString &)>(1, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { QMetaType::QString, 4 },
        }}),
        // Method 'getValue'
        QtMocHelpers::MethodData<double(const QString &, const QString &)>(5, 3, QMC::AccessPublic, QMetaType::Double, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 4 },
        }}),
        // Method 'setValue'
        QtMocHelpers::MethodData<void(const QString &, const QString &, double)>(7, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 4 }, { QMetaType::Double, 8 },
        }}),
        // Method 'getParameter'
        QtMocHelpers::MethodData<double(const QString &, const QString &)>(9, 3, QMC::AccessPublic, QMetaType::Double, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 4 },
        }}),
        // Method 'setParameter'
        QtMocHelpers::MethodData<void(const QString &, const QString &, double)>(10, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 4 }, { QMetaType::Double, 8 },
        }}),
        // Method 'getParameterForValue'
        QtMocHelpers::MethodData<double(const QString &, const QString &, double)>(11, 3, QMC::AccessPublic, QMetaType::Double, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 4 }, { QMetaType::Double, 12 },
        }}),
        // Method 'reset'
        QtMocHelpers::MethodData<void(const QString &, const QString &)>(13, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 4 },
        }}),
        // Method 'getDefaultValue'
        QtMocHelpers::MethodData<double(const QString &, const QString &)>(14, 3, QMC::AccessPublic, QMetaType::Double, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 4 },
        }}),
        // Method 'getDefaultParameter'
        QtMocHelpers::MethodData<double(const QString &, const QString &)>(15, 3, QMC::AccessPublic, QMetaType::Double, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 4 },
        }}),
        // Method 'makeConnection'
        QtMocHelpers::MethodData<QJSValue(const QString &, const QString &, const QJSValue &)>(16, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 4 }, { 0x80000000 | 2, 17 },
        }}),
        // Method 'makeUnbufferedConnection'
        QtMocHelpers::MethodData<QJSValue(const QString &, const QString &, const QJSValue &)>(18, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 4 }, { 0x80000000 | 2, 17 },
        }}),
        // Method 'connectControl'
        QtMocHelpers::MethodData<QJSValue(const QString &, const QString &, const QJSValue &, bool)>(19, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 4 }, { 0x80000000 | 2, 20 }, { QMetaType::Bool, 21 },
        }}),
        // Method 'connectControl'
        QtMocHelpers::MethodData<QJSValue(const QString &, const QString &, const QJSValue &)>(19, 3, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 2, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 4 }, { 0x80000000 | 2, 20 },
        }}),
        // Method 'trigger'
        QtMocHelpers::MethodData<void(const QString &, const QString &)>(22, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 4 },
        }}),
        // Method 'log'
        QtMocHelpers::MethodData<void(const QString &)>(23, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 24 },
        }}),
        // Method 'beginTimer'
        QtMocHelpers::MethodData<int(int, QJSValue, bool)>(25, 3, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::Int, 26 }, { 0x80000000 | 2, 27 }, { QMetaType::Bool, 28 },
        }}),
        // Method 'beginTimer'
        QtMocHelpers::MethodData<int(int, QJSValue)>(25, 3, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::Int, 26 }, { 0x80000000 | 2, 27 },
        }}),
        // Method 'stopTimer'
        QtMocHelpers::MethodData<void(int)>(29, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 30 },
        }}),
        // Method 'scratchEnable'
        QtMocHelpers::MethodData<void(int, int, double, double, double, bool)>(31, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 32 }, { QMetaType::Int, 33 }, { QMetaType::Double, 34 }, { QMetaType::Double, 35 },
            { QMetaType::Double, 36 }, { QMetaType::Bool, 37 },
        }}),
        // Method 'scratchEnable'
        QtMocHelpers::MethodData<void(int, int, double, double, double)>(31, 3, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 32 }, { QMetaType::Int, 33 }, { QMetaType::Double, 34 }, { QMetaType::Double, 35 },
            { QMetaType::Double, 36 },
        }}),
        // Method 'scratchTick'
        QtMocHelpers::MethodData<void(int, int)>(38, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 32 }, { QMetaType::Int, 26 },
        }}),
        // Method 'scratchDisable'
        QtMocHelpers::MethodData<void(int, bool)>(39, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 32 }, { QMetaType::Bool, 37 },
        }}),
        // Method 'scratchDisable'
        QtMocHelpers::MethodData<void(int)>(39, 3, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 32 },
        }}),
        // Method 'isScratching'
        QtMocHelpers::MethodData<bool(int)>(40, 3, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Int, 32 },
        }}),
        // Method 'softTakeover'
        QtMocHelpers::MethodData<void(const QString &, const QString &, bool)>(41, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 4 }, { QMetaType::Bool, 42 },
        }}),
        // Method 'softTakeoverIgnoreNextValue'
        QtMocHelpers::MethodData<void(const QString &, const QString &)>(43, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 4 },
        }}),
        // Method 'brake'
        QtMocHelpers::MethodData<void(const int, bool, double, const double)>(44, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 32 }, { QMetaType::Bool, 45 }, { QMetaType::Double, 46 }, { QMetaType::Double, 47 },
        }}),
        // Method 'brake'
        QtMocHelpers::MethodData<void(const int, bool, double)>(44, 3, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 32 }, { QMetaType::Bool, 45 }, { QMetaType::Double, 46 },
        }}),
        // Method 'brake'
        QtMocHelpers::MethodData<void(const int, bool)>(44, 3, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 32 }, { QMetaType::Bool, 45 },
        }}),
        // Method 'spinback'
        QtMocHelpers::MethodData<void(const int, bool, double, const double)>(48, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 32 }, { QMetaType::Bool, 45 }, { QMetaType::Double, 46 }, { QMetaType::Double, 47 },
        }}),
        // Method 'spinback'
        QtMocHelpers::MethodData<void(const int, bool, double)>(48, 3, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 32 }, { QMetaType::Bool, 45 }, { QMetaType::Double, 46 },
        }}),
        // Method 'spinback'
        QtMocHelpers::MethodData<void(const int, bool)>(48, 3, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 32 }, { QMetaType::Bool, 45 },
        }}),
        // Method 'softStart'
        QtMocHelpers::MethodData<void(const int, bool, double)>(49, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 32 }, { QMetaType::Bool, 45 }, { QMetaType::Double, 46 },
        }}),
        // Method 'softStart'
        QtMocHelpers::MethodData<void(const int, bool)>(49, 3, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 32 }, { QMetaType::Bool, 45 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ControllerScriptInterfaceLegacy, qt_meta_tag_ZN31ControllerScriptInterfaceLegacyE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ControllerScriptInterfaceLegacy::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31ControllerScriptInterfaceLegacyE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31ControllerScriptInterfaceLegacyE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN31ControllerScriptInterfaceLegacyE_t>.metaTypes,
    nullptr
} };

void ControllerScriptInterfaceLegacy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ControllerScriptInterfaceLegacy *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QJSValue _r = _t->getSetting((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 1: { double _r = _t->getValue((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 2: _t->setValue((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3]))); break;
        case 3: { double _r = _t->getParameter((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 4: _t->setParameter((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3]))); break;
        case 5: { double _r = _t->getParameterForValue((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])));
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 6: _t->reset((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 7: { double _r = _t->getDefaultValue((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 8: { double _r = _t->getDefaultParameter((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<double*>(_a[0]) = std::move(_r); }  break;
        case 9: { QJSValue _r = _t->makeConnection((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 10: { QJSValue _r = _t->makeUnbufferedConnection((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 11: { QJSValue _r = _t->connectControl((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])));
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 12: { QJSValue _r = _t->connectControl((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 13: _t->trigger((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 14: _t->log((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 15: { int _r = _t->beginTimer((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 16: { int _r = _t->beginTimer((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[2])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 17: _t->stopTimer((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 18: _t->scratchEnable((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6]))); break;
        case 19: _t->scratchEnable((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[5]))); break;
        case 20: _t->scratchTick((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 21: _t->scratchDisable((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 22: _t->scratchDisable((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 23: { bool _r = _t->isScratching((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 24: _t->softTakeover((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 25: _t->softTakeoverIgnoreNextValue((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 26: _t->brake((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4]))); break;
        case 27: _t->brake((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3]))); break;
        case 28: _t->brake((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 29: _t->spinback((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[4]))); break;
        case 30: _t->spinback((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3]))); break;
        case 31: _t->spinback((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 32: _t->softStart((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3]))); break;
        case 33: _t->softStart((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QJSValue >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QJSValue >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QJSValue >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QJSValue >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QJSValue >(); break;
            }
            break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QJSValue >(); break;
            }
            break;
        }
    }
}

const QMetaObject *ControllerScriptInterfaceLegacy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ControllerScriptInterfaceLegacy::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31ControllerScriptInterfaceLegacyE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ControllerScriptInterfaceLegacy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 34)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 34;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 34)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 34;
    }
    return _id;
}
QT_WARNING_POP
