/****************************************************************************
** Meta object code from reading C++ file 'libraryfeature.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/library/libraryfeature.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'libraryfeature.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14LibraryFeatureE_t {};
} // unnamed namespace

template <> constexpr inline auto LibraryFeature::qt_create_metaobjectdata<qt_meta_tag_ZN14LibraryFeatureE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LibraryFeature",
        "showTrackModel",
        "",
        "QAbstractItemModel*",
        "model",
        "restoreState",
        "switchToView",
        "view",
        "loadTrack",
        "TrackPointer",
        "pTrack",
        "loadTrackToPlayer",
        "group",
        "play",
        "saveModelState",
        "restoreModelState",
        "restoreSearch",
        "disableSearch",
        "pasteFromSidebar",
        "featureIsLoading",
        "LibraryFeature*",
        "selectFeature",
        "featureLoadingFinished",
        "s",
        "featureSelect",
        "pFeature",
        "QModelIndex",
        "index",
        "enableCoverArtDisplay",
        "trackSelected",
        "activate",
        "activateChild",
        "onRightClick",
        "QPoint",
        "globalPos",
        "onRightClickChild",
        "renameItem",
        "deleteItem",
        "onLazyChildExpandation"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'showTrackModel'
        QtMocHelpers::SignalData<void(QAbstractItemModel *, bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Bool, 5 },
        }}),
        // Signal 'showTrackModel'
        QtMocHelpers::SignalData<void(QAbstractItemModel *)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'switchToView'
        QtMocHelpers::SignalData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Signal 'loadTrack'
        QtMocHelpers::SignalData<void(TrackPointer)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'loadTrackToPlayer'
        QtMocHelpers::SignalData<void(TrackPointer, const QString &, bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::QString, 12 }, { QMetaType::Bool, 13 },
        }}),
        // Signal 'loadTrackToPlayer'
        QtMocHelpers::SignalData<void(TrackPointer, const QString &)>(11, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::QString, 12 },
        }}),
        // Signal 'saveModelState'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'restoreModelState'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'restoreSearch'
        QtMocHelpers::SignalData<void(const QString &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'disableSearch'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pasteFromSidebar'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'featureIsLoading'
        QtMocHelpers::SignalData<void(LibraryFeature *, bool)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 2 }, { QMetaType::Bool, 21 },
        }}),
        // Signal 'featureLoadingFinished'
        QtMocHelpers::SignalData<void(LibraryFeature *)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 23 },
        }}),
        // Signal 'featureSelect'
        QtMocHelpers::SignalData<void(LibraryFeature *, const QModelIndex &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 25 }, { 0x80000000 | 26, 27 },
        }}),
        // Signal 'enableCoverArtDisplay'
        QtMocHelpers::SignalData<void(bool)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'trackSelected'
        QtMocHelpers::SignalData<void(TrackPointer)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'activate'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'activateChild'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Slot 'onRightClick'
        QtMocHelpers::SlotData<void(const QPoint &)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 33, 34 },
        }}),
        // Slot 'onRightClickChild'
        QtMocHelpers::SlotData<void(const QPoint &, const QModelIndex &)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 33, 34 }, { 0x80000000 | 26, 27 },
        }}),
        // Slot 'renameItem'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Slot 'deleteItem'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Slot 'onLazyChildExpandation'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LibraryFeature, qt_meta_tag_ZN14LibraryFeatureE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject LibraryFeature::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14LibraryFeatureE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14LibraryFeatureE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14LibraryFeatureE_t>.metaTypes,
    nullptr
} };

void LibraryFeature::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LibraryFeature *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->showTrackModel((*reinterpret_cast<std::add_pointer_t<QAbstractItemModel*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 1: _t->showTrackModel((*reinterpret_cast<std::add_pointer_t<QAbstractItemModel*>>(_a[1]))); break;
        case 2: _t->switchToView((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->loadTrack((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1]))); break;
        case 4: _t->loadTrackToPlayer((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 5: _t->loadTrackToPlayer((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 6: _t->saveModelState(); break;
        case 7: _t->restoreModelState(); break;
        case 8: _t->restoreSearch((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->disableSearch(); break;
        case 10: _t->pasteFromSidebar(); break;
        case 11: _t->featureIsLoading((*reinterpret_cast<std::add_pointer_t<LibraryFeature*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 12: _t->featureLoadingFinished((*reinterpret_cast<std::add_pointer_t<LibraryFeature*>>(_a[1]))); break;
        case 13: _t->featureSelect((*reinterpret_cast<std::add_pointer_t<LibraryFeature*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 14: _t->enableCoverArtDisplay((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: _t->trackSelected((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1]))); break;
        case 16: _t->activate(); break;
        case 17: _t->activateChild((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 18: _t->onRightClick((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 19: _t->onRightClickChild((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 20: _t->renameItem((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 21: _t->deleteItem((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 22: _t->onLazyChildExpandation((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractItemModel* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractItemModel* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< LibraryFeature* >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< LibraryFeature* >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< LibraryFeature* >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (LibraryFeature::*)(QAbstractItemModel * , bool )>(_a, &LibraryFeature::showTrackModel, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (LibraryFeature::*)(const QString & )>(_a, &LibraryFeature::switchToView, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (LibraryFeature::*)(TrackPointer )>(_a, &LibraryFeature::loadTrack, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (LibraryFeature::*)(TrackPointer , const QString & , bool )>(_a, &LibraryFeature::loadTrackToPlayer, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (LibraryFeature::*)()>(_a, &LibraryFeature::saveModelState, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (LibraryFeature::*)()>(_a, &LibraryFeature::restoreModelState, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (LibraryFeature::*)(const QString & )>(_a, &LibraryFeature::restoreSearch, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (LibraryFeature::*)()>(_a, &LibraryFeature::disableSearch, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (LibraryFeature::*)()>(_a, &LibraryFeature::pasteFromSidebar, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (LibraryFeature::*)(LibraryFeature * , bool )>(_a, &LibraryFeature::featureIsLoading, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (LibraryFeature::*)(LibraryFeature * )>(_a, &LibraryFeature::featureLoadingFinished, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (LibraryFeature::*)(LibraryFeature * , const QModelIndex & )>(_a, &LibraryFeature::featureSelect, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (LibraryFeature::*)(bool )>(_a, &LibraryFeature::enableCoverArtDisplay, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (LibraryFeature::*)(TrackPointer )>(_a, &LibraryFeature::trackSelected, 15))
            return;
    }
}

const QMetaObject *LibraryFeature::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LibraryFeature::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14LibraryFeatureE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int LibraryFeature::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    return _id;
}

// SIGNAL 0
void LibraryFeature::showTrackModel(QAbstractItemModel * _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 2
void LibraryFeature::switchToView(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void LibraryFeature::loadTrack(TrackPointer _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void LibraryFeature::loadTrackToPlayer(TrackPointer _t1, const QString & _t2, bool _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2, _t3);
}

// SIGNAL 6
void LibraryFeature::saveModelState()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void LibraryFeature::restoreModelState()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void LibraryFeature::restoreSearch(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void LibraryFeature::disableSearch()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void LibraryFeature::pasteFromSidebar()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void LibraryFeature::featureIsLoading(LibraryFeature * _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1, _t2);
}

// SIGNAL 12
void LibraryFeature::featureLoadingFinished(LibraryFeature * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void LibraryFeature::featureSelect(LibraryFeature * _t1, const QModelIndex & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1, _t2);
}

// SIGNAL 14
void LibraryFeature::enableCoverArtDisplay(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void LibraryFeature::trackSelected(TrackPointer _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}
QT_WARNING_POP
