/****************************************************************************
** Meta object code from reading C++ file 'wlibrary.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/widget/wlibrary.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wlibrary.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8WLibraryE_t {};
} // unnamed namespace

template <> constexpr inline auto WLibrary::qt_create_metaobjectdata<qt_meta_tag_ZN8WLibraryE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "WLibrary",
        "setLibraryFocus",
        "FocusWidget",
        "",
        "newFocus",
        "switchToView",
        "name",
        "slotSelectTrackInActiveTrackView",
        "TrackId",
        "trackId",
        "pasteFromSidebar",
        "search"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'setLibraryFocus'
        QtMocHelpers::SignalData<FocusWidget(FocusWidget)>(1, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { 0x80000000 | 2, 4 },
        }}),
        // Slot 'switchToView'
        QtMocHelpers::SlotData<void(const QString &)>(5, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Slot 'slotSelectTrackInActiveTrackView'
        QtMocHelpers::SlotData<void(const TrackId &)>(7, 3, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'pasteFromSidebar'
        QtMocHelpers::SlotData<void()>(10, 3, QMC::AccessPublic, QMetaType::Void),
        // Slot 'search'
        QtMocHelpers::SlotData<void(const QString &)>(11, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WLibrary, qt_meta_tag_ZN8WLibraryE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject WLibrary::staticMetaObject = { {
    QMetaObject::SuperData::link<QStackedWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8WLibraryE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8WLibraryE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8WLibraryE_t>.metaTypes,
    nullptr
} };

void WLibrary::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WLibrary *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { FocusWidget _r = _t->setLibraryFocus((*reinterpret_cast<std::add_pointer_t<FocusWidget>>(_a[1])));
            if (_a[0]) *reinterpret_cast<FocusWidget*>(_a[0]) = std::move(_r); }  break;
        case 1: _t->switchToView((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->slotSelectTrackInActiveTrackView((*reinterpret_cast<std::add_pointer_t<TrackId>>(_a[1]))); break;
        case 3: _t->pasteFromSidebar(); break;
        case 4: _t->search((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackId >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<FocusWidget (WLibrary::*)(FocusWidget )>(_a, &WLibrary::setLibraryFocus, 0))
            return;
    }
}

const QMetaObject *WLibrary::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *WLibrary::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8WLibraryE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "WBaseWidget"))
        return static_cast< WBaseWidget*>(this);
    return QStackedWidget::qt_metacast(_clname);
}

int WLibrary::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QStackedWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
FocusWidget WLibrary::setLibraryFocus(FocusWidget _t1)
{
    FocusWidget _t0{};
    QMetaObject::activate<FocusWidget>(this, &staticMetaObject, 0, std::addressof(_t0), _t1);
    return _t0;
}
QT_WARNING_POP
