/********************************************************************************
** Form generated from reading UI file 'dlgprefautodjdlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLGPREFAUTODJDLG_H
#define UI_DLGPREFAUTODJDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTimeEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DlgPrefAutoDJDlg
{
public:
    QVBoxLayout *AutoDJGridLayout;
    QGroupBox *RequeueOptions;
    QGridLayout *RequeGridLayout;
    QLabel *MinimumAvailableLabel;
    QSpinBox *MinimumAvailableSpinBox;
    QLabel *RequeueIgnoreLabel;
    QCheckBox *RequeueIgnoreCheckBox;
    QLabel *RequeueIgnoreTimeLabel;
    QTimeEdit *RequeueIgnoreTimeEdit;
    QSpacerItem *horizontalSpacerRequeue;
    QGroupBox *AddRandomOptions;
    QGridLayout *AddRandomGridLayout;
    QLabel *RandomQueueLabel;
    QCheckBox *RandomQueueCheckBox;
    QLabel *RandomQueueMinimumLabel;
    QSpinBox *RandomQueueMinimumSpinBox;
    QSpacerItem *horizontalSpacerRandom;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *DlgPrefAutoDJDlg)
    {
        if (DlgPrefAutoDJDlg->objectName().isEmpty())
            DlgPrefAutoDJDlg->setObjectName("DlgPrefAutoDJDlg");
        DlgPrefAutoDJDlg->resize(658, 239);
        AutoDJGridLayout = new QVBoxLayout(DlgPrefAutoDJDlg);
        AutoDJGridLayout->setObjectName("AutoDJGridLayout");
        RequeueOptions = new QGroupBox(DlgPrefAutoDJDlg);
        RequeueOptions->setObjectName("RequeueOptions");
        RequeueOptions->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        RequeGridLayout = new QGridLayout(RequeueOptions);
        RequeGridLayout->setObjectName("RequeGridLayout");
        MinimumAvailableLabel = new QLabel(RequeueOptions);
        MinimumAvailableLabel->setObjectName("MinimumAvailableLabel");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(MinimumAvailableLabel->sizePolicy().hasHeightForWidth());
        MinimumAvailableLabel->setSizePolicy(sizePolicy);

        RequeGridLayout->addWidget(MinimumAvailableLabel, 0, 0, 1, 1);

        MinimumAvailableSpinBox = new QSpinBox(RequeueOptions);
        MinimumAvailableSpinBox->setObjectName("MinimumAvailableSpinBox");
        MinimumAvailableSpinBox->setMaximum(100);
        MinimumAvailableSpinBox->setValue(20);
        MinimumAvailableSpinBox->setMinimumSize(QSize(60, 0));
        MinimumAvailableSpinBox->setMaximumSize(QSize(80, 16777215));

        RequeGridLayout->addWidget(MinimumAvailableSpinBox, 0, 1, 1, 1);

        RequeueIgnoreLabel = new QLabel(RequeueOptions);
        RequeueIgnoreLabel->setObjectName("RequeueIgnoreLabel");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(RequeueIgnoreLabel->sizePolicy().hasHeightForWidth());
        RequeueIgnoreLabel->setSizePolicy(sizePolicy1);

        RequeGridLayout->addWidget(RequeueIgnoreLabel, 1, 0, 1, 1);

        RequeueIgnoreCheckBox = new QCheckBox(RequeueOptions);
        RequeueIgnoreCheckBox->setObjectName("RequeueIgnoreCheckBox");

        RequeGridLayout->addWidget(RequeueIgnoreCheckBox, 1, 1, 1, 1);

        RequeueIgnoreTimeLabel = new QLabel(RequeueOptions);
        RequeueIgnoreTimeLabel->setObjectName("RequeueIgnoreTimeLabel");
        sizePolicy1.setHeightForWidth(RequeueIgnoreTimeLabel->sizePolicy().hasHeightForWidth());
        RequeueIgnoreTimeLabel->setSizePolicy(sizePolicy1);

        RequeGridLayout->addWidget(RequeueIgnoreTimeLabel, 2, 0, 1, 1);

        RequeueIgnoreTimeEdit = new QTimeEdit(RequeueOptions);
        RequeueIgnoreTimeEdit->setObjectName("RequeueIgnoreTimeEdit");
        RequeueIgnoreTimeEdit->setMinimumSize(QSize(60, 0));
        RequeueIgnoreTimeEdit->setMaximumSize(QSize(80, 16777215));

        RequeGridLayout->addWidget(RequeueIgnoreTimeEdit, 2, 1, 1, 1);

        horizontalSpacerRequeue = new QSpacerItem(0, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        RequeGridLayout->addItem(horizontalSpacerRequeue, 2, 2, 1, 1);


        AutoDJGridLayout->addWidget(RequeueOptions);

        AddRandomOptions = new QGroupBox(DlgPrefAutoDJDlg);
        AddRandomOptions->setObjectName("AddRandomOptions");
        AddRandomOptions->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        AddRandomGridLayout = new QGridLayout(AddRandomOptions);
        AddRandomGridLayout->setObjectName("AddRandomGridLayout");
        RandomQueueLabel = new QLabel(AddRandomOptions);
        RandomQueueLabel->setObjectName("RandomQueueLabel");
        sizePolicy1.setHeightForWidth(RandomQueueLabel->sizePolicy().hasHeightForWidth());
        RandomQueueLabel->setSizePolicy(sizePolicy1);

        AddRandomGridLayout->addWidget(RandomQueueLabel, 0, 0, 1, 1);

        RandomQueueCheckBox = new QCheckBox(AddRandomOptions);
        RandomQueueCheckBox->setObjectName("RandomQueueCheckBox");

        AddRandomGridLayout->addWidget(RandomQueueCheckBox, 0, 1, 1, 1);

        RandomQueueMinimumLabel = new QLabel(AddRandomOptions);
        RandomQueueMinimumLabel->setObjectName("RandomQueueMinimumLabel");
        sizePolicy1.setHeightForWidth(RandomQueueMinimumLabel->sizePolicy().hasHeightForWidth());
        RandomQueueMinimumLabel->setSizePolicy(sizePolicy1);

        AddRandomGridLayout->addWidget(RandomQueueMinimumLabel, 1, 0, 1, 1);

        RandomQueueMinimumSpinBox = new QSpinBox(AddRandomOptions);
        RandomQueueMinimumSpinBox->setObjectName("RandomQueueMinimumSpinBox");
        RandomQueueMinimumSpinBox->setEnabled(true);
        RandomQueueMinimumSpinBox->setMinimum(1);
        RandomQueueMinimumSpinBox->setMaximum(20);
        RandomQueueMinimumSpinBox->setMinimumSize(QSize(60, 0));
        RandomQueueMinimumSpinBox->setMaximumSize(QSize(80, 16777215));

        AddRandomGridLayout->addWidget(RandomQueueMinimumSpinBox, 1, 1, 1, 1);

        horizontalSpacerRandom = new QSpacerItem(0, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        AddRandomGridLayout->addItem(horizontalSpacerRandom, 1, 2, 1, 1);


        AutoDJGridLayout->addWidget(AddRandomOptions);

        verticalSpacer = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        AutoDJGridLayout->addItem(verticalSpacer);

#if QT_CONFIG(shortcut)
        RequeueIgnoreLabel->setBuddy(RequeueIgnoreCheckBox);
        RandomQueueLabel->setBuddy(RandomQueueCheckBox);
#endif // QT_CONFIG(shortcut)

        retranslateUi(DlgPrefAutoDJDlg);

        QMetaObject::connectSlotsByName(DlgPrefAutoDJDlg);
    } // setupUi

    void retranslateUi(QWidget *DlgPrefAutoDJDlg)
    {
        DlgPrefAutoDJDlg->setWindowTitle(QCoreApplication::translate("DlgPrefAutoDJDlg", "Auto DJ Preferences", nullptr));
        RequeueOptions->setTitle(QCoreApplication::translate("DlgPrefAutoDJDlg", "Re-queue Tracks", nullptr));
        MinimumAvailableLabel->setText(QCoreApplication::translate("DlgPrefAutoDJDlg", "Minimum available tracks in Track Source", nullptr));
#if QT_CONFIG(tooltip)
        MinimumAvailableSpinBox->setToolTip(QCoreApplication::translate("DlgPrefAutoDJDlg", "This percentage of tracks are always available for selecting, regardless of when they were last played.", nullptr));
#endif // QT_CONFIG(tooltip)
        MinimumAvailableSpinBox->setSuffix(QCoreApplication::translate("DlgPrefAutoDJDlg", "%", nullptr));
#if QT_CONFIG(tooltip)
        RequeueIgnoreLabel->setToolTip(QCoreApplication::translate("DlgPrefAutoDJDlg", "Uncheck, to ignore all played tracks.", nullptr));
#endif // QT_CONFIG(tooltip)
        RequeueIgnoreLabel->setText(QCoreApplication::translate("DlgPrefAutoDJDlg", "Suspend track in Track Source from re-queue", nullptr));
#if QT_CONFIG(tooltip)
        RequeueIgnoreCheckBox->setToolTip(QCoreApplication::translate("DlgPrefAutoDJDlg", "Uncheck, to ignore all played tracks.", nullptr));
#endif // QT_CONFIG(tooltip)
        RequeueIgnoreCheckBox->setText(QString());
        RequeueIgnoreTimeLabel->setText(QCoreApplication::translate("DlgPrefAutoDJDlg", "Suspension period for track selection", nullptr));
#if QT_CONFIG(tooltip)
        RequeueIgnoreTimeEdit->setToolTip(QCoreApplication::translate("DlgPrefAutoDJDlg", "Duration after which a track is eligible for selection by Auto DJ again", nullptr));
#endif // QT_CONFIG(tooltip)
        RequeueIgnoreTimeEdit->setDisplayFormat(QCoreApplication::translate("DlgPrefAutoDJDlg", "hh:mm", nullptr));
        AddRandomOptions->setTitle(QCoreApplication::translate("DlgPrefAutoDJDlg", "Add Random Tracks", nullptr));
        RandomQueueLabel->setText(QCoreApplication::translate("DlgPrefAutoDJDlg", "Enable random track addition to queue", nullptr));
#if QT_CONFIG(tooltip)
        RandomQueueCheckBox->setToolTip(QCoreApplication::translate("DlgPrefAutoDJDlg", "Add random tracks from Track Source if the specified minimum tracks remain", nullptr));
#endif // QT_CONFIG(tooltip)
        RandomQueueMinimumLabel->setText(QCoreApplication::translate("DlgPrefAutoDJDlg", "Minimum allowed tracks before addition", nullptr));
#if QT_CONFIG(tooltip)
        RandomQueueMinimumSpinBox->setToolTip(QCoreApplication::translate("DlgPrefAutoDJDlg", "Minimum number of tracks after which random tracks may be added", nullptr));
#endif // QT_CONFIG(tooltip)
    } // retranslateUi

};

namespace Ui {
    class DlgPrefAutoDJDlg: public Ui_DlgPrefAutoDJDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLGPREFAUTODJDLG_H
