/****************************************************************************
** Meta object code from reading C++ file 'wsearchlineedit.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/widget/wsearchlineedit.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wsearchlineedit.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15WSearchLineEditE_t {};
} // unnamed namespace

template <> constexpr inline auto WSearchLineEdit::qt_create_metaobjectdata<qt_meta_tag_ZN15WSearchLineEditE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "WSearchLineEdit",
        "search",
        "",
        "text",
        "setLibraryFocus",
        "FocusWidget",
        "newFocusWidget",
        "slotSetFont",
        "QFont",
        "font",
        "slotRestoreSearch",
        "slotDisableSearch",
        "slotClearSearch",
        "slotClearSearchIfClearButtonHasFocus",
        "slotMoveSelectedHistory",
        "steps",
        "slotDeleteCurrentItem",
        "slotSetShortcutFocus",
        "slotTextChanged",
        "slotIndexChanged",
        "index",
        "slotTriggerSearch",
        "slotSaveSearch"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'search'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'setLibraryFocus'
        QtMocHelpers::SignalData<FocusWidget(FocusWidget)>(4, 2, QMC::AccessPublic, 0x80000000 | 5, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'slotSetFont'
        QtMocHelpers::SlotData<void(const QFont &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'slotRestoreSearch'
        QtMocHelpers::SlotData<void(const QString &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'slotDisableSearch'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotClearSearch'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotClearSearchIfClearButtonHasFocus'
        QtMocHelpers::SlotData<bool()>(13, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'slotMoveSelectedHistory'
        QtMocHelpers::SlotData<void(int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 15 },
        }}),
        // Slot 'slotDeleteCurrentItem'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSetShortcutFocus'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotTextChanged'
        QtMocHelpers::SlotData<void(const QString &)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'slotIndexChanged'
        QtMocHelpers::SlotData<void(int)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 20 },
        }}),
        // Slot 'slotTriggerSearch'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSaveSearch'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WSearchLineEdit, qt_meta_tag_ZN15WSearchLineEditE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject WSearchLineEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<QComboBox::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15WSearchLineEditE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15WSearchLineEditE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15WSearchLineEditE_t>.metaTypes,
    nullptr
} };

void WSearchLineEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WSearchLineEdit *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->search((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: { FocusWidget _r = _t->setLibraryFocus((*reinterpret_cast<std::add_pointer_t<FocusWidget>>(_a[1])));
            if (_a[0]) *reinterpret_cast<FocusWidget*>(_a[0]) = std::move(_r); }  break;
        case 2: _t->slotSetFont((*reinterpret_cast<std::add_pointer_t<QFont>>(_a[1]))); break;
        case 3: _t->slotRestoreSearch((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->slotDisableSearch(); break;
        case 5: _t->slotClearSearch(); break;
        case 6: { bool _r = _t->slotClearSearchIfClearButtonHasFocus();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->slotMoveSelectedHistory((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->slotDeleteCurrentItem(); break;
        case 9: _t->slotSetShortcutFocus(); break;
        case 10: _t->slotTextChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->slotIndexChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->slotTriggerSearch(); break;
        case 13: _t->slotSaveSearch(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WSearchLineEdit::*)(const QString & )>(_a, &WSearchLineEdit::search, 0))
            return;
        if (QtMocHelpers::indexOfMethod<FocusWidget (WSearchLineEdit::*)(FocusWidget )>(_a, &WSearchLineEdit::setLibraryFocus, 1))
            return;
    }
}

const QMetaObject *WSearchLineEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *WSearchLineEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15WSearchLineEditE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "WBaseWidget"))
        return static_cast< WBaseWidget*>(this);
    return QComboBox::qt_metacast(_clname);
}

int WSearchLineEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QComboBox::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void WSearchLineEdit::search(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
FocusWidget WSearchLineEdit::setLibraryFocus(FocusWidget _t1)
{
    FocusWidget _t0{};
    QMetaObject::activate<FocusWidget>(this, &staticMetaObject, 1, std::addressof(_t0), _t1);
    return _t0;
}
QT_WARNING_POP
