/****************************************************************************
** Meta object code from reading C++ file 'trackdao.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/library/dao/trackdao.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#include <QtCore/QSet>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'trackdao.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8TrackDAOE_t {};
} // unnamed namespace

template <> constexpr inline auto TrackDAO::qt_create_metaobjectdata<qt_meta_tag_ZN8TrackDAOE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TrackDAO",
        "trackDirty",
        "",
        "TrackId",
        "trackId",
        "trackClean",
        "tracksAdded",
        "QSet<TrackId>",
        "trackIds",
        "tracksChanged",
        "tracksRemoved",
        "progressVerifyTracksOutside",
        "path",
        "progressCoverArt",
        "file",
        "forceModelUpdate",
        "removeTrackRows",
        "slotDatabaseTracksChanged",
        "changedTrackIds",
        "slotDatabaseTracksRelocated",
        "QList<RelocatedTrack>",
        "relocatedTracks"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'trackDirty'
        QtMocHelpers::SignalData<void(TrackId)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'trackClean'
        QtMocHelpers::SignalData<void(TrackId)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'tracksAdded'
        QtMocHelpers::SignalData<void(const QSet<TrackId> &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'tracksChanged'
        QtMocHelpers::SignalData<void(const QSet<TrackId> &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'tracksRemoved'
        QtMocHelpers::SignalData<void(const QSet<TrackId> &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'progressVerifyTracksOutside'
        QtMocHelpers::SignalData<void(const QString &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 },
        }}),
        // Signal 'progressCoverArt'
        QtMocHelpers::SignalData<void(const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Signal 'forceModelUpdate'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'removeTrackRows'
        QtMocHelpers::SignalData<void(const QSet<TrackId> &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'slotDatabaseTracksChanged'
        QtMocHelpers::SlotData<void(const QSet<TrackId> &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 18 },
        }}),
        // Slot 'slotDatabaseTracksRelocated'
        QtMocHelpers::SlotData<void(const QList<RelocatedTrack> &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TrackDAO, qt_meta_tag_ZN8TrackDAOE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TrackDAO::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8TrackDAOE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8TrackDAOE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8TrackDAOE_t>.metaTypes,
    nullptr
} };

void TrackDAO::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TrackDAO *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->trackDirty((*reinterpret_cast<std::add_pointer_t<TrackId>>(_a[1]))); break;
        case 1: _t->trackClean((*reinterpret_cast<std::add_pointer_t<TrackId>>(_a[1]))); break;
        case 2: _t->tracksAdded((*reinterpret_cast<std::add_pointer_t<QSet<TrackId>>>(_a[1]))); break;
        case 3: _t->tracksChanged((*reinterpret_cast<std::add_pointer_t<QSet<TrackId>>>(_a[1]))); break;
        case 4: _t->tracksRemoved((*reinterpret_cast<std::add_pointer_t<QSet<TrackId>>>(_a[1]))); break;
        case 5: _t->progressVerifyTracksOutside((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->progressCoverArt((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->forceModelUpdate(); break;
        case 8: _t->removeTrackRows((*reinterpret_cast<std::add_pointer_t<QSet<TrackId>>>(_a[1]))); break;
        case 9: _t->slotDatabaseTracksChanged((*reinterpret_cast<std::add_pointer_t<QSet<TrackId>>>(_a[1]))); break;
        case 10: _t->slotDatabaseTracksRelocated((*reinterpret_cast<std::add_pointer_t<QList<RelocatedTrack>>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackId >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackId >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<TrackId> >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<TrackId> >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<TrackId> >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<TrackId> >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<TrackId> >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<RelocatedTrack> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TrackDAO::*)(TrackId )>(_a, &TrackDAO::trackDirty, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TrackDAO::*)(TrackId )>(_a, &TrackDAO::trackClean, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TrackDAO::*)(const QSet<TrackId> & )>(_a, &TrackDAO::tracksAdded, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TrackDAO::*)(const QSet<TrackId> & )>(_a, &TrackDAO::tracksChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (TrackDAO::*)(const QSet<TrackId> & )>(_a, &TrackDAO::tracksRemoved, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (TrackDAO::*)(const QString & )>(_a, &TrackDAO::progressVerifyTracksOutside, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (TrackDAO::*)(const QString & )>(_a, &TrackDAO::progressCoverArt, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (TrackDAO::*)()>(_a, &TrackDAO::forceModelUpdate, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (TrackDAO::*)(const QSet<TrackId> & )>(_a, &TrackDAO::removeTrackRows, 8))
            return;
    }
}

const QMetaObject *TrackDAO::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TrackDAO::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8TrackDAOE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "DAO"))
        return static_cast< DAO*>(this);
    if (!strcmp(_clname, "GlobalTrackCacheRelocator"))
        return static_cast< GlobalTrackCacheRelocator*>(this);
    return QObject::qt_metacast(_clname);
}

int TrackDAO::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void TrackDAO::trackDirty(TrackId _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void TrackDAO::trackClean(TrackId _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void TrackDAO::tracksAdded(const QSet<TrackId> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void TrackDAO::tracksChanged(const QSet<TrackId> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void TrackDAO::tracksRemoved(const QSet<TrackId> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void TrackDAO::progressVerifyTracksOutside(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void TrackDAO::progressCoverArt(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void TrackDAO::forceModelUpdate()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void TrackDAO::removeTrackRows(const QSet<TrackId> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}
QT_WARNING_POP
