/****************************************************************************
** Meta object code from reading C++ file 'dlgcontrollerlearning.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/controllers/dlgcontrollerlearning.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dlgcontrollerlearning.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN21DlgControllerLearningE_t {};
} // unnamed namespace

template <> constexpr inline auto DlgControllerLearning::qt_create_metaobjectdata<qt_meta_tag_ZN21DlgControllerLearningE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DlgControllerLearning",
        "learnTemporaryInputMappings",
        "",
        "MidiInputMappings",
        "mappings",
        "clearTemporaryInputMappings",
        "commitTemporaryInputMappings",
        "inputMappingsLearned",
        "startLearning",
        "stopLearning",
        "listenForClicks",
        "stopListeningForClicks",
        "controlPicked",
        "ConfigKey",
        "control",
        "controlClicked",
        "controlKey",
        "comboboxIndexChanged",
        "index",
        "slotMessageReceived",
        "status",
        "value",
        "slotCancelLearn",
        "slotChooseControlPressed",
        "slotTimerExpired",
        "slotFirstMessageTimeout",
        "slotRetry",
        "slotStartLearningPressed",
        "slotMidiOptionsChanged",
        "showControlMenu"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'learnTemporaryInputMappings'
        QtMocHelpers::SignalData<void(const MidiInputMappings &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'clearTemporaryInputMappings'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'commitTemporaryInputMappings'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'inputMappingsLearned'
        QtMocHelpers::SignalData<void(const MidiInputMappings &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'startLearning'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'stopLearning'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'listenForClicks'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'stopListeningForClicks'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'controlPicked'
        QtMocHelpers::SlotData<void(const ConfigKey &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'controlClicked'
        QtMocHelpers::SlotData<void(const ConfigKey &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 16 },
        }}),
        // Slot 'comboboxIndexChanged'
        QtMocHelpers::SlotData<void(int)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 18 },
        }}),
        // Slot 'slotMessageReceived'
        QtMocHelpers::SlotData<void(unsigned char, unsigned char, unsigned char)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UChar, 20 }, { QMetaType::UChar, 14 }, { QMetaType::UChar, 21 },
        }}),
        // Slot 'slotCancelLearn'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotChooseControlPressed'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotTimerExpired'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFirstMessageTimeout'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotRetry'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotStartLearningPressed'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotMidiOptionsChanged'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showControlMenu'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DlgControllerLearning, qt_meta_tag_ZN21DlgControllerLearningE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DlgControllerLearning::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21DlgControllerLearningE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21DlgControllerLearningE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21DlgControllerLearningE_t>.metaTypes,
    nullptr
} };

void DlgControllerLearning::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DlgControllerLearning *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->learnTemporaryInputMappings((*reinterpret_cast<std::add_pointer_t<MidiInputMappings>>(_a[1]))); break;
        case 1: _t->clearTemporaryInputMappings(); break;
        case 2: _t->commitTemporaryInputMappings(); break;
        case 3: _t->inputMappingsLearned((*reinterpret_cast<std::add_pointer_t<MidiInputMappings>>(_a[1]))); break;
        case 4: _t->startLearning(); break;
        case 5: _t->stopLearning(); break;
        case 6: _t->listenForClicks(); break;
        case 7: _t->stopListeningForClicks(); break;
        case 8: _t->controlPicked((*reinterpret_cast<std::add_pointer_t<ConfigKey>>(_a[1]))); break;
        case 9: _t->controlClicked((*reinterpret_cast<std::add_pointer_t<ConfigKey>>(_a[1]))); break;
        case 10: _t->comboboxIndexChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->slotMessageReceived((*reinterpret_cast<std::add_pointer_t<uchar>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uchar>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<uchar>>(_a[3]))); break;
        case 12: _t->slotCancelLearn(); break;
        case 13: _t->slotChooseControlPressed(); break;
        case 14: _t->slotTimerExpired(); break;
        case 15: _t->slotFirstMessageTimeout(); break;
        case 16: _t->slotRetry(); break;
        case 17: _t->slotStartLearningPressed(); break;
        case 18: _t->slotMidiOptionsChanged(); break;
        case 19: _t->showControlMenu(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ConfigKey >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ConfigKey >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DlgControllerLearning::*)(const MidiInputMappings & )>(_a, &DlgControllerLearning::learnTemporaryInputMappings, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgControllerLearning::*)()>(_a, &DlgControllerLearning::clearTemporaryInputMappings, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgControllerLearning::*)()>(_a, &DlgControllerLearning::commitTemporaryInputMappings, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgControllerLearning::*)(const MidiInputMappings & )>(_a, &DlgControllerLearning::inputMappingsLearned, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgControllerLearning::*)()>(_a, &DlgControllerLearning::startLearning, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgControllerLearning::*)()>(_a, &DlgControllerLearning::stopLearning, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgControllerLearning::*)()>(_a, &DlgControllerLearning::listenForClicks, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgControllerLearning::*)()>(_a, &DlgControllerLearning::stopListeningForClicks, 7))
            return;
    }
}

const QMetaObject *DlgControllerLearning::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DlgControllerLearning::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21DlgControllerLearningE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::DlgControllerLearning"))
        return static_cast< Ui::DlgControllerLearning*>(this);
    return QDialog::qt_metacast(_clname);
}

int DlgControllerLearning::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    return _id;
}

// SIGNAL 0
void DlgControllerLearning::learnTemporaryInputMappings(const MidiInputMappings & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void DlgControllerLearning::clearTemporaryInputMappings()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void DlgControllerLearning::commitTemporaryInputMappings()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void DlgControllerLearning::inputMappingsLearned(const MidiInputMappings & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void DlgControllerLearning::startLearning()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void DlgControllerLearning::stopLearning()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void DlgControllerLearning::listenForClicks()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void DlgControllerLearning::stopListeningForClicks()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
