/****************************************************************************
** Meta object code from reading C++ file 'dlgprefdeck.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/preferences/dialog/dlgprefdeck.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dlgprefdeck.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11DlgPrefDeckE_t {};
} // unnamed namespace

template <> constexpr inline auto DlgPrefDeck::qt_create_metaobjectdata<qt_meta_tag_ZN11DlgPrefDeckE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DlgPrefDeck",
        "slotUpdate",
        "",
        "slotApply",
        "slotResetToDefaults",
        "slotMoveIntroStartCheckbox",
        "checked",
        "slotRateRangeComboBox",
        "index",
        "slotRateInversionCheckbox",
        "invert",
        "slotKeyLockModeSelected",
        "QAbstractButton*",
        "slotKeyUnlockModeSelected",
        "slotRateTempCoarseSpinbox",
        "slotRateTempFineSpinbox",
        "slotRatePermCoarseSpinbox",
        "slotRatePermFineSpinbox",
        "slotSetTrackTimeDisplay",
        "slotCueModeCombobox",
        "slotSetTrackLoadMode",
        "comboboxIndex",
        "slotLoadWhenDeckPlayingIndexChanged",
        "slotCloneDeckOnLoadDoubleTapCheckbox",
        "slotRateRampingModeLinearButton",
        "slotRateRampSensitivitySlider",
        "slotTimeFormatChanged",
        "slotNumDecksChanged",
        "initializing",
        "slotNumSamplersChanged",
        "slotUpdateSpeedAutoReset",
        "slotUpdatePitchAutoReset"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotUpdate'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotApply'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotResetToDefaults'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotMoveIntroStartCheckbox'
        QtMocHelpers::SlotData<void(bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Slot 'slotRateRangeComboBox'
        QtMocHelpers::SlotData<void(int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'slotRateInversionCheckbox'
        QtMocHelpers::SlotData<void(bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Slot 'slotKeyLockModeSelected'
        QtMocHelpers::SlotData<void(QAbstractButton *)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 2 },
        }}),
        // Slot 'slotKeyUnlockModeSelected'
        QtMocHelpers::SlotData<void(QAbstractButton *)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 2 },
        }}),
        // Slot 'slotRateTempCoarseSpinbox'
        QtMocHelpers::SlotData<void(double)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotRateTempFineSpinbox'
        QtMocHelpers::SlotData<void(double)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotRatePermCoarseSpinbox'
        QtMocHelpers::SlotData<void(double)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotRatePermFineSpinbox'
        QtMocHelpers::SlotData<void(double)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotSetTrackTimeDisplay'
        QtMocHelpers::SlotData<void(QAbstractButton *)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 2 },
        }}),
        // Slot 'slotSetTrackTimeDisplay'
        QtMocHelpers::SlotData<void(double)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotCueModeCombobox'
        QtMocHelpers::SlotData<void(int)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'slotSetTrackLoadMode'
        QtMocHelpers::SlotData<void(int)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 21 },
        }}),
        // Slot 'slotLoadWhenDeckPlayingIndexChanged'
        QtMocHelpers::SlotData<void(int)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 21 },
        }}),
        // Slot 'slotCloneDeckOnLoadDoubleTapCheckbox'
        QtMocHelpers::SlotData<void(bool)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'slotRateRampingModeLinearButton'
        QtMocHelpers::SlotData<void(bool)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'slotRateRampSensitivitySlider'
        QtMocHelpers::SlotData<void(int)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'slotTimeFormatChanged'
        QtMocHelpers::SlotData<void(double)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotNumDecksChanged'
        QtMocHelpers::SlotData<void(double, bool)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 }, { QMetaType::Bool, 28 },
        }}),
        // Slot 'slotNumDecksChanged'
        QtMocHelpers::SlotData<void(double)>(27, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotNumSamplersChanged'
        QtMocHelpers::SlotData<void(double, bool)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 }, { QMetaType::Bool, 28 },
        }}),
        // Slot 'slotNumSamplersChanged'
        QtMocHelpers::SlotData<void(double)>(29, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotUpdateSpeedAutoReset'
        QtMocHelpers::SlotData<void(bool)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'slotUpdatePitchAutoReset'
        QtMocHelpers::SlotData<void(bool)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DlgPrefDeck, qt_meta_tag_ZN11DlgPrefDeckE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DlgPrefDeck::staticMetaObject = { {
    QMetaObject::SuperData::link<DlgPreferencePage::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11DlgPrefDeckE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11DlgPrefDeckE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11DlgPrefDeckE_t>.metaTypes,
    nullptr
} };

void DlgPrefDeck::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DlgPrefDeck *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotUpdate(); break;
        case 1: _t->slotApply(); break;
        case 2: _t->slotResetToDefaults(); break;
        case 3: _t->slotMoveIntroStartCheckbox((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->slotRateRangeComboBox((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->slotRateInversionCheckbox((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->slotKeyLockModeSelected((*reinterpret_cast<std::add_pointer_t<QAbstractButton*>>(_a[1]))); break;
        case 7: _t->slotKeyUnlockModeSelected((*reinterpret_cast<std::add_pointer_t<QAbstractButton*>>(_a[1]))); break;
        case 8: _t->slotRateTempCoarseSpinbox((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 9: _t->slotRateTempFineSpinbox((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 10: _t->slotRatePermCoarseSpinbox((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 11: _t->slotRatePermFineSpinbox((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 12: _t->slotSetTrackTimeDisplay((*reinterpret_cast<std::add_pointer_t<QAbstractButton*>>(_a[1]))); break;
        case 13: _t->slotSetTrackTimeDisplay((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 14: _t->slotCueModeCombobox((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 15: _t->slotSetTrackLoadMode((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 16: _t->slotLoadWhenDeckPlayingIndexChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 17: _t->slotCloneDeckOnLoadDoubleTapCheckbox((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: _t->slotRateRampingModeLinearButton((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 19: _t->slotRateRampSensitivitySlider((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 20: _t->slotTimeFormatChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 21: _t->slotNumDecksChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 22: _t->slotNumDecksChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 23: _t->slotNumSamplersChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 24: _t->slotNumSamplersChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 25: _t->slotUpdateSpeedAutoReset((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 26: _t->slotUpdatePitchAutoReset((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *DlgPrefDeck::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DlgPrefDeck::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11DlgPrefDeckE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::DlgPrefDeckDlg"))
        return static_cast< Ui::DlgPrefDeckDlg*>(this);
    return DlgPreferencePage::qt_metacast(_clname);
}

int DlgPrefDeck::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DlgPreferencePage::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 27)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 27;
    }
    return _id;
}
QT_WARNING_POP
