/****************************************************************************
** Meta object code from reading C++ file 'dlgpreferences.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/preferences/dialog/dlgpreferences.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dlgpreferences.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14DlgPreferencesE_t {};
} // unnamed namespace

template <> constexpr inline auto DlgPreferences::qt_create_metaobjectdata<qt_meta_tag_ZN14DlgPreferencesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DlgPreferences",
        "closeDlg",
        "",
        "showDlg",
        "applyPreferences",
        "cancelPreferences",
        "resetToDefaults",
        "reloadUserInterface",
        "tooltipModeChanged",
        "mixxx::preferences::Tooltips",
        "tooltipMode",
        "menuBarAutoHideChanged",
        "changePage",
        "QTreeWidgetItem*",
        "pCurrent",
        "pPrevious",
        "showSoundHardwarePage",
        "slotButtonPressed",
        "QAbstractButton*",
        "pButton"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'closeDlg'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'showDlg'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'applyPreferences'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cancelPreferences'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'resetToDefaults'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'reloadUserInterface'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'tooltipModeChanged'
        QtMocHelpers::SignalData<void(mixxx::preferences::Tooltips)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'menuBarAutoHideChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'changePage'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *, QTreeWidgetItem *)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { 0x80000000 | 13, 15 },
        }}),
        // Slot 'showSoundHardwarePage'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotButtonPressed'
        QtMocHelpers::SlotData<void(QAbstractButton *)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DlgPreferences, qt_meta_tag_ZN14DlgPreferencesE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DlgPreferences::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14DlgPreferencesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14DlgPreferencesE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14DlgPreferencesE_t>.metaTypes,
    nullptr
} };

void DlgPreferences::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DlgPreferences *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->closeDlg(); break;
        case 1: _t->showDlg(); break;
        case 2: _t->applyPreferences(); break;
        case 3: _t->cancelPreferences(); break;
        case 4: _t->resetToDefaults(); break;
        case 5: _t->reloadUserInterface(); break;
        case 6: _t->tooltipModeChanged((*reinterpret_cast<std::add_pointer_t<mixxx::preferences::Tooltips>>(_a[1]))); break;
        case 7: _t->menuBarAutoHideChanged(); break;
        case 8: _t->changePage((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[2]))); break;
        case 9: _t->showSoundHardwarePage(); break;
        case 10: _t->slotButtonPressed((*reinterpret_cast<std::add_pointer_t<QAbstractButton*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DlgPreferences::*)()>(_a, &DlgPreferences::closeDlg, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgPreferences::*)()>(_a, &DlgPreferences::showDlg, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgPreferences::*)()>(_a, &DlgPreferences::applyPreferences, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgPreferences::*)()>(_a, &DlgPreferences::cancelPreferences, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgPreferences::*)()>(_a, &DlgPreferences::resetToDefaults, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgPreferences::*)()>(_a, &DlgPreferences::reloadUserInterface, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgPreferences::*)(mixxx::preferences::Tooltips )>(_a, &DlgPreferences::tooltipModeChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgPreferences::*)()>(_a, &DlgPreferences::menuBarAutoHideChanged, 7))
            return;
    }
}

const QMetaObject *DlgPreferences::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DlgPreferences::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14DlgPreferencesE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::DlgPreferencesDlg"))
        return static_cast< Ui::DlgPreferencesDlg*>(this);
    return QDialog::qt_metacast(_clname);
}

int DlgPreferences::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void DlgPreferences::closeDlg()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void DlgPreferences::showDlg()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void DlgPreferences::applyPreferences()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void DlgPreferences::cancelPreferences()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void DlgPreferences::resetToDefaults()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void DlgPreferences::reloadUserInterface()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void DlgPreferences::tooltipModeChanged(mixxx::preferences::Tooltips _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void DlgPreferences::menuBarAutoHideChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
