/****************************************************************************
** Meta object code from reading C++ file 'wcoverart.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/widget/wcoverart.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wcoverart.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9WCoverArtE_t {};
} // unnamed namespace

template <> constexpr inline auto WCoverArt::qt_create_metaobjectdata<qt_meta_tag_ZN9WCoverArtE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "WCoverArt",
        "trackDropped",
        "",
        "filename",
        "group",
        "cloneDeck",
        "sourceGroup",
        "targetGroup",
        "slotLoadTrack",
        "TrackPointer",
        "slotLoadingTrack",
        "pNewTrack",
        "pOldTrack",
        "slotReset",
        "slotEnable",
        "slotCoverFound",
        "const QObject*",
        "pRequester",
        "CoverInfo",
        "coverInfo",
        "QPixmap",
        "pixmap",
        "slotCoverInfoSelected",
        "CoverInfoRelative",
        "slotReloadCoverArt",
        "slotTrackCoverArtUpdated"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'trackDropped'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 },
        }}),
        // Signal 'cloneDeck'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 7 },
        }}),
        // Slot 'slotLoadTrack'
        QtMocHelpers::SlotData<void(TrackPointer)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 2 },
        }}),
        // Slot 'slotLoadingTrack'
        QtMocHelpers::SlotData<void(TrackPointer, TrackPointer)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 11 }, { 0x80000000 | 9, 12 },
        }}),
        // Slot 'slotReset'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotEnable'
        QtMocHelpers::SlotData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'slotCoverFound'
        QtMocHelpers::SlotData<void(const QObject *, const CoverInfo &, const QPixmap &)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 16, 17 }, { 0x80000000 | 18, 19 }, { 0x80000000 | 20, 21 },
        }}),
        // Slot 'slotCoverInfoSelected'
        QtMocHelpers::SlotData<void(const CoverInfoRelative &)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 23, 19 },
        }}),
        // Slot 'slotReloadCoverArt'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotTrackCoverArtUpdated'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WCoverArt, qt_meta_tag_ZN9WCoverArtE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject WCoverArt::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9WCoverArtE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9WCoverArtE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9WCoverArtE_t>.metaTypes,
    nullptr
} };

void WCoverArt::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WCoverArt *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->trackDropped((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 1: _t->cloneDeck((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 2: _t->slotLoadTrack((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1]))); break;
        case 3: _t->slotLoadingTrack((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[2]))); break;
        case 4: _t->slotReset(); break;
        case 5: _t->slotEnable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->slotCoverFound((*reinterpret_cast<std::add_pointer_t<const QObject*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CoverInfo>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QPixmap>>(_a[3]))); break;
        case 7: _t->slotCoverInfoSelected((*reinterpret_cast<std::add_pointer_t<CoverInfoRelative>>(_a[1]))); break;
        case 8: _t->slotReloadCoverArt(); break;
        case 9: _t->slotTrackCoverArtUpdated(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WCoverArt::*)(const QString & , const QString & )>(_a, &WCoverArt::trackDropped, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WCoverArt::*)(const QString & , const QString & )>(_a, &WCoverArt::cloneDeck, 1))
            return;
    }
}

const QMetaObject *WCoverArt::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *WCoverArt::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9WCoverArtE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "WBaseWidget"))
        return static_cast< WBaseWidget*>(this);
    if (!strcmp(_clname, "TrackDropTarget"))
        return static_cast< TrackDropTarget*>(this);
    return QWidget::qt_metacast(_clname);
}

int WCoverArt::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void WCoverArt::trackDropped(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void WCoverArt::cloneDeck(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}
QT_WARNING_POP
