/********************************************************************************
** Form generated from reading UI file 'dlgprefrecorddlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLGPREFRECORDDLG_H
#define UI_DLGPREFRECORDDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DlgPrefRecordDlg
{
public:
    QGridLayout *gridLayout_2;
    QGroupBox *groupBoxRecordingsPath;
    QGridLayout *gridLayout_Recordings;
    QLabel *TextLabel_Recordings;
    QLineEdit *LineEditRecordings;
    QPushButton *PushButtonBrowseRecordings;
    QHBoxLayout *splittingLayout;
    QLabel *LabelSplitting;
    QComboBox *comboBoxSplitting;
    QSpacerItem *horizontalSpacer_split;
    QCheckBox *CheckBoxRecordCueFile;
    QGroupBox *groupBoxFileformat;
    QGridLayout *gridLayout_4;
    QLabel *LabelLossless;
    QHBoxLayout *LosslessEncLayout;
    QLabel *LabelLossy;
    QHBoxLayout *LossyEncLayout;
    QFrame *line;
    QLabel *labelOptionGroup;
    QHBoxLayout *OptionGroupsLayout;
    QWidget *qualityGroup;
    QHBoxLayout *qualityHLayout;
    QLabel *LabelQuality;
    QSlider *SliderQuality;
    QLabel *TextQuality;
    QWidget *compressionGroup;
    QHBoxLayout *compressionHLayout;
    QLabel *LabelCompression;
    QSlider *SliderCompression;
    QLabel *TextCompression;
    QGroupBox *groupBoxTags;
    QFormLayout *formLayout;
    QLabel *LabelTitle;
    QLineEdit *LineEditTitle;
    QLabel *LabelAuthor;
    QLineEdit *LineEditAuthor;
    QLabel *LabelAlbum;
    QLineEdit *LineEditAlbum;
    QSpacerItem *spacerItem;

    void setupUi(QWidget *DlgPrefRecordDlg)
    {
        if (DlgPrefRecordDlg->objectName().isEmpty())
            DlgPrefRecordDlg->setObjectName("DlgPrefRecordDlg");
        DlgPrefRecordDlg->resize(603, 568);
        gridLayout_2 = new QGridLayout(DlgPrefRecordDlg);
        gridLayout_2->setSpacing(6);
        gridLayout_2->setContentsMargins(11, 11, 11, 11);
        gridLayout_2->setObjectName("gridLayout_2");
        groupBoxRecordingsPath = new QGroupBox(DlgPrefRecordDlg);
        groupBoxRecordingsPath->setObjectName("groupBoxRecordingsPath");
        gridLayout_Recordings = new QGridLayout(groupBoxRecordingsPath);
        gridLayout_Recordings->setSpacing(6);
        gridLayout_Recordings->setContentsMargins(11, 11, 11, 11);
        gridLayout_Recordings->setObjectName("gridLayout_Recordings");
        TextLabel_Recordings = new QLabel(groupBoxRecordingsPath);
        TextLabel_Recordings->setObjectName("TextLabel_Recordings");
        TextLabel_Recordings->setEnabled(true);
        TextLabel_Recordings->setMinimumSize(QSize(100, 10));
        QFont font;
        TextLabel_Recordings->setFont(font);
        TextLabel_Recordings->setWordWrap(false);

        gridLayout_Recordings->addWidget(TextLabel_Recordings, 0, 0, 1, 1);

        LineEditRecordings = new QLineEdit(groupBoxRecordingsPath);
        LineEditRecordings->setObjectName("LineEditRecordings");
        LineEditRecordings->setMinimumSize(QSize(100, 10));
        LineEditRecordings->setFont(font);

        gridLayout_Recordings->addWidget(LineEditRecordings, 0, 1, 1, 1);

        PushButtonBrowseRecordings = new QPushButton(groupBoxRecordingsPath);
        PushButtonBrowseRecordings->setObjectName("PushButtonBrowseRecordings");
        PushButtonBrowseRecordings->setFont(font);

        gridLayout_Recordings->addWidget(PushButtonBrowseRecordings, 0, 2, 1, 1);

        splittingLayout = new QHBoxLayout();
        splittingLayout->setSpacing(6);
        splittingLayout->setObjectName("splittingLayout");
        splittingLayout->setContentsMargins(0, 0, 0, 0);
        LabelSplitting = new QLabel(groupBoxRecordingsPath);
        LabelSplitting->setObjectName("LabelSplitting");

        splittingLayout->addWidget(LabelSplitting);

        comboBoxSplitting = new QComboBox(groupBoxRecordingsPath);
        comboBoxSplitting->setObjectName("comboBoxSplitting");

        splittingLayout->addWidget(comboBoxSplitting);

        horizontalSpacer_split = new QSpacerItem(0, 0, QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Minimum);

        splittingLayout->addItem(horizontalSpacer_split);


        gridLayout_Recordings->addLayout(splittingLayout, 1, 0, 1, 3);

        CheckBoxRecordCueFile = new QCheckBox(groupBoxRecordingsPath);
        CheckBoxRecordCueFile->setObjectName("CheckBoxRecordCueFile");

        gridLayout_Recordings->addWidget(CheckBoxRecordCueFile, 2, 0, 1, 3);


        gridLayout_2->addWidget(groupBoxRecordingsPath, 0, 0, 1, 3);

        groupBoxFileformat = new QGroupBox(DlgPrefRecordDlg);
        groupBoxFileformat->setObjectName("groupBoxFileformat");
        gridLayout_4 = new QGridLayout(groupBoxFileformat);
        gridLayout_4->setSpacing(6);
        gridLayout_4->setContentsMargins(11, 11, 11, 11);
        gridLayout_4->setObjectName("gridLayout_4");
        LabelLossless = new QLabel(groupBoxFileformat);
        LabelLossless->setObjectName("LabelLossless");

        gridLayout_4->addWidget(LabelLossless, 1, 0, 1, 1);

        LosslessEncLayout = new QHBoxLayout();
        LosslessEncLayout->setSpacing(6);
        LosslessEncLayout->setObjectName("LosslessEncLayout");
        LosslessEncLayout->setContentsMargins(-1, 0, -1, -1);

        gridLayout_4->addLayout(LosslessEncLayout, 1, 1, 1, 2);

        LabelLossy = new QLabel(groupBoxFileformat);
        LabelLossy->setObjectName("LabelLossy");

        gridLayout_4->addWidget(LabelLossy, 2, 0, 1, 1);

        LossyEncLayout = new QHBoxLayout();
        LossyEncLayout->setSpacing(6);
        LossyEncLayout->setObjectName("LossyEncLayout");

        gridLayout_4->addLayout(LossyEncLayout, 2, 1, 1, 2);

        line = new QFrame(groupBoxFileformat);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayout_4->addWidget(line, 3, 0, 1, 3);

        labelOptionGroup = new QLabel(groupBoxFileformat);
        labelOptionGroup->setObjectName("labelOptionGroup");
        labelOptionGroup->setMinimumSize(QSize(0, 0));

        gridLayout_4->addWidget(labelOptionGroup, 4, 0, 1, 1);

        OptionGroupsLayout = new QHBoxLayout();
        OptionGroupsLayout->setSpacing(6);
        OptionGroupsLayout->setObjectName("OptionGroupsLayout");

        gridLayout_4->addLayout(OptionGroupsLayout, 4, 1, 1, 2);

        qualityGroup = new QWidget(groupBoxFileformat);
        qualityGroup->setObjectName("qualityGroup");
        qualityHLayout = new QHBoxLayout(qualityGroup);
        qualityHLayout->setSpacing(6);
        qualityHLayout->setContentsMargins(11, 11, 11, 11);
        qualityHLayout->setObjectName("qualityHLayout");
        qualityHLayout->setContentsMargins(0, 0, 0, 0);
        LabelQuality = new QLabel(qualityGroup);
        LabelQuality->setObjectName("LabelQuality");

        qualityHLayout->addWidget(LabelQuality);

        SliderQuality = new QSlider(qualityGroup);
        SliderQuality->setObjectName("SliderQuality");
        SliderQuality->setMinimum(1);
        SliderQuality->setMaximum(11);
        SliderQuality->setSingleStep(1);
        SliderQuality->setPageStep(1);
        SliderQuality->setValue(9);
        SliderQuality->setOrientation(Qt::Horizontal);
        SliderQuality->setTickInterval(5);

        qualityHLayout->addWidget(SliderQuality);

        TextQuality = new QLabel(qualityGroup);
        TextQuality->setObjectName("TextQuality");
        TextQuality->setMinimumSize(QSize(60, 0));
        TextQuality->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        TextQuality->setWordWrap(false);

        qualityHLayout->addWidget(TextQuality);


        gridLayout_4->addWidget(qualityGroup, 5, 0, 1, 3);

        compressionGroup = new QWidget(groupBoxFileformat);
        compressionGroup->setObjectName("compressionGroup");
        compressionHLayout = new QHBoxLayout(compressionGroup);
        compressionHLayout->setSpacing(6);
        compressionHLayout->setContentsMargins(11, 11, 11, 11);
        compressionHLayout->setObjectName("compressionHLayout");
        compressionHLayout->setContentsMargins(0, 0, 0, 0);
        LabelCompression = new QLabel(compressionGroup);
        LabelCompression->setObjectName("LabelCompression");

        compressionHLayout->addWidget(LabelCompression);

        SliderCompression = new QSlider(compressionGroup);
        SliderCompression->setObjectName("SliderCompression");
        SliderCompression->setOrientation(Qt::Horizontal);

        compressionHLayout->addWidget(SliderCompression);

        TextCompression = new QLabel(compressionGroup);
        TextCompression->setObjectName("TextCompression");
        TextCompression->setMinimumSize(QSize(60, 0));
        TextCompression->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        compressionHLayout->addWidget(TextCompression);


        gridLayout_4->addWidget(compressionGroup, 6, 0, 1, 3);


        gridLayout_2->addWidget(groupBoxFileformat, 1, 0, 1, 3);

        groupBoxTags = new QGroupBox(DlgPrefRecordDlg);
        groupBoxTags->setObjectName("groupBoxTags");
        formLayout = new QFormLayout(groupBoxTags);
        formLayout->setSpacing(6);
        formLayout->setContentsMargins(11, 11, 11, 11);
        formLayout->setObjectName("formLayout");
        LabelTitle = new QLabel(groupBoxTags);
        LabelTitle->setObjectName("LabelTitle");
        LabelTitle->setEnabled(true);
        LabelTitle->setFont(font);
        LabelTitle->setWordWrap(false);

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, LabelTitle);

        LineEditTitle = new QLineEdit(groupBoxTags);
        LineEditTitle->setObjectName("LineEditTitle");

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, LineEditTitle);

        LabelAuthor = new QLabel(groupBoxTags);
        LabelAuthor->setObjectName("LabelAuthor");
        LabelAuthor->setEnabled(true);
        LabelAuthor->setFont(font);
        LabelAuthor->setWordWrap(false);

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, LabelAuthor);

        LineEditAuthor = new QLineEdit(groupBoxTags);
        LineEditAuthor->setObjectName("LineEditAuthor");

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, LineEditAuthor);

        LabelAlbum = new QLabel(groupBoxTags);
        LabelAlbum->setObjectName("LabelAlbum");
        LabelAlbum->setEnabled(true);
        LabelAlbum->setFont(font);
        LabelAlbum->setWordWrap(false);

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, LabelAlbum);

        LineEditAlbum = new QLineEdit(groupBoxTags);
        LineEditAlbum->setObjectName("LineEditAlbum");

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, LineEditAlbum);


        gridLayout_2->addWidget(groupBoxTags, 2, 0, 1, 3);

        spacerItem = new QSpacerItem(20, 51, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_2->addItem(spacerItem, 7, 1, 1, 1);

#if QT_CONFIG(shortcut)
        TextLabel_Recordings->setBuddy(LineEditRecordings);
        LabelSplitting->setBuddy(comboBoxSplitting);
        TextQuality->setBuddy(SliderQuality);
        TextCompression->setBuddy(SliderCompression);
        LabelTitle->setBuddy(LineEditTitle);
        LabelAuthor->setBuddy(LineEditAuthor);
        LabelAlbum->setBuddy(LineEditAlbum);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(LineEditRecordings, PushButtonBrowseRecordings);
        QWidget::setTabOrder(PushButtonBrowseRecordings, comboBoxSplitting);
        QWidget::setTabOrder(comboBoxSplitting, CheckBoxRecordCueFile);
        QWidget::setTabOrder(CheckBoxRecordCueFile, SliderCompression);
        QWidget::setTabOrder(SliderCompression, SliderQuality);
        QWidget::setTabOrder(SliderQuality, LineEditTitle);
        QWidget::setTabOrder(LineEditTitle, LineEditAuthor);
        QWidget::setTabOrder(LineEditAuthor, LineEditAlbum);

        retranslateUi(DlgPrefRecordDlg);

        QMetaObject::connectSlotsByName(DlgPrefRecordDlg);
    } // setupUi

    void retranslateUi(QWidget *DlgPrefRecordDlg)
    {
        DlgPrefRecordDlg->setWindowTitle(QCoreApplication::translate("DlgPrefRecordDlg", "Recording Preferences", nullptr));
        groupBoxRecordingsPath->setTitle(QCoreApplication::translate("DlgPrefRecordDlg", "Recording Files", nullptr));
        TextLabel_Recordings->setText(QCoreApplication::translate("DlgPrefRecordDlg", "Directory:", nullptr));
        PushButtonBrowseRecordings->setText(QCoreApplication::translate("DlgPrefRecordDlg", "Browse...", nullptr));
        LabelSplitting->setText(QCoreApplication::translate("DlgPrefRecordDlg", "Split recordings at", nullptr));
        CheckBoxRecordCueFile->setText(QCoreApplication::translate("DlgPrefRecordDlg", "Create a CUE file", nullptr));
        groupBoxFileformat->setTitle(QCoreApplication::translate("DlgPrefRecordDlg", "Output File Format", nullptr));
        LabelLossless->setText(QCoreApplication::translate("DlgPrefRecordDlg", "Lossless", nullptr));
        LabelLossy->setText(QCoreApplication::translate("DlgPrefRecordDlg", "Lossy", nullptr));
        labelOptionGroup->setText(QString());
        LabelQuality->setText(QCoreApplication::translate("DlgPrefRecordDlg", "Quality", nullptr));
        TextQuality->setText(QCoreApplication::translate("DlgPrefRecordDlg", "Quality", nullptr));
        LabelCompression->setText(QCoreApplication::translate("DlgPrefRecordDlg", "Compression Level", nullptr));
        TextCompression->setText(QCoreApplication::translate("DlgPrefRecordDlg", "Compression", nullptr));
        groupBoxTags->setTitle(QCoreApplication::translate("DlgPrefRecordDlg", "Tags", nullptr));
        LabelTitle->setText(QCoreApplication::translate("DlgPrefRecordDlg", "Title", nullptr));
        LabelAuthor->setText(QCoreApplication::translate("DlgPrefRecordDlg", "Author", nullptr));
        LabelAlbum->setText(QCoreApplication::translate("DlgPrefRecordDlg", "Album", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgPrefRecordDlg: public Ui_DlgPrefRecordDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLGPREFRECORDDLG_H
