/****************************************************************************
** Meta object code from reading C++ file 'basetrackplayer.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/mixer/basetrackplayer.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'basetrackplayer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15BaseTrackPlayerE_t {};
} // unnamed namespace

template <> constexpr inline auto BaseTrackPlayer::qt_create_metaobjectdata<qt_meta_tag_ZN15BaseTrackPlayerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "BaseTrackPlayer",
        "newTrackLoaded",
        "",
        "TrackPointer",
        "pLoadedTrack",
        "trackUnloaded",
        "pUnloadedTrack",
        "loadingTrack",
        "pNewTrack",
        "pOldTrack",
        "playerEmpty",
        "noVinylControlInputConfigured",
        "trackRatingChanged",
        "rating",
        "trackMenuChangeRequest",
        "show",
        "slotLoadTrack",
        "pTrack",
        "bPlay",
        "slotCloneFromGroup",
        "group",
        "slotCloneDeck",
        "slotEjectTrack",
        "slotSetAndConfirmTrackMenuControl",
        "slotTrackRatingChangeRequest"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'newTrackLoaded'
        QtMocHelpers::SignalData<void(TrackPointer)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'trackUnloaded'
        QtMocHelpers::SignalData<void(TrackPointer)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 6 },
        }}),
        // Signal 'loadingTrack'
        QtMocHelpers::SignalData<void(TrackPointer, TrackPointer)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 8 }, { 0x80000000 | 3, 9 },
        }}),
        // Signal 'playerEmpty'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'noVinylControlInputConfigured'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'trackRatingChanged'
        QtMocHelpers::SignalData<void(int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 13 },
        }}),
        // Signal 'trackMenuChangeRequest'
        QtMocHelpers::SignalData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
        // Slot 'slotLoadTrack'
        QtMocHelpers::SlotData<void(TrackPointer, bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 17 }, { QMetaType::Bool, 18 },
        }}),
        // Slot 'slotLoadTrack'
        QtMocHelpers::SlotData<void(TrackPointer)>(16, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 3, 17 },
        }}),
        // Slot 'slotCloneFromGroup'
        QtMocHelpers::SlotData<void(const QString &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 },
        }}),
        // Slot 'slotCloneDeck'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotEjectTrack'
        QtMocHelpers::SlotData<void(double)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotSetAndConfirmTrackMenuControl'
        QtMocHelpers::SlotData<void(bool)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'slotTrackRatingChangeRequest'
        QtMocHelpers::SlotData<void(int)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<BaseTrackPlayer, qt_meta_tag_ZN15BaseTrackPlayerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject BaseTrackPlayer::staticMetaObject = { {
    QMetaObject::SuperData::link<BasePlayer::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15BaseTrackPlayerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15BaseTrackPlayerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15BaseTrackPlayerE_t>.metaTypes,
    nullptr
} };

void BaseTrackPlayer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<BaseTrackPlayer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->newTrackLoaded((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1]))); break;
        case 1: _t->trackUnloaded((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1]))); break;
        case 2: _t->loadingTrack((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[2]))); break;
        case 3: _t->playerEmpty(); break;
        case 4: _t->noVinylControlInputConfigured(); break;
        case 5: _t->trackRatingChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->trackMenuChangeRequest((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->slotLoadTrack((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 8: _t->slotLoadTrack((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1]))); break;
        case 9: _t->slotCloneFromGroup((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->slotCloneDeck(); break;
        case 11: _t->slotEjectTrack((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 12: _t->slotSetAndConfirmTrackMenuControl((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->slotTrackRatingChangeRequest((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (BaseTrackPlayer::*)(TrackPointer )>(_a, &BaseTrackPlayer::newTrackLoaded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (BaseTrackPlayer::*)(TrackPointer )>(_a, &BaseTrackPlayer::trackUnloaded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (BaseTrackPlayer::*)(TrackPointer , TrackPointer )>(_a, &BaseTrackPlayer::loadingTrack, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (BaseTrackPlayer::*)()>(_a, &BaseTrackPlayer::playerEmpty, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (BaseTrackPlayer::*)()>(_a, &BaseTrackPlayer::noVinylControlInputConfigured, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (BaseTrackPlayer::*)(int )>(_a, &BaseTrackPlayer::trackRatingChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (BaseTrackPlayer::*)(bool )>(_a, &BaseTrackPlayer::trackMenuChangeRequest, 6))
            return;
    }
}

const QMetaObject *BaseTrackPlayer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *BaseTrackPlayer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15BaseTrackPlayerE_t>.strings))
        return static_cast<void*>(this);
    return BasePlayer::qt_metacast(_clname);
}

int BaseTrackPlayer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = BasePlayer::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void BaseTrackPlayer::newTrackLoaded(TrackPointer _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void BaseTrackPlayer::trackUnloaded(TrackPointer _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void BaseTrackPlayer::loadingTrack(TrackPointer _t1, TrackPointer _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void BaseTrackPlayer::playerEmpty()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void BaseTrackPlayer::noVinylControlInputConfigured()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void BaseTrackPlayer::trackRatingChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void BaseTrackPlayer::trackMenuChangeRequest(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN19BaseTrackPlayerImplE_t {};
} // unnamed namespace

template <> constexpr inline auto BaseTrackPlayerImpl::qt_create_metaobjectdata<qt_meta_tag_ZN19BaseTrackPlayerImplE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "BaseTrackPlayerImpl",
        "slotLoadTrack",
        "",
        "TrackPointer",
        "track",
        "bPlay",
        "slotEjectTrack",
        "slotCloneFromGroup",
        "group",
        "slotCloneDeck",
        "slotTrackLoaded",
        "pNewTrack",
        "pOldTrack",
        "slotLoadFailed",
        "pTrack",
        "reason",
        "slotSetReplayGain",
        "mixxx::ReplayGain",
        "replayGain",
        "slotAdjustReplayGain",
        "requestingPlayerGroup",
        "slotSetTrackColor",
        "mixxx::RgbColor::optional_t",
        "color",
        "slotTrackColorSelector",
        "steps",
        "slotSetAndConfirmTrackMenuControl",
        "visible",
        "slotTrackRatingChangeRequest",
        "rating",
        "slotPlayToggled",
        "slotCloneChannel",
        "EngineChannel*",
        "pChannel",
        "slotCloneFromDeck",
        "deck",
        "slotCloneFromSampler",
        "sampler",
        "loadTrackFromGroup",
        "slotLoadTrackFromDeck",
        "slotLoadTrackFromSampler",
        "slotTrackColorChangeRequest",
        "value",
        "slotTrackRatingChangeRequestRelative",
        "change",
        "slotWaveformZoomValueChangeRequest",
        "pressed",
        "slotWaveformZoomUp",
        "slotWaveformZoomDown",
        "slotWaveformZoomSetDefault",
        "slotShiftCuesMillis",
        "milliseconds",
        "slotShiftCuesMillisButton",
        "slotUpdateReplayGainFromPregain"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotLoadTrack'
        QtMocHelpers::SlotData<void(TrackPointer, bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Bool, 5 },
        }}),
        // Slot 'slotEjectTrack'
        QtMocHelpers::SlotData<void(double)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotCloneFromGroup'
        QtMocHelpers::SlotData<void(const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Slot 'slotCloneDeck'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotTrackLoaded'
        QtMocHelpers::SlotData<void(TrackPointer, TrackPointer)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 11 }, { 0x80000000 | 3, 12 },
        }}),
        // Slot 'slotLoadFailed'
        QtMocHelpers::SlotData<void(TrackPointer, const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 14 }, { QMetaType::QString, 15 },
        }}),
        // Slot 'slotSetReplayGain'
        QtMocHelpers::SlotData<void(mixxx::ReplayGain)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'slotAdjustReplayGain'
        QtMocHelpers::SlotData<void(mixxx::ReplayGain, const QString &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { QMetaType::QString, 20 },
        }}),
        // Slot 'slotSetTrackColor'
        QtMocHelpers::SlotData<void(const mixxx::RgbColor::optional_t &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'slotTrackColorSelector'
        QtMocHelpers::SlotData<void(int)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 25 },
        }}),
        // Slot 'slotSetAndConfirmTrackMenuControl'
        QtMocHelpers::SlotData<void(bool)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 27 },
        }}),
        // Slot 'slotTrackRatingChangeRequest'
        QtMocHelpers::SlotData<void(int)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 29 },
        }}),
        // Slot 'slotPlayToggled'
        QtMocHelpers::SlotData<void(double)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotCloneChannel'
        QtMocHelpers::SlotData<void(EngineChannel *)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Slot 'slotCloneFromDeck'
        QtMocHelpers::SlotData<void(double)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 35 },
        }}),
        // Slot 'slotCloneFromSampler'
        QtMocHelpers::SlotData<void(double)>(36, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 37 },
        }}),
        // Slot 'loadTrackFromGroup'
        QtMocHelpers::SlotData<void(const QString &)>(38, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Slot 'slotLoadTrackFromDeck'
        QtMocHelpers::SlotData<void(double)>(39, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 35 },
        }}),
        // Slot 'slotLoadTrackFromSampler'
        QtMocHelpers::SlotData<void(double)>(40, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 37 },
        }}),
        // Slot 'slotTrackColorChangeRequest'
        QtMocHelpers::SlotData<void(double)>(41, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 42 },
        }}),
        // Slot 'slotTrackRatingChangeRequestRelative'
        QtMocHelpers::SlotData<void(int)>(43, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 44 },
        }}),
        // Slot 'slotWaveformZoomValueChangeRequest'
        QtMocHelpers::SlotData<void(double)>(45, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 46 },
        }}),
        // Slot 'slotWaveformZoomUp'
        QtMocHelpers::SlotData<void(double)>(47, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 46 },
        }}),
        // Slot 'slotWaveformZoomDown'
        QtMocHelpers::SlotData<void(double)>(48, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 46 },
        }}),
        // Slot 'slotWaveformZoomSetDefault'
        QtMocHelpers::SlotData<void(double)>(49, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 46 },
        }}),
        // Slot 'slotShiftCuesMillis'
        QtMocHelpers::SlotData<void(double)>(50, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 51 },
        }}),
        // Slot 'slotShiftCuesMillisButton'
        QtMocHelpers::SlotData<void(double, double)>(52, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 42 }, { QMetaType::Double, 51 },
        }}),
        // Slot 'slotUpdateReplayGainFromPregain'
        QtMocHelpers::SlotData<void(double)>(53, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 46 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<BaseTrackPlayerImpl, qt_meta_tag_ZN19BaseTrackPlayerImplE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject BaseTrackPlayerImpl::staticMetaObject = { {
    QMetaObject::SuperData::link<BaseTrackPlayer::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19BaseTrackPlayerImplE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19BaseTrackPlayerImplE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19BaseTrackPlayerImplE_t>.metaTypes,
    nullptr
} };

void BaseTrackPlayerImpl::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<BaseTrackPlayerImpl *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotLoadTrack((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 1: _t->slotEjectTrack((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 2: _t->slotCloneFromGroup((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->slotCloneDeck(); break;
        case 4: _t->slotTrackLoaded((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[2]))); break;
        case 5: _t->slotLoadFailed((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 6: _t->slotSetReplayGain((*reinterpret_cast<std::add_pointer_t<mixxx::ReplayGain>>(_a[1]))); break;
        case 7: _t->slotAdjustReplayGain((*reinterpret_cast<std::add_pointer_t<mixxx::ReplayGain>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 8: _t->slotSetTrackColor((*reinterpret_cast<std::add_pointer_t<mixxx::RgbColor::optional_t>>(_a[1]))); break;
        case 9: _t->slotTrackColorSelector((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->slotSetAndConfirmTrackMenuControl((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->slotTrackRatingChangeRequest((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->slotPlayToggled((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 13: _t->slotCloneChannel((*reinterpret_cast<std::add_pointer_t<EngineChannel*>>(_a[1]))); break;
        case 14: _t->slotCloneFromDeck((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 15: _t->slotCloneFromSampler((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 16: _t->loadTrackFromGroup((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 17: _t->slotLoadTrackFromDeck((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 18: _t->slotLoadTrackFromSampler((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 19: _t->slotTrackColorChangeRequest((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 20: _t->slotTrackRatingChangeRequestRelative((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 21: _t->slotWaveformZoomValueChangeRequest((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 22: _t->slotWaveformZoomUp((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 23: _t->slotWaveformZoomDown((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 24: _t->slotWaveformZoomSetDefault((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 25: _t->slotShiftCuesMillis((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 26: _t->slotShiftCuesMillisButton((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 27: _t->slotUpdateReplayGainFromPregain((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< mixxx::ReplayGain >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< mixxx::ReplayGain >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< EngineChannel* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *BaseTrackPlayerImpl::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *BaseTrackPlayerImpl::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19BaseTrackPlayerImplE_t>.strings))
        return static_cast<void*>(this);
    return BaseTrackPlayer::qt_metacast(_clname);
}

int BaseTrackPlayerImpl::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = BaseTrackPlayer::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 28)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 28;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 28)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 28;
    }
    return _id;
}
QT_WARNING_POP
