/****************************************************************************
** Meta object code from reading C++ file 'dlgcoverartfullsize.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/library/dlgcoverartfullsize.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dlgcoverartfullsize.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19DlgCoverArtFullSizeE_t {};
} // unnamed namespace

template <> constexpr inline auto DlgCoverArtFullSize::qt_create_metaobjectdata<qt_meta_tag_ZN19DlgCoverArtFullSizeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DlgCoverArtFullSize",
        "slotLoadTrack",
        "",
        "TrackPointer",
        "slotCoverFound",
        "const QObject*",
        "pRequester",
        "CoverInfo",
        "coverInfo",
        "QPixmap",
        "pixmap",
        "slotTrackCoverArtUpdated",
        "adjustImageAndDialogSize",
        "slotCoverMenu",
        "QPoint",
        "pos",
        "slotCoverInfoSelected",
        "CoverInfoRelative",
        "slotReloadCoverArt"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotLoadTrack'
        QtMocHelpers::SlotData<void(TrackPointer)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Slot 'slotCoverFound'
        QtMocHelpers::SlotData<void(const QObject *, const CoverInfo &, const QPixmap &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 7, 8 }, { 0x80000000 | 9, 10 },
        }}),
        // Slot 'slotTrackCoverArtUpdated'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'adjustImageAndDialogSize'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotCoverMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'slotCoverInfoSelected'
        QtMocHelpers::SlotData<void(const CoverInfoRelative &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 8 },
        }}),
        // Slot 'slotReloadCoverArt'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DlgCoverArtFullSize, qt_meta_tag_ZN19DlgCoverArtFullSizeE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DlgCoverArtFullSize::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19DlgCoverArtFullSizeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19DlgCoverArtFullSizeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19DlgCoverArtFullSizeE_t>.metaTypes,
    nullptr
} };

void DlgCoverArtFullSize::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DlgCoverArtFullSize *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotLoadTrack((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1]))); break;
        case 1: _t->slotCoverFound((*reinterpret_cast<std::add_pointer_t<const QObject*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CoverInfo>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QPixmap>>(_a[3]))); break;
        case 2: _t->slotTrackCoverArtUpdated(); break;
        case 3: _t->adjustImageAndDialogSize(); break;
        case 4: _t->slotCoverMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 5: _t->slotCoverInfoSelected((*reinterpret_cast<std::add_pointer_t<CoverInfoRelative>>(_a[1]))); break;
        case 6: _t->slotReloadCoverArt(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        }
    }
}

const QMetaObject *DlgCoverArtFullSize::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DlgCoverArtFullSize::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19DlgCoverArtFullSizeE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::DlgCoverArtFullSize"))
        return static_cast< Ui::DlgCoverArtFullSize*>(this);
    return QDialog::qt_metacast(_clname);
}

int DlgCoverArtFullSize::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}
QT_WARNING_POP
