/********************************************************************************
** Form generated from reading UI file 'dlgprefvinyldlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLGPREFVINYLDLG_H
#define UI_DLGPREFVINYLDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DlgPrefVinylDlg
{
public:
    QGridLayout *VinylPrefLayout;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout;
    QLabel *textLabel1_3;
    QSlider *SliderVinylGain;
    QHBoxLayout *horizontalLayout;
    QLabel *textLabelPreampMin;
    QSpacerItem *horizontalSpacer_1;
    QLabel *textLabelPreampCurrent;
    QSpacerItem *horizontalSpacer_2;
    QLabel *textLabelPreampMax;
    QGroupBox *vinylConfigGroupBox;
    QGridLayout *gridLayout_2;
    QLabel *VinylTypeLabel;
    QLabel *LeadInLabel;
    QLabel *PitchEstimatorLabel;
    QLabel *VinylLabel1;
    QLabel *VinylLabel2;
    QLabel *VinylLabel3;
    QLabel *VinylLabel4;
    QComboBox *ComboBoxVinylType1;
    QComboBox *ComboBoxVinylType2;
    QComboBox *ComboBoxVinylType3;
    QComboBox *ComboBoxVinylType4;
    QComboBox *ComboBoxVinylSpeed1;
    QComboBox *ComboBoxVinylSpeed2;
    QComboBox *ComboBoxVinylSpeed3;
    QComboBox *ComboBoxVinylSpeed4;
    QSpinBox *LeadinTime1;
    QSpinBox *LeadinTime2;
    QSpinBox *LeadinTime3;
    QSpinBox *LeadinTime4;
    QComboBox *ComboBoxPitchEstimator1;
    QComboBox *ComboBoxPitchEstimator2;
    QComboBox *ComboBoxPitchEstimator3;
    QComboBox *ComboBoxPitchEstimator4;
    QSpacerItem *horizontalSpacer_3;
    QGroupBox *groupBoxSignalQuality;
    QHBoxLayout *signalQualityLayout;
    QGroupBox *groupBoxSignalQuality2;
    QGridLayout *gridLayout;
    QCheckBox *SignalQualityEnable;
    QGroupBox *Hints;
    QGridLayout *gridLayout_6;
    QLabel *TroubleshootingLink;
    QLabel *label_11;
    QLabel *label;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *DlgPrefVinylDlg)
    {
        if (DlgPrefVinylDlg->objectName().isEmpty())
            DlgPrefVinylDlg->setObjectName("DlgPrefVinylDlg");
        DlgPrefVinylDlg->resize(502, 654);
        VinylPrefLayout = new QGridLayout(DlgPrefVinylDlg);
        VinylPrefLayout->setSpacing(6);
        VinylPrefLayout->setContentsMargins(11, 11, 11, 11);
        VinylPrefLayout->setObjectName("VinylPrefLayout");
        groupBox_2 = new QGroupBox(DlgPrefVinylDlg);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout = new QVBoxLayout(groupBox_2);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        textLabel1_3 = new QLabel(groupBox_2);
        textLabel1_3->setObjectName("textLabel1_3");
        textLabel1_3->setWordWrap(false);

        verticalLayout->addWidget(textLabel1_3);

        SliderVinylGain = new QSlider(groupBox_2);
        SliderVinylGain->setObjectName("SliderVinylGain");
        SliderVinylGain->setMinimum(0);
        SliderVinylGain->setMaximum(44);
        SliderVinylGain->setOrientation(Qt::Horizontal);

        verticalLayout->addWidget(SliderVinylGain);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        textLabelPreampMin = new QLabel(groupBox_2);
        textLabelPreampMin->setObjectName("textLabelPreampMin");
        textLabelPreampMin->setWordWrap(false);

        horizontalLayout->addWidget(textLabelPreampMin);

        horizontalSpacer_1 = new QSpacerItem(30, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_1);

        textLabelPreampCurrent = new QLabel(groupBox_2);
        textLabelPreampCurrent->setObjectName("textLabelPreampCurrent");
        textLabelPreampCurrent->setText(QString::fromUtf8("TextLabel"));

        horizontalLayout->addWidget(textLabelPreampCurrent);

        horizontalSpacer_2 = new QSpacerItem(30, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);

        textLabelPreampMax = new QLabel(groupBox_2);
        textLabelPreampMax->setObjectName("textLabelPreampMax");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(textLabelPreampMax->sizePolicy().hasHeightForWidth());
        textLabelPreampMax->setSizePolicy(sizePolicy);
        textLabelPreampMax->setWordWrap(false);

        horizontalLayout->addWidget(textLabelPreampMax);


        verticalLayout->addLayout(horizontalLayout);


        VinylPrefLayout->addWidget(groupBox_2, 0, 0, 1, 2);

        vinylConfigGroupBox = new QGroupBox(DlgPrefVinylDlg);
        vinylConfigGroupBox->setObjectName("vinylConfigGroupBox");
        gridLayout_2 = new QGridLayout(vinylConfigGroupBox);
        gridLayout_2->setSpacing(6);
        gridLayout_2->setContentsMargins(11, 11, 11, 11);
        gridLayout_2->setObjectName("gridLayout_2");
        VinylTypeLabel = new QLabel(vinylConfigGroupBox);
        VinylTypeLabel->setObjectName("VinylTypeLabel");
        VinylTypeLabel->setEnabled(true);
        sizePolicy.setHeightForWidth(VinylTypeLabel->sizePolicy().hasHeightForWidth());
        VinylTypeLabel->setSizePolicy(sizePolicy);
        VinylTypeLabel->setWordWrap(false);
        VinylTypeLabel->setContentsMargins(3, -1, -1, -1);

        gridLayout_2->addWidget(VinylTypeLabel, 0, 1, 1, 2);

        LeadInLabel = new QLabel(vinylConfigGroupBox);
        LeadInLabel->setObjectName("LeadInLabel");
        LeadInLabel->setEnabled(true);
        sizePolicy.setHeightForWidth(LeadInLabel->sizePolicy().hasHeightForWidth());
        LeadInLabel->setSizePolicy(sizePolicy);
        LeadInLabel->setWordWrap(false);
        LeadInLabel->setContentsMargins(3, -1, -1, -1);

        gridLayout_2->addWidget(LeadInLabel, 0, 3, 1, 1);

        PitchEstimatorLabel = new QLabel(vinylConfigGroupBox);
        PitchEstimatorLabel->setObjectName("PitchEstimatorLabel");
        PitchEstimatorLabel->setEnabled(true);
        sizePolicy.setHeightForWidth(PitchEstimatorLabel->sizePolicy().hasHeightForWidth());
        PitchEstimatorLabel->setSizePolicy(sizePolicy);
        PitchEstimatorLabel->setWordWrap(false);
        PitchEstimatorLabel->setContentsMargins(3, -1, -1, -1);

        gridLayout_2->addWidget(PitchEstimatorLabel, 0, 4, 1, 1);

        VinylLabel1 = new QLabel(vinylConfigGroupBox);
        VinylLabel1->setObjectName("VinylLabel1");
        VinylLabel1->setEnabled(true);
        sizePolicy.setHeightForWidth(VinylLabel1->sizePolicy().hasHeightForWidth());
        VinylLabel1->setSizePolicy(sizePolicy);
        QFont font;
        VinylLabel1->setFont(font);
        VinylLabel1->setWordWrap(false);

        gridLayout_2->addWidget(VinylLabel1, 1, 0, 1, 1);

        VinylLabel2 = new QLabel(vinylConfigGroupBox);
        VinylLabel2->setObjectName("VinylLabel2");
        VinylLabel2->setEnabled(true);
        sizePolicy.setHeightForWidth(VinylLabel2->sizePolicy().hasHeightForWidth());
        VinylLabel2->setSizePolicy(sizePolicy);
        VinylLabel2->setFont(font);
        VinylLabel2->setWordWrap(false);

        gridLayout_2->addWidget(VinylLabel2, 2, 0, 1, 1);

        VinylLabel3 = new QLabel(vinylConfigGroupBox);
        VinylLabel3->setObjectName("VinylLabel3");
        VinylLabel3->setEnabled(true);
        sizePolicy.setHeightForWidth(VinylLabel3->sizePolicy().hasHeightForWidth());
        VinylLabel3->setSizePolicy(sizePolicy);
        VinylLabel3->setFont(font);
        VinylLabel3->setWordWrap(false);

        gridLayout_2->addWidget(VinylLabel3, 3, 0, 1, 1);

        VinylLabel4 = new QLabel(vinylConfigGroupBox);
        VinylLabel4->setObjectName("VinylLabel4");
        VinylLabel4->setEnabled(true);
        sizePolicy.setHeightForWidth(VinylLabel4->sizePolicy().hasHeightForWidth());
        VinylLabel4->setSizePolicy(sizePolicy);
        VinylLabel4->setFont(font);
        VinylLabel4->setWordWrap(false);

        gridLayout_2->addWidget(VinylLabel4, 4, 0, 1, 1);

        ComboBoxVinylType1 = new QComboBox(vinylConfigGroupBox);
        ComboBoxVinylType1->setObjectName("ComboBoxVinylType1");
        ComboBoxVinylType1->setEnabled(true);
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(ComboBoxVinylType1->sizePolicy().hasHeightForWidth());
        ComboBoxVinylType1->setSizePolicy(sizePolicy1);
        ComboBoxVinylType1->setFont(font);

        gridLayout_2->addWidget(ComboBoxVinylType1, 1, 1, 1, 1);

        ComboBoxVinylType2 = new QComboBox(vinylConfigGroupBox);
        ComboBoxVinylType2->setObjectName("ComboBoxVinylType2");
        ComboBoxVinylType2->setEnabled(true);
        sizePolicy1.setHeightForWidth(ComboBoxVinylType2->sizePolicy().hasHeightForWidth());
        ComboBoxVinylType2->setSizePolicy(sizePolicy1);
        ComboBoxVinylType2->setFont(font);

        gridLayout_2->addWidget(ComboBoxVinylType2, 2, 1, 1, 1);

        ComboBoxVinylType3 = new QComboBox(vinylConfigGroupBox);
        ComboBoxVinylType3->setObjectName("ComboBoxVinylType3");
        ComboBoxVinylType3->setEnabled(true);
        sizePolicy1.setHeightForWidth(ComboBoxVinylType3->sizePolicy().hasHeightForWidth());
        ComboBoxVinylType3->setSizePolicy(sizePolicy1);
        ComboBoxVinylType3->setFont(font);

        gridLayout_2->addWidget(ComboBoxVinylType3, 3, 1, 1, 1);

        ComboBoxVinylType4 = new QComboBox(vinylConfigGroupBox);
        ComboBoxVinylType4->setObjectName("ComboBoxVinylType4");
        ComboBoxVinylType4->setEnabled(true);
        sizePolicy1.setHeightForWidth(ComboBoxVinylType4->sizePolicy().hasHeightForWidth());
        ComboBoxVinylType4->setSizePolicy(sizePolicy1);
        ComboBoxVinylType4->setFont(font);

        gridLayout_2->addWidget(ComboBoxVinylType4, 4, 1, 1, 1);

        ComboBoxVinylSpeed1 = new QComboBox(vinylConfigGroupBox);
        ComboBoxVinylSpeed1->setObjectName("ComboBoxVinylSpeed1");
        ComboBoxVinylSpeed1->setEnabled(true);
        sizePolicy1.setHeightForWidth(ComboBoxVinylSpeed1->sizePolicy().hasHeightForWidth());
        ComboBoxVinylSpeed1->setSizePolicy(sizePolicy1);
        ComboBoxVinylSpeed1->setFont(font);

        gridLayout_2->addWidget(ComboBoxVinylSpeed1, 1, 2, 1, 1);

        ComboBoxVinylSpeed2 = new QComboBox(vinylConfigGroupBox);
        ComboBoxVinylSpeed2->setObjectName("ComboBoxVinylSpeed2");
        ComboBoxVinylSpeed2->setEnabled(true);
        sizePolicy1.setHeightForWidth(ComboBoxVinylSpeed2->sizePolicy().hasHeightForWidth());
        ComboBoxVinylSpeed2->setSizePolicy(sizePolicy1);
        ComboBoxVinylSpeed2->setFont(font);

        gridLayout_2->addWidget(ComboBoxVinylSpeed2, 2, 2, 1, 1);

        ComboBoxVinylSpeed3 = new QComboBox(vinylConfigGroupBox);
        ComboBoxVinylSpeed3->setObjectName("ComboBoxVinylSpeed3");
        ComboBoxVinylSpeed3->setEnabled(true);
        sizePolicy1.setHeightForWidth(ComboBoxVinylSpeed3->sizePolicy().hasHeightForWidth());
        ComboBoxVinylSpeed3->setSizePolicy(sizePolicy1);
        ComboBoxVinylSpeed3->setFont(font);

        gridLayout_2->addWidget(ComboBoxVinylSpeed3, 3, 2, 1, 1);

        ComboBoxVinylSpeed4 = new QComboBox(vinylConfigGroupBox);
        ComboBoxVinylSpeed4->setObjectName("ComboBoxVinylSpeed4");
        ComboBoxVinylSpeed4->setEnabled(true);
        sizePolicy1.setHeightForWidth(ComboBoxVinylSpeed4->sizePolicy().hasHeightForWidth());
        ComboBoxVinylSpeed4->setSizePolicy(sizePolicy1);
        ComboBoxVinylSpeed4->setFont(font);

        gridLayout_2->addWidget(ComboBoxVinylSpeed4, 4, 2, 1, 1);

        LeadinTime1 = new QSpinBox(vinylConfigGroupBox);
        LeadinTime1->setObjectName("LeadinTime1");
        LeadinTime1->setMinimum(0);
        sizePolicy1.setHeightForWidth(LeadinTime1->sizePolicy().hasHeightForWidth());
        LeadinTime1->setSizePolicy(sizePolicy1);
        LeadinTime1->setMinimumSize(QSize(55, 0));
        LeadinTime1->setMaximumSize(QSize(70, 16777215));
        LeadinTime1->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(LeadinTime1, 1, 3, 1, 1);

        LeadinTime2 = new QSpinBox(vinylConfigGroupBox);
        LeadinTime2->setObjectName("LeadinTime2");
        LeadinTime2->setMinimum(0);
        sizePolicy1.setHeightForWidth(LeadinTime2->sizePolicy().hasHeightForWidth());
        LeadinTime2->setSizePolicy(sizePolicy1);
        LeadinTime2->setMinimumSize(QSize(55, 0));
        LeadinTime2->setMaximumSize(QSize(70, 16777215));
        LeadinTime2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(LeadinTime2, 2, 3, 1, 1);

        LeadinTime3 = new QSpinBox(vinylConfigGroupBox);
        LeadinTime3->setObjectName("LeadinTime3");
        LeadinTime3->setMinimum(0);
        sizePolicy1.setHeightForWidth(LeadinTime3->sizePolicy().hasHeightForWidth());
        LeadinTime3->setSizePolicy(sizePolicy1);
        LeadinTime3->setMinimumSize(QSize(55, 0));
        LeadinTime3->setMaximumSize(QSize(70, 16777215));
        LeadinTime3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(LeadinTime3, 3, 3, 1, 1);

        LeadinTime4 = new QSpinBox(vinylConfigGroupBox);
        LeadinTime4->setObjectName("LeadinTime4");
        LeadinTime4->setMinimum(0);
        sizePolicy1.setHeightForWidth(LeadinTime4->sizePolicy().hasHeightForWidth());
        LeadinTime4->setSizePolicy(sizePolicy1);
        LeadinTime4->setMinimumSize(QSize(55, 0));
        LeadinTime4->setMaximumSize(QSize(70, 16777215));
        LeadinTime4->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_2->addWidget(LeadinTime4, 4, 3, 1, 1);

        ComboBoxPitchEstimator1 = new QComboBox(vinylConfigGroupBox);
        ComboBoxPitchEstimator1->setObjectName("ComboBoxPitchEstimator1");
        ComboBoxPitchEstimator1->setEnabled(true);
        sizePolicy1.setHeightForWidth(ComboBoxPitchEstimator1->sizePolicy().hasHeightForWidth());
        ComboBoxPitchEstimator1->setSizePolicy(sizePolicy1);
        ComboBoxPitchEstimator1->setFont(font);
        ComboBoxPitchEstimator1->setMinimumSize(QSize(100, 0));
        ComboBoxPitchEstimator1->setMaximumSize(QSize(200, 16777215));

        gridLayout_2->addWidget(ComboBoxPitchEstimator1, 1, 4, 1, 1);

        ComboBoxPitchEstimator2 = new QComboBox(vinylConfigGroupBox);
        ComboBoxPitchEstimator2->setObjectName("ComboBoxPitchEstimator2");
        ComboBoxPitchEstimator2->setEnabled(true);
        sizePolicy1.setHeightForWidth(ComboBoxPitchEstimator2->sizePolicy().hasHeightForWidth());
        ComboBoxPitchEstimator2->setSizePolicy(sizePolicy1);
        ComboBoxPitchEstimator2->setFont(font);
        ComboBoxPitchEstimator2->setMinimumSize(QSize(100, 0));
        ComboBoxPitchEstimator2->setMaximumSize(QSize(200, 16777215));

        gridLayout_2->addWidget(ComboBoxPitchEstimator2, 2, 4, 1, 1);

        ComboBoxPitchEstimator3 = new QComboBox(vinylConfigGroupBox);
        ComboBoxPitchEstimator3->setObjectName("ComboBoxPitchEstimator3");
        ComboBoxPitchEstimator3->setEnabled(true);
        sizePolicy1.setHeightForWidth(ComboBoxPitchEstimator3->sizePolicy().hasHeightForWidth());
        ComboBoxPitchEstimator3->setSizePolicy(sizePolicy1);
        ComboBoxPitchEstimator3->setFont(font);
        ComboBoxPitchEstimator3->setMinimumSize(QSize(100, 0));
        ComboBoxPitchEstimator3->setMaximumSize(QSize(200, 16777215));

        gridLayout_2->addWidget(ComboBoxPitchEstimator3, 3, 4, 1, 1);

        ComboBoxPitchEstimator4 = new QComboBox(vinylConfigGroupBox);
        ComboBoxPitchEstimator4->setObjectName("ComboBoxPitchEstimator4");
        ComboBoxPitchEstimator4->setEnabled(true);
        sizePolicy1.setHeightForWidth(ComboBoxPitchEstimator4->sizePolicy().hasHeightForWidth());
        ComboBoxPitchEstimator4->setSizePolicy(sizePolicy1);
        ComboBoxPitchEstimator4->setFont(font);
        ComboBoxPitchEstimator4->setMinimumSize(QSize(100, 0));
        ComboBoxPitchEstimator4->setMaximumSize(QSize(200, 16777215));

        gridLayout_2->addWidget(ComboBoxPitchEstimator4, 4, 4, 1, 1);

        horizontalSpacer_3 = new QSpacerItem(0, 0, QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Minimum);

        gridLayout_2->addItem(horizontalSpacer_3, 1, 4, 4, 1);


        VinylPrefLayout->addWidget(vinylConfigGroupBox, 1, 0, 1, 2);

        groupBoxSignalQuality = new QGroupBox(DlgPrefVinylDlg);
        groupBoxSignalQuality->setObjectName("groupBoxSignalQuality");
        groupBoxSignalQuality->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(groupBoxSignalQuality->sizePolicy().hasHeightForWidth());
        groupBoxSignalQuality->setSizePolicy(sizePolicy2);
        signalQualityLayout = new QHBoxLayout(groupBoxSignalQuality);
        signalQualityLayout->setSpacing(6);
        signalQualityLayout->setContentsMargins(11, 11, 11, 11);
        signalQualityLayout->setObjectName("signalQualityLayout");

        VinylPrefLayout->addWidget(groupBoxSignalQuality, 2, 0, 1, 1);

        groupBoxSignalQuality2 = new QGroupBox(DlgPrefVinylDlg);
        groupBoxSignalQuality2->setObjectName("groupBoxSignalQuality2");
        groupBoxSignalQuality2->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        groupBoxSignalQuality2->setContentsMargins(6, 0, 0, 0);
        gridLayout = new QGridLayout(groupBoxSignalQuality2);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        SignalQualityEnable = new QCheckBox(groupBoxSignalQuality2);
        SignalQualityEnable->setObjectName("SignalQualityEnable");

        gridLayout->addWidget(SignalQualityEnable, 0, 0, 1, 1);


        VinylPrefLayout->addWidget(groupBoxSignalQuality2, 3, 0, 1, 2);

        Hints = new QGroupBox(DlgPrefVinylDlg);
        Hints->setObjectName("Hints");
        gridLayout_6 = new QGridLayout(Hints);
        gridLayout_6->setSpacing(6);
        gridLayout_6->setContentsMargins(11, 11, 11, 11);
        gridLayout_6->setObjectName("gridLayout_6");
        TroubleshootingLink = new QLabel(Hints);
        TroubleshootingLink->setObjectName("TroubleshootingLink");
        TroubleshootingLink->setWordWrap(true);
        TroubleshootingLink->setOpenExternalLinks(true);

        gridLayout_6->addWidget(TroubleshootingLink, 4, 0, 1, 1);

        label_11 = new QLabel(Hints);
        label_11->setObjectName("label_11");
        label_11->setWordWrap(true);

        gridLayout_6->addWidget(label_11, 0, 0, 1, 1);


        VinylPrefLayout->addWidget(Hints, 4, 0, 1, 2);

        label = new QLabel(DlgPrefVinylDlg);
        label->setObjectName("label");
        label->setEnabled(false);
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy3);
        label->setAlignment(Qt::AlignBottom|Qt::AlignLeading|Qt::AlignRight);

        VinylPrefLayout->addWidget(label, 5, 0, 1, 2);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        VinylPrefLayout->addItem(verticalSpacer, 6, 2, 1, 1);

#if QT_CONFIG(shortcut)
        textLabel1_3->setBuddy(SliderVinylGain);
        VinylLabel1->setBuddy(ComboBoxVinylType1);
        VinylLabel2->setBuddy(ComboBoxVinylType2);
        VinylLabel3->setBuddy(ComboBoxVinylType3);
        VinylLabel4->setBuddy(ComboBoxVinylType4);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(SliderVinylGain, ComboBoxVinylType1);
        QWidget::setTabOrder(ComboBoxVinylType1, ComboBoxVinylSpeed1);
        QWidget::setTabOrder(ComboBoxVinylSpeed1, LeadinTime1);
        QWidget::setTabOrder(LeadinTime1, ComboBoxVinylType2);
        QWidget::setTabOrder(ComboBoxVinylType2, ComboBoxVinylSpeed2);
        QWidget::setTabOrder(ComboBoxVinylSpeed2, LeadinTime2);
        QWidget::setTabOrder(LeadinTime2, ComboBoxVinylType3);
        QWidget::setTabOrder(ComboBoxVinylType3, ComboBoxVinylSpeed3);
        QWidget::setTabOrder(ComboBoxVinylSpeed3, LeadinTime3);
        QWidget::setTabOrder(LeadinTime3, ComboBoxVinylType4);
        QWidget::setTabOrder(ComboBoxVinylType4, ComboBoxVinylSpeed4);
        QWidget::setTabOrder(ComboBoxVinylSpeed4, LeadinTime4);

        retranslateUi(DlgPrefVinylDlg);

        QMetaObject::connectSlotsByName(DlgPrefVinylDlg);
    } // setupUi

    void retranslateUi(QWidget *DlgPrefVinylDlg)
    {
        DlgPrefVinylDlg->setWindowTitle(QCoreApplication::translate("DlgPrefVinylDlg", "Vinyl Control Preferences", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("DlgPrefVinylDlg", "Input", nullptr));
        textLabel1_3->setText(QCoreApplication::translate("DlgPrefVinylDlg", "Turntable Input Signal Boost", nullptr));
        textLabelPreampMin->setText(QCoreApplication::translate("DlgPrefVinylDlg", "0 dB", nullptr));
        textLabelPreampMax->setText(QCoreApplication::translate("DlgPrefVinylDlg", "44 dB", nullptr));
        vinylConfigGroupBox->setTitle(QCoreApplication::translate("DlgPrefVinylDlg", "Vinyl Configuration", nullptr));
        VinylTypeLabel->setText(QCoreApplication::translate("DlgPrefVinylDlg", "Vinyl Type", nullptr));
        LeadInLabel->setText(QCoreApplication::translate("DlgPrefVinylDlg", "Lead-In", nullptr));
        PitchEstimatorLabel->setText(QCoreApplication::translate("DlgPrefVinylDlg", "Pitch estimator", nullptr));
        VinylLabel1->setText(QCoreApplication::translate("DlgPrefVinylDlg", "Deck 1", nullptr));
        VinylLabel2->setText(QCoreApplication::translate("DlgPrefVinylDlg", "Deck 2", nullptr));
        VinylLabel3->setText(QCoreApplication::translate("DlgPrefVinylDlg", "Deck 3", nullptr));
        VinylLabel4->setText(QCoreApplication::translate("DlgPrefVinylDlg", "Deck 4", nullptr));
        groupBoxSignalQuality->setTitle(QCoreApplication::translate("DlgPrefVinylDlg", "Signal Quality", nullptr));
        SignalQualityEnable->setText(QCoreApplication::translate("DlgPrefVinylDlg", "Show Signal Quality in Skin", nullptr));
        Hints->setTitle(QCoreApplication::translate("DlgPrefVinylDlg", "Hints", nullptr));
        label_11->setText(QCoreApplication::translate("DlgPrefVinylDlg", "Select sound devices for Vinyl Control in the Sound Hardware pane.", nullptr));
#if QT_CONFIG(tooltip)
        label->setToolTip(QCoreApplication::translate("DlgPrefVinylDlg", "http://www.xwax.co.uk", nullptr));
#endif // QT_CONFIG(tooltip)
        label->setText(QCoreApplication::translate("DlgPrefVinylDlg", "Powered by xwax", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgPrefVinylDlg: public Ui_DlgPrefVinylDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLGPREFVINYLDLG_H
