// This file is autogenerated by CMake. Do not edit.

#include <QtCore/qtsymbolmacros.h>
#include <QtQml/qqmlextensionplugin.h>


QT_DECLARE_EXTERN_RESOURCE(qmlcache_mixxx_qml_mixxxcontrols)
QT_DECLARE_EXTERN_RESOURCE(qmake_Mixxx_Controls)
QT_DECLARE_EXTERN_RESOURCE(mixxx_qml_mixxxcontrols_raw_qml_0)


QT_DECLARE_EXTERN_SYMBOL_VOID(qml_register_types_Mixxx_Controls)

class Mixxx_ControlsPlugin : public QQmlEngineExtensionPlugin
{
    Q_OBJECT
    Q_PLUGIN_METADATA(IID QQmlEngineExtensionInterface_iid)

public:
    Mixxx_ControlsPlugin(QObject *parent = nullptr) : QQmlEngineExtensionPlugin(parent)
    {

QT_KEEP_SYMBOL(qml_register_types_Mixxx_Controls)
QT_KEEP_RESOURCE(qmlcache_mixxx_qml_mixxxcontrols)
QT_KEEP_RESOURCE(qmake_Mixxx_Controls)
QT_KEEP_RESOURCE(mixxx_qml_mixxxcontrols_raw_qml_0)
    }
};



#include "mixxx-qml-mixxxcontrolsplugin_Mixxx_ControlsPlugin.moc"
