/****************************************************************************
** Meta object code from reading C++ file 'qmlplayerproxy.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/qml/qmlplayerproxy.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qmlplayerproxy.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN5mixxx3qml14QmlPlayerProxyE_t {};
} // unnamed namespace

template <> constexpr inline auto mixxx::qml::QmlPlayerProxy::qt_create_metaobjectdata<qt_meta_tag_ZN5mixxx3qml14QmlPlayerProxyE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "mixxx::qml::QmlPlayerProxy",
        "QML.Element",
        "Player",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Only accessible via Mixxx.PlayerManager.getPlayer(group)",
        "trackLoading",
        "",
        "trackLoaded",
        "trackUnloaded",
        "trackChanged",
        "cloneFromGroup",
        "group",
        "albumChanged",
        "titleChanged",
        "artistChanged",
        "albumArtistChanged",
        "genreChanged",
        "composerChanged",
        "groupingChanged",
        "yearChanged",
        "trackNumberChanged",
        "trackTotalChanged",
        "commentChanged",
        "keyTextChanged",
        "colorChanged",
        "coverArtUrlChanged",
        "trackLocationUrlChanged",
        "cuesChanged",
        "loadTrackFromLocationRequested",
        "trackLocation",
        "play",
        "waveformLengthChanged",
        "waveformTextureChanged",
        "waveformTextureSizeChanged",
        "waveformTextureStrideChanged",
        "slotTrackLoaded",
        "TrackPointer",
        "pTrack",
        "slotLoadingTrack",
        "pNewTrack",
        "pOldTrack",
        "slotTrackChanged",
        "slotWaveformChanged",
        "slotBeatsChanged",
        "slotHotcuesChanged",
        "setArtist",
        "artist",
        "setTitle",
        "title",
        "setAlbum",
        "album",
        "setAlbumArtist",
        "albumArtist",
        "setComposer",
        "composer",
        "setGrouping",
        "grouping",
        "setYear",
        "year",
        "setTrackNumber",
        "trackNumber",
        "setTrackTotal",
        "trackTotal",
        "setComment",
        "comment",
        "setKeyText",
        "keyText",
        "setColor",
        "QColor",
        "color",
        "loadTrackFromLocation",
        "loadTrackFromLocationUrl",
        "QUrl",
        "trackLocationUrl",
        "isLoaded",
        "genre",
        "coverArtUrl",
        "waveformLength",
        "waveformTexture",
        "waveformTextureSize",
        "waveformTextureStride",
        "beatsModel",
        "mixxx::qml::QmlBeatsModel*",
        "hotcuesModel",
        "mixxx::qml::QmlCuesModel*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'trackLoading'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'trackLoaded'
        QtMocHelpers::SignalData<void()>(9, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'trackUnloaded'
        QtMocHelpers::SignalData<void()>(10, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'trackChanged'
        QtMocHelpers::SignalData<void()>(11, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cloneFromGroup'
        QtMocHelpers::SignalData<void(const QString &)>(12, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Signal 'albumChanged'
        QtMocHelpers::SignalData<void()>(14, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'titleChanged'
        QtMocHelpers::SignalData<void()>(15, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'artistChanged'
        QtMocHelpers::SignalData<void()>(16, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'albumArtistChanged'
        QtMocHelpers::SignalData<void()>(17, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'genreChanged'
        QtMocHelpers::SignalData<void()>(18, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'composerChanged'
        QtMocHelpers::SignalData<void()>(19, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'groupingChanged'
        QtMocHelpers::SignalData<void()>(20, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'yearChanged'
        QtMocHelpers::SignalData<void()>(21, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'trackNumberChanged'
        QtMocHelpers::SignalData<void()>(22, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'trackTotalChanged'
        QtMocHelpers::SignalData<void()>(23, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'commentChanged'
        QtMocHelpers::SignalData<void()>(24, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'keyTextChanged'
        QtMocHelpers::SignalData<void()>(25, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'colorChanged'
        QtMocHelpers::SignalData<void()>(26, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'coverArtUrlChanged'
        QtMocHelpers::SignalData<void()>(27, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'trackLocationUrlChanged'
        QtMocHelpers::SignalData<void()>(28, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cuesChanged'
        QtMocHelpers::SignalData<void()>(29, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'loadTrackFromLocationRequested'
        QtMocHelpers::SignalData<void(const QString &, bool)>(30, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 31 }, { QMetaType::Bool, 32 },
        }}),
        // Signal 'waveformLengthChanged'
        QtMocHelpers::SignalData<void()>(33, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'waveformTextureChanged'
        QtMocHelpers::SignalData<void()>(34, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'waveformTextureSizeChanged'
        QtMocHelpers::SignalData<void()>(35, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'waveformTextureStrideChanged'
        QtMocHelpers::SignalData<void()>(36, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotTrackLoaded'
        QtMocHelpers::SlotData<void(TrackPointer)>(37, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 38, 39 },
        }}),
        // Slot 'slotLoadingTrack'
        QtMocHelpers::SlotData<void(TrackPointer, TrackPointer)>(40, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 38, 41 }, { 0x80000000 | 38, 42 },
        }}),
        // Slot 'slotTrackChanged'
        QtMocHelpers::SlotData<void()>(43, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotWaveformChanged'
        QtMocHelpers::SlotData<void()>(44, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotBeatsChanged'
        QtMocHelpers::SlotData<void()>(45, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotHotcuesChanged'
        QtMocHelpers::SlotData<void()>(46, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setArtist'
        QtMocHelpers::SlotData<void(const QString &)>(47, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 48 },
        }}),
        // Slot 'setTitle'
        QtMocHelpers::SlotData<void(const QString &)>(49, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 50 },
        }}),
        // Slot 'setAlbum'
        QtMocHelpers::SlotData<void(const QString &)>(51, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 52 },
        }}),
        // Slot 'setAlbumArtist'
        QtMocHelpers::SlotData<void(const QString &)>(53, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 54 },
        }}),
        // Slot 'setComposer'
        QtMocHelpers::SlotData<void(const QString &)>(55, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 56 },
        }}),
        // Slot 'setGrouping'
        QtMocHelpers::SlotData<void(const QString &)>(57, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 58 },
        }}),
        // Slot 'setYear'
        QtMocHelpers::SlotData<void(const QString &)>(59, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 60 },
        }}),
        // Slot 'setTrackNumber'
        QtMocHelpers::SlotData<void(const QString &)>(61, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 62 },
        }}),
        // Slot 'setTrackTotal'
        QtMocHelpers::SlotData<void(const QString &)>(63, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 64 },
        }}),
        // Slot 'setComment'
        QtMocHelpers::SlotData<void(const QString &)>(65, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 66 },
        }}),
        // Slot 'setKeyText'
        QtMocHelpers::SlotData<void(const QString &)>(67, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 68 },
        }}),
        // Slot 'setColor'
        QtMocHelpers::SlotData<void(const QColor &)>(69, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 70, 71 },
        }}),
        // Method 'loadTrackFromLocation'
        QtMocHelpers::MethodData<void(const QString &, bool)>(72, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 31 }, { QMetaType::Bool, 32 },
        }}),
        // Method 'loadTrackFromLocation'
        QtMocHelpers::MethodData<void(const QString &)>(72, 8, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 31 },
        }}),
        // Method 'loadTrackFromLocationUrl'
        QtMocHelpers::MethodData<void(const QUrl &, bool)>(73, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 74, 75 }, { QMetaType::Bool, 32 },
        }}),
        // Method 'loadTrackFromLocationUrl'
        QtMocHelpers::MethodData<void(const QUrl &)>(73, 8, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 74, 75 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'isLoaded'
        QtMocHelpers::PropertyData<bool>(76, QMetaType::Bool, QMC::DefaultPropertyFlags, 3),
        // property 'artist'
        QtMocHelpers::PropertyData<QString>(48, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'title'
        QtMocHelpers::PropertyData<QString>(50, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'album'
        QtMocHelpers::PropertyData<QString>(52, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'albumArtist'
        QtMocHelpers::PropertyData<QString>(54, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'genre'
        QtMocHelpers::PropertyData<QString>(77, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable, 9),
        // property 'composer'
        QtMocHelpers::PropertyData<QString>(56, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 10),
        // property 'grouping'
        QtMocHelpers::PropertyData<QString>(58, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11),
        // property 'year'
        QtMocHelpers::PropertyData<QString>(60, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 12),
        // property 'trackNumber'
        QtMocHelpers::PropertyData<QString>(62, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 13),
        // property 'trackTotal'
        QtMocHelpers::PropertyData<QString>(64, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 14),
        // property 'comment'
        QtMocHelpers::PropertyData<QString>(66, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 15),
        // property 'keyText'
        QtMocHelpers::PropertyData<QString>(68, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 16),
        // property 'color'
        QtMocHelpers::PropertyData<QColor>(71, 0x80000000 | 70, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 17),
        // property 'coverArtUrl'
        QtMocHelpers::PropertyData<QUrl>(78, 0x80000000 | 74, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 18),
        // property 'trackLocationUrl'
        QtMocHelpers::PropertyData<QUrl>(75, 0x80000000 | 74, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 19),
        // property 'waveformLength'
        QtMocHelpers::PropertyData<int>(79, QMetaType::Int, QMC::DefaultPropertyFlags, 22),
        // property 'waveformTexture'
        QtMocHelpers::PropertyData<QString>(80, QMetaType::QString, QMC::DefaultPropertyFlags, 23),
        // property 'waveformTextureSize'
        QtMocHelpers::PropertyData<int>(81, QMetaType::Int, QMC::DefaultPropertyFlags, 24),
        // property 'waveformTextureStride'
        QtMocHelpers::PropertyData<int>(82, QMetaType::Int, QMC::DefaultPropertyFlags, 25),
        // property 'beatsModel'
        QtMocHelpers::PropertyData<mixxx::qml::QmlBeatsModel*>(83, 0x80000000 | 84, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'hotcuesModel'
        QtMocHelpers::PropertyData<mixxx::qml::QmlCuesModel*>(85, 0x80000000 | 86, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QmlPlayerProxy, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject mixxx::qml::QmlPlayerProxy::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5mixxx3qml14QmlPlayerProxyE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5mixxx3qml14QmlPlayerProxyE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN5mixxx3qml14QmlPlayerProxyE_t>.metaTypes,
    nullptr
} };

void mixxx::qml::QmlPlayerProxy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QmlPlayerProxy *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->trackLoading(); break;
        case 1: _t->trackLoaded(); break;
        case 2: _t->trackUnloaded(); break;
        case 3: _t->trackChanged(); break;
        case 4: _t->cloneFromGroup((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->albumChanged(); break;
        case 6: _t->titleChanged(); break;
        case 7: _t->artistChanged(); break;
        case 8: _t->albumArtistChanged(); break;
        case 9: _t->genreChanged(); break;
        case 10: _t->composerChanged(); break;
        case 11: _t->groupingChanged(); break;
        case 12: _t->yearChanged(); break;
        case 13: _t->trackNumberChanged(); break;
        case 14: _t->trackTotalChanged(); break;
        case 15: _t->commentChanged(); break;
        case 16: _t->keyTextChanged(); break;
        case 17: _t->colorChanged(); break;
        case 18: _t->coverArtUrlChanged(); break;
        case 19: _t->trackLocationUrlChanged(); break;
        case 20: _t->cuesChanged(); break;
        case 21: _t->loadTrackFromLocationRequested((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 22: _t->waveformLengthChanged(); break;
        case 23: _t->waveformTextureChanged(); break;
        case 24: _t->waveformTextureSizeChanged(); break;
        case 25: _t->waveformTextureStrideChanged(); break;
        case 26: _t->slotTrackLoaded((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1]))); break;
        case 27: _t->slotLoadingTrack((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[2]))); break;
        case 28: _t->slotTrackChanged(); break;
        case 29: _t->slotWaveformChanged(); break;
        case 30: _t->slotBeatsChanged(); break;
        case 31: _t->slotHotcuesChanged(); break;
        case 32: _t->setArtist((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 33: _t->setTitle((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 34: _t->setAlbum((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 35: _t->setAlbumArtist((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 36: _t->setComposer((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 37: _t->setGrouping((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 38: _t->setYear((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 39: _t->setTrackNumber((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 40: _t->setTrackTotal((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 41: _t->setComment((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 42: _t->setKeyText((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 43: _t->setColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 44: _t->loadTrackFromLocation((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 45: _t->loadTrackFromLocation((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 46: _t->loadTrackFromLocationUrl((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 47: _t->loadTrackFromLocationUrl((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 26:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 27:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QmlPlayerProxy::*)()>(_a, &QmlPlayerProxy::trackLoading, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlPlayerProxy::*)()>(_a, &QmlPlayerProxy::trackLoaded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlPlayerProxy::*)()>(_a, &QmlPlayerProxy::trackUnloaded, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlPlayerProxy::*)()>(_a, &QmlPlayerProxy::trackChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlPlayerProxy::*)(const QString & )>(_a, &QmlPlayerProxy::cloneFromGroup, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlPlayerProxy::*)()>(_a, &QmlPlayerProxy::albumChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlPlayerProxy::*)()>(_a, &QmlPlayerProxy::titleChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlPlayerProxy::*)()>(_a, &QmlPlayerProxy::artistChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlPlayerProxy::*)()>(_a, &QmlPlayerProxy::albumArtistChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlPlayerProxy::*)()>(_a, &QmlPlayerProxy::genreChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlPlayerProxy::*)()>(_a, &QmlPlayerProxy::composerChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlPlayerProxy::*)()>(_a, &QmlPlayerProxy::groupingChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlPlayerProxy::*)()>(_a, &QmlPlayerProxy::yearChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlPlayerProxy::*)()>(_a, &QmlPlayerProxy::trackNumberChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlPlayerProxy::*)()>(_a, &QmlPlayerProxy::trackTotalChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlPlayerProxy::*)()>(_a, &QmlPlayerProxy::commentChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlPlayerProxy::*)()>(_a, &QmlPlayerProxy::keyTextChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlPlayerProxy::*)()>(_a, &QmlPlayerProxy::colorChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlPlayerProxy::*)()>(_a, &QmlPlayerProxy::coverArtUrlChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlPlayerProxy::*)()>(_a, &QmlPlayerProxy::trackLocationUrlChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlPlayerProxy::*)()>(_a, &QmlPlayerProxy::cuesChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlPlayerProxy::*)(const QString & , bool )>(_a, &QmlPlayerProxy::loadTrackFromLocationRequested, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlPlayerProxy::*)()>(_a, &QmlPlayerProxy::waveformLengthChanged, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlPlayerProxy::*)()>(_a, &QmlPlayerProxy::waveformTextureChanged, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlPlayerProxy::*)()>(_a, &QmlPlayerProxy::waveformTextureSizeChanged, 24))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlPlayerProxy::*)()>(_a, &QmlPlayerProxy::waveformTextureStrideChanged, 25))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 20:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< mixxx::qml::QmlBeatsModel* >(); break;
        case 21:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< mixxx::qml::QmlCuesModel* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isLoaded(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->getArtist(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->getTitle(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->getAlbum(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->getAlbumArtist(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->getGenre(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->getComposer(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->getGrouping(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->getYear(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->getTrackNumber(); break;
        case 10: *reinterpret_cast<QString*>(_v) = _t->getTrackTotal(); break;
        case 11: *reinterpret_cast<QString*>(_v) = _t->getComment(); break;
        case 12: *reinterpret_cast<QString*>(_v) = _t->getKeyText(); break;
        case 13: *reinterpret_cast<QColor*>(_v) = _t->getColor(); break;
        case 14: *reinterpret_cast<QUrl*>(_v) = _t->getCoverArtUrl(); break;
        case 15: *reinterpret_cast<QUrl*>(_v) = _t->getTrackLocationUrl(); break;
        case 16: *reinterpret_cast<int*>(_v) = _t->getWaveformLength(); break;
        case 17: *reinterpret_cast<QString*>(_v) = _t->getWaveformTexture(); break;
        case 18: *reinterpret_cast<int*>(_v) = _t->getWaveformTextureSize(); break;
        case 19: *reinterpret_cast<int*>(_v) = _t->getWaveformTextureStride(); break;
        case 20: *reinterpret_cast<mixxx::qml::QmlBeatsModel**>(_v) = _t->m_pBeatsModel; break;
        case 21: *reinterpret_cast<mixxx::qml::QmlCuesModel**>(_v) = _t->m_pHotcuesModel; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setArtist(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setTitle(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setAlbum(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setAlbumArtist(*reinterpret_cast<QString*>(_v)); break;
        case 6: _t->setComposer(*reinterpret_cast<QString*>(_v)); break;
        case 7: _t->setGrouping(*reinterpret_cast<QString*>(_v)); break;
        case 8: _t->setYear(*reinterpret_cast<QString*>(_v)); break;
        case 9: _t->setTrackNumber(*reinterpret_cast<QString*>(_v)); break;
        case 10: _t->setTrackTotal(*reinterpret_cast<QString*>(_v)); break;
        case 11: _t->setComment(*reinterpret_cast<QString*>(_v)); break;
        case 12: _t->setKeyText(*reinterpret_cast<QString*>(_v)); break;
        case 13: _t->setColor(*reinterpret_cast<QColor*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *mixxx::qml::QmlPlayerProxy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *mixxx::qml::QmlPlayerProxy::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5mixxx3qml14QmlPlayerProxyE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int mixxx::qml::QmlPlayerProxy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 48)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 48;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 48)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 48;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    return _id;
}

// SIGNAL 0
void mixxx::qml::QmlPlayerProxy::trackLoading()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void mixxx::qml::QmlPlayerProxy::trackLoaded()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void mixxx::qml::QmlPlayerProxy::trackUnloaded()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void mixxx::qml::QmlPlayerProxy::trackChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void mixxx::qml::QmlPlayerProxy::cloneFromGroup(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void mixxx::qml::QmlPlayerProxy::albumChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void mixxx::qml::QmlPlayerProxy::titleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void mixxx::qml::QmlPlayerProxy::artistChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void mixxx::qml::QmlPlayerProxy::albumArtistChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void mixxx::qml::QmlPlayerProxy::genreChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void mixxx::qml::QmlPlayerProxy::composerChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void mixxx::qml::QmlPlayerProxy::groupingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void mixxx::qml::QmlPlayerProxy::yearChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void mixxx::qml::QmlPlayerProxy::trackNumberChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void mixxx::qml::QmlPlayerProxy::trackTotalChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void mixxx::qml::QmlPlayerProxy::commentChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void mixxx::qml::QmlPlayerProxy::keyTextChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void mixxx::qml::QmlPlayerProxy::colorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void mixxx::qml::QmlPlayerProxy::coverArtUrlChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void mixxx::qml::QmlPlayerProxy::trackLocationUrlChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void mixxx::qml::QmlPlayerProxy::cuesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 20, nullptr);
}

// SIGNAL 21
void mixxx::qml::QmlPlayerProxy::loadTrackFromLocationRequested(const QString & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1, _t2);
}

// SIGNAL 22
void mixxx::qml::QmlPlayerProxy::waveformLengthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 22, nullptr);
}

// SIGNAL 23
void mixxx::qml::QmlPlayerProxy::waveformTextureChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 23, nullptr);
}

// SIGNAL 24
void mixxx::qml::QmlPlayerProxy::waveformTextureSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 24, nullptr);
}

// SIGNAL 25
void mixxx::qml::QmlPlayerProxy::waveformTextureStrideChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 25, nullptr);
}
QT_WARNING_POP
