/********************************************************************************
** Form generated from reading UI file 'dlgprefbroadcastdlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLGPREFBROADCASTDLG_H
#define UI_DLGPREFBROADCASTDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTableView>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DlgPrefBroadcastDlg
{
public:
    QGridLayout *gridLayout;
    QGroupBox *groupBoxProfiles;
    QVBoxLayout *verticalLayout_7;
    QTableView *connectionList;
    QGridLayout *gridLayout_4;
    QPushButton *btnRemoveConnection;
    QPushButton *btnDisconnectAll;
    QSpacerItem *horizontalSpacer_3;
    QPushButton *btnCreateConnection;
    QSpacerItem *horizontalSpacer;
    QPushButton *btnRenameConnection;
    QCheckBox *connectOnApply;
    QGroupBox *groupBoxProfileSettings;
    QGridLayout *gridLayout_8;
    QGroupBox *groupBoxAutomaticReconnect;
    QVBoxLayout *verticalLayout_3;
    QVBoxLayout *verticalLayout_5;
    QCheckBox *checkBoxEnableReconnect;
    QWidget *widgetReconnectControls;
    QGridLayout *gridLayout_5;
    QGridLayout *gridLayout_2;
    QLabel *label_18;
    QDoubleSpinBox *spinBoxReconnectPeriod;
    QSpinBox *spinBoxMaximumRetries;
    QLabel *label_17;
    QCheckBox *checkBoxLimitReconnects;
    QDoubleSpinBox *spinBoxFirstDelay;
    QLabel *label_19;
    QGroupBox *groupBoxEncoding;
    QVBoxLayout *verticalLayout_4;
    QGridLayout *gridLayout_3;
    QLabel *label_8;
    QLabel *label_4;
    QLabel *label_9;
    QComboBox *comboBoxEncodingChannels;
    QComboBox *comboBoxEncodingFormat;
    QComboBox *comboBoxEncodingBitrate;
    QSpacerItem *verticalSpacer_2;
    QGroupBox *groupBoxMetadata;
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout_1;
    QLabel *label_16;
    QLineEdit *metadata_format;
    QLabel *label_14;
    QLineEdit *custom_artist;
    QLabel *label_15;
    QLineEdit *custom_title;
    QCheckBox *enableCustomMetadata;
    QCheckBox *ogg_dynamicupdate;
    QCheckBox *enableUtf8Metadata;
    QSpacerItem *verticalSpacer_3;
    QGroupBox *groupBoxStreamInfo;
    QVBoxLayout *verticalLayout_2;
    QGridLayout *gridLayout1;
    QLabel *stream_ICQ_label;
    QLabel *stream_AIM_label;
    QLineEdit *stream_name;
    QLabel *label_10;
    QLabel *label_11;
    QLineEdit *stream_website;
    QLabel *label_12;
    QTextEdit *stream_desc;
    QLabel *label_13;
    QLineEdit *stream_genre;
    QCheckBox *stream_public;
    QLabel *stream_IRC_label;
    QLineEdit *stream_IRC;
    QLineEdit *stream_AIM;
    QLineEdit *stream_ICQ;
    QLabel *labelHowtoSettings;
    QGroupBox *groupBoxServerConnection;
    QGridLayout *gridLayout2;
    QGridLayout *gridLayoutSC;
    QLabel *label;
    QComboBox *comboBoxServerType;
    QWidget *widget;
    QLabel *label_5;
    QLineEdit *mountpoint;
    QLabel *label_2;
    QLineEdit *host;
    QLabel *label_3;
    QLineEdit *port;
    QLabel *label_6;
    QLineEdit *login;
    QLabel *label_7;
    QLineEdit *password;
    QWidget *groupPasswordStorage;
    QHBoxLayout *horizontalLayout_2;
    QHBoxLayout *horizontalLayout;
    QLabel *label_20;
    QRadioButton *rbPasswordCleartext;
    QRadioButton *rbPasswordKeychain;
    QSpacerItem *horizontalSpacer_2;
    QSpacerItem *verticalSpacer;
    QButtonGroup *rbGroupPassword;

    void setupUi(QWidget *DlgPrefBroadcastDlg)
    {
        if (DlgPrefBroadcastDlg->objectName().isEmpty())
            DlgPrefBroadcastDlg->setObjectName("DlgPrefBroadcastDlg");
        DlgPrefBroadcastDlg->resize(715, 1012);
        gridLayout = new QGridLayout(DlgPrefBroadcastDlg);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        groupBoxProfiles = new QGroupBox(DlgPrefBroadcastDlg);
        groupBoxProfiles->setObjectName("groupBoxProfiles");
        verticalLayout_7 = new QVBoxLayout(groupBoxProfiles);
        verticalLayout_7->setSpacing(6);
        verticalLayout_7->setContentsMargins(11, 11, 11, 11);
        verticalLayout_7->setObjectName("verticalLayout_7");
        connectionList = new QTableView(groupBoxProfiles);
        connectionList->setObjectName("connectionList");
        connectionList->setMinimumSize(QSize(0, 130));
        connectionList->setStyleSheet(QString::fromUtf8(""));
        connectionList->setSelectionMode(QAbstractItemView::SingleSelection);
        connectionList->setSelectionBehavior(QAbstractItemView::SelectRows);
        connectionList->setShowGrid(true);
        connectionList->setGridStyle(Qt::NoPen);
        connectionList->horizontalHeader()->setHighlightSections(true);
        connectionList->horizontalHeader()->setStretchLastSection(true);
        connectionList->verticalHeader()->setVisible(false);
        connectionList->verticalHeader()->setHighlightSections(false);

        verticalLayout_7->addWidget(connectionList);

        gridLayout_4 = new QGridLayout();
        gridLayout_4->setSpacing(6);
        gridLayout_4->setObjectName("gridLayout_4");
        btnRemoveConnection = new QPushButton(groupBoxProfiles);
        btnRemoveConnection->setObjectName("btnRemoveConnection");

        gridLayout_4->addWidget(btnRemoveConnection, 0, 0, 1, 1);

        btnDisconnectAll = new QPushButton(groupBoxProfiles);
        btnDisconnectAll->setObjectName("btnDisconnectAll");

        gridLayout_4->addWidget(btnDisconnectAll, 0, 3, 1, 1);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_4->addItem(horizontalSpacer_3, 0, 4, 1, 1);

        btnCreateConnection = new QPushButton(groupBoxProfiles);
        btnCreateConnection->setObjectName("btnCreateConnection");

        gridLayout_4->addWidget(btnCreateConnection, 0, 5, 1, 1);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_4->addItem(horizontalSpacer, 0, 2, 1, 1);

        btnRenameConnection = new QPushButton(groupBoxProfiles);
        btnRenameConnection->setObjectName("btnRenameConnection");

        gridLayout_4->addWidget(btnRenameConnection, 0, 1, 1, 1);


        verticalLayout_7->addLayout(gridLayout_4);

        connectOnApply = new QCheckBox(groupBoxProfiles);
        connectOnApply->setObjectName("connectOnApply");
        connectOnApply->setChecked(false);

        verticalLayout_7->addWidget(connectOnApply);


        gridLayout->addWidget(groupBoxProfiles, 0, 0, 1, 2);

        groupBoxProfileSettings = new QGroupBox(DlgPrefBroadcastDlg);
        groupBoxProfileSettings->setObjectName("groupBoxProfileSettings");
        gridLayout_8 = new QGridLayout(groupBoxProfileSettings);
        gridLayout_8->setSpacing(6);
        gridLayout_8->setContentsMargins(11, 11, 11, 11);
        gridLayout_8->setObjectName("gridLayout_8");
        groupBoxAutomaticReconnect = new QGroupBox(groupBoxProfileSettings);
        groupBoxAutomaticReconnect->setObjectName("groupBoxAutomaticReconnect");
        verticalLayout_3 = new QVBoxLayout(groupBoxAutomaticReconnect);
        verticalLayout_3->setSpacing(6);
        verticalLayout_3->setContentsMargins(11, 11, 11, 11);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_5 = new QVBoxLayout();
        verticalLayout_5->setSpacing(6);
        verticalLayout_5->setObjectName("verticalLayout_5");
        checkBoxEnableReconnect = new QCheckBox(groupBoxAutomaticReconnect);
        checkBoxEnableReconnect->setObjectName("checkBoxEnableReconnect");

        verticalLayout_5->addWidget(checkBoxEnableReconnect);

        widgetReconnectControls = new QWidget(groupBoxAutomaticReconnect);
        widgetReconnectControls->setObjectName("widgetReconnectControls");
        gridLayout_5 = new QGridLayout(widgetReconnectControls);
        gridLayout_5->setSpacing(6);
        gridLayout_5->setContentsMargins(0, 0, 0, 0);
        gridLayout_5->setObjectName("gridLayout_5");
        gridLayout_2 = new QGridLayout();
        gridLayout_2->setSpacing(6);
        gridLayout_2->setObjectName("gridLayout_2");
        label_18 = new QLabel(widgetReconnectControls);
        label_18->setObjectName("label_18");

        gridLayout_2->addWidget(label_18, 1, 0, 1, 1);

        spinBoxReconnectPeriod = new QDoubleSpinBox(widgetReconnectControls);
        spinBoxReconnectPeriod->setObjectName("spinBoxReconnectPeriod");
        spinBoxReconnectPeriod->setDecimals(2);
        spinBoxReconnectPeriod->setMaximum(3600.000000000000000);

        gridLayout_2->addWidget(spinBoxReconnectPeriod, 1, 1, 1, 1);

        spinBoxMaximumRetries = new QSpinBox(widgetReconnectControls);
        spinBoxMaximumRetries->setObjectName("spinBoxMaximumRetries");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(spinBoxMaximumRetries->sizePolicy().hasHeightForWidth());
        spinBoxMaximumRetries->setSizePolicy(sizePolicy);
        spinBoxMaximumRetries->setMinimum(1);
        spinBoxMaximumRetries->setMaximum(100);

        gridLayout_2->addWidget(spinBoxMaximumRetries, 3, 1, 1, 1);

        label_17 = new QLabel(widgetReconnectControls);
        label_17->setObjectName("label_17");

        gridLayout_2->addWidget(label_17, 3, 0, 1, 1);

        checkBoxLimitReconnects = new QCheckBox(widgetReconnectControls);
        checkBoxLimitReconnects->setObjectName("checkBoxLimitReconnects");

        gridLayout_2->addWidget(checkBoxLimitReconnects, 2, 0, 1, 2);

        spinBoxFirstDelay = new QDoubleSpinBox(widgetReconnectControls);
        spinBoxFirstDelay->setObjectName("spinBoxFirstDelay");
        spinBoxFirstDelay->setMaximum(3600.000000000000000);

        gridLayout_2->addWidget(spinBoxFirstDelay, 0, 1, 1, 1);

        label_19 = new QLabel(widgetReconnectControls);
        label_19->setObjectName("label_19");

        gridLayout_2->addWidget(label_19, 0, 0, 1, 1);


        gridLayout_5->addLayout(gridLayout_2, 0, 0, 1, 1);


        verticalLayout_5->addWidget(widgetReconnectControls);


        verticalLayout_3->addLayout(verticalLayout_5);


        gridLayout_8->addWidget(groupBoxAutomaticReconnect, 2, 0, 1, 1);

        groupBoxEncoding = new QGroupBox(groupBoxProfileSettings);
        groupBoxEncoding->setObjectName("groupBoxEncoding");
        verticalLayout_4 = new QVBoxLayout(groupBoxEncoding);
        verticalLayout_4->setSpacing(6);
        verticalLayout_4->setContentsMargins(11, 11, 11, 11);
        verticalLayout_4->setObjectName("verticalLayout_4");
        gridLayout_3 = new QGridLayout();
        gridLayout_3->setSpacing(6);
        gridLayout_3->setObjectName("gridLayout_3");
        label_8 = new QLabel(groupBoxEncoding);
        label_8->setObjectName("label_8");

        gridLayout_3->addWidget(label_8, 1, 0, 1, 1);

        label_4 = new QLabel(groupBoxEncoding);
        label_4->setObjectName("label_4");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label_4->sizePolicy().hasHeightForWidth());
        label_4->setSizePolicy(sizePolicy1);

        gridLayout_3->addWidget(label_4, 0, 0, 1, 1);

        label_9 = new QLabel(groupBoxEncoding);
        label_9->setObjectName("label_9");
        label_9->setFrameShape(QFrame::NoFrame);
        label_9->setLineWidth(1);
        label_9->setTextFormat(Qt::AutoText);
        label_9->setScaledContents(false);

        gridLayout_3->addWidget(label_9, 2, 0, 1, 1);

        comboBoxEncodingChannels = new QComboBox(groupBoxEncoding);
        comboBoxEncodingChannels->setObjectName("comboBoxEncodingChannels");
        sizePolicy.setHeightForWidth(comboBoxEncodingChannels->sizePolicy().hasHeightForWidth());
        comboBoxEncodingChannels->setSizePolicy(sizePolicy);
        comboBoxEncodingChannels->setMaximumSize(QSize(16777215, 16777215));

        gridLayout_3->addWidget(comboBoxEncodingChannels, 2, 1, 1, 1);

        comboBoxEncodingFormat = new QComboBox(groupBoxEncoding);
        comboBoxEncodingFormat->setObjectName("comboBoxEncodingFormat");
        sizePolicy.setHeightForWidth(comboBoxEncodingFormat->sizePolicy().hasHeightForWidth());
        comboBoxEncodingFormat->setSizePolicy(sizePolicy);

        gridLayout_3->addWidget(comboBoxEncodingFormat, 1, 1, 1, 1);

        comboBoxEncodingBitrate = new QComboBox(groupBoxEncoding);
        comboBoxEncodingBitrate->setObjectName("comboBoxEncodingBitrate");
        sizePolicy.setHeightForWidth(comboBoxEncodingBitrate->sizePolicy().hasHeightForWidth());
        comboBoxEncodingBitrate->setSizePolicy(sizePolicy);

        gridLayout_3->addWidget(comboBoxEncodingBitrate, 0, 1, 1, 1);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_3->addItem(verticalSpacer_2, 3, 1, 1, 1);


        verticalLayout_4->addLayout(gridLayout_3);


        gridLayout_8->addWidget(groupBoxEncoding, 2, 1, 1, 1);

        groupBoxMetadata = new QGroupBox(groupBoxProfileSettings);
        groupBoxMetadata->setObjectName("groupBoxMetadata");
        groupBoxMetadata->setFlat(false);
        verticalLayout = new QVBoxLayout(groupBoxMetadata);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout_1 = new QGridLayout();
        gridLayout_1->setSpacing(6);
        gridLayout_1->setObjectName("gridLayout_1");
        label_16 = new QLabel(groupBoxMetadata);
        label_16->setObjectName("label_16");

        gridLayout_1->addWidget(label_16, 0, 0, 1, 1);

        metadata_format = new QLineEdit(groupBoxMetadata);
        metadata_format->setObjectName("metadata_format");
        metadata_format->setText(QString::fromUtf8("$artist - $title"));

        gridLayout_1->addWidget(metadata_format, 0, 1, 1, 1);

        label_14 = new QLabel(groupBoxMetadata);
        label_14->setObjectName("label_14");

        gridLayout_1->addWidget(label_14, 2, 0, 1, 1);

        custom_artist = new QLineEdit(groupBoxMetadata);
        custom_artist->setObjectName("custom_artist");
        custom_artist->setText(QString::fromUtf8("%mainartist"));

        gridLayout_1->addWidget(custom_artist, 2, 1, 1, 1);

        label_15 = new QLabel(groupBoxMetadata);
        label_15->setObjectName("label_15");

        gridLayout_1->addWidget(label_15, 3, 0, 1, 1);

        custom_title = new QLineEdit(groupBoxMetadata);
        custom_title->setObjectName("custom_title");
        custom_title->setText(QString::fromUtf8("%maintitle"));

        gridLayout_1->addWidget(custom_title, 3, 1, 1, 1);

        enableCustomMetadata = new QCheckBox(groupBoxMetadata);
        enableCustomMetadata->setObjectName("enableCustomMetadata");
        enableCustomMetadata->setChecked(true);

        gridLayout_1->addWidget(enableCustomMetadata, 1, 0, 1, 2);

        ogg_dynamicupdate = new QCheckBox(groupBoxMetadata);
        ogg_dynamicupdate->setObjectName("ogg_dynamicupdate");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(ogg_dynamicupdate->sizePolicy().hasHeightForWidth());
        ogg_dynamicupdate->setSizePolicy(sizePolicy2);

        gridLayout_1->addWidget(ogg_dynamicupdate, 4, 0, 1, 2);

        enableUtf8Metadata = new QCheckBox(groupBoxMetadata);
        enableUtf8Metadata->setObjectName("enableUtf8Metadata");

        gridLayout_1->addWidget(enableUtf8Metadata, 5, 0, 1, 2);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_1->addItem(verticalSpacer_3, 6, 1, 1, 1);


        verticalLayout->addLayout(gridLayout_1);


        gridLayout_8->addWidget(groupBoxMetadata, 4, 0, 1, 2);

        groupBoxStreamInfo = new QGroupBox(groupBoxProfileSettings);
        groupBoxStreamInfo->setObjectName("groupBoxStreamInfo");
        groupBoxStreamInfo->setMinimumSize(QSize(300, 0));
        verticalLayout_2 = new QVBoxLayout(groupBoxStreamInfo);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        gridLayout1 = new QGridLayout();
        gridLayout1->setSpacing(6);
        gridLayout1->setObjectName("gridLayout1");
        stream_ICQ_label = new QLabel(groupBoxStreamInfo);
        stream_ICQ_label->setObjectName("stream_ICQ_label");

        gridLayout1->addWidget(stream_ICQ_label, 7, 0, 1, 1);

        stream_AIM_label = new QLabel(groupBoxStreamInfo);
        stream_AIM_label->setObjectName("stream_AIM_label");

        gridLayout1->addWidget(stream_AIM_label, 6, 0, 1, 1);

        stream_name = new QLineEdit(groupBoxStreamInfo);
        stream_name->setObjectName("stream_name");
        sizePolicy.setHeightForWidth(stream_name->sizePolicy().hasHeightForWidth());
        stream_name->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(stream_name, 1, 1, 1, 1);

        label_10 = new QLabel(groupBoxStreamInfo);
        label_10->setObjectName("label_10");
        sizePolicy1.setHeightForWidth(label_10->sizePolicy().hasHeightForWidth());
        label_10->setSizePolicy(sizePolicy1);

        gridLayout1->addWidget(label_10, 1, 0, 1, 1);

        label_11 = new QLabel(groupBoxStreamInfo);
        label_11->setObjectName("label_11");
        sizePolicy1.setHeightForWidth(label_11->sizePolicy().hasHeightForWidth());
        label_11->setSizePolicy(sizePolicy1);

        gridLayout1->addWidget(label_11, 2, 0, 1, 1);

        stream_website = new QLineEdit(groupBoxStreamInfo);
        stream_website->setObjectName("stream_website");
        sizePolicy.setHeightForWidth(stream_website->sizePolicy().hasHeightForWidth());
        stream_website->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(stream_website, 2, 1, 1, 1);

        label_12 = new QLabel(groupBoxStreamInfo);
        label_12->setObjectName("label_12");
        sizePolicy1.setHeightForWidth(label_12->sizePolicy().hasHeightForWidth());
        label_12->setSizePolicy(sizePolicy1);

        gridLayout1->addWidget(label_12, 3, 0, 1, 1);

        stream_desc = new QTextEdit(groupBoxStreamInfo);
        stream_desc->setObjectName("stream_desc");
        sizePolicy.setHeightForWidth(stream_desc->sizePolicy().hasHeightForWidth());
        stream_desc->setSizePolicy(sizePolicy);
        stream_desc->setMinimumSize(QSize(0, 50));
        stream_desc->setMaximumSize(QSize(16777215, 50));
        stream_desc->setTabChangesFocus(true);
        stream_desc->setHtml(QString::fromUtf8("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'.SF NS Text'; font-size:13pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:'Ubuntu'; font-size:10pt;\">  </span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:'Lucida Grande';\"><br /></p></body></html>"));

        gridLayout1->addWidget(stream_desc, 3, 1, 1, 1);

        label_13 = new QLabel(groupBoxStreamInfo);
        label_13->setObjectName("label_13");
        sizePolicy1.setHeightForWidth(label_13->sizePolicy().hasHeightForWidth());
        label_13->setSizePolicy(sizePolicy1);

        gridLayout1->addWidget(label_13, 4, 0, 1, 1);

        stream_genre = new QLineEdit(groupBoxStreamInfo);
        stream_genre->setObjectName("stream_genre");
        sizePolicy.setHeightForWidth(stream_genre->sizePolicy().hasHeightForWidth());
        stream_genre->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(stream_genre, 4, 1, 1, 1);

        stream_public = new QCheckBox(groupBoxStreamInfo);
        stream_public->setObjectName("stream_public");

        gridLayout1->addWidget(stream_public, 0, 0, 1, 2);

        stream_IRC_label = new QLabel(groupBoxStreamInfo);
        stream_IRC_label->setObjectName("stream_IRC_label");

        gridLayout1->addWidget(stream_IRC_label, 5, 0, 1, 1);

        stream_IRC = new QLineEdit(groupBoxStreamInfo);
        stream_IRC->setObjectName("stream_IRC");

        gridLayout1->addWidget(stream_IRC, 5, 1, 1, 1);

        stream_AIM = new QLineEdit(groupBoxStreamInfo);
        stream_AIM->setObjectName("stream_AIM");

        gridLayout1->addWidget(stream_AIM, 6, 1, 1, 1);

        stream_ICQ = new QLineEdit(groupBoxStreamInfo);
        stream_ICQ->setObjectName("stream_ICQ");

        gridLayout1->addWidget(stream_ICQ, 7, 1, 1, 1);


        verticalLayout_2->addLayout(gridLayout1);


        gridLayout_8->addWidget(groupBoxStreamInfo, 3, 0, 1, 2);

        labelHowtoSettings = new QLabel(groupBoxProfileSettings);
        labelHowtoSettings->setObjectName("labelHowtoSettings");

        gridLayout_8->addWidget(labelHowtoSettings, 0, 0, 1, 2);

        groupBoxServerConnection = new QGroupBox(groupBoxProfileSettings);
        groupBoxServerConnection->setObjectName("groupBoxServerConnection");
        gridLayout2 = new QGridLayout(groupBoxServerConnection);
        gridLayout2->setSpacing(6);
        gridLayout2->setContentsMargins(11, 11, 11, 11);
        gridLayout2->setObjectName("gridLayout2");
        gridLayoutSC = new QGridLayout();
        gridLayoutSC->setSpacing(6);
        gridLayoutSC->setObjectName("gridLayoutSC");
        label = new QLabel(groupBoxServerConnection);
        label->setObjectName("label");

        gridLayoutSC->addWidget(label, 0, 0, 1, 1);

        comboBoxServerType = new QComboBox(groupBoxServerConnection);
        comboBoxServerType->setObjectName("comboBoxServerType");
        sizePolicy.setHeightForWidth(comboBoxServerType->sizePolicy().hasHeightForWidth());
        comboBoxServerType->setSizePolicy(sizePolicy);

        gridLayoutSC->addWidget(comboBoxServerType, 0, 1, 1, 1);

        widget = new QWidget(groupBoxServerConnection);
        widget->setObjectName("widget");
        widget->setMinimumSize(QSize(10, 0));

        gridLayoutSC->addWidget(widget, 0, 2, 1, 1);

        label_5 = new QLabel(groupBoxServerConnection);
        label_5->setObjectName("label_5");

        gridLayoutSC->addWidget(label_5, 0, 3, 1, 1);

        mountpoint = new QLineEdit(groupBoxServerConnection);
        mountpoint->setObjectName("mountpoint");
        mountpoint->setText(QString::fromUtf8(""));

        gridLayoutSC->addWidget(mountpoint, 0, 4, 1, 1);

        label_2 = new QLabel(groupBoxServerConnection);
        label_2->setObjectName("label_2");

        gridLayoutSC->addWidget(label_2, 1, 0, 1, 1);

        host = new QLineEdit(groupBoxServerConnection);
        host->setObjectName("host");
        host->setText(QString::fromUtf8(""));

        gridLayoutSC->addWidget(host, 1, 1, 1, 1);

        label_3 = new QLabel(groupBoxServerConnection);
        label_3->setObjectName("label_3");

        gridLayoutSC->addWidget(label_3, 1, 3, 1, 1);

        port = new QLineEdit(groupBoxServerConnection);
        port->setObjectName("port");
        sizePolicy2.setHeightForWidth(port->sizePolicy().hasHeightForWidth());
        port->setSizePolicy(sizePolicy2);
        port->setText(QString::fromUtf8(""));

        gridLayoutSC->addWidget(port, 1, 4, 1, 1);

        label_6 = new QLabel(groupBoxServerConnection);
        label_6->setObjectName("label_6");

        gridLayoutSC->addWidget(label_6, 2, 0, 1, 1);

        login = new QLineEdit(groupBoxServerConnection);
        login->setObjectName("login");
        login->setText(QString::fromUtf8("source"));

        gridLayoutSC->addWidget(login, 2, 1, 1, 1);

        label_7 = new QLabel(groupBoxServerConnection);
        label_7->setObjectName("label_7");

        gridLayoutSC->addWidget(label_7, 2, 3, 1, 1);

        password = new QLineEdit(groupBoxServerConnection);
        password->setObjectName("password");
        password->setText(QString::fromUtf8(""));
        password->setEchoMode(QLineEdit::Password);

        gridLayoutSC->addWidget(password, 2, 4, 1, 1);


        gridLayout2->addLayout(gridLayoutSC, 0, 0, 1, 1);

        groupPasswordStorage = new QWidget(groupBoxServerConnection);
        groupPasswordStorage->setObjectName("groupPasswordStorage");
        horizontalLayout_2 = new QHBoxLayout(groupPasswordStorage);
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        label_20 = new QLabel(groupPasswordStorage);
        label_20->setObjectName("label_20");

        horizontalLayout->addWidget(label_20);

        rbPasswordCleartext = new QRadioButton(groupPasswordStorage);
        rbGroupPassword = new QButtonGroup(DlgPrefBroadcastDlg);
        rbGroupPassword->setObjectName("rbGroupPassword");
        rbGroupPassword->addButton(rbPasswordCleartext);
        rbPasswordCleartext->setObjectName("rbPasswordCleartext");

        horizontalLayout->addWidget(rbPasswordCleartext);

        rbPasswordKeychain = new QRadioButton(groupPasswordStorage);
        rbGroupPassword->addButton(rbPasswordKeychain);
        rbPasswordKeychain->setObjectName("rbPasswordKeychain");

        horizontalLayout->addWidget(rbPasswordKeychain);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);


        horizontalLayout_2->addLayout(horizontalLayout);


        gridLayout2->addWidget(groupPasswordStorage, 1, 0, 1, 1);


        gridLayout_8->addWidget(groupBoxServerConnection, 1, 0, 1, 2);


        gridLayout->addWidget(groupBoxProfileSettings, 2, 0, 1, 2);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 6, 0, 1, 1);

#if QT_CONFIG(shortcut)
        label_8->setBuddy(comboBoxEncodingFormat);
        label_9->setBuddy(comboBoxEncodingChannels);
        label_14->setBuddy(custom_artist);
        label_15->setBuddy(custom_title);
        label_10->setBuddy(comboBoxServerType);
        label_11->setBuddy(mountpoint);
        label_12->setBuddy(stream_desc);
        label_13->setBuddy(stream_genre);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(connectionList, btnRemoveConnection);
        QWidget::setTabOrder(btnRemoveConnection, btnRenameConnection);
        QWidget::setTabOrder(btnRenameConnection, btnDisconnectAll);
        QWidget::setTabOrder(btnDisconnectAll, btnCreateConnection);
        QWidget::setTabOrder(btnCreateConnection, connectOnApply);
        QWidget::setTabOrder(connectOnApply, comboBoxServerType);
        QWidget::setTabOrder(comboBoxServerType, mountpoint);
        QWidget::setTabOrder(mountpoint, host);
        QWidget::setTabOrder(host, port);
        QWidget::setTabOrder(port, login);
        QWidget::setTabOrder(login, password);
        QWidget::setTabOrder(password, rbPasswordCleartext);
        QWidget::setTabOrder(rbPasswordCleartext, checkBoxEnableReconnect);
        QWidget::setTabOrder(checkBoxEnableReconnect, spinBoxFirstDelay);
        QWidget::setTabOrder(spinBoxFirstDelay, spinBoxReconnectPeriod);
        QWidget::setTabOrder(spinBoxReconnectPeriod, checkBoxLimitReconnects);
        QWidget::setTabOrder(checkBoxLimitReconnects, spinBoxMaximumRetries);
        QWidget::setTabOrder(spinBoxMaximumRetries, comboBoxEncodingBitrate);
        QWidget::setTabOrder(comboBoxEncodingBitrate, comboBoxEncodingFormat);
        QWidget::setTabOrder(comboBoxEncodingFormat, comboBoxEncodingChannels);
        QWidget::setTabOrder(comboBoxEncodingChannels, stream_public);
        QWidget::setTabOrder(stream_public, stream_name);
        QWidget::setTabOrder(stream_name, stream_website);
        QWidget::setTabOrder(stream_website, stream_desc);
        QWidget::setTabOrder(stream_desc, stream_genre);
        QWidget::setTabOrder(stream_genre, stream_IRC);
        QWidget::setTabOrder(stream_IRC, stream_AIM);
        QWidget::setTabOrder(stream_AIM, stream_ICQ);
        QWidget::setTabOrder(stream_ICQ, metadata_format);
        QWidget::setTabOrder(metadata_format, enableCustomMetadata);
        QWidget::setTabOrder(enableCustomMetadata, custom_artist);
        QWidget::setTabOrder(custom_artist, custom_title);
        QWidget::setTabOrder(custom_title, ogg_dynamicupdate);
        QWidget::setTabOrder(ogg_dynamicupdate, enableUtf8Metadata);

        retranslateUi(DlgPrefBroadcastDlg);

        QMetaObject::connectSlotsByName(DlgPrefBroadcastDlg);
    } // setupUi

    void retranslateUi(QWidget *DlgPrefBroadcastDlg)
    {
        DlgPrefBroadcastDlg->setWindowTitle(QCoreApplication::translate("DlgPrefBroadcastDlg", "Live Broadcasting Preferences", nullptr));
        groupBoxProfiles->setTitle(QCoreApplication::translate("DlgPrefBroadcastDlg", "Live Broadcasting source connections", nullptr));
        btnRemoveConnection->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Delete selected", nullptr));
        btnDisconnectAll->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Disconnect all", nullptr));
        btnCreateConnection->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Create new connection", nullptr));
        btnRenameConnection->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Rename selected", nullptr));
        connectOnApply->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Turn on Live Broadcasting when applying these settings", nullptr));
        groupBoxProfileSettings->setTitle(QCoreApplication::translate("DlgPrefBroadcastDlg", "Settings for %1", nullptr));
        groupBoxAutomaticReconnect->setTitle(QCoreApplication::translate("DlgPrefBroadcastDlg", "Automatic reconnect", nullptr));
#if QT_CONFIG(tooltip)
        checkBoxEnableReconnect->setToolTip(QCoreApplication::translate("DlgPrefBroadcastDlg", "Reconnect if the connection to the streaming server is lost.", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBoxEnableReconnect->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Enable automatic reconnect", nullptr));
        label_18->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Reconnect period", nullptr));
#if QT_CONFIG(tooltip)
        spinBoxReconnectPeriod->setToolTip(QCoreApplication::translate("DlgPrefBroadcastDlg", "Time to wait between two reconnection attempts.", nullptr));
#endif // QT_CONFIG(tooltip)
        spinBoxReconnectPeriod->setSuffix(QCoreApplication::translate("DlgPrefBroadcastDlg", " seconds", nullptr));
        label_17->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Maximum retries", nullptr));
#if QT_CONFIG(tooltip)
        checkBoxLimitReconnects->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        checkBoxLimitReconnects->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Limit number of reconnection attempts", nullptr));
#if QT_CONFIG(tooltip)
        spinBoxFirstDelay->setToolTip(QCoreApplication::translate("DlgPrefBroadcastDlg", "Time to wait before the first reconnection attempt is made.", nullptr));
#endif // QT_CONFIG(tooltip)
        spinBoxFirstDelay->setSuffix(QCoreApplication::translate("DlgPrefBroadcastDlg", " seconds", nullptr));
        label_19->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Wait until first attempt", nullptr));
        groupBoxEncoding->setTitle(QCoreApplication::translate("DlgPrefBroadcastDlg", "Encoding", nullptr));
        label_8->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Format", nullptr));
        label_4->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Bitrate", nullptr));
        label_9->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Channels", nullptr));
        groupBoxMetadata->setTitle(QCoreApplication::translate("DlgPrefBroadcastDlg", "Metadata", nullptr));
        label_16->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Format", nullptr));
        label_14->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Static artist", nullptr));
        label_15->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Static title", nullptr));
        enableCustomMetadata->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Use static artist and title.", nullptr));
#if QT_CONFIG(tooltip)
        ogg_dynamicupdate->setToolTip(QCoreApplication::translate("DlgPrefBroadcastDlg", "Due to flaws in some streaming clients, updating Ogg Vorbis metadata dynamically can cause listener glitches and disconnections. Check this box to update the metadata anyway.", nullptr));
#endif // QT_CONFIG(tooltip)
        ogg_dynamicupdate->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Dynamically update Ogg Vorbis metadata.", nullptr));
        enableUtf8Metadata->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Use UTF-8 encoding for metadata.", nullptr));
        groupBoxStreamInfo->setTitle(QCoreApplication::translate("DlgPrefBroadcastDlg", "Stream info", nullptr));
        stream_ICQ_label->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "ICQ", nullptr));
        stream_AIM_label->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "AIM", nullptr));
        stream_name->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Mixxx Icecast Testing", nullptr));
        label_10->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Stream name", nullptr));
        label_11->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Website", nullptr));
        stream_website->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "http://www.mixxx.org", nullptr));
        label_12->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Description", nullptr));
        label_13->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Genre", nullptr));
        stream_genre->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Live mix", nullptr));
        stream_public->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Public stream", nullptr));
        stream_IRC_label->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "IRC", nullptr));
        labelHowtoSettings->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Select a source connection above to edit its settings here", nullptr));
        groupBoxServerConnection->setTitle(QCoreApplication::translate("DlgPrefBroadcastDlg", "Server connection", nullptr));
        label->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Type", nullptr));
        label_5->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Mount", nullptr));
        label_2->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Host", nullptr));
        label_3->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Port", nullptr));
        label_6->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Login", nullptr));
        label_7->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Password", nullptr));
        label_20->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Password storage", nullptr));
        rbPasswordCleartext->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Plain text", nullptr));
        rbPasswordKeychain->setText(QCoreApplication::translate("DlgPrefBroadcastDlg", "Secure storage (OS keychain)", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgPrefBroadcastDlg: public Ui_DlgPrefBroadcastDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLGPREFBROADCASTDLG_H
