#include "waveform/renderers/allshader/waveformrenderersignalbase.h"

using namespace allshader;

allshader::WaveformRendererSignalBase::WaveformRendererSignalBase(
        WaveformWidgetRenderer* waveformWidget)
        : ::WaveformRendererSignalBase(waveformWidget) {
}
