/********************************************************************************
** Form generated from reading UI file 'dlgprefsounddlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLGPREFSOUNDDLG_H
#define UI_DLGPREFSOUNDDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DlgPrefSoundDlg
{
public:
    QVBoxLayout *verticalLayout_2;
    QGridLayout *gridLayout;
    QLabel *apiLabel;
    QComboBox *apiComboBox;
    QLabel *sampleRateLabel;
    QComboBox *sampleRateComboBox;
    QLabel *audioBufferLabel;
    QComboBox *audioBufferComboBox;
    QLabel *deviceSyncLabel;
    QComboBox *deviceSyncComboBox;
    QLabel *engineClockLabel;
    QComboBox *engineClockComboBox;
    QLabel *keylockLabel;
    QComboBox *keylockComboBox;
    QLabel *masteMixLabel;
    QComboBox *mainMixComboBox;
    QComboBox *mainOutputModeComboBox;
    QLabel *mainMonoLabel;
    QComboBox *micMonitorModeComboBox;
    QLabel *micMonitorModeLabel;
    QLabel *latencyCompensationLabel;
    QDoubleSpinBox *latencyCompensationSpinBox;
    QLabel *mainDelayLabel;
    QDoubleSpinBox *mainDelaySpinBox;
    QLabel *headDelayLabel;
    QDoubleSpinBox *headDelaySpinBox;
    QLabel *boothDelayLabel;
    QDoubleSpinBox *boothDelaySpinBox;
    QLabel *latencyCompensationWarningLabel;
    QFrame *line_2;
    QHBoxLayout *buttonsHLayout;
    QPushButton *queryButton;
    QSpacerItem *buttonsHSpacer;
    QTabWidget *ioTabs;
    QWidget *outputTab;
    QVBoxLayout *outputVLayout;
    QSpacerItem *outputVSpacer;
    QWidget *inputTab;
    QVBoxLayout *inputVLayout;
    QSpacerItem *inputVSpacer;
    QGroupBox *Hints;
    QGridLayout *gridLayout_6;
    QLabel *deckBusHint;
    QLabel *underflowCounterHint;
    QLabel *audioBufferHint;
    QLabel *realtimeHint;
    QLabel *hardwareGuide;
    QFrame *line;
    QLabel *latencyLabel;
    QLabel *currentLatency;
    QLabel *underflowLabel;
    QLabel *bufferUnderflowCount;

    void setupUi(QWidget *DlgPrefSoundDlg)
    {
        if (DlgPrefSoundDlg->objectName().isEmpty())
            DlgPrefSoundDlg->setObjectName("DlgPrefSoundDlg");
        DlgPrefSoundDlg->resize(686, 852);
        verticalLayout_2 = new QVBoxLayout(DlgPrefSoundDlg);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        apiLabel = new QLabel(DlgPrefSoundDlg);
        apiLabel->setObjectName("apiLabel");
        apiLabel->setOpenExternalLinks(true);

        gridLayout->addWidget(apiLabel, 0, 0, 1, 1);

        apiComboBox = new QComboBox(DlgPrefSoundDlg);
        apiComboBox->setObjectName("apiComboBox");

        gridLayout->addWidget(apiComboBox, 0, 1, 1, 1);

        sampleRateLabel = new QLabel(DlgPrefSoundDlg);
        sampleRateLabel->setObjectName("sampleRateLabel");

        gridLayout->addWidget(sampleRateLabel, 1, 0, 1, 1);

        sampleRateComboBox = new QComboBox(DlgPrefSoundDlg);
        sampleRateComboBox->setObjectName("sampleRateComboBox");

        gridLayout->addWidget(sampleRateComboBox, 1, 1, 1, 1);

        audioBufferLabel = new QLabel(DlgPrefSoundDlg);
        audioBufferLabel->setObjectName("audioBufferLabel");

        gridLayout->addWidget(audioBufferLabel, 2, 0, 1, 1);

        audioBufferComboBox = new QComboBox(DlgPrefSoundDlg);
        audioBufferComboBox->setObjectName("audioBufferComboBox");

        gridLayout->addWidget(audioBufferComboBox, 2, 1, 1, 1);

        deviceSyncLabel = new QLabel(DlgPrefSoundDlg);
        deviceSyncLabel->setObjectName("deviceSyncLabel");

        gridLayout->addWidget(deviceSyncLabel, 3, 0, 1, 1);

        deviceSyncComboBox = new QComboBox(DlgPrefSoundDlg);
        deviceSyncComboBox->setObjectName("deviceSyncComboBox");

        gridLayout->addWidget(deviceSyncComboBox, 3, 1, 1, 1);

        engineClockLabel = new QLabel(DlgPrefSoundDlg);
        engineClockLabel->setObjectName("engineClockLabel");

        gridLayout->addWidget(engineClockLabel, 4, 0, 1, 1);

        engineClockComboBox = new QComboBox(DlgPrefSoundDlg);
        engineClockComboBox->setObjectName("engineClockComboBox");

        gridLayout->addWidget(engineClockComboBox, 4, 1, 1, 1);

        keylockLabel = new QLabel(DlgPrefSoundDlg);
        keylockLabel->setObjectName("keylockLabel");

        gridLayout->addWidget(keylockLabel, 5, 0, 1, 1);

        keylockComboBox = new QComboBox(DlgPrefSoundDlg);
        keylockComboBox->setObjectName("keylockComboBox");

        gridLayout->addWidget(keylockComboBox, 5, 1, 1, 1);

        masteMixLabel = new QLabel(DlgPrefSoundDlg);
        masteMixLabel->setObjectName("masteMixLabel");

        gridLayout->addWidget(masteMixLabel, 6, 0, 1, 1);

        mainMixComboBox = new QComboBox(DlgPrefSoundDlg);
        mainMixComboBox->setObjectName("mainMixComboBox");

        gridLayout->addWidget(mainMixComboBox, 6, 1, 1, 1);

        mainOutputModeComboBox = new QComboBox(DlgPrefSoundDlg);
        mainOutputModeComboBox->setObjectName("mainOutputModeComboBox");

        gridLayout->addWidget(mainOutputModeComboBox, 7, 1, 1, 1);

        mainMonoLabel = new QLabel(DlgPrefSoundDlg);
        mainMonoLabel->setObjectName("mainMonoLabel");

        gridLayout->addWidget(mainMonoLabel, 7, 0, 1, 1);

        micMonitorModeComboBox = new QComboBox(DlgPrefSoundDlg);
        micMonitorModeComboBox->setObjectName("micMonitorModeComboBox");

        gridLayout->addWidget(micMonitorModeComboBox, 8, 1, 1, 1);

        micMonitorModeLabel = new QLabel(DlgPrefSoundDlg);
        micMonitorModeLabel->setObjectName("micMonitorModeLabel");
        micMonitorModeLabel->setOpenExternalLinks(true);

        gridLayout->addWidget(micMonitorModeLabel, 8, 0, 1, 1);

        latencyCompensationLabel = new QLabel(DlgPrefSoundDlg);
        latencyCompensationLabel->setObjectName("latencyCompensationLabel");
        latencyCompensationLabel->setOpenExternalLinks(true);

        gridLayout->addWidget(latencyCompensationLabel, 9, 0, 1, 1);

        latencyCompensationSpinBox = new QDoubleSpinBox(DlgPrefSoundDlg);
        latencyCompensationSpinBox->setObjectName("latencyCompensationSpinBox");
        latencyCompensationSpinBox->setDecimals(3);
        latencyCompensationSpinBox->setMaximum(500.000000000000000);
        latencyCompensationSpinBox->setSingleStep(0.100000000000000);

        gridLayout->addWidget(latencyCompensationSpinBox, 9, 1, 1, 1);

        mainDelayLabel = new QLabel(DlgPrefSoundDlg);
        mainDelayLabel->setObjectName("mainDelayLabel");

        gridLayout->addWidget(mainDelayLabel, 11, 0, 1, 1);

        mainDelaySpinBox = new QDoubleSpinBox(DlgPrefSoundDlg);
        mainDelaySpinBox->setObjectName("mainDelaySpinBox");
        mainDelaySpinBox->setDecimals(3);
        mainDelaySpinBox->setMaximum(500.000000000000000);
        mainDelaySpinBox->setSingleStep(0.100000000000000);

        gridLayout->addWidget(mainDelaySpinBox, 11, 1, 1, 1);

        headDelayLabel = new QLabel(DlgPrefSoundDlg);
        headDelayLabel->setObjectName("headDelayLabel");

        gridLayout->addWidget(headDelayLabel, 12, 0, 1, 1);

        headDelaySpinBox = new QDoubleSpinBox(DlgPrefSoundDlg);
        headDelaySpinBox->setObjectName("headDelaySpinBox");
        headDelaySpinBox->setDecimals(3);
        headDelaySpinBox->setMaximum(500.000000000000000);
        headDelaySpinBox->setSingleStep(0.100000000000000);

        gridLayout->addWidget(headDelaySpinBox, 12, 1, 1, 1);

        boothDelayLabel = new QLabel(DlgPrefSoundDlg);
        boothDelayLabel->setObjectName("boothDelayLabel");

        gridLayout->addWidget(boothDelayLabel, 13, 0, 1, 1);

        boothDelaySpinBox = new QDoubleSpinBox(DlgPrefSoundDlg);
        boothDelaySpinBox->setObjectName("boothDelaySpinBox");
        boothDelaySpinBox->setDecimals(3);
        boothDelaySpinBox->setMaximum(500.000000000000000);
        boothDelaySpinBox->setSingleStep(0.100000000000000);

        gridLayout->addWidget(boothDelaySpinBox, 13, 1, 1, 1);

        latencyCompensationWarningLabel = new QLabel(DlgPrefSoundDlg);
        latencyCompensationWarningLabel->setObjectName("latencyCompensationWarningLabel");
        latencyCompensationWarningLabel->setText(QString::fromUtf8("warning goes here"));

        gridLayout->addWidget(latencyCompensationWarningLabel, 14, 0, 1, 2);


        verticalLayout_2->addLayout(gridLayout);

        line_2 = new QFrame(DlgPrefSoundDlg);
        line_2->setObjectName("line_2");
        line_2->setFrameShape(QFrame::Shape::HLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_2->addWidget(line_2);

        buttonsHLayout = new QHBoxLayout();
        buttonsHLayout->setSpacing(6);
        buttonsHLayout->setObjectName("buttonsHLayout");
        queryButton = new QPushButton(DlgPrefSoundDlg);
        queryButton->setObjectName("queryButton");

        buttonsHLayout->addWidget(queryButton);

        buttonsHSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        buttonsHLayout->addItem(buttonsHSpacer);


        verticalLayout_2->addLayout(buttonsHLayout);

        ioTabs = new QTabWidget(DlgPrefSoundDlg);
        ioTabs->setObjectName("ioTabs");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(ioTabs->sizePolicy().hasHeightForWidth());
        ioTabs->setSizePolicy(sizePolicy);
        outputTab = new QWidget();
        outputTab->setObjectName("outputTab");
        outputVLayout = new QVBoxLayout(outputTab);
        outputVLayout->setSpacing(6);
        outputVLayout->setContentsMargins(11, 11, 11, 11);
        outputVLayout->setObjectName("outputVLayout");
        outputVSpacer = new QSpacerItem(20, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::MinimumExpanding);

        outputVLayout->addItem(outputVSpacer);

        ioTabs->addTab(outputTab, QString());
        inputTab = new QWidget();
        inputTab->setObjectName("inputTab");
        inputVLayout = new QVBoxLayout(inputTab);
        inputVLayout->setSpacing(6);
        inputVLayout->setContentsMargins(11, 11, 11, 11);
        inputVLayout->setObjectName("inputVLayout");
        inputVSpacer = new QSpacerItem(20, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::MinimumExpanding);

        inputVLayout->addItem(inputVSpacer);

        ioTabs->addTab(inputTab, QString());

        verticalLayout_2->addWidget(ioTabs);

        Hints = new QGroupBox(DlgPrefSoundDlg);
        Hints->setObjectName("Hints");
        gridLayout_6 = new QGridLayout(Hints);
        gridLayout_6->setSpacing(6);
        gridLayout_6->setContentsMargins(11, 11, 11, 11);
        gridLayout_6->setObjectName("gridLayout_6");
        deckBusHint = new QLabel(Hints);
        deckBusHint->setObjectName("deckBusHint");
        deckBusHint->setWordWrap(true);
        deckBusHint->setOpenExternalLinks(true);

        gridLayout_6->addWidget(deckBusHint, 0, 0, 1, 2);

        underflowCounterHint = new QLabel(Hints);
        underflowCounterHint->setObjectName("underflowCounterHint");
        underflowCounterHint->setWordWrap(true);

        gridLayout_6->addWidget(underflowCounterHint, 1, 0, 1, 2);

        audioBufferHint = new QLabel(Hints);
        audioBufferHint->setObjectName("audioBufferHint");
        audioBufferHint->setWordWrap(true);

        gridLayout_6->addWidget(audioBufferHint, 2, 0, 1, 2);

        realtimeHint = new QLabel(Hints);
        realtimeHint->setObjectName("realtimeHint");
        realtimeHint->setWordWrap(true);
        realtimeHint->setOpenExternalLinks(true);

        gridLayout_6->addWidget(realtimeHint, 3, 0, 1, 2);

        hardwareGuide = new QLabel(Hints);
        hardwareGuide->setObjectName("hardwareGuide");
        hardwareGuide->setWordWrap(true);
        hardwareGuide->setOpenExternalLinks(true);

        gridLayout_6->addWidget(hardwareGuide, 4, 0, 1, 2);

        line = new QFrame(Hints);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayout_6->addWidget(line, 5, 0, 1, 2);

        latencyLabel = new QLabel(Hints);
        latencyLabel->setObjectName("latencyLabel");

        gridLayout_6->addWidget(latencyLabel, 6, 0, 1, 1);

        currentLatency = new QLabel(Hints);
        currentLatency->setObjectName("currentLatency");

        gridLayout_6->addWidget(currentLatency, 6, 1, 1, 1);

        underflowLabel = new QLabel(Hints);
        underflowLabel->setObjectName("underflowLabel");

        gridLayout_6->addWidget(underflowLabel, 7, 0, 1, 1);

        bufferUnderflowCount = new QLabel(Hints);
        bufferUnderflowCount->setObjectName("bufferUnderflowCount");

        gridLayout_6->addWidget(bufferUnderflowCount, 7, 1, 1, 1);


        verticalLayout_2->addWidget(Hints);

#if QT_CONFIG(shortcut)
        apiLabel->setBuddy(apiComboBox);
        sampleRateLabel->setBuddy(sampleRateComboBox);
        audioBufferLabel->setBuddy(audioBufferComboBox);
        keylockLabel->setBuddy(keylockComboBox);
        latencyLabel->setBuddy(audioBufferComboBox);
        underflowLabel->setBuddy(audioBufferComboBox);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(apiComboBox, sampleRateComboBox);
        QWidget::setTabOrder(sampleRateComboBox, audioBufferComboBox);
        QWidget::setTabOrder(audioBufferComboBox, deviceSyncComboBox);
        QWidget::setTabOrder(deviceSyncComboBox, engineClockComboBox);
        QWidget::setTabOrder(engineClockComboBox, keylockComboBox);
        QWidget::setTabOrder(keylockComboBox, mainMixComboBox);
        QWidget::setTabOrder(mainMixComboBox, mainOutputModeComboBox);
        QWidget::setTabOrder(mainOutputModeComboBox, micMonitorModeComboBox);
        QWidget::setTabOrder(micMonitorModeComboBox, latencyCompensationSpinBox);
        QWidget::setTabOrder(latencyCompensationSpinBox, mainDelaySpinBox);
        QWidget::setTabOrder(mainDelaySpinBox, headDelaySpinBox);
        QWidget::setTabOrder(headDelaySpinBox, boothDelaySpinBox);
        QWidget::setTabOrder(boothDelaySpinBox, queryButton);
        QWidget::setTabOrder(queryButton, ioTabs);

        retranslateUi(DlgPrefSoundDlg);

        ioTabs->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(DlgPrefSoundDlg);
    } // setupUi

    void retranslateUi(QWidget *DlgPrefSoundDlg)
    {
        DlgPrefSoundDlg->setWindowTitle(QCoreApplication::translate("DlgPrefSoundDlg", "Sound Hardware Preferences", nullptr));
        apiLabel->setText(QCoreApplication::translate("DlgPrefSoundDlg", "Sound API", nullptr));
        sampleRateLabel->setText(QCoreApplication::translate("DlgPrefSoundDlg", "Sample Rate", nullptr));
        audioBufferLabel->setText(QCoreApplication::translate("DlgPrefSoundDlg", "Audio Buffer", nullptr));
        deviceSyncLabel->setText(QCoreApplication::translate("DlgPrefSoundDlg", "Multi-Soundcard Synchronization", nullptr));
        engineClockLabel->setText(QCoreApplication::translate("DlgPrefSoundDlg", "Engine Clock", nullptr));
#if QT_CONFIG(tooltip)
        engineClockComboBox->setToolTip(QCoreApplication::translate("DlgPrefSoundDlg", "Use soundcard clock for live audience setups and lowest latency.<br>Use network clock for broadcasting without a live audience.", nullptr));
#endif // QT_CONFIG(tooltip)
        keylockLabel->setText(QCoreApplication::translate("DlgPrefSoundDlg", "Keylock/Pitch-Bending Engine", nullptr));
        masteMixLabel->setText(QCoreApplication::translate("DlgPrefSoundDlg", "Main Mix", nullptr));
        mainMonoLabel->setText(QCoreApplication::translate("DlgPrefSoundDlg", "Main Output Mode", nullptr));
        micMonitorModeLabel->setText(QCoreApplication::translate("DlgPrefSoundDlg", "Microphone Monitor Mode", nullptr));
        latencyCompensationLabel->setText(QCoreApplication::translate("DlgPrefSoundDlg", "Microphone Latency Compensation", nullptr));
        latencyCompensationSpinBox->setSuffix(QCoreApplication::translate("DlgPrefSoundDlg", " ms", nullptr));
        mainDelayLabel->setText(QCoreApplication::translate("DlgPrefSoundDlg", "Main Output Delay", nullptr));
        mainDelaySpinBox->setSuffix(QCoreApplication::translate("DlgPrefSoundDlg", " ms", nullptr));
        headDelayLabel->setText(QCoreApplication::translate("DlgPrefSoundDlg", "Headphone Output Delay", nullptr));
        headDelaySpinBox->setSuffix(QCoreApplication::translate("DlgPrefSoundDlg", " ms", nullptr));
        boothDelayLabel->setText(QCoreApplication::translate("DlgPrefSoundDlg", "Booth Output Delay", nullptr));
        boothDelaySpinBox->setSuffix(QCoreApplication::translate("DlgPrefSoundDlg", " ms", nullptr));
        queryButton->setText(QCoreApplication::translate("DlgPrefSoundDlg", "Query Devices", nullptr));
        ioTabs->setTabText(ioTabs->indexOf(outputTab), QCoreApplication::translate("DlgPrefSoundDlg", "Output", nullptr));
        ioTabs->setTabText(ioTabs->indexOf(inputTab), QCoreApplication::translate("DlgPrefSoundDlg", "Input", nullptr));
        Hints->setTitle(QCoreApplication::translate("DlgPrefSoundDlg", "Hints and Diagnostics", nullptr));
        deckBusHint->setText(QCoreApplication::translate("DlgPrefSoundDlg", "Deck and Bus outputs are for external mixers. They are post-fader and include effects and crossfader (for Auto DJ). For external mixing, make sure all Mixxx faders and EQ knobs are set to their default position (right- or double-click). ", nullptr));
        underflowCounterHint->setText(QCoreApplication::translate("DlgPrefSoundDlg", "Enlarge your audio buffer if the underflow counter is increasing or you hear pops during playback.", nullptr));
        audioBufferHint->setText(QCoreApplication::translate("DlgPrefSoundDlg", "Downsize your audio buffer to improve Mixxx's responsiveness.", nullptr));
        latencyLabel->setText(QCoreApplication::translate("DlgPrefSoundDlg", "System Reported Latency", nullptr));
        currentLatency->setText(QCoreApplication::translate("DlgPrefSoundDlg", "20 ms", nullptr));
        underflowLabel->setText(QCoreApplication::translate("DlgPrefSoundDlg", "Buffer Underflow Count", nullptr));
        bufferUnderflowCount->setText(QCoreApplication::translate("DlgPrefSoundDlg", "0", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgPrefSoundDlg: public Ui_DlgPrefSoundDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLGPREFSOUNDDLG_H
