/********************************************************************************
** Form generated from reading UI file 'dlgaboutdlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLGABOUTDLG_H
#define UI_DLGABOUTDLG_H

#include <QtCore/QVariant>
#include <QtSvgWidgets/QSvgWidget>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DlgAboutDlg
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QSvgWidget *mixxx_icon;
    QSvgWidget *mixxx_logo;
    QSpacerItem *horizontalSpacer;
    QWidget *widget;
    QVBoxLayout *verticalLayout_2;
    QLabel *version_label;
    QGridLayout *gridLayout;
    QLabel *label;
    QLabel *git_version_label;
    QLabel *label_2;
    QLabel *qt_version_label;
    QLabel *label_3;
    QLabel *date_label;
    QLabel *label_4;
    QLabel *platform_label;
    QTabWidget *tabs;
    QWidget *creditsTab;
    QGridLayout *creditsTabLayout;
    QTextBrowser *textBrowser;
    QWidget *licenseTab;
    QGridLayout *licenseTabLayout;
    QTextBrowser *licenseText;
    QHBoxLayout *horizontalLayout_2;
    QLabel *textWebsiteLink;
    QSpacerItem *horizontalSpacer_2;
    QPushButton *btnDonate;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgAboutDlg)
    {
        if (DlgAboutDlg->objectName().isEmpty())
            DlgAboutDlg->setObjectName("DlgAboutDlg");
        DlgAboutDlg->resize(494, 513);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(DlgAboutDlg->sizePolicy().hasHeightForWidth());
        DlgAboutDlg->setSizePolicy(sizePolicy);
        DlgAboutDlg->setMinimumSize(QSize(0, 0));
        verticalLayout = new QVBoxLayout(DlgAboutDlg);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        mixxx_icon = new QSvgWidget(DlgAboutDlg);
        mixxx_icon->setObjectName("mixxx_icon");
        mixxx_icon->setMaximumSize(QSize(32, 32));

        horizontalLayout->addWidget(mixxx_icon);

        mixxx_logo = new QSvgWidget(DlgAboutDlg);
        mixxx_logo->setObjectName("mixxx_logo");
        mixxx_logo->setMaximumSize(QSize(170, 68));

        horizontalLayout->addWidget(mixxx_logo);

        horizontalSpacer = new QSpacerItem(1, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout);

        widget = new QWidget(DlgAboutDlg);
        widget->setObjectName("widget");
        verticalLayout_2 = new QVBoxLayout(widget);
        verticalLayout_2->setObjectName("verticalLayout_2");
        version_label = new QLabel(widget);
        version_label->setObjectName("version_label");
        version_label->setText(QString::fromUtf8("Mixxx 2.x.x"));
        version_label->setTextInteractionFlags(Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        verticalLayout_2->addWidget(version_label);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        label = new QLabel(widget);
        label->setObjectName("label");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(label, 0, 0, 1, 1);

        git_version_label = new QLabel(widget);
        git_version_label->setObjectName("git_version_label");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(git_version_label->sizePolicy().hasHeightForWidth());
        git_version_label->setSizePolicy(sizePolicy2);
        git_version_label->setTextInteractionFlags(Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        gridLayout->addWidget(git_version_label, 0, 1, 1, 1);

        label_2 = new QLabel(widget);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        qt_version_label = new QLabel(widget);
        qt_version_label->setObjectName("qt_version_label");
        qt_version_label->setTextInteractionFlags(Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        gridLayout->addWidget(qt_version_label, 1, 1, 1, 1);

        label_3 = new QLabel(widget);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 2, 0, 1, 1);

        date_label = new QLabel(widget);
        date_label->setObjectName("date_label");
        date_label->setTextInteractionFlags(Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        gridLayout->addWidget(date_label, 2, 1, 1, 1);

        label_4 = new QLabel(widget);
        label_4->setObjectName("label_4");

        gridLayout->addWidget(label_4, 3, 0, 1, 1);

        platform_label = new QLabel(widget);
        platform_label->setObjectName("platform_label");
        platform_label->setTextInteractionFlags(Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        gridLayout->addWidget(platform_label, 3, 1, 1, 1);


        verticalLayout_2->addLayout(gridLayout);


        verticalLayout->addWidget(widget);

        tabs = new QTabWidget(DlgAboutDlg);
        tabs->setObjectName("tabs");
        tabs->setDocumentMode(false);
        tabs->setMovable(false);
        creditsTab = new QWidget();
        creditsTab->setObjectName("creditsTab");
        creditsTabLayout = new QGridLayout(creditsTab);
        creditsTabLayout->setObjectName("creditsTabLayout");
        creditsTabLayout->setContentsMargins(12, 12, 12, 12);
        textBrowser = new QTextBrowser(creditsTab);
        textBrowser->setObjectName("textBrowser");
        textBrowser->setHtml(QString::fromUtf8("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Cantarell'; font-size:11pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:'Lucida Grande'; font-size:13pt;\">            </span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:'Sans Serif'; font-size:9pt;\">Credits go here</span><span style=\" font-family:'Lucida Grande'; font-size:13pt;\">            </span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:'Sans Serif'; font-size:9pt;\""
                        "><br /></p></body></html>"));
        textBrowser->setTextInteractionFlags(Qt::NoTextInteraction);

        creditsTabLayout->addWidget(textBrowser, 0, 0, 1, 1);

        tabs->addTab(creditsTab, QString());
        licenseTab = new QWidget();
        licenseTab->setObjectName("licenseTab");
        licenseTabLayout = new QGridLayout(licenseTab);
        licenseTabLayout->setObjectName("licenseTabLayout");
        licenseTabLayout->setContentsMargins(12, 12, 12, 12);
        licenseText = new QTextBrowser(licenseTab);
        licenseText->setObjectName("licenseText");
        licenseText->setTextInteractionFlags(Qt::NoTextInteraction);

        licenseTabLayout->addWidget(licenseText, 0, 0, 1, 1);

        tabs->addTab(licenseTab, QString());

        verticalLayout->addWidget(tabs);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        textWebsiteLink = new QLabel(DlgAboutDlg);
        textWebsiteLink->setObjectName("textWebsiteLink");
        textWebsiteLink->setOpenExternalLinks(true);
        textWebsiteLink->setTextInteractionFlags(Qt::LinksAccessibleByMouse);

        horizontalLayout_2->addWidget(textWebsiteLink);

        horizontalSpacer_2 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);

        btnDonate = new QPushButton(DlgAboutDlg);
        btnDonate->setObjectName("btnDonate");

        horizontalLayout_2->addWidget(btnDonate);

        buttonBox = new QDialogButtonBox(DlgAboutDlg);
        buttonBox->setObjectName("buttonBox");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(buttonBox->sizePolicy().hasHeightForWidth());
        buttonBox->setSizePolicy(sizePolicy3);
        buttonBox->setLayoutDirection(Qt::LeftToRight);
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Ok);

        horizontalLayout_2->addWidget(buttonBox);


        verticalLayout->addLayout(horizontalLayout_2);


        retranslateUi(DlgAboutDlg);

        tabs->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(DlgAboutDlg);
    } // setupUi

    void retranslateUi(QDialog *DlgAboutDlg)
    {
        DlgAboutDlg->setWindowTitle(QCoreApplication::translate("DlgAboutDlg", "About Mixxx", nullptr));
        label->setText(QCoreApplication::translate("DlgAboutDlg", "Git Version:", nullptr));
        git_version_label->setText(QCoreApplication::translate("DlgAboutDlg", "Unknown", nullptr));
        label_2->setText(QCoreApplication::translate("DlgAboutDlg", "Qt Version:", nullptr));
        qt_version_label->setText(QCoreApplication::translate("DlgAboutDlg", "Unknown", nullptr));
        label_3->setText(QCoreApplication::translate("DlgAboutDlg", "Date:", nullptr));
        date_label->setText(QCoreApplication::translate("DlgAboutDlg", "Unknown", nullptr));
        label_4->setText(QCoreApplication::translate("DlgAboutDlg", "Platform:", nullptr));
        platform_label->setText(QCoreApplication::translate("DlgAboutDlg", "Unknown", nullptr));
        tabs->setTabText(tabs->indexOf(creditsTab), QCoreApplication::translate("DlgAboutDlg", "Credits", nullptr));
        tabs->setTabText(tabs->indexOf(licenseTab), QCoreApplication::translate("DlgAboutDlg", "License", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgAboutDlg: public Ui_DlgAboutDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLGABOUTDLG_H
