/****************************************************************************
** Meta object code from reading C++ file 'setlogfeature.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/library/trackset/setlogfeature.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QSet>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'setlogfeature.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13SetlogFeatureE_t {};
} // unnamed namespace

template <> constexpr inline auto SetlogFeature::qt_create_metaobjectdata<qt_meta_tag_ZN13SetlogFeatureE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SetlogFeature",
        "onRightClick",
        "",
        "QPoint",
        "globalPos",
        "onRightClickChild",
        "QModelIndex",
        "index",
        "slotJoinWithPrevious",
        "slotMarkAllTracksPlayed",
        "slotLockAllChildPlaylists",
        "slotUnlockAllChildPlaylists",
        "slotDeletePlaylist",
        "slotGetNewPlaylist",
        "activate",
        "activateChild",
        "slotPlayingTrackChanged",
        "TrackPointer",
        "currentPlayingTrack",
        "slotPlaylistTableChanged",
        "playlistId",
        "slotPlaylistContentOrLockChanged",
        "QSet<int>",
        "playlistIds",
        "slotPlaylistTableRenamed",
        "newName",
        "slotDeleteAllUnlockedChildPlaylists"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'onRightClick'
        QtMocHelpers::SlotData<void(const QPoint &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'onRightClickChild'
        QtMocHelpers::SlotData<void(const QPoint &, const QModelIndex &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 6, 7 },
        }}),
        // Slot 'slotJoinWithPrevious'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotMarkAllTracksPlayed'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotLockAllChildPlaylists'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotUnlockAllChildPlaylists'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotDeletePlaylist'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotGetNewPlaylist'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'activate'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'activateChild'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'slotPlayingTrackChanged'
        QtMocHelpers::SlotData<void(TrackPointer)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'slotPlaylistTableChanged'
        QtMocHelpers::SlotData<void(int)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 20 },
        }}),
        // Slot 'slotPlaylistContentOrLockChanged'
        QtMocHelpers::SlotData<void(const QSet<int> &)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'slotPlaylistTableRenamed'
        QtMocHelpers::SlotData<void(int, const QString &)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 20 }, { QMetaType::QString, 25 },
        }}),
        // Slot 'slotDeleteAllUnlockedChildPlaylists'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SetlogFeature, qt_meta_tag_ZN13SetlogFeatureE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SetlogFeature::staticMetaObject = { {
    QMetaObject::SuperData::link<BasePlaylistFeature::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13SetlogFeatureE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13SetlogFeatureE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13SetlogFeatureE_t>.metaTypes,
    nullptr
} };

void SetlogFeature::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SetlogFeature *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->onRightClick((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 1: _t->onRightClickChild((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 2: _t->slotJoinWithPrevious(); break;
        case 3: _t->slotMarkAllTracksPlayed(); break;
        case 4: _t->slotLockAllChildPlaylists(); break;
        case 5: _t->slotUnlockAllChildPlaylists(); break;
        case 6: _t->slotDeletePlaylist(); break;
        case 7: _t->slotGetNewPlaylist(); break;
        case 8: _t->activate(); break;
        case 9: _t->activateChild((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 10: _t->slotPlayingTrackChanged((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1]))); break;
        case 11: _t->slotPlaylistTableChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->slotPlaylistContentOrLockChanged((*reinterpret_cast<std::add_pointer_t<QSet<int>>>(_a[1]))); break;
        case 13: _t->slotPlaylistTableRenamed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 14: _t->slotDeleteAllUnlockedChildPlaylists(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<int> >(); break;
            }
            break;
        }
    }
}

const QMetaObject *SetlogFeature::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SetlogFeature::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13SetlogFeatureE_t>.strings))
        return static_cast<void*>(this);
    return BasePlaylistFeature::qt_metacast(_clname);
}

int SetlogFeature::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = BasePlaylistFeature::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    return _id;
}
QT_WARNING_POP
