/****************************************************************************
** Meta object code from reading C++ file 'dlganalysis.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/library/analysis/dlganalysis.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dlganalysis.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11DlgAnalysisE_t {};
} // unnamed namespace

template <> constexpr inline auto DlgAnalysis::qt_create_metaobjectdata<qt_meta_tag_ZN11DlgAnalysisE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DlgAnalysis",
        "loadTrack",
        "",
        "TrackPointer",
        "pTrack",
        "loadTrackToPlayer",
        "player",
        "analyzeTracks",
        "QList<AnalyzerScheduledTrack>",
        "tracks",
        "stopAnalysis",
        "trackSelected",
        "tableSelectionChanged",
        "QItemSelection",
        "selected",
        "deselected",
        "selectAll",
        "analyze",
        "slotAnalysisActive",
        "bActive",
        "onTrackAnalysisSchedulerProgress",
        "AnalyzerProgress",
        "analyzerProgress",
        "finishedCount",
        "totalCount",
        "onTrackAnalysisSchedulerFinished",
        "slotShowRecentSongs",
        "slotShowAllSongs",
        "installEventFilter",
        "pFilter"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'loadTrack'
        QtMocHelpers::SignalData<void(TrackPointer)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'loadTrackToPlayer'
        QtMocHelpers::SignalData<void(TrackPointer, const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 6 },
        }}),
        // Signal 'analyzeTracks'
        QtMocHelpers::SignalData<void(const QList<AnalyzerScheduledTrack> &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'stopAnalysis'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'trackSelected'
        QtMocHelpers::SignalData<void(TrackPointer)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'tableSelectionChanged'
        QtMocHelpers::SlotData<void(const QItemSelection &, const QItemSelection &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { 0x80000000 | 13, 15 },
        }}),
        // Slot 'selectAll'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'analyze'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotAnalysisActive'
        QtMocHelpers::SlotData<void(bool)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 19 },
        }}),
        // Slot 'onTrackAnalysisSchedulerProgress'
        QtMocHelpers::SlotData<void(AnalyzerProgress, int, int)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 22 }, { QMetaType::Int, 23 }, { QMetaType::Int, 24 },
        }}),
        // Slot 'onTrackAnalysisSchedulerFinished'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotShowRecentSongs'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotShowAllSongs'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'installEventFilter'
        QtMocHelpers::SlotData<void(QObject *)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QObjectStar, 29 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DlgAnalysis, qt_meta_tag_ZN11DlgAnalysisE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DlgAnalysis::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11DlgAnalysisE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11DlgAnalysisE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11DlgAnalysisE_t>.metaTypes,
    nullptr
} };

void DlgAnalysis::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DlgAnalysis *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->loadTrack((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1]))); break;
        case 1: _t->loadTrackToPlayer((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 2: _t->analyzeTracks((*reinterpret_cast<std::add_pointer_t<QList<AnalyzerScheduledTrack>>>(_a[1]))); break;
        case 3: _t->stopAnalysis(); break;
        case 4: _t->trackSelected((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1]))); break;
        case 5: _t->tableSelectionChanged((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[2]))); break;
        case 6: _t->selectAll(); break;
        case 7: _t->analyze(); break;
        case 8: _t->slotAnalysisActive((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->onTrackAnalysisSchedulerProgress((*reinterpret_cast<std::add_pointer_t<AnalyzerProgress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 10: _t->onTrackAnalysisSchedulerFinished(); break;
        case 11: _t->slotShowRecentSongs(); break;
        case 12: _t->slotShowAllSongs(); break;
        case 13: _t->installEventFilter((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AnalyzerProgress >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DlgAnalysis::*)(TrackPointer )>(_a, &DlgAnalysis::loadTrack, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgAnalysis::*)(TrackPointer , const QString & )>(_a, &DlgAnalysis::loadTrackToPlayer, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgAnalysis::*)(const QList<AnalyzerScheduledTrack> & )>(_a, &DlgAnalysis::analyzeTracks, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgAnalysis::*)()>(_a, &DlgAnalysis::stopAnalysis, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgAnalysis::*)(TrackPointer )>(_a, &DlgAnalysis::trackSelected, 4))
            return;
    }
}

const QMetaObject *DlgAnalysis::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DlgAnalysis::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11DlgAnalysisE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::DlgAnalysis"))
        return static_cast< Ui::DlgAnalysis*>(this);
    if (!strcmp(_clname, "LibraryView"))
        return static_cast< LibraryView*>(this);
    return QWidget::qt_metacast(_clname);
}

int DlgAnalysis::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void DlgAnalysis::loadTrack(TrackPointer _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void DlgAnalysis::loadTrackToPlayer(TrackPointer _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void DlgAnalysis::analyzeTracks(const QList<AnalyzerScheduledTrack> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void DlgAnalysis::stopAnalysis()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void DlgAnalysis::trackSelected(TrackPointer _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
