/****************************************************************************
** Meta object code from reading C++ file 'dlgtrackinfo.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/library/dlgtrackinfo.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dlgtrackinfo.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12DlgTrackInfoE_t {};
} // unnamed namespace

template <> constexpr inline auto DlgTrackInfo::qt_create_metaobjectdata<qt_meta_tag_ZN12DlgTrackInfoE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DlgTrackInfo",
        "next",
        "",
        "previous",
        "loadTrack",
        "TrackPointer",
        "pTrack",
        "QModelIndex",
        "index",
        "focusField",
        "property",
        "slotNextButton",
        "slotPrevButton",
        "slotNextDlgTagFetcher",
        "slotPrevDlgTagFetcher",
        "slotOk",
        "slotApply",
        "slotCancel",
        "slotBpmScale",
        "mixxx::Beats::BpmScale",
        "bpmScale",
        "slotBpmClear",
        "slotBpmConstChanged",
        "Qt::CheckState",
        "state",
        "slotBpmTap",
        "averageLength",
        "numSamples",
        "slotSpinBpmValueChanged",
        "value",
        "slotKeyTextChanged",
        "slotRatingChanged",
        "rating",
        "slotImportMetadataFromFile",
        "slotImportMetadataFromMusicBrainz",
        "slotTrackChanged",
        "TrackId",
        "trackId",
        "slotOpenInFileBrowser",
        "slotColorButtonClicked",
        "slotCoverFound",
        "const QObject*",
        "pRequester",
        "CoverInfo",
        "info",
        "QPixmap",
        "pixmap",
        "slotCoverInfoSelected",
        "CoverInfoRelative",
        "coverInfo",
        "slotReloadCoverArt"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'next'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'previous'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'loadTrack'
        QtMocHelpers::SlotData<void(TrackPointer)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'loadTrack'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'focusField'
        QtMocHelpers::SlotData<void(const QString &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Slot 'slotNextButton'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotPrevButton'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotNextDlgTagFetcher'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotPrevDlgTagFetcher'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotOk'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotApply'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotCancel'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotBpmScale'
        QtMocHelpers::SlotData<void(mixxx::Beats::BpmScale)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Slot 'slotBpmClear'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotBpmConstChanged'
        QtMocHelpers::SlotData<void(Qt::CheckState)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Slot 'slotBpmTap'
        QtMocHelpers::SlotData<void(double, int)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 26 }, { QMetaType::Int, 27 },
        }}),
        // Slot 'slotSpinBpmValueChanged'
        QtMocHelpers::SlotData<void(double)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 29 },
        }}),
        // Slot 'slotKeyTextChanged'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotRatingChanged'
        QtMocHelpers::SlotData<void(int)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 32 },
        }}),
        // Slot 'slotImportMetadataFromFile'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotImportMetadataFromMusicBrainz'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotTrackChanged'
        QtMocHelpers::SlotData<void(TrackId)>(35, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 36, 37 },
        }}),
        // Slot 'slotOpenInFileBrowser'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotColorButtonClicked'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotCoverFound'
        QtMocHelpers::SlotData<void(const QObject *, const CoverInfo &, const QPixmap &)>(40, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 41, 42 }, { 0x80000000 | 43, 44 }, { 0x80000000 | 45, 46 },
        }}),
        // Slot 'slotCoverInfoSelected'
        QtMocHelpers::SlotData<void(const CoverInfoRelative &)>(47, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 48, 49 },
        }}),
        // Slot 'slotReloadCoverArt'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DlgTrackInfo, qt_meta_tag_ZN12DlgTrackInfoE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DlgTrackInfo::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12DlgTrackInfoE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12DlgTrackInfoE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12DlgTrackInfoE_t>.metaTypes,
    nullptr
} };

void DlgTrackInfo::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DlgTrackInfo *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->next(); break;
        case 1: _t->previous(); break;
        case 2: _t->loadTrack((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1]))); break;
        case 3: _t->loadTrack((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 4: _t->focusField((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->slotNextButton(); break;
        case 6: _t->slotPrevButton(); break;
        case 7: _t->slotNextDlgTagFetcher(); break;
        case 8: _t->slotPrevDlgTagFetcher(); break;
        case 9: _t->slotOk(); break;
        case 10: _t->slotApply(); break;
        case 11: _t->slotCancel(); break;
        case 12: _t->slotBpmScale((*reinterpret_cast<std::add_pointer_t<mixxx::Beats::BpmScale>>(_a[1]))); break;
        case 13: _t->slotBpmClear(); break;
        case 14: _t->slotBpmConstChanged((*reinterpret_cast<std::add_pointer_t<Qt::CheckState>>(_a[1]))); break;
        case 15: _t->slotBpmTap((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 16: _t->slotSpinBpmValueChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 17: _t->slotKeyTextChanged(); break;
        case 18: _t->slotRatingChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 19: _t->slotImportMetadataFromFile(); break;
        case 20: _t->slotImportMetadataFromMusicBrainz(); break;
        case 21: _t->slotTrackChanged((*reinterpret_cast<std::add_pointer_t<TrackId>>(_a[1]))); break;
        case 22: _t->slotOpenInFileBrowser(); break;
        case 23: _t->slotColorButtonClicked(); break;
        case 24: _t->slotCoverFound((*reinterpret_cast<std::add_pointer_t<const QObject*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CoverInfo>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QPixmap>>(_a[3]))); break;
        case 25: _t->slotCoverInfoSelected((*reinterpret_cast<std::add_pointer_t<CoverInfoRelative>>(_a[1]))); break;
        case 26: _t->slotReloadCoverArt(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackId >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DlgTrackInfo::*)()>(_a, &DlgTrackInfo::next, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgTrackInfo::*)()>(_a, &DlgTrackInfo::previous, 1))
            return;
    }
}

const QMetaObject *DlgTrackInfo::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DlgTrackInfo::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12DlgTrackInfoE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::DlgTrackInfo"))
        return static_cast< Ui::DlgTrackInfo*>(this);
    return QDialog::qt_metacast(_clname);
}

int DlgTrackInfo::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    return _id;
}

// SIGNAL 0
void DlgTrackInfo::next()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void DlgTrackInfo::previous()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
