/********************************************************************************
** Form generated from reading UI file 'dlgtrackinfo.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLGTRACKINFO_H
#define UI_DLGTRACKINFO_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DlgTrackInfo
{
public:
    QVBoxLayout *verticalLayout_2;
    QTabWidget *tabWidget;
    QWidget *tabSummary;
    QVBoxLayout *summary_layout;
    QGroupBox *tags_groupBox;
    QGridLayout *tags_layout;
    QLabel *lblTitle;
    QLineEdit *txtTitle;
    QWidget *coverWidget;
    QHBoxLayout *coverLayout;
    QLabel *lblArtist;
    QLineEdit *txtArtist;
    QLabel *lblAlbum;
    QLineEdit *txtAlbum;
    QLabel *lblAlbumArtist;
    QLineEdit *txtAlbumArtist;
    QWidget *verticalWidgetStars;
    QHBoxLayout *starsLayout;
    QLabel *lblComposer;
    QLineEdit *txtComposer;
    QLabel *lblYear;
    QLineEdit *txtYear;
    QLabel *lblGenre;
    QLineEdit *txtGenre;
    QLabel *lblKey;
    QLineEdit *txtKey;
    QLabel *lblGrouping;
    QLineEdit *txtGrouping;
    QLabel *lblTrackNumber;
    QLineEdit *txtTrackNumber;
    QLabel *lblTrackComment;
    QPlainTextEdit *txtComment;
    QHBoxLayout *horizontalLayout_3;
    QPushButton *btnImportMetadataFromMusicBrainz;
    QPushButton *btnImportMetadataFromFile;
    QSpacerItem *horizontalSpacer_4;
    QLabel *lblTrackColor;
    QPushButton *btnColorPicker;
    QGroupBox *properties_groupBox;
    QGridLayout *properties_layout;
    QLabel *lblDuration;
    QLabel *txtDuration;
    QLabel *lblType;
    QLabel *txtType;
    QLabel *lblBpm;
    QLabel *txtBpm;
    QLabel *lblBitrate;
    QLabel *txtBitrate;
    QLabel *lblDateAdded;
    QLabel *txtDateAdded;
    QLabel *lblSamplerate;
    QLabel *txtSamplerate;
    QLabel *lblReplayGain;
    QLabel *txtReplayGain;
    QLabel *lblLocation;
    QLabel *txtLocation;
    QHBoxLayout *openFileBrowser_layout;
    QPushButton *btnOpenFileBrowser;
    QSpacerItem *horizontalSpacer_6;
    QSpacerItem *verticalSpacer_2;
    QWidget *tabBPM;
    QVBoxLayout *verticalLayout_4;
    QHBoxLayout *horizontalLayout_4;
    QGridLayout *gridLayout;
    QLabel *labelBpm;
    QDoubleSpinBox *spinBpm;
    QCheckBox *bpmConst;
    QPushButton *bpmDouble;
    QPushButton *bpmHalve;
    QPushButton *bpmTwoThirds;
    QPushButton *bpmThreeFourth;
    QPushButton *bpmThreeHalves;
    QPushButton *bpmFourThirds;
    QPushButton *bpmTap;
    QFrame *line;
    QPushButton *bpmClear;
    QLabel *label;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *buttonLayout;
    QPushButton *btnPrev;
    QPushButton *btnNext;
    QSpacerItem *horizontalSpacer;
    QPushButton *btnCancel;
    QPushButton *btnApply;
    QPushButton *btnOK;

    void setupUi(QDialog *DlgTrackInfo)
    {
        if (DlgTrackInfo->objectName().isEmpty())
            DlgTrackInfo->setObjectName("DlgTrackInfo");
        DlgTrackInfo->resize(575, 642);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(DlgTrackInfo->sizePolicy().hasHeightForWidth());
        DlgTrackInfo->setSizePolicy(sizePolicy);
        DlgTrackInfo->setSizeGripEnabled(true);
        verticalLayout_2 = new QVBoxLayout(DlgTrackInfo);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        tabWidget = new QTabWidget(DlgTrackInfo);
        tabWidget->setObjectName("tabWidget");
        sizePolicy.setHeightForWidth(tabWidget->sizePolicy().hasHeightForWidth());
        tabWidget->setSizePolicy(sizePolicy);
        tabWidget->setMinimumSize(QSize(0, 0));
        tabSummary = new QWidget();
        tabSummary->setObjectName("tabSummary");
        summary_layout = new QVBoxLayout(tabSummary);
        summary_layout->setSpacing(6);
        summary_layout->setContentsMargins(11, 11, 11, 11);
        summary_layout->setObjectName("summary_layout");
        tags_groupBox = new QGroupBox(tabSummary);
        tags_groupBox->setObjectName("tags_groupBox");
        tags_layout = new QGridLayout(tags_groupBox);
        tags_layout->setSpacing(6);
        tags_layout->setContentsMargins(11, 11, 11, 11);
        tags_layout->setObjectName("tags_layout");
        lblTitle = new QLabel(tags_groupBox);
        lblTitle->setObjectName("lblTitle");
        sizePolicy.setHeightForWidth(lblTitle->sizePolicy().hasHeightForWidth());
        lblTitle->setSizePolicy(sizePolicy);
        QFont font;
        font.setItalic(false);
        lblTitle->setFont(font);
        lblTitle->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        tags_layout->addWidget(lblTitle, 0, 0, 1, 1);

        txtTitle = new QLineEdit(tags_groupBox);
        txtTitle->setObjectName("txtTitle");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(txtTitle->sizePolicy().hasHeightForWidth());
        txtTitle->setSizePolicy(sizePolicy1);
        txtTitle->setMinimumSize(QSize(0, 0));

        tags_layout->addWidget(txtTitle, 0, 1, 1, 1);

        coverWidget = new QWidget(tags_groupBox);
        coverWidget->setObjectName("coverWidget");
        sizePolicy.setHeightForWidth(coverWidget->sizePolicy().hasHeightForWidth());
        coverWidget->setSizePolicy(sizePolicy);
        coverLayout = new QHBoxLayout(coverWidget);
        coverLayout->setSpacing(0);
        coverLayout->setContentsMargins(11, 11, 11, 11);
        coverLayout->setObjectName("coverLayout");
        coverLayout->setSizeConstraint(QLayout::SetDefaultConstraint);

        tags_layout->addWidget(coverWidget, 0, 2, 3, 2);

        lblArtist = new QLabel(tags_groupBox);
        lblArtist->setObjectName("lblArtist");
        sizePolicy.setHeightForWidth(lblArtist->sizePolicy().hasHeightForWidth());
        lblArtist->setSizePolicy(sizePolicy);
        lblArtist->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        tags_layout->addWidget(lblArtist, 1, 0, 1, 1);

        txtArtist = new QLineEdit(tags_groupBox);
        txtArtist->setObjectName("txtArtist");
        sizePolicy1.setHeightForWidth(txtArtist->sizePolicy().hasHeightForWidth());
        txtArtist->setSizePolicy(sizePolicy1);
        txtArtist->setMinimumSize(QSize(0, 0));

        tags_layout->addWidget(txtArtist, 1, 1, 1, 1);

        lblAlbum = new QLabel(tags_groupBox);
        lblAlbum->setObjectName("lblAlbum");
        sizePolicy.setHeightForWidth(lblAlbum->sizePolicy().hasHeightForWidth());
        lblAlbum->setSizePolicy(sizePolicy);
        lblAlbum->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        tags_layout->addWidget(lblAlbum, 2, 0, 1, 1);

        txtAlbum = new QLineEdit(tags_groupBox);
        txtAlbum->setObjectName("txtAlbum");
        sizePolicy1.setHeightForWidth(txtAlbum->sizePolicy().hasHeightForWidth());
        txtAlbum->setSizePolicy(sizePolicy1);
        txtAlbum->setMinimumSize(QSize(0, 0));

        tags_layout->addWidget(txtAlbum, 2, 1, 1, 1);

        lblAlbumArtist = new QLabel(tags_groupBox);
        lblAlbumArtist->setObjectName("lblAlbumArtist");
        sizePolicy.setHeightForWidth(lblAlbumArtist->sizePolicy().hasHeightForWidth());
        lblAlbumArtist->setSizePolicy(sizePolicy);
        lblAlbumArtist->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        tags_layout->addWidget(lblAlbumArtist, 3, 0, 1, 1);

        txtAlbumArtist = new QLineEdit(tags_groupBox);
        txtAlbumArtist->setObjectName("txtAlbumArtist");
        sizePolicy1.setHeightForWidth(txtAlbumArtist->sizePolicy().hasHeightForWidth());
        txtAlbumArtist->setSizePolicy(sizePolicy1);
        txtAlbumArtist->setMinimumSize(QSize(0, 0));

        tags_layout->addWidget(txtAlbumArtist, 3, 1, 1, 1);

        verticalWidgetStars = new QWidget(tags_groupBox);
        verticalWidgetStars->setObjectName("verticalWidgetStars");
        sizePolicy.setHeightForWidth(verticalWidgetStars->sizePolicy().hasHeightForWidth());
        verticalWidgetStars->setSizePolicy(sizePolicy);
        starsLayout = new QHBoxLayout(verticalWidgetStars);
        starsLayout->setSpacing(0);
        starsLayout->setContentsMargins(11, 11, 11, 11);
        starsLayout->setObjectName("starsLayout");
        starsLayout->setSizeConstraint(QLayout::SetDefaultConstraint);

        tags_layout->addWidget(verticalWidgetStars, 3, 2, 1, 2);

        lblComposer = new QLabel(tags_groupBox);
        lblComposer->setObjectName("lblComposer");
        lblComposer->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        tags_layout->addWidget(lblComposer, 4, 0, 1, 1);

        txtComposer = new QLineEdit(tags_groupBox);
        txtComposer->setObjectName("txtComposer");
        sizePolicy1.setHeightForWidth(txtComposer->sizePolicy().hasHeightForWidth());
        txtComposer->setSizePolicy(sizePolicy1);
        txtComposer->setMinimumSize(QSize(0, 0));

        tags_layout->addWidget(txtComposer, 4, 1, 1, 1);

        lblYear = new QLabel(tags_groupBox);
        lblYear->setObjectName("lblYear");
        lblYear->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        tags_layout->addWidget(lblYear, 4, 2, 1, 1);

        txtYear = new QLineEdit(tags_groupBox);
        txtYear->setObjectName("txtYear");
        sizePolicy1.setHeightForWidth(txtYear->sizePolicy().hasHeightForWidth());
        txtYear->setSizePolicy(sizePolicy1);

        tags_layout->addWidget(txtYear, 4, 3, 1, 1);

        lblGenre = new QLabel(tags_groupBox);
        lblGenre->setObjectName("lblGenre");
        lblGenre->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        tags_layout->addWidget(lblGenre, 5, 0, 1, 1);

        txtGenre = new QLineEdit(tags_groupBox);
        txtGenre->setObjectName("txtGenre");
        sizePolicy1.setHeightForWidth(txtGenre->sizePolicy().hasHeightForWidth());
        txtGenre->setSizePolicy(sizePolicy1);
        txtGenre->setMinimumSize(QSize(0, 0));

        tags_layout->addWidget(txtGenre, 5, 1, 1, 1);

        lblKey = new QLabel(tags_groupBox);
        lblKey->setObjectName("lblKey");
        lblKey->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        tags_layout->addWidget(lblKey, 5, 2, 1, 1);

        txtKey = new QLineEdit(tags_groupBox);
        txtKey->setObjectName("txtKey");
        sizePolicy1.setHeightForWidth(txtKey->sizePolicy().hasHeightForWidth());
        txtKey->setSizePolicy(sizePolicy1);
        txtKey->setMinimumSize(QSize(0, 0));

        tags_layout->addWidget(txtKey, 5, 3, 1, 1);

        lblGrouping = new QLabel(tags_groupBox);
        lblGrouping->setObjectName("lblGrouping");
        lblGrouping->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        tags_layout->addWidget(lblGrouping, 6, 0, 1, 1);

        txtGrouping = new QLineEdit(tags_groupBox);
        txtGrouping->setObjectName("txtGrouping");
        sizePolicy1.setHeightForWidth(txtGrouping->sizePolicy().hasHeightForWidth());
        txtGrouping->setSizePolicy(sizePolicy1);
        txtGrouping->setMinimumSize(QSize(0, 0));

        tags_layout->addWidget(txtGrouping, 6, 1, 1, 1);

        lblTrackNumber = new QLabel(tags_groupBox);
        lblTrackNumber->setObjectName("lblTrackNumber");
        lblTrackNumber->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        tags_layout->addWidget(lblTrackNumber, 6, 2, 1, 1);

        txtTrackNumber = new QLineEdit(tags_groupBox);
        txtTrackNumber->setObjectName("txtTrackNumber");
        sizePolicy1.setHeightForWidth(txtTrackNumber->sizePolicy().hasHeightForWidth());
        txtTrackNumber->setSizePolicy(sizePolicy1);
        txtTrackNumber->setMinimumSize(QSize(0, 0));

        tags_layout->addWidget(txtTrackNumber, 6, 3, 1, 1);

        lblTrackComment = new QLabel(tags_groupBox);
        lblTrackComment->setObjectName("lblTrackComment");
        lblTrackComment->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        tags_layout->addWidget(lblTrackComment, 7, 0, 1, 1);

        txtComment = new QPlainTextEdit(tags_groupBox);
        txtComment->setObjectName("txtComment");
        txtComment->setTabChangesFocus(true);

        tags_layout->addWidget(txtComment, 7, 1, 1, 3);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setSpacing(6);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        btnImportMetadataFromMusicBrainz = new QPushButton(tags_groupBox);
        btnImportMetadataFromMusicBrainz->setObjectName("btnImportMetadataFromMusicBrainz");

        horizontalLayout_3->addWidget(btnImportMetadataFromMusicBrainz);

        btnImportMetadataFromFile = new QPushButton(tags_groupBox);
        btnImportMetadataFromFile->setObjectName("btnImportMetadataFromFile");

        horizontalLayout_3->addWidget(btnImportMetadataFromFile);

        horizontalSpacer_4 = new QSpacerItem(40, 10, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_4);


        tags_layout->addLayout(horizontalLayout_3, 8, 1, 1, 3);

        lblTrackColor = new QLabel(tags_groupBox);
        lblTrackColor->setObjectName("lblTrackColor");
        lblTrackColor->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        tags_layout->addWidget(lblTrackColor, 9, 0, 1, 1);

        btnColorPicker = new QPushButton(tags_groupBox);
        btnColorPicker->setObjectName("btnColorPicker");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(btnColorPicker->sizePolicy().hasHeightForWidth());
        btnColorPicker->setSizePolicy(sizePolicy2);

        tags_layout->addWidget(btnColorPicker, 9, 1, 1, 3);

        tags_layout->setColumnStretch(1, 10);
        tags_layout->setColumnStretch(3, 2);

        summary_layout->addWidget(tags_groupBox);

        properties_groupBox = new QGroupBox(tabSummary);
        properties_groupBox->setObjectName("properties_groupBox");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(properties_groupBox->sizePolicy().hasHeightForWidth());
        properties_groupBox->setSizePolicy(sizePolicy3);
        properties_layout = new QGridLayout(properties_groupBox);
        properties_layout->setSpacing(6);
        properties_layout->setContentsMargins(11, 11, 11, 11);
        properties_layout->setObjectName("properties_layout");
        lblDuration = new QLabel(properties_groupBox);
        lblDuration->setObjectName("lblDuration");

        properties_layout->addWidget(lblDuration, 0, 0, 1, 1);

        txtDuration = new QLabel(properties_groupBox);
        txtDuration->setObjectName("txtDuration");
        QFont font1;
        font1.setBold(true);
        txtDuration->setFont(font1);
        txtDuration->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        properties_layout->addWidget(txtDuration, 0, 1, 1, 1);

        lblType = new QLabel(properties_groupBox);
        lblType->setObjectName("lblType");

        properties_layout->addWidget(lblType, 0, 2, 1, 1);

        txtType = new QLabel(properties_groupBox);
        txtType->setObjectName("txtType");
        txtType->setFont(font1);
        txtType->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        properties_layout->addWidget(txtType, 0, 3, 1, 1);

        lblBpm = new QLabel(properties_groupBox);
        lblBpm->setObjectName("lblBpm");

        properties_layout->addWidget(lblBpm, 1, 0, 1, 1);

        txtBpm = new QLabel(properties_groupBox);
        txtBpm->setObjectName("txtBpm");
        txtBpm->setFont(font1);
        txtBpm->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        properties_layout->addWidget(txtBpm, 1, 1, 1, 1);

        lblBitrate = new QLabel(properties_groupBox);
        lblBitrate->setObjectName("lblBitrate");

        properties_layout->addWidget(lblBitrate, 1, 2, 1, 1);

        txtBitrate = new QLabel(properties_groupBox);
        txtBitrate->setObjectName("txtBitrate");
        txtBitrate->setFont(font1);
        txtBitrate->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        properties_layout->addWidget(txtBitrate, 1, 3, 1, 1);

        lblDateAdded = new QLabel(properties_groupBox);
        lblDateAdded->setObjectName("lblDateAdded");

        properties_layout->addWidget(lblDateAdded, 2, 0, 1, 1);

        txtDateAdded = new QLabel(properties_groupBox);
        txtDateAdded->setObjectName("txtDateAdded");
        txtDateAdded->setFont(font1);
        txtDateAdded->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        properties_layout->addWidget(txtDateAdded, 2, 1, 1, 1);

        lblSamplerate = new QLabel(properties_groupBox);
        lblSamplerate->setObjectName("lblSamplerate");

        properties_layout->addWidget(lblSamplerate, 2, 2, 1, 1);

        txtSamplerate = new QLabel(properties_groupBox);
        txtSamplerate->setObjectName("txtSamplerate");
        txtSamplerate->setFont(font1);

        properties_layout->addWidget(txtSamplerate, 2, 3, 1, 1);

        lblReplayGain = new QLabel(properties_groupBox);
        lblReplayGain->setObjectName("lblReplayGain");

        properties_layout->addWidget(lblReplayGain, 3, 0, 1, 1);

        txtReplayGain = new QLabel(properties_groupBox);
        txtReplayGain->setObjectName("txtReplayGain");
        txtReplayGain->setFont(font1);
        txtReplayGain->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        properties_layout->addWidget(txtReplayGain, 3, 1, 1, 1);

        lblLocation = new QLabel(properties_groupBox);
        lblLocation->setObjectName("lblLocation");
        lblLocation->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        properties_layout->addWidget(lblLocation, 4, 0, 1, 1);

        txtLocation = new QLabel(properties_groupBox);
        txtLocation->setObjectName("txtLocation");
        txtLocation->setWordWrap(true);
        txtLocation->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        properties_layout->addWidget(txtLocation, 4, 1, 1, 3);

        openFileBrowser_layout = new QHBoxLayout();
        openFileBrowser_layout->setSpacing(6);
        openFileBrowser_layout->setObjectName("openFileBrowser_layout");
        btnOpenFileBrowser = new QPushButton(properties_groupBox);
        btnOpenFileBrowser->setObjectName("btnOpenFileBrowser");

        openFileBrowser_layout->addWidget(btnOpenFileBrowser);

        horizontalSpacer_6 = new QSpacerItem(40, 10, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        openFileBrowser_layout->addItem(horizontalSpacer_6);


        properties_layout->addLayout(openFileBrowser_layout, 5, 1, 1, 3);

        properties_layout->setColumnStretch(1, 1);
        properties_layout->setColumnStretch(3, 1);

        summary_layout->addWidget(properties_groupBox);

        verticalSpacer_2 = new QSpacerItem(20, 2, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::MinimumExpanding);

        summary_layout->addItem(verticalSpacer_2);

        tabWidget->addTab(tabSummary, QString());
        tabBPM = new QWidget();
        tabBPM->setObjectName("tabBPM");
        verticalLayout_4 = new QVBoxLayout(tabBPM);
        verticalLayout_4->setSpacing(6);
        verticalLayout_4->setContentsMargins(11, 11, 11, 11);
        verticalLayout_4->setObjectName("verticalLayout_4");
        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setSpacing(6);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName("gridLayout");
        labelBpm = new QLabel(tabBPM);
        labelBpm->setObjectName("labelBpm");

        gridLayout->addWidget(labelBpm, 0, 0, 1, 1);

        spinBpm = new QDoubleSpinBox(tabBPM);
        spinBpm->setObjectName("spinBpm");
        spinBpm->setDecimals(8);
        spinBpm->setMaximum(500.000000000000000);
        spinBpm->setSingleStep(1.000000000000000);

        gridLayout->addWidget(spinBpm, 0, 1, 1, 1);

        bpmConst = new QCheckBox(tabBPM);
        bpmConst->setObjectName("bpmConst");
        bpmConst->setLayoutDirection(Qt::RightToLeft);

        gridLayout->addWidget(bpmConst, 1, 0, 1, 2);

        bpmDouble = new QPushButton(tabBPM);
        bpmDouble->setObjectName("bpmDouble");
        bpmDouble->setMinimumSize(QSize(125, 0));

        gridLayout->addWidget(bpmDouble, 2, 0, 1, 1);

        bpmHalve = new QPushButton(tabBPM);
        bpmHalve->setObjectName("bpmHalve");
        bpmHalve->setMinimumSize(QSize(125, 0));

        gridLayout->addWidget(bpmHalve, 2, 1, 1, 1);

        bpmTwoThirds = new QPushButton(tabBPM);
        bpmTwoThirds->setObjectName("bpmTwoThirds");
        bpmTwoThirds->setMinimumSize(QSize(125, 0));

        gridLayout->addWidget(bpmTwoThirds, 3, 0, 1, 1);

        bpmThreeFourth = new QPushButton(tabBPM);
        bpmThreeFourth->setObjectName("bpmThreeFourth");
        bpmThreeFourth->setMinimumSize(QSize(125, 0));

        gridLayout->addWidget(bpmThreeFourth, 3, 1, 1, 1);

        bpmThreeHalves = new QPushButton(tabBPM);
        bpmThreeHalves->setObjectName("bpmThreeHalves");
        bpmThreeHalves->setMinimumSize(QSize(125, 0));

        gridLayout->addWidget(bpmThreeHalves, 4, 0, 1, 1);

        bpmFourThirds = new QPushButton(tabBPM);
        bpmFourThirds->setObjectName("bpmFourThirds");
        bpmFourThirds->setMinimumSize(QSize(125, 0));

        gridLayout->addWidget(bpmFourThirds, 4, 1, 1, 1);

        bpmTap = new QPushButton(tabBPM);
        bpmTap->setObjectName("bpmTap");
        bpmTap->setMinimumSize(QSize(121, 91));
        bpmTap->setMaximumSize(QSize(123, 128));

        gridLayout->addWidget(bpmTap, 2, 2, 3, 1);


        horizontalLayout_4->addLayout(gridLayout);


        verticalLayout_4->addLayout(horizontalLayout_4);

        line = new QFrame(tabBPM);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_4->addWidget(line);

        bpmClear = new QPushButton(tabBPM);
        bpmClear->setObjectName("bpmClear");

        verticalLayout_4->addWidget(bpmClear);

        label = new QLabel(tabBPM);
        label->setObjectName("label");

        verticalLayout_4->addWidget(label);

        verticalSpacer = new QSpacerItem(0, 200, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer);

        tabWidget->addTab(tabBPM, QString());

        verticalLayout_2->addWidget(tabWidget);

        buttonLayout = new QHBoxLayout();
        buttonLayout->setSpacing(6);
        buttonLayout->setObjectName("buttonLayout");
        btnPrev = new QPushButton(DlgTrackInfo);
        btnPrev->setObjectName("btnPrev");
        btnPrev->setMinimumSize(QSize(100, 0));
        btnPrev->setMaximumSize(QSize(95, 16777215));

        buttonLayout->addWidget(btnPrev);

        btnNext = new QPushButton(DlgTrackInfo);
        btnNext->setObjectName("btnNext");
        btnNext->setMinimumSize(QSize(100, 0));

        buttonLayout->addWidget(btnNext);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        buttonLayout->addItem(horizontalSpacer);

        btnCancel = new QPushButton(DlgTrackInfo);
        btnCancel->setObjectName("btnCancel");
        btnCancel->setMinimumSize(QSize(100, 0));

        buttonLayout->addWidget(btnCancel);

        btnApply = new QPushButton(DlgTrackInfo);
        btnApply->setObjectName("btnApply");
        btnApply->setMinimumSize(QSize(100, 0));

        buttonLayout->addWidget(btnApply);

        btnOK = new QPushButton(DlgTrackInfo);
        btnOK->setObjectName("btnOK");
        btnOK->setMinimumSize(QSize(100, 0));

        buttonLayout->addWidget(btnOK);


        verticalLayout_2->addLayout(buttonLayout);

        QWidget::setTabOrder(tabWidget, txtTitle);
        QWidget::setTabOrder(txtTitle, txtArtist);
        QWidget::setTabOrder(txtArtist, txtAlbum);
        QWidget::setTabOrder(txtAlbum, txtAlbumArtist);
        QWidget::setTabOrder(txtAlbumArtist, txtComposer);
        QWidget::setTabOrder(txtComposer, txtGenre);
        QWidget::setTabOrder(txtGenre, txtGrouping);
        QWidget::setTabOrder(txtGrouping, txtComment);
        QWidget::setTabOrder(txtComment, txtYear);
        QWidget::setTabOrder(txtYear, txtKey);
        QWidget::setTabOrder(txtKey, txtTrackNumber);
        QWidget::setTabOrder(txtTrackNumber, btnImportMetadataFromMusicBrainz);
        QWidget::setTabOrder(btnImportMetadataFromMusicBrainz, btnImportMetadataFromFile);
        QWidget::setTabOrder(btnImportMetadataFromFile, btnColorPicker);
        QWidget::setTabOrder(btnColorPicker, btnOpenFileBrowser);
        QWidget::setTabOrder(btnOpenFileBrowser, spinBpm);
        QWidget::setTabOrder(spinBpm, bpmConst);
        QWidget::setTabOrder(bpmConst, bpmTap);
        QWidget::setTabOrder(bpmTap, bpmDouble);
        QWidget::setTabOrder(bpmDouble, bpmHalve);
        QWidget::setTabOrder(bpmHalve, bpmTwoThirds);
        QWidget::setTabOrder(bpmTwoThirds, bpmThreeFourth);
        QWidget::setTabOrder(bpmThreeFourth, bpmThreeHalves);
        QWidget::setTabOrder(bpmThreeHalves, bpmFourThirds);
        QWidget::setTabOrder(bpmFourThirds, bpmClear);
        QWidget::setTabOrder(bpmClear, btnPrev);
        QWidget::setTabOrder(btnPrev, btnNext);
        QWidget::setTabOrder(btnNext, btnCancel);
        QWidget::setTabOrder(btnCancel, btnApply);
        QWidget::setTabOrder(btnApply, btnOK);

        retranslateUi(DlgTrackInfo);

        tabWidget->setCurrentIndex(0);
        btnCancel->setDefault(false);
        btnApply->setDefault(true);
        btnOK->setDefault(false);


        QMetaObject::connectSlotsByName(DlgTrackInfo);
    } // setupUi

    void retranslateUi(QDialog *DlgTrackInfo)
    {
        DlgTrackInfo->setWindowTitle(QCoreApplication::translate("DlgTrackInfo", "Track Editor", nullptr));
#if QT_CONFIG(tooltip)
        DlgTrackInfo->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        lblTitle->setText(QCoreApplication::translate("DlgTrackInfo", "Title", nullptr));
        lblArtist->setText(QCoreApplication::translate("DlgTrackInfo", "Artist", nullptr));
        lblAlbum->setText(QCoreApplication::translate("DlgTrackInfo", "Album", nullptr));
        lblAlbumArtist->setText(QCoreApplication::translate("DlgTrackInfo", "Album Artist", nullptr));
        lblComposer->setText(QCoreApplication::translate("DlgTrackInfo", "Composer", nullptr));
        lblYear->setText(QCoreApplication::translate("DlgTrackInfo", "Year", nullptr));
        lblGenre->setText(QCoreApplication::translate("DlgTrackInfo", "Genre", nullptr));
        lblKey->setText(QCoreApplication::translate("DlgTrackInfo", "Key", nullptr));
        lblGrouping->setText(QCoreApplication::translate("DlgTrackInfo", "Grouping", nullptr));
        lblTrackNumber->setText(QCoreApplication::translate("DlgTrackInfo", "Track #", nullptr));
        lblTrackComment->setText(QCoreApplication::translate("DlgTrackInfo", "Comments", nullptr));
        btnImportMetadataFromMusicBrainz->setText(QCoreApplication::translate("DlgTrackInfo", "Import Metadata from MusicBrainz", nullptr));
        btnImportMetadataFromFile->setText(QCoreApplication::translate("DlgTrackInfo", "Re-Import Metadata from file", nullptr));
        lblTrackColor->setText(QCoreApplication::translate("DlgTrackInfo", "Color", nullptr));
        lblDuration->setText(QCoreApplication::translate("DlgTrackInfo", "Duration:", nullptr));
        txtDuration->setText(QString());
        lblType->setText(QCoreApplication::translate("DlgTrackInfo", "Filetype:", nullptr));
        txtType->setText(QString());
        lblBpm->setText(QCoreApplication::translate("DlgTrackInfo", "BPM:", nullptr));
        txtBpm->setText(QString());
        lblBitrate->setText(QCoreApplication::translate("DlgTrackInfo", "Bitrate:", nullptr));
        txtBitrate->setText(QString());
        lblDateAdded->setText(QCoreApplication::translate("DlgTrackInfo", "Date added:", nullptr));
        txtDateAdded->setText(QString());
        lblSamplerate->setText(QCoreApplication::translate("DlgTrackInfo", "Samplerate:", nullptr));
        txtSamplerate->setText(QString());
        lblReplayGain->setText(QCoreApplication::translate("DlgTrackInfo", "ReplayGain:", nullptr));
        txtReplayGain->setText(QString());
        lblLocation->setText(QCoreApplication::translate("DlgTrackInfo", "Location:", nullptr));
        txtLocation->setText(QString());
        btnOpenFileBrowser->setText(QCoreApplication::translate("DlgTrackInfo", "Open in File Browser", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabSummary), QCoreApplication::translate("DlgTrackInfo", "Summary", nullptr));
        labelBpm->setText(QCoreApplication::translate("DlgTrackInfo", "Track BPM: ", nullptr));
#if QT_CONFIG(tooltip)
        spinBpm->setToolTip(QCoreApplication::translate("DlgTrackInfo", "Displays the BPM of the selected track.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        bpmConst->setToolTip(QCoreApplication::translate("DlgTrackInfo", "Converts beats detected by the analyzer into a fixed-tempo beatgrid.\n"
"Use this setting if your tracks have a constant tempo (e.g. most electronic music).\n"
"Often results in higher quality beatgrids, but will not do well on tracks that have tempo shifts.", nullptr));
#endif // QT_CONFIG(tooltip)
        bpmConst->setText(QCoreApplication::translate("DlgTrackInfo", "Assume constant tempo", nullptr));
#if QT_CONFIG(tooltip)
        bpmDouble->setToolTip(QCoreApplication::translate("DlgTrackInfo", "Sets the BPM to 200% of the current value.", nullptr));
#endif // QT_CONFIG(tooltip)
        bpmDouble->setText(QCoreApplication::translate("DlgTrackInfo", "Double BPM", nullptr));
#if QT_CONFIG(tooltip)
        bpmHalve->setToolTip(QCoreApplication::translate("DlgTrackInfo", "Sets the BPM to 50% of the current value.", nullptr));
#endif // QT_CONFIG(tooltip)
        bpmHalve->setText(QCoreApplication::translate("DlgTrackInfo", "Halve BPM", nullptr));
#if QT_CONFIG(tooltip)
        bpmTwoThirds->setToolTip(QCoreApplication::translate("DlgTrackInfo", "Sets the BPM to 66% of the current value.", nullptr));
#endif // QT_CONFIG(tooltip)
        bpmTwoThirds->setText(QCoreApplication::translate("DlgTrackInfo", "2/3 BPM", nullptr));
#if QT_CONFIG(tooltip)
        bpmThreeFourth->setToolTip(QCoreApplication::translate("DlgTrackInfo", "Sets the BPM to 75% of the current value.", nullptr));
#endif // QT_CONFIG(tooltip)
        bpmThreeFourth->setText(QCoreApplication::translate("DlgTrackInfo", "3/4 BPM", nullptr));
#if QT_CONFIG(tooltip)
        bpmThreeHalves->setToolTip(QCoreApplication::translate("DlgTrackInfo", "Sets the BPM to 150% of the current value.", nullptr));
#endif // QT_CONFIG(tooltip)
        bpmThreeHalves->setText(QCoreApplication::translate("DlgTrackInfo", "3/2 BPM", nullptr));
#if QT_CONFIG(tooltip)
        bpmFourThirds->setToolTip(QCoreApplication::translate("DlgTrackInfo", "Sets the BPM to 133% of the current value.", nullptr));
#endif // QT_CONFIG(tooltip)
        bpmFourThirds->setText(QCoreApplication::translate("DlgTrackInfo", "4/3 BPM", nullptr));
#if QT_CONFIG(tooltip)
        bpmTap->setToolTip(QCoreApplication::translate("DlgTrackInfo", "Tap with the beat to set the BPM to the speed you are tapping.", nullptr));
#endif // QT_CONFIG(tooltip)
        bpmTap->setText(QCoreApplication::translate("DlgTrackInfo", "Tap to Beat", nullptr));
        bpmClear->setText(QCoreApplication::translate("DlgTrackInfo", "Clear BPM and Beatgrid", nullptr));
        label->setText(QCoreApplication::translate("DlgTrackInfo", "Hint: Use the Library Analyze view to run BPM detection.", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabBPM), QCoreApplication::translate("DlgTrackInfo", "BPM", nullptr));
#if QT_CONFIG(tooltip)
        btnPrev->setToolTip(QCoreApplication::translate("DlgTrackInfo", "Move to the previous item.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnPrev->setText(QCoreApplication::translate("DlgTrackInfo", "&Previous", nullptr));
#if QT_CONFIG(tooltip)
        btnNext->setToolTip(QCoreApplication::translate("DlgTrackInfo", "Move to the next item.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnNext->setText(QCoreApplication::translate("DlgTrackInfo", "&Next", nullptr));
#if QT_CONFIG(tooltip)
        btnCancel->setToolTip(QCoreApplication::translate("DlgTrackInfo", "Discard changes and close the window.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnCancel->setText(QCoreApplication::translate("DlgTrackInfo", "&Cancel", nullptr));
#if QT_CONFIG(tooltip)
        btnApply->setToolTip(QCoreApplication::translate("DlgTrackInfo", "Save changes and keep the window open.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnApply->setText(QCoreApplication::translate("DlgTrackInfo", "&Apply", nullptr));
#if QT_CONFIG(tooltip)
        btnOK->setToolTip(QCoreApplication::translate("DlgTrackInfo", "Save changes and close the window.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnOK->setText(QCoreApplication::translate("DlgTrackInfo", "&OK", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgTrackInfo: public Ui_DlgTrackInfo {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLGTRACKINFO_H
