/****************************************************************************
** Meta object code from reading C++ file 'enginebuffer.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/engine/enginebuffer.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'enginebuffer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12EngineBufferE_t {};
} // unnamed namespace

template <> constexpr inline auto EngineBuffer::qt_create_metaobjectdata<qt_meta_tag_ZN12EngineBufferE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "EngineBuffer",
        "trackLoaded",
        "",
        "TrackPointer",
        "pNewTrack",
        "pOldTrack",
        "trackLoadFailed",
        "pTrack",
        "reason",
        "noVinylControlInputConfigured",
        "slotControlPlayRequest",
        "slotControlPlayFromStart",
        "slotControlJumpToStartAndStop",
        "slotControlStop",
        "slotControlStart",
        "slotControlEnd",
        "slotControlSeek",
        "slotKeylockEngineChanged",
        "slotTrackLoading",
        "slotTrackLoaded",
        "mixxx::audio::SampleRate",
        "trackSampleRate",
        "trackNumSamples",
        "slotTrackLoadFailed",
        "slotPassthroughChanged",
        "v",
        "slotUpdatedTrackBeats"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'trackLoaded'
        QtMocHelpers::SignalData<void(TrackPointer, TrackPointer)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 3, 5 },
        }}),
        // Signal 'trackLoadFailed'
        QtMocHelpers::SignalData<void(TrackPointer, const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 7 }, { QMetaType::QString, 8 },
        }}),
        // Signal 'noVinylControlInputConfigured'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotControlPlayRequest'
        QtMocHelpers::SlotData<void(double)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotControlPlayFromStart'
        QtMocHelpers::SlotData<void(double)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotControlJumpToStartAndStop'
        QtMocHelpers::SlotData<void(double)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotControlStop'
        QtMocHelpers::SlotData<void(double)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotControlStart'
        QtMocHelpers::SlotData<void(double)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotControlEnd'
        QtMocHelpers::SlotData<void(double)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotControlSeek'
        QtMocHelpers::SlotData<void(double)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotKeylockEngineChanged'
        QtMocHelpers::SlotData<void(double)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotTrackLoading'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotTrackLoaded'
        QtMocHelpers::SlotData<void(TrackPointer, mixxx::audio::SampleRate, double)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 7 }, { 0x80000000 | 20, 21 }, { QMetaType::Double, 22 },
        }}),
        // Slot 'slotTrackLoadFailed'
        QtMocHelpers::SlotData<void(TrackPointer, const QString &)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 7 }, { QMetaType::QString, 8 },
        }}),
        // Slot 'slotPassthroughChanged'
        QtMocHelpers::SlotData<void(double)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 25 },
        }}),
        // Slot 'slotUpdatedTrackBeats'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<EngineBuffer, qt_meta_tag_ZN12EngineBufferE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject EngineBuffer::staticMetaObject = { {
    QMetaObject::SuperData::link<EngineObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12EngineBufferE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12EngineBufferE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12EngineBufferE_t>.metaTypes,
    nullptr
} };

void EngineBuffer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<EngineBuffer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->trackLoaded((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[2]))); break;
        case 1: _t->trackLoadFailed((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 2: _t->noVinylControlInputConfigured(); break;
        case 3: _t->slotControlPlayRequest((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 4: _t->slotControlPlayFromStart((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 5: _t->slotControlJumpToStartAndStop((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 6: _t->slotControlStop((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 7: _t->slotControlStart((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 8: _t->slotControlEnd((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 9: _t->slotControlSeek((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 10: _t->slotKeylockEngineChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 11: _t->slotTrackLoading(); break;
        case 12: _t->slotTrackLoaded((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<mixxx::audio::SampleRate>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[3]))); break;
        case 13: _t->slotTrackLoadFailed((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 14: _t->slotPassthroughChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 15: _t->slotUpdatedTrackBeats(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< mixxx::audio::SampleRate >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (EngineBuffer::*)(TrackPointer , TrackPointer )>(_a, &EngineBuffer::trackLoaded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (EngineBuffer::*)(TrackPointer , const QString & )>(_a, &EngineBuffer::trackLoadFailed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (EngineBuffer::*)()>(_a, &EngineBuffer::noVinylControlInputConfigured, 2))
            return;
    }
}

const QMetaObject *EngineBuffer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *EngineBuffer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12EngineBufferE_t>.strings))
        return static_cast<void*>(this);
    return EngineObject::qt_metacast(_clname);
}

int EngineBuffer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = EngineObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void EngineBuffer::trackLoaded(TrackPointer _t1, TrackPointer _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void EngineBuffer::trackLoadFailed(TrackPointer _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void EngineBuffer::noVinylControlInputConfigured()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
