/****************************************************************************
** Meta object code from reading C++ file 'wtracktableview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/widget/wtracktableview.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wtracktableview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15WTrackTableViewE_t {};
} // unnamed namespace

template <> constexpr inline auto WTrackTableView::qt_create_metaobjectdata<qt_meta_tag_ZN15WTrackTableViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "WTrackTableView",
        "trackMenuVisible",
        "",
        "visible",
        "focusBorderColorChanged",
        "QColor",
        "col",
        "trackPlayedColorChanged",
        "trackMissingColorChanged",
        "loadTrackModel",
        "QAbstractItemModel*",
        "model",
        "restoreState",
        "slotMouseDoubleClicked",
        "QModelIndex",
        "slotUnhide",
        "slotPurge",
        "slotDeleteTracksFromDisk",
        "slotShowHideTrackMenu",
        "show",
        "slotSaveCurrentViewState",
        "slotRestoreCurrentViewState",
        "slotrestoreCurrentIndex",
        "doSortByColumn",
        "headerSection",
        "Qt::SortOrder",
        "sortOrder",
        "applySortingIfVisible",
        "applySorting",
        "slotGuiTick50ms",
        "slotScrollValueChanged",
        "slotSortingChanged",
        "order",
        "keyNotationChanged",
        "focusBorderColor",
        "trackPlayedColor",
        "trackMissingColor"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'trackMenuVisible'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'focusBorderColorChanged'
        QtMocHelpers::SignalData<void(QColor)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'trackPlayedColorChanged'
        QtMocHelpers::SignalData<void(QColor)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'trackMissingColorChanged'
        QtMocHelpers::SignalData<void(QColor)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'loadTrackModel'
        QtMocHelpers::SlotData<void(QAbstractItemModel *, bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 }, { QMetaType::Bool, 12 },
        }}),
        // Slot 'loadTrackModel'
        QtMocHelpers::SlotData<void(QAbstractItemModel *)>(9, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'slotMouseDoubleClicked'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 2 },
        }}),
        // Slot 'slotUnhide'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotPurge'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotDeleteTracksFromDisk'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotShowHideTrackMenu'
        QtMocHelpers::SlotData<void(bool)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 19 },
        }}),
        // Slot 'slotSaveCurrentViewState'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotRestoreCurrentViewState'
        QtMocHelpers::SlotData<bool()>(21, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'slotrestoreCurrentIndex'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'doSortByColumn'
        QtMocHelpers::SlotData<void(int, Qt::SortOrder)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 24 }, { 0x80000000 | 25, 26 },
        }}),
        // Slot 'applySortingIfVisible'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'applySorting'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotGuiTick50ms'
        QtMocHelpers::SlotData<void(double)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotScrollValueChanged'
        QtMocHelpers::SlotData<void(int)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'slotSortingChanged'
        QtMocHelpers::SlotData<void(int, Qt::SortOrder)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 24 }, { 0x80000000 | 25, 32 },
        }}),
        // Slot 'keyNotationChanged'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'focusBorderColor'
        QtMocHelpers::PropertyData<QColor>(34, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, 1),
        // property 'trackPlayedColor'
        QtMocHelpers::PropertyData<QColor>(35, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, 2),
        // property 'trackMissingColor'
        QtMocHelpers::PropertyData<QColor>(36, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, 3),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WTrackTableView, qt_meta_tag_ZN15WTrackTableViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject WTrackTableView::staticMetaObject = { {
    QMetaObject::SuperData::link<WLibraryTableView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15WTrackTableViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15WTrackTableViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15WTrackTableViewE_t>.metaTypes,
    nullptr
} };

void WTrackTableView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WTrackTableView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->trackMenuVisible((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->focusBorderColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 2: _t->trackPlayedColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 3: _t->trackMissingColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 4: _t->loadTrackModel((*reinterpret_cast<std::add_pointer_t<QAbstractItemModel*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 5: _t->loadTrackModel((*reinterpret_cast<std::add_pointer_t<QAbstractItemModel*>>(_a[1]))); break;
        case 6: _t->slotMouseDoubleClicked((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 7: _t->slotUnhide(); break;
        case 8: _t->slotPurge(); break;
        case 9: _t->slotDeleteTracksFromDisk(); break;
        case 10: _t->slotShowHideTrackMenu((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->slotSaveCurrentViewState(); break;
        case 12: { bool _r = _t->slotRestoreCurrentViewState();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 13: _t->slotrestoreCurrentIndex(); break;
        case 14: _t->doSortByColumn((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::SortOrder>>(_a[2]))); break;
        case 15: _t->applySortingIfVisible(); break;
        case 16: _t->applySorting(); break;
        case 17: _t->slotGuiTick50ms((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 18: _t->slotScrollValueChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 19: _t->slotSortingChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::SortOrder>>(_a[2]))); break;
        case 20: _t->keyNotationChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractItemModel* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractItemModel* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WTrackTableView::*)(bool )>(_a, &WTrackTableView::trackMenuVisible, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WTrackTableView::*)(QColor )>(_a, &WTrackTableView::focusBorderColorChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WTrackTableView::*)(QColor )>(_a, &WTrackTableView::trackPlayedColorChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WTrackTableView::*)(QColor )>(_a, &WTrackTableView::trackMissingColorChanged, 3))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QColor*>(_v) = _t->m_focusBorderColor; break;
        case 1: *reinterpret_cast<QColor*>(_v) = _t->m_trackPlayedColor; break;
        case 2: *reinterpret_cast<QColor*>(_v) = _t->m_trackMissingColor; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0:
            if (QtMocHelpers::setProperty(_t->m_focusBorderColor, *reinterpret_cast<QColor*>(_v)))
                Q_EMIT _t->focusBorderColorChanged(_t->m_focusBorderColor);
            break;
        case 1:
            if (QtMocHelpers::setProperty(_t->m_trackPlayedColor, *reinterpret_cast<QColor*>(_v)))
                Q_EMIT _t->trackPlayedColorChanged(_t->m_trackPlayedColor);
            break;
        case 2:
            if (QtMocHelpers::setProperty(_t->m_trackMissingColor, *reinterpret_cast<QColor*>(_v)))
                Q_EMIT _t->trackMissingColorChanged(_t->m_trackMissingColor);
            break;
        default: break;
        }
    }
}

const QMetaObject *WTrackTableView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *WTrackTableView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15WTrackTableViewE_t>.strings))
        return static_cast<void*>(this);
    return WLibraryTableView::qt_metacast(_clname);
}

int WTrackTableView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = WLibraryTableView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void WTrackTableView::trackMenuVisible(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void WTrackTableView::focusBorderColorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void WTrackTableView::trackPlayedColorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void WTrackTableView::trackMissingColorChanged(QColor _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
