/********************************************************************************
** Form generated from reading UI file 'dlgprefcontrollersdlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLGPREFCONTROLLERSDLG_H
#define UI_DLGPREFCONTROLLERSDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DlgPrefControllersDlg
{
public:
    QGridLayout *gridLayout_3;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout;
    QLabel *txtControllersOverview;
    QLabel *txtNoControllersAvailable;
    QGroupBox *groupBox;
    QGridLayout *gridLayout;
    QLabel *txtMappingsOverview;
    QPushButton *btnOpenUserMappings;
    QLabel *createMapping;
    QGroupBox *groupBox_3;
    QGridLayout *gridLayout_2;
    QLabel *txtHardwareCompatibility;
    QLabel *txtControllerForums;
    QLabel *txtControllerMappingFormat;
    QLabel *txtControllerScripting;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *DlgPrefControllersDlg)
    {
        if (DlgPrefControllersDlg->objectName().isEmpty())
            DlgPrefControllersDlg->setObjectName("DlgPrefControllersDlg");
        DlgPrefControllersDlg->resize(0, 0);
        gridLayout_3 = new QGridLayout(DlgPrefControllersDlg);
        gridLayout_3->setObjectName("gridLayout_3");
        groupBox_2 = new QGroupBox(DlgPrefControllersDlg);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout = new QVBoxLayout(groupBox_2);
        verticalLayout->setObjectName("verticalLayout");
        txtControllersOverview = new QLabel(groupBox_2);
        txtControllersOverview->setObjectName("txtControllersOverview");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Ignored, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(txtControllersOverview->sizePolicy().hasHeightForWidth());
        txtControllersOverview->setSizePolicy(sizePolicy);
        txtControllersOverview->setWordWrap(true);

        verticalLayout->addWidget(txtControllersOverview);

        txtNoControllersAvailable = new QLabel(groupBox_2);
        txtNoControllersAvailable->setObjectName("txtNoControllersAvailable");
        txtNoControllersAvailable->setEnabled(true);
        sizePolicy.setHeightForWidth(txtNoControllersAvailable->sizePolicy().hasHeightForWidth());
        txtNoControllersAvailable->setSizePolicy(sizePolicy);
        txtNoControllersAvailable->setMinimumSize(QSize(100, 10));
        QFont font;
        font.setBold(true);
        txtNoControllersAvailable->setFont(font);
        txtNoControllersAvailable->setStyleSheet(QString::fromUtf8("QLabel { font-weight: bold; }"));
        txtNoControllersAvailable->setWordWrap(true);

        verticalLayout->addWidget(txtNoControllersAvailable);


        gridLayout_3->addWidget(groupBox_2, 0, 0, 1, 1);

        groupBox = new QGroupBox(DlgPrefControllersDlg);
        groupBox->setObjectName("groupBox");
        gridLayout = new QGridLayout(groupBox);
        gridLayout->setObjectName("gridLayout");
        txtMappingsOverview = new QLabel(groupBox);
        txtMappingsOverview->setObjectName("txtMappingsOverview");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(txtMappingsOverview->sizePolicy().hasHeightForWidth());
        txtMappingsOverview->setSizePolicy(sizePolicy1);
        txtMappingsOverview->setWordWrap(true);
        txtMappingsOverview->setOpenExternalLinks(true);

        gridLayout->addWidget(txtMappingsOverview, 0, 0, 1, 2);

        btnOpenUserMappings = new QPushButton(groupBox);
        btnOpenUserMappings->setObjectName("btnOpenUserMappings");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(btnOpenUserMappings->sizePolicy().hasHeightForWidth());
        btnOpenUserMappings->setSizePolicy(sizePolicy2);

        gridLayout->addWidget(btnOpenUserMappings, 1, 0, 1, 1);

        createMapping = new QLabel(groupBox);
        createMapping->setObjectName("createMapping");
        sizePolicy.setHeightForWidth(createMapping->sizePolicy().hasHeightForWidth());
        createMapping->setSizePolicy(sizePolicy);
        createMapping->setWordWrap(true);

        gridLayout->addWidget(createMapping, 2, 0, 1, 2);


        gridLayout_3->addWidget(groupBox, 1, 0, 1, 1);

        groupBox_3 = new QGroupBox(DlgPrefControllersDlg);
        groupBox_3->setObjectName("groupBox_3");
        gridLayout_2 = new QGridLayout(groupBox_3);
        gridLayout_2->setObjectName("gridLayout_2");
        txtHardwareCompatibility = new QLabel(groupBox_3);
        txtHardwareCompatibility->setObjectName("txtHardwareCompatibility");
        txtHardwareCompatibility->setOpenExternalLinks(true);

        gridLayout_2->addWidget(txtHardwareCompatibility, 0, 0, 1, 1);

        txtControllerForums = new QLabel(groupBox_3);
        txtControllerForums->setObjectName("txtControllerForums");
        txtControllerForums->setOpenExternalLinks(true);

        gridLayout_2->addWidget(txtControllerForums, 0, 1, 1, 1);

        txtControllerMappingFormat = new QLabel(groupBox_3);
        txtControllerMappingFormat->setObjectName("txtControllerMappingFormat");
        txtControllerMappingFormat->setOpenExternalLinks(true);

        gridLayout_2->addWidget(txtControllerMappingFormat, 1, 0, 1, 1);

        txtControllerScripting = new QLabel(groupBox_3);
        txtControllerScripting->setObjectName("txtControllerScripting");
        txtControllerScripting->setOpenExternalLinks(true);

        gridLayout_2->addWidget(txtControllerScripting, 1, 1, 1, 1);


        gridLayout_3->addWidget(groupBox_3, 2, 0, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_3->addItem(verticalSpacer, 3, 0, 1, 1);


        retranslateUi(DlgPrefControllersDlg);

        QMetaObject::connectSlotsByName(DlgPrefControllersDlg);
    } // setupUi

    void retranslateUi(QWidget *DlgPrefControllersDlg)
    {
        DlgPrefControllersDlg->setWindowTitle(QCoreApplication::translate("DlgPrefControllersDlg", "Controller Preferences", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("DlgPrefControllersDlg", "Controllers", nullptr));
        txtControllersOverview->setText(QCoreApplication::translate("DlgPrefControllersDlg", "Controllers are physical devices that send MIDI or HID signals to your computer over a USB connection. These allow you to control Mixxx in a more hands-on way than a keyboard and mouse. Attached controllers that Mixxx recognizes are shown in the \"Controllers\" section in the sidebar.", nullptr));
        txtNoControllersAvailable->setText(QCoreApplication::translate("DlgPrefControllersDlg", "Mixxx did not detect any controllers. If you connected the controller while Mixxx was running you must restart Mixxx first.", nullptr));
        groupBox->setTitle(QCoreApplication::translate("DlgPrefControllersDlg", "Mappings", nullptr));
        btnOpenUserMappings->setText(QCoreApplication::translate("DlgPrefControllersDlg", "Open User Mapping Folder", nullptr));
        createMapping->setText(QCoreApplication::translate("DlgPrefControllersDlg", "You can create your own mapping by using the MIDI Learning Wizard when you select your controller in the sidebar. You can edit mappings by selecting the \"Input Mappings\" and \"Output Mappings\" tabs in the preference page for your controller. See the Resources below for more details on making mappings.", nullptr));
        groupBox_3->setTitle(QCoreApplication::translate("DlgPrefControllersDlg", "Resources", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgPrefControllersDlg: public Ui_DlgPrefControllersDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLGPREFCONTROLLERSDLG_H
