/****************************************************************************
** Meta object code from reading C++ file 'wmainmenubar.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/widget/wmainmenubar.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wmainmenubar.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN27VisibilityControlConnectionE_t {};
} // unnamed namespace

template <> constexpr inline auto VisibilityControlConnection::qt_create_metaobjectdata<qt_meta_tag_ZN27VisibilityControlConnectionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "VisibilityControlConnection",
        "slotClearControl",
        "",
        "slotReconnectControl",
        "slotControlChanged",
        "slotActionToggled",
        "toggle"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotClearControl'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotReconnectControl'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotControlChanged'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotActionToggled'
        QtMocHelpers::SlotData<void(bool)>(5, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<VisibilityControlConnection, qt_meta_tag_ZN27VisibilityControlConnectionE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject VisibilityControlConnection::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27VisibilityControlConnectionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27VisibilityControlConnectionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN27VisibilityControlConnectionE_t>.metaTypes,
    nullptr
} };

void VisibilityControlConnection::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<VisibilityControlConnection *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotClearControl(); break;
        case 1: _t->slotReconnectControl(); break;
        case 2: _t->slotControlChanged(); break;
        case 3: _t->slotActionToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *VisibilityControlConnection::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *VisibilityControlConnection::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27VisibilityControlConnectionE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int VisibilityControlConnection::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN12WMainMenuBarE_t {};
} // unnamed namespace

template <> constexpr inline auto WMainMenuBar::qt_create_metaobjectdata<qt_meta_tag_ZN12WMainMenuBarE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "WMainMenuBar",
        "createCrate",
        "",
        "createPlaylist",
        "loadTrackToDeck",
        "deck",
        "reloadSkin",
        "rescanLibrary",
        "exportLibrary",
        "menubarAutoHideChanged",
        "autohide",
        "showAbout",
        "showKeywheel",
        "visible",
        "showPreferences",
        "toggleDeveloperTools",
        "toggle",
        "toggleFullScreen",
        "toggleKeyboardShortcuts",
        "toggleBroadcasting",
        "toggleRecording",
        "enabled",
        "toggleVinylControl",
        "visitUrl",
        "url",
        "quit",
        "internalRecordingStateChange",
        "recording",
        "internalBroadcastingStateChange",
        "broadcasting",
        "internalFullScreenStateChange",
        "fullscreen",
        "internalLibraryScanActive",
        "active",
        "internalDeveloperToolsStateChange",
        "internalKeywheelStateChanged",
        "state",
        "internalOnNewSkinLoaded",
        "internalOnNewSkinAboutToLoad",
        "onLibraryScanStarted",
        "onLibraryScanFinished",
        "onRecordingStateChange",
        "onBroadcastingStateChange",
        "onNewSkinAboutToLoad",
        "onNewSkinLoaded",
        "onDeveloperToolsHidden",
        "onDeveloperToolsShown",
        "onFullScreenStateChange",
        "onVinylControlDeckEnabledStateChange",
        "onNumberOfDecksChanged",
        "decks",
        "onKeywheelChange",
        "slotToggleMenuBar",
        "slotAutoHideMenuBarToggled",
        "autoHide",
        "slotDeveloperStatsExperiment",
        "enable",
        "slotDeveloperStatsBase",
        "slotDeveloperDebugger",
        "slotVisitUrl",
        "QUrl"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'createCrate'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'createPlaylist'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'loadTrackToDeck'
        QtMocHelpers::SignalData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Signal 'reloadSkin'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'rescanLibrary'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'exportLibrary'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'menubarAutoHideChanged'
        QtMocHelpers::SignalData<void(bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Signal 'showAbout'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'showKeywheel'
        QtMocHelpers::SignalData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
        // Signal 'showPreferences'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'toggleDeveloperTools'
        QtMocHelpers::SignalData<void(bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Signal 'toggleFullScreen'
        QtMocHelpers::SignalData<void(bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Signal 'toggleKeyboardShortcuts'
        QtMocHelpers::SignalData<void(bool)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Signal 'toggleBroadcasting'
        QtMocHelpers::SignalData<void(bool)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Signal 'toggleRecording'
        QtMocHelpers::SignalData<void(bool)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 21 },
        }}),
        // Signal 'toggleVinylControl'
        QtMocHelpers::SignalData<void(int)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Signal 'visitUrl'
        QtMocHelpers::SignalData<void(const QString &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 24 },
        }}),
        // Signal 'quit'
        QtMocHelpers::SignalData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'internalRecordingStateChange'
        QtMocHelpers::SignalData<void(bool)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 27 },
        }}),
        // Signal 'internalBroadcastingStateChange'
        QtMocHelpers::SignalData<void(bool)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 29 },
        }}),
        // Signal 'internalFullScreenStateChange'
        QtMocHelpers::SignalData<void(bool)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 31 },
        }}),
        // Signal 'internalLibraryScanActive'
        QtMocHelpers::SignalData<void(bool)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 33 },
        }}),
        // Signal 'internalDeveloperToolsStateChange'
        QtMocHelpers::SignalData<void(bool)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
        // Signal 'internalKeywheelStateChanged'
        QtMocHelpers::SignalData<void(int)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 36 },
        }}),
        // Signal 'internalOnNewSkinLoaded'
        QtMocHelpers::SignalData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'internalOnNewSkinAboutToLoad'
        QtMocHelpers::SignalData<void()>(38, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onLibraryScanStarted'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onLibraryScanFinished'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onRecordingStateChange'
        QtMocHelpers::SlotData<void(bool)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 27 },
        }}),
        // Slot 'onBroadcastingStateChange'
        QtMocHelpers::SlotData<void(bool)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 29 },
        }}),
        // Slot 'onNewSkinAboutToLoad'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onNewSkinLoaded'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onDeveloperToolsHidden'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onDeveloperToolsShown'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onFullScreenStateChange'
        QtMocHelpers::SlotData<void(bool)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 31 },
        }}),
        // Slot 'onVinylControlDeckEnabledStateChange'
        QtMocHelpers::SlotData<void(int, bool)>(48, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 }, { QMetaType::Bool, 21 },
        }}),
        // Slot 'onNumberOfDecksChanged'
        QtMocHelpers::SlotData<void(int)>(49, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 50 },
        }}),
        // Slot 'onKeywheelChange'
        QtMocHelpers::SlotData<void(int)>(51, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 36 },
        }}),
        // Slot 'slotToggleMenuBar'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotAutoHideMenuBarToggled'
        QtMocHelpers::SlotData<void(bool)>(53, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 54 },
        }}),
        // Slot 'slotDeveloperStatsExperiment'
        QtMocHelpers::SlotData<void(bool)>(55, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 56 },
        }}),
        // Slot 'slotDeveloperStatsBase'
        QtMocHelpers::SlotData<void(bool)>(57, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 56 },
        }}),
        // Slot 'slotDeveloperDebugger'
        QtMocHelpers::SlotData<void(bool)>(58, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Slot 'slotVisitUrl'
        QtMocHelpers::SlotData<void(const QUrl &)>(59, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 60, 24 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WMainMenuBar, qt_meta_tag_ZN12WMainMenuBarE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject WMainMenuBar::staticMetaObject = { {
    QMetaObject::SuperData::link<QMenuBar::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12WMainMenuBarE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12WMainMenuBarE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12WMainMenuBarE_t>.metaTypes,
    nullptr
} };

void WMainMenuBar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WMainMenuBar *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->createCrate(); break;
        case 1: _t->createPlaylist(); break;
        case 2: _t->loadTrackToDeck((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->reloadSkin(); break;
        case 4: _t->rescanLibrary(); break;
        case 5: _t->exportLibrary(); break;
        case 6: _t->menubarAutoHideChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->showAbout(); break;
        case 8: _t->showKeywheel((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->showPreferences(); break;
        case 10: _t->toggleDeveloperTools((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->toggleFullScreen((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->toggleKeyboardShortcuts((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->toggleBroadcasting((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->toggleRecording((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: _t->toggleVinylControl((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 16: _t->visitUrl((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 17: _t->quit(); break;
        case 18: _t->internalRecordingStateChange((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 19: _t->internalBroadcastingStateChange((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 20: _t->internalFullScreenStateChange((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 21: _t->internalLibraryScanActive((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 22: _t->internalDeveloperToolsStateChange((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 23: _t->internalKeywheelStateChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 24: _t->internalOnNewSkinLoaded(); break;
        case 25: _t->internalOnNewSkinAboutToLoad(); break;
        case 26: _t->onLibraryScanStarted(); break;
        case 27: _t->onLibraryScanFinished(); break;
        case 28: _t->onRecordingStateChange((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 29: _t->onBroadcastingStateChange((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 30: _t->onNewSkinAboutToLoad(); break;
        case 31: _t->onNewSkinLoaded(); break;
        case 32: _t->onDeveloperToolsHidden(); break;
        case 33: _t->onDeveloperToolsShown(); break;
        case 34: _t->onFullScreenStateChange((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 35: _t->onVinylControlDeckEnabledStateChange((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 36: _t->onNumberOfDecksChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 37: _t->onKeywheelChange((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 38: _t->slotToggleMenuBar(); break;
        case 39: _t->slotAutoHideMenuBarToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 40: _t->slotDeveloperStatsExperiment((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 41: _t->slotDeveloperStatsBase((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 42: _t->slotDeveloperDebugger((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 43: _t->slotVisitUrl((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WMainMenuBar::*)()>(_a, &WMainMenuBar::createCrate, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WMainMenuBar::*)()>(_a, &WMainMenuBar::createPlaylist, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WMainMenuBar::*)(int )>(_a, &WMainMenuBar::loadTrackToDeck, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WMainMenuBar::*)()>(_a, &WMainMenuBar::reloadSkin, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (WMainMenuBar::*)()>(_a, &WMainMenuBar::rescanLibrary, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (WMainMenuBar::*)()>(_a, &WMainMenuBar::exportLibrary, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (WMainMenuBar::*)(bool )>(_a, &WMainMenuBar::menubarAutoHideChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (WMainMenuBar::*)()>(_a, &WMainMenuBar::showAbout, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (WMainMenuBar::*)(bool )>(_a, &WMainMenuBar::showKeywheel, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (WMainMenuBar::*)()>(_a, &WMainMenuBar::showPreferences, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (WMainMenuBar::*)(bool )>(_a, &WMainMenuBar::toggleDeveloperTools, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (WMainMenuBar::*)(bool )>(_a, &WMainMenuBar::toggleFullScreen, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (WMainMenuBar::*)(bool )>(_a, &WMainMenuBar::toggleKeyboardShortcuts, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (WMainMenuBar::*)(bool )>(_a, &WMainMenuBar::toggleBroadcasting, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (WMainMenuBar::*)(bool )>(_a, &WMainMenuBar::toggleRecording, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (WMainMenuBar::*)(int )>(_a, &WMainMenuBar::toggleVinylControl, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (WMainMenuBar::*)(const QString & )>(_a, &WMainMenuBar::visitUrl, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (WMainMenuBar::*)()>(_a, &WMainMenuBar::quit, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (WMainMenuBar::*)(bool )>(_a, &WMainMenuBar::internalRecordingStateChange, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (WMainMenuBar::*)(bool )>(_a, &WMainMenuBar::internalBroadcastingStateChange, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (WMainMenuBar::*)(bool )>(_a, &WMainMenuBar::internalFullScreenStateChange, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (WMainMenuBar::*)(bool )>(_a, &WMainMenuBar::internalLibraryScanActive, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (WMainMenuBar::*)(bool )>(_a, &WMainMenuBar::internalDeveloperToolsStateChange, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (WMainMenuBar::*)(int )>(_a, &WMainMenuBar::internalKeywheelStateChanged, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (WMainMenuBar::*)()>(_a, &WMainMenuBar::internalOnNewSkinLoaded, 24))
            return;
        if (QtMocHelpers::indexOfMethod<void (WMainMenuBar::*)()>(_a, &WMainMenuBar::internalOnNewSkinAboutToLoad, 25))
            return;
    }
}

const QMetaObject *WMainMenuBar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *WMainMenuBar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12WMainMenuBarE_t>.strings))
        return static_cast<void*>(this);
    return QMenuBar::qt_metacast(_clname);
}

int WMainMenuBar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMenuBar::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 44)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 44;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 44)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 44;
    }
    return _id;
}

// SIGNAL 0
void WMainMenuBar::createCrate()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void WMainMenuBar::createPlaylist()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void WMainMenuBar::loadTrackToDeck(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void WMainMenuBar::reloadSkin()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void WMainMenuBar::rescanLibrary()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void WMainMenuBar::exportLibrary()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void WMainMenuBar::menubarAutoHideChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void WMainMenuBar::showAbout()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void WMainMenuBar::showKeywheel(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void WMainMenuBar::showPreferences()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void WMainMenuBar::toggleDeveloperTools(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void WMainMenuBar::toggleFullScreen(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void WMainMenuBar::toggleKeyboardShortcuts(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void WMainMenuBar::toggleBroadcasting(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void WMainMenuBar::toggleRecording(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void WMainMenuBar::toggleVinylControl(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void WMainMenuBar::visitUrl(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void WMainMenuBar::quit()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void WMainMenuBar::internalRecordingStateChange(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void WMainMenuBar::internalBroadcastingStateChange(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}

// SIGNAL 20
void WMainMenuBar::internalFullScreenStateChange(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void WMainMenuBar::internalLibraryScanActive(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1);
}

// SIGNAL 22
void WMainMenuBar::internalDeveloperToolsStateChange(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 22, nullptr, _t1);
}

// SIGNAL 23
void WMainMenuBar::internalKeywheelStateChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 23, nullptr, _t1);
}

// SIGNAL 24
void WMainMenuBar::internalOnNewSkinLoaded()
{
    QMetaObject::activate(this, &staticMetaObject, 24, nullptr);
}

// SIGNAL 25
void WMainMenuBar::internalOnNewSkinAboutToLoad()
{
    QMetaObject::activate(this, &staticMetaObject, 25, nullptr);
}
QT_WARNING_POP
