/********************************************************************************
** Form generated from reading UI file 'dlgprefsounditem.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLGPREFSOUNDITEM_H
#define UI_DLGPREFSOUNDITEM_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DlgPrefSoundItem
{
public:
    QHBoxLayout *horizontalLayout;
    QLabel *typeLabel;
    QComboBox *deviceComboBox;
    QComboBox *channelComboBox;

    void setupUi(QWidget *DlgPrefSoundItem)
    {
        if (DlgPrefSoundItem->objectName().isEmpty())
            DlgPrefSoundItem->setObjectName("DlgPrefSoundItem");
        DlgPrefSoundItem->resize(283, 26);
        horizontalLayout = new QHBoxLayout(DlgPrefSoundItem);
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        horizontalLayout->setObjectName("horizontalLayout");
        typeLabel = new QLabel(DlgPrefSoundItem);
        typeLabel->setObjectName("typeLabel");

        horizontalLayout->addWidget(typeLabel);

        deviceComboBox = new QComboBox(DlgPrefSoundItem);
        deviceComboBox->setObjectName("deviceComboBox");

        horizontalLayout->addWidget(deviceComboBox);

        channelComboBox = new QComboBox(DlgPrefSoundItem);
        channelComboBox->setObjectName("channelComboBox");

        horizontalLayout->addWidget(channelComboBox);

#if QT_CONFIG(shortcut)
        typeLabel->setBuddy(deviceComboBox);
#endif // QT_CONFIG(shortcut)

        retranslateUi(DlgPrefSoundItem);

        QMetaObject::connectSlotsByName(DlgPrefSoundItem);
    } // setupUi

    void retranslateUi(QWidget *DlgPrefSoundItem)
    {
        DlgPrefSoundItem->setWindowTitle(QCoreApplication::translate("DlgPrefSoundItem", "Sound Item Preferences", nullptr));
        typeLabel->setText(QCoreApplication::translate("DlgPrefSoundItem", "Type (#)", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgPrefSoundItem: public Ui_DlgPrefSoundItem {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLGPREFSOUNDITEM_H
