/****************************************************************************
** Meta object code from reading C++ file 'wlibrarysidebar.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/widget/wlibrarysidebar.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wlibrarysidebar.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15WLibrarySidebarE_t {};
} // unnamed namespace

template <> constexpr inline auto WLibrarySidebar::qt_create_metaobjectdata<qt_meta_tag_ZN15WLibrarySidebarE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "WLibrarySidebar",
        "rightClicked",
        "",
        "QPoint",
        "QModelIndex",
        "renameItem",
        "deleteItem",
        "setLibraryFocus",
        "FocusWidget",
        "newFocus",
        "selectIndex",
        "selectChildIndex",
        "selectItem",
        "slotSetFont",
        "QFont",
        "font"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'rightClicked'
        QtMocHelpers::SignalData<void(const QPoint &, const QModelIndex &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 4, 2 },
        }}),
        // Signal 'renameItem'
        QtMocHelpers::SignalData<void(const QModelIndex &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 2 },
        }}),
        // Signal 'deleteItem'
        QtMocHelpers::SignalData<void(const QModelIndex &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 2 },
        }}),
        // Signal 'setLibraryFocus'
        QtMocHelpers::SignalData<FocusWidget(FocusWidget)>(7, 2, QMC::AccessPublic, 0x80000000 | 8, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'selectIndex'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 2 },
        }}),
        // Slot 'selectChildIndex'
        QtMocHelpers::SlotData<void(const QModelIndex &, bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 2 }, { QMetaType::Bool, 12 },
        }}),
        // Slot 'selectChildIndex'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(11, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 4, 2 },
        }}),
        // Slot 'slotSetFont'
        QtMocHelpers::SlotData<void(const QFont &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WLibrarySidebar, qt_meta_tag_ZN15WLibrarySidebarE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject WLibrarySidebar::staticMetaObject = { {
    QMetaObject::SuperData::link<QTreeView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15WLibrarySidebarE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15WLibrarySidebarE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15WLibrarySidebarE_t>.metaTypes,
    nullptr
} };

void WLibrarySidebar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WLibrarySidebar *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->rightClicked((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 1: _t->renameItem((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 2: _t->deleteItem((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 3: { FocusWidget _r = _t->setLibraryFocus((*reinterpret_cast<std::add_pointer_t<FocusWidget>>(_a[1])));
            if (_a[0]) *reinterpret_cast<FocusWidget*>(_a[0]) = std::move(_r); }  break;
        case 4: _t->selectIndex((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 5: _t->selectChildIndex((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 6: _t->selectChildIndex((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 7: _t->slotSetFont((*reinterpret_cast<std::add_pointer_t<QFont>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WLibrarySidebar::*)(const QPoint & , const QModelIndex & )>(_a, &WLibrarySidebar::rightClicked, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WLibrarySidebar::*)(const QModelIndex & )>(_a, &WLibrarySidebar::renameItem, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WLibrarySidebar::*)(const QModelIndex & )>(_a, &WLibrarySidebar::deleteItem, 2))
            return;
        if (QtMocHelpers::indexOfMethod<FocusWidget (WLibrarySidebar::*)(FocusWidget )>(_a, &WLibrarySidebar::setLibraryFocus, 3))
            return;
    }
}

const QMetaObject *WLibrarySidebar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *WLibrarySidebar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15WLibrarySidebarE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "WBaseWidget"))
        return static_cast< WBaseWidget*>(this);
    return QTreeView::qt_metacast(_clname);
}

int WLibrarySidebar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTreeView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void WLibrarySidebar::rightClicked(const QPoint & _t1, const QModelIndex & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void WLibrarySidebar::renameItem(const QModelIndex & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void WLibrarySidebar::deleteItem(const QModelIndex & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
FocusWidget WLibrarySidebar::setLibraryFocus(FocusWidget _t1)
{
    FocusWidget _t0{};
    QMetaObject::activate<FocusWidget>(this, &staticMetaObject, 3, std::addressof(_t0), _t1);
    return _t0;
}
QT_WARNING_POP
