/****************************************************************************
** Meta object code from reading C++ file 'cuecontrol.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/engine/controls/cuecontrol.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'cuecontrol.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13HotcueControlE_t {};
} // unnamed namespace

template <> constexpr inline auto HotcueControl::qt_create_metaobjectdata<qt_meta_tag_ZN13HotcueControlE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "HotcueControl",
        "hotcueSet",
        "",
        "HotcueControl*",
        "pHotcue",
        "v",
        "HotcueSetMode",
        "mode",
        "hotcueGoto",
        "hotcueGotoAndPlay",
        "hotcueGotoAndStop",
        "hotcueGotoAndLoop",
        "hotcueCueLoop",
        "hotcueActivate",
        "hotcueActivatePreview",
        "hotcueClear",
        "hotcuePositionChanged",
        "newPosition",
        "hotcueEndPositionChanged",
        "newEndPosition",
        "hotcuePlay",
        "slotHotcueSet",
        "slotHotcueSetCue",
        "slotHotcueSetLoop",
        "slotHotcueGoto",
        "slotHotcueGotoAndPlay",
        "slotHotcueGotoAndStop",
        "slotHotcueGotoAndLoop",
        "slotHotcueCueLoop",
        "slotHotcueActivate",
        "slotHotcueActivateCue",
        "slotHotcueActivateLoop",
        "slotHotcueActivatePreview",
        "slotHotcueClear",
        "slotHotcueEndPositionChanged",
        "slotHotcuePositionChanged",
        "slotHotcueColorChangeRequest",
        "newColor"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'hotcueSet'
        QtMocHelpers::SignalData<void(HotcueControl *, double, HotcueSetMode)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Double, 5 }, { 0x80000000 | 6, 7 },
        }}),
        // Signal 'hotcueGoto'
        QtMocHelpers::SignalData<void(HotcueControl *, double)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Double, 5 },
        }}),
        // Signal 'hotcueGotoAndPlay'
        QtMocHelpers::SignalData<void(HotcueControl *, double)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Double, 5 },
        }}),
        // Signal 'hotcueGotoAndStop'
        QtMocHelpers::SignalData<void(HotcueControl *, double)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Double, 5 },
        }}),
        // Signal 'hotcueGotoAndLoop'
        QtMocHelpers::SignalData<void(HotcueControl *, double)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Double, 5 },
        }}),
        // Signal 'hotcueCueLoop'
        QtMocHelpers::SignalData<void(HotcueControl *, double)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Double, 5 },
        }}),
        // Signal 'hotcueActivate'
        QtMocHelpers::SignalData<void(HotcueControl *, double, HotcueSetMode)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Double, 5 }, { 0x80000000 | 6, 7 },
        }}),
        // Signal 'hotcueActivatePreview'
        QtMocHelpers::SignalData<void(HotcueControl *, double)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Double, 5 },
        }}),
        // Signal 'hotcueClear'
        QtMocHelpers::SignalData<void(HotcueControl *, double)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Double, 5 },
        }}),
        // Signal 'hotcuePositionChanged'
        QtMocHelpers::SignalData<void(HotcueControl *, double)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Double, 17 },
        }}),
        // Signal 'hotcueEndPositionChanged'
        QtMocHelpers::SignalData<void(HotcueControl *, double)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Double, 19 },
        }}),
        // Signal 'hotcuePlay'
        QtMocHelpers::SignalData<void(double)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 5 },
        }}),
        // Slot 'slotHotcueSet'
        QtMocHelpers::SlotData<void(double)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 5 },
        }}),
        // Slot 'slotHotcueSetCue'
        QtMocHelpers::SlotData<void(double)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 5 },
        }}),
        // Slot 'slotHotcueSetLoop'
        QtMocHelpers::SlotData<void(double)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 5 },
        }}),
        // Slot 'slotHotcueGoto'
        QtMocHelpers::SlotData<void(double)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 5 },
        }}),
        // Slot 'slotHotcueGotoAndPlay'
        QtMocHelpers::SlotData<void(double)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 5 },
        }}),
        // Slot 'slotHotcueGotoAndStop'
        QtMocHelpers::SlotData<void(double)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 5 },
        }}),
        // Slot 'slotHotcueGotoAndLoop'
        QtMocHelpers::SlotData<void(double)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 5 },
        }}),
        // Slot 'slotHotcueCueLoop'
        QtMocHelpers::SlotData<void(double)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 5 },
        }}),
        // Slot 'slotHotcueActivate'
        QtMocHelpers::SlotData<void(double)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 5 },
        }}),
        // Slot 'slotHotcueActivateCue'
        QtMocHelpers::SlotData<void(double)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 5 },
        }}),
        // Slot 'slotHotcueActivateLoop'
        QtMocHelpers::SlotData<void(double)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 5 },
        }}),
        // Slot 'slotHotcueActivatePreview'
        QtMocHelpers::SlotData<void(double)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 5 },
        }}),
        // Slot 'slotHotcueClear'
        QtMocHelpers::SlotData<void(double)>(33, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 5 },
        }}),
        // Slot 'slotHotcueEndPositionChanged'
        QtMocHelpers::SlotData<void(double)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 17 },
        }}),
        // Slot 'slotHotcuePositionChanged'
        QtMocHelpers::SlotData<void(double)>(35, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 17 },
        }}),
        // Slot 'slotHotcueColorChangeRequest'
        QtMocHelpers::SlotData<void(double)>(36, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 37 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<HotcueControl, qt_meta_tag_ZN13HotcueControlE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject HotcueControl::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13HotcueControlE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13HotcueControlE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13HotcueControlE_t>.metaTypes,
    nullptr
} };

void HotcueControl::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<HotcueControl *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->hotcueSet((*reinterpret_cast<std::add_pointer_t<HotcueControl*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<HotcueSetMode>>(_a[3]))); break;
        case 1: _t->hotcueGoto((*reinterpret_cast<std::add_pointer_t<HotcueControl*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 2: _t->hotcueGotoAndPlay((*reinterpret_cast<std::add_pointer_t<HotcueControl*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 3: _t->hotcueGotoAndStop((*reinterpret_cast<std::add_pointer_t<HotcueControl*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 4: _t->hotcueGotoAndLoop((*reinterpret_cast<std::add_pointer_t<HotcueControl*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 5: _t->hotcueCueLoop((*reinterpret_cast<std::add_pointer_t<HotcueControl*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 6: _t->hotcueActivate((*reinterpret_cast<std::add_pointer_t<HotcueControl*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<HotcueSetMode>>(_a[3]))); break;
        case 7: _t->hotcueActivatePreview((*reinterpret_cast<std::add_pointer_t<HotcueControl*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 8: _t->hotcueClear((*reinterpret_cast<std::add_pointer_t<HotcueControl*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 9: _t->hotcuePositionChanged((*reinterpret_cast<std::add_pointer_t<HotcueControl*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 10: _t->hotcueEndPositionChanged((*reinterpret_cast<std::add_pointer_t<HotcueControl*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 11: _t->hotcuePlay((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 12: _t->slotHotcueSet((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 13: _t->slotHotcueSetCue((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 14: _t->slotHotcueSetLoop((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 15: _t->slotHotcueGoto((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 16: _t->slotHotcueGotoAndPlay((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 17: _t->slotHotcueGotoAndStop((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 18: _t->slotHotcueGotoAndLoop((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 19: _t->slotHotcueCueLoop((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 20: _t->slotHotcueActivate((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 21: _t->slotHotcueActivateCue((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 22: _t->slotHotcueActivateLoop((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 23: _t->slotHotcueActivatePreview((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 24: _t->slotHotcueClear((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 25: _t->slotHotcueEndPositionChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 26: _t->slotHotcuePositionChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 27: _t->slotHotcueColorChangeRequest((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< HotcueControl* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< HotcueControl* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< HotcueControl* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< HotcueControl* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< HotcueControl* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< HotcueControl* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< HotcueControl* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< HotcueControl* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< HotcueControl* >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< HotcueControl* >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< HotcueControl* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (HotcueControl::*)(HotcueControl * , double , HotcueSetMode )>(_a, &HotcueControl::hotcueSet, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (HotcueControl::*)(HotcueControl * , double )>(_a, &HotcueControl::hotcueGoto, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (HotcueControl::*)(HotcueControl * , double )>(_a, &HotcueControl::hotcueGotoAndPlay, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (HotcueControl::*)(HotcueControl * , double )>(_a, &HotcueControl::hotcueGotoAndStop, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (HotcueControl::*)(HotcueControl * , double )>(_a, &HotcueControl::hotcueGotoAndLoop, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (HotcueControl::*)(HotcueControl * , double )>(_a, &HotcueControl::hotcueCueLoop, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (HotcueControl::*)(HotcueControl * , double , HotcueSetMode )>(_a, &HotcueControl::hotcueActivate, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (HotcueControl::*)(HotcueControl * , double )>(_a, &HotcueControl::hotcueActivatePreview, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (HotcueControl::*)(HotcueControl * , double )>(_a, &HotcueControl::hotcueClear, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (HotcueControl::*)(HotcueControl * , double )>(_a, &HotcueControl::hotcuePositionChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (HotcueControl::*)(HotcueControl * , double )>(_a, &HotcueControl::hotcueEndPositionChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (HotcueControl::*)(double )>(_a, &HotcueControl::hotcuePlay, 11))
            return;
    }
}

const QMetaObject *HotcueControl::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *HotcueControl::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13HotcueControlE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int HotcueControl::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 28)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 28;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 28)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 28;
    }
    return _id;
}

// SIGNAL 0
void HotcueControl::hotcueSet(HotcueControl * _t1, double _t2, HotcueSetMode _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void HotcueControl::hotcueGoto(HotcueControl * _t1, double _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void HotcueControl::hotcueGotoAndPlay(HotcueControl * _t1, double _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void HotcueControl::hotcueGotoAndStop(HotcueControl * _t1, double _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void HotcueControl::hotcueGotoAndLoop(HotcueControl * _t1, double _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void HotcueControl::hotcueCueLoop(HotcueControl * _t1, double _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void HotcueControl::hotcueActivate(HotcueControl * _t1, double _t2, HotcueSetMode _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2, _t3);
}

// SIGNAL 7
void HotcueControl::hotcueActivatePreview(HotcueControl * _t1, double _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}

// SIGNAL 8
void HotcueControl::hotcueClear(HotcueControl * _t1, double _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2);
}

// SIGNAL 9
void HotcueControl::hotcuePositionChanged(HotcueControl * _t1, double _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2);
}

// SIGNAL 10
void HotcueControl::hotcueEndPositionChanged(HotcueControl * _t1, double _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1, _t2);
}

// SIGNAL 11
void HotcueControl::hotcuePlay(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN10CueControlE_t {};
} // unnamed namespace

template <> constexpr inline auto CueControl::qt_create_metaobjectdata<qt_meta_tag_ZN10CueControlE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CueControl",
        "loopRemove",
        "",
        "slotLoopReset",
        "slotLoopEnabledChanged",
        "enabled",
        "slotLoopUpdated",
        "mixxx::audio::FramePos",
        "startPosition",
        "endPosition",
        "quantizeChanged",
        "v",
        "slotCueModeChanged",
        "cueUpdated",
        "trackAnalyzed",
        "trackCuesUpdated",
        "hotcueSet",
        "HotcueControl*",
        "pControl",
        "HotcueSetMode",
        "mode",
        "hotcueGoto",
        "hotcueGotoAndPlay",
        "hotcueGotoAndStop",
        "hotcueGotoAndLoop",
        "hotcueCueLoop",
        "hotcueActivate",
        "hotcueActivatePreview",
        "updateCurrentlyPreviewingIndex",
        "hotcueIndex",
        "hotcueClear",
        "hotcuePositionChanged",
        "newPosition",
        "hotcueEndPositionChanged",
        "newEndPosition",
        "hotcueFocusColorNext",
        "hotcueFocusColorPrev",
        "passthroughChanged",
        "cueSet",
        "cueClear",
        "cueGoto",
        "cueGotoAndPlay",
        "cueGotoAndStop",
        "cuePreview",
        "cueCDJ",
        "cueDenon",
        "cuePlay",
        "cueDefault",
        "pause",
        "playStutter",
        "introStartSet",
        "introStartClear",
        "introStartActivate",
        "introEndSet",
        "introEndClear",
        "introEndActivate",
        "outroStartSet",
        "outroStartClear",
        "outroStartActivate",
        "outroEndSet",
        "outroEndClear",
        "outroEndActivate"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'loopRemove'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotLoopReset'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotLoopEnabledChanged'
        QtMocHelpers::SlotData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Slot 'slotLoopUpdated'
        QtMocHelpers::SlotData<void(mixxx::audio::FramePos, mixxx::audio::FramePos)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 }, { 0x80000000 | 7, 9 },
        }}),
        // Slot 'quantizeChanged'
        QtMocHelpers::SlotData<void(double)>(10, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Slot 'slotCueModeChanged'
        QtMocHelpers::SlotData<void(double)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Slot 'cueUpdated'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'trackAnalyzed'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'trackCuesUpdated'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'hotcueSet'
        QtMocHelpers::SlotData<void(HotcueControl *, double, HotcueSetMode)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { QMetaType::Double, 11 }, { 0x80000000 | 19, 20 },
        }}),
        // Slot 'hotcueGoto'
        QtMocHelpers::SlotData<void(HotcueControl *, double)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { QMetaType::Double, 11 },
        }}),
        // Slot 'hotcueGotoAndPlay'
        QtMocHelpers::SlotData<void(HotcueControl *, double)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { QMetaType::Double, 11 },
        }}),
        // Slot 'hotcueGotoAndStop'
        QtMocHelpers::SlotData<void(HotcueControl *, double)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { QMetaType::Double, 11 },
        }}),
        // Slot 'hotcueGotoAndLoop'
        QtMocHelpers::SlotData<void(HotcueControl *, double)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { QMetaType::Double, 11 },
        }}),
        // Slot 'hotcueCueLoop'
        QtMocHelpers::SlotData<void(HotcueControl *, double)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { QMetaType::Double, 11 },
        }}),
        // Slot 'hotcueActivate'
        QtMocHelpers::SlotData<void(HotcueControl *, double, HotcueSetMode)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { QMetaType::Double, 11 }, { 0x80000000 | 19, 20 },
        }}),
        // Slot 'hotcueActivatePreview'
        QtMocHelpers::SlotData<void(HotcueControl *, double)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { QMetaType::Double, 11 },
        }}),
        // Slot 'updateCurrentlyPreviewingIndex'
        QtMocHelpers::SlotData<void(int)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 29 },
        }}),
        // Slot 'hotcueClear'
        QtMocHelpers::SlotData<void(HotcueControl *, double)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { QMetaType::Double, 11 },
        }}),
        // Slot 'hotcuePositionChanged'
        QtMocHelpers::SlotData<void(HotcueControl *, double)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { QMetaType::Double, 32 },
        }}),
        // Slot 'hotcueEndPositionChanged'
        QtMocHelpers::SlotData<void(HotcueControl *, double)>(33, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { QMetaType::Double, 34 },
        }}),
        // Slot 'hotcueFocusColorNext'
        QtMocHelpers::SlotData<void(double)>(35, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Slot 'hotcueFocusColorPrev'
        QtMocHelpers::SlotData<void(double)>(36, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Slot 'passthroughChanged'
        QtMocHelpers::SlotData<void(double)>(37, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Slot 'cueSet'
        QtMocHelpers::SlotData<void(double)>(38, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Slot 'cueClear'
        QtMocHelpers::SlotData<void(double)>(39, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Slot 'cueGoto'
        QtMocHelpers::SlotData<void(double)>(40, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Slot 'cueGotoAndPlay'
        QtMocHelpers::SlotData<void(double)>(41, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Slot 'cueGotoAndStop'
        QtMocHelpers::SlotData<void(double)>(42, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Slot 'cuePreview'
        QtMocHelpers::SlotData<void(double)>(43, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Slot 'cueCDJ'
        QtMocHelpers::SlotData<void(double)>(44, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Slot 'cueDenon'
        QtMocHelpers::SlotData<void(double)>(45, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Slot 'cuePlay'
        QtMocHelpers::SlotData<void(double)>(46, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Slot 'cueDefault'
        QtMocHelpers::SlotData<void(double)>(47, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Slot 'pause'
        QtMocHelpers::SlotData<void(double)>(48, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Slot 'playStutter'
        QtMocHelpers::SlotData<void(double)>(49, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Slot 'introStartSet'
        QtMocHelpers::SlotData<void(double)>(50, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Slot 'introStartClear'
        QtMocHelpers::SlotData<void(double)>(51, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Slot 'introStartActivate'
        QtMocHelpers::SlotData<void(double)>(52, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Slot 'introEndSet'
        QtMocHelpers::SlotData<void(double)>(53, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Slot 'introEndClear'
        QtMocHelpers::SlotData<void(double)>(54, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Slot 'introEndActivate'
        QtMocHelpers::SlotData<void(double)>(55, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Slot 'outroStartSet'
        QtMocHelpers::SlotData<void(double)>(56, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Slot 'outroStartClear'
        QtMocHelpers::SlotData<void(double)>(57, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Slot 'outroStartActivate'
        QtMocHelpers::SlotData<void(double)>(58, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Slot 'outroEndSet'
        QtMocHelpers::SlotData<void(double)>(59, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Slot 'outroEndClear'
        QtMocHelpers::SlotData<void(double)>(60, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Slot 'outroEndActivate'
        QtMocHelpers::SlotData<void(double)>(61, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CueControl, qt_meta_tag_ZN10CueControlE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CueControl::staticMetaObject = { {
    QMetaObject::SuperData::link<EngineControl::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10CueControlE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10CueControlE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10CueControlE_t>.metaTypes,
    nullptr
} };

void CueControl::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CueControl *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->loopRemove(); break;
        case 1: _t->slotLoopReset(); break;
        case 2: _t->slotLoopEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->slotLoopUpdated((*reinterpret_cast<std::add_pointer_t<mixxx::audio::FramePos>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<mixxx::audio::FramePos>>(_a[2]))); break;
        case 4: _t->quantizeChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 5: _t->slotCueModeChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 6: _t->cueUpdated(); break;
        case 7: _t->trackAnalyzed(); break;
        case 8: _t->trackCuesUpdated(); break;
        case 9: _t->hotcueSet((*reinterpret_cast<std::add_pointer_t<HotcueControl*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<HotcueSetMode>>(_a[3]))); break;
        case 10: _t->hotcueGoto((*reinterpret_cast<std::add_pointer_t<HotcueControl*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 11: _t->hotcueGotoAndPlay((*reinterpret_cast<std::add_pointer_t<HotcueControl*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 12: _t->hotcueGotoAndStop((*reinterpret_cast<std::add_pointer_t<HotcueControl*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 13: _t->hotcueGotoAndLoop((*reinterpret_cast<std::add_pointer_t<HotcueControl*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 14: _t->hotcueCueLoop((*reinterpret_cast<std::add_pointer_t<HotcueControl*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 15: _t->hotcueActivate((*reinterpret_cast<std::add_pointer_t<HotcueControl*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<HotcueSetMode>>(_a[3]))); break;
        case 16: _t->hotcueActivatePreview((*reinterpret_cast<std::add_pointer_t<HotcueControl*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 17: _t->updateCurrentlyPreviewingIndex((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 18: _t->hotcueClear((*reinterpret_cast<std::add_pointer_t<HotcueControl*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 19: _t->hotcuePositionChanged((*reinterpret_cast<std::add_pointer_t<HotcueControl*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 20: _t->hotcueEndPositionChanged((*reinterpret_cast<std::add_pointer_t<HotcueControl*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 21: _t->hotcueFocusColorNext((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 22: _t->hotcueFocusColorPrev((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 23: _t->passthroughChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 24: _t->cueSet((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 25: _t->cueClear((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 26: _t->cueGoto((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 27: _t->cueGotoAndPlay((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 28: _t->cueGotoAndStop((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 29: _t->cuePreview((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 30: _t->cueCDJ((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 31: _t->cueDenon((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 32: _t->cuePlay((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 33: _t->cueDefault((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 34: _t->pause((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 35: _t->playStutter((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 36: _t->introStartSet((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 37: _t->introStartClear((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 38: _t->introStartActivate((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 39: _t->introEndSet((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 40: _t->introEndClear((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 41: _t->introEndActivate((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 42: _t->outroStartSet((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 43: _t->outroStartClear((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 44: _t->outroStartActivate((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 45: _t->outroEndSet((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 46: _t->outroEndClear((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 47: _t->outroEndActivate((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< mixxx::audio::FramePos >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< HotcueControl* >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< HotcueControl* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< HotcueControl* >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< HotcueControl* >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< HotcueControl* >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< HotcueControl* >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< HotcueControl* >(); break;
            }
            break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< HotcueControl* >(); break;
            }
            break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< HotcueControl* >(); break;
            }
            break;
        case 19:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< HotcueControl* >(); break;
            }
            break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< HotcueControl* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CueControl::*)()>(_a, &CueControl::loopRemove, 0))
            return;
    }
}

const QMetaObject *CueControl::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CueControl::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10CueControlE_t>.strings))
        return static_cast<void*>(this);
    return EngineControl::qt_metacast(_clname);
}

int CueControl::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = EngineControl::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 48)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 48;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 48)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 48;
    }
    return _id;
}

// SIGNAL 0
void CueControl::loopRemove()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
