/********************************************************************************
** Form generated from reading UI file 'dlgprefcolorsdlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLGPREFCOLORSDLG_H
#define UI_DLGPREFCOLORSDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DlgPrefColorsDlg
{
public:
    QGridLayout *gridLayout;
    QLabel *labelTrackColors;
    QComboBox *comboBoxTrackColors;
    QPushButton *pushButtonEditTrackPalette;
    QLabel *labelHotcueColors;
    QComboBox *comboBoxHotcueColors;
    QPushButton *pushButtonEditHotcuePalette;
    QLabel *labelHotcueDefaultColor;
    QComboBox *comboBoxHotcueDefaultColor;
    QPushButton *pushButtonReplaceCueColor;
    QLabel *labelLoopDefaultColor;
    QComboBox *comboBoxLoopDefaultColor;
    QSpacerItem *spacerItem;

    void setupUi(QWidget *DlgPrefColorsDlg)
    {
        if (DlgPrefColorsDlg->objectName().isEmpty())
            DlgPrefColorsDlg->setObjectName("DlgPrefColorsDlg");
        DlgPrefColorsDlg->resize(656, 333);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(DlgPrefColorsDlg->sizePolicy().hasHeightForWidth());
        DlgPrefColorsDlg->setSizePolicy(sizePolicy);
        gridLayout = new QGridLayout(DlgPrefColorsDlg);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        labelTrackColors = new QLabel(DlgPrefColorsDlg);
        labelTrackColors->setObjectName("labelTrackColors");

        gridLayout->addWidget(labelTrackColors, 0, 0, 1, 1);

        comboBoxTrackColors = new QComboBox(DlgPrefColorsDlg);
        comboBoxTrackColors->setObjectName("comboBoxTrackColors");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(comboBoxTrackColors->sizePolicy().hasHeightForWidth());
        comboBoxTrackColors->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(comboBoxTrackColors, 0, 1, 1, 1);

        pushButtonEditTrackPalette = new QPushButton(DlgPrefColorsDlg);
        pushButtonEditTrackPalette->setObjectName("pushButtonEditTrackPalette");

        gridLayout->addWidget(pushButtonEditTrackPalette, 0, 2, 1, 1);

        labelHotcueColors = new QLabel(DlgPrefColorsDlg);
        labelHotcueColors->setObjectName("labelHotcueColors");

        gridLayout->addWidget(labelHotcueColors, 1, 0, 1, 1);

        comboBoxHotcueColors = new QComboBox(DlgPrefColorsDlg);
        comboBoxHotcueColors->setObjectName("comboBoxHotcueColors");
        sizePolicy1.setHeightForWidth(comboBoxHotcueColors->sizePolicy().hasHeightForWidth());
        comboBoxHotcueColors->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(comboBoxHotcueColors, 1, 1, 1, 1);

        pushButtonEditHotcuePalette = new QPushButton(DlgPrefColorsDlg);
        pushButtonEditHotcuePalette->setObjectName("pushButtonEditHotcuePalette");

        gridLayout->addWidget(pushButtonEditHotcuePalette, 1, 2, 1, 1);

        labelHotcueDefaultColor = new QLabel(DlgPrefColorsDlg);
        labelHotcueDefaultColor->setObjectName("labelHotcueDefaultColor");

        gridLayout->addWidget(labelHotcueDefaultColor, 2, 0, 1, 1);

        comboBoxHotcueDefaultColor = new QComboBox(DlgPrefColorsDlg);
        comboBoxHotcueDefaultColor->setObjectName("comboBoxHotcueDefaultColor");

        gridLayout->addWidget(comboBoxHotcueDefaultColor, 2, 1, 1, 1);

        pushButtonReplaceCueColor = new QPushButton(DlgPrefColorsDlg);
        pushButtonReplaceCueColor->setObjectName("pushButtonReplaceCueColor");

        gridLayout->addWidget(pushButtonReplaceCueColor, 2, 2, 1, 1);

        labelLoopDefaultColor = new QLabel(DlgPrefColorsDlg);
        labelLoopDefaultColor->setObjectName("labelLoopDefaultColor");

        gridLayout->addWidget(labelLoopDefaultColor, 3, 0, 1, 1);

        comboBoxLoopDefaultColor = new QComboBox(DlgPrefColorsDlg);
        comboBoxLoopDefaultColor->setObjectName("comboBoxLoopDefaultColor");

        gridLayout->addWidget(comboBoxLoopDefaultColor, 3, 1, 1, 1);

        spacerItem = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(spacerItem, 4, 0, 1, 2);

        QWidget::setTabOrder(comboBoxTrackColors, pushButtonEditTrackPalette);
        QWidget::setTabOrder(pushButtonEditTrackPalette, comboBoxHotcueColors);
        QWidget::setTabOrder(comboBoxHotcueColors, pushButtonEditHotcuePalette);
        QWidget::setTabOrder(pushButtonEditHotcuePalette, comboBoxHotcueDefaultColor);
        QWidget::setTabOrder(comboBoxHotcueDefaultColor, pushButtonReplaceCueColor);

        retranslateUi(DlgPrefColorsDlg);

        QMetaObject::connectSlotsByName(DlgPrefColorsDlg);
    } // setupUi

    void retranslateUi(QWidget *DlgPrefColorsDlg)
    {
        DlgPrefColorsDlg->setWindowTitle(QCoreApplication::translate("DlgPrefColorsDlg", "Color Preferences", nullptr));
        labelTrackColors->setText(QCoreApplication::translate("DlgPrefColorsDlg", "Track palette", nullptr));
        pushButtonEditTrackPalette->setText(QCoreApplication::translate("DlgPrefColorsDlg", "Edit\342\200\246", nullptr));
        labelHotcueColors->setText(QCoreApplication::translate("DlgPrefColorsDlg", "Hotcue palette", nullptr));
        pushButtonEditHotcuePalette->setText(QCoreApplication::translate("DlgPrefColorsDlg", "Edit\342\200\246", nullptr));
        labelHotcueDefaultColor->setText(QCoreApplication::translate("DlgPrefColorsDlg", "Hotcue default color", nullptr));
        pushButtonReplaceCueColor->setText(QCoreApplication::translate("DlgPrefColorsDlg", "Replace\342\200\246", nullptr));
        labelLoopDefaultColor->setText(QCoreApplication::translate("DlgPrefColorsDlg", "Loop default color", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgPrefColorsDlg: public Ui_DlgPrefColorsDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLGPREFCOLORSDLG_H
