/****************************************************************************
** Meta object code from reading C++ file 'trackmetadata_file.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/track/taglib/trackmetadata_file.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'trackmetadata_file.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN5mixxx6taglibE_t {};
} // unnamed namespace

template <> constexpr inline auto mixxx::taglib::qt_create_metaobjectdata<qt_meta_tag_ZN5mixxx6taglibE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "mixxx::taglib",
        "FileType",
        "Unknown",
        "MPEG",
        "OggVorbis",
        "FLAC",
        "MPC",
        "OggFlac",
        "WavPack",
        "Speex",
        "TrueAudio",
        "MP4",
        "ASF",
        "AIFF",
        "WAV",
        "APE",
        "IT",
        "Mod",
        "S3M",
        "XM",
        "Opus",
        "DSF",
        "DSDIFF"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'FileType'
        QtMocHelpers::EnumData<FileType>(1, 1, QMC::EnumIsScoped).add({
            {    2, FileType::Unknown },
            {    3, FileType::MPEG },
            {    4, FileType::OggVorbis },
            {    5, FileType::FLAC },
            {    6, FileType::MPC },
            {    7, FileType::OggFlac },
            {    8, FileType::WavPack },
            {    9, FileType::Speex },
            {   10, FileType::TrueAudio },
            {   11, FileType::MP4 },
            {   12, FileType::ASF },
            {   13, FileType::AIFF },
            {   14, FileType::WAV },
            {   15, FileType::APE },
            {   16, FileType::IT },
            {   17, FileType::Mod },
            {   18, FileType::S3M },
            {   19, FileType::XM },
            {   20, FileType::Opus },
            {   21, FileType::DSF },
            {   22, FileType::DSDIFF },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN5mixxx6taglibE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN5mixxx6taglibE =
    mixxx::taglib::qt_create_metaobjectdata<qt_meta_tag_ZN5mixxx6taglibE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN5mixxx6taglibE =
    qt_staticMetaObjectContent_ZN5mixxx6taglibE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN5mixxx6taglibE =
    qt_staticMetaObjectContent_ZN5mixxx6taglibE.relocatingData;

Q_CONSTINIT const QMetaObject mixxx::taglib::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN5mixxx6taglibE.stringdata,
    qt_staticMetaObjectStaticContent_ZN5mixxx6taglibE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN5mixxx6taglibE.metaTypes,
    nullptr
} };

QT_WARNING_POP
