// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: headers.proto
// Protobuf C++ Version: 7.35.0

#ifndef headers_2eproto_2epb_2eh
#define headers_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

// clang-format off
#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 7035000
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_headers_2eproto

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct TableStruct_headers_2eproto {
  static const ::uint32_t offsets[];
};
namespace mixxx {
namespace library {
class HeaderViewState;
struct HeaderViewStateGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern HeaderViewStateGlobalsTypeInternal HeaderViewState_globals_;
extern const ::google::protobuf::internal::ClassDataLite HeaderViewState_class_data_;
#else
extern const HeaderViewStateGlobalsTypeInternal HeaderViewState_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class HeaderViewState_HeaderState;
struct HeaderViewState_HeaderStateGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern HeaderViewState_HeaderStateGlobalsTypeInternal HeaderViewState_HeaderState_globals_;
extern const ::google::protobuf::internal::ClassDataLite HeaderViewState_HeaderState_class_data_;
#else
extern const HeaderViewState_HeaderStateGlobalsTypeInternal HeaderViewState_HeaderState_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
}  // namespace library
}  // namespace mixxx
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google

namespace mixxx {
namespace library {

// ===================================================================


// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED HeaderViewState_HeaderState final : public ::google::protobuf::MessageLite
/* @@protoc_insertion_point(class_definition:mixxx.library.HeaderViewState.HeaderState) */ {
 public:
  inline HeaderViewState_HeaderState() : HeaderViewState_HeaderState(nullptr) {}
  ~HeaderViewState_HeaderState() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(HeaderViewState_HeaderState* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(HeaderViewState_HeaderState));
  }
#endif

  template <typename = void>
  explicit constexpr HeaderViewState_HeaderState(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline HeaderViewState_HeaderState(const HeaderViewState_HeaderState& from) : HeaderViewState_HeaderState(nullptr, from) {}
  inline HeaderViewState_HeaderState(HeaderViewState_HeaderState&& from) noexcept : HeaderViewState_HeaderState(nullptr, ::std::move(from)) {}
  inline HeaderViewState_HeaderState& operator=(const HeaderViewState_HeaderState& from) {
    CopyFrom(from);
    return *this;
  }
  inline HeaderViewState_HeaderState& operator=(HeaderViewState_HeaderState&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::std::string& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString);
  }
  [[nodiscard]] inline ::std::string* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::std::string>();
  }

  [[nodiscard]] static const HeaderViewState_HeaderState& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<HeaderViewState_HeaderState>(&HeaderViewState_HeaderState_globals_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(HeaderViewState_HeaderState& a, HeaderViewState_HeaderState& b) { a.Swap(&b); }
  inline void Swap(HeaderViewState_HeaderState* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(HeaderViewState_HeaderState* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] HeaderViewState_HeaderState* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::MessageLite::DefaultConstruct<HeaderViewState_HeaderState>(arena);
  }
  void CopyFrom(const HeaderViewState_HeaderState& from);
  void MergeFrom(const HeaderViewState_HeaderState& from) { HeaderViewState_HeaderState::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(HeaderViewState_HeaderState* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "mixxx.library.HeaderViewState.HeaderState"; }

  explicit HeaderViewState_HeaderState(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  HeaderViewState_HeaderState(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const HeaderViewState_HeaderState& from);
  HeaderViewState_HeaderState(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, HeaderViewState_HeaderState&& from) noexcept
      : HeaderViewState_HeaderState(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kColumnNameFieldNumber = 5,
    kHiddenFieldNumber = 1,
    kSizeFieldNumber = 2,
    kLogicalIndexFieldNumber = 3,
    kVisualIndexFieldNumber = 4,
  };
  // optional string column_name = 5;
  [[nodiscard]] bool has_column_name()
      const;
  void clear_column_name() ;
  [[nodiscard]] const ::std::string& column_name() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_column_name(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_column_name();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_column_name();
  void set_allocated_column_name(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_column_name() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_column_name(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_column_name();

  public:
  // optional bool hidden = 1;
  [[nodiscard]] bool has_hidden()
      const;
  void clear_hidden() ;
  [[nodiscard]] bool hidden() const;
  void set_hidden(bool value);

  private:
  bool _internal_hidden() const;
  void _internal_set_hidden(bool value);

  public:
  // optional int32 size = 2;
  [[nodiscard]] bool has_size()
      const;
  void clear_size() ;
  [[nodiscard]] ::int32_t size() const;
  void set_size(::int32_t value);

  private:
  ::int32_t _internal_size() const;
  void _internal_set_size(::int32_t value);

  public:
  // optional int32 logical_index = 3;
  [[nodiscard]] bool has_logical_index()
      const;
  void clear_logical_index() ;
  [[nodiscard]] ::int32_t logical_index() const;
  void set_logical_index(::int32_t value);

  private:
  ::int32_t _internal_logical_index() const;
  void _internal_set_logical_index(::int32_t value);

  public:
  // optional int32 visual_index = 4;
  [[nodiscard]] bool has_visual_index()
      const;
  void clear_visual_index() ;
  [[nodiscard]] ::int32_t visual_index() const;
  void set_visual_index(::int32_t value);

  private:
  ::int32_t _internal_visual_index() const;
  void _internal_set_visual_index(::int32_t value);

  public:
  // @@protoc_insertion_point(class_scope:mixxx.library.HeaderViewState.HeaderState)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 5,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const HeaderViewState_HeaderState& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr column_name_;
    bool hidden_;
    ::int32_t size_;
    ::int32_t logical_index_;
    ::int32_t visual_index_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_headers_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED HeaderViewState final : public ::google::protobuf::MessageLite
/* @@protoc_insertion_point(class_definition:mixxx.library.HeaderViewState) */ {
 public:
  inline HeaderViewState() : HeaderViewState(nullptr) {}
  ~HeaderViewState() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(HeaderViewState* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(HeaderViewState));
  }
#endif

  template <typename = void>
  explicit constexpr HeaderViewState(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline HeaderViewState(const HeaderViewState& from) : HeaderViewState(nullptr, from) {}
  inline HeaderViewState(HeaderViewState&& from) noexcept : HeaderViewState(nullptr, ::std::move(from)) {}
  inline HeaderViewState& operator=(const HeaderViewState& from) {
    CopyFrom(from);
    return *this;
  }
  inline HeaderViewState& operator=(HeaderViewState&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::std::string& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString);
  }
  [[nodiscard]] inline ::std::string* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::std::string>();
  }

  [[nodiscard]] static const HeaderViewState& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<HeaderViewState>(&HeaderViewState_globals_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(HeaderViewState& a, HeaderViewState& b) { a.Swap(&b); }
  inline void Swap(HeaderViewState* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(HeaderViewState* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] HeaderViewState* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::MessageLite::DefaultConstruct<HeaderViewState>(arena);
  }
  void CopyFrom(const HeaderViewState& from);
  void MergeFrom(const HeaderViewState& from) { HeaderViewState::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(HeaderViewState* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "mixxx.library.HeaderViewState"; }

  explicit HeaderViewState(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  HeaderViewState(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const HeaderViewState& from);
  HeaderViewState(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, HeaderViewState&& from) noexcept
      : HeaderViewState(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  // nested types ----------------------------------------------------
  using HeaderState = HeaderViewState_HeaderState;

  // accessors -------------------------------------------------------
  enum : int {
    kHeaderStateFieldNumber = 1,
    kSortIndicatorShownFieldNumber = 2,
    kSortIndicatorSectionFieldNumber = 3,
    kSortOrderFieldNumber = 4,
  };
  // repeated .mixxx.library.HeaderViewState.HeaderState header_state = 1;
  [[nodiscard]] int header_state_size()
      const;
  private:
  int _internal_header_state_size() const;

  public:
  void clear_header_state() ;
  [[nodiscard]] const ::mixxx::library::HeaderViewState_HeaderState& header_state(int index) const;
  [[nodiscard]] ::mixxx::library::HeaderViewState_HeaderState* PROTOBUF_NONNULL mutable_header_state(int index);
  ::mixxx::library::HeaderViewState_HeaderState* PROTOBUF_NONNULL add_header_state();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::mixxx::library::HeaderViewState_HeaderState>&
  header_state() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::mixxx::library::HeaderViewState_HeaderState>* PROTOBUF_NONNULL
  mutable_header_state();

  private:
  const ::google::protobuf::RepeatedPtrField<::mixxx::library::HeaderViewState_HeaderState>& _internal_header_state() const;
  ::google::protobuf::RepeatedPtrField<::mixxx::library::HeaderViewState_HeaderState>* PROTOBUF_NONNULL _internal_mutable_header_state();

  public:
  // optional bool sort_indicator_shown = 2;
  [[nodiscard]] bool has_sort_indicator_shown()
      const;
  void clear_sort_indicator_shown() ;
  [[nodiscard]] bool sort_indicator_shown() const;
  void set_sort_indicator_shown(bool value);

  private:
  bool _internal_sort_indicator_shown() const;
  void _internal_set_sort_indicator_shown(bool value);

  public:
  // optional int32 sort_indicator_section = 3;
  [[nodiscard]] bool has_sort_indicator_section()
      const;
  void clear_sort_indicator_section() ;
  [[nodiscard]] ::int32_t sort_indicator_section() const;
  void set_sort_indicator_section(::int32_t value);

  private:
  ::int32_t _internal_sort_indicator_section() const;
  void _internal_set_sort_indicator_section(::int32_t value);

  public:
  // optional int32 sort_order = 4;
  [[nodiscard]] bool has_sort_order()
      const;
  void clear_sort_order() ;
  [[nodiscard]] ::int32_t sort_order() const;
  void set_sort_order(::int32_t value);

  private:
  ::int32_t _internal_sort_order() const;
  void _internal_set_sort_order(::int32_t value);

  public:
  // @@protoc_insertion_point(class_scope:mixxx.library.HeaderViewState)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<2, 4,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const HeaderViewState& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::mixxx::library::HeaderViewState_HeaderState > header_state_;
    bool sort_indicator_shown_;
    ::int32_t sort_indicator_section_;
    ::int32_t sort_order_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_headers_2eproto;
};

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// HeaderViewState_HeaderState

// optional bool hidden = 1;
inline bool HeaderViewState_HeaderState::has_hidden() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  return value;
}
inline void HeaderViewState_HeaderState::clear_hidden() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.hidden_ = false;
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline bool HeaderViewState_HeaderState::hidden() const {
  // @@protoc_insertion_point(field_get:mixxx.library.HeaderViewState.HeaderState.hidden)
  return _internal_hidden();
}
inline void HeaderViewState_HeaderState::set_hidden(bool value) {
  _internal_set_hidden(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:mixxx.library.HeaderViewState.HeaderState.hidden)
}
inline bool HeaderViewState_HeaderState::_internal_hidden() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.hidden_;
}
inline void HeaderViewState_HeaderState::_internal_set_hidden(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.hidden_ = value;
}

// optional int32 size = 2;
inline bool HeaderViewState_HeaderState::has_size() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  return value;
}
inline void HeaderViewState_HeaderState::clear_size() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.size_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline ::int32_t HeaderViewState_HeaderState::size() const {
  // @@protoc_insertion_point(field_get:mixxx.library.HeaderViewState.HeaderState.size)
  return _internal_size();
}
inline void HeaderViewState_HeaderState::set_size(::int32_t value) {
  _internal_set_size(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:mixxx.library.HeaderViewState.HeaderState.size)
}
inline ::int32_t HeaderViewState_HeaderState::_internal_size() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.size_;
}
inline void HeaderViewState_HeaderState::_internal_set_size(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.size_ = value;
}

// optional int32 logical_index = 3;
inline bool HeaderViewState_HeaderState::has_logical_index() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000008U);
  return value;
}
inline void HeaderViewState_HeaderState::clear_logical_index() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.logical_index_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline ::int32_t HeaderViewState_HeaderState::logical_index() const {
  // @@protoc_insertion_point(field_get:mixxx.library.HeaderViewState.HeaderState.logical_index)
  return _internal_logical_index();
}
inline void HeaderViewState_HeaderState::set_logical_index(::int32_t value) {
  _internal_set_logical_index(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:mixxx.library.HeaderViewState.HeaderState.logical_index)
}
inline ::int32_t HeaderViewState_HeaderState::_internal_logical_index() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.logical_index_;
}
inline void HeaderViewState_HeaderState::_internal_set_logical_index(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.logical_index_ = value;
}

// optional int32 visual_index = 4;
inline bool HeaderViewState_HeaderState::has_visual_index() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000010U);
  return value;
}
inline void HeaderViewState_HeaderState::clear_visual_index() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.visual_index_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::int32_t HeaderViewState_HeaderState::visual_index() const {
  // @@protoc_insertion_point(field_get:mixxx.library.HeaderViewState.HeaderState.visual_index)
  return _internal_visual_index();
}
inline void HeaderViewState_HeaderState::set_visual_index(::int32_t value) {
  _internal_set_visual_index(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:mixxx.library.HeaderViewState.HeaderState.visual_index)
}
inline ::int32_t HeaderViewState_HeaderState::_internal_visual_index() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.visual_index_;
}
inline void HeaderViewState_HeaderState::_internal_set_visual_index(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.visual_index_ = value;
}

// optional string column_name = 5;
inline bool HeaderViewState_HeaderState::has_column_name() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  return value;
}
inline void HeaderViewState_HeaderState::clear_column_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.column_name_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& HeaderViewState_HeaderState::column_name() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:mixxx.library.HeaderViewState.HeaderState.column_name)
  return _internal_column_name();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void HeaderViewState_HeaderState::set_column_name(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.column_name_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:mixxx.library.HeaderViewState.HeaderState.column_name)
}
inline ::std::string* PROTOBUF_NONNULL HeaderViewState_HeaderState::mutable_column_name()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_column_name();
  // @@protoc_insertion_point(field_mutable:mixxx.library.HeaderViewState.HeaderState.column_name)
  return _s;
}
inline const ::std::string& HeaderViewState_HeaderState::_internal_column_name() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.column_name_.Get();
}
inline void HeaderViewState_HeaderState::_internal_set_column_name(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.column_name_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL HeaderViewState_HeaderState::_internal_mutable_column_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.column_name_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE HeaderViewState_HeaderState::release_column_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:mixxx.library.HeaderViewState.HeaderState.column_name)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.column_name_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.column_name_.Set("", GetArena());
  }
  return released;
}
inline void HeaderViewState_HeaderState::set_allocated_column_name(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.column_name_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.column_name_.IsDefault()) {
    _impl_.column_name_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:mixxx.library.HeaderViewState.HeaderState.column_name)
}

// -------------------------------------------------------------------

// HeaderViewState

// repeated .mixxx.library.HeaderViewState.HeaderState header_state = 1;
inline int HeaderViewState::_internal_header_state_size() const {
  return _internal_header_state().size();
}
inline int HeaderViewState::header_state_size() const {
  return _internal_header_state_size();
}
inline void HeaderViewState::clear_header_state() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.header_state_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::mixxx::library::HeaderViewState_HeaderState& HeaderViewState::header_state(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:mixxx.library.HeaderViewState.header_state)
  return _internal_header_state().Get(index);
}
inline ::mixxx::library::HeaderViewState_HeaderState* PROTOBUF_NONNULL HeaderViewState::mutable_header_state(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:mixxx.library.HeaderViewState.header_state)
  return _internal_mutable_header_state()->Mutable(index);
}
inline ::mixxx::library::HeaderViewState_HeaderState* PROTOBUF_NONNULL HeaderViewState::add_header_state()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::mixxx::library::HeaderViewState_HeaderState* _add =
      _internal_mutable_header_state()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:mixxx.library.HeaderViewState.header_state)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::mixxx::library::HeaderViewState_HeaderState>& HeaderViewState::header_state() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:mixxx.library.HeaderViewState.header_state)
  return _internal_header_state();
}
inline ::google::protobuf::RepeatedPtrField<::mixxx::library::HeaderViewState_HeaderState>* PROTOBUF_NONNULL
HeaderViewState::mutable_header_state() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:mixxx.library.HeaderViewState.header_state)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_header_state();
}
inline const ::google::protobuf::RepeatedPtrField<::mixxx::library::HeaderViewState_HeaderState>&
HeaderViewState::_internal_header_state() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.header_state_;
}
inline ::google::protobuf::RepeatedPtrField<::mixxx::library::HeaderViewState_HeaderState>* PROTOBUF_NONNULL
HeaderViewState::_internal_mutable_header_state() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.header_state_;
}

// optional bool sort_indicator_shown = 2;
inline bool HeaderViewState::has_sort_indicator_shown() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  return value;
}
inline void HeaderViewState::clear_sort_indicator_shown() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sort_indicator_shown_ = false;
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline bool HeaderViewState::sort_indicator_shown() const {
  // @@protoc_insertion_point(field_get:mixxx.library.HeaderViewState.sort_indicator_shown)
  return _internal_sort_indicator_shown();
}
inline void HeaderViewState::set_sort_indicator_shown(bool value) {
  _internal_set_sort_indicator_shown(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:mixxx.library.HeaderViewState.sort_indicator_shown)
}
inline bool HeaderViewState::_internal_sort_indicator_shown() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sort_indicator_shown_;
}
inline void HeaderViewState::_internal_set_sort_indicator_shown(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sort_indicator_shown_ = value;
}

// optional int32 sort_indicator_section = 3;
inline bool HeaderViewState::has_sort_indicator_section() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  return value;
}
inline void HeaderViewState::clear_sort_indicator_section() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sort_indicator_section_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline ::int32_t HeaderViewState::sort_indicator_section() const {
  // @@protoc_insertion_point(field_get:mixxx.library.HeaderViewState.sort_indicator_section)
  return _internal_sort_indicator_section();
}
inline void HeaderViewState::set_sort_indicator_section(::int32_t value) {
  _internal_set_sort_indicator_section(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:mixxx.library.HeaderViewState.sort_indicator_section)
}
inline ::int32_t HeaderViewState::_internal_sort_indicator_section() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sort_indicator_section_;
}
inline void HeaderViewState::_internal_set_sort_indicator_section(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sort_indicator_section_ = value;
}

// optional int32 sort_order = 4;
inline bool HeaderViewState::has_sort_order() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000008U);
  return value;
}
inline void HeaderViewState::clear_sort_order() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sort_order_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline ::int32_t HeaderViewState::sort_order() const {
  // @@protoc_insertion_point(field_get:mixxx.library.HeaderViewState.sort_order)
  return _internal_sort_order();
}
inline void HeaderViewState::set_sort_order(::int32_t value) {
  _internal_set_sort_order(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:mixxx.library.HeaderViewState.sort_order)
}
inline ::int32_t HeaderViewState::_internal_sort_order() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sort_order_;
}
inline void HeaderViewState::_internal_set_sort_order(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sort_order_ = value;
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace library
}  // namespace mixxx


// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"
// clang-format on

#endif  // headers_2eproto_2epb_2eh
