/********************************************************************************
** Form generated from reading UI file 'dlgrecording.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLGRECORDING_H
#define UI_DLGRECORDING_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DlgRecording
{
public:
    QVBoxLayout *verticalLayout;
    QWidget *LibraryFeatureControls;
    QHBoxLayout *horizontalLayout;
    QPushButton *pushButtonRecording;
    QLabel *labelRecPrefix;
    QLabel *labelRecFilename;
    QLabel *labelRecStatistics;
    QSpacerItem *horizontalSpacer;
    QTableView *m_pTrackTablePlaceholder;

    void setupUi(QWidget *DlgRecording)
    {
        if (DlgRecording->objectName().isEmpty())
            DlgRecording->setObjectName("DlgRecording");
        DlgRecording->resize(582, 399);
        verticalLayout = new QVBoxLayout(DlgRecording);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        LibraryFeatureControls = new QWidget(DlgRecording);
        LibraryFeatureControls->setObjectName("LibraryFeatureControls");
        horizontalLayout = new QHBoxLayout(LibraryFeatureControls);
        horizontalLayout->setSpacing(0);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        pushButtonRecording = new QPushButton(LibraryFeatureControls);
        pushButtonRecording->setObjectName("pushButtonRecording");
        pushButtonRecording->setFocusPolicy(Qt::NoFocus);
        pushButtonRecording->setCheckable(true);

        horizontalLayout->addWidget(pushButtonRecording);

        labelRecPrefix = new QLabel(LibraryFeatureControls);
        labelRecPrefix->setObjectName("labelRecPrefix");

        horizontalLayout->addWidget(labelRecPrefix);

        labelRecFilename = new QLabel(LibraryFeatureControls);
        labelRecFilename->setObjectName("labelRecFilename");

        horizontalLayout->addWidget(labelRecFilename);

        labelRecStatistics = new QLabel(LibraryFeatureControls);
        labelRecStatistics->setObjectName("labelRecStatistics");

        horizontalLayout->addWidget(labelRecStatistics);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout->addWidget(LibraryFeatureControls);

        m_pTrackTablePlaceholder = new QTableView(DlgRecording);
        m_pTrackTablePlaceholder->setObjectName("m_pTrackTablePlaceholder");
        m_pTrackTablePlaceholder->setShowGrid(true);

        verticalLayout->addWidget(m_pTrackTablePlaceholder);


        retranslateUi(DlgRecording);

        QMetaObject::connectSlotsByName(DlgRecording);
    } // setupUi

    void retranslateUi(QWidget *DlgRecording)
    {
        DlgRecording->setWindowTitle(QCoreApplication::translate("DlgRecording", "Recordings", nullptr));
        pushButtonRecording->setText(QCoreApplication::translate("DlgRecording", "Start Recording", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgRecording: public Ui_DlgRecording {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLGRECORDING_H
