/********************************************************************************
** Form generated from reading UI file 'dlgtagfetcher.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLGTAGFETCHER_H
#define UI_DLGTAGFETCHER_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DlgTagFetcher
{
public:
    QVBoxLayout *verticalLayout_3;
    QStackedWidget *stack;
    QWidget *submit_page;
    QVBoxLayout *verticalLayout_5;
    QHBoxLayout *horizontalLayout_2;
    QLineEdit *apikey;
    QPushButton *btnApikey;
    QSpacerItem *horizontalSpacer_3;
    QSpacerItem *horizontalSpacer_2;
    QPushButton *btnSubmit;
    QTreeWidget *submit_tree;
    QWidget *results_page;
    QGridLayout *gridLayout;
    QLabel *labelSelect;
    QTreeWidget *tags;
    QVBoxLayout *coverFetcherLayout;
    QLabel *currentCoverLabel;
    QWidget *currentCover;
    QVBoxLayout *currentCoverArtLayout;
    QLabel *fetchedCoverLabel;
    QWidget *fetchedCover;
    QVBoxLayout *fetchedCoverArtLayout;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *progressRetryLayout;
    QVBoxLayout *progressMessageLayout;
    QProgressBar *loadingProgressBar;
    QLabel *statusMessage;
    QVBoxLayout *verticalLayout_2;
    QSpacerItem *horizontalSpacer;
    QPushButton *btnRetry;
    QHBoxLayout *btnsLowerLayout;
    QPushButton *btnPrev;
    QPushButton *btnNext;
    QSpacerItem *btnSpacer;
    QCheckBox *checkBoxTags;
    QCheckBox *checkBoxCover;
    QPushButton *btnApply;
    QPushButton *btnQuit;

    void setupUi(QDialog *DlgTagFetcher)
    {
        if (DlgTagFetcher->objectName().isEmpty())
            DlgTagFetcher->setObjectName("DlgTagFetcher");
        DlgTagFetcher->resize(810, 393);
        verticalLayout_3 = new QVBoxLayout(DlgTagFetcher);
        verticalLayout_3->setObjectName("verticalLayout_3");
        stack = new QStackedWidget(DlgTagFetcher);
        stack->setObjectName("stack");
        submit_page = new QWidget();
        submit_page->setObjectName("submit_page");
        verticalLayout_5 = new QVBoxLayout(submit_page);
        verticalLayout_5->setObjectName("verticalLayout_5");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        apikey = new QLineEdit(submit_page);
        apikey->setObjectName("apikey");

        horizontalLayout_2->addWidget(apikey);

        btnApikey = new QPushButton(submit_page);
        btnApikey->setObjectName("btnApikey");

        horizontalLayout_2->addWidget(btnApikey);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_3);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);

        btnSubmit = new QPushButton(submit_page);
        btnSubmit->setObjectName("btnSubmit");

        horizontalLayout_2->addWidget(btnSubmit);


        verticalLayout_5->addLayout(horizontalLayout_2);

        submit_tree = new QTreeWidget(submit_page);
        new QTreeWidgetItem(submit_tree);
        submit_tree->setObjectName("submit_tree");

        verticalLayout_5->addWidget(submit_tree);

        stack->addWidget(submit_page);
        results_page = new QWidget();
        results_page->setObjectName("results_page");
        results_page->setLayoutDirection(Qt::LeftToRight);
        gridLayout = new QGridLayout(results_page);
        gridLayout->setObjectName("gridLayout");
        labelSelect = new QLabel(results_page);
        labelSelect->setObjectName("labelSelect");
        labelSelect->setStyleSheet(QString::fromUtf8("QLabel { font-weight: bold; }"));

        gridLayout->addWidget(labelSelect, 0, 0, 1, 1);

        tags = new QTreeWidget(results_page);
        tags->setObjectName("tags");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(1);
        sizePolicy.setHeightForWidth(tags->sizePolicy().hasHeightForWidth());
        tags->setSizePolicy(sizePolicy);
        tags->setEditTriggers(QAbstractItemView::DoubleClicked|QAbstractItemView::EditKeyPressed);
        tags->setRootIsDecorated(false);
        tags->header()->setMinimumSectionSize(50);
        tags->header()->setDefaultSectionSize(150);
        tags->header()->setStretchLastSection(true);

        gridLayout->addWidget(tags, 0, 0, 1, 1);

        coverFetcherLayout = new QVBoxLayout();
        coverFetcherLayout->setObjectName("coverFetcherLayout");
        currentCoverLabel = new QLabel(results_page);
        currentCoverLabel->setObjectName("currentCoverLabel");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Maximum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(currentCoverLabel->sizePolicy().hasHeightForWidth());
        currentCoverLabel->setSizePolicy(sizePolicy1);
        currentCoverLabel->setAlignment(Qt::AlignCenter);
        currentCoverLabel->setStyleSheet(QString::fromUtf8("QLabel { font-weight: bold; }"));

        coverFetcherLayout->addWidget(currentCoverLabel);

        currentCover = new QWidget(results_page);
        currentCover->setObjectName("currentCover");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(250);
        sizePolicy2.setVerticalStretch(250);
        sizePolicy2.setHeightForWidth(currentCover->sizePolicy().hasHeightForWidth());
        currentCover->setSizePolicy(sizePolicy2);
        currentCoverArtLayout = new QVBoxLayout(currentCover);
        currentCoverArtLayout->setSpacing(0);
        currentCoverArtLayout->setObjectName("currentCoverArtLayout");
        currentCoverArtLayout->setAlignment(Qt::AlignCenter);
        currentCoverArtLayout->setSizeConstraint(QLayout::SetDefaultConstraint);
        currentCoverArtLayout->setContentsMargins(0, 0, 0, 0);

        coverFetcherLayout->addWidget(currentCover);

        fetchedCoverLabel = new QLabel(results_page);
        fetchedCoverLabel->setObjectName("fetchedCoverLabel");
        sizePolicy1.setHeightForWidth(fetchedCoverLabel->sizePolicy().hasHeightForWidth());
        fetchedCoverLabel->setSizePolicy(sizePolicy1);
        fetchedCoverLabel->setAlignment(Qt::AlignCenter);
        fetchedCoverLabel->setStyleSheet(QString::fromUtf8("QLabel { font-weight: bold; }"));

        coverFetcherLayout->addWidget(fetchedCoverLabel);

        fetchedCover = new QWidget(results_page);
        fetchedCover->setObjectName("fetchedCover");
        sizePolicy2.setHeightForWidth(fetchedCover->sizePolicy().hasHeightForWidth());
        fetchedCover->setSizePolicy(sizePolicy2);
        fetchedCoverArtLayout = new QVBoxLayout(fetchedCover);
        fetchedCoverArtLayout->setSpacing(0);
        fetchedCoverArtLayout->setObjectName("fetchedCoverArtLayout");
        fetchedCoverArtLayout->setAlignment(Qt::AlignCenter);
        fetchedCoverArtLayout->setSizeConstraint(QLayout::SetDefaultConstraint);
        fetchedCoverArtLayout->setContentsMargins(0, 0, 0, 0);

        coverFetcherLayout->addWidget(fetchedCover);

        verticalSpacer = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        coverFetcherLayout->addItem(verticalSpacer);


        gridLayout->addLayout(coverFetcherLayout, 0, 1, 1, 1);

        stack->addWidget(results_page);

        verticalLayout_3->addWidget(stack);

        progressRetryLayout = new QHBoxLayout();
        progressRetryLayout->setObjectName("progressRetryLayout");
        progressMessageLayout = new QVBoxLayout();
        progressMessageLayout->setObjectName("progressMessageLayout");
        progressMessageLayout->setSizeConstraint(QLayout::SetMaximumSize);
        loadingProgressBar = new QProgressBar(DlgTagFetcher);
        loadingProgressBar->setObjectName("loadingProgressBar");
        loadingProgressBar->setValue(24);

        progressMessageLayout->addWidget(loadingProgressBar);

        statusMessage = new QLabel(DlgTagFetcher);
        statusMessage->setObjectName("statusMessage");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Preferred);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(statusMessage->sizePolicy().hasHeightForWidth());
        statusMessage->setSizePolicy(sizePolicy3);
        statusMessage->setAlignment(Qt::AlignCenter);
        statusMessage->setLayoutDirection(Qt::LeftToRight);

        progressMessageLayout->addWidget(statusMessage);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        horizontalSpacer = new QSpacerItem(20000, 0, QSizePolicy::Policy::Maximum, QSizePolicy::Policy::Minimum);

        verticalLayout_2->addItem(horizontalSpacer);


        progressMessageLayout->addLayout(verticalLayout_2);


        progressRetryLayout->addLayout(progressMessageLayout);

        btnRetry = new QPushButton(DlgTagFetcher);
        btnRetry->setObjectName("btnRetry");

        progressRetryLayout->addWidget(btnRetry);


        verticalLayout_3->addLayout(progressRetryLayout);

        btnsLowerLayout = new QHBoxLayout();
        btnsLowerLayout->setObjectName("btnsLowerLayout");
        btnPrev = new QPushButton(DlgTagFetcher);
        btnPrev->setObjectName("btnPrev");

        btnsLowerLayout->addWidget(btnPrev);

        btnNext = new QPushButton(DlgTagFetcher);
        btnNext->setObjectName("btnNext");

        btnsLowerLayout->addWidget(btnNext);

        btnSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        btnsLowerLayout->addItem(btnSpacer);

        checkBoxTags = new QCheckBox(DlgTagFetcher);
        checkBoxTags->setObjectName("checkBoxTags");

        btnsLowerLayout->addWidget(checkBoxTags);

        checkBoxCover = new QCheckBox(DlgTagFetcher);
        checkBoxCover->setObjectName("checkBoxCover");

        btnsLowerLayout->addWidget(checkBoxCover);

        btnApply = new QPushButton(DlgTagFetcher);
        btnApply->setObjectName("btnApply");

        btnsLowerLayout->addWidget(btnApply);

        btnQuit = new QPushButton(DlgTagFetcher);
        btnQuit->setObjectName("btnQuit");

        btnsLowerLayout->addWidget(btnQuit);


        verticalLayout_3->addLayout(btnsLowerLayout);

        QWidget::setTabOrder(tags, apikey);
        QWidget::setTabOrder(apikey, btnApikey);
        QWidget::setTabOrder(btnApikey, btnSubmit);
        QWidget::setTabOrder(btnSubmit, submit_tree);
        QWidget::setTabOrder(submit_tree, btnPrev);
        QWidget::setTabOrder(btnPrev, btnNext);
        QWidget::setTabOrder(btnNext, checkBoxTags);
        QWidget::setTabOrder(checkBoxTags, checkBoxCover);
        QWidget::setTabOrder(checkBoxCover, btnApply);
        QWidget::setTabOrder(btnApply, btnQuit);
        QWidget::setTabOrder(btnQuit, btnRetry);

        retranslateUi(DlgTagFetcher);

        stack->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(DlgTagFetcher);
    } // setupUi

    void retranslateUi(QDialog *DlgTagFetcher)
    {
        DlgTagFetcher->setWindowTitle(QCoreApplication::translate("DlgTagFetcher", "MusicBrainz", nullptr));
        apikey->setText(QString());
        btnApikey->setText(QCoreApplication::translate("DlgTagFetcher", "Get API-Key", "To be able to submit audio fingerprints to the MusicBrainz database, a free application programming interface key (API key) is required."));
        btnSubmit->setText(QCoreApplication::translate("DlgTagFetcher", "Submit", "Submits audio fingerprints to the MusicBrainz database."));
        QTreeWidgetItem *___qtreewidgetitem = submit_tree->headerItem();
        ___qtreewidgetitem->setText(4, QCoreApplication::translate("DlgTagFetcher", "Album", nullptr));
        ___qtreewidgetitem->setText(3, QCoreApplication::translate("DlgTagFetcher", "Artist", nullptr));
        ___qtreewidgetitem->setText(2, QCoreApplication::translate("DlgTagFetcher", "New Column", nullptr));
        ___qtreewidgetitem->setText(1, QCoreApplication::translate("DlgTagFetcher", "Year", nullptr));
        ___qtreewidgetitem->setText(0, QCoreApplication::translate("DlgTagFetcher", "Track", nullptr));

        const bool __sortingEnabled = submit_tree->isSortingEnabled();
        submit_tree->setSortingEnabled(false);
        QTreeWidgetItem *___qtreewidgetitem1 = submit_tree->topLevelItem(0);
        ___qtreewidgetitem1->setText(0, QCoreApplication::translate("DlgTagFetcher", "New Item", nullptr));
        submit_tree->setSortingEnabled(__sortingEnabled);

        labelSelect->setText(QCoreApplication::translate("DlgTagFetcher", "Select best possible match", nullptr));
        QTreeWidgetItem *___qtreewidgetitem2 = tags->headerItem();
        ___qtreewidgetitem2->setText(5, QCoreApplication::translate("DlgTagFetcher", "Album Artist", nullptr));
        ___qtreewidgetitem2->setText(4, QCoreApplication::translate("DlgTagFetcher", "Track", nullptr));
        ___qtreewidgetitem2->setText(3, QCoreApplication::translate("DlgTagFetcher", "Year", nullptr));
        ___qtreewidgetitem2->setText(2, QCoreApplication::translate("DlgTagFetcher", "Album", nullptr));
        ___qtreewidgetitem2->setText(1, QCoreApplication::translate("DlgTagFetcher", "Artist", nullptr));
        ___qtreewidgetitem2->setText(0, QCoreApplication::translate("DlgTagFetcher", "Title", nullptr));
        currentCoverLabel->setText(QCoreApplication::translate("DlgTagFetcher", "Current Cover Art", nullptr));
        fetchedCoverLabel->setText(QCoreApplication::translate("DlgTagFetcher", "Found Cover Art", nullptr));
        statusMessage->setText(QCoreApplication::translate("DlgTagFetcher", "The results are ready to be applied.", nullptr));
        btnRetry->setText(QCoreApplication::translate("DlgTagFetcher", "Retry", nullptr));
        btnPrev->setText(QCoreApplication::translate("DlgTagFetcher", "&Previous", nullptr));
        btnNext->setText(QCoreApplication::translate("DlgTagFetcher", "&Next", nullptr));
        checkBoxTags->setText(QCoreApplication::translate("DlgTagFetcher", "Tags", nullptr));
        checkBoxCover->setText(QCoreApplication::translate("DlgTagFetcher", "Cover", nullptr));
        btnApply->setText(QCoreApplication::translate("DlgTagFetcher", "&Apply", nullptr));
        btnQuit->setText(QCoreApplication::translate("DlgTagFetcher", "&Close", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgTagFetcher: public Ui_DlgTagFetcher {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLGTAGFETCHER_H
