/****************************************************************************
** Meta object code from reading C++ file 'soundmanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/soundio/soundmanager.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'soundmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12SoundManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto SoundManager::qt_create_metaobjectdata<qt_meta_tag_ZN12SoundManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SoundManager",
        "devicesUpdated",
        "",
        "devicesSetup",
        "outputRegistered",
        "AudioOutput",
        "output",
        "AudioSource*",
        "src",
        "inputRegistered",
        "AudioInput",
        "input",
        "AudioDestination*",
        "dest"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'devicesUpdated'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'devicesSetup'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'outputRegistered'
        QtMocHelpers::SignalData<void(const AudioOutput &, AudioSource *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 7, 8 },
        }}),
        // Signal 'inputRegistered'
        QtMocHelpers::SignalData<void(const AudioInput &, AudioDestination *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 }, { 0x80000000 | 12, 13 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SoundManager, qt_meta_tag_ZN12SoundManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SoundManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12SoundManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12SoundManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12SoundManagerE_t>.metaTypes,
    nullptr
} };

void SoundManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SoundManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->devicesUpdated(); break;
        case 1: _t->devicesSetup(); break;
        case 2: _t->outputRegistered((*reinterpret_cast<std::add_pointer_t<AudioOutput>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<AudioSource*>>(_a[2]))); break;
        case 3: _t->inputRegistered((*reinterpret_cast<std::add_pointer_t<AudioInput>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<AudioDestination*>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SoundManager::*)()>(_a, &SoundManager::devicesUpdated, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SoundManager::*)()>(_a, &SoundManager::devicesSetup, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (SoundManager::*)(const AudioOutput & , AudioSource * )>(_a, &SoundManager::outputRegistered, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (SoundManager::*)(const AudioInput & , AudioDestination * )>(_a, &SoundManager::inputRegistered, 3))
            return;
    }
}

const QMetaObject *SoundManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SoundManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12SoundManagerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int SoundManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void SoundManager::devicesUpdated()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void SoundManager::devicesSetup()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void SoundManager::outputRegistered(const AudioOutput & _t1, AudioSource * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void SoundManager::inputRegistered(const AudioInput & _t1, AudioDestination * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}
QT_WARNING_POP
