/****************************************************************************
** Meta object code from reading C++ file 'dlgrecording.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/library/recording/dlgrecording.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dlgrecording.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12DlgRecordingE_t {};
} // unnamed namespace

template <> constexpr inline auto DlgRecording::qt_create_metaobjectdata<qt_meta_tag_ZN12DlgRecordingE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DlgRecording",
        "loadTrack",
        "",
        "TrackPointer",
        "tio",
        "loadTrackToPlayer",
        "group",
        "play",
        "restoreSearch",
        "search",
        "restoreModelState",
        "slotRecordingStateChanged",
        "slotBytesRecorded",
        "refreshBrowseModel",
        "slotRestoreSearch",
        "slotDurationRecorded",
        "durationRecorded"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'loadTrack'
        QtMocHelpers::SignalData<void(TrackPointer)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'loadTrackToPlayer'
        QtMocHelpers::SignalData<void(TrackPointer, const QString &, bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 6 }, { QMetaType::Bool, 7 },
        }}),
        // Signal 'restoreSearch'
        QtMocHelpers::SignalData<void(const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Signal 'restoreModelState'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotRecordingStateChanged'
        QtMocHelpers::SlotData<void(bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'slotBytesRecorded'
        QtMocHelpers::SlotData<void(int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'refreshBrowseModel'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotRestoreSearch'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotDurationRecorded'
        QtMocHelpers::SlotData<void(const QString &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DlgRecording, qt_meta_tag_ZN12DlgRecordingE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DlgRecording::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12DlgRecordingE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12DlgRecordingE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12DlgRecordingE_t>.metaTypes,
    nullptr
} };

void DlgRecording::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DlgRecording *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->loadTrack((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1]))); break;
        case 1: _t->loadTrackToPlayer((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 2: _t->restoreSearch((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->restoreModelState(); break;
        case 4: _t->slotRecordingStateChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->slotBytesRecorded((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->refreshBrowseModel(); break;
        case 7: _t->slotRestoreSearch(); break;
        case 8: _t->slotDurationRecorded((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DlgRecording::*)(TrackPointer )>(_a, &DlgRecording::loadTrack, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgRecording::*)(TrackPointer , const QString & , bool )>(_a, &DlgRecording::loadTrackToPlayer, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgRecording::*)(const QString & )>(_a, &DlgRecording::restoreSearch, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgRecording::*)()>(_a, &DlgRecording::restoreModelState, 3))
            return;
    }
}

const QMetaObject *DlgRecording::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DlgRecording::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12DlgRecordingE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::DlgRecording"))
        return static_cast< Ui::DlgRecording*>(this);
    if (!strcmp(_clname, "LibraryView"))
        return static_cast< LibraryView*>(this);
    return QWidget::qt_metacast(_clname);
}

int DlgRecording::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void DlgRecording::loadTrack(TrackPointer _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void DlgRecording::loadTrackToPlayer(TrackPointer _t1, const QString & _t2, bool _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}

// SIGNAL 2
void DlgRecording::restoreSearch(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void DlgRecording::restoreModelState()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
