// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: keys.proto
// Protobuf C++ Version: 7.35.0

#include "keys.pb.h"

#include <algorithm>
#include <type_traits>
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/generated_message_tctable_impl.h"
#include "google/protobuf/internal_visibility.h"
#include "google/protobuf/extension_set.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/wire_format_lite.h"
#include "google/protobuf/io/zero_copy_stream_impl_lite.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"
PROTOBUF_PRAGMA_INIT_SEG
namespace _pb = ::google::protobuf;
namespace _pbi = ::google::protobuf::internal;
namespace _fl = ::google::protobuf::internal::field_layout;
namespace mixxx {
namespace track {
namespace io {
namespace key {
class KeyMap_KeyChange::_Internal {
 public:
  using HasBits = decltype(::std::declval<KeyMap_KeyChange>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(KeyMap_KeyChange, _impl_._has_bits_);
};

constexpr KeyMap_KeyChange::ParseTableT_ KeyMap_KeyChange::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(KeyMap_KeyChange, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallbackLite,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::mixxx::track::io::key::KeyMap_KeyChange>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // optional .mixxx.track.io.key.ChromaticKey key = 2;
      {::_pbi::TcParser::FastEr0S1,
       {16, 1, 24,
        PROTOBUF_FIELD_OFFSET(KeyMap_KeyChange, _impl_.key_)}},
      // optional int32 frame_position = 1;
      {::_pbi::TcParser::FastV32S1,
       {8, 0, 0,
        PROTOBUF_FIELD_OFFSET(KeyMap_KeyChange, _impl_.frame_position_)}},
    }}, {{
      65535, 65535
    }}, {{
      // optional int32 frame_position = 1;
      {PROTOBUF_FIELD_OFFSET(KeyMap_KeyChange, _impl_.frame_position_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // optional .mixxx.track.io.key.ChromaticKey key = 2;
      {PROTOBUF_FIELD_OFFSET(KeyMap_KeyChange, _impl_.key_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kEnumRange)},
    }},
    {{
        {0, 24},
    }},
    {{
    }},
  };
}


inline constexpr KeyMap_KeyChange::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        frame_position_{0},
        key_{static_cast< ::mixxx::track::io::key::ChromaticKey >(0)} {}

template <typename>
constexpr KeyMap_KeyChange::KeyMap_KeyChange(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::MessageLite(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL KeyMap_KeyChange::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) KeyMap_KeyChange(arena);
}
constexpr auto KeyMap_KeyChange::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(KeyMap_KeyChange), alignof(KeyMap_KeyChange));
}
constexpr auto KeyMap_KeyChange::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataLite{
      {
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &KeyMap_KeyChange::MergeImpl,
          ::google::protobuf::MessageLite::GetNewImpl<KeyMap_KeyChange>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &KeyMap_KeyChange::SharedDtor,
          ::google::protobuf::MessageLite::GetClearImpl<KeyMap_KeyChange>(), &KeyMap_KeyChange::ByteSizeLong,
              &KeyMap_KeyChange::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(KeyMap_KeyChange, _impl_._cached_size_),
          true,
      },
      "mixxx.track.io.key.KeyMap.KeyChange",
  };
}
struct KeyMap_KeyChangeGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr KeyMap_KeyChangeGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 KeyMap_KeyChange_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(KeyMap_KeyChange::InternalGenerateClassData_(
            _default, &KeyMap_KeyChange_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<KeyMap_KeyChange>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~KeyMap_KeyChangeGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) KeyMap_KeyChange _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<KeyMap_KeyChange>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(KeyMap_KeyChangeGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST KeyMap_KeyChangeGlobalsTypeInternal KeyMap_KeyChange_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* KeyMap_KeyChange_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return KeyMap_KeyChange_globals_.GetClassData();
#else
  return KeyMap_KeyChange_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class KeyMap::_Internal {
 public:
  using HasBits = decltype(::std::declval<KeyMap>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(KeyMap, _impl_._has_bits_);
};

constexpr KeyMap::ParseTableT_ KeyMap::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(KeyMap, _impl_._has_bits_),
      0, // no _extensions_
      4, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967280,  // skipmap
      offsetof(ParseTableT_, field_entries),
      4,  // num_field_entries
      3,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallbackLite,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::mixxx::track::io::key::KeyMap>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // optional .mixxx.track.io.key.Source source = 4 [default = ANALYZER];
      {::_pbi::TcParser::FastEr0S1,
       {32, 3, 2,
        PROTOBUF_FIELD_OFFSET(KeyMap, _impl_.source_)}},
      // optional .mixxx.track.io.key.ChromaticKey global_key = 1 [default = INVALID];
      {::_pbi::TcParser::FastEr0S1,
       {8, 2, 24,
        PROTOBUF_FIELD_OFFSET(KeyMap, _impl_.global_key_)}},
      // optional string global_key_text = 2;
      {::_pbi::TcParser::FastBS1,
       {18, 1, 0,
        PROTOBUF_FIELD_OFFSET(KeyMap, _impl_.global_key_text_)}},
      // repeated .mixxx.track.io.key.KeyMap.KeyChange key_change = 3;
      {::_pbi::TcParser::FastMtR1,
       {26, 0, 0,
        PROTOBUF_FIELD_OFFSET(KeyMap, _impl_.key_change_)}},
    }}, {{
      65535, 65535
    }}, {{
      // optional .mixxx.track.io.key.ChromaticKey global_key = 1 [default = INVALID];
      {PROTOBUF_FIELD_OFFSET(KeyMap, _impl_.global_key_), _Internal::kHasBitsOffset + 2, 1, (0 | ::_fl::kFcOptional | ::_fl::kEnumRange)},
      // optional string global_key_text = 2;
      {PROTOBUF_FIELD_OFFSET(KeyMap, _impl_.global_key_text_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kBytes | ::_fl::kRepAString)},
      // repeated .mixxx.track.io.key.KeyMap.KeyChange key_change = 3;
      {PROTOBUF_FIELD_OFFSET(KeyMap, _impl_.key_change_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kMessage | ::_fl::kTvTable)},
      // optional .mixxx.track.io.key.Source source = 4 [default = ANALYZER];
      {PROTOBUF_FIELD_OFFSET(KeyMap, _impl_.source_), _Internal::kHasBitsOffset + 3, 2, (0 | ::_fl::kFcOptional | ::_fl::kEnumRange)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::mixxx::track::io::key::KeyMap_KeyChange>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::mixxx::track::io::key::KeyMap_KeyChange_globals_},
        #endif
        {0, 24},
        {0, 2},
    }},
    {{
    }},
  };
}


inline constexpr KeyMap::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        key_change_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::mixxx::track::io::key::KeyMap,
            PROTOBUF_FIELD_OFFSET(::mixxx::track::io::key::KeyMap, _impl_.key_change_)>()
         }
        ,
        global_key_text_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        global_key_{static_cast< ::mixxx::track::io::key::ChromaticKey >(0)},
        source_{static_cast< ::mixxx::track::io::key::Source >(0)} {}

template <typename>
constexpr KeyMap::KeyMap(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::MessageLite(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL KeyMap::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) KeyMap(arena);
}
constexpr auto KeyMap::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(KeyMap), alignof(KeyMap));
}
constexpr auto KeyMap::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataLite{
      {
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &KeyMap::MergeImpl,
          ::google::protobuf::MessageLite::GetNewImpl<KeyMap>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &KeyMap::SharedDtor,
          ::google::protobuf::MessageLite::GetClearImpl<KeyMap>(), &KeyMap::ByteSizeLong,
              &KeyMap::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(KeyMap, _impl_._cached_size_),
          true,
      },
      "mixxx.track.io.key.KeyMap",
  };
}
struct KeyMapGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr KeyMapGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 KeyMap_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(KeyMap::InternalGenerateClassData_(
            _default, &KeyMap_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<KeyMap>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~KeyMapGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) KeyMap _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<KeyMap>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(KeyMapGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST KeyMapGlobalsTypeInternal KeyMap_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* KeyMap_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return KeyMap_globals_.GetClassData();
#else
  return KeyMap_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
}  // namespace key
}  // namespace io
}  // namespace track
}  // namespace mixxx
namespace mixxx {
namespace track {
namespace io {
namespace key {
PROTOBUF_CONSTINIT const uint32_t Source_internal_data_[] = {
    196608u, 0u, };
static const char Source_names[] = {
    "ANALYZER"
    "FILE_METADATA"
    "USER"
};

static const ::google::protobuf::internal::EnumEntry Source_entries[] = {
    {{&Source_names[0], 8}, 0},
    {{&Source_names[8], 13}, 1},
    {{&Source_names[21], 4}, 2},
};

static const int Source_entries_by_number[] = {
    0,  // 0 -> ANALYZER
    1,  // 1 -> FILE_METADATA
    2,  // 2 -> USER
};

[[nodiscard]] bool Source_Parse(::absl::string_view name,
                                  Source* PROTOBUF_NONNULL value) {
  int int_value;
  bool success = ::google::protobuf::internal::LookUpEnumValue(
      Source_entries, 3, name, &int_value);
  if (success) {
    *value = static_cast<Source>(int_value);
  }
  return success;
}
static ::google::protobuf::internal::ExplicitlyConstructed<::std::string>
    Source_strings[3] = {};

[[nodiscard]] const ::std::string& Source_Name(Source value) {
  static const bool kDummy = ::google::protobuf::internal::InitializeEnumStrings(
      Source_entries, Source_entries_by_number, 3,
      Source_strings);
  (void)kDummy;

  int idx = ::google::protobuf::internal::LookUpEnumName(Source_entries,
                                  Source_entries_by_number,
                                  3, value);
  return idx == -1 ? ::google::protobuf::internal::GetEmptyString() : Source_strings[idx].get();
}
PROTOBUF_CONSTINIT const uint32_t ChromaticKey_internal_data_[] = {
    1638400u, 0u, };
static const char ChromaticKey_names[] = {
    "A_FLAT_MAJOR"
    "A_MAJOR"
    "A_MINOR"
    "B_FLAT_MAJOR"
    "B_FLAT_MINOR"
    "B_MAJOR"
    "B_MINOR"
    "C_MAJOR"
    "C_MINOR"
    "C_SHARP_MINOR"
    "D_FLAT_MAJOR"
    "D_MAJOR"
    "D_MINOR"
    "E_FLAT_MAJOR"
    "E_FLAT_MINOR"
    "E_MAJOR"
    "E_MINOR"
    "F_MAJOR"
    "F_MINOR"
    "F_SHARP_MAJOR"
    "F_SHARP_MINOR"
    "G_MAJOR"
    "G_MINOR"
    "G_SHARP_MINOR"
    "INVALID"
};

static const ::google::protobuf::internal::EnumEntry ChromaticKey_entries[] = {
    {{&ChromaticKey_names[0], 12}, 9},
    {{&ChromaticKey_names[12], 7}, 10},
    {{&ChromaticKey_names[19], 7}, 22},
    {{&ChromaticKey_names[26], 12}, 11},
    {{&ChromaticKey_names[38], 12}, 23},
    {{&ChromaticKey_names[50], 7}, 12},
    {{&ChromaticKey_names[57], 7}, 24},
    {{&ChromaticKey_names[64], 7}, 1},
    {{&ChromaticKey_names[71], 7}, 13},
    {{&ChromaticKey_names[78], 13}, 14},
    {{&ChromaticKey_names[91], 12}, 2},
    {{&ChromaticKey_names[103], 7}, 3},
    {{&ChromaticKey_names[110], 7}, 15},
    {{&ChromaticKey_names[117], 12}, 4},
    {{&ChromaticKey_names[129], 12}, 16},
    {{&ChromaticKey_names[141], 7}, 5},
    {{&ChromaticKey_names[148], 7}, 17},
    {{&ChromaticKey_names[155], 7}, 6},
    {{&ChromaticKey_names[162], 7}, 18},
    {{&ChromaticKey_names[169], 13}, 7},
    {{&ChromaticKey_names[182], 13}, 19},
    {{&ChromaticKey_names[195], 7}, 8},
    {{&ChromaticKey_names[202], 7}, 20},
    {{&ChromaticKey_names[209], 13}, 21},
    {{&ChromaticKey_names[222], 7}, 0},
};

static const int ChromaticKey_entries_by_number[] = {
    24,  // 0 -> INVALID
    7,  // 1 -> C_MAJOR
    10,  // 2 -> D_FLAT_MAJOR
    11,  // 3 -> D_MAJOR
    13,  // 4 -> E_FLAT_MAJOR
    15,  // 5 -> E_MAJOR
    17,  // 6 -> F_MAJOR
    19,  // 7 -> F_SHARP_MAJOR
    21,  // 8 -> G_MAJOR
    0,  // 9 -> A_FLAT_MAJOR
    1,  // 10 -> A_MAJOR
    3,  // 11 -> B_FLAT_MAJOR
    5,  // 12 -> B_MAJOR
    8,  // 13 -> C_MINOR
    9,  // 14 -> C_SHARP_MINOR
    12,  // 15 -> D_MINOR
    14,  // 16 -> E_FLAT_MINOR
    16,  // 17 -> E_MINOR
    18,  // 18 -> F_MINOR
    20,  // 19 -> F_SHARP_MINOR
    22,  // 20 -> G_MINOR
    23,  // 21 -> G_SHARP_MINOR
    2,  // 22 -> A_MINOR
    4,  // 23 -> B_FLAT_MINOR
    6,  // 24 -> B_MINOR
};

[[nodiscard]] bool ChromaticKey_Parse(::absl::string_view name,
                                  ChromaticKey* PROTOBUF_NONNULL value) {
  int int_value;
  bool success = ::google::protobuf::internal::LookUpEnumValue(
      ChromaticKey_entries, 25, name, &int_value);
  if (success) {
    *value = static_cast<ChromaticKey>(int_value);
  }
  return success;
}
static ::google::protobuf::internal::ExplicitlyConstructed<::std::string>
    ChromaticKey_strings[25] = {};

[[nodiscard]] const ::std::string& ChromaticKey_Name(ChromaticKey value) {
  static const bool kDummy = ::google::protobuf::internal::InitializeEnumStrings(
      ChromaticKey_entries, ChromaticKey_entries_by_number, 25,
      ChromaticKey_strings);
  (void)kDummy;

  int idx = ::google::protobuf::internal::LookUpEnumName(ChromaticKey_entries,
                                  ChromaticKey_entries_by_number,
                                  25, value);
  return idx == -1 ? ::google::protobuf::internal::GetEmptyString() : ChromaticKey_strings[idx].get();
}
// ===================================================================

KeyMap_KeyChange::KeyMap_KeyChange(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::MessageLite(arena, KeyMap_KeyChange_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::MessageLite(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:mixxx.track.io.key.KeyMap.KeyChange)
}
KeyMap_KeyChange::KeyMap_KeyChange(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const KeyMap_KeyChange& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::MessageLite(arena, KeyMap_KeyChange_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::MessageLite(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::std::string>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE KeyMap_KeyChange::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void KeyMap_KeyChange::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, frame_position_),
           0,
           offsetof(Impl_, key_) -
               offsetof(Impl_, frame_position_) +
               sizeof(Impl_::key_));
}
KeyMap_KeyChange::~KeyMap_KeyChange() {
  // @@protoc_insertion_point(destructor:mixxx.track.io.key.KeyMap.KeyChange)
  SharedDtor(*this);
}
inline void KeyMap_KeyChange::SharedDtor(MessageLite& self) {
  KeyMap_KeyChange& this_ = static_cast<KeyMap_KeyChange&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::std::string>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1
const ::google::protobuf::internal::ClassDataLite KeyMap_KeyChange_class_data_ =
    KeyMap_KeyChange::InternalGenerateClassData_(KeyMap_KeyChange_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
KeyMap_KeyChange::GetClassData() const {
  return KeyMap_KeyChange_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
KeyMap_KeyChange::GetClassData() const {
  return KeyMap_KeyChange_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const KeyMap_KeyChange::ParseTableT_
    KeyMap_KeyChange::_table_ =
        KeyMap_KeyChange::InternalGenerateParseTable_(KeyMap_KeyChange_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void KeyMap_KeyChange::Clear() {
// @@protoc_insertion_point(message_clear_start:mixxx.track.io.key.KeyMap.KeyChange)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    ::memset(&_impl_.frame_position_, 0, static_cast<::size_t>(
        reinterpret_cast<char*>(&_impl_.key_) -
        reinterpret_cast<char*>(&_impl_.frame_position_)) + sizeof(_impl_.key_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::std::string>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL KeyMap_KeyChange::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const KeyMap_KeyChange& this_ = static_cast<const KeyMap_KeyChange&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL KeyMap_KeyChange::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const KeyMap_KeyChange& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:mixxx.track.io.key.KeyMap.KeyChange)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // optional int32 frame_position = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target =
        ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<1>(
            stream, this_._internal_frame_position(), target);
  }

  // optional .mixxx.track.io.key.ChromaticKey key = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteEnumToArray(
        2, this_._internal_key(), target);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target = stream->WriteRaw(
        this_._internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString).data(),
        static_cast<int>(this_._internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString).size()), target);
  }
  // @@protoc_insertion_point(serialize_to_array_end:mixxx.track.io.key.KeyMap.KeyChange)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t KeyMap_KeyChange::ByteSizeLong(const MessageLite& base) {
  const KeyMap_KeyChange& this_ = static_cast<const KeyMap_KeyChange&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t KeyMap_KeyChange::ByteSizeLong() const {
  const KeyMap_KeyChange& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:mixxx.track.io.key.KeyMap.KeyChange)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // optional int32 frame_position = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
          this_._internal_frame_position());
    }
    // optional .mixxx.track.io.key.ChromaticKey key = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::_pbi::WireFormatLite::EnumSize(this_._internal_key());
    }
  }
  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    total_size += this_._internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString).size();
  }
  this_._impl_._cached_size_.Set(::_pbi::ToCachedSize(total_size));
  return total_size;
}

void KeyMap_KeyChange::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<KeyMap_KeyChange*>(&to_msg);
  auto& from = static_cast<const KeyMap_KeyChange&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:mixxx.track.io.key.KeyMap.KeyChange)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_impl_.frame_position_ = from._impl_.frame_position_;
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _this->_impl_.key_ = from._impl_.key_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::std::string>(
      from._internal_metadata_);
}

void KeyMap_KeyChange::CopyFrom(const KeyMap_KeyChange& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:mixxx.track.io.key.KeyMap.KeyChange)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void KeyMap_KeyChange::InternalSwap(KeyMap_KeyChange* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(KeyMap_KeyChange, _impl_.key_)
      + sizeof(KeyMap_KeyChange::_impl_.key_)
      - PROTOBUF_FIELD_OFFSET(KeyMap_KeyChange, _impl_.frame_position_)>(
          reinterpret_cast<char*>(&_impl_.frame_position_),
          reinterpret_cast<char*>(&other->_impl_.frame_position_));
}

// ===================================================================

KeyMap::KeyMap(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::MessageLite(arena, KeyMap_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::MessageLite(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:mixxx.track.io.key.KeyMap)
}
PROTOBUF_NDEBUG_INLINE KeyMap::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::mixxx::track::io::key::KeyMap& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        key_change_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::mixxx::track::io::key::KeyMap,
              PROTOBUF_FIELD_OFFSET(::mixxx::track::io::key::KeyMap, _impl_.key_change_)>()
          , from.key_change_
        }
        ,
        global_key_text_(arena, from.global_key_text_) {}

KeyMap::KeyMap(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const KeyMap& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::MessageLite(arena, KeyMap_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::MessageLite(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  KeyMap* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::std::string>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, global_key_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, global_key_),
           offsetof(Impl_, source_) -
               offsetof(Impl_, global_key_) +
               sizeof(Impl_::source_));

  // @@protoc_insertion_point(copy_constructor:mixxx.track.io.key.KeyMap)
}
PROTOBUF_NDEBUG_INLINE KeyMap::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        key_change_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::mixxx::track::io::key::KeyMap,
            PROTOBUF_FIELD_OFFSET(::mixxx::track::io::key::KeyMap, _impl_.key_change_)>()
         }
        ,
        global_key_text_(arena) {}

inline void KeyMap::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, global_key_),
           0,
           offsetof(Impl_, source_) -
               offsetof(Impl_, global_key_) +
               sizeof(Impl_::source_));
}
KeyMap::~KeyMap() {
  // @@protoc_insertion_point(destructor:mixxx.track.io.key.KeyMap)
  SharedDtor(*this);
}
inline void KeyMap::SharedDtor(MessageLite& self) {
  KeyMap& this_ = static_cast<KeyMap&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::std::string>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.global_key_text_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1
const ::google::protobuf::internal::ClassDataLite KeyMap_class_data_ =
    KeyMap::InternalGenerateClassData_(KeyMap_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
KeyMap::GetClassData() const {
  return KeyMap_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
KeyMap::GetClassData() const {
  return KeyMap_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const KeyMap::ParseTableT_
    KeyMap::_table_ =
        KeyMap::InternalGenerateParseTable_(KeyMap_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void KeyMap::Clear() {
// @@protoc_insertion_point(message_clear_start:mixxx.track.io.key.KeyMap)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.key_change_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.global_key_text_.ClearNonDefaultToEmpty();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x0000000cU)) {
    ::memset(&_impl_.global_key_, 0, static_cast<::size_t>(
        reinterpret_cast<char*>(&_impl_.source_) -
        reinterpret_cast<char*>(&_impl_.global_key_)) + sizeof(_impl_.source_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::std::string>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL KeyMap::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const KeyMap& this_ = static_cast<const KeyMap&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL KeyMap::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const KeyMap& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:mixxx.track.io.key.KeyMap)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // optional .mixxx.track.io.key.ChromaticKey global_key = 1 [default = INVALID];
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteEnumToArray(
        1, this_._internal_global_key(), target);
  }

  // optional string global_key_text = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    const ::std::string& _s = this_._internal_global_key_text();
    target = stream->WriteStringMaybeAliased(2, _s, target);
  }

  // repeated .mixxx.track.io.key.KeyMap.KeyChange key_change = 3;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (unsigned i = 0, n = static_cast<unsigned>(
                             this_._internal_key_change_size());
         i < n; i++) {
      const auto& repfield = this_._internal_key_change().Get(i);
      target =
          ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
              3, repfield, repfield.GetCachedSize(),
              target, stream);
    }
  }

  // optional .mixxx.track.io.key.Source source = 4 [default = ANALYZER];
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteEnumToArray(
        4, this_._internal_source(), target);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target = stream->WriteRaw(
        this_._internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString).data(),
        static_cast<int>(this_._internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString).size()), target);
  }
  // @@protoc_insertion_point(serialize_to_array_end:mixxx.track.io.key.KeyMap)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t KeyMap::ByteSizeLong(const MessageLite& base) {
  const KeyMap& this_ = static_cast<const KeyMap&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t KeyMap::ByteSizeLong() const {
  const KeyMap& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:mixxx.track.io.key.KeyMap)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    // repeated .mixxx.track.io.key.KeyMap.KeyChange key_change = 3;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1UL * this_._internal_key_change_size();
      for (const auto& msg : this_._internal_key_change()) {
        total_size += ::google::protobuf::internal::WireFormatLite::MessageSize(msg);
      }
    }
    // optional string global_key_text = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                      this_._internal_global_key_text());
    }
    // optional .mixxx.track.io.key.ChromaticKey global_key = 1 [default = INVALID];
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += 1 +
                    ::_pbi::WireFormatLite::EnumSize(this_._internal_global_key());
    }
    // optional .mixxx.track.io.key.Source source = 4 [default = ANALYZER];
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      total_size += 1 +
                    ::_pbi::WireFormatLite::EnumSize(this_._internal_source());
    }
  }
  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    total_size += this_._internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString).size();
  }
  this_._impl_._cached_size_.Set(::_pbi::ToCachedSize(total_size));
  return total_size;
}

void KeyMap::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<KeyMap*>(&to_msg);
  auto& from = static_cast<const KeyMap&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:mixxx.track.io.key.KeyMap)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_key_change()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_key_change());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _this->_internal_set_global_key_text(from._internal_global_key_text());
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _this->_impl_.global_key_ = from._impl_.global_key_;
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      _this->_impl_.source_ = from._impl_.source_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::std::string>(
      from._internal_metadata_);
}

void KeyMap::CopyFrom(const KeyMap& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:mixxx.track.io.key.KeyMap)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void KeyMap::InternalSwap(KeyMap* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.key_change_.InternalSwap(&other->_impl_.key_change_);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.global_key_text_, &other->_impl_.global_key_text_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(KeyMap, _impl_.source_)
      + sizeof(KeyMap::_impl_.source_)
      - PROTOBUF_FIELD_OFFSET(KeyMap, _impl_.global_key_)>(
          reinterpret_cast<char*>(&_impl_.global_key_),
          reinterpret_cast<char*>(&other->_impl_.global_key_));
}

// @@protoc_insertion_point(namespace_scope)
}  // namespace key
}  // namespace io
}  // namespace track
}  // namespace mixxx
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google
// @@protoc_insertion_point(global_scope)
#include "google/protobuf/port_undef.inc"
