/****************************************************************************
** Meta object code from reading C++ file 'tagfetcher.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/musicbrainz/tagfetcher.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tagfetcher.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10TagFetcherE_t {};
} // unnamed namespace

template <> constexpr inline auto TagFetcher::qt_create_metaobjectdata<qt_meta_tag_ZN10TagFetcherE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TagFetcher",
        "resultAvailable",
        "",
        "TrackPointer",
        "pTrack",
        "QList<mixxx::musicbrainz::TrackRelease>",
        "guessedTrackReleases",
        "whyEmptyMessage",
        "fetchProgress",
        "message",
        "numberOfRecordingsFoundFromAcoustId",
        "totalNumberOfRecordings",
        "currentRecordingFetchedFromMusicBrainz",
        "networkError",
        "httpStatus",
        "app",
        "code",
        "fetchedCoverUpdate",
        "coverInfo",
        "coverArtImageFetchAvailable",
        "QUuid",
        "albumReleaseId",
        "coverArtBytes",
        "coverArtArchiveLinksAvailable",
        "allUrls",
        "coverArtLinkNotFound",
        "cancel",
        "slotFingerprintReady",
        "slotAcoustIdTaskSucceeded",
        "QList<QUuid>",
        "recordingIds",
        "slotAcoustIdTaskFailed",
        "mixxx::network::JsonWebResponse",
        "response",
        "slotAcoustIdTaskAborted",
        "slotAcoustIdTaskNetworkError",
        "QNetworkReply::NetworkError",
        "errorCode",
        "errorString",
        "mixxx::network::WebResponseWithContent",
        "responseWithContent",
        "slotMusicBrainzTaskSucceeded",
        "slotMusicBrainzTaskFailed",
        "mixxx::network::WebResponse",
        "errorMessage",
        "slotMusicBrainzTaskAborted",
        "slotMusicBrainzTaskNetworkError",
        "slotCoverArtArchiveLinksTaskSucceeded",
        "slotCoverArtArchiveLinksTaskFailed",
        "slotCoverArtArchiveLinksTaskAborted",
        "slotCoverArtArchiveLinksTaskNetworkError",
        "slotCoverArtArchiveImageTaskSucceeded",
        "slotCoverArtArchiveImageTaskFailed",
        "slotCoverArtArchiveImageTaskAborted",
        "slotCoverArtArchiveImageTaskNetworkError"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'resultAvailable'
        QtMocHelpers::SignalData<void(TrackPointer, const QList<mixxx::musicbrainz::TrackRelease> &, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 }, { QMetaType::QString, 7 },
        }}),
        // Signal 'fetchProgress'
        QtMocHelpers::SignalData<void(const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Signal 'numberOfRecordingsFoundFromAcoustId'
        QtMocHelpers::SignalData<void(int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Signal 'currentRecordingFetchedFromMusicBrainz'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'networkError'
        QtMocHelpers::SignalData<void(int, const QString &, const QString &, int)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 14 }, { QMetaType::QString, 15 }, { QMetaType::QString, 9 }, { QMetaType::Int, 16 },
        }}),
        // Signal 'fetchedCoverUpdate'
        QtMocHelpers::SignalData<void(const QByteArray &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 18 },
        }}),
        // Signal 'coverArtImageFetchAvailable'
        QtMocHelpers::SignalData<void(const QUuid &, const QByteArray &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 }, { QMetaType::QByteArray, 22 },
        }}),
        // Signal 'coverArtArchiveLinksAvailable'
        QtMocHelpers::SignalData<void(const QUuid &, const QList<QString> &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 }, { QMetaType::QStringList, 24 },
        }}),
        // Signal 'coverArtLinkNotFound'
        QtMocHelpers::SignalData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'cancel'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFingerprintReady'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotAcoustIdTaskSucceeded'
        QtMocHelpers::SlotData<void(QList<QUuid>)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 29, 30 },
        }}),
        // Slot 'slotAcoustIdTaskFailed'
        QtMocHelpers::SlotData<void(const mixxx::network::JsonWebResponse &)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Slot 'slotAcoustIdTaskAborted'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotAcoustIdTaskNetworkError'
        QtMocHelpers::SlotData<void(QNetworkReply::NetworkError, const QString &, const mixxx::network::WebResponseWithContent &)>(35, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 36, 37 }, { QMetaType::QString, 38 }, { 0x80000000 | 39, 40 },
        }}),
        // Slot 'slotMusicBrainzTaskSucceeded'
        QtMocHelpers::SlotData<void(const QList<mixxx::musicbrainz::TrackRelease> &)>(41, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'slotMusicBrainzTaskFailed'
        QtMocHelpers::SlotData<void(const mixxx::network::WebResponse &, int, const QString &)>(42, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 43, 33 }, { QMetaType::Int, 37 }, { QMetaType::QString, 44 },
        }}),
        // Slot 'slotMusicBrainzTaskAborted'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotMusicBrainzTaskNetworkError'
        QtMocHelpers::SlotData<void(QNetworkReply::NetworkError, const QString &, const mixxx::network::WebResponseWithContent &)>(46, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 36, 37 }, { QMetaType::QString, 38 }, { 0x80000000 | 39, 40 },
        }}),
        // Slot 'slotCoverArtArchiveLinksTaskSucceeded'
        QtMocHelpers::SlotData<void(const QUuid &, const QList<QString> &)>(47, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 20, 21 }, { QMetaType::QStringList, 24 },
        }}),
        // Slot 'slotCoverArtArchiveLinksTaskFailed'
        QtMocHelpers::SlotData<void(const mixxx::network::JsonWebResponse &)>(48, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Slot 'slotCoverArtArchiveLinksTaskAborted'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotCoverArtArchiveLinksTaskNetworkError'
        QtMocHelpers::SlotData<void(QNetworkReply::NetworkError, const QString &, const mixxx::network::WebResponseWithContent &)>(50, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 36, 37 }, { QMetaType::QString, 38 }, { 0x80000000 | 39, 40 },
        }}),
        // Slot 'slotCoverArtArchiveImageTaskSucceeded'
        QtMocHelpers::SlotData<void(const QUuid &, const QByteArray &)>(51, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 20, 21 }, { QMetaType::QByteArray, 22 },
        }}),
        // Slot 'slotCoverArtArchiveImageTaskFailed'
        QtMocHelpers::SlotData<void(const mixxx::network::WebResponse &, int, const QString &)>(52, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 43, 33 }, { QMetaType::Int, 37 }, { QMetaType::QString, 44 },
        }}),
        // Slot 'slotCoverArtArchiveImageTaskAborted'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotCoverArtArchiveImageTaskNetworkError'
        QtMocHelpers::SlotData<void(QNetworkReply::NetworkError, const QString &, const mixxx::network::WebResponseWithContent &)>(54, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 36, 37 }, { QMetaType::QString, 38 }, { 0x80000000 | 39, 40 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TagFetcher, qt_meta_tag_ZN10TagFetcherE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TagFetcher::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10TagFetcherE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10TagFetcherE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10TagFetcherE_t>.metaTypes,
    nullptr
} };

void TagFetcher::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TagFetcher *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->resultAvailable((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<mixxx::musicbrainz::TrackRelease>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 1: _t->fetchProgress((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->numberOfRecordingsFoundFromAcoustId((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->currentRecordingFetchedFromMusicBrainz(); break;
        case 4: _t->networkError((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 5: _t->fetchedCoverUpdate((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 6: _t->coverArtImageFetchAvailable((*reinterpret_cast<std::add_pointer_t<QUuid>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 7: _t->coverArtArchiveLinksAvailable((*reinterpret_cast<std::add_pointer_t<QUuid>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QString>>>(_a[2]))); break;
        case 8: _t->coverArtLinkNotFound(); break;
        case 9: _t->cancel(); break;
        case 10: _t->slotFingerprintReady(); break;
        case 11: _t->slotAcoustIdTaskSucceeded((*reinterpret_cast<std::add_pointer_t<QList<QUuid>>>(_a[1]))); break;
        case 12: _t->slotAcoustIdTaskFailed((*reinterpret_cast<std::add_pointer_t<mixxx::network::JsonWebResponse>>(_a[1]))); break;
        case 13: _t->slotAcoustIdTaskAborted(); break;
        case 14: _t->slotAcoustIdTaskNetworkError((*reinterpret_cast<std::add_pointer_t<QNetworkReply::NetworkError>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<mixxx::network::WebResponseWithContent>>(_a[3]))); break;
        case 15: _t->slotMusicBrainzTaskSucceeded((*reinterpret_cast<std::add_pointer_t<QList<mixxx::musicbrainz::TrackRelease>>>(_a[1]))); break;
        case 16: _t->slotMusicBrainzTaskFailed((*reinterpret_cast<std::add_pointer_t<mixxx::network::WebResponse>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 17: _t->slotMusicBrainzTaskAborted(); break;
        case 18: _t->slotMusicBrainzTaskNetworkError((*reinterpret_cast<std::add_pointer_t<QNetworkReply::NetworkError>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<mixxx::network::WebResponseWithContent>>(_a[3]))); break;
        case 19: _t->slotCoverArtArchiveLinksTaskSucceeded((*reinterpret_cast<std::add_pointer_t<QUuid>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QString>>>(_a[2]))); break;
        case 20: _t->slotCoverArtArchiveLinksTaskFailed((*reinterpret_cast<std::add_pointer_t<mixxx::network::JsonWebResponse>>(_a[1]))); break;
        case 21: _t->slotCoverArtArchiveLinksTaskAborted(); break;
        case 22: _t->slotCoverArtArchiveLinksTaskNetworkError((*reinterpret_cast<std::add_pointer_t<QNetworkReply::NetworkError>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<mixxx::network::WebResponseWithContent>>(_a[3]))); break;
        case 23: _t->slotCoverArtArchiveImageTaskSucceeded((*reinterpret_cast<std::add_pointer_t<QUuid>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 24: _t->slotCoverArtArchiveImageTaskFailed((*reinterpret_cast<std::add_pointer_t<mixxx::network::WebResponse>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 25: _t->slotCoverArtArchiveImageTaskAborted(); break;
        case 26: _t->slotCoverArtArchiveImageTaskNetworkError((*reinterpret_cast<std::add_pointer_t<QNetworkReply::NetworkError>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<mixxx::network::WebResponseWithContent>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<mixxx::musicbrainz::TrackRelease> >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< mixxx::network::JsonWebResponse >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNetworkReply::NetworkError >(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< mixxx::network::WebResponseWithContent >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<mixxx::musicbrainz::TrackRelease> >(); break;
            }
            break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< mixxx::network::WebResponse >(); break;
            }
            break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNetworkReply::NetworkError >(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< mixxx::network::WebResponseWithContent >(); break;
            }
            break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< mixxx::network::JsonWebResponse >(); break;
            }
            break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNetworkReply::NetworkError >(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< mixxx::network::WebResponseWithContent >(); break;
            }
            break;
        case 24:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< mixxx::network::WebResponse >(); break;
            }
            break;
        case 26:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNetworkReply::NetworkError >(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< mixxx::network::WebResponseWithContent >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TagFetcher::*)(TrackPointer , const QList<mixxx::musicbrainz::TrackRelease> & , const QString & )>(_a, &TagFetcher::resultAvailable, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagFetcher::*)(const QString & )>(_a, &TagFetcher::fetchProgress, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagFetcher::*)(int )>(_a, &TagFetcher::numberOfRecordingsFoundFromAcoustId, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagFetcher::*)()>(_a, &TagFetcher::currentRecordingFetchedFromMusicBrainz, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagFetcher::*)(int , const QString & , const QString & , int )>(_a, &TagFetcher::networkError, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagFetcher::*)(const QByteArray & )>(_a, &TagFetcher::fetchedCoverUpdate, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagFetcher::*)(const QUuid & , const QByteArray & )>(_a, &TagFetcher::coverArtImageFetchAvailable, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagFetcher::*)(const QUuid & , const QList<QString> & )>(_a, &TagFetcher::coverArtArchiveLinksAvailable, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagFetcher::*)()>(_a, &TagFetcher::coverArtLinkNotFound, 8))
            return;
    }
}

const QMetaObject *TagFetcher::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TagFetcher::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10TagFetcherE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int TagFetcher::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    return _id;
}

// SIGNAL 0
void TagFetcher::resultAvailable(TrackPointer _t1, const QList<mixxx::musicbrainz::TrackRelease> & _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void TagFetcher::fetchProgress(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void TagFetcher::numberOfRecordingsFoundFromAcoustId(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void TagFetcher::currentRecordingFetchedFromMusicBrainz()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void TagFetcher::networkError(int _t1, const QString & _t2, const QString & _t3, int _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 5
void TagFetcher::fetchedCoverUpdate(const QByteArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void TagFetcher::coverArtImageFetchAvailable(const QUuid & _t1, const QByteArray & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void TagFetcher::coverArtArchiveLinksAvailable(const QUuid & _t1, const QList<QString> & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}

// SIGNAL 8
void TagFetcher::coverArtLinkNotFound()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}
QT_WARNING_POP
