/****************************************************************************
** Meta object code from reading C++ file 'librarycontrol.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/library/librarycontrol.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'librarycontrol.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN21LoadToGroupControllerE_t {};
} // unnamed namespace

template <> constexpr inline auto LoadToGroupController::qt_create_metaobjectdata<qt_meta_tag_ZN21LoadToGroupControllerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LoadToGroupController",
        "loadToGroup",
        "",
        "group",
        "slotLoadToGroup",
        "v",
        "slotLoadToGroupAndPlay"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'loadToGroup'
        QtMocHelpers::SignalData<void(const QString &, bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Bool, 2 },
        }}),
        // Slot 'slotLoadToGroup'
        QtMocHelpers::SlotData<void(double)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 5 },
        }}),
        // Slot 'slotLoadToGroupAndPlay'
        QtMocHelpers::SlotData<void(double)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LoadToGroupController, qt_meta_tag_ZN21LoadToGroupControllerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject LoadToGroupController::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21LoadToGroupControllerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21LoadToGroupControllerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21LoadToGroupControllerE_t>.metaTypes,
    nullptr
} };

void LoadToGroupController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LoadToGroupController *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->loadToGroup((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 1: _t->slotLoadToGroup((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 2: _t->slotLoadToGroupAndPlay((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (LoadToGroupController::*)(const QString & , bool )>(_a, &LoadToGroupController::loadToGroup, 0))
            return;
    }
}

const QMetaObject *LoadToGroupController::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LoadToGroupController::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21LoadToGroupControllerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int LoadToGroupController::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void LoadToGroupController::loadToGroup(const QString & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN14LibraryControlE_t {};
} // unnamed namespace

template <> constexpr inline auto LibraryControl::qt_create_metaobjectdata<qt_meta_tag_ZN14LibraryControlE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LibraryControl",
        "clearSearchIfClearButtonHasFocus",
        "",
        "showHideTrackMenu",
        "show",
        "slotLoadSelectedTrackToGroup",
        "group",
        "play",
        "slotUpdateTrackMenuControl",
        "visible",
        "libraryWidgetDeleted",
        "sidebarWidgetDeleted",
        "searchboxWidgetDeleted",
        "slotFocusedWidgetChanged",
        "QWidget*",
        "oldW",
        "newW",
        "updateFocusedWidgetControls",
        "refocusPrevLibraryWidget",
        "slotMoveUp",
        "slotMoveDown",
        "slotMoveVertical",
        "slotScrollUp",
        "slotScrollDown",
        "slotScrollVertical",
        "slotMoveLeft",
        "slotMoveRight",
        "slotMoveHorizontal",
        "slotMoveFocusForward",
        "slotMoveFocusBackward",
        "slotMoveFocus",
        "slotMoveTrackUp",
        "slotMoveTrackDown",
        "slotMoveTrack",
        "slotGoToItem",
        "v",
        "slotTrackColorPrev",
        "slotTrackColorNext",
        "slotSelectNextTrack",
        "slotSelectPrevTrack",
        "slotSelectTrack",
        "slotSelectSidebarItem",
        "slotSelectNextSidebarItem",
        "slotSelectPrevSidebarItem",
        "slotToggleSelectedSidebarItem",
        "slotLoadSelectedIntoFirstStopped",
        "slotAutoDjAddTop",
        "slotAutoDjAddBottom",
        "slotAutoDjAddReplace",
        "maybeCreateGroupController",
        "slotNumDecksChanged",
        "slotNumSamplersChanged",
        "slotNumPreviewDecksChanged",
        "slotSortColumn",
        "slotSortColumnToggle",
        "slotFontSize",
        "slotIncrementFontSize",
        "slotDecrementFontSize"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'clearSearchIfClearButtonHasFocus'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'showHideTrackMenu'
        QtMocHelpers::SignalData<void(bool)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 4 },
        }}),
        // Slot 'slotLoadSelectedTrackToGroup'
        QtMocHelpers::SlotData<void(const QString &, bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { QMetaType::Bool, 7 },
        }}),
        // Slot 'slotUpdateTrackMenuControl'
        QtMocHelpers::SlotData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Slot 'libraryWidgetDeleted'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'sidebarWidgetDeleted'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'searchboxWidgetDeleted'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotFocusedWidgetChanged'
        QtMocHelpers::SlotData<void(QWidget *, QWidget *)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 14, 15 }, { 0x80000000 | 14, 16 },
        }}),
        // Slot 'updateFocusedWidgetControls'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'refocusPrevLibraryWidget'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotMoveUp'
        QtMocHelpers::SlotData<void(double)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotMoveDown'
        QtMocHelpers::SlotData<void(double)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotMoveVertical'
        QtMocHelpers::SlotData<void(double)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotScrollUp'
        QtMocHelpers::SlotData<void(double)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotScrollDown'
        QtMocHelpers::SlotData<void(double)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotScrollVertical'
        QtMocHelpers::SlotData<void(double)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotMoveLeft'
        QtMocHelpers::SlotData<void(double)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotMoveRight'
        QtMocHelpers::SlotData<void(double)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotMoveHorizontal'
        QtMocHelpers::SlotData<void(double)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotMoveFocusForward'
        QtMocHelpers::SlotData<void(double)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotMoveFocusBackward'
        QtMocHelpers::SlotData<void(double)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotMoveFocus'
        QtMocHelpers::SlotData<void(double)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotMoveTrackUp'
        QtMocHelpers::SlotData<void(double)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotMoveTrackDown'
        QtMocHelpers::SlotData<void(double)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotMoveTrack'
        QtMocHelpers::SlotData<void(double)>(33, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'slotGoToItem'
        QtMocHelpers::SlotData<void(double)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 35 },
        }}),
        // Slot 'slotTrackColorPrev'
        QtMocHelpers::SlotData<void(double)>(36, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 35 },
        }}),
        // Slot 'slotTrackColorNext'
        QtMocHelpers::SlotData<void(double)>(37, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 35 },
        }}),
        // Slot 'slotSelectNextTrack'
        QtMocHelpers::SlotData<void(double)>(38, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 35 },
        }}),
        // Slot 'slotSelectPrevTrack'
        QtMocHelpers::SlotData<void(double)>(39, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 35 },
        }}),
        // Slot 'slotSelectTrack'
        QtMocHelpers::SlotData<void(double)>(40, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 35 },
        }}),
        // Slot 'slotSelectSidebarItem'
        QtMocHelpers::SlotData<void(double)>(41, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 35 },
        }}),
        // Slot 'slotSelectNextSidebarItem'
        QtMocHelpers::SlotData<void(double)>(42, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 35 },
        }}),
        // Slot 'slotSelectPrevSidebarItem'
        QtMocHelpers::SlotData<void(double)>(43, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 35 },
        }}),
        // Slot 'slotToggleSelectedSidebarItem'
        QtMocHelpers::SlotData<void(double)>(44, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 35 },
        }}),
        // Slot 'slotLoadSelectedIntoFirstStopped'
        QtMocHelpers::SlotData<void(double)>(45, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 35 },
        }}),
        // Slot 'slotAutoDjAddTop'
        QtMocHelpers::SlotData<void(double)>(46, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 35 },
        }}),
        // Slot 'slotAutoDjAddBottom'
        QtMocHelpers::SlotData<void(double)>(47, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 35 },
        }}),
        // Slot 'slotAutoDjAddReplace'
        QtMocHelpers::SlotData<void(double)>(48, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 35 },
        }}),
        // Slot 'maybeCreateGroupController'
        QtMocHelpers::SlotData<void(const QString &)>(49, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Slot 'slotNumDecksChanged'
        QtMocHelpers::SlotData<void(double)>(50, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 35 },
        }}),
        // Slot 'slotNumSamplersChanged'
        QtMocHelpers::SlotData<void(double)>(51, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 35 },
        }}),
        // Slot 'slotNumPreviewDecksChanged'
        QtMocHelpers::SlotData<void(double)>(52, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 35 },
        }}),
        // Slot 'slotSortColumn'
        QtMocHelpers::SlotData<void(double)>(53, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 35 },
        }}),
        // Slot 'slotSortColumnToggle'
        QtMocHelpers::SlotData<void(double)>(54, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 35 },
        }}),
        // Slot 'slotFontSize'
        QtMocHelpers::SlotData<void(double)>(55, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 35 },
        }}),
        // Slot 'slotIncrementFontSize'
        QtMocHelpers::SlotData<void(double)>(56, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 35 },
        }}),
        // Slot 'slotDecrementFontSize'
        QtMocHelpers::SlotData<void(double)>(57, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 35 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LibraryControl, qt_meta_tag_ZN14LibraryControlE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject LibraryControl::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14LibraryControlE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14LibraryControlE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14LibraryControlE_t>.metaTypes,
    nullptr
} };

void LibraryControl::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LibraryControl *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->clearSearchIfClearButtonHasFocus(); break;
        case 1: _t->showHideTrackMenu((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->slotLoadSelectedTrackToGroup((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 3: _t->slotUpdateTrackMenuControl((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->libraryWidgetDeleted(); break;
        case 5: _t->sidebarWidgetDeleted(); break;
        case 6: _t->searchboxWidgetDeleted(); break;
        case 7: _t->slotFocusedWidgetChanged((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[2]))); break;
        case 8: _t->updateFocusedWidgetControls(); break;
        case 9: _t->refocusPrevLibraryWidget(); break;
        case 10: _t->slotMoveUp((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 11: _t->slotMoveDown((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 12: _t->slotMoveVertical((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 13: _t->slotScrollUp((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 14: _t->slotScrollDown((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 15: _t->slotScrollVertical((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 16: _t->slotMoveLeft((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 17: _t->slotMoveRight((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 18: _t->slotMoveHorizontal((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 19: _t->slotMoveFocusForward((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 20: _t->slotMoveFocusBackward((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 21: _t->slotMoveFocus((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 22: _t->slotMoveTrackUp((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 23: _t->slotMoveTrackDown((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 24: _t->slotMoveTrack((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 25: _t->slotGoToItem((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 26: _t->slotTrackColorPrev((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 27: _t->slotTrackColorNext((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 28: _t->slotSelectNextTrack((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 29: _t->slotSelectPrevTrack((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 30: _t->slotSelectTrack((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 31: _t->slotSelectSidebarItem((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 32: _t->slotSelectNextSidebarItem((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 33: _t->slotSelectPrevSidebarItem((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 34: _t->slotToggleSelectedSidebarItem((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 35: _t->slotLoadSelectedIntoFirstStopped((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 36: _t->slotAutoDjAddTop((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 37: _t->slotAutoDjAddBottom((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 38: _t->slotAutoDjAddReplace((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 39: _t->maybeCreateGroupController((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 40: _t->slotNumDecksChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 41: _t->slotNumSamplersChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 42: _t->slotNumPreviewDecksChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 43: _t->slotSortColumn((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 44: _t->slotSortColumnToggle((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 45: _t->slotFontSize((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 46: _t->slotIncrementFontSize((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 47: _t->slotDecrementFontSize((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (LibraryControl::*)()>(_a, &LibraryControl::clearSearchIfClearButtonHasFocus, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (LibraryControl::*)(bool )>(_a, &LibraryControl::showHideTrackMenu, 1))
            return;
    }
}

const QMetaObject *LibraryControl::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LibraryControl::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14LibraryControlE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int LibraryControl::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 48)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 48;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 48)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 48;
    }
    return _id;
}

// SIGNAL 0
void LibraryControl::clearSearchIfClearButtonHasFocus()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void LibraryControl::showHideTrackMenu(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
