/****************************************************************************
** Meta object code from reading C++ file 'mixxxmainwindow.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/mixxxmainwindow.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mixxxmainwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15MixxxMainWindowE_t {};
} // unnamed namespace

template <> constexpr inline auto MixxxMainWindow::qt_create_metaobjectdata<qt_meta_tag_ZN15MixxxMainWindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MixxxMainWindow",
        "skinLoaded",
        "",
        "developerToolsDlgClosed",
        "r",
        "closeDeveloperToolsDlgChecked",
        "fullScreenChanged",
        "fullscreen",
        "rebootMixxxView",
        "slotFileLoadSongPlayer",
        "deck",
        "slotOptionsPreferences",
        "slotHelpAbout",
        "slotShowKeywheel",
        "toggle",
        "slotViewFullScreen",
        "slotDeveloperTools",
        "enable",
        "slotDeveloperToolsClosed",
        "slotUpdateWindowTitle",
        "TrackPointer",
        "pTrack",
        "slotNoMicrophoneInputConfigured",
        "slotNoAuxiliaryInputConfigured",
        "slotNoDeckPassthroughInputConfigured",
        "slotNoVinylControlInputConfigured",
        "slotUpdateMenuBarAltKeyConnection",
        "initializationProgressUpdate",
        "progress",
        "serviceName",
        "slotTooltipModeChanged",
        "mixxx::preferences::Tooltips",
        "tt"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'skinLoaded'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'developerToolsDlgClosed'
        QtMocHelpers::SignalData<void(int)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Signal 'closeDeveloperToolsDlgChecked'
        QtMocHelpers::SignalData<void(int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Signal 'fullScreenChanged'
        QtMocHelpers::SignalData<void(bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'rebootMixxxView'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFileLoadSongPlayer'
        QtMocHelpers::SlotData<void(int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 },
        }}),
        // Slot 'slotOptionsPreferences'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotHelpAbout'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotShowKeywheel'
        QtMocHelpers::SlotData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Slot 'slotViewFullScreen'
        QtMocHelpers::SlotData<void(bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Slot 'slotDeveloperTools'
        QtMocHelpers::SlotData<void(bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'slotDeveloperToolsClosed'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotUpdateWindowTitle'
        QtMocHelpers::SlotData<void(TrackPointer)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'slotNoMicrophoneInputConfigured'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotNoAuxiliaryInputConfigured'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotNoDeckPassthroughInputConfigured'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotNoVinylControlInputConfigured'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotUpdateMenuBarAltKeyConnection'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'initializationProgressUpdate'
        QtMocHelpers::SlotData<void(int, const QString &)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 28 }, { QMetaType::QString, 29 },
        }}),
        // Slot 'slotTooltipModeChanged'
        QtMocHelpers::SlotData<void(mixxx::preferences::Tooltips)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 31, 32 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MixxxMainWindow, qt_meta_tag_ZN15MixxxMainWindowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MixxxMainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MixxxMainWindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MixxxMainWindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15MixxxMainWindowE_t>.metaTypes,
    nullptr
} };

void MixxxMainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MixxxMainWindow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->skinLoaded(); break;
        case 1: _t->developerToolsDlgClosed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->closeDeveloperToolsDlgChecked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->fullScreenChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->rebootMixxxView(); break;
        case 5: _t->slotFileLoadSongPlayer((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->slotOptionsPreferences(); break;
        case 7: _t->slotHelpAbout(); break;
        case 8: _t->slotShowKeywheel((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->slotViewFullScreen((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->slotDeveloperTools((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->slotDeveloperToolsClosed(); break;
        case 12: _t->slotUpdateWindowTitle((*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[1]))); break;
        case 13: _t->slotNoMicrophoneInputConfigured(); break;
        case 14: _t->slotNoAuxiliaryInputConfigured(); break;
        case 15: _t->slotNoDeckPassthroughInputConfigured(); break;
        case 16: _t->slotNoVinylControlInputConfigured(); break;
        case 17: _t->slotUpdateMenuBarAltKeyConnection(); break;
        case 18: _t->initializationProgressUpdate((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 19: _t->slotTooltipModeChanged((*reinterpret_cast<std::add_pointer_t<mixxx::preferences::Tooltips>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MixxxMainWindow::*)()>(_a, &MixxxMainWindow::skinLoaded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MixxxMainWindow::*)(int )>(_a, &MixxxMainWindow::developerToolsDlgClosed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MixxxMainWindow::*)(int )>(_a, &MixxxMainWindow::closeDeveloperToolsDlgChecked, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (MixxxMainWindow::*)(bool )>(_a, &MixxxMainWindow::fullScreenChanged, 3))
            return;
    }
}

const QMetaObject *MixxxMainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MixxxMainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15MixxxMainWindowE_t>.strings))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int MixxxMainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    return _id;
}

// SIGNAL 0
void MixxxMainWindow::skinLoaded()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void MixxxMainWindow::developerToolsDlgClosed(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void MixxxMainWindow::closeDeveloperToolsDlgChecked(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void MixxxMainWindow::fullScreenChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
