// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: keys.proto
// Protobuf C++ Version: 7.35.0

#ifndef keys_2eproto_2epb_2eh
#define keys_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

// clang-format off
#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 7035000
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/generated_enum_util.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_keys_2eproto

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct TableStruct_keys_2eproto {
  static const ::uint32_t offsets[];
};
namespace mixxx {
namespace track {
namespace io {
namespace key {
enum ChromaticKey : int;
extern const uint32_t ChromaticKey_internal_data_[];
enum Source : int;
extern const uint32_t Source_internal_data_[];
class KeyMap;
struct KeyMapGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern KeyMapGlobalsTypeInternal KeyMap_globals_;
extern const ::google::protobuf::internal::ClassDataLite KeyMap_class_data_;
#else
extern const KeyMapGlobalsTypeInternal KeyMap_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class KeyMap_KeyChange;
struct KeyMap_KeyChangeGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern KeyMap_KeyChangeGlobalsTypeInternal KeyMap_KeyChange_globals_;
extern const ::google::protobuf::internal::ClassDataLite KeyMap_KeyChange_class_data_;
#else
extern const KeyMap_KeyChangeGlobalsTypeInternal KeyMap_KeyChange_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
}  // namespace key
}  // namespace io
}  // namespace track
}  // namespace mixxx
namespace google {
namespace protobuf {
template <>
internal::EnumTraitsT<::mixxx::track::io::key::ChromaticKey_internal_data_>
    internal::EnumTraitsImpl::value<::mixxx::track::io::key::ChromaticKey>;
template <>
internal::EnumTraitsT<::mixxx::track::io::key::Source_internal_data_>
    internal::EnumTraitsImpl::value<::mixxx::track::io::key::Source>;
}  // namespace protobuf
}  // namespace google

namespace mixxx {
namespace track {
namespace io {
namespace key {
enum Source : int {
  ANALYZER = 0,
  FILE_METADATA = 1,
  USER = 2,
};

extern const uint32_t Source_internal_data_[];
inline constexpr Source Source_MIN =
    static_cast<Source>(0);
inline constexpr Source Source_MAX =
    static_cast<Source>(2);
[[nodiscard]] inline bool Source_IsValid(int value) {
  return 0 <= value && value <= 2;
}
inline constexpr int Source_ARRAYSIZE = 2 + 1;
[[nodiscard]] const ::std::string& Source_Name(Source value);
template <typename T>
[[nodiscard]] const ::std::string& Source_Name(T value) {
  static_assert(::std::is_same<T, Source>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to Source_Name().");
  return Source_Name(static_cast<Source>(value));
}
[[nodiscard]] bool Source_Parse(
    ::absl::string_view name, Source* PROTOBUF_NONNULL value);
enum ChromaticKey : int {
  INVALID = 0,
  C_MAJOR = 1,
  D_FLAT_MAJOR = 2,
  D_MAJOR = 3,
  E_FLAT_MAJOR = 4,
  E_MAJOR = 5,
  F_MAJOR = 6,
  F_SHARP_MAJOR = 7,
  G_MAJOR = 8,
  A_FLAT_MAJOR = 9,
  A_MAJOR = 10,
  B_FLAT_MAJOR = 11,
  B_MAJOR = 12,
  C_MINOR = 13,
  C_SHARP_MINOR = 14,
  D_MINOR = 15,
  E_FLAT_MINOR = 16,
  E_MINOR = 17,
  F_MINOR = 18,
  F_SHARP_MINOR = 19,
  G_MINOR = 20,
  G_SHARP_MINOR = 21,
  A_MINOR = 22,
  B_FLAT_MINOR = 23,
  B_MINOR = 24,
};

extern const uint32_t ChromaticKey_internal_data_[];
inline constexpr ChromaticKey ChromaticKey_MIN =
    static_cast<ChromaticKey>(0);
inline constexpr ChromaticKey ChromaticKey_MAX =
    static_cast<ChromaticKey>(24);
[[nodiscard]] inline bool ChromaticKey_IsValid(int value) {
  return 0 <= value && value <= 24;
}
inline constexpr int ChromaticKey_ARRAYSIZE = 24 + 1;
[[nodiscard]] const ::std::string& ChromaticKey_Name(ChromaticKey value);
template <typename T>
[[nodiscard]] const ::std::string& ChromaticKey_Name(T value) {
  static_assert(::std::is_same<T, ChromaticKey>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to ChromaticKey_Name().");
  return ChromaticKey_Name(static_cast<ChromaticKey>(value));
}
[[nodiscard]] bool ChromaticKey_Parse(
    ::absl::string_view name, ChromaticKey* PROTOBUF_NONNULL value);
using ::google::protobuf::internal::generated_enum::AbslParseFlag;
using ::google::protobuf::internal::generated_enum::AbslUnparseFlag;

// ===================================================================


// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED KeyMap_KeyChange final : public ::google::protobuf::MessageLite
/* @@protoc_insertion_point(class_definition:mixxx.track.io.key.KeyMap.KeyChange) */ {
 public:
  inline KeyMap_KeyChange() : KeyMap_KeyChange(nullptr) {}
  ~KeyMap_KeyChange() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(KeyMap_KeyChange* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(KeyMap_KeyChange));
  }
#endif

  template <typename = void>
  explicit constexpr KeyMap_KeyChange(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline KeyMap_KeyChange(const KeyMap_KeyChange& from) : KeyMap_KeyChange(nullptr, from) {}
  inline KeyMap_KeyChange(KeyMap_KeyChange&& from) noexcept : KeyMap_KeyChange(nullptr, ::std::move(from)) {}
  inline KeyMap_KeyChange& operator=(const KeyMap_KeyChange& from) {
    CopyFrom(from);
    return *this;
  }
  inline KeyMap_KeyChange& operator=(KeyMap_KeyChange&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::std::string& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString);
  }
  [[nodiscard]] inline ::std::string* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::std::string>();
  }

  [[nodiscard]] static const KeyMap_KeyChange& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<KeyMap_KeyChange>(&KeyMap_KeyChange_globals_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(KeyMap_KeyChange& a, KeyMap_KeyChange& b) { a.Swap(&b); }
  inline void Swap(KeyMap_KeyChange* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(KeyMap_KeyChange* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] KeyMap_KeyChange* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::MessageLite::DefaultConstruct<KeyMap_KeyChange>(arena);
  }
  void CopyFrom(const KeyMap_KeyChange& from);
  void MergeFrom(const KeyMap_KeyChange& from) { KeyMap_KeyChange::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(KeyMap_KeyChange* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "mixxx.track.io.key.KeyMap.KeyChange"; }

  explicit KeyMap_KeyChange(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  KeyMap_KeyChange(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const KeyMap_KeyChange& from);
  KeyMap_KeyChange(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, KeyMap_KeyChange&& from) noexcept
      : KeyMap_KeyChange(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kFramePositionFieldNumber = 1,
    kKeyFieldNumber = 2,
  };
  // optional int32 frame_position = 1;
  [[nodiscard]] bool has_frame_position()
      const;
  void clear_frame_position() ;
  [[nodiscard]] ::int32_t frame_position() const;
  void set_frame_position(::int32_t value);

  private:
  ::int32_t _internal_frame_position() const;
  void _internal_set_frame_position(::int32_t value);

  public:
  // optional .mixxx.track.io.key.ChromaticKey key = 2;
  [[nodiscard]] bool has_key()
      const;
  void clear_key() ;
  [[nodiscard]] ::mixxx::track::io::key::ChromaticKey key() const;
  void set_key(::mixxx::track::io::key::ChromaticKey value);

  private:
  ::mixxx::track::io::key::ChromaticKey _internal_key() const;
  void _internal_set_key(::mixxx::track::io::key::ChromaticKey value);

  public:
  // @@protoc_insertion_point(class_scope:mixxx.track.io.key.KeyMap.KeyChange)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<1, 2,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const KeyMap_KeyChange& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::int32_t frame_position_;
    int key_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_keys_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED KeyMap final : public ::google::protobuf::MessageLite
/* @@protoc_insertion_point(class_definition:mixxx.track.io.key.KeyMap) */ {
 public:
  inline KeyMap() : KeyMap(nullptr) {}
  ~KeyMap() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(KeyMap* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(KeyMap));
  }
#endif

  template <typename = void>
  explicit constexpr KeyMap(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline KeyMap(const KeyMap& from) : KeyMap(nullptr, from) {}
  inline KeyMap(KeyMap&& from) noexcept : KeyMap(nullptr, ::std::move(from)) {}
  inline KeyMap& operator=(const KeyMap& from) {
    CopyFrom(from);
    return *this;
  }
  inline KeyMap& operator=(KeyMap&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::std::string& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::std::string>(::google::protobuf::internal::GetEmptyString);
  }
  [[nodiscard]] inline ::std::string* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::std::string>();
  }

  [[nodiscard]] static const KeyMap& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<KeyMap>(&KeyMap_globals_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(KeyMap& a, KeyMap& b) { a.Swap(&b); }
  inline void Swap(KeyMap* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(KeyMap* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] KeyMap* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::MessageLite::DefaultConstruct<KeyMap>(arena);
  }
  void CopyFrom(const KeyMap& from);
  void MergeFrom(const KeyMap& from) { KeyMap::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(KeyMap* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "mixxx.track.io.key.KeyMap"; }

  explicit KeyMap(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  KeyMap(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const KeyMap& from);
  KeyMap(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, KeyMap&& from) noexcept
      : KeyMap(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  // nested types ----------------------------------------------------
  using KeyChange = KeyMap_KeyChange;

  // accessors -------------------------------------------------------
  enum : int {
    kKeyChangeFieldNumber = 3,
    kGlobalKeyTextFieldNumber = 2,
    kGlobalKeyFieldNumber = 1,
    kSourceFieldNumber = 4,
  };
  // repeated .mixxx.track.io.key.KeyMap.KeyChange key_change = 3;
  [[nodiscard]] int key_change_size()
      const;
  private:
  int _internal_key_change_size() const;

  public:
  void clear_key_change() ;
  [[nodiscard]] const ::mixxx::track::io::key::KeyMap_KeyChange& key_change(int index) const;
  [[nodiscard]] ::mixxx::track::io::key::KeyMap_KeyChange* PROTOBUF_NONNULL mutable_key_change(int index);
  ::mixxx::track::io::key::KeyMap_KeyChange* PROTOBUF_NONNULL add_key_change();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::mixxx::track::io::key::KeyMap_KeyChange>&
  key_change() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::mixxx::track::io::key::KeyMap_KeyChange>* PROTOBUF_NONNULL
  mutable_key_change();

  private:
  const ::google::protobuf::RepeatedPtrField<::mixxx::track::io::key::KeyMap_KeyChange>& _internal_key_change() const;
  ::google::protobuf::RepeatedPtrField<::mixxx::track::io::key::KeyMap_KeyChange>* PROTOBUF_NONNULL _internal_mutable_key_change();

  public:
  // optional string global_key_text = 2;
  [[nodiscard]] bool has_global_key_text()
      const;
  void clear_global_key_text() ;
  [[nodiscard]] const ::std::string& global_key_text() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_global_key_text(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_global_key_text();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_global_key_text();
  void set_allocated_global_key_text(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_global_key_text() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_global_key_text(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_global_key_text();

  public:
  // optional .mixxx.track.io.key.ChromaticKey global_key = 1 [default = INVALID];
  [[nodiscard]] bool has_global_key()
      const;
  void clear_global_key() ;
  [[nodiscard]] ::mixxx::track::io::key::ChromaticKey global_key() const;
  void set_global_key(::mixxx::track::io::key::ChromaticKey value);

  private:
  ::mixxx::track::io::key::ChromaticKey _internal_global_key() const;
  void _internal_set_global_key(::mixxx::track::io::key::ChromaticKey value);

  public:
  // optional .mixxx.track.io.key.Source source = 4 [default = ANALYZER];
  [[nodiscard]] bool has_source()
      const;
  void clear_source() ;
  [[nodiscard]] ::mixxx::track::io::key::Source source() const;
  void set_source(::mixxx::track::io::key::Source value);

  private:
  ::mixxx::track::io::key::Source _internal_source() const;
  void _internal_set_source(::mixxx::track::io::key::Source value);

  public:
  // @@protoc_insertion_point(class_scope:mixxx.track.io.key.KeyMap)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<2, 4,
                          3, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const KeyMap& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::mixxx::track::io::key::KeyMap_KeyChange > key_change_;
    ::google::protobuf::internal::ArenaStringPtr global_key_text_;
    int global_key_;
    int source_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_keys_2eproto;
};

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// KeyMap_KeyChange

// optional int32 frame_position = 1;
inline bool KeyMap_KeyChange::has_frame_position() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  return value;
}
inline void KeyMap_KeyChange::clear_frame_position() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.frame_position_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline ::int32_t KeyMap_KeyChange::frame_position() const {
  // @@protoc_insertion_point(field_get:mixxx.track.io.key.KeyMap.KeyChange.frame_position)
  return _internal_frame_position();
}
inline void KeyMap_KeyChange::set_frame_position(::int32_t value) {
  _internal_set_frame_position(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_set:mixxx.track.io.key.KeyMap.KeyChange.frame_position)
}
inline ::int32_t KeyMap_KeyChange::_internal_frame_position() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.frame_position_;
}
inline void KeyMap_KeyChange::_internal_set_frame_position(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.frame_position_ = value;
}

// optional .mixxx.track.io.key.ChromaticKey key = 2;
inline bool KeyMap_KeyChange::has_key() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  return value;
}
inline void KeyMap_KeyChange::clear_key() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.key_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::mixxx::track::io::key::ChromaticKey KeyMap_KeyChange::key() const {
  // @@protoc_insertion_point(field_get:mixxx.track.io.key.KeyMap.KeyChange.key)
  return _internal_key();
}
inline void KeyMap_KeyChange::set_key(::mixxx::track::io::key::ChromaticKey value) {
  _internal_set_key(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:mixxx.track.io.key.KeyMap.KeyChange.key)
}
inline ::mixxx::track::io::key::ChromaticKey KeyMap_KeyChange::_internal_key() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::mixxx::track::io::key::ChromaticKey>(_impl_.key_);
}
inline void KeyMap_KeyChange::_internal_set_key(::mixxx::track::io::key::ChromaticKey value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);

                                          assert(::google::protobuf::internal::ValidateEnum(
                                              value, ::mixxx::track::io::key::ChromaticKey_internal_data_));
                                          _impl_.key_ = value;
}

// -------------------------------------------------------------------

// KeyMap

// optional .mixxx.track.io.key.ChromaticKey global_key = 1 [default = INVALID];
inline bool KeyMap::has_global_key() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  return value;
}
inline void KeyMap::clear_global_key() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.global_key_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline ::mixxx::track::io::key::ChromaticKey KeyMap::global_key() const {
  // @@protoc_insertion_point(field_get:mixxx.track.io.key.KeyMap.global_key)
  return _internal_global_key();
}
inline void KeyMap::set_global_key(::mixxx::track::io::key::ChromaticKey value) {
  _internal_set_global_key(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:mixxx.track.io.key.KeyMap.global_key)
}
inline ::mixxx::track::io::key::ChromaticKey KeyMap::_internal_global_key() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::mixxx::track::io::key::ChromaticKey>(_impl_.global_key_);
}
inline void KeyMap::_internal_set_global_key(::mixxx::track::io::key::ChromaticKey value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);

                                          assert(::google::protobuf::internal::ValidateEnum(
                                              value, ::mixxx::track::io::key::ChromaticKey_internal_data_));
                                          _impl_.global_key_ = value;
}

// optional string global_key_text = 2;
inline bool KeyMap::has_global_key_text() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  return value;
}
inline void KeyMap::clear_global_key_text() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.global_key_text_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& KeyMap::global_key_text() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:mixxx.track.io.key.KeyMap.global_key_text)
  return _internal_global_key_text();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void KeyMap::set_global_key_text(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.global_key_text_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:mixxx.track.io.key.KeyMap.global_key_text)
}
inline ::std::string* PROTOBUF_NONNULL KeyMap::mutable_global_key_text()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_global_key_text();
  // @@protoc_insertion_point(field_mutable:mixxx.track.io.key.KeyMap.global_key_text)
  return _s;
}
inline const ::std::string& KeyMap::_internal_global_key_text() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.global_key_text_.Get();
}
inline void KeyMap::_internal_set_global_key_text(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.global_key_text_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL KeyMap::_internal_mutable_global_key_text() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.global_key_text_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE KeyMap::release_global_key_text() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:mixxx.track.io.key.KeyMap.global_key_text)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.global_key_text_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.global_key_text_.Set("", GetArena());
  }
  return released;
}
inline void KeyMap::set_allocated_global_key_text(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.global_key_text_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.global_key_text_.IsDefault()) {
    _impl_.global_key_text_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:mixxx.track.io.key.KeyMap.global_key_text)
}

// repeated .mixxx.track.io.key.KeyMap.KeyChange key_change = 3;
inline int KeyMap::_internal_key_change_size() const {
  return _internal_key_change().size();
}
inline int KeyMap::key_change_size() const {
  return _internal_key_change_size();
}
inline void KeyMap::clear_key_change() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.key_change_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::mixxx::track::io::key::KeyMap_KeyChange& KeyMap::key_change(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:mixxx.track.io.key.KeyMap.key_change)
  return _internal_key_change().Get(index);
}
inline ::mixxx::track::io::key::KeyMap_KeyChange* PROTOBUF_NONNULL KeyMap::mutable_key_change(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:mixxx.track.io.key.KeyMap.key_change)
  return _internal_mutable_key_change()->Mutable(index);
}
inline ::mixxx::track::io::key::KeyMap_KeyChange* PROTOBUF_NONNULL KeyMap::add_key_change()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::mixxx::track::io::key::KeyMap_KeyChange* _add =
      _internal_mutable_key_change()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:mixxx.track.io.key.KeyMap.key_change)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::mixxx::track::io::key::KeyMap_KeyChange>& KeyMap::key_change() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:mixxx.track.io.key.KeyMap.key_change)
  return _internal_key_change();
}
inline ::google::protobuf::RepeatedPtrField<::mixxx::track::io::key::KeyMap_KeyChange>* PROTOBUF_NONNULL
KeyMap::mutable_key_change() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:mixxx.track.io.key.KeyMap.key_change)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_key_change();
}
inline const ::google::protobuf::RepeatedPtrField<::mixxx::track::io::key::KeyMap_KeyChange>&
KeyMap::_internal_key_change() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.key_change_;
}
inline ::google::protobuf::RepeatedPtrField<::mixxx::track::io::key::KeyMap_KeyChange>* PROTOBUF_NONNULL
KeyMap::_internal_mutable_key_change() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.key_change_;
}

// optional .mixxx.track.io.key.Source source = 4 [default = ANALYZER];
inline bool KeyMap::has_source() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000008U);
  return value;
}
inline void KeyMap::clear_source() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.source_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline ::mixxx::track::io::key::Source KeyMap::source() const {
  // @@protoc_insertion_point(field_get:mixxx.track.io.key.KeyMap.source)
  return _internal_source();
}
inline void KeyMap::set_source(::mixxx::track::io::key::Source value) {
  _internal_set_source(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:mixxx.track.io.key.KeyMap.source)
}
inline ::mixxx::track::io::key::Source KeyMap::_internal_source() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::mixxx::track::io::key::Source>(_impl_.source_);
}
inline void KeyMap::_internal_set_source(::mixxx::track::io::key::Source value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);

                                          assert(::google::protobuf::internal::ValidateEnum(
                                              value, ::mixxx::track::io::key::Source_internal_data_));
                                          _impl_.source_ = value;
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace key
}  // namespace io
}  // namespace track
}  // namespace mixxx


namespace google {
namespace protobuf {

template <>
struct is_proto_enum<::mixxx::track::io::key::Source> : std::true_type {};
template <>
struct internal::LiteEnumFuncs<::mixxx::track::io::key::Source> {
  static constexpr bool kIsDefined = true;
  static constexpr auto kParseFunc = ::mixxx::track::io::key::Source_Parse;
  static constexpr auto kNameFunc = ::mixxx::track::io::key::Source_Name<int>;
};
template <>
struct is_proto_enum<::mixxx::track::io::key::ChromaticKey> : std::true_type {};
template <>
struct internal::LiteEnumFuncs<::mixxx::track::io::key::ChromaticKey> {
  static constexpr bool kIsDefined = true;
  static constexpr auto kParseFunc = ::mixxx::track::io::key::ChromaticKey_Parse;
  static constexpr auto kNameFunc = ::mixxx::track::io::key::ChromaticKey_Name<int>;
};

}  // namespace protobuf
}  // namespace google

// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"
// clang-format on

#endif  // keys_2eproto_2epb_2eh
