/****************************************************************************
** Meta object code from reading C++ file 'track.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/track/track.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'track.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN5TrackE_t {};
} // unnamed namespace

template <> constexpr inline auto Track::qt_create_metaobjectdata<qt_meta_tag_ZN5TrackE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Track",
        "artistChanged",
        "",
        "titleChanged",
        "albumChanged",
        "albumArtistChanged",
        "genreChanged",
        "composerChanged",
        "groupingChanged",
        "yearChanged",
        "trackNumberChanged",
        "trackTotalChanged",
        "commentChanged",
        "bpmChanged",
        "bpmLockChanged",
        "locked",
        "keyChanged",
        "timesPlayedChanged",
        "durationChanged",
        "infoChanged",
        "waveformUpdated",
        "waveformSummaryUpdated",
        "coverArtUpdated",
        "beatsUpdated",
        "replayGainUpdated",
        "mixxx::ReplayGain",
        "replayGain",
        "replayGainAdjusted",
        "requestingPlayerGroup",
        "colorUpdated",
        "mixxx::RgbColor::optional_t",
        "color",
        "ratingUpdated",
        "rating",
        "cuesUpdated",
        "loopRemove",
        "analyzed",
        "changed",
        "TrackId",
        "trackId",
        "dirty",
        "clean",
        "slotCueUpdated",
        "artist",
        "title",
        "album",
        "albumArtist",
        "genre",
        "composer",
        "grouping",
        "year",
        "trackNumber",
        "trackTotal",
        "timesPlayed",
        "comment",
        "bpm",
        "bpmText",
        "keyText",
        "duration",
        "durationTextSeconds",
        "durationTextCentiseconds",
        "durationTextMilliseconds",
        "info",
        "titleInfo",
        "sourceSynchronizedAt"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'artistChanged'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'titleChanged'
        QtMocHelpers::SignalData<void(const QString &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'albumChanged'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'albumArtistChanged'
        QtMocHelpers::SignalData<void(const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'genreChanged'
        QtMocHelpers::SignalData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'composerChanged'
        QtMocHelpers::SignalData<void(const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'groupingChanged'
        QtMocHelpers::SignalData<void(const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'yearChanged'
        QtMocHelpers::SignalData<void(const QString &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'trackNumberChanged'
        QtMocHelpers::SignalData<void(const QString &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'trackTotalChanged'
        QtMocHelpers::SignalData<void(const QString &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'commentChanged'
        QtMocHelpers::SignalData<void(const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'bpmChanged'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'bpmLockChanged'
        QtMocHelpers::SignalData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
        // Signal 'keyChanged'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'timesPlayedChanged'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'durationChanged'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'infoChanged'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'waveformUpdated'
        QtMocHelpers::SignalData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'waveformSummaryUpdated'
        QtMocHelpers::SignalData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'coverArtUpdated'
        QtMocHelpers::SignalData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'beatsUpdated'
        QtMocHelpers::SignalData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'replayGainUpdated'
        QtMocHelpers::SignalData<void(mixxx::ReplayGain)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Signal 'replayGainAdjusted'
        QtMocHelpers::SignalData<void(const mixxx::ReplayGain &, const QString &)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 2 }, { QMetaType::QString, 28 },
        }}),
        // Signal 'colorUpdated'
        QtMocHelpers::SignalData<void(const mixxx::RgbColor::optional_t &)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 30, 31 },
        }}),
        // Signal 'ratingUpdated'
        QtMocHelpers::SignalData<void(int)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 33 },
        }}),
        // Signal 'cuesUpdated'
        QtMocHelpers::SignalData<void()>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'loopRemove'
        QtMocHelpers::SignalData<void()>(35, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'analyzed'
        QtMocHelpers::SignalData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'changed'
        QtMocHelpers::SignalData<void(TrackId)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 38, 39 },
        }}),
        // Signal 'dirty'
        QtMocHelpers::SignalData<void(TrackId)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 38, 39 },
        }}),
        // Signal 'clean'
        QtMocHelpers::SignalData<void(TrackId)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 38, 39 },
        }}),
        // Slot 'slotCueUpdated'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'artist'
        QtMocHelpers::PropertyData<QString>(43, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'title'
        QtMocHelpers::PropertyData<QString>(44, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'album'
        QtMocHelpers::PropertyData<QString>(45, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'albumArtist'
        QtMocHelpers::PropertyData<QString>(46, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'genre'
        QtMocHelpers::PropertyData<QString>(47, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable, 4),
        // property 'composer'
        QtMocHelpers::PropertyData<QString>(48, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'grouping'
        QtMocHelpers::PropertyData<QString>(49, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'year'
        QtMocHelpers::PropertyData<QString>(50, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'trackNumber'
        QtMocHelpers::PropertyData<QString>(51, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'trackTotal'
        QtMocHelpers::PropertyData<QString>(52, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'timesPlayed'
        QtMocHelpers::PropertyData<int>(53, QMetaType::Int, QMC::DefaultPropertyFlags, 14),
        // property 'comment'
        QtMocHelpers::PropertyData<QString>(54, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 10),
        // property 'bpm'
        QtMocHelpers::PropertyData<double>(55, QMetaType::Double, QMC::DefaultPropertyFlags, 11),
        // property 'bpmText'
        QtMocHelpers::PropertyData<QString>(56, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable, 11),
        // property 'keyText'
        QtMocHelpers::PropertyData<QString>(57, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 13),
        // property 'duration'
        QtMocHelpers::PropertyData<double>(58, QMetaType::Double, QMC::DefaultPropertyFlags, 15),
        // property 'durationTextSeconds'
        QtMocHelpers::PropertyData<QString>(59, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable, 15),
        // property 'durationTextCentiseconds'
        QtMocHelpers::PropertyData<QString>(60, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable, 15),
        // property 'durationTextMilliseconds'
        QtMocHelpers::PropertyData<QString>(61, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable, 15),
        // property 'info'
        QtMocHelpers::PropertyData<QString>(62, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable, 16),
        // property 'titleInfo'
        QtMocHelpers::PropertyData<QString>(63, QMetaType::QString, QMC::Readable | QMC::Designable | QMC::Scriptable, 16),
        // property 'sourceSynchronizedAt'
        QtMocHelpers::PropertyData<QDateTime>(64, QMetaType::QDateTime, QMC::Readable | QMC::Designable | QMC::Scriptable),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Track, qt_meta_tag_ZN5TrackE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Track::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5TrackE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5TrackE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN5TrackE_t>.metaTypes,
    nullptr
} };

void Track::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Track *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->artistChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->titleChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->albumChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->albumArtistChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->genreChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->composerChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->groupingChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->yearChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->trackNumberChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->trackTotalChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->commentChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->bpmChanged(); break;
        case 12: _t->bpmLockChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->keyChanged(); break;
        case 14: _t->timesPlayedChanged(); break;
        case 15: _t->durationChanged(); break;
        case 16: _t->infoChanged(); break;
        case 17: _t->waveformUpdated(); break;
        case 18: _t->waveformSummaryUpdated(); break;
        case 19: _t->coverArtUpdated(); break;
        case 20: _t->beatsUpdated(); break;
        case 21: _t->replayGainUpdated((*reinterpret_cast<std::add_pointer_t<mixxx::ReplayGain>>(_a[1]))); break;
        case 22: _t->replayGainAdjusted((*reinterpret_cast<std::add_pointer_t<mixxx::ReplayGain>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 23: _t->colorUpdated((*reinterpret_cast<std::add_pointer_t<mixxx::RgbColor::optional_t>>(_a[1]))); break;
        case 24: _t->ratingUpdated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 25: _t->cuesUpdated(); break;
        case 26: _t->loopRemove(); break;
        case 27: _t->analyzed(); break;
        case 28: _t->changed((*reinterpret_cast<std::add_pointer_t<TrackId>>(_a[1]))); break;
        case 29: _t->dirty((*reinterpret_cast<std::add_pointer_t<TrackId>>(_a[1]))); break;
        case 30: _t->clean((*reinterpret_cast<std::add_pointer_t<TrackId>>(_a[1]))); break;
        case 31: _t->slotCueUpdated(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< mixxx::ReplayGain >(); break;
            }
            break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< mixxx::ReplayGain >(); break;
            }
            break;
        case 28:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackId >(); break;
            }
            break;
        case 29:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackId >(); break;
            }
            break;
        case 30:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackId >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Track::*)(const QString & )>(_a, &Track::artistChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Track::*)(const QString & )>(_a, &Track::titleChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Track::*)(const QString & )>(_a, &Track::albumChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Track::*)(const QString & )>(_a, &Track::albumArtistChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Track::*)(const QString & )>(_a, &Track::genreChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Track::*)(const QString & )>(_a, &Track::composerChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Track::*)(const QString & )>(_a, &Track::groupingChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Track::*)(const QString & )>(_a, &Track::yearChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Track::*)(const QString & )>(_a, &Track::trackNumberChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (Track::*)(const QString & )>(_a, &Track::trackTotalChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (Track::*)(const QString & )>(_a, &Track::commentChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (Track::*)()>(_a, &Track::bpmChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (Track::*)(bool )>(_a, &Track::bpmLockChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (Track::*)()>(_a, &Track::keyChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (Track::*)()>(_a, &Track::timesPlayedChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (Track::*)()>(_a, &Track::durationChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (Track::*)()>(_a, &Track::infoChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (Track::*)()>(_a, &Track::waveformUpdated, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (Track::*)()>(_a, &Track::waveformSummaryUpdated, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (Track::*)()>(_a, &Track::coverArtUpdated, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (Track::*)()>(_a, &Track::beatsUpdated, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (Track::*)(mixxx::ReplayGain )>(_a, &Track::replayGainUpdated, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (Track::*)(const mixxx::ReplayGain & , const QString & )>(_a, &Track::replayGainAdjusted, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (Track::*)(const mixxx::RgbColor::optional_t & )>(_a, &Track::colorUpdated, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (Track::*)(int )>(_a, &Track::ratingUpdated, 24))
            return;
        if (QtMocHelpers::indexOfMethod<void (Track::*)()>(_a, &Track::cuesUpdated, 25))
            return;
        if (QtMocHelpers::indexOfMethod<void (Track::*)()>(_a, &Track::loopRemove, 26))
            return;
        if (QtMocHelpers::indexOfMethod<void (Track::*)()>(_a, &Track::analyzed, 27))
            return;
        if (QtMocHelpers::indexOfMethod<void (Track::*)(TrackId )>(_a, &Track::changed, 28))
            return;
        if (QtMocHelpers::indexOfMethod<void (Track::*)(TrackId )>(_a, &Track::dirty, 29))
            return;
        if (QtMocHelpers::indexOfMethod<void (Track::*)(TrackId )>(_a, &Track::clean, 30))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->getArtist(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->getTitle(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->getAlbum(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->getAlbumArtist(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->getGenre(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->getComposer(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->getGrouping(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->getYear(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->getTrackNumber(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->getTrackTotal(); break;
        case 10: *reinterpret_cast<int*>(_v) = _t->getTimesPlayed(); break;
        case 11: *reinterpret_cast<QString*>(_v) = _t->getComment(); break;
        case 12: *reinterpret_cast<double*>(_v) = _t->getBpm(); break;
        case 13: *reinterpret_cast<QString*>(_v) = _t->getBpmText(); break;
        case 14: *reinterpret_cast<QString*>(_v) = _t->getKeyText(); break;
        case 15: *reinterpret_cast<double*>(_v) = _t->getDuration(); break;
        case 16: *reinterpret_cast<QString*>(_v) = _t->getDurationTextSeconds(); break;
        case 17: *reinterpret_cast<QString*>(_v) = _t->getDurationTextCentiseconds(); break;
        case 18: *reinterpret_cast<QString*>(_v) = _t->getDurationTextMilliseconds(); break;
        case 19: *reinterpret_cast<QString*>(_v) = _t->getInfo(); break;
        case 20: *reinterpret_cast<QString*>(_v) = _t->getTitleInfo(); break;
        case 21: *reinterpret_cast<QDateTime*>(_v) = _t->getSourceSynchronizedAt(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setArtist(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setTitle(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setAlbum(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setAlbumArtist(*reinterpret_cast<QString*>(_v)); break;
        case 5: _t->setComposer(*reinterpret_cast<QString*>(_v)); break;
        case 6: _t->setGrouping(*reinterpret_cast<QString*>(_v)); break;
        case 7: _t->setYear(*reinterpret_cast<QString*>(_v)); break;
        case 8: _t->setTrackNumber(*reinterpret_cast<QString*>(_v)); break;
        case 9: _t->setTrackTotal(*reinterpret_cast<QString*>(_v)); break;
        case 11: _t->setComment(*reinterpret_cast<QString*>(_v)); break;
        case 14: _t->setKeyText(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Track::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Track::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5TrackE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Track::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 32)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 32;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 32)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 32;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    return _id;
}

// SIGNAL 0
void Track::artistChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Track::titleChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Track::albumChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Track::albumArtistChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Track::genreChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void Track::composerChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void Track::groupingChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void Track::yearChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void Track::trackNumberChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void Track::trackTotalChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void Track::commentChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void Track::bpmChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void Track::bpmLockChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void Track::keyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void Track::timesPlayedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void Track::durationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void Track::infoChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void Track::waveformUpdated()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void Track::waveformSummaryUpdated()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void Track::coverArtUpdated()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void Track::beatsUpdated()
{
    QMetaObject::activate(this, &staticMetaObject, 20, nullptr);
}

// SIGNAL 21
void Track::replayGainUpdated(mixxx::ReplayGain _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1);
}

// SIGNAL 22
void Track::replayGainAdjusted(const mixxx::ReplayGain & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 22, nullptr, _t1, _t2);
}

// SIGNAL 23
void Track::colorUpdated(const mixxx::RgbColor::optional_t & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 23, nullptr, _t1);
}

// SIGNAL 24
void Track::ratingUpdated(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 24, nullptr, _t1);
}

// SIGNAL 25
void Track::cuesUpdated()
{
    QMetaObject::activate(this, &staticMetaObject, 25, nullptr);
}

// SIGNAL 26
void Track::loopRemove()
{
    QMetaObject::activate(this, &staticMetaObject, 26, nullptr);
}

// SIGNAL 27
void Track::analyzed()
{
    QMetaObject::activate(this, &staticMetaObject, 27, nullptr);
}

// SIGNAL 28
void Track::changed(TrackId _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 28, nullptr, _t1);
}

// SIGNAL 29
void Track::dirty(TrackId _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 29, nullptr, _t1);
}

// SIGNAL 30
void Track::clean(TrackId _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 30, nullptr, _t1);
}
QT_WARNING_POP
