/********************************************************************************
** Form generated from reading UI file 'dlgprefdeckdlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLGPREFDECKDLG_H
#define UI_DLGPREFDECKDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DlgPrefDeckDlg
{
public:
    QGridLayout *gridLayout_3;
    QGroupBox *groupBoxDeckOptions;
    QGridLayout *GridLayout1;
    QLabel *labelLoadWhenDeckPlaying;
    QLabel *labelCueMode;
    QComboBox *comboBoxLoadPoint;
    QCheckBox *checkBoxIntroStartMove;
    QLabel *labelIntroStartMove;
    QComboBox *ComboBoxCueMode;
    QLabel *labelCloneDeckOnLoadDoubleTap;
    QLabel *labelTimeDisplay;
    QComboBox *comboBoxTimeFormat;
    QLabel *labelLoadPoint;
    QLabel *labelPositionDisplay;
    QComboBox *comboBoxLoadWhenDeckPlaying;
    QHBoxLayout *horizontalLayout;
    QRadioButton *radioButtonElapsed;
    QRadioButton *radioButtonRemaining;
    QRadioButton *radioButtonElapsedAndRemaining;
    QCheckBox *checkBoxCloneDeckOnLoadDoubleTap;
    QGroupBox *groupBoxSpeedPitchOptions;
    QGridLayout *gridLayout_7;
    QGridLayout *GridLayoutSpeedOptions;
    QDoubleSpinBox *spinBoxPermanentRateCoarse;
    QDoubleSpinBox *spinBoxPermanentRateFine;
    QLabel *labelSpeedPitchReset;
    QRadioButton *radioButtonCurrentKey;
    QDoubleSpinBox *spinBoxTemporaryRateFine;
    QLabel *labelSpeedPermanent;
    QSlider *SliderRateRampSensitivity;
    QLabel *labelSpeedTemporary;
    QLabel *labelSyncMode;
    QLabel *labelKeylockMode;
    QLabel *labelSpeedRampSensitivity;
    QLabel *labelSpeedBendBehavior;
    QRadioButton *radioButtonOriginalKey;
    QDoubleSpinBox *spinBoxTemporaryRateCoarse;
    QCheckBox *checkBoxResetSpeed;
    QCheckBox *checkBoxResetPitch;
    QLabel *labelSpeedAdjustment;
    QRadioButton *radioButtonSoftLeader;
    QLabel *labelSpeedCoarse;
    QLabel *labelSpeedFine;
    QCheckBox *checkBoxInvertSpeedSlider;
    QLabel *labelSpeedSliderrange;
    QComboBox *ComboBoxRateRange;
    QRadioButton *radioButtonRateRampModeStepping;
    QRadioButton *radioButtonRateRampModeLinear;
    QLabel *labelKeyunlockMode;
    QRadioButton *radioButtonResetUnlockedKey;
    QRadioButton *radioButtonKeepUnlockedKey;
    QRadioButton *radioButtonLockBpm;
    QSpacerItem *verticalSpacer1;
    QSpacerItem *verticalSpacer2;
    QButtonGroup *buttonGroupTrackTime;
    QButtonGroup *buttonGroupKeyUnlockMode;
    QButtonGroup *buttonGroupKeyLockMode;
    QButtonGroup *buttonGroupSpeedBendBehavior;
    QButtonGroup *buttonGroupSyncMode;

    void setupUi(QWidget *DlgPrefDeckDlg)
    {
        if (DlgPrefDeckDlg->objectName().isEmpty())
            DlgPrefDeckDlg->setObjectName("DlgPrefDeckDlg");
        DlgPrefDeckDlg->resize(710, 723);
        gridLayout_3 = new QGridLayout(DlgPrefDeckDlg);
        gridLayout_3->setSpacing(6);
        gridLayout_3->setContentsMargins(11, 11, 11, 11);
        gridLayout_3->setObjectName("gridLayout_3");
        groupBoxDeckOptions = new QGroupBox(DlgPrefDeckDlg);
        groupBoxDeckOptions->setObjectName("groupBoxDeckOptions");
        GridLayout1 = new QGridLayout(groupBoxDeckOptions);
        GridLayout1->setSpacing(10);
        GridLayout1->setContentsMargins(11, 11, 11, 11);
        GridLayout1->setObjectName("GridLayout1");
        labelLoadWhenDeckPlaying = new QLabel(groupBoxDeckOptions);
        labelLoadWhenDeckPlaying->setObjectName("labelLoadWhenDeckPlaying");

        GridLayout1->addWidget(labelLoadWhenDeckPlaying, 5, 0, 1, 1);

        labelCueMode = new QLabel(groupBoxDeckOptions);
        labelCueMode->setObjectName("labelCueMode");
        labelCueMode->setOpenExternalLinks(true);

        GridLayout1->addWidget(labelCueMode, 0, 0, 1, 1);

        comboBoxLoadPoint = new QComboBox(groupBoxDeckOptions);
        comboBoxLoadPoint->setObjectName("comboBoxLoadPoint");

        GridLayout1->addWidget(comboBoxLoadPoint, 4, 1, 1, 2);

        checkBoxIntroStartMove = new QCheckBox(groupBoxDeckOptions);
        checkBoxIntroStartMove->setObjectName("checkBoxIntroStartMove");

        GridLayout1->addWidget(checkBoxIntroStartMove, 1, 1, 1, 2);

        labelIntroStartMove = new QLabel(groupBoxDeckOptions);
        labelIntroStartMove->setObjectName("labelIntroStartMove");

        GridLayout1->addWidget(labelIntroStartMove, 1, 0, 1, 1);

        ComboBoxCueMode = new QComboBox(groupBoxDeckOptions);
        ComboBoxCueMode->setObjectName("ComboBoxCueMode");

        GridLayout1->addWidget(ComboBoxCueMode, 0, 1, 1, 2);

        labelCloneDeckOnLoadDoubleTap = new QLabel(groupBoxDeckOptions);
        labelCloneDeckOnLoadDoubleTap->setObjectName("labelCloneDeckOnLoadDoubleTap");

        GridLayout1->addWidget(labelCloneDeckOnLoadDoubleTap, 6, 0, 1, 1);

        labelTimeDisplay = new QLabel(groupBoxDeckOptions);
        labelTimeDisplay->setObjectName("labelTimeDisplay");

        GridLayout1->addWidget(labelTimeDisplay, 3, 0, 1, 1);

        comboBoxTimeFormat = new QComboBox(groupBoxDeckOptions);
        comboBoxTimeFormat->setObjectName("comboBoxTimeFormat");

        GridLayout1->addWidget(comboBoxTimeFormat, 3, 1, 1, 1);

        labelLoadPoint = new QLabel(groupBoxDeckOptions);
        labelLoadPoint->setObjectName("labelLoadPoint");

        GridLayout1->addWidget(labelLoadPoint, 4, 0, 1, 1);

        labelPositionDisplay = new QLabel(groupBoxDeckOptions);
        labelPositionDisplay->setObjectName("labelPositionDisplay");
        labelPositionDisplay->setEnabled(true);
        QFont font;
        labelPositionDisplay->setFont(font);
        labelPositionDisplay->setWordWrap(false);

        GridLayout1->addWidget(labelPositionDisplay, 2, 0, 1, 1);

        comboBoxLoadWhenDeckPlaying = new QComboBox(groupBoxDeckOptions);
        comboBoxLoadWhenDeckPlaying->setObjectName("comboBoxLoadWhenDeckPlaying");

        GridLayout1->addWidget(comboBoxLoadWhenDeckPlaying, 5, 1, 1, 2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        radioButtonElapsed = new QRadioButton(groupBoxDeckOptions);
        buttonGroupTrackTime = new QButtonGroup(DlgPrefDeckDlg);
        buttonGroupTrackTime->setObjectName("buttonGroupTrackTime");
        buttonGroupTrackTime->addButton(radioButtonElapsed);
        radioButtonElapsed->setObjectName("radioButtonElapsed");

        horizontalLayout->addWidget(radioButtonElapsed);

        radioButtonRemaining = new QRadioButton(groupBoxDeckOptions);
        buttonGroupTrackTime->addButton(radioButtonRemaining);
        radioButtonRemaining->setObjectName("radioButtonRemaining");

        horizontalLayout->addWidget(radioButtonRemaining);

        radioButtonElapsedAndRemaining = new QRadioButton(groupBoxDeckOptions);
        buttonGroupTrackTime->addButton(radioButtonElapsedAndRemaining);
        radioButtonElapsedAndRemaining->setObjectName("radioButtonElapsedAndRemaining");

        horizontalLayout->addWidget(radioButtonElapsedAndRemaining);


        GridLayout1->addLayout(horizontalLayout, 2, 1, 1, 2);

        checkBoxCloneDeckOnLoadDoubleTap = new QCheckBox(groupBoxDeckOptions);
        checkBoxCloneDeckOnLoadDoubleTap->setObjectName("checkBoxCloneDeckOnLoadDoubleTap");

        GridLayout1->addWidget(checkBoxCloneDeckOnLoadDoubleTap, 6, 1, 1, 2);


        gridLayout_3->addWidget(groupBoxDeckOptions, 0, 0, 1, 1);

        groupBoxSpeedPitchOptions = new QGroupBox(DlgPrefDeckDlg);
        groupBoxSpeedPitchOptions->setObjectName("groupBoxSpeedPitchOptions");
        gridLayout_7 = new QGridLayout(groupBoxSpeedPitchOptions);
        gridLayout_7->setSpacing(6);
        gridLayout_7->setContentsMargins(11, 11, 11, 11);
        gridLayout_7->setObjectName("gridLayout_7");
        GridLayoutSpeedOptions = new QGridLayout();
        GridLayoutSpeedOptions->setSpacing(6);
        GridLayoutSpeedOptions->setObjectName("GridLayoutSpeedOptions");
        spinBoxPermanentRateCoarse = new QDoubleSpinBox(groupBoxSpeedPitchOptions);
        spinBoxPermanentRateCoarse->setObjectName("spinBoxPermanentRateCoarse");
        spinBoxPermanentRateCoarse->setAccelerated(true);
        spinBoxPermanentRateCoarse->setDecimals(2);
        spinBoxPermanentRateCoarse->setMinimum(0.010000000000000);
        spinBoxPermanentRateCoarse->setMaximum(10.000000000000000);
        spinBoxPermanentRateCoarse->setSingleStep(0.010000000000000);
        spinBoxPermanentRateCoarse->setValue(0.500000000000000);

        GridLayoutSpeedOptions->addWidget(spinBoxPermanentRateCoarse, 11, 2, 1, 1);

        spinBoxPermanentRateFine = new QDoubleSpinBox(groupBoxSpeedPitchOptions);
        spinBoxPermanentRateFine->setObjectName("spinBoxPermanentRateFine");
        spinBoxPermanentRateFine->setAccelerated(true);
        spinBoxPermanentRateFine->setDecimals(2);
        spinBoxPermanentRateFine->setMinimum(0.010000000000000);
        spinBoxPermanentRateFine->setMaximum(10.000000000000000);
        spinBoxPermanentRateFine->setSingleStep(0.010000000000000);
        spinBoxPermanentRateFine->setValue(0.050000000000000);

        GridLayoutSpeedOptions->addWidget(spinBoxPermanentRateFine, 11, 1, 1, 1);

        labelSpeedPitchReset = new QLabel(groupBoxSpeedPitchOptions);
        labelSpeedPitchReset->setObjectName("labelSpeedPitchReset");

        GridLayoutSpeedOptions->addWidget(labelSpeedPitchReset, 1, 0, 1, 1);

        radioButtonCurrentKey = new QRadioButton(groupBoxSpeedPitchOptions);
        buttonGroupKeyLockMode = new QButtonGroup(DlgPrefDeckDlg);
        buttonGroupKeyLockMode->setObjectName("buttonGroupKeyLockMode");
        buttonGroupKeyLockMode->addButton(radioButtonCurrentKey);
        radioButtonCurrentKey->setObjectName("radioButtonCurrentKey");

        GridLayoutSpeedOptions->addWidget(radioButtonCurrentKey, 3, 2, 1, 1);

        spinBoxTemporaryRateFine = new QDoubleSpinBox(groupBoxSpeedPitchOptions);
        spinBoxTemporaryRateFine->setObjectName("spinBoxTemporaryRateFine");
        spinBoxTemporaryRateFine->setEnabled(false);
        spinBoxTemporaryRateFine->setAccelerated(true);
        spinBoxTemporaryRateFine->setDecimals(2);
        spinBoxTemporaryRateFine->setMinimum(0.010000000000000);
        spinBoxTemporaryRateFine->setMaximum(10.000000000000000);
        spinBoxTemporaryRateFine->setSingleStep(0.010000000000000);
        spinBoxTemporaryRateFine->setValue(1.000000000000000);

        GridLayoutSpeedOptions->addWidget(spinBoxTemporaryRateFine, 10, 1, 1, 1);

        labelSpeedPermanent = new QLabel(groupBoxSpeedPitchOptions);
        labelSpeedPermanent->setObjectName("labelSpeedPermanent");
        labelSpeedPermanent->setAlignment(Qt::AlignCenter);

        GridLayoutSpeedOptions->addWidget(labelSpeedPermanent, 11, 0, 1, 1);

        SliderRateRampSensitivity = new QSlider(groupBoxSpeedPitchOptions);
        SliderRateRampSensitivity->setObjectName("SliderRateRampSensitivity");
        SliderRateRampSensitivity->setEnabled(false);
        SliderRateRampSensitivity->setMinimum(100);
        SliderRateRampSensitivity->setMaximum(2500);
        SliderRateRampSensitivity->setSingleStep(50);
        SliderRateRampSensitivity->setValue(250);
        SliderRateRampSensitivity->setOrientation(Qt::Horizontal);

        GridLayoutSpeedOptions->addWidget(SliderRateRampSensitivity, 7, 1, 1, 2);

        labelSpeedTemporary = new QLabel(groupBoxSpeedPitchOptions);
        labelSpeedTemporary->setObjectName("labelSpeedTemporary");
        labelSpeedTemporary->setEnabled(false);
        labelSpeedTemporary->setAlignment(Qt::AlignCenter);

        GridLayoutSpeedOptions->addWidget(labelSpeedTemporary, 10, 0, 1, 1);

        labelSyncMode = new QLabel(groupBoxSpeedPitchOptions);
        labelSyncMode->setObjectName("labelSyncMode");
        labelSyncMode->setOpenExternalLinks(true);

        GridLayoutSpeedOptions->addWidget(labelSyncMode, 2, 0, 1, 1);

        labelKeylockMode = new QLabel(groupBoxSpeedPitchOptions);
        labelKeylockMode->setObjectName("labelKeylockMode");

        GridLayoutSpeedOptions->addWidget(labelKeylockMode, 3, 0, 1, 1);

        labelSpeedRampSensitivity = new QLabel(groupBoxSpeedPitchOptions);
        labelSpeedRampSensitivity->setObjectName("labelSpeedRampSensitivity");
        labelSpeedRampSensitivity->setEnabled(false);

        GridLayoutSpeedOptions->addWidget(labelSpeedRampSensitivity, 7, 0, 1, 1);

        labelSpeedBendBehavior = new QLabel(groupBoxSpeedPitchOptions);
        labelSpeedBendBehavior->setObjectName("labelSpeedBendBehavior");

        GridLayoutSpeedOptions->addWidget(labelSpeedBendBehavior, 6, 0, 1, 1);

        radioButtonOriginalKey = new QRadioButton(groupBoxSpeedPitchOptions);
        buttonGroupKeyLockMode->addButton(radioButtonOriginalKey);
        radioButtonOriginalKey->setObjectName("radioButtonOriginalKey");

        GridLayoutSpeedOptions->addWidget(radioButtonOriginalKey, 3, 1, 1, 1);

        spinBoxTemporaryRateCoarse = new QDoubleSpinBox(groupBoxSpeedPitchOptions);
        spinBoxTemporaryRateCoarse->setObjectName("spinBoxTemporaryRateCoarse");
        spinBoxTemporaryRateCoarse->setEnabled(false);
        spinBoxTemporaryRateCoarse->setAccelerated(true);
        spinBoxTemporaryRateCoarse->setDecimals(2);
        spinBoxTemporaryRateCoarse->setMinimum(0.010000000000000);
        spinBoxTemporaryRateCoarse->setMaximum(10.000000000000000);
        spinBoxTemporaryRateCoarse->setSingleStep(0.010000000000000);
        spinBoxTemporaryRateCoarse->setValue(4.000000000000000);

        GridLayoutSpeedOptions->addWidget(spinBoxTemporaryRateCoarse, 10, 2, 1, 1);

        checkBoxResetSpeed = new QCheckBox(groupBoxSpeedPitchOptions);
        checkBoxResetSpeed->setObjectName("checkBoxResetSpeed");

        GridLayoutSpeedOptions->addWidget(checkBoxResetSpeed, 1, 2, 1, 1);

        checkBoxResetPitch = new QCheckBox(groupBoxSpeedPitchOptions);
        checkBoxResetPitch->setObjectName("checkBoxResetPitch");

        GridLayoutSpeedOptions->addWidget(checkBoxResetPitch, 1, 1, 1, 1);

        labelSpeedAdjustment = new QLabel(groupBoxSpeedPitchOptions);
        labelSpeedAdjustment->setObjectName("labelSpeedAdjustment");

        GridLayoutSpeedOptions->addWidget(labelSpeedAdjustment, 9, 0, 1, 1);

        radioButtonSoftLeader = new QRadioButton(groupBoxSpeedPitchOptions);
        buttonGroupSyncMode = new QButtonGroup(DlgPrefDeckDlg);
        buttonGroupSyncMode->setObjectName("buttonGroupSyncMode");
        buttonGroupSyncMode->addButton(radioButtonSoftLeader);
        radioButtonSoftLeader->setObjectName("radioButtonSoftLeader");

        GridLayoutSpeedOptions->addWidget(radioButtonSoftLeader, 2, 1, 1, 1);

        labelSpeedCoarse = new QLabel(groupBoxSpeedPitchOptions);
        labelSpeedCoarse->setObjectName("labelSpeedCoarse");
        labelSpeedCoarse->setEnabled(true);
        labelSpeedCoarse->setFont(font);
        labelSpeedCoarse->setAlignment(Qt::AlignCenter);
        labelSpeedCoarse->setWordWrap(false);

        GridLayoutSpeedOptions->addWidget(labelSpeedCoarse, 9, 2, 1, 1);

        labelSpeedFine = new QLabel(groupBoxSpeedPitchOptions);
        labelSpeedFine->setObjectName("labelSpeedFine");
        labelSpeedFine->setEnabled(true);
        labelSpeedFine->setFont(font);
        labelSpeedFine->setAlignment(Qt::AlignCenter);
        labelSpeedFine->setWordWrap(false);

        GridLayoutSpeedOptions->addWidget(labelSpeedFine, 9, 1, 1, 1);

        checkBoxInvertSpeedSlider = new QCheckBox(groupBoxSpeedPitchOptions);
        checkBoxInvertSpeedSlider->setObjectName("checkBoxInvertSpeedSlider");

        GridLayoutSpeedOptions->addWidget(checkBoxInvertSpeedSlider, 0, 2, 1, 1);

        labelSpeedSliderrange = new QLabel(groupBoxSpeedPitchOptions);
        labelSpeedSliderrange->setObjectName("labelSpeedSliderrange");
        labelSpeedSliderrange->setEnabled(true);
        labelSpeedSliderrange->setFont(font);
        labelSpeedSliderrange->setWordWrap(false);

        GridLayoutSpeedOptions->addWidget(labelSpeedSliderrange, 0, 0, 1, 1);

        ComboBoxRateRange = new QComboBox(groupBoxSpeedPitchOptions);
        ComboBoxRateRange->setObjectName("ComboBoxRateRange");
        ComboBoxRateRange->setFont(font);

        GridLayoutSpeedOptions->addWidget(ComboBoxRateRange, 0, 1, 1, 1);

        radioButtonRateRampModeStepping = new QRadioButton(groupBoxSpeedPitchOptions);
        buttonGroupSpeedBendBehavior = new QButtonGroup(DlgPrefDeckDlg);
        buttonGroupSpeedBendBehavior->setObjectName("buttonGroupSpeedBendBehavior");
        buttonGroupSpeedBendBehavior->addButton(radioButtonRateRampModeStepping);
        radioButtonRateRampModeStepping->setObjectName("radioButtonRateRampModeStepping");

        GridLayoutSpeedOptions->addWidget(radioButtonRateRampModeStepping, 6, 2, 1, 1);

        radioButtonRateRampModeLinear = new QRadioButton(groupBoxSpeedPitchOptions);
        buttonGroupSpeedBendBehavior->addButton(radioButtonRateRampModeLinear);
        radioButtonRateRampModeLinear->setObjectName("radioButtonRateRampModeLinear");

        GridLayoutSpeedOptions->addWidget(radioButtonRateRampModeLinear, 6, 1, 1, 1);

        labelKeyunlockMode = new QLabel(groupBoxSpeedPitchOptions);
        labelKeyunlockMode->setObjectName("labelKeyunlockMode");

        GridLayoutSpeedOptions->addWidget(labelKeyunlockMode, 4, 0, 1, 1);

        radioButtonResetUnlockedKey = new QRadioButton(groupBoxSpeedPitchOptions);
        buttonGroupKeyUnlockMode = new QButtonGroup(DlgPrefDeckDlg);
        buttonGroupKeyUnlockMode->setObjectName("buttonGroupKeyUnlockMode");
        buttonGroupKeyUnlockMode->addButton(radioButtonResetUnlockedKey);
        radioButtonResetUnlockedKey->setObjectName("radioButtonResetUnlockedKey");

        GridLayoutSpeedOptions->addWidget(radioButtonResetUnlockedKey, 4, 1, 1, 1);

        radioButtonKeepUnlockedKey = new QRadioButton(groupBoxSpeedPitchOptions);
        buttonGroupKeyUnlockMode->addButton(radioButtonKeepUnlockedKey);
        radioButtonKeepUnlockedKey->setObjectName("radioButtonKeepUnlockedKey");

        GridLayoutSpeedOptions->addWidget(radioButtonKeepUnlockedKey, 4, 2, 1, 1);

        radioButtonLockBpm = new QRadioButton(groupBoxSpeedPitchOptions);
        buttonGroupSyncMode->addButton(radioButtonLockBpm);
        radioButtonLockBpm->setObjectName("radioButtonLockBpm");

        GridLayoutSpeedOptions->addWidget(radioButtonLockBpm, 2, 2, 1, 1);

        verticalSpacer1 = new QSpacerItem(20, 15, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        GridLayoutSpeedOptions->addItem(verticalSpacer1, 5, 0, 1, 1);


        gridLayout_7->addLayout(GridLayoutSpeedOptions, 0, 0, 1, 1);


        gridLayout_3->addWidget(groupBoxSpeedPitchOptions, 1, 0, 1, 1);

        verticalSpacer2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_3->addItem(verticalSpacer2, 2, 0, 1, 1);

#if QT_CONFIG(shortcut)
        labelLoadWhenDeckPlaying->setBuddy(comboBoxLoadWhenDeckPlaying);
        labelCueMode->setBuddy(ComboBoxCueMode);
        labelIntroStartMove->setBuddy(ComboBoxCueMode);
        labelCloneDeckOnLoadDoubleTap->setBuddy(checkBoxCloneDeckOnLoadDoubleTap);
        labelPositionDisplay->setBuddy(radioButtonElapsed);
        labelSpeedPitchReset->setBuddy(checkBoxResetPitch);
        labelKeylockMode->setBuddy(radioButtonOriginalKey);
        labelSpeedRampSensitivity->setBuddy(SliderRateRampSensitivity);
        labelSpeedBendBehavior->setBuddy(radioButtonRateRampModeStepping);
        labelSpeedCoarse->setBuddy(spinBoxPermanentRateCoarse);
        labelSpeedFine->setBuddy(spinBoxPermanentRateFine);
        labelSpeedSliderrange->setBuddy(ComboBoxRateRange);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(ComboBoxCueMode, checkBoxIntroStartMove);
        QWidget::setTabOrder(checkBoxIntroStartMove, radioButtonElapsed);
        QWidget::setTabOrder(radioButtonElapsed, radioButtonRemaining);
        QWidget::setTabOrder(radioButtonRemaining, radioButtonElapsedAndRemaining);
        QWidget::setTabOrder(radioButtonElapsedAndRemaining, comboBoxTimeFormat);
        QWidget::setTabOrder(comboBoxTimeFormat, comboBoxLoadPoint);
        QWidget::setTabOrder(comboBoxLoadPoint, comboBoxLoadWhenDeckPlaying);
        QWidget::setTabOrder(comboBoxLoadWhenDeckPlaying, checkBoxCloneDeckOnLoadDoubleTap);
        QWidget::setTabOrder(checkBoxCloneDeckOnLoadDoubleTap, ComboBoxRateRange);
        QWidget::setTabOrder(ComboBoxRateRange, checkBoxInvertSpeedSlider);
        QWidget::setTabOrder(checkBoxInvertSpeedSlider, checkBoxResetPitch);
        QWidget::setTabOrder(checkBoxResetPitch, checkBoxResetSpeed);
        QWidget::setTabOrder(checkBoxResetSpeed, radioButtonSoftLeader);
        QWidget::setTabOrder(radioButtonSoftLeader, radioButtonOriginalKey);
        QWidget::setTabOrder(radioButtonOriginalKey, radioButtonCurrentKey);
        QWidget::setTabOrder(radioButtonCurrentKey, radioButtonResetUnlockedKey);
        QWidget::setTabOrder(radioButtonResetUnlockedKey, radioButtonKeepUnlockedKey);
        QWidget::setTabOrder(radioButtonKeepUnlockedKey, radioButtonRateRampModeLinear);
        QWidget::setTabOrder(radioButtonRateRampModeLinear, radioButtonRateRampModeStepping);
        QWidget::setTabOrder(radioButtonRateRampModeStepping, SliderRateRampSensitivity);
        QWidget::setTabOrder(SliderRateRampSensitivity, spinBoxTemporaryRateFine);
        QWidget::setTabOrder(spinBoxTemporaryRateFine, spinBoxTemporaryRateCoarse);
        QWidget::setTabOrder(spinBoxTemporaryRateCoarse, spinBoxPermanentRateFine);
        QWidget::setTabOrder(spinBoxPermanentRateFine, spinBoxPermanentRateCoarse);

        retranslateUi(DlgPrefDeckDlg);

        QMetaObject::connectSlotsByName(DlgPrefDeckDlg);
    } // setupUi

    void retranslateUi(QWidget *DlgPrefDeckDlg)
    {
        DlgPrefDeckDlg->setWindowTitle(QCoreApplication::translate("DlgPrefDeckDlg", "Deck Preferences", nullptr));
        groupBoxDeckOptions->setTitle(QCoreApplication::translate("DlgPrefDeckDlg", "Deck options", nullptr));
        labelLoadWhenDeckPlaying->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Loading a track, when deck is playing", nullptr));
        labelCueMode->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Cue mode", nullptr));
#if QT_CONFIG(tooltip)
        checkBoxIntroStartMove->setToolTip(QCoreApplication::translate("DlgPrefDeckDlg", "When the analyzer places the intro start point automatically,\n"
"it will place it at the main cue point if the main cue point has been set previously.\n"
"This may be helpful for upgrading to Mixxx 2.3 from earlier versions.\n"
"\n"
"If this option is disabled, the intro start point is automatically placed at the first sound.", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBoxIntroStartMove->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Set intro start to main cue when analyzing tracks", nullptr));
        labelIntroStartMove->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Intro start", nullptr));
#if QT_CONFIG(tooltip)
        ComboBoxCueMode->setToolTip(QCoreApplication::translate("DlgPrefDeckDlg", "Mixxx mode:\n"
"- Cue button while pause at cue point = preview\n"
"- Cue button while pause not at cue point = set cue point\n"
"- Cue button while playing = pause at cue point\n"
"Mixxx mode (no blinking):\n"
"- Same as Mixxx mode but with no blinking indicators\n"
"Pioneer mode:\n"
"- Same as Mixxx mode with a flashing play button\n"
"Denon mode:\n"
"- Cue button at cue point = preview\n"
"- Cue button not at cue point = pause at cue point\n"
"- Play = set cue point\n"
"Numark mode:\n"
"- Same as Denon mode, but without a flashing play button\n"
"CUP mode:\n"
"- Cue button while pause at cue point = play after release\n"
"- Cue button while pause not at cue point = set cue point and play after release\n"
"- Cue button while playing = go to cue point and play after release\n"
"", nullptr));
#endif // QT_CONFIG(tooltip)
        labelCloneDeckOnLoadDoubleTap->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Clone deck", nullptr));
        labelTimeDisplay->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Time Format", nullptr));
        labelLoadPoint->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Track load point", nullptr));
        labelPositionDisplay->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Track time display", nullptr));
        radioButtonElapsed->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Elapsed", nullptr));
        radioButtonRemaining->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Remaining", nullptr));
        radioButtonElapsedAndRemaining->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Elapsed and Remaining", nullptr));
#if QT_CONFIG(tooltip)
        checkBoxCloneDeckOnLoadDoubleTap->setToolTip(QCoreApplication::translate("DlgPrefDeckDlg", "Create a playing clone of the first playing deck by double-tapping a Load button on a controller or keyboard.\n"
"You can always drag-and-drop tracks on screen to clone a deck.", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBoxCloneDeckOnLoadDoubleTap->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Double-press Load button to clone playing track", nullptr));
        groupBoxSpeedPitchOptions->setTitle(QCoreApplication::translate("DlgPrefDeckDlg", "Speed (Tempo) and Key (Pitch) options", nullptr));
#if QT_CONFIG(tooltip)
        spinBoxPermanentRateCoarse->setToolTip(QCoreApplication::translate("DlgPrefDeckDlg", "Permanent rate change when left-clicking", nullptr));
#endif // QT_CONFIG(tooltip)
        spinBoxPermanentRateCoarse->setSuffix(QCoreApplication::translate("DlgPrefDeckDlg", "%", nullptr));
#if QT_CONFIG(tooltip)
        spinBoxPermanentRateFine->setToolTip(QCoreApplication::translate("DlgPrefDeckDlg", "Permanent rate change when right-clicking", nullptr));
#endif // QT_CONFIG(tooltip)
        spinBoxPermanentRateFine->setSuffix(QCoreApplication::translate("DlgPrefDeckDlg", "%", nullptr));
        labelSpeedPitchReset->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Reset on track load", nullptr));
        radioButtonCurrentKey->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Current key", nullptr));
#if QT_CONFIG(tooltip)
        spinBoxTemporaryRateFine->setToolTip(QCoreApplication::translate("DlgPrefDeckDlg", "Temporary rate change when right-clicking", nullptr));
#endif // QT_CONFIG(tooltip)
        spinBoxTemporaryRateFine->setSuffix(QCoreApplication::translate("DlgPrefDeckDlg", "%", nullptr));
        labelSpeedPermanent->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Permanent", nullptr));
        labelSpeedTemporary->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Temporary", nullptr));
        labelSyncMode->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Sync mode (Dynamic tempo tracks)", nullptr));
        labelKeylockMode->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Keylock mode", nullptr));
        labelSpeedRampSensitivity->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Ramping sensitivity", nullptr));
        labelSpeedBendBehavior->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Pitch bend behavior", nullptr));
        radioButtonOriginalKey->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Original key", nullptr));
#if QT_CONFIG(tooltip)
        spinBoxTemporaryRateCoarse->setToolTip(QCoreApplication::translate("DlgPrefDeckDlg", "Temporary rate change when left-clicking", nullptr));
#endif // QT_CONFIG(tooltip)
        spinBoxTemporaryRateCoarse->setSuffix(QCoreApplication::translate("DlgPrefDeckDlg", "%", nullptr));
        checkBoxResetSpeed->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Speed/Tempo", nullptr));
        checkBoxResetPitch->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Key/Pitch", nullptr));
        labelSpeedAdjustment->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Adjustment buttons:", nullptr));
#if QT_CONFIG(tooltip)
        radioButtonSoftLeader->setToolTip(QCoreApplication::translate("DlgPrefDeckDlg", "Apply tempo changes from a soft-leading track (usually the leaving track in a transition) to the follower tracks. After the transition, the follower track will continue with the previous leader's very last tempo. Changes from explicit selected leaders are always applied.", nullptr));
#endif // QT_CONFIG(tooltip)
        radioButtonSoftLeader->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Follow soft leader's tempo", nullptr));
#if QT_CONFIG(tooltip)
        labelSpeedCoarse->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        labelSpeedCoarse->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Coarse", nullptr));
#if QT_CONFIG(tooltip)
        labelSpeedFine->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        labelSpeedFine->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Fine", nullptr));
#if QT_CONFIG(tooltip)
        checkBoxInvertSpeedSlider->setToolTip(QCoreApplication::translate("DlgPrefDeckDlg", "Make the speed sliders work like those on DJ turntables and CDJs where moving downward increases the speed", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBoxInvertSpeedSlider->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Down increases speed", nullptr));
        labelSpeedSliderrange->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Slider range", nullptr));
#if QT_CONFIG(tooltip)
        ComboBoxRateRange->setToolTip(QCoreApplication::translate("DlgPrefDeckDlg", "Adjusts the range of the speed (Vinyl \"Pitch\") slider.", nullptr));
#endif // QT_CONFIG(tooltip)
        radioButtonRateRampModeStepping->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Abrupt jump", nullptr));
#if QT_CONFIG(tooltip)
        radioButtonRateRampModeLinear->setToolTip(QCoreApplication::translate("DlgPrefDeckDlg", "Smoothly adjusts deck speed when temporary change buttons are held down", nullptr));
#endif // QT_CONFIG(tooltip)
        radioButtonRateRampModeLinear->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Smooth ramping", nullptr));
        labelKeyunlockMode->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Keyunlock mode", nullptr));
        radioButtonResetUnlockedKey->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Reset key", nullptr));
        radioButtonKeepUnlockedKey->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Keep key", nullptr));
#if QT_CONFIG(tooltip)
        radioButtonLockBpm->setToolTip(QCoreApplication::translate("DlgPrefDeckDlg", "The tempo of a previous soft leader track at the beginning of the transition is kept steady. After the transition, the follower track will maintain this original tempo. This technique serves as a workaround to avoid dynamic tempo changes, as seen during the outro of rubato-style tracks. For instance, it prevents the follower track from continuing with a slowed-down tempo of the soft leader. This corresponds to the behavior before Mixxx 2.4. Changes from explicit selected leaders are always applied.", nullptr));
#endif // QT_CONFIG(tooltip)
        radioButtonLockBpm->setText(QCoreApplication::translate("DlgPrefDeckDlg", "Use steady tempo", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgPrefDeckDlg: public Ui_DlgPrefDeckDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLGPREFDECKDLG_H
