/********************************************************************************
** Form generated from reading UI file 'dlgprefbeatsdlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLGPREFBEATSDLG_H
#define UI_DLGPREFBEATSDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DlgBeatsDlg
{
public:
    QGridLayout *gridLayout;
    QCheckBox *checkBoxAnalyzerEnabled;
    QSpacerItem *verticalSpacer;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout;
    QCheckBox *checkBoxFixedTempo;
    QCheckBox *checkBoxFastAnalysis;
    QCheckBox *checkBoxReanalyze;
    QCheckBox *checkBoxReanalyzeImported;
    QGroupBox *groupBox;
    QHBoxLayout *horizontalLayout;
    QComboBox *comboBoxBeatPlugin;

    void setupUi(QWidget *DlgBeatsDlg)
    {
        if (DlgBeatsDlg->objectName().isEmpty())
            DlgBeatsDlg->setObjectName("DlgBeatsDlg");
        DlgBeatsDlg->resize(576, 400);
        gridLayout = new QGridLayout(DlgBeatsDlg);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        checkBoxAnalyzerEnabled = new QCheckBox(DlgBeatsDlg);
        checkBoxAnalyzerEnabled->setObjectName("checkBoxAnalyzerEnabled");

        gridLayout->addWidget(checkBoxAnalyzerEnabled, 0, 0, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 5, 0, 1, 1);

        groupBox_2 = new QGroupBox(DlgBeatsDlg);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout = new QVBoxLayout(groupBox_2);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        checkBoxFixedTempo = new QCheckBox(groupBox_2);
        checkBoxFixedTempo->setObjectName("checkBoxFixedTempo");

        verticalLayout->addWidget(checkBoxFixedTempo);

        checkBoxFastAnalysis = new QCheckBox(groupBox_2);
        checkBoxFastAnalysis->setObjectName("checkBoxFastAnalysis");

        verticalLayout->addWidget(checkBoxFastAnalysis);

        checkBoxReanalyze = new QCheckBox(groupBox_2);
        checkBoxReanalyze->setObjectName("checkBoxReanalyze");

        verticalLayout->addWidget(checkBoxReanalyze);

        checkBoxReanalyzeImported = new QCheckBox(groupBox_2);
        checkBoxReanalyzeImported->setObjectName("checkBoxReanalyzeImported");

        verticalLayout->addWidget(checkBoxReanalyzeImported);


        gridLayout->addWidget(groupBox_2, 3, 0, 1, 1);

        groupBox = new QGroupBox(DlgBeatsDlg);
        groupBox->setObjectName("groupBox");
        groupBox->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        horizontalLayout = new QHBoxLayout(groupBox);
        horizontalLayout->setSpacing(6);
        horizontalLayout->setContentsMargins(11, 11, 11, 11);
        horizontalLayout->setObjectName("horizontalLayout");
        comboBoxBeatPlugin = new QComboBox(groupBox);
        comboBoxBeatPlugin->setObjectName("comboBoxBeatPlugin");
        comboBoxBeatPlugin->setMinimumSize(QSize(200, 0));
        comboBoxBeatPlugin->setMaximumSize(QSize(2600, 26));
        comboBoxBeatPlugin->setLayoutDirection(Qt::LeftToRight);

        horizontalLayout->addWidget(comboBoxBeatPlugin);


        gridLayout->addWidget(groupBox, 2, 0, 1, 1);

        QWidget::setTabOrder(checkBoxAnalyzerEnabled, comboBoxBeatPlugin);
        QWidget::setTabOrder(comboBoxBeatPlugin, checkBoxFixedTempo);
        QWidget::setTabOrder(checkBoxFixedTempo, checkBoxReanalyze);
        QWidget::setTabOrder(checkBoxReanalyze, checkBoxReanalyzeImported);

        retranslateUi(DlgBeatsDlg);

        QMetaObject::connectSlotsByName(DlgBeatsDlg);
    } // setupUi

    void retranslateUi(QWidget *DlgBeatsDlg)
    {
        DlgBeatsDlg->setWindowTitle(QCoreApplication::translate("DlgBeatsDlg", "Beat Detection Preferences", nullptr));
#if QT_CONFIG(tooltip)
        checkBoxAnalyzerEnabled->setToolTip(QCoreApplication::translate("DlgBeatsDlg", "When beat detection is enabled, Mixxx detects the beats per minute and beats of your tracks,\n"
"automatically shows a beat-grid for them, and allows you to synchronize tracks using their beat information.", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBoxAnalyzerEnabled->setText(QCoreApplication::translate("DlgBeatsDlg", "Enable BPM and Beat Detection", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("DlgBeatsDlg", "Analyzer Settings", nullptr));
#if QT_CONFIG(tooltip)
        checkBoxFixedTempo->setToolTip(QCoreApplication::translate("DlgBeatsDlg", "Converts beats detected by the analyzer into a fixed-tempo beatgrid.\n"
"Use this setting if your tracks have a constant tempo (e.g. most electronic music).\n"
"Often results in higher quality beatgrids, but will not do well on tracks that have tempo shifts.", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBoxFixedTempo->setText(QCoreApplication::translate("DlgBeatsDlg", "Assume constant tempo (Recommended)", nullptr));
#if QT_CONFIG(tooltip)
        checkBoxFastAnalysis->setToolTip(QCoreApplication::translate("DlgBeatsDlg", "Enable fast beat detection.\n"
"If activated Mixxx only analyzes the first minute of a track for beat information.\n"
"This can speed up beat detection on slower computers but may result in lower quality beatgrids.", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBoxFastAnalysis->setText(QCoreApplication::translate("DlgBeatsDlg", "Enable Fast Analysis (For slow computers, may be less accurate)", nullptr));
#if QT_CONFIG(tooltip)
        checkBoxReanalyze->setToolTip(QCoreApplication::translate("DlgBeatsDlg", "e.g. from 3rd-party programs or Mixxx versions before 1.11.\n"
"(Not checked: Analyze only, if no beats exist.)", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBoxReanalyze->setText(QCoreApplication::translate("DlgBeatsDlg", "Re-analyze beats when settings change or beat detection data is outdated", nullptr));
        checkBoxReanalyzeImported->setText(QCoreApplication::translate("DlgBeatsDlg", "Re-analyze beatgrids imported from other DJ software", nullptr));
        groupBox->setTitle(QCoreApplication::translate("DlgBeatsDlg", "Choose Analyzer", nullptr));
#if QT_CONFIG(tooltip)
        comboBoxBeatPlugin->setToolTip(QCoreApplication::translate("DlgBeatsDlg", "Choose between different algorithms to detect beats.", nullptr));
#endif // QT_CONFIG(tooltip)
    } // retranslateUi

};

namespace Ui {
    class DlgBeatsDlg: public Ui_DlgBeatsDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLGPREFBEATSDLG_H
