/****************************************************************************
** Meta object code from reading C++ file 'autodjcratesdao.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/library/dao/autodjcratesdao.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'autodjcratesdao.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15AutoDJCratesDAOE_t {};
} // unnamed namespace

template <> constexpr inline auto AutoDJCratesDAO::qt_create_metaobjectdata<qt_meta_tag_ZN15AutoDJCratesDAOE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AutoDJCratesDAO",
        "slotTrackDirty",
        "",
        "TrackId",
        "trackId",
        "slotCrateInserted",
        "CrateId",
        "crateId",
        "slotCrateUpdated",
        "slotCrateDeleted",
        "slotCrateTracksChanged",
        "QList<TrackId>",
        "addedTrackIds",
        "removedTrackIds",
        "slotPlaylistAdded",
        "playlistId",
        "slotPlaylistDeleted",
        "slotPlaylistTrackAdded",
        "position",
        "slotPlaylistTrackRemoved",
        "slotPlayerInfoTrackChanged",
        "group",
        "TrackPointer",
        "pNewTrack",
        "pOldTrack"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotTrackDirty'
        QtMocHelpers::SlotData<void(TrackId)>(1, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'slotCrateInserted'
        QtMocHelpers::SlotData<void(CrateId)>(5, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'slotCrateUpdated'
        QtMocHelpers::SlotData<void(CrateId)>(8, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'slotCrateDeleted'
        QtMocHelpers::SlotData<void(CrateId)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'slotCrateTracksChanged'
        QtMocHelpers::SlotData<void(CrateId, const QList<TrackId> &, const QList<TrackId> &)>(10, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 11, 12 }, { 0x80000000 | 11, 13 },
        }}),
        // Slot 'slotPlaylistAdded'
        QtMocHelpers::SlotData<void(int)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 15 },
        }}),
        // Slot 'slotPlaylistDeleted'
        QtMocHelpers::SlotData<void(int)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 15 },
        }}),
        // Slot 'slotPlaylistTrackAdded'
        QtMocHelpers::SlotData<void(int, TrackId, int)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 15 }, { 0x80000000 | 3, 4 }, { QMetaType::Int, 18 },
        }}),
        // Slot 'slotPlaylistTrackRemoved'
        QtMocHelpers::SlotData<void(int, TrackId, int)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 15 }, { 0x80000000 | 3, 4 }, { QMetaType::Int, 18 },
        }}),
        // Slot 'slotPlayerInfoTrackChanged'
        QtMocHelpers::SlotData<void(const QString &, TrackPointer, TrackPointer)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 21 }, { 0x80000000 | 22, 23 }, { 0x80000000 | 22, 24 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AutoDJCratesDAO, qt_meta_tag_ZN15AutoDJCratesDAOE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AutoDJCratesDAO::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15AutoDJCratesDAOE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15AutoDJCratesDAOE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15AutoDJCratesDAOE_t>.metaTypes,
    nullptr
} };

void AutoDJCratesDAO::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AutoDJCratesDAO *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotTrackDirty((*reinterpret_cast<std::add_pointer_t<TrackId>>(_a[1]))); break;
        case 1: _t->slotCrateInserted((*reinterpret_cast<std::add_pointer_t<CrateId>>(_a[1]))); break;
        case 2: _t->slotCrateUpdated((*reinterpret_cast<std::add_pointer_t<CrateId>>(_a[1]))); break;
        case 3: _t->slotCrateDeleted((*reinterpret_cast<std::add_pointer_t<CrateId>>(_a[1]))); break;
        case 4: _t->slotCrateTracksChanged((*reinterpret_cast<std::add_pointer_t<CrateId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<TrackId>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QList<TrackId>>>(_a[3]))); break;
        case 5: _t->slotPlaylistAdded((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->slotPlaylistDeleted((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->slotPlaylistTrackAdded((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<TrackId>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 8: _t->slotPlaylistTrackRemoved((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<TrackId>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 9: _t->slotPlayerInfoTrackChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<TrackPointer>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackId >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CrateId >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CrateId >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CrateId >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CrateId >(); break;
            case 2:
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<TrackId> >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackId >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackId >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackPointer >(); break;
            }
            break;
        }
    }
}

const QMetaObject *AutoDJCratesDAO::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AutoDJCratesDAO::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15AutoDJCratesDAOE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int AutoDJCratesDAO::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}
QT_WARNING_POP
