/********************************************************************************
** Form generated from reading UI file 'dlgprefmodplugdlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLGPREFMODPLUGDLG_H
#define UI_DLGPREFMODPLUGDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DlgPrefModplug
{
public:
    QGridLayout *gridLayout;
    QCheckBox *showAdvanced;
    QSpacerItem *verticalSpacer_2;
    QGroupBox *advancedSettings;
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout_2;
    QSlider *maxMixChannels;
    QSlider *surroundDelay;
    QSlider *bassCutoff;
    QSlider *stereoSeparation;
    QSlider *bassDepth;
    QSlider *reverbDepth;
    QLabel *label_5;
    QSlider *surroundDepth;
    QLabel *label_2;
    QLabel *label_7;
    QLabel *label_13;
    QLabel *label_6;
    QSpacerItem *horizontalSpacer;
    QLabel *label_8;
    QCheckBox *megabass;
    QSlider *reverbDelay;
    QLabel *label_3;
    QLabel *label_18;
    QLabel *label_9;
    QLabel *label_17;
    QCheckBox *surround;
    QLabel *label_15;
    QLabel *label_14;
    QCheckBox *reverb;
    QLabel *label_4;
    QLabel *label_16;
    QLabel *label_19;
    QLabel *label_20;
    QLabel *label_22;
    QLabel *label_21;
    QLabel *label_23;
    QLabel *label_24;
    QLabel *label_25;
    QCheckBox *noiseReduction;
    QGroupBox *Hints;
    QVBoxLayout *verticalLayout_3;
    QLabel *label_11;
    QLabel *modplugSettingsHint;
    QGroupBox *decodingOptions;
    QVBoxLayout *verticalLayout_4;
    QGridLayout *gridLayout_5;
    QSlider *memoryLimit;
    QComboBox *resampleMode;
    QLabel *label;
    QSpinBox *memoryLimitSpin;
    QLabel *label_10;
    QCheckBox *oversampling;

    void setupUi(QWidget *DlgPrefModplug)
    {
        if (DlgPrefModplug->objectName().isEmpty())
            DlgPrefModplug->setObjectName("DlgPrefModplug");
        DlgPrefModplug->resize(590, 704);
        gridLayout = new QGridLayout(DlgPrefModplug);
        gridLayout->setSpacing(10);
        gridLayout->setObjectName("gridLayout");
        showAdvanced = new QCheckBox(DlgPrefModplug);
        showAdvanced->setObjectName("showAdvanced");

        gridLayout->addWidget(showAdvanced, 8, 0, 1, 1);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer_2, 12, 1, 1, 1);

        advancedSettings = new QGroupBox(DlgPrefModplug);
        advancedSettings->setObjectName("advancedSettings");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(advancedSettings->sizePolicy().hasHeightForWidth());
        advancedSettings->setSizePolicy(sizePolicy);
        verticalLayout = new QVBoxLayout(advancedSettings);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout_2 = new QGridLayout();
        gridLayout_2->setSpacing(15);
        gridLayout_2->setObjectName("gridLayout_2");
        maxMixChannels = new QSlider(advancedSettings);
        maxMixChannels->setObjectName("maxMixChannels");
        maxMixChannels->setMinimum(32);
        maxMixChannels->setMaximum(256);
        maxMixChannels->setValue(128);
        maxMixChannels->setOrientation(Qt::Horizontal);

        gridLayout_2->addWidget(maxMixChannels, 4, 5, 1, 1);

        surroundDelay = new QSlider(advancedSettings);
        surroundDelay->setObjectName("surroundDelay");
        surroundDelay->setMinimum(5);
        surroundDelay->setMaximum(50);
        surroundDelay->setValue(10);
        surroundDelay->setOrientation(Qt::Horizontal);

        gridLayout_2->addWidget(surroundDelay, 11, 5, 1, 1);

        bassCutoff = new QSlider(advancedSettings);
        bassCutoff->setObjectName("bassCutoff");
        bassCutoff->setMinimum(10);
        bassCutoff->setMaximum(100);
        bassCutoff->setValue(80);
        bassCutoff->setOrientation(Qt::Horizontal);

        gridLayout_2->addWidget(bassCutoff, 7, 5, 1, 1);

        stereoSeparation = new QSlider(advancedSettings);
        stereoSeparation->setObjectName("stereoSeparation");
        stereoSeparation->setMinimum(1);
        stereoSeparation->setMaximum(256);
        stereoSeparation->setValue(1);
        stereoSeparation->setOrientation(Qt::Horizontal);
        stereoSeparation->setTickPosition(QSlider::NoTicks);

        gridLayout_2->addWidget(stereoSeparation, 4, 1, 1, 1);

        bassDepth = new QSlider(advancedSettings);
        bassDepth->setObjectName("bassDepth");
        bassDepth->setMaximum(100);
        bassDepth->setValue(30);
        bassDepth->setOrientation(Qt::Horizontal);

        gridLayout_2->addWidget(bassDepth, 7, 1, 1, 1);

        reverbDepth = new QSlider(advancedSettings);
        reverbDepth->setObjectName("reverbDepth");
        reverbDepth->setMaximum(100);
        reverbDepth->setValue(30);
        reverbDepth->setOrientation(Qt::Horizontal);

        gridLayout_2->addWidget(reverbDepth, 9, 1, 1, 1);

        label_5 = new QLabel(advancedSettings);
        label_5->setObjectName("label_5");

        gridLayout_2->addWidget(label_5, 3, 4, 1, 3);

        surroundDepth = new QSlider(advancedSettings);
        surroundDepth->setObjectName("surroundDepth");
        surroundDepth->setMaximum(100);
        surroundDepth->setValue(30);
        surroundDepth->setOrientation(Qt::Horizontal);

        gridLayout_2->addWidget(surroundDepth, 11, 1, 1, 1);

        label_2 = new QLabel(advancedSettings);
        label_2->setObjectName("label_2");

        gridLayout_2->addWidget(label_2, 7, 0, 1, 1);

        label_7 = new QLabel(advancedSettings);
        label_7->setObjectName("label_7");

        gridLayout_2->addWidget(label_7, 8, 4, 1, 3);

        label_13 = new QLabel(advancedSettings);
        label_13->setObjectName("label_13");

        gridLayout_2->addWidget(label_13, 9, 0, 1, 1);

        label_6 = new QLabel(advancedSettings);
        label_6->setObjectName("label_6");

        gridLayout_2->addWidget(label_6, 7, 2, 1, 1);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);

        gridLayout_2->addItem(horizontalSpacer, 4, 3, 1, 1);

        label_8 = new QLabel(advancedSettings);
        label_8->setObjectName("label_8");

        gridLayout_2->addWidget(label_8, 4, 0, 1, 1);

        megabass = new QCheckBox(advancedSettings);
        megabass->setObjectName("megabass");

        gridLayout_2->addWidget(megabass, 6, 0, 1, 3);

        reverbDelay = new QSlider(advancedSettings);
        reverbDelay->setObjectName("reverbDelay");
        reverbDelay->setMinimum(10);
        reverbDelay->setMaximum(250);
        reverbDelay->setValue(50);
        reverbDelay->setOrientation(Qt::Horizontal);

        gridLayout_2->addWidget(reverbDelay, 9, 5, 1, 1);

        label_3 = new QLabel(advancedSettings);
        label_3->setObjectName("label_3");

        gridLayout_2->addWidget(label_3, 6, 4, 1, 3);

        label_18 = new QLabel(advancedSettings);
        label_18->setObjectName("label_18");

        gridLayout_2->addWidget(label_18, 4, 4, 1, 1);

        label_9 = new QLabel(advancedSettings);
        label_9->setObjectName("label_9");

        gridLayout_2->addWidget(label_9, 10, 4, 1, 3);

        label_17 = new QLabel(advancedSettings);
        label_17->setObjectName("label_17");

        gridLayout_2->addWidget(label_17, 11, 2, 1, 1);

        surround = new QCheckBox(advancedSettings);
        surround->setObjectName("surround");

        gridLayout_2->addWidget(surround, 10, 0, 1, 3);

        label_15 = new QLabel(advancedSettings);
        label_15->setObjectName("label_15");

        gridLayout_2->addWidget(label_15, 4, 2, 1, 1);

        label_14 = new QLabel(advancedSettings);
        label_14->setObjectName("label_14");

        gridLayout_2->addWidget(label_14, 11, 0, 1, 1);

        reverb = new QCheckBox(advancedSettings);
        reverb->setObjectName("reverb");

        gridLayout_2->addWidget(reverb, 8, 0, 1, 3);

        label_4 = new QLabel(advancedSettings);
        label_4->setObjectName("label_4");

        gridLayout_2->addWidget(label_4, 3, 0, 1, 3);

        label_16 = new QLabel(advancedSettings);
        label_16->setObjectName("label_16");

        gridLayout_2->addWidget(label_16, 9, 2, 1, 1);

        label_19 = new QLabel(advancedSettings);
        label_19->setObjectName("label_19");

        gridLayout_2->addWidget(label_19, 7, 4, 1, 1);

        label_20 = new QLabel(advancedSettings);
        label_20->setObjectName("label_20");

        gridLayout_2->addWidget(label_20, 9, 4, 1, 1);

        label_22 = new QLabel(advancedSettings);
        label_22->setObjectName("label_22");

        gridLayout_2->addWidget(label_22, 4, 6, 1, 1);

        label_21 = new QLabel(advancedSettings);
        label_21->setObjectName("label_21");

        gridLayout_2->addWidget(label_21, 11, 4, 1, 1);

        label_23 = new QLabel(advancedSettings);
        label_23->setObjectName("label_23");

        gridLayout_2->addWidget(label_23, 7, 6, 1, 1);

        label_24 = new QLabel(advancedSettings);
        label_24->setObjectName("label_24");

        gridLayout_2->addWidget(label_24, 9, 6, 1, 1);

        label_25 = new QLabel(advancedSettings);
        label_25->setObjectName("label_25");

        gridLayout_2->addWidget(label_25, 11, 6, 1, 1);

        noiseReduction = new QCheckBox(advancedSettings);
        noiseReduction->setObjectName("noiseReduction");

        gridLayout_2->addWidget(noiseReduction, 5, 0, 1, 3);


        verticalLayout->addLayout(gridLayout_2);


        gridLayout->addWidget(advancedSettings, 9, 0, 1, 6);

        Hints = new QGroupBox(DlgPrefModplug);
        Hints->setObjectName("Hints");
        sizePolicy.setHeightForWidth(Hints->sizePolicy().hasHeightForWidth());
        Hints->setSizePolicy(sizePolicy);
        verticalLayout_3 = new QVBoxLayout(Hints);
        verticalLayout_3->setObjectName("verticalLayout_3");
        label_11 = new QLabel(Hints);
        label_11->setObjectName("label_11");
        label_11->setWordWrap(true);

        verticalLayout_3->addWidget(label_11);

        modplugSettingsHint = new QLabel(Hints);
        modplugSettingsHint->setObjectName("modplugSettingsHint");
        modplugSettingsHint->setWordWrap(true);
        modplugSettingsHint->setOpenExternalLinks(true);

        verticalLayout_3->addWidget(modplugSettingsHint);


        gridLayout->addWidget(Hints, 10, 0, 1, 6);

        decodingOptions = new QGroupBox(DlgPrefModplug);
        decodingOptions->setObjectName("decodingOptions");
        sizePolicy.setHeightForWidth(decodingOptions->sizePolicy().hasHeightForWidth());
        decodingOptions->setSizePolicy(sizePolicy);
        verticalLayout_4 = new QVBoxLayout(decodingOptions);
        verticalLayout_4->setObjectName("verticalLayout_4");
        gridLayout_5 = new QGridLayout();
        gridLayout_5->setObjectName("gridLayout_5");
        memoryLimit = new QSlider(decodingOptions);
        memoryLimit->setObjectName("memoryLimit");
        memoryLimit->setMinimum(16);
        memoryLimit->setMaximum(512);
        memoryLimit->setValue(256);
        memoryLimit->setSliderPosition(256);
        memoryLimit->setOrientation(Qt::Horizontal);

        gridLayout_5->addWidget(memoryLimit, 0, 1, 1, 1);

        resampleMode = new QComboBox(decodingOptions);
        resampleMode->addItem(QString());
        resampleMode->addItem(QString());
        resampleMode->addItem(QString());
        resampleMode->addItem(QString());
        resampleMode->setObjectName("resampleMode");

        gridLayout_5->addWidget(resampleMode, 2, 1, 1, 2);

        label = new QLabel(decodingOptions);
        label->setObjectName("label");

        gridLayout_5->addWidget(label, 2, 0, 1, 1);

        memoryLimitSpin = new QSpinBox(decodingOptions);
        memoryLimitSpin->setObjectName("memoryLimitSpin");
        memoryLimitSpin->setMinimum(16);
        memoryLimitSpin->setMaximum(512);
        memoryLimitSpin->setValue(256);

        gridLayout_5->addWidget(memoryLimitSpin, 0, 2, 1, 1);

        label_10 = new QLabel(decodingOptions);
        label_10->setObjectName("label_10");

        gridLayout_5->addWidget(label_10, 0, 0, 1, 1);

        oversampling = new QCheckBox(decodingOptions);
        oversampling->setObjectName("oversampling");
        oversampling->setChecked(true);

        gridLayout_5->addWidget(oversampling, 1, 0, 1, 1);


        verticalLayout_4->addLayout(gridLayout_5);


        gridLayout->addWidget(decodingOptions, 1, 0, 1, 6);

        QWidget::setTabOrder(memoryLimit, memoryLimitSpin);
        QWidget::setTabOrder(memoryLimitSpin, oversampling);
        QWidget::setTabOrder(oversampling, resampleMode);
        QWidget::setTabOrder(resampleMode, showAdvanced);
        QWidget::setTabOrder(showAdvanced, stereoSeparation);
        QWidget::setTabOrder(stereoSeparation, maxMixChannels);
        QWidget::setTabOrder(maxMixChannels, noiseReduction);
        QWidget::setTabOrder(noiseReduction, megabass);
        QWidget::setTabOrder(megabass, bassDepth);
        QWidget::setTabOrder(bassDepth, bassCutoff);
        QWidget::setTabOrder(bassCutoff, reverb);
        QWidget::setTabOrder(reverb, reverbDepth);
        QWidget::setTabOrder(reverbDepth, reverbDelay);
        QWidget::setTabOrder(reverbDelay, surround);
        QWidget::setTabOrder(surround, surroundDepth);
        QWidget::setTabOrder(surroundDepth, surroundDelay);

        retranslateUi(DlgPrefModplug);

        resampleMode->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(DlgPrefModplug);
    } // setupUi

    void retranslateUi(QWidget *DlgPrefModplug)
    {
        DlgPrefModplug->setWindowTitle(QCoreApplication::translate("DlgPrefModplug", "Modplug Preferences", nullptr));
        showAdvanced->setText(QCoreApplication::translate("DlgPrefModplug", "Show Advanced Settings", nullptr));
        advancedSettings->setTitle(QString());
        label_5->setText(QCoreApplication::translate("DlgPrefModplug", "Maximum Number of Mixing Channels:", nullptr));
        label_2->setText(QCoreApplication::translate("DlgPrefModplug", "Low", nullptr));
        label_7->setText(QCoreApplication::translate("DlgPrefModplug", "Reverb Delay:", nullptr));
        label_13->setText(QCoreApplication::translate("DlgPrefModplug", "Low", nullptr));
        label_6->setText(QCoreApplication::translate("DlgPrefModplug", "High", nullptr));
        label_8->setText(QCoreApplication::translate("DlgPrefModplug", "None", nullptr));
        megabass->setText(QCoreApplication::translate("DlgPrefModplug", "Bass Expansion", nullptr));
        label_3->setText(QCoreApplication::translate("DlgPrefModplug", "Bass Range:", nullptr));
        label_18->setText(QCoreApplication::translate("DlgPrefModplug", "16", nullptr));
        label_9->setText(QCoreApplication::translate("DlgPrefModplug", "Front/Rear Delay:", nullptr));
        label_17->setText(QCoreApplication::translate("DlgPrefModplug", "High", nullptr));
        surround->setText(QCoreApplication::translate("DlgPrefModplug", "Pro-Logic Surround", nullptr));
        label_15->setText(QCoreApplication::translate("DlgPrefModplug", "Full", nullptr));
        label_14->setText(QCoreApplication::translate("DlgPrefModplug", "Low", nullptr));
        reverb->setText(QCoreApplication::translate("DlgPrefModplug", "Reverb", nullptr));
        label_4->setText(QCoreApplication::translate("DlgPrefModplug", "Stereo separation", nullptr));
        label_16->setText(QCoreApplication::translate("DlgPrefModplug", "High", nullptr));
        label_19->setText(QCoreApplication::translate("DlgPrefModplug", "10Hz", nullptr));
        label_20->setText(QCoreApplication::translate("DlgPrefModplug", "10ms", nullptr));
        label_22->setText(QCoreApplication::translate("DlgPrefModplug", "256", nullptr));
        label_21->setText(QCoreApplication::translate("DlgPrefModplug", "5ms", nullptr));
        label_23->setText(QCoreApplication::translate("DlgPrefModplug", "100Hz", nullptr));
        label_24->setText(QCoreApplication::translate("DlgPrefModplug", "250ms", nullptr));
        label_25->setText(QCoreApplication::translate("DlgPrefModplug", "50ms", nullptr));
        noiseReduction->setText(QCoreApplication::translate("DlgPrefModplug", "Noise reduction", nullptr));
        Hints->setTitle(QCoreApplication::translate("DlgPrefModplug", "Hints", nullptr));
        label_11->setText(QCoreApplication::translate("DlgPrefModplug", "Module files are decoded at once and kept in RAM to allow for seeking and smooth operation in Mixxx. About 10MB of RAM are required for 1 minute of audio.", nullptr));
#if QT_CONFIG(tooltip)
        decodingOptions->setToolTip(QCoreApplication::translate("DlgPrefModplug", "Decoding options for libmodplug, a software library for loading and rendering module files (MOD music, tracker music).", nullptr));
#endif // QT_CONFIG(tooltip)
        decodingOptions->setTitle(QCoreApplication::translate("DlgPrefModplug", "Decoding Options", nullptr));
#if QT_CONFIG(tooltip)
        memoryLimit->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        resampleMode->setItemText(0, QCoreApplication::translate("DlgPrefModplug", "Nearest (very fast, extremely bad quality)", nullptr));
        resampleMode->setItemText(1, QCoreApplication::translate("DlgPrefModplug", "Linear (fast, good quality)", nullptr));
        resampleMode->setItemText(2, QCoreApplication::translate("DlgPrefModplug", "Cubic Spline (high quality)", nullptr));
        resampleMode->setItemText(3, QCoreApplication::translate("DlgPrefModplug", "8-tap FIR (extremely high quality)", nullptr));

        label->setText(QCoreApplication::translate("DlgPrefModplug", "Resampling mode (interpolation)", nullptr));
        label_10->setText(QCoreApplication::translate("DlgPrefModplug", "Memory limit for single track (MB)", nullptr));
        oversampling->setText(QCoreApplication::translate("DlgPrefModplug", "Enable oversampling", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgPrefModplug: public Ui_DlgPrefModplug {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLGPREFMODPLUGDLG_H
