/****************************************************************************
** Meta object code from reading C++ file 'waveformwidgetfactory.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/waveform/waveformwidgetfactory.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'waveformwidgetfactory.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN21WaveformWidgetFactoryE_t {};
} // unnamed namespace

template <> constexpr inline auto WaveformWidgetFactory::qt_create_metaobjectdata<qt_meta_tag_ZN21WaveformWidgetFactoryE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "WaveformWidgetFactory",
        "waveformUpdateTick",
        "",
        "waveformMeasured",
        "frameRate",
        "droppedFrames",
        "renderSpinnies",
        "VSyncThread*",
        "swapSpinnies",
        "renderVuMeters",
        "swapVuMeters",
        "overviewNormalizeChanged",
        "overallVisualGainChanged",
        "slotSkinLoaded",
        "render",
        "swap",
        "swapAndRender",
        "slotFrameSwapped"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'waveformUpdateTick'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'waveformMeasured'
        QtMocHelpers::SignalData<void(float, int)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 4 }, { QMetaType::Int, 5 },
        }}),
        // Signal 'renderSpinnies'
        QtMocHelpers::SignalData<void(VSyncThread *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 2 },
        }}),
        // Signal 'swapSpinnies'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'renderVuMeters'
        QtMocHelpers::SignalData<void(VSyncThread *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 2 },
        }}),
        // Signal 'swapVuMeters'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'overviewNormalizeChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'overallVisualGainChanged'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSkinLoaded'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'render'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'swap'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'swapAndRender'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotFrameSwapped'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WaveformWidgetFactory, qt_meta_tag_ZN21WaveformWidgetFactoryE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject WaveformWidgetFactory::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21WaveformWidgetFactoryE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21WaveformWidgetFactoryE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21WaveformWidgetFactoryE_t>.metaTypes,
    nullptr
} };

void WaveformWidgetFactory::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WaveformWidgetFactory *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->waveformUpdateTick(); break;
        case 1: _t->waveformMeasured((*reinterpret_cast<std::add_pointer_t<float>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 2: _t->renderSpinnies((*reinterpret_cast<std::add_pointer_t<VSyncThread*>>(_a[1]))); break;
        case 3: _t->swapSpinnies(); break;
        case 4: _t->renderVuMeters((*reinterpret_cast<std::add_pointer_t<VSyncThread*>>(_a[1]))); break;
        case 5: _t->swapVuMeters(); break;
        case 6: _t->overviewNormalizeChanged(); break;
        case 7: _t->overallVisualGainChanged(); break;
        case 8: _t->slotSkinLoaded(); break;
        case 9: _t->render(); break;
        case 10: _t->swap(); break;
        case 11: _t->swapAndRender(); break;
        case 12: _t->slotFrameSwapped(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WaveformWidgetFactory::*)()>(_a, &WaveformWidgetFactory::waveformUpdateTick, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WaveformWidgetFactory::*)(float , int )>(_a, &WaveformWidgetFactory::waveformMeasured, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WaveformWidgetFactory::*)(VSyncThread * )>(_a, &WaveformWidgetFactory::renderSpinnies, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WaveformWidgetFactory::*)()>(_a, &WaveformWidgetFactory::swapSpinnies, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (WaveformWidgetFactory::*)(VSyncThread * )>(_a, &WaveformWidgetFactory::renderVuMeters, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (WaveformWidgetFactory::*)()>(_a, &WaveformWidgetFactory::swapVuMeters, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (WaveformWidgetFactory::*)()>(_a, &WaveformWidgetFactory::overviewNormalizeChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (WaveformWidgetFactory::*)()>(_a, &WaveformWidgetFactory::overallVisualGainChanged, 7))
            return;
    }
}

const QMetaObject *WaveformWidgetFactory::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *WaveformWidgetFactory::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21WaveformWidgetFactoryE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Singleton<WaveformWidgetFactory>"))
        return static_cast< Singleton<WaveformWidgetFactory>*>(this);
    return QObject::qt_metacast(_clname);
}

int WaveformWidgetFactory::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void WaveformWidgetFactory::waveformUpdateTick()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void WaveformWidgetFactory::waveformMeasured(float _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void WaveformWidgetFactory::renderSpinnies(VSyncThread * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void WaveformWidgetFactory::swapSpinnies()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void WaveformWidgetFactory::renderVuMeters(VSyncThread * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void WaveformWidgetFactory::swapVuMeters()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void WaveformWidgetFactory::overviewNormalizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void WaveformWidgetFactory::overallVisualGainChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
