/********************************************************************************
** Form generated from reading UI file 'dlgdevelopertoolsdlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLGDEVELOPERTOOLSDLG_H
#define UI_DLGDEVELOPERTOOLSDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTableView>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "widget/wsearchlineedit.h"

QT_BEGIN_NAMESPACE

class Ui_DlgDeveloperTools
{
public:
    QVBoxLayout *verticalLayout;
    QTabWidget *toolTabWidget;
    QWidget *controlsTab;
    QGridLayout *gridLayout;
    WSearchLineEdit *controlSearch;
    QSpacerItem *horizontalSpacer;
    QPushButton *controlDump;
    QTableView *controlsTable;
    QWidget *logTab;
    QGridLayout *gridLayout_2;
    QLineEdit *logSearch;
    QPushButton *logSearchButton;
    QSpacerItem *horizontalSpacer_2;
    QTextEdit *logTextView;
    QWidget *statsTab;
    QVBoxLayout *verticalLayout_4;
    QTableView *statsTable;

    void setupUi(QDialog *DlgDeveloperTools)
    {
        if (DlgDeveloperTools->objectName().isEmpty())
            DlgDeveloperTools->setObjectName("DlgDeveloperTools");
        DlgDeveloperTools->resize(628, 485);
        verticalLayout = new QVBoxLayout(DlgDeveloperTools);
        verticalLayout->setObjectName("verticalLayout");
        toolTabWidget = new QTabWidget(DlgDeveloperTools);
        toolTabWidget->setObjectName("toolTabWidget");
        controlsTab = new QWidget();
        controlsTab->setObjectName("controlsTab");
        gridLayout = new QGridLayout(controlsTab);
        gridLayout->setObjectName("gridLayout");
        controlSearch = new WSearchLineEdit(controlsTab);
        controlSearch->setObjectName("controlSearch");

        gridLayout->addWidget(controlSearch, 0, 4, 1, 1);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer, 0, 3, 1, 1);

        controlDump = new QPushButton(controlsTab);
        controlDump->setObjectName("controlDump");

        gridLayout->addWidget(controlDump, 0, 1, 1, 1);

        controlsTable = new QTableView(controlsTab);
        controlsTable->setObjectName("controlsTable");
        controlsTable->setEditTriggers(QAbstractItemView::AnyKeyPressed|QAbstractItemView::DoubleClicked|QAbstractItemView::EditKeyPressed|QAbstractItemView::SelectedClicked);
        controlsTable->setAlternatingRowColors(true);
        controlsTable->setSelectionBehavior(QAbstractItemView::SelectItems);
        controlsTable->setTabKeyNavigation(false);
        controlsTable->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
        controlsTable->setSortingEnabled(true);
        controlsTable->setWordWrap(false);
        controlsTable->setCornerButtonEnabled(false);
        controlsTable->verticalHeader()->setVisible(false);

        gridLayout->addWidget(controlsTable, 1, 1, 1, 4);

        toolTabWidget->addTab(controlsTab, QString());
        logTab = new QWidget();
        logTab->setObjectName("logTab");
        gridLayout_2 = new QGridLayout(logTab);
        gridLayout_2->setObjectName("gridLayout_2");
        logSearch = new QLineEdit(logTab);
        logSearch->setObjectName("logSearch");

        gridLayout_2->addWidget(logSearch, 0, 2, 1, 1);

        logSearchButton = new QPushButton(logTab);
        logSearchButton->setObjectName("logSearchButton");

        gridLayout_2->addWidget(logSearchButton, 0, 3, 1, 1);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_2->addItem(horizontalSpacer_2, 0, 1, 1, 1);

        logTextView = new QTextEdit(logTab);
        logTextView->setObjectName("logTextView");
        logTextView->setReadOnly(true);
        logTextView->setHtml(QString::fromUtf8("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Ubuntu'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:'.Lucida Grande UI'; font-size:13pt;\"><br /></p></body></html>"));
        logTextView->setOverwriteMode(true);
        logTextView->setAcceptRichText(false);

        gridLayout_2->addWidget(logTextView, 1, 1, 1, 3);

        toolTabWidget->addTab(logTab, QString());
        statsTab = new QWidget();
        statsTab->setObjectName("statsTab");
        verticalLayout_4 = new QVBoxLayout(statsTab);
        verticalLayout_4->setObjectName("verticalLayout_4");
        statsTable = new QTableView(statsTab);
        statsTable->setObjectName("statsTable");
        statsTable->setEditTriggers(QAbstractItemView::AnyKeyPressed|QAbstractItemView::DoubleClicked|QAbstractItemView::EditKeyPressed);
        statsTable->setAlternatingRowColors(true);
        statsTable->setSelectionBehavior(QAbstractItemView::SelectRows);
        statsTable->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
        statsTable->setSortingEnabled(true);
        statsTable->verticalHeader()->setVisible(false);

        verticalLayout_4->addWidget(statsTable);

        toolTabWidget->addTab(statsTab, QString());

        verticalLayout->addWidget(toolTabWidget);


        retranslateUi(DlgDeveloperTools);

        toolTabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(DlgDeveloperTools);
    } // setupUi

    void retranslateUi(QDialog *DlgDeveloperTools)
    {
        DlgDeveloperTools->setWindowTitle(QCoreApplication::translate("DlgDeveloperTools", "Developer Tools", nullptr));
#if QT_CONFIG(tooltip)
        controlDump->setToolTip(QCoreApplication::translate("DlgDeveloperTools", "Dumps all ControlObject values to a csv-file saved in the settings path (e.g. ~/.mixxx)", nullptr));
#endif // QT_CONFIG(tooltip)
        controlDump->setText(QCoreApplication::translate("DlgDeveloperTools", "Dump to csv", nullptr));
        toolTabWidget->setTabText(toolTabWidget->indexOf(controlsTab), QCoreApplication::translate("DlgDeveloperTools", "Controls", nullptr));
        logSearchButton->setText(QCoreApplication::translate("DlgDeveloperTools", "Search", nullptr));
        toolTabWidget->setTabText(toolTabWidget->indexOf(logTab), QCoreApplication::translate("DlgDeveloperTools", "Log", nullptr));
        toolTabWidget->setTabText(toolTabWidget->indexOf(statsTab), QCoreApplication::translate("DlgDeveloperTools", "Stats", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgDeveloperTools: public Ui_DlgDeveloperTools {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLGDEVELOPERTOOLSDLG_H
