/****************************************************************************
** Meta object code from reading C++ file 'playlistdao.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/library/dao/playlistdao.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QSet>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'playlistdao.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11PlaylistDAOE_t {};
} // unnamed namespace

template <> constexpr inline auto PlaylistDAO::qt_create_metaobjectdata<qt_meta_tag_ZN11PlaylistDAOE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PlaylistDAO",
        "added",
        "",
        "playlistId",
        "deleted",
        "renamed",
        "newName",
        "lockChanged",
        "QSet<int>",
        "playlistIds",
        "trackAdded",
        "TrackId",
        "trackId",
        "position",
        "trackRemoved",
        "playlistContentChanged",
        "tracksAdded",
        "tracksMoved",
        "tracksRemoved",
        "tracksRemovedFromPlayedHistory",
        "QSet<TrackId>",
        "playedTrackIds"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'added'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'deleted'
        QtMocHelpers::SignalData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'renamed'
        QtMocHelpers::SignalData<void(int, const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::QString, 6 },
        }}),
        // Signal 'lockChanged'
        QtMocHelpers::SignalData<void(const QSet<int> &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'trackAdded'
        QtMocHelpers::SignalData<void(int, TrackId, int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { 0x80000000 | 11, 12 }, { QMetaType::Int, 13 },
        }}),
        // Signal 'trackRemoved'
        QtMocHelpers::SignalData<void(int, TrackId, int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { 0x80000000 | 11, 12 }, { QMetaType::Int, 13 },
        }}),
        // Signal 'playlistContentChanged'
        QtMocHelpers::SignalData<void(const QSet<int> &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'tracksAdded'
        QtMocHelpers::SignalData<void(const QSet<int> &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'tracksMoved'
        QtMocHelpers::SignalData<void(const QSet<int> &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'tracksRemoved'
        QtMocHelpers::SignalData<void(const QSet<int> &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'tracksRemovedFromPlayedHistory'
        QtMocHelpers::SignalData<void(const QSet<TrackId> &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PlaylistDAO, qt_meta_tag_ZN11PlaylistDAOE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PlaylistDAO::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11PlaylistDAOE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11PlaylistDAOE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11PlaylistDAOE_t>.metaTypes,
    nullptr
} };

void PlaylistDAO::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PlaylistDAO *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->added((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->deleted((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->renamed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 3: _t->lockChanged((*reinterpret_cast<std::add_pointer_t<QSet<int>>>(_a[1]))); break;
        case 4: _t->trackAdded((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<TrackId>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 5: _t->trackRemoved((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<TrackId>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 6: _t->playlistContentChanged((*reinterpret_cast<std::add_pointer_t<QSet<int>>>(_a[1]))); break;
        case 7: _t->tracksAdded((*reinterpret_cast<std::add_pointer_t<QSet<int>>>(_a[1]))); break;
        case 8: _t->tracksMoved((*reinterpret_cast<std::add_pointer_t<QSet<int>>>(_a[1]))); break;
        case 9: _t->tracksRemoved((*reinterpret_cast<std::add_pointer_t<QSet<int>>>(_a[1]))); break;
        case 10: _t->tracksRemovedFromPlayedHistory((*reinterpret_cast<std::add_pointer_t<QSet<TrackId>>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<int> >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackId >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackId >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<int> >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<int> >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<int> >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<int> >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<TrackId> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PlaylistDAO::*)(int )>(_a, &PlaylistDAO::added, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistDAO::*)(int )>(_a, &PlaylistDAO::deleted, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistDAO::*)(int , const QString & )>(_a, &PlaylistDAO::renamed, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistDAO::*)(const QSet<int> & )>(_a, &PlaylistDAO::lockChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistDAO::*)(int , TrackId , int )>(_a, &PlaylistDAO::trackAdded, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistDAO::*)(int , TrackId , int )>(_a, &PlaylistDAO::trackRemoved, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistDAO::*)(const QSet<int> & )>(_a, &PlaylistDAO::playlistContentChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistDAO::*)(const QSet<int> & )>(_a, &PlaylistDAO::tracksAdded, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistDAO::*)(const QSet<int> & )>(_a, &PlaylistDAO::tracksMoved, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistDAO::*)(const QSet<int> & )>(_a, &PlaylistDAO::tracksRemoved, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistDAO::*)(const QSet<TrackId> & )>(_a, &PlaylistDAO::tracksRemovedFromPlayedHistory, 10))
            return;
    }
}

const QMetaObject *PlaylistDAO::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PlaylistDAO::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11PlaylistDAOE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "DAO"))
        return static_cast< DAO*>(this);
    return QObject::qt_metacast(_clname);
}

int PlaylistDAO::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void PlaylistDAO::added(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void PlaylistDAO::deleted(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void PlaylistDAO::renamed(int _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void PlaylistDAO::lockChanged(const QSet<int> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void PlaylistDAO::trackAdded(int _t1, TrackId _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2, _t3);
}

// SIGNAL 5
void PlaylistDAO::trackRemoved(int _t1, TrackId _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2, _t3);
}

// SIGNAL 6
void PlaylistDAO::playlistContentChanged(const QSet<int> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void PlaylistDAO::tracksAdded(const QSet<int> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void PlaylistDAO::tracksMoved(const QSet<int> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void PlaylistDAO::tracksRemoved(const QSet<int> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void PlaylistDAO::tracksRemovedFromPlayedHistory(const QSet<TrackId> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}
QT_WARNING_POP
