/********************************************************************************
** Form generated from reading UI file 'dlgprefmixerdlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLGPREFMIXERDLG_H
#define UI_DLGPREFMIXERDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGraphicsView>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DlgPrefMixerDlg
{
public:
    QVBoxLayout *verticalLayout_2;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout;
    QRadioButton *radioButtonAdditive;
    QRadioButton *radioButtonConstantPower;
    QHBoxLayout *hboxLayout;
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout1;
    QLabel *label_xf_mixing;
    QSpacerItem *spacerItem;
    QLabel *label_xf_scratching;
    QSlider *SliderXFader;
    QHBoxLayout *hboxLayout2;
    QLabel *label_linear;
    QSpacerItem *spacerItem1;
    QLabel *label_logarithmic;
    QGraphicsView *graphicsViewXfader;
    QCheckBox *checkBoxReverse;
    QGroupBox *DeckEqualizerOptions;
    QVBoxLayout *DeckEqualizerCheckboxes;
    QCheckBox *CheckBoxEqOnly;
    QCheckBox *CheckBoxSingleEqEffect;
    QWidget *GridWidgetDecks;
    QGridLayout *gridLayout_3;
    QWidget *widget;
    QLabel *label_eq_plugin;
    QLabel *label_quickeffect;
    QSpacerItem *horizontalSpacer;
    QCheckBox *CheckBoxBypass;
    QCheckBox *CheckBoxEqAutoReset;
    QCheckBox *CheckBoxGainAutoReset;
    QGroupBox *GroupEQSliders;
    QVBoxLayout *vBoxLayout_1;
    QWidget *GroupHiEQ;
    QVBoxLayout *vBoxLayout_2;
    QLabel *label_highshelf;
    QSlider *SliderHiEQ;
    QHBoxLayout *sliderHiEQ_labels;
    QLabel *label_16hz;
    QSpacerItem *spacerItem2;
    QLabel *TextHiEQ;
    QSpacerItem *spacerItem3;
    QLabel *label_20khz;
    QWidget *GroupLoEQ;
    QVBoxLayout *vBoxLayout_3;
    QLabel *label_lowshelf;
    QSlider *SliderLoEQ;
    QHBoxLayout *sliderLoEQ_labels;
    QLabel *label_16hz_lo;
    QSpacerItem *spacerItem4;
    QLabel *TextLoEQ;
    QSpacerItem *spacerItem5;
    QLabel *label_20khz_lo;
    QGroupBox *MainMainEQ;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout_2;
    QComboBox *comboBoxMainEq;
    QPushButton *pbResetMainEq;
    QSpacerItem *horizontalSpacer_2;
    QHBoxLayout *horizontalLayout;
    QGridLayout *slidersGridLayout;
    QSpacerItem *horizontalSpacer_3;
    QSpacerItem *spacerItem6;
    QButtonGroup *buttonGroupCrossfaderModes;

    void setupUi(QWidget *DlgPrefMixerDlg)
    {
        if (DlgPrefMixerDlg->objectName().isEmpty())
            DlgPrefMixerDlg->setObjectName("DlgPrefMixerDlg");
        DlgPrefMixerDlg->resize(427, 333);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(DlgPrefMixerDlg->sizePolicy().hasHeightForWidth());
        DlgPrefMixerDlg->setSizePolicy(sizePolicy);
        verticalLayout_2 = new QVBoxLayout(DlgPrefMixerDlg);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        groupBox = new QGroupBox(DlgPrefMixerDlg);
        groupBox->setObjectName("groupBox");
        verticalLayout = new QVBoxLayout(groupBox);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        radioButtonAdditive = new QRadioButton(groupBox);
        buttonGroupCrossfaderModes = new QButtonGroup(DlgPrefMixerDlg);
        buttonGroupCrossfaderModes->setObjectName("buttonGroupCrossfaderModes");
        buttonGroupCrossfaderModes->addButton(radioButtonAdditive);
        radioButtonAdditive->setObjectName("radioButtonAdditive");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(radioButtonAdditive->sizePolicy().hasHeightForWidth());
        radioButtonAdditive->setSizePolicy(sizePolicy1);

        verticalLayout->addWidget(radioButtonAdditive);

        radioButtonConstantPower = new QRadioButton(groupBox);
        buttonGroupCrossfaderModes->addButton(radioButtonConstantPower);
        radioButtonConstantPower->setObjectName("radioButtonConstantPower");

        verticalLayout->addWidget(radioButtonConstantPower);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        vboxLayout = new QVBoxLayout();
        vboxLayout->setSpacing(6);
        vboxLayout->setObjectName("vboxLayout");
        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName("hboxLayout1");
        label_xf_mixing = new QLabel(groupBox);
        label_xf_mixing->setObjectName("label_xf_mixing");

        hboxLayout1->addWidget(label_xf_mixing);

        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(spacerItem);

        label_xf_scratching = new QLabel(groupBox);
        label_xf_scratching->setObjectName("label_xf_scratching");

        hboxLayout1->addWidget(label_xf_scratching);


        vboxLayout->addLayout(hboxLayout1);

        SliderXFader = new QSlider(groupBox);
        SliderXFader->setObjectName("SliderXFader");
        SliderXFader->setMinimum(0);
        SliderXFader->setMaximum(100000);
        SliderXFader->setSingleStep(1000);
        SliderXFader->setPageStep(1000);
        SliderXFader->setValue(50000);
        SliderXFader->setOrientation(Qt::Horizontal);

        vboxLayout->addWidget(SliderXFader);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setSpacing(6);
        hboxLayout2->setObjectName("hboxLayout2");
        label_linear = new QLabel(groupBox);
        label_linear->setObjectName("label_linear");
        label_linear->setWordWrap(false);

        hboxLayout2->addWidget(label_linear);

        spacerItem1 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout2->addItem(spacerItem1);

        label_logarithmic = new QLabel(groupBox);
        label_logarithmic->setObjectName("label_logarithmic");
        label_logarithmic->setWordWrap(false);

        hboxLayout2->addWidget(label_logarithmic);


        vboxLayout->addLayout(hboxLayout2);


        hboxLayout->addLayout(vboxLayout);

        graphicsViewXfader = new QGraphicsView(groupBox);
        graphicsViewXfader->setObjectName("graphicsViewXfader");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(graphicsViewXfader->sizePolicy().hasHeightForWidth());
        graphicsViewXfader->setSizePolicy(sizePolicy2);
        graphicsViewXfader->setFixedSize(QSize(123, 83));
        graphicsViewXfader->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        graphicsViewXfader->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

        hboxLayout->addWidget(graphicsViewXfader);


        verticalLayout->addLayout(hboxLayout);

        checkBoxReverse = new QCheckBox(groupBox);
        checkBoxReverse->setObjectName("checkBoxReverse");
        sizePolicy1.setHeightForWidth(checkBoxReverse->sizePolicy().hasHeightForWidth());
        checkBoxReverse->setSizePolicy(sizePolicy1);

        verticalLayout->addWidget(checkBoxReverse);


        verticalLayout_2->addWidget(groupBox);

        DeckEqualizerOptions = new QGroupBox(DlgPrefMixerDlg);
        DeckEqualizerOptions->setObjectName("DeckEqualizerOptions");
        DeckEqualizerCheckboxes = new QVBoxLayout(DeckEqualizerOptions);
        DeckEqualizerCheckboxes->setSpacing(6);
        DeckEqualizerCheckboxes->setContentsMargins(11, 11, 11, 11);
        DeckEqualizerCheckboxes->setObjectName("DeckEqualizerCheckboxes");
        CheckBoxEqOnly = new QCheckBox(DeckEqualizerOptions);
        CheckBoxEqOnly->setObjectName("CheckBoxEqOnly");
        CheckBoxEqOnly->setChecked(true);

        DeckEqualizerCheckboxes->addWidget(CheckBoxEqOnly);

        CheckBoxSingleEqEffect = new QCheckBox(DeckEqualizerOptions);
        CheckBoxSingleEqEffect->setObjectName("CheckBoxSingleEqEffect");
        CheckBoxSingleEqEffect->setChecked(true);

        DeckEqualizerCheckboxes->addWidget(CheckBoxSingleEqEffect);

        GridWidgetDecks = new QWidget(DeckEqualizerOptions);
        GridWidgetDecks->setObjectName("GridWidgetDecks");
        gridLayout_3 = new QGridLayout(GridWidgetDecks);
        gridLayout_3->setSpacing(6);
        gridLayout_3->setContentsMargins(11, 11, 11, 11);
        gridLayout_3->setObjectName("gridLayout_3");
        widget = new QWidget(GridWidgetDecks);
        widget->setObjectName("widget");

        gridLayout_3->addWidget(widget, 0, 0, 1, 1);

        label_eq_plugin = new QLabel(GridWidgetDecks);
        label_eq_plugin->setObjectName("label_eq_plugin");
        QFont font;
        font.setBold(true);
        label_eq_plugin->setFont(font);
        label_eq_plugin->setContentsMargins(3, -1, -1, -1);

        gridLayout_3->addWidget(label_eq_plugin, 0, 1, 1, 1);

        label_quickeffect = new QLabel(GridWidgetDecks);
        label_quickeffect->setObjectName("label_quickeffect");
        label_quickeffect->setFont(font);
        label_quickeffect->setContentsMargins(3, -1, -1, -1);

        gridLayout_3->addWidget(label_quickeffect, 0, 2, 1, 1);

        horizontalSpacer = new QSpacerItem(40, 5, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_3->addItem(horizontalSpacer, 0, 3, 1, 1);


        DeckEqualizerCheckboxes->addWidget(GridWidgetDecks);

        CheckBoxBypass = new QCheckBox(DeckEqualizerOptions);
        CheckBoxBypass->setObjectName("CheckBoxBypass");

        DeckEqualizerCheckboxes->addWidget(CheckBoxBypass);

        CheckBoxEqAutoReset = new QCheckBox(DeckEqualizerOptions);
        CheckBoxEqAutoReset->setObjectName("CheckBoxEqAutoReset");

        DeckEqualizerCheckboxes->addWidget(CheckBoxEqAutoReset);

        CheckBoxGainAutoReset = new QCheckBox(DeckEqualizerOptions);
        CheckBoxGainAutoReset->setObjectName("CheckBoxGainAutoReset");

        DeckEqualizerCheckboxes->addWidget(CheckBoxGainAutoReset);


        verticalLayout_2->addWidget(DeckEqualizerOptions);

        GroupEQSliders = new QGroupBox(DlgPrefMixerDlg);
        GroupEQSliders->setObjectName("GroupEQSliders");
        vBoxLayout_1 = new QVBoxLayout(GroupEQSliders);
        vBoxLayout_1->setSpacing(6);
        vBoxLayout_1->setContentsMargins(11, 11, 11, 11);
        vBoxLayout_1->setObjectName("vBoxLayout_1");
        GroupHiEQ = new QWidget(GroupEQSliders);
        GroupHiEQ->setObjectName("GroupHiEQ");
        vBoxLayout_2 = new QVBoxLayout(GroupHiEQ);
        vBoxLayout_2->setSpacing(6);
        vBoxLayout_2->setContentsMargins(11, 11, 11, 11);
        vBoxLayout_2->setObjectName("vBoxLayout_2");
        label_highshelf = new QLabel(GroupHiEQ);
        label_highshelf->setObjectName("label_highshelf");
        label_highshelf->setFont(font);
        label_highshelf->setWordWrap(false);

        vBoxLayout_2->addWidget(label_highshelf);

        SliderHiEQ = new QSlider(GroupHiEQ);
        SliderHiEQ->setObjectName("SliderHiEQ");
        SliderHiEQ->setMinimum(100);
        SliderHiEQ->setMaximum(100000);
        SliderHiEQ->setSingleStep(30);
        SliderHiEQ->setPageStep(300);
        SliderHiEQ->setValue(100);
        SliderHiEQ->setOrientation(Qt::Horizontal);

        vBoxLayout_2->addWidget(SliderHiEQ);

        sliderHiEQ_labels = new QHBoxLayout();
        sliderHiEQ_labels->setSpacing(6);
        sliderHiEQ_labels->setObjectName("sliderHiEQ_labels");
        label_16hz = new QLabel(GroupHiEQ);
        label_16hz->setObjectName("label_16hz");
        label_16hz->setWordWrap(false);

        sliderHiEQ_labels->addWidget(label_16hz);

        spacerItem2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        sliderHiEQ_labels->addItem(spacerItem2);

        TextHiEQ = new QLabel(GroupHiEQ);
        TextHiEQ->setObjectName("TextHiEQ");
        TextHiEQ->setText(QString::fromUtf8("textLabel1"));
        TextHiEQ->setWordWrap(false);

        sliderHiEQ_labels->addWidget(TextHiEQ);

        spacerItem3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        sliderHiEQ_labels->addItem(spacerItem3);

        label_20khz = new QLabel(GroupHiEQ);
        label_20khz->setObjectName("label_20khz");
        label_20khz->setWordWrap(false);

        sliderHiEQ_labels->addWidget(label_20khz);


        vBoxLayout_2->addLayout(sliderHiEQ_labels);


        vBoxLayout_1->addWidget(GroupHiEQ);

        GroupLoEQ = new QWidget(GroupEQSliders);
        GroupLoEQ->setObjectName("GroupLoEQ");
        vBoxLayout_3 = new QVBoxLayout(GroupLoEQ);
        vBoxLayout_3->setSpacing(6);
        vBoxLayout_3->setContentsMargins(11, 11, 11, 11);
        vBoxLayout_3->setObjectName("vBoxLayout_3");
        label_lowshelf = new QLabel(GroupLoEQ);
        label_lowshelf->setObjectName("label_lowshelf");
        label_lowshelf->setFont(font);
        label_lowshelf->setWordWrap(false);

        vBoxLayout_3->addWidget(label_lowshelf);

        SliderLoEQ = new QSlider(GroupLoEQ);
        SliderLoEQ->setObjectName("SliderLoEQ");
        SliderLoEQ->setMinimum(100);
        SliderLoEQ->setMaximum(100000);
        SliderLoEQ->setSingleStep(30);
        SliderLoEQ->setPageStep(300);
        SliderLoEQ->setValue(100);
        SliderLoEQ->setOrientation(Qt::Horizontal);

        vBoxLayout_3->addWidget(SliderLoEQ);

        sliderLoEQ_labels = new QHBoxLayout();
        sliderLoEQ_labels->setSpacing(6);
        sliderLoEQ_labels->setObjectName("sliderLoEQ_labels");
        label_16hz_lo = new QLabel(GroupLoEQ);
        label_16hz_lo->setObjectName("label_16hz_lo");
        label_16hz_lo->setWordWrap(false);

        sliderLoEQ_labels->addWidget(label_16hz_lo);

        spacerItem4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        sliderLoEQ_labels->addItem(spacerItem4);

        TextLoEQ = new QLabel(GroupLoEQ);
        TextLoEQ->setObjectName("TextLoEQ");
        TextLoEQ->setText(QString::fromUtf8("textLabel2"));
        TextLoEQ->setWordWrap(false);

        sliderLoEQ_labels->addWidget(TextLoEQ);

        spacerItem5 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        sliderLoEQ_labels->addItem(spacerItem5);

        label_20khz_lo = new QLabel(GroupLoEQ);
        label_20khz_lo->setObjectName("label_20khz_lo");
        label_20khz_lo->setWordWrap(false);

        sliderLoEQ_labels->addWidget(label_20khz_lo);


        vBoxLayout_3->addLayout(sliderLoEQ_labels);


        vBoxLayout_1->addWidget(GroupLoEQ);


        verticalLayout_2->addWidget(GroupEQSliders);

        MainMainEQ = new QGroupBox(DlgPrefMixerDlg);
        MainMainEQ->setObjectName("MainMainEQ");
        verticalLayout_3 = new QVBoxLayout(MainMainEQ);
        verticalLayout_3->setSpacing(6);
        verticalLayout_3->setContentsMargins(11, 11, 11, 11);
        verticalLayout_3->setObjectName("verticalLayout_3");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        comboBoxMainEq = new QComboBox(MainMainEQ);
        comboBoxMainEq->setObjectName("comboBoxMainEq");

        horizontalLayout_2->addWidget(comboBoxMainEq);

        pbResetMainEq = new QPushButton(MainMainEQ);
        pbResetMainEq->setObjectName("pbResetMainEq");

        horizontalLayout_2->addWidget(pbResetMainEq);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);


        verticalLayout_3->addLayout(horizontalLayout_2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        slidersGridLayout = new QGridLayout();
        slidersGridLayout->setSpacing(6);
        slidersGridLayout->setObjectName("slidersGridLayout");

        horizontalLayout->addLayout(slidersGridLayout);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_3);


        verticalLayout_3->addLayout(horizontalLayout);


        verticalLayout_2->addWidget(MainMainEQ);

        spacerItem6 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(spacerItem6);

#if QT_CONFIG(shortcut)
        TextHiEQ->setBuddy(SliderHiEQ);
        TextLoEQ->setBuddy(SliderLoEQ);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(radioButtonAdditive, radioButtonConstantPower);
        QWidget::setTabOrder(radioButtonConstantPower, SliderXFader);
        QWidget::setTabOrder(SliderXFader, checkBoxReverse);
        QWidget::setTabOrder(checkBoxReverse, CheckBoxEqOnly);
        QWidget::setTabOrder(CheckBoxEqOnly, CheckBoxSingleEqEffect);
        QWidget::setTabOrder(CheckBoxSingleEqEffect, CheckBoxBypass);
        QWidget::setTabOrder(CheckBoxBypass, CheckBoxEqAutoReset);
        QWidget::setTabOrder(CheckBoxEqAutoReset, CheckBoxGainAutoReset);
        QWidget::setTabOrder(CheckBoxGainAutoReset, SliderHiEQ);
        QWidget::setTabOrder(SliderHiEQ, SliderLoEQ);
        QWidget::setTabOrder(SliderLoEQ, comboBoxMainEq);
        QWidget::setTabOrder(comboBoxMainEq, pbResetMainEq);

        retranslateUi(DlgPrefMixerDlg);

        QMetaObject::connectSlotsByName(DlgPrefMixerDlg);
    } // setupUi

    void retranslateUi(QWidget *DlgPrefMixerDlg)
    {
        DlgPrefMixerDlg->setWindowTitle(QCoreApplication::translate("DlgPrefMixerDlg", "Crossfader Preferences", nullptr));
        groupBox->setTitle(QCoreApplication::translate("DlgPrefMixerDlg", "Crossfader Curve", nullptr));
        radioButtonAdditive->setText(QCoreApplication::translate("DlgPrefMixerDlg", "Slow fade/Fast cut (additive)", nullptr));
        radioButtonConstantPower->setText(QCoreApplication::translate("DlgPrefMixerDlg", "Constant power", nullptr));
        label_xf_mixing->setText(QCoreApplication::translate("DlgPrefMixerDlg", "Mixing", nullptr));
        label_xf_scratching->setText(QCoreApplication::translate("DlgPrefMixerDlg", "Scratching", nullptr));
        label_linear->setText(QCoreApplication::translate("DlgPrefMixerDlg", "Linear", nullptr));
        label_logarithmic->setText(QCoreApplication::translate("DlgPrefMixerDlg", "Logarithmic", nullptr));
        checkBoxReverse->setText(QCoreApplication::translate("DlgPrefMixerDlg", "Reverse crossfader (Hamster Style)", nullptr));
        DeckEqualizerOptions->setTitle(QCoreApplication::translate("DlgPrefMixerDlg", "Deck Equalizers", nullptr));
        CheckBoxEqOnly->setText(QCoreApplication::translate("DlgPrefMixerDlg", "Only allow EQ knobs to control EQ-specific effects", nullptr));
#if QT_CONFIG(tooltip)
        CheckBoxEqOnly->setToolTip(QCoreApplication::translate("DlgPrefMixerDlg", "Uncheck to allow any effect to be loaded into the EQ knobs.", nullptr));
#endif // QT_CONFIG(tooltip)
        CheckBoxSingleEqEffect->setText(QCoreApplication::translate("DlgPrefMixerDlg", "Use the same EQ filter for all decks", nullptr));
#if QT_CONFIG(tooltip)
        CheckBoxSingleEqEffect->setToolTip(QCoreApplication::translate("DlgPrefMixerDlg", "Uncheck to allow different decks to use different EQ effects.", nullptr));
#endif // QT_CONFIG(tooltip)
        label_eq_plugin->setText(QCoreApplication::translate("DlgPrefMixerDlg", "Equalizer Plugin", nullptr));
        label_quickeffect->setText(QCoreApplication::translate("DlgPrefMixerDlg", "Quick Effect", nullptr));
        CheckBoxBypass->setText(QCoreApplication::translate("DlgPrefMixerDlg", "Bypass EQ effect processing", nullptr));
#if QT_CONFIG(tooltip)
        CheckBoxBypass->setToolTip(QCoreApplication::translate("DlgPrefMixerDlg", "When checked, EQs are not processed, improving performance on slower computers.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        CheckBoxEqAutoReset->setToolTip(QCoreApplication::translate("DlgPrefMixerDlg", "Resets the equalizers to their default values when loading a track.", nullptr));
#endif // QT_CONFIG(tooltip)
        CheckBoxEqAutoReset->setText(QCoreApplication::translate("DlgPrefMixerDlg", "Reset equalizers on track load", nullptr));
#if QT_CONFIG(tooltip)
        CheckBoxGainAutoReset->setToolTip(QCoreApplication::translate("DlgPrefMixerDlg", "Resets the deck gain to unity when loading a track.", nullptr));
#endif // QT_CONFIG(tooltip)
        CheckBoxGainAutoReset->setText(QCoreApplication::translate("DlgPrefMixerDlg", "Reset gain on track load", nullptr));
        GroupEQSliders->setTitle(QCoreApplication::translate("DlgPrefMixerDlg", "Equalizer frequency Shelves", nullptr));
        label_highshelf->setText(QCoreApplication::translate("DlgPrefMixerDlg", "High EQ", nullptr));
        label_16hz->setText(QCoreApplication::translate("DlgPrefMixerDlg", "16 Hz", nullptr));
        label_20khz->setText(QCoreApplication::translate("DlgPrefMixerDlg", "20.05 kHz", nullptr));
        label_lowshelf->setText(QCoreApplication::translate("DlgPrefMixerDlg", "Low EQ", nullptr));
        label_16hz_lo->setText(QCoreApplication::translate("DlgPrefMixerDlg", "16 Hz", nullptr));
        label_20khz_lo->setText(QCoreApplication::translate("DlgPrefMixerDlg", "20.05 kHz", nullptr));
        MainMainEQ->setTitle(QCoreApplication::translate("DlgPrefMixerDlg", "Main EQ", nullptr));
        pbResetMainEq->setText(QCoreApplication::translate("DlgPrefMixerDlg", "Reset Parameter", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgPrefMixerDlg: public Ui_DlgPrefMixerDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLGPREFMIXERDLG_H
