/****************************************************************************
** Meta object code from reading C++ file 'analysisfeature.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/library/analysis/analysisfeature.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'analysisfeature.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15AnalysisFeatureE_t {};
} // unnamed namespace

template <> constexpr inline auto AnalysisFeature::qt_create_metaobjectdata<qt_meta_tag_ZN15AnalysisFeatureE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AnalysisFeature",
        "analysisActive",
        "",
        "bActive",
        "trackProgress",
        "TrackId",
        "trackId",
        "AnalyzerProgress",
        "progress",
        "activate",
        "analyzeTracks",
        "QList<AnalyzerScheduledTrack>",
        "tracks",
        "suspendAnalysis",
        "resumeAnalysis",
        "stopAnalysis",
        "onTrackAnalysisSchedulerProgress",
        "currentTrackProgress",
        "currentTrackNumber",
        "totalTracksCount",
        "onTrackAnalysisSchedulerFinished"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'analysisActive'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'trackProgress'
        QtMocHelpers::SignalData<void(TrackId, AnalyzerProgress)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 7, 8 },
        }}),
        // Slot 'activate'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'analyzeTracks'
        QtMocHelpers::SlotData<void(const QList<AnalyzerScheduledTrack> &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'suspendAnalysis'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resumeAnalysis'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'stopAnalysis'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onTrackAnalysisSchedulerProgress'
        QtMocHelpers::SlotData<void(AnalyzerProgress, int, int)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 17 }, { QMetaType::Int, 18 }, { QMetaType::Int, 19 },
        }}),
        // Slot 'onTrackAnalysisSchedulerFinished'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AnalysisFeature, qt_meta_tag_ZN15AnalysisFeatureE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AnalysisFeature::staticMetaObject = { {
    QMetaObject::SuperData::link<LibraryFeature::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15AnalysisFeatureE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15AnalysisFeatureE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15AnalysisFeatureE_t>.metaTypes,
    nullptr
} };

void AnalysisFeature::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AnalysisFeature *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->analysisActive((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->trackProgress((*reinterpret_cast<std::add_pointer_t<TrackId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<AnalyzerProgress>>(_a[2]))); break;
        case 2: _t->activate(); break;
        case 3: _t->analyzeTracks((*reinterpret_cast<std::add_pointer_t<QList<AnalyzerScheduledTrack>>>(_a[1]))); break;
        case 4: _t->suspendAnalysis(); break;
        case 5: _t->resumeAnalysis(); break;
        case 6: _t->stopAnalysis(); break;
        case 7: _t->onTrackAnalysisSchedulerProgress((*reinterpret_cast<std::add_pointer_t<AnalyzerProgress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 8: _t->onTrackAnalysisSchedulerFinished(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AnalyzerProgress >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TrackId >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AnalyzerProgress >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AnalysisFeature::*)(bool )>(_a, &AnalysisFeature::analysisActive, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AnalysisFeature::*)(TrackId , AnalyzerProgress )>(_a, &AnalysisFeature::trackProgress, 1))
            return;
    }
}

const QMetaObject *AnalysisFeature::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AnalysisFeature::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15AnalysisFeatureE_t>.strings))
        return static_cast<void*>(this);
    return LibraryFeature::qt_metacast(_clname);
}

int AnalysisFeature::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = LibraryFeature::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void AnalysisFeature::analysisActive(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void AnalysisFeature::trackProgress(TrackId _t1, AnalyzerProgress _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}
QT_WARNING_POP
