// This file is autogenerated by CMake. Do not edit.

#include <QtCore/qtsymbolmacros.h>
#include <QtQml/qqmlextensionplugin.h>


QT_DECLARE_EXTERN_RESOURCE(qmlcache_mixxx_qml_lib)
QT_DECLARE_EXTERN_RESOURCE(qmake_Mixxx)
QT_DECLARE_EXTERN_RESOURCE(mixxx_qml_lib_raw_qml_0)


QT_DECLARE_EXTERN_SYMBOL_VOID(qml_register_types_Mixxx)

class MixxxPlugin : public QQmlEngineExtensionPlugin
{
    Q_OBJECT
    Q_PLUGIN_METADATA(IID QQmlEngineExtensionInterface_iid)

public:
    MixxxPlugin(QObject *parent = nullptr) : QQmlEngineExtensionPlugin(parent)
    {

QT_KEEP_SYMBOL(qml_register_types_Mixxx)
QT_KEEP_RESOURCE(qmlcache_mixxx_qml_lib)
QT_KEEP_RESOURCE(qmake_Mixxx)
QT_KEEP_RESOURCE(mixxx_qml_lib_raw_qml_0)
    }
};



#include "mixxx-qml-libplugin_MixxxPlugin.moc"
