/********************************************************************************
** Form generated from reading UI file 'dlgprefreplaygaindlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLGPREFREPLAYGAINDLG_H
#define UI_DLGPREFREPLAYGAINDLG_H

#include <QtCore/QLocale>
#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DlgPrefReplayGainDlg
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_2;
    QCheckBox *EnableGain;
    QLabel *label_boost;
    QSlider *SliderReplayGainBoost;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_3;
    QSpacerItem *spacerItem;
    QLabel *LabelCurrentReplayGainBoost;
    QSpacerItem *horizontalSpacer;
    QLabel *label_4;
    QLabel *label;
    QSlider *SliderDefaultBoost;
    QHBoxLayout *horizontalLayout;
    QLabel *label_2;
    QSpacerItem *horizontalSpacer_2;
    QLabel *LabelCurrentDefaultBoost;
    QSpacerItem *horizontalSpacer_3;
    QLabel *label_6;
    QGroupBox *groupBoxAnalysis;
    QVBoxLayout *verticalLayout_3;
    QRadioButton *radioButtonRG2;
    QRadioButton *radioButtonRG1;
    QRadioButton *radioButtonDisable;
    QCheckBox *checkBoxReanalyze;
    QGroupBox *Hints;
    QGridLayout *gridLayout_6;
    QLabel *label_8;
    QLabel *label_26;
    QSpacerItem *verticalSpacer;
    QButtonGroup *buttonGroupAnalyzer;

    void setupUi(QWidget *DlgPrefReplayGainDlg)
    {
        if (DlgPrefReplayGainDlg->objectName().isEmpty())
            DlgPrefReplayGainDlg->setObjectName("DlgPrefReplayGainDlg");
        DlgPrefReplayGainDlg->resize(509, 609);
        verticalLayout = new QVBoxLayout(DlgPrefReplayGainDlg);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        groupBox = new QGroupBox(DlgPrefReplayGainDlg);
        groupBox->setObjectName("groupBox");
        verticalLayout_2 = new QVBoxLayout(groupBox);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        EnableGain = new QCheckBox(groupBox);
        EnableGain->setObjectName("EnableGain");

        verticalLayout_2->addWidget(EnableGain);

        label_boost = new QLabel(groupBox);
        label_boost->setObjectName("label_boost");

        verticalLayout_2->addWidget(label_boost);

        SliderReplayGainBoost = new QSlider(groupBox);
        SliderReplayGainBoost->setObjectName("SliderReplayGainBoost");
        SliderReplayGainBoost->setLocale(QLocale(QLocale::C, QLocale::AnyCountry));
        SliderReplayGainBoost->setMinimum(-12);
        SliderReplayGainBoost->setMaximum(12);
        SliderReplayGainBoost->setSingleStep(1);
        SliderReplayGainBoost->setPageStep(1);
        SliderReplayGainBoost->setValue(0);
        SliderReplayGainBoost->setOrientation(Qt::Horizontal);
        SliderReplayGainBoost->setTickInterval(5);

        verticalLayout_2->addWidget(SliderReplayGainBoost);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalLayout_2->setContentsMargins(-1, -1, 0, 20);
        label_3 = new QLabel(groupBox);
        label_3->setObjectName("label_3");

        horizontalLayout_2->addWidget(label_3);

        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(spacerItem);

        LabelCurrentReplayGainBoost = new QLabel(groupBox);
        LabelCurrentReplayGainBoost->setObjectName("LabelCurrentReplayGainBoost");
        LabelCurrentReplayGainBoost->setText(QString::fromUtf8("TextLabel"));

        horizontalLayout_2->addWidget(LabelCurrentReplayGainBoost);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        label_4 = new QLabel(groupBox);
        label_4->setObjectName("label_4");

        horizontalLayout_2->addWidget(label_4);


        verticalLayout_2->addLayout(horizontalLayout_2);

        label = new QLabel(groupBox);
        label->setObjectName("label");

        verticalLayout_2->addWidget(label);

        SliderDefaultBoost = new QSlider(groupBox);
        SliderDefaultBoost->setObjectName("SliderDefaultBoost");
        SliderDefaultBoost->setMinimum(-12);
        SliderDefaultBoost->setMaximum(12);
        SliderDefaultBoost->setPageStep(1);
        SliderDefaultBoost->setSliderPosition(0);
        SliderDefaultBoost->setOrientation(Qt::Horizontal);

        verticalLayout_2->addWidget(SliderDefaultBoost);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(-1, -1, -1, 0);
        label_2 = new QLabel(groupBox);
        label_2->setObjectName("label_2");

        horizontalLayout->addWidget(label_2);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);

        LabelCurrentDefaultBoost = new QLabel(groupBox);
        LabelCurrentDefaultBoost->setObjectName("LabelCurrentDefaultBoost");
        LabelCurrentDefaultBoost->setText(QString::fromUtf8("TextLabel"));

        horizontalLayout->addWidget(LabelCurrentDefaultBoost);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_3);

        label_6 = new QLabel(groupBox);
        label_6->setObjectName("label_6");

        horizontalLayout->addWidget(label_6);


        verticalLayout_2->addLayout(horizontalLayout);


        verticalLayout->addWidget(groupBox);

        groupBoxAnalysis = new QGroupBox(DlgPrefReplayGainDlg);
        groupBoxAnalysis->setObjectName("groupBoxAnalysis");
        verticalLayout_3 = new QVBoxLayout(groupBoxAnalysis);
        verticalLayout_3->setSpacing(6);
        verticalLayout_3->setContentsMargins(11, 11, 11, 11);
        verticalLayout_3->setObjectName("verticalLayout_3");
        radioButtonRG2 = new QRadioButton(groupBoxAnalysis);
        buttonGroupAnalyzer = new QButtonGroup(DlgPrefReplayGainDlg);
        buttonGroupAnalyzer->setObjectName("buttonGroupAnalyzer");
        buttonGroupAnalyzer->addButton(radioButtonRG2);
        radioButtonRG2->setObjectName("radioButtonRG2");

        verticalLayout_3->addWidget(radioButtonRG2);

        radioButtonRG1 = new QRadioButton(groupBoxAnalysis);
        buttonGroupAnalyzer->addButton(radioButtonRG1);
        radioButtonRG1->setObjectName("radioButtonRG1");

        verticalLayout_3->addWidget(radioButtonRG1);

        radioButtonDisable = new QRadioButton(groupBoxAnalysis);
        buttonGroupAnalyzer->addButton(radioButtonDisable);
        radioButtonDisable->setObjectName("radioButtonDisable");

        verticalLayout_3->addWidget(radioButtonDisable);

        checkBoxReanalyze = new QCheckBox(groupBoxAnalysis);
        checkBoxReanalyze->setObjectName("checkBoxReanalyze");

        verticalLayout_3->addWidget(checkBoxReanalyze);


        verticalLayout->addWidget(groupBoxAnalysis);

        Hints = new QGroupBox(DlgPrefReplayGainDlg);
        Hints->setObjectName("Hints");
        gridLayout_6 = new QGridLayout(Hints);
        gridLayout_6->setSpacing(6);
        gridLayout_6->setContentsMargins(11, 11, 11, 11);
        gridLayout_6->setObjectName("gridLayout_6");
        label_8 = new QLabel(Hints);
        label_8->setObjectName("label_8");
        label_8->setScaledContents(true);
        label_8->setWordWrap(true);

        gridLayout_6->addWidget(label_8, 0, 0, 1, 1);

        label_26 = new QLabel(Hints);
        label_26->setObjectName("label_26");
        label_26->setWordWrap(true);

        gridLayout_6->addWidget(label_26, 1, 0, 1, 1);


        verticalLayout->addWidget(Hints);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

#if QT_CONFIG(shortcut)
        label_boost->setBuddy(SliderReplayGainBoost);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(EnableGain, SliderReplayGainBoost);
        QWidget::setTabOrder(SliderReplayGainBoost, SliderDefaultBoost);
        QWidget::setTabOrder(SliderDefaultBoost, radioButtonRG1);
        QWidget::setTabOrder(radioButtonRG1, radioButtonRG2);
        QWidget::setTabOrder(radioButtonRG2, radioButtonDisable);
        QWidget::setTabOrder(radioButtonDisable, checkBoxReanalyze);

        retranslateUi(DlgPrefReplayGainDlg);

        QMetaObject::connectSlotsByName(DlgPrefReplayGainDlg);
    } // setupUi

    void retranslateUi(QWidget *DlgPrefReplayGainDlg)
    {
        DlgPrefReplayGainDlg->setWindowTitle(QCoreApplication::translate("DlgPrefReplayGainDlg", "Normalization Preferences", nullptr));
        groupBox->setTitle(QCoreApplication::translate("DlgPrefReplayGainDlg", "ReplayGain Loudness Normalization", nullptr));
#if QT_CONFIG(tooltip)
        EnableGain->setToolTip(QCoreApplication::translate("DlgPrefReplayGainDlg", "Apply loudness normalization to loaded tracks.", nullptr));
#endif // QT_CONFIG(tooltip)
        EnableGain->setText(QCoreApplication::translate("DlgPrefReplayGainDlg", "Apply ReplayGain", nullptr));
#if QT_CONFIG(tooltip)
        label_boost->setToolTip(QCoreApplication::translate("DlgPrefReplayGainDlg", "For tracks with ReplayGain, adjust the target loudness to this LUFS value (Loudness Units relative to Full Scale).", nullptr));
#endif // QT_CONFIG(tooltip)
        label_boost->setText(QCoreApplication::translate("DlgPrefReplayGainDlg", "Target loudness", nullptr));
#if QT_CONFIG(tooltip)
        SliderReplayGainBoost->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        label_3->setText(QCoreApplication::translate("DlgPrefReplayGainDlg", "-30 LUFS", nullptr));
        label_4->setText(QCoreApplication::translate("DlgPrefReplayGainDlg", "-6 LUFS", nullptr));
#if QT_CONFIG(tooltip)
        label->setToolTip(QCoreApplication::translate("DlgPrefReplayGainDlg", "When ReplayGain is enabled, adjust tracks lacking ReplayGain information by this amount.", nullptr));
#endif // QT_CONFIG(tooltip)
        label->setText(QCoreApplication::translate("DlgPrefReplayGainDlg", "Initial boost without ReplayGain data", nullptr));
        label_2->setText(QCoreApplication::translate("DlgPrefReplayGainDlg", "-12 dB", nullptr));
        label_6->setText(QCoreApplication::translate("DlgPrefReplayGainDlg", "+12 dB", nullptr));
        groupBoxAnalysis->setTitle(QCoreApplication::translate("DlgPrefReplayGainDlg", "Analysis", nullptr));
        radioButtonRG2->setText(QCoreApplication::translate("DlgPrefReplayGainDlg", "ReplayGain 2.0 (ITU-R BS.1770)", nullptr));
        radioButtonRG1->setText(QCoreApplication::translate("DlgPrefReplayGainDlg", "ReplayGain 1.0", nullptr));
        radioButtonDisable->setText(QCoreApplication::translate("DlgPrefReplayGainDlg", "Disabled", nullptr));
        checkBoxReanalyze->setText(QCoreApplication::translate("DlgPrefReplayGainDlg", "Re-analyze and override an existing value ", nullptr));
        Hints->setTitle(QCoreApplication::translate("DlgPrefReplayGainDlg", "Hints", nullptr));
        label_8->setText(QCoreApplication::translate("DlgPrefReplayGainDlg", "ReplayGain targets a reference loudness of -18 LUFS (Loudness Units relative to Full Scale). You may increase it if you find Mixxx is too quiet or reduce it if you find that your tracks are clipping. You may also want to decrease the volume of unanalyzed tracks if you find they are often louder than ReplayGained tracks. For podcasting a loudness of -16 LUFS is recommended.\n"
"\n"
"The loudness target is approximate and assumes track pregain and main output level are unchanged.", nullptr));
        label_26->setText(QCoreApplication::translate("DlgPrefReplayGainDlg", "When an unanalyzed track is playing, Mixxx will avoid an abrupt volume change by not applying a newly calculated ReplayGain value.", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgPrefReplayGainDlg: public Ui_DlgPrefReplayGainDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLGPREFREPLAYGAINDLG_H
