/****************************************************************************
** Meta object code from reading C++ file 'dlgprefcontroller.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/controllers/dlgprefcontroller.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dlgprefcontroller.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17DlgPrefControllerE_t {};
} // unnamed namespace

template <> constexpr inline auto DlgPrefController::qt_create_metaobjectdata<qt_meta_tag_ZN17DlgPrefControllerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DlgPrefController",
        "applyMapping",
        "",
        "Controller*",
        "pController",
        "std::shared_ptr<LegacyControllerMapping>",
        "pMapping",
        "bEnabled",
        "mappingStarted",
        "mappingEnded",
        "slotUpdate",
        "slotApply",
        "slotHide",
        "slotResetToDefaults",
        "slotRecreateControlPickerMenu",
        "slotMappingSelected",
        "index",
        "slotInputControlSearch",
        "slotOutputControlSearch",
        "slotStopLearning",
        "enableWizardAndIOTabs",
        "enable",
        "addInputMapping",
        "showLearningWizard",
        "removeInputMappings",
        "clearAllInputMappings",
        "addOutputMapping",
        "removeOutputMappings",
        "clearAllOutputMappings",
        "midiInputMappingsLearned",
        "MidiInputMappings",
        "mappings"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'applyMapping'
        QtMocHelpers::SignalData<void(Controller *, std::shared_ptr<LegacyControllerMapping>, bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 }, { QMetaType::Bool, 7 },
        }}),
        // Signal 'mappingStarted'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'mappingEnded'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotUpdate'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotApply'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotHide'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotResetToDefaults'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotRecreateControlPickerMenu'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotMappingSelected'
        QtMocHelpers::SlotData<void(int)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 16 },
        }}),
        // Slot 'slotInputControlSearch'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotOutputControlSearch'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotStopLearning'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'enableWizardAndIOTabs'
        QtMocHelpers::SlotData<void(bool)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 21 },
        }}),
        // Slot 'addInputMapping'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showLearningWizard'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'removeInputMappings'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'clearAllInputMappings'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'addOutputMapping'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'removeOutputMappings'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'clearAllOutputMappings'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'midiInputMappingsLearned'
        QtMocHelpers::SlotData<void(const MidiInputMappings &)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 30, 31 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DlgPrefController, qt_meta_tag_ZN17DlgPrefControllerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DlgPrefController::staticMetaObject = { {
    QMetaObject::SuperData::link<DlgPreferencePage::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17DlgPrefControllerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17DlgPrefControllerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17DlgPrefControllerE_t>.metaTypes,
    nullptr
} };

void DlgPrefController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DlgPrefController *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->applyMapping((*reinterpret_cast<std::add_pointer_t<Controller*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<std::shared_ptr<LegacyControllerMapping>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 1: _t->mappingStarted(); break;
        case 2: _t->mappingEnded(); break;
        case 3: _t->slotUpdate(); break;
        case 4: _t->slotApply(); break;
        case 5: _t->slotHide(); break;
        case 6: _t->slotResetToDefaults(); break;
        case 7: _t->slotRecreateControlPickerMenu(); break;
        case 8: _t->slotMappingSelected((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->slotInputControlSearch(); break;
        case 10: _t->slotOutputControlSearch(); break;
        case 11: _t->slotStopLearning(); break;
        case 12: _t->enableWizardAndIOTabs((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->addInputMapping(); break;
        case 14: _t->showLearningWizard(); break;
        case 15: _t->removeInputMappings(); break;
        case 16: _t->clearAllInputMappings(); break;
        case 17: _t->addOutputMapping(); break;
        case 18: _t->removeOutputMappings(); break;
        case 19: _t->clearAllOutputMappings(); break;
        case 20: _t->midiInputMappingsLearned((*reinterpret_cast<std::add_pointer_t<MidiInputMappings>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DlgPrefController::*)(Controller * , std::shared_ptr<LegacyControllerMapping> , bool )>(_a, &DlgPrefController::applyMapping, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgPrefController::*)()>(_a, &DlgPrefController::mappingStarted, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DlgPrefController::*)()>(_a, &DlgPrefController::mappingEnded, 2))
            return;
    }
}

const QMetaObject *DlgPrefController::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DlgPrefController::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17DlgPrefControllerE_t>.strings))
        return static_cast<void*>(this);
    return DlgPreferencePage::qt_metacast(_clname);
}

int DlgPrefController::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DlgPreferencePage::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 21;
    }
    return _id;
}

// SIGNAL 0
void DlgPrefController::applyMapping(Controller * _t1, std::shared_ptr<LegacyControllerMapping> _t2, bool _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void DlgPrefController::mappingStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void DlgPrefController::mappingEnded()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
