/****************************************************************************
** Meta object code from reading C++ file 'controlpotmeter.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../mixxx-2.5.6/src/control/controlpotmeter.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'controlpotmeter.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16PotmeterControlsE_t {};
} // unnamed namespace

template <> constexpr inline auto PotmeterControls::qt_create_metaobjectdata<qt_meta_tag_ZN16PotmeterControlsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PotmeterControls",
        "incValue",
        "",
        "decValue",
        "incSmallValue",
        "decSmallValue",
        "setToOne",
        "setToMinusOne",
        "setToZero",
        "setToDefault",
        "toggleValue",
        "toggleMinusValue",
        "setIsDefault",
        "isDefault"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'incValue'
        QtMocHelpers::SlotData<void(double)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'decValue'
        QtMocHelpers::SlotData<void(double)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'incSmallValue'
        QtMocHelpers::SlotData<void(double)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'decSmallValue'
        QtMocHelpers::SlotData<void(double)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'setToOne'
        QtMocHelpers::SlotData<void(double)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'setToMinusOne'
        QtMocHelpers::SlotData<void(double)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'setToZero'
        QtMocHelpers::SlotData<void(double)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'setToDefault'
        QtMocHelpers::SlotData<void(double)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'toggleValue'
        QtMocHelpers::SlotData<void(double)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'toggleMinusValue'
        QtMocHelpers::SlotData<void(double)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
        // Slot 'setIsDefault'
        QtMocHelpers::SlotData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PotmeterControls, qt_meta_tag_ZN16PotmeterControlsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PotmeterControls::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16PotmeterControlsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16PotmeterControlsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16PotmeterControlsE_t>.metaTypes,
    nullptr
} };

void PotmeterControls::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PotmeterControls *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->incValue((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 1: _t->decValue((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 2: _t->incSmallValue((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 3: _t->decSmallValue((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 4: _t->setToOne((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 5: _t->setToMinusOne((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 6: _t->setToZero((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 7: _t->setToDefault((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 8: _t->toggleValue((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 9: _t->toggleMinusValue((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 10: _t->setIsDefault((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *PotmeterControls::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PotmeterControls::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16PotmeterControlsE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int PotmeterControls::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN15ControlPotmeterE_t {};
} // unnamed namespace

template <> constexpr inline auto ControlPotmeter::qt_create_metaobjectdata<qt_meta_tag_ZN15ControlPotmeterE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ControlPotmeter",
        "privateValueChanged",
        "",
        "dValue",
        "pSender"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'privateValueChanged'
        QtMocHelpers::SlotData<void(double, QObject *)>(1, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 3 }, { QMetaType::QObjectStar, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ControlPotmeter, qt_meta_tag_ZN15ControlPotmeterE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ControlPotmeter::staticMetaObject = { {
    QMetaObject::SuperData::link<ControlObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15ControlPotmeterE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15ControlPotmeterE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15ControlPotmeterE_t>.metaTypes,
    nullptr
} };

void ControlPotmeter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ControlPotmeter *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->privateValueChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[2]))); break;
        default: ;
        }
    }
}

const QMetaObject *ControlPotmeter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ControlPotmeter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15ControlPotmeterE_t>.strings))
        return static_cast<void*>(this);
    return ControlObject::qt_metacast(_clname);
}

int ControlPotmeter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ControlObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
QT_WARNING_POP
