/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<controlsortfiltermodel.h>)
#  include <controlsortfiltermodel.h>
#endif
#if __has_include(<qmlconfigproxy.h>)
#  include <qmlconfigproxy.h>
#endif
#if __has_include(<qmlcontrolproxy.h>)
#  include <qmlcontrolproxy.h>
#endif
#if __has_include(<qmldlgpreferencesproxy.h>)
#  include <qmldlgpreferencesproxy.h>
#endif
#if __has_include(<qmleffectmanifestparametersmodel.h>)
#  include <qmleffectmanifestparametersmodel.h>
#endif
#if __has_include(<qmleffectslotproxy.h>)
#  include <qmleffectslotproxy.h>
#endif
#if __has_include(<qmleffectsmanagerproxy.h>)
#  include <qmleffectsmanagerproxy.h>
#endif
#if __has_include(<qmllibraryproxy.h>)
#  include <qmllibraryproxy.h>
#endif
#if __has_include(<qmllibrarytracklistmodel.h>)
#  include <qmllibrarytracklistmodel.h>
#endif
#if __has_include(<qmlplayermanagerproxy.h>)
#  include <qmlplayermanagerproxy.h>
#endif
#if __has_include(<qmlplayerproxy.h>)
#  include <qmlplayerproxy.h>
#endif
#if __has_include(<qmlvisibleeffectsmodel.h>)
#  include <qmlvisibleeffectsmodel.h>
#endif
#if __has_include(<qmlwaveformoverview.h>)
#  include <qmlwaveformoverview.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
Q_QMLTYPE_EXPORT void qml_register_types_Mixxx()
{
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    qmlRegisterTypesAndRevisions<ControlSortFilterModel>("Mixxx", 1);
    qmlRegisterAnonymousType<QAbstractItemModel, 254>("Mixxx", 1);
    QMetaType::fromType<QAbstractProxyModel *>().id();
    QMetaType::fromType<QIdentityProxyModel *>().id();
    QMetaType::fromType<QSortFilterProxyModel *>().id();
    qmlRegisterTypesAndRevisions<mixxx::qml::QmlConfigProxy>("Mixxx", 1);
    qmlRegisterTypesAndRevisions<mixxx::qml::QmlControlProxy>("Mixxx", 1);
    qmlRegisterTypesAndRevisions<mixxx::qml::QmlDlgPreferencesProxy>("Mixxx", 1);
    qmlRegisterTypesAndRevisions<mixxx::qml::QmlEffectManifestParametersModel>("Mixxx", 1);
    qmlRegisterEnum<mixxx::qml::QmlEffectManifestParametersModel::Roles>("mixxx::qml::QmlEffectManifestParametersModel::Roles");
    qmlRegisterTypesAndRevisions<mixxx::qml::QmlEffectSlotProxy>("Mixxx", 1);
    qmlRegisterTypesAndRevisions<mixxx::qml::QmlEffectsManagerProxy>("Mixxx", 1);
    qmlRegisterTypesAndRevisions<mixxx::qml::QmlLibraryProxy>("Mixxx", 1);
    qmlRegisterTypesAndRevisions<mixxx::qml::QmlLibraryTrackListModel>("Mixxx", 1);
    qmlRegisterEnum<mixxx::qml::QmlLibraryTrackListModel::Roles>("mixxx::qml::QmlLibraryTrackListModel::Roles");
    qmlRegisterTypesAndRevisions<mixxx::qml::QmlPlayerManagerProxy>("Mixxx", 1);
    qmlRegisterTypesAndRevisions<mixxx::qml::QmlPlayerProxy>("Mixxx", 1);
    qmlRegisterTypesAndRevisions<mixxx::qml::QmlVisibleEffectsModel>("Mixxx", 1);
    qmlRegisterEnum<mixxx::qml::QmlVisibleEffectsModel::Roles>("mixxx::qml::QmlVisibleEffectsModel::Roles");
    qmlRegisterTypesAndRevisions<mixxx::qml::QmlWaveformOverview>("Mixxx", 1);
    qmlRegisterAnonymousType<QQuickItem, 254>("Mixxx", 1);
    qmlRegisterEnum<mixxx::qml::QmlWaveformOverview::Channels>("mixxx::qml::QmlWaveformOverview::Channels");
    qmlRegisterEnum<mixxx::qml::QmlWaveformOverview::ChannelFlag>("mixxx::qml::QmlWaveformOverview::ChannelFlag");
    qmlRegisterEnum<mixxx::qml::QmlWaveformOverview::Renderer>("mixxx::qml::QmlWaveformOverview::Renderer");
    QT_WARNING_POP
    qmlRegisterModule("Mixxx", 1, 0);
}

static const QQmlModuleRegistration mixxxRegistration("Mixxx", qml_register_types_Mixxx);
