/********************************************************************************
** Form generated from reading UI file 'dlgprefeffectsdlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLGPREFEFFECTSDLG_H
#define UI_DLGPREFEFFECTSDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTableView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DlgPrefEffectsDlg
{
public:
    QVBoxLayout *verticalLayout;
    QTabWidget *effectsTabs;
    QWidget *chainPresetsTab;
    QGridLayout *chainPresetGridLayout;
    QLabel *chainPresetsTabHeader;
    QLabel *presetListControllerLabel;
    QLabel *label;
    QLabel *chainPresetsLabel;
    QListView *quickEffectListView;
    QVBoxLayout *chainDetailsVBoxLayout;
    QLabel *effectsInChainHeader;
    QLabel *effect1Name;
    QLabel *effect2Name;
    QLabel *effect3Name;
    QHBoxLayout *chainButtonsTopRow;
    QPushButton *chainPresetImportButton;
    QPushButton *chainPresetRenameButton;
    QHBoxLayout *chainButtonsBottomRow;
    QPushButton *chainPresetExportButton;
    QPushButton *chainPresetDeleteButton;
    QLabel *quickEffectChainPresetsLabel;
    QListView *chainListView;
    QWidget *visibleEffectsTab;
    QHBoxLayout *horizontalLayout;
    QGridLayout *effectsTablesGridLayout;
    QLabel *effectsTablesHeaderLabel;
    QLabel *visibleEffectsTableLabel;
    QLabel *hiddenEffectsTableLabel;
    QTableView *visibleEffectsTableView;
    QVBoxLayout *moveButtonslayout;
    QSpacerItem *hideShowVSpacerTop;
    QPushButton *hideButton;
    QPushButton *unhideButton;
    QSpacerItem *hideShowVSpacerBottom;
    QTableView *hiddenEffectsTableView;
    QGroupBox *groupBox;
    QGridLayout *gridLayout;
    QLabel *effectNameLabel;
    QLabel *effectName;
    QLabel *effectVersionLabel;
    QLabel *effectVersion;
    QLabel *effectTypeLabel;
    QLabel *effectType;
    QLabel *effectAuthorLabel;
    QLabel *effectAuthor;
    QLabel *effectDescriptionLabel;
    QLabel *effectDescription;
    QGroupBox *groupMetaKnobOption;
    QHBoxLayout *horizontalLayoutMetaKnob;
    QRadioButton *radioButtonKeepMetaknobPosition;
    QRadioButton *radioButtonMetaknobLoadDefault;
    QButtonGroup *buttonGroupMetaKnob;
    QButtonGroup *buttonGroupPresetActions;

    void setupUi(QWidget *DlgPrefEffectsDlg)
    {
        if (DlgPrefEffectsDlg->objectName().isEmpty())
            DlgPrefEffectsDlg->setObjectName("DlgPrefEffectsDlg");
        DlgPrefEffectsDlg->resize(1434, 823);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Ignored, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(DlgPrefEffectsDlg->sizePolicy().hasHeightForWidth());
        DlgPrefEffectsDlg->setSizePolicy(sizePolicy);
        verticalLayout = new QVBoxLayout(DlgPrefEffectsDlg);
        verticalLayout->setObjectName("verticalLayout");
        effectsTabs = new QTabWidget(DlgPrefEffectsDlg);
        effectsTabs->setObjectName("effectsTabs");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Ignored, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(effectsTabs->sizePolicy().hasHeightForWidth());
        effectsTabs->setSizePolicy(sizePolicy1);
        chainPresetsTab = new QWidget();
        chainPresetsTab->setObjectName("chainPresetsTab");
        sizePolicy.setHeightForWidth(chainPresetsTab->sizePolicy().hasHeightForWidth());
        chainPresetsTab->setSizePolicy(sizePolicy);
        chainPresetGridLayout = new QGridLayout(chainPresetsTab);
        chainPresetGridLayout->setObjectName("chainPresetGridLayout");
        chainPresetsTabHeader = new QLabel(chainPresetsTab);
        chainPresetsTabHeader->setObjectName("chainPresetsTabHeader");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(chainPresetsTabHeader->sizePolicy().hasHeightForWidth());
        chainPresetsTabHeader->setSizePolicy(sizePolicy2);
        chainPresetsTabHeader->setWordWrap(true);

        chainPresetGridLayout->addWidget(chainPresetsTabHeader, 1, 0, 1, 2);

        presetListControllerLabel = new QLabel(chainPresetsTab);
        presetListControllerLabel->setObjectName("presetListControllerLabel");
        presetListControllerLabel->setWordWrap(true);

        chainPresetGridLayout->addWidget(presetListControllerLabel, 0, 0, 1, 2);

        label = new QLabel(chainPresetsTab);
        label->setObjectName("label");

        chainPresetGridLayout->addWidget(label, 7, 0, 1, 1);

        chainPresetsLabel = new QLabel(chainPresetsTab);
        chainPresetsLabel->setObjectName("chainPresetsLabel");
        sizePolicy2.setHeightForWidth(chainPresetsLabel->sizePolicy().hasHeightForWidth());
        chainPresetsLabel->setSizePolicy(sizePolicy2);
        QFont font;
        font.setBold(true);
        chainPresetsLabel->setFont(font);
        chainPresetsLabel->setContentsMargins(-1, 6, -1, -1);

        chainPresetGridLayout->addWidget(chainPresetsLabel, 8, 0, 1, 1);

        quickEffectListView = new QListView(chainPresetsTab);
        quickEffectListView->setObjectName("quickEffectListView");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(quickEffectListView->sizePolicy().hasHeightForWidth());
        quickEffectListView->setSizePolicy(sizePolicy3);

        chainPresetGridLayout->addWidget(quickEffectListView, 9, 1, 1, 1);

        chainDetailsVBoxLayout = new QVBoxLayout();
        chainDetailsVBoxLayout->setObjectName("chainDetailsVBoxLayout");
        effectsInChainHeader = new QLabel(chainPresetsTab);
        effectsInChainHeader->setObjectName("effectsInChainHeader");

        chainDetailsVBoxLayout->addWidget(effectsInChainHeader);

        effect1Name = new QLabel(chainPresetsTab);
        effect1Name->setObjectName("effect1Name");

        chainDetailsVBoxLayout->addWidget(effect1Name);

        effect2Name = new QLabel(chainPresetsTab);
        effect2Name->setObjectName("effect2Name");

        chainDetailsVBoxLayout->addWidget(effect2Name);

        effect3Name = new QLabel(chainPresetsTab);
        effect3Name->setObjectName("effect3Name");

        chainDetailsVBoxLayout->addWidget(effect3Name);

        chainButtonsTopRow = new QHBoxLayout();
        chainButtonsTopRow->setObjectName("chainButtonsTopRow");
        chainPresetImportButton = new QPushButton(chainPresetsTab);
        buttonGroupPresetActions = new QButtonGroup(DlgPrefEffectsDlg);
        buttonGroupPresetActions->setObjectName("buttonGroupPresetActions");
        buttonGroupPresetActions->addButton(chainPresetImportButton);
        chainPresetImportButton->setObjectName("chainPresetImportButton");

        chainButtonsTopRow->addWidget(chainPresetImportButton);

        chainPresetRenameButton = new QPushButton(chainPresetsTab);
        buttonGroupPresetActions->addButton(chainPresetRenameButton);
        chainPresetRenameButton->setObjectName("chainPresetRenameButton");
        chainPresetRenameButton->setEnabled(false);

        chainButtonsTopRow->addWidget(chainPresetRenameButton);


        chainDetailsVBoxLayout->addLayout(chainButtonsTopRow);

        chainButtonsBottomRow = new QHBoxLayout();
        chainButtonsBottomRow->setObjectName("chainButtonsBottomRow");
        chainPresetExportButton = new QPushButton(chainPresetsTab);
        buttonGroupPresetActions->addButton(chainPresetExportButton);
        chainPresetExportButton->setObjectName("chainPresetExportButton");
        chainPresetExportButton->setEnabled(false);

        chainButtonsBottomRow->addWidget(chainPresetExportButton);

        chainPresetDeleteButton = new QPushButton(chainPresetsTab);
        buttonGroupPresetActions->addButton(chainPresetDeleteButton);
        chainPresetDeleteButton->setObjectName("chainPresetDeleteButton");
        chainPresetDeleteButton->setEnabled(false);

        chainButtonsBottomRow->addWidget(chainPresetDeleteButton);


        chainDetailsVBoxLayout->addLayout(chainButtonsBottomRow);


        chainPresetGridLayout->addLayout(chainDetailsVBoxLayout, 10, 0, 1, 2);

        quickEffectChainPresetsLabel = new QLabel(chainPresetsTab);
        quickEffectChainPresetsLabel->setObjectName("quickEffectChainPresetsLabel");
        sizePolicy2.setHeightForWidth(quickEffectChainPresetsLabel->sizePolicy().hasHeightForWidth());
        quickEffectChainPresetsLabel->setSizePolicy(sizePolicy2);
        quickEffectChainPresetsLabel->setFont(font);
        quickEffectChainPresetsLabel->setContentsMargins(-1, 6, -1, -1);

        chainPresetGridLayout->addWidget(quickEffectChainPresetsLabel, 8, 1, 1, 1);

        chainListView = new QListView(chainPresetsTab);
        chainListView->setObjectName("chainListView");
        sizePolicy3.setHeightForWidth(chainListView->sizePolicy().hasHeightForWidth());
        chainListView->setSizePolicy(sizePolicy3);

        chainPresetGridLayout->addWidget(chainListView, 9, 0, 1, 1);

        effectsTabs->addTab(chainPresetsTab, QString());
        visibleEffectsTab = new QWidget();
        visibleEffectsTab->setObjectName("visibleEffectsTab");
        sizePolicy.setHeightForWidth(visibleEffectsTab->sizePolicy().hasHeightForWidth());
        visibleEffectsTab->setSizePolicy(sizePolicy);
        horizontalLayout = new QHBoxLayout(visibleEffectsTab);
        horizontalLayout->setObjectName("horizontalLayout");
        effectsTablesGridLayout = new QGridLayout();
        effectsTablesGridLayout->setObjectName("effectsTablesGridLayout");
        effectsTablesHeaderLabel = new QLabel(visibleEffectsTab);
        effectsTablesHeaderLabel->setObjectName("effectsTablesHeaderLabel");

        effectsTablesGridLayout->addWidget(effectsTablesHeaderLabel, 0, 0, 1, 3);

        visibleEffectsTableLabel = new QLabel(visibleEffectsTab);
        visibleEffectsTableLabel->setObjectName("visibleEffectsTableLabel");
        visibleEffectsTableLabel->setFont(font);
        visibleEffectsTableLabel->setContentsMargins(-1, 6, -1, -1);

        effectsTablesGridLayout->addWidget(visibleEffectsTableLabel, 1, 0, 1, 1);

        hiddenEffectsTableLabel = new QLabel(visibleEffectsTab);
        hiddenEffectsTableLabel->setObjectName("hiddenEffectsTableLabel");
        hiddenEffectsTableLabel->setFont(font);
        hiddenEffectsTableLabel->setContentsMargins(-1, 6, -1, -1);

        effectsTablesGridLayout->addWidget(hiddenEffectsTableLabel, 1, 2, 1, 1);

        visibleEffectsTableView = new QTableView(visibleEffectsTab);
        visibleEffectsTableView->setObjectName("visibleEffectsTableView");

        effectsTablesGridLayout->addWidget(visibleEffectsTableView, 2, 0, 1, 1);

        moveButtonslayout = new QVBoxLayout();
        moveButtonslayout->setObjectName("moveButtonslayout");
        hideShowVSpacerTop = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        moveButtonslayout->addItem(hideShowVSpacerTop);

        hideButton = new QPushButton(visibleEffectsTab);
        hideButton->setObjectName("hideButton");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(hideButton->sizePolicy().hasHeightForWidth());
        hideButton->setSizePolicy(sizePolicy4);
        hideButton->setFixedSize(QSize(20, 40));

        moveButtonslayout->addWidget(hideButton);

        unhideButton = new QPushButton(visibleEffectsTab);
        unhideButton->setObjectName("unhideButton");
        sizePolicy4.setHeightForWidth(unhideButton->sizePolicy().hasHeightForWidth());
        unhideButton->setSizePolicy(sizePolicy4);
        unhideButton->setFixedSize(QSize(20, 40));

        moveButtonslayout->addWidget(unhideButton);

        hideShowVSpacerBottom = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        moveButtonslayout->addItem(hideShowVSpacerBottom);


        effectsTablesGridLayout->addLayout(moveButtonslayout, 2, 1, 1, 1);

        hiddenEffectsTableView = new QTableView(visibleEffectsTab);
        hiddenEffectsTableView->setObjectName("hiddenEffectsTableView");

        effectsTablesGridLayout->addWidget(hiddenEffectsTableView, 2, 2, 1, 1);

        groupBox = new QGroupBox(visibleEffectsTab);
        groupBox->setObjectName("groupBox");
        QSizePolicy sizePolicy5(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(groupBox->sizePolicy().hasHeightForWidth());
        groupBox->setSizePolicy(sizePolicy5);
        gridLayout = new QGridLayout(groupBox);
        gridLayout->setObjectName("gridLayout");
        effectNameLabel = new QLabel(groupBox);
        effectNameLabel->setObjectName("effectNameLabel");
        QSizePolicy sizePolicy6(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy6.setHorizontalStretch(0);
        sizePolicy6.setVerticalStretch(0);
        sizePolicy6.setHeightForWidth(effectNameLabel->sizePolicy().hasHeightForWidth());
        effectNameLabel->setSizePolicy(sizePolicy6);
        effectNameLabel->setAlignment(Qt::AlignRight|Qt::AlignTop|Qt::AlignTrailing);

        gridLayout->addWidget(effectNameLabel, 0, 0, 1, 1);

        effectName = new QLabel(groupBox);
        effectName->setObjectName("effectName");
        QSizePolicy sizePolicy7(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Preferred);
        sizePolicy7.setHorizontalStretch(0);
        sizePolicy7.setVerticalStretch(0);
        sizePolicy7.setHeightForWidth(effectName->sizePolicy().hasHeightForWidth());
        effectName->setSizePolicy(sizePolicy7);
        effectName->setText(QString::fromUtf8("(effect name)"));
        effectName->setWordWrap(true);

        gridLayout->addWidget(effectName, 0, 1, 1, 1);

        effectVersionLabel = new QLabel(groupBox);
        effectVersionLabel->setObjectName("effectVersionLabel");
        sizePolicy6.setHeightForWidth(effectVersionLabel->sizePolicy().hasHeightForWidth());
        effectVersionLabel->setSizePolicy(sizePolicy6);
        effectVersionLabel->setAlignment(Qt::AlignRight|Qt::AlignTop|Qt::AlignTrailing);

        gridLayout->addWidget(effectVersionLabel, 1, 0, 1, 1);

        effectVersion = new QLabel(groupBox);
        effectVersion->setObjectName("effectVersion");
        sizePolicy7.setHeightForWidth(effectVersion->sizePolicy().hasHeightForWidth());
        effectVersion->setSizePolicy(sizePolicy7);
        effectVersion->setText(QString::fromUtf8("(effect version)"));
        effectVersion->setWordWrap(true);

        gridLayout->addWidget(effectVersion, 1, 1, 1, 1);

        effectTypeLabel = new QLabel(groupBox);
        effectTypeLabel->setObjectName("effectTypeLabel");
        sizePolicy6.setHeightForWidth(effectTypeLabel->sizePolicy().hasHeightForWidth());
        effectTypeLabel->setSizePolicy(sizePolicy6);
        effectTypeLabel->setAlignment(Qt::AlignRight|Qt::AlignTop|Qt::AlignTrailing);

        gridLayout->addWidget(effectTypeLabel, 2, 0, 1, 1);

        effectType = new QLabel(groupBox);
        effectType->setObjectName("effectType");
        sizePolicy7.setHeightForWidth(effectType->sizePolicy().hasHeightForWidth());
        effectType->setSizePolicy(sizePolicy7);
        effectType->setText(QString::fromUtf8("(effect type)"));
        effectType->setWordWrap(true);

        gridLayout->addWidget(effectType, 2, 1, 1, 1);

        effectAuthorLabel = new QLabel(groupBox);
        effectAuthorLabel->setObjectName("effectAuthorLabel");
        sizePolicy6.setHeightForWidth(effectAuthorLabel->sizePolicy().hasHeightForWidth());
        effectAuthorLabel->setSizePolicy(sizePolicy6);
        effectAuthorLabel->setAlignment(Qt::AlignRight|Qt::AlignTop|Qt::AlignTrailing);

        gridLayout->addWidget(effectAuthorLabel, 3, 0, 1, 1);

        effectAuthor = new QLabel(groupBox);
        effectAuthor->setObjectName("effectAuthor");
        sizePolicy7.setHeightForWidth(effectAuthor->sizePolicy().hasHeightForWidth());
        effectAuthor->setSizePolicy(sizePolicy7);
        effectAuthor->setText(QString::fromUtf8("(effect author)"));
        effectAuthor->setWordWrap(true);

        gridLayout->addWidget(effectAuthor, 3, 1, 1, 1);

        effectDescriptionLabel = new QLabel(groupBox);
        effectDescriptionLabel->setObjectName("effectDescriptionLabel");
        sizePolicy6.setHeightForWidth(effectDescriptionLabel->sizePolicy().hasHeightForWidth());
        effectDescriptionLabel->setSizePolicy(sizePolicy6);
        effectDescriptionLabel->setAlignment(Qt::AlignRight|Qt::AlignTop|Qt::AlignTrailing);

        gridLayout->addWidget(effectDescriptionLabel, 4, 0, 1, 1);

        effectDescription = new QLabel(groupBox);
        effectDescription->setObjectName("effectDescription");
        sizePolicy7.setHeightForWidth(effectDescription->sizePolicy().hasHeightForWidth());
        effectDescription->setSizePolicy(sizePolicy7);
        effectDescription->setText(QString::fromUtf8("(effect description)"));
        effectDescription->setWordWrap(true);

        gridLayout->addWidget(effectDescription, 4, 1, 1, 1);


        effectsTablesGridLayout->addWidget(groupBox, 3, 0, 1, 3);


        horizontalLayout->addLayout(effectsTablesGridLayout);

        effectsTabs->addTab(visibleEffectsTab, QString());

        verticalLayout->addWidget(effectsTabs);

        groupMetaKnobOption = new QGroupBox(DlgPrefEffectsDlg);
        groupMetaKnobOption->setObjectName("groupMetaKnobOption");
        horizontalLayoutMetaKnob = new QHBoxLayout(groupMetaKnobOption);
        horizontalLayoutMetaKnob->setObjectName("horizontalLayoutMetaKnob");
        radioButtonKeepMetaknobPosition = new QRadioButton(groupMetaKnobOption);
        buttonGroupMetaKnob = new QButtonGroup(DlgPrefEffectsDlg);
        buttonGroupMetaKnob->setObjectName("buttonGroupMetaKnob");
        buttonGroupMetaKnob->addButton(radioButtonKeepMetaknobPosition);
        radioButtonKeepMetaknobPosition->setObjectName("radioButtonKeepMetaknobPosition");

        horizontalLayoutMetaKnob->addWidget(radioButtonKeepMetaknobPosition);

        radioButtonMetaknobLoadDefault = new QRadioButton(groupMetaKnobOption);
        buttonGroupMetaKnob->addButton(radioButtonMetaknobLoadDefault);
        radioButtonMetaknobLoadDefault->setObjectName("radioButtonMetaknobLoadDefault");

        horizontalLayoutMetaKnob->addWidget(radioButtonMetaknobLoadDefault);


        verticalLayout->addWidget(groupMetaKnobOption);

        QWidget::setTabOrder(chainListView, quickEffectListView);
        QWidget::setTabOrder(quickEffectListView, visibleEffectsTableView);
        QWidget::setTabOrder(visibleEffectsTableView, hiddenEffectsTableView);
        QWidget::setTabOrder(hiddenEffectsTableView, hideButton);
        QWidget::setTabOrder(hideButton, unhideButton);
        QWidget::setTabOrder(unhideButton, chainPresetImportButton);
        QWidget::setTabOrder(chainPresetImportButton, chainPresetExportButton);
        QWidget::setTabOrder(chainPresetExportButton, chainPresetRenameButton);
        QWidget::setTabOrder(chainPresetRenameButton, chainPresetDeleteButton);
        QWidget::setTabOrder(chainPresetDeleteButton, radioButtonKeepMetaknobPosition);
        QWidget::setTabOrder(radioButtonKeepMetaknobPosition, radioButtonMetaknobLoadDefault);

        retranslateUi(DlgPrefEffectsDlg);

        effectsTabs->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(DlgPrefEffectsDlg);
    } // setupUi

    void retranslateUi(QWidget *DlgPrefEffectsDlg)
    {
        DlgPrefEffectsDlg->setWindowTitle(QCoreApplication::translate("DlgPrefEffectsDlg", "Effects Preferences", nullptr));
        chainPresetsTabHeader->setText(QCoreApplication::translate("DlgPrefEffectsDlg", "Drag and drop to rearrange lists and copy chains between lists. Create and edit chain presets in the effect units in the main window. Please refer the manual for further details.", nullptr));
        presetListControllerLabel->setText(QCoreApplication::translate("DlgPrefEffectsDlg", "Chain presets from these lists will be selectable in the given order in the main window and from controllers (depending on the controller mapping).", nullptr));
        label->setText(QString());
        chainPresetsLabel->setText(QCoreApplication::translate("DlgPrefEffectsDlg", "Effect Chain Presets", nullptr));
        effectsInChainHeader->setText(QCoreApplication::translate("DlgPrefEffectsDlg", "Effects in this chain preset:", nullptr));
        effect1Name->setText(QCoreApplication::translate("DlgPrefEffectsDlg", "effect 1 name", nullptr));
        effect2Name->setText(QCoreApplication::translate("DlgPrefEffectsDlg", "effect 2 name", nullptr));
        effect3Name->setText(QCoreApplication::translate("DlgPrefEffectsDlg", "effect 3 name", nullptr));
        chainPresetImportButton->setText(QCoreApplication::translate("DlgPrefEffectsDlg", "Import", nullptr));
        chainPresetRenameButton->setText(QCoreApplication::translate("DlgPrefEffectsDlg", "Rename", nullptr));
        chainPresetExportButton->setText(QCoreApplication::translate("DlgPrefEffectsDlg", "Export", nullptr));
        chainPresetDeleteButton->setText(QCoreApplication::translate("DlgPrefEffectsDlg", "Delete", nullptr));
        quickEffectChainPresetsLabel->setText(QCoreApplication::translate("DlgPrefEffectsDlg", "Quick Effect Chain Presets", nullptr));
        effectsTabs->setTabText(effectsTabs->indexOf(chainPresetsTab), QCoreApplication::translate("DlgPrefEffectsDlg", "Effect Chain Presets", nullptr));
        effectsTablesHeaderLabel->setText(QCoreApplication::translate("DlgPrefEffectsDlg", "Drag and drop to rearrange lists and show or hide effects.", nullptr));
        visibleEffectsTableLabel->setText(QCoreApplication::translate("DlgPrefEffectsDlg", "Visible Effects", nullptr));
        hiddenEffectsTableLabel->setText(QCoreApplication::translate("DlgPrefEffectsDlg", "Hidden Effects", nullptr));
        hideButton->setText(QCoreApplication::translate("DlgPrefEffectsDlg", "\342\235\257", nullptr));
        unhideButton->setText(QCoreApplication::translate("DlgPrefEffectsDlg", "\342\235\256", nullptr));
        groupBox->setTitle(QCoreApplication::translate("DlgPrefEffectsDlg", "Effect Info", nullptr));
        effectNameLabel->setText(QCoreApplication::translate("DlgPrefEffectsDlg", "Name:", nullptr));
        effectVersionLabel->setText(QCoreApplication::translate("DlgPrefEffectsDlg", "Version:", nullptr));
        effectTypeLabel->setText(QCoreApplication::translate("DlgPrefEffectsDlg", "Type:", nullptr));
        effectAuthorLabel->setText(QCoreApplication::translate("DlgPrefEffectsDlg", "Author:", nullptr));
        effectDescriptionLabel->setText(QCoreApplication::translate("DlgPrefEffectsDlg", "Description:", nullptr));
        effectsTabs->setTabText(effectsTabs->indexOf(visibleEffectsTab), QCoreApplication::translate("DlgPrefEffectsDlg", "Visible Effects", nullptr));
        groupMetaKnobOption->setTitle(QCoreApplication::translate("DlgPrefEffectsDlg", "Effect load behavior", nullptr));
        radioButtonKeepMetaknobPosition->setText(QCoreApplication::translate("DlgPrefEffectsDlg", "Keep metaknob position", nullptr));
        radioButtonMetaknobLoadDefault->setText(QCoreApplication::translate("DlgPrefEffectsDlg", "Reset metaknob to effect default", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgPrefEffectsDlg: public Ui_DlgPrefEffectsDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLGPREFEFFECTSDLG_H
