/********************************************************************************
** Form generated from reading UI file 'dlgpreferencesdlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLGPREFERENCESDLG_H
#define UI_DLGPREFERENCESDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DlgPreferencesDlg
{
public:
    QHBoxLayout *horizontalLayout;
    QSplitter *splitter;
    QTreeWidget *contentsTreeWidget;
    QWidget *layoutWidget;
    QVBoxLayout *vboxLayout;
    QStackedWidget *pagesWidget;
    QWidget *page;
    QHBoxLayout *horizontalLayout_2;
    QLabel *labelWarningIcon;
    QLabel *labelWarning;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgPreferencesDlg)
    {
        if (DlgPreferencesDlg->objectName().isEmpty())
            DlgPreferencesDlg->setObjectName("DlgPreferencesDlg");
        DlgPreferencesDlg->resize(681, 445);
        horizontalLayout = new QHBoxLayout(DlgPreferencesDlg);
        horizontalLayout->setObjectName("horizontalLayout");
        splitter = new QSplitter(DlgPreferencesDlg);
        splitter->setObjectName("splitter");
        splitter->setOrientation(Qt::Horizontal);
        splitter->setChildrenCollapsible(false);
        contentsTreeWidget = new QTreeWidget(splitter);
        contentsTreeWidget->setObjectName("contentsTreeWidget");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(contentsTreeWidget->sizePolicy().hasHeightForWidth());
        contentsTreeWidget->setSizePolicy(sizePolicy);
        contentsTreeWidget->setMinimumSize(QSize(100, 0));
        contentsTreeWidget->setMaximumSize(QSize(1500, 16777215));
        contentsTreeWidget->setAutoFillBackground(false);
        contentsTreeWidget->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContentsOnFirstShow);
        contentsTreeWidget->setProperty("showDropIndicator", QVariant(false));
        contentsTreeWidget->setSelectionMode(QAbstractItemView::SingleSelection);
        contentsTreeWidget->setRootIsDecorated(true);
        splitter->addWidget(contentsTreeWidget);
        layoutWidget = new QWidget(splitter);
        layoutWidget->setObjectName("layoutWidget");
        vboxLayout = new QVBoxLayout(layoutWidget);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        pagesWidget = new QStackedWidget(layoutWidget);
        pagesWidget->setObjectName("pagesWidget");
        sizePolicy.setHeightForWidth(pagesWidget->sizePolicy().hasHeightForWidth());
        pagesWidget->setSizePolicy(sizePolicy);
        pagesWidget->setBaseSize(QSize(0, 0));
        page = new QWidget();
        page->setObjectName("page");
        pagesWidget->addWidget(page);

        vboxLayout->addWidget(pagesWidget);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        labelWarningIcon = new QLabel(layoutWidget);
        labelWarningIcon->setObjectName("labelWarningIcon");
        labelWarningIcon->setMinimumSize(QSize(16, 16));
        labelWarningIcon->setMaximumSize(QSize(16, 16));
        QFont font;
        font.setBold(false);
        labelWarningIcon->setFont(font);

        horizontalLayout_2->addWidget(labelWarningIcon);

        labelWarning = new QLabel(layoutWidget);
        labelWarning->setObjectName("labelWarning");

        horizontalLayout_2->addWidget(labelWarning);


        vboxLayout->addLayout(horizontalLayout_2);

        buttonBox = new QDialogButtonBox(layoutWidget);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Apply|QDialogButtonBox::Cancel|QDialogButtonBox::Help|QDialogButtonBox::Ok|QDialogButtonBox::RestoreDefaults);
        buttonBox->setCenterButtons(false);

        vboxLayout->addWidget(buttonBox);

        splitter->addWidget(layoutWidget);

        horizontalLayout->addWidget(splitter);


        retranslateUi(DlgPreferencesDlg);

        QMetaObject::connectSlotsByName(DlgPreferencesDlg);
    } // setupUi

    void retranslateUi(QDialog *DlgPreferencesDlg)
    {
        DlgPreferencesDlg->setWindowTitle(QCoreApplication::translate("DlgPreferencesDlg", "Preferences", nullptr));
        QTreeWidgetItem *___qtreewidgetitem = contentsTreeWidget->headerItem();
        ___qtreewidgetitem->setText(0, QCoreApplication::translate("DlgPreferencesDlg", "1", nullptr));
        labelWarningIcon->setText(QCoreApplication::translate("DlgPreferencesDlg", "TextLabel", nullptr));
        labelWarning->setText(QCoreApplication::translate("DlgPreferencesDlg", "TextLabel", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgPreferencesDlg: public Ui_DlgPreferencesDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLGPREFERENCESDLG_H
