/********************************************************************************
** Form generated from reading UI file 'dlganalysis.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLGANALYSIS_H
#define UI_DLGANALYSIS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DlgAnalysis
{
public:
    QVBoxLayout *verticalLayout;
    QWidget *LibraryFeatureControls;
    QHBoxLayout *horizontalLayout;
    QRadioButton *radioButtonRecentlyAdded;
    QRadioButton *radioButtonAllSongs;
    QPushButton *pushButtonSelectAll;
    QPushButton *pushButtonAnalyze;
    QLabel *labelProgress;
    QSpacerItem *horizontalSpacer;
    QTableView *m_pTrackTablePlaceholder;

    void setupUi(QWidget *DlgAnalysis)
    {
        if (DlgAnalysis->objectName().isEmpty())
            DlgAnalysis->setObjectName("DlgAnalysis");
        DlgAnalysis->resize(500, 399);
        verticalLayout = new QVBoxLayout(DlgAnalysis);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        LibraryFeatureControls = new QWidget(DlgAnalysis);
        LibraryFeatureControls->setObjectName("LibraryFeatureControls");
        horizontalLayout = new QHBoxLayout(LibraryFeatureControls);
        horizontalLayout->setSpacing(0);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        radioButtonRecentlyAdded = new QRadioButton(LibraryFeatureControls);
        radioButtonRecentlyAdded->setObjectName("radioButtonRecentlyAdded");
        radioButtonRecentlyAdded->setFocusPolicy(Qt::NoFocus);

        horizontalLayout->addWidget(radioButtonRecentlyAdded);

        radioButtonAllSongs = new QRadioButton(LibraryFeatureControls);
        radioButtonAllSongs->setObjectName("radioButtonAllSongs");
        radioButtonAllSongs->setFocusPolicy(Qt::NoFocus);

        horizontalLayout->addWidget(radioButtonAllSongs);

        pushButtonSelectAll = new QPushButton(LibraryFeatureControls);
        pushButtonSelectAll->setObjectName("pushButtonSelectAll");
        pushButtonSelectAll->setFocusPolicy(Qt::NoFocus);

        horizontalLayout->addWidget(pushButtonSelectAll);

        pushButtonAnalyze = new QPushButton(LibraryFeatureControls);
        pushButtonAnalyze->setObjectName("pushButtonAnalyze");
        pushButtonAnalyze->setFocusPolicy(Qt::NoFocus);
        pushButtonAnalyze->setCheckable(true);

        horizontalLayout->addWidget(pushButtonAnalyze);

        labelProgress = new QLabel(LibraryFeatureControls);
        labelProgress->setObjectName("labelProgress");

        horizontalLayout->addWidget(labelProgress);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout->addWidget(LibraryFeatureControls);

        m_pTrackTablePlaceholder = new QTableView(DlgAnalysis);
        m_pTrackTablePlaceholder->setObjectName("m_pTrackTablePlaceholder");
        m_pTrackTablePlaceholder->setShowGrid(true);

        verticalLayout->addWidget(m_pTrackTablePlaceholder);


        retranslateUi(DlgAnalysis);

        QMetaObject::connectSlotsByName(DlgAnalysis);
    } // setupUi

    void retranslateUi(QWidget *DlgAnalysis)
    {
        DlgAnalysis->setWindowTitle(QCoreApplication::translate("DlgAnalysis", "Analyze", nullptr));
#if QT_CONFIG(tooltip)
        radioButtonRecentlyAdded->setToolTip(QCoreApplication::translate("DlgAnalysis", "Shows tracks added to the library within the last 7 days.", nullptr));
#endif // QT_CONFIG(tooltip)
        radioButtonRecentlyAdded->setText(QCoreApplication::translate("DlgAnalysis", "New", nullptr));
#if QT_CONFIG(tooltip)
        radioButtonAllSongs->setToolTip(QCoreApplication::translate("DlgAnalysis", "Shows all tracks in the library.", nullptr));
#endif // QT_CONFIG(tooltip)
        radioButtonAllSongs->setText(QCoreApplication::translate("DlgAnalysis", "All", nullptr));
#if QT_CONFIG(tooltip)
        pushButtonSelectAll->setToolTip(QCoreApplication::translate("DlgAnalysis", "Selects all tracks in the table below.", nullptr));
#endif // QT_CONFIG(tooltip)
        pushButtonSelectAll->setText(QCoreApplication::translate("DlgAnalysis", "Select All", nullptr));
#if QT_CONFIG(tooltip)
        pushButtonAnalyze->setToolTip(QCoreApplication::translate("DlgAnalysis", "Runs beatgrid, key, and ReplayGain detection on the selected tracks. Does not generate waveforms for the selected tracks to save disk space.", nullptr));
#endif // QT_CONFIG(tooltip)
        pushButtonAnalyze->setText(QCoreApplication::translate("DlgAnalysis", "Analyze", nullptr));
        labelProgress->setText(QCoreApplication::translate("DlgAnalysis", "Progress", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgAnalysis: public Ui_DlgAnalysis {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLGANALYSIS_H
